	
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.alert.framework;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.StringTokenizer;

import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletLog;

import com.ibm.ccd.ui.context.IWPCUser;
import com.ibm.ccd.ui.context.WPCUser;
import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.mediator.IWPCMediator;
import com.ibm.ccd.ui.mediator.WPCMediator;
import com.ibm.ccd.ui.mediator.WPCMediatorException;
import com.ibm.ccd.ui.portlets.alert.ITransactionDisplay;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;


/**
 * TransactionExchangeService class Service class that holds all user Transactions
 * Also stores and loads the transactions file.
 */
public class TransactionExchangeService implements IAlertsDataSource {
    private static PortletContext pc = null;
    private static PortletLog log = null;
    private ITransactions transMap = null;
    private String userId = null;

    private TransactionExchangeService() {
    }

    public static IAlertsDataSource getInstance(PortletContext portletContext) {
        pc = portletContext;

        TransactionExchangeService objectInstance = new TransactionExchangeService();

        return ((IAlertsDataSource) objectInstance);
    }

    private void initTransactionService() {
        log = pc.getLog();
        log("initializeService - > ");

        Properties prop = new Properties();
        prop = loadUserTrans(userId);

        Enumeration enumNames = prop.propertyNames();
        transMap = new Transactions();

        while (enumNames.hasMoreElements()) {
            String keyTransId = (String) enumNames.nextElement();
            String value = prop.getProperty(keyTransId);
            StringTokenizer strtoken = new StringTokenizer(value, TOKENIZER);

            TransactionBean transBean = new TransactionBean();
            transBean.setTransactionID(keyTransId);
            transBean.setTransactionDescription(getReplaceEmptyString(
                    getTokenString(strtoken)));
            transBean.setTransactionStartTime(getReplaceEmptyString(
                    getTokenString(strtoken)));
            transBean.setTransactionEndTime(getReplaceEmptyString(
                    getTokenString(strtoken)));
            transBean.setTransactionStatus(getReplaceEmptyString(getTokenString(
                        strtoken)));
            transBean.setAttachmentFilePath(getReplaceEmptyString(
                    getTokenString(strtoken)));
            transBean.setXlsAttachmentFilePath(getReplaceEmptyString(
                    getTokenString(strtoken)));

            transMap.addTransaction(transBean);
        }

        log(" initializeService <- ");
    }

    private String getTokenString(StringTokenizer strtoken) {
        String tokenStr = "";

        try {
            tokenStr = strtoken.nextToken();
        } catch (java.util.NoSuchElementException e) {
            tokenStr = "";
        }

        return tokenStr;
    }

    public void addTransaction(String transactionID, WPSContext context) {
        log(" addTransaction -> transactionID");
        addTransaction(transactionID, "", "", "",
            String.valueOf(ITransactionStatus.IN_PROGRESS), context);
        log(" addTransaction <- ");
    }

    public void addTransaction(ITransaction transactionObject,
        WPSContext context) {
        WPCUser user = null;
        user = context.getWpcUser();
        log(" addTransaction -> ITransaction ");
        userId = (String) user.getProperty(IWPCUser.KEY_USER_ID);
        initTransactionService();
        transMap.addTransaction(transactionObject);
        saveUserTrans();
        log(" addTransaction <- ");
    }

    public void addTransaction(String transactionID, String transactionDesc,
        String startTime, WPSContext context) {
        log(" addTransaction ->transactionID,desc,starttime");
        addTransaction(transactionID, transactionDesc, startTime, "",
            String.valueOf(ITransactionStatus.IN_PROGRESS), context);

        log(" addTransaction <- ");
    }

    public void addTransaction(String transactionID, String transactionDesc,
        String startTime, String endTime, String status, WPSContext context) {
        log(" addTransaction ->transactionId,desc,starttime, endtime,status ");

        ITransaction transaction = createTransactionBean(transactionID,
                transactionDesc, startTime, endTime, status);
        addTransaction(transaction, context);
    }

    private ITransaction createTransactionBean(String transactionID,
        String transactionDesc, String startTime, String endTime, String status) {
    	log("createTransactionBean ->"+transactionID+","+transactionDesc+","+startTime +","+endTime +","+status);
        if (getReplaceEmptyString(transactionID).equals("")) {
            log.error(UIUtils.getTextMessages(pc, Locale.US, "MESSAGE_CODE_302"));
        }

        TransactionBean transactionbean = new TransactionBean();

        if (getReplaceEmptyString(startTime).equals("")) {
            startTime = String.valueOf(new Date());
        }

        if (getReplaceEmptyString(endTime).equals("")) {
            endTime = String.valueOf(new Date());
        }

        try {
            Integer.parseInt(status);
        } catch (NumberFormatException numb) {
            status = String.valueOf(ITransactionStatus.IN_PROGRESS);
        }

        transactionbean.setTransactionID(transactionID);
        transactionbean.setTransactionDescription(getReplaceEmptyString(
                transactionDesc));
        transactionbean.setTransactionStartTime(startTime);
        transactionbean.setTransactionEndTime(endTime);
        transactionbean.setTransactionStatus(status);
        transactionbean.setAttachmentFilePath("");
        transactionbean.setXlsAttachmentFilePath("");

        ITransaction transaction = (ITransaction) transactionbean;

        return transaction;
    }

    private ITransaction getTransaction(String transactionID, String user) {
        log(" getTransaction -> transactionID ,userId ");

        return transMap.getTransaction(transactionID);
    }

    public ITransactions getTransactions(WPSContext context) {
        log(" getTransactions -> ");

        WPCUser user = null;
        user = context.getWpcUser();
        userId = (String) user.getProperty(IWPCUser.KEY_USER_ID);

        Map map = null;

        // loading the transactions from props 
        initTransactionService();

        //	fetching the transactions from WPC
        IWPCMediator wpcMediator = null;

        if (transMap.getTransactionsSize() != 0) {
            wpcMediator = (IWPCMediator) WPCMediator.getInstance();

            try {
                // fetch the transactions from wpc and update the status of all the transactions
            	String[] inCompleteTrIdsArray = transMap.getInCompletedTransactionIDsArray();
            	if (inCompleteTrIdsArray!= null && inCompleteTrIdsArray.length != 0) {
            		map = wpcMediator.getTransactionsStatus(inCompleteTrIdsArray,context);	
				}
                
            } catch (WPCMediatorException e) {
                e.printStackTrace();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        Iterator mapIterator = null;

        if (map == null) {
            map = new HashMap();
            mapIterator = map.keySet().iterator();
        } else {
            mapIterator = map.keySet().iterator();
        }

        boolean save_flag = false;

        while (mapIterator.hasNext()) {
            String transactionId = ((String) mapIterator.next()).trim();
            String status = (String) map.get(transactionId);

            if (transMap.getTransaction(transactionId) != null) {
                if (status.equals(STATUS_COMPLETED)) {
                    status = String.valueOf(ITransactionStatus.COMPLETED);
                } else {
                    status = String.valueOf(ITransactionStatus.IN_PROGRESS);
                }
                if (transMap.getTransaction(transactionId).getTransactionStatus().equals(status)) {
					save_flag = save_flag | false;
				}else{
					save_flag = save_flag | true;
				}
                transMap.getTransaction(transactionId).setTransactionStatus(status);
                
            } else {
            	System.out.println(" New transaction id, not available in list");
                ITransaction transaction = createTransactionBean(transactionId,
                        null, null, null, status);

                transMap.addTransaction(transaction);
                save_flag = true;
            }
        }

        //	saving the transactions objects (changed status)
        if (save_flag) {
            saveUserTrans();
        }

        return transMap;
    }

    private Properties loadUserTrans(String userId) {
        InputStream instream = null;
        Properties prop = new Properties();

        if (userId != null) {
            try {
                instream = new FileInputStream(getFilePath());
            } catch (FileNotFoundException e) {
                log.error(UIUtils.getTextMessages(pc, Locale.US,
                        "MESSAGE_CODE_301"));
                e.printStackTrace();
            }
        }

        if (instream != null) {
            try {
                prop.load(instream);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        return prop;
    }

    private Properties saveUserTrans() {
        log(" saveUserTrans ->");

        OutputStream outStream = null;
        Properties prop = new Properties();

        if (userId != null) {
            try {
                outStream = new FileOutputStream(getFilePath());
            } catch (FileNotFoundException e) {
                log.error(UIUtils.getTextMessages(pc, Locale.US,
                        "MESSAGE_CODE_301"));
                e.printStackTrace();
            }
        }

        if (outStream != null) {
            try {
                Iterator itr = transMap.getTransactionsKeyIterator();

                while (itr.hasNext()) {
                    String transactionId = (String) itr.next();
                    ITransaction transaction = (ITransaction) transMap.getTransaction(transactionId);
                    String transactionDesc = transaction.getTransactionDescription();
                    String transactionStartTime = transaction.getTransactionStartTime();
                    String transactionEndTime = transaction.getTransactionEndTime();
                    String transactionStatus = transaction.getTransactionStatus();
                    StringBuffer value = new StringBuffer();

                    value = new StringBuffer().append(getReplaceNullString(
                                transactionDesc)).append(TOKENIZER)
                                              .append(getReplaceNullString(
                                transactionStartTime)).append(TOKENIZER)
                                              .append(getReplaceNullString(
                                transactionEndTime)).append(TOKENIZER)
                                              .append(getReplaceNullString(
                                transactionStatus)).append(TOKENIZER)
                                              .append(getReplaceNullString(
                                transaction.getAttachmentFilePath()))
                                              .append(TOKENIZER).append(getReplaceNullString(
                                transaction.getXlsAttachmentFilePath()));

                    prop.setProperty(transaction.getTransactionID(),
                        value.toString());
                }

                prop.store(outStream, null);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        log(" saveUserTrans <-");

        return prop;
    }

    private String getReplaceNullString(String str) {
        if (str.equals("")) {
            return "null";
        } else {
            return str;
        }
    }

    public void saveAttachmentFile(ITransaction transactionObj,
        Object attachObj, String extension) {
        String fileName = saveAttachment(transactionObj, attachObj, extension);

        if (extension.equals(ITransactionDisplay.ATTACHMENT_FILE_EXTENSION_XML)) {
            transactionObj.setAttachmentFilePath(fileName);
        } else if (extension.equals(
                    ITransactionDisplay.ATTACHMENT_FILE_EXTENSION_XLS)) {
            transactionObj.setXlsAttachmentFilePath(fileName);
        }

        saveUserTrans();
    }

    /**
     * Saving the file in appropriate location.
     * with ATTACH_<USER_ID>_<TRANSACTION_ID>_<FILENAME>
     * @param transactionId
     * @param attachObj
     */
    private String saveAttachment(ITransaction transactionObj,
        Object attachObj, String extension) {
        log.debug(" saveAttachmentFile ->");

        if (attachObj == null) {
            return ITransactionDisplay.TRANSACTION_NO_ATTACH;
        }

        String fileName = "";
        String attachFileName = "ATTACH" + userId + "_" +
            transactionObj.getTransactionID() + fileName + extension;

        File fileObj = null;
        FileOutputStream outputStream = null;
        String fileAttachLocation = "";

        try {
            fileAttachLocation = getAttachmentPath();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }

        String completePath = fileAttachLocation + attachFileName;
        InputStream inputstream = (InputStream) attachObj;
        UIUtils.saveFileStream(inputstream, completePath);

        return attachFileName;
    }

    public String getAttachmentPath() throws FileNotFoundException {
        String fileAttachLocation;
        File attachdbDir = new File(pc.getRealPath("/") + File.separator +
                ITransactionDisplay.TRANSACTION_ATTACHMENT_DIR);

        if (!attachdbDir.exists()) {
            log.warn(UIUtils.getTextMessages(pc, Locale.US, "MESSAGE_CODE_303"));

            if (!attachdbDir.mkdir()) {
                log.warn(UIUtils.getTextMessages(pc, Locale.US,
                        "MESSAGE_CODE_304"));
                throw new FileNotFoundException();
            }
        }

        fileAttachLocation = pc.getRealPath("/") + File.separator +
            ITransactionDisplay.TRANSACTION_ATTACHMENT_DIR + File.separator;

        //fileAttachLocation = pc.getRealPath("/")+ File.separator+"WEB-INF"+File.separator+IReference.TRANSACTION_PATH +File.separator;
        return fileAttachLocation;
    }

    private String getReplaceEmptyString(String str) {
        if ((str == null) || (str.equals("null"))) {
            return "";
        } else {
            return str;
        }
    }

    private File getFilePath() throws FileNotFoundException {
        //		System.out.println(" init getFilePath ="+pc.getRealPath("/WEB-INF/"));		
        File trandbDir = new File(pc.getRealPath("/WEB-INF/") +
                IReference.TRANSACTION_PATH);

        if (!trandbDir.exists()) {
            log.warn(UIUtils.getTextMessages(pc, Locale.US, "MESSAGE_CODE_303"));

            if (!trandbDir.mkdir()) {
                log.warn(UIUtils.getTextMessages(pc, Locale.US,
                        "MESSAGE_CODE_304"));
                throw new FileNotFoundException();
            }
        }

        return new File(trandbDir + File.separator + userId + "_" +
            IReference.TRANS_LOG_FILE);
    }

    private static void log(String str) {
       // 	System.out.println("TransactionExchangeService: "+str);
    }
}
