
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.alert.framework;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;


/**
 * This class holds Transaction Object list in HashMap
 */
public class Transactions implements ITransactions, Serializable {
    /**
     * transactions stores list of transaction details(all attributes)
     * It is collections of Transaction object referred by ITransaction.
     */
    private HashMap transactions = new HashMap();

    public ITransaction getTransaction(String transactionID) {
        if (transactions.get(transactionID) == null) {
            log("No transactions available in the list");

            return null;
        } else {
            return (ITransaction) transactions.get(transactionID);
        }
    }

    public void addTransaction(ITransaction transactionObject) {
        log(" addTransaction (ITransaction )->");
        transactions.put(transactionObject.getTransactionID(), transactionObject);
    }

    public void addTransaction(String transactionID, String transactionDesc,
        String startTime) {
        log(" addTransaction ->transactionId,desc,starttime ");

        TransactionBean transactionbean = (TransactionBean) transactions.get(transactionID);

        if (transactionbean == null) {
            transactionbean = new TransactionBean();
        } else {
            log("updating the transaction=" + transactionID);
            transactionbean = (TransactionBean) transactions.get(transactionID);
        }

        transactionbean.setTransactionID(transactionID);
        transactionbean.setTransactionDescription(transactionDesc);
        transactionbean.setTransactionStartTime(startTime);

        ITransaction transaction = (ITransaction) transactionbean;
        transactions.put(transactionID, transaction);
    }

    public void addTransaction(String transactionID) {
        log("addTransaction -> transactionid");

        TransactionBean transactionbean = (TransactionBean) transactions.get(transactionID);

        if (transactionbean == null) {
            transactionbean = new TransactionBean();
        } else {
            transactionbean = (TransactionBean) transactions.get(transactionID);
        }

        transactionbean.setTransactionID(transactionID);
        transactions.put(transactionID, transactionbean);
    }

    public Iterator getTransactionsKeyIterator() {
        return transactions.keySet().iterator();
    }

    /**
     *  Returns Iterator with keys in sorted Order.
     */
    public Iterator getTransactionsSortedKeyIterator() {
        SortedMap transSortedMap = new TreeMap(transactions);
        Iterator iterator = transSortedMap.keySet().iterator();

        return iterator;
    }

  public int getTransactionsSize() {
		return transactions.size();
	}
  
  /**
   *  Returns only Incompleted(In progress transactions)
   */
  public String[] getInCompletedTransactionIDsArray() {
		Iterator itTemp = transactions.values().iterator();
		ArrayList arrayList = new ArrayList();
		while (itTemp.hasNext()) {
			TransactionBean element = (TransactionBean) itTemp.next();
			String status = element.getTransactionStatus();
			if (status.equals(ITransactionStatus.IN_PROGRESS + "")) {
				arrayList.add(element.getTransactionID());
			}
		}
		String strArray[] = new String[arrayList.size()];
		arrayList.toArray(strArray);
		return strArray;
	}

    private void log(String str) {
        // System.out.println("Transactions "+str);
    }
}
