package com.ibm.ccd.ui.cache;

import java.util.Map;

import com.ibm.ccd.ui.mediator.ConfigManager;

/**
 * @author vaibhav
 *
 * This class access cached object from Cachemanager. For templates,it also takes care of reloading 
 * the cached contents if cache objects are removed.It also checks if cache is disable or enable.
 */
public class CacheFactory implements ICacheFactory {
	private static CacheFactory instance = null;

	public static CacheFactory getInstance() {
		if (instance == null) {
			synchronized (ConfigManager.class) {
				instance = new CacheFactory();
			}
		}
		return instance;
	}

	/**
	 * Getting the cached objects using groupkey and key.It checks for cache enable/disable 
	 * and reloading the templates cache objects.
	 * @param groupkey
	 * @param key 
	 */
	public Object getCachedObject(String groupkey, String key) {
		Object value = null;
		CacheManager cacheManager = null;
		if (CacheManager.isCacheEnabled()) {
			try {
				// Cache related initialization
				cacheManager = CacheManager.getInstance();
			} catch (CacheNotEnabledException e) {
			}
			value = cacheManager.getFromCache(groupkey, key);

			/*
			 * Reloading the templates if not available
			 */
			if ((value == null) && groupkey.equals(CacheConstants.TEMPLATES)) {
				ConfigManager.getInstance().cacheXMLTemplates(null);
				value = cacheManager.getFromCache(groupkey, key);
			}

		} else {
			if (groupkey.equals(CacheConstants.TEMPLATES)) {
				Map templateMap = ConfigManager.getInstance()
						.getCacheConfigTemplates();
				value = templateMap.get(key);
			}
		}
		return value;
	}
}