/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
package com.ibm.ccd.ui.cache;

import java.util.Set;

import com.ibm.ccd.ui.mediator.ConfigManager;
import com.ibm.ccd.ui.util.IReference;

/**
 * Created on Jun 13, 2006
 */
public class CacheManager {
	
	//static variables
	private static CacheManager cacheManager;
	
	//Instance variables
	private CacheRegistry cacheRegistry = null;
	
	//Constructor
	private CacheManager() {
		
		cacheRegistry = CacheRegistry.getInstance();
	}
	
	/*
	 * Method to get an instance of CacheManager
	 */
	public static CacheManager getInstance() throws CacheNotEnabledException{
		
		if(! CacheManager.isCacheEnabled())
			throw new CacheNotEnabledException();
			
		if(cacheManager == null)
			cacheManager = new CacheManager();
		
		return cacheManager;
	}
	
	/*
	 * Method to put a value in to cache
	 */
	public void putInCache(String groupId, String key, Object value){
		
		this.cacheRegistry.putValue(groupId, key, value);
	}
	
	/*
	 * Method to get a value from cache
	 */
	public Object getFromCache(String groupId, String key) {
		
		return this.cacheRegistry.getValue(groupId,key);
	}
	
	/*
	 * Method to reset a particular group
	 */
	public void resetCacheValue(String groupId, String key)  {
		
		this.cacheRegistry.removeCacheKey(groupId,key);
	}

	public void resetGroup(String groupId)  {
		
		this.cacheRegistry.resetGroup(groupId);
	}

	/*
	 * Method to reset cacheRegistry
	 */
	public void resetCacheRegistry() throws CacheException{
		
		this.cacheRegistry.resetCacheRegistry();
	}
	
	public boolean isCached(String groupId, String key){
		
		Object obj = this.cacheRegistry.getValue(groupId,key);
		
		return ((obj == null) ? false : true);
	}
	
	public Set getCacheGroups(){
		
		return this.cacheRegistry.getCacheGroups();
	}
	
	public Set getCacheKeys(String groupId){
		
		return this.cacheRegistry.getCacheKeys(groupId);
	}
	
	public static boolean isCacheEnabled(){
		
		String cacheEnable = (String) ConfigManager.getInstance().getFrameworkConfig().get(IReference.CACHE_ENABLED);
		boolean enabledFlag = cacheEnable.equalsIgnoreCase("enable") ? true : false;
		
		return enabledFlag;
	}
}
