/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
package com.ibm.ccd.ui.cache;

import java.util.HashMap;
import java.util.Set;

/**
 * Created on Jun 13, 2006
 */
public class CacheRegistry {
	
	private static CacheRegistry cacheRegistry; 
	
	//To store the caching services after grouping them
	private HashMap cacheRegistryMap;
	
	private CacheRegistry(){
		
		this.cacheRegistryMap = new HashMap(10);
	}
	
	public static CacheRegistry getInstance()  {
		
		if(cacheRegistry == null)
			cacheRegistry = new CacheRegistry();

		return cacheRegistry;
	}
	
	/*
	 * Method to create a new group
	 */
	public void createGroup(String groupId) {
		
		if(this.cacheRegistryMap == null)
			cacheRegistryMap = new HashMap(10);
		
		this.cacheRegistryMap.put(groupId,new SimpleCacheService());
	}
	
	/*
	 * Method to reset a group based on its id
	 */
	public void resetGroup(String groupId) {
		
		SimpleCacheService sc = (SimpleCacheService) cacheRegistryMap.get(groupId);
		sc.resetCache();
	}
	
	/*
	 * Method to reset cache Registry
	 */
	public void resetCacheRegistry(){
		
		if(this.cacheRegistryMap != null)
			this.cacheRegistryMap = new HashMap();
	}
	
	/*
	 * Method to retrieve cached value
	 */
	public Object getValue(String groupId, String key)  {
		
		SimpleCacheService sc = (SimpleCacheService) cacheRegistryMap.get(groupId);
		
		return ((sc != null) ? sc.getCacheValue(key) : null);
	}
	
	/*
	 * Method to put a cached value
	 */
	public void putValue(String groupId, String key, Object value)  {
		
		if(!this.cacheRegistryMap.containsKey(groupId))
			createGroup(groupId);

		SimpleCacheService sc = (SimpleCacheService) cacheRegistryMap.get(groupId);
		
		if(sc != null)
			sc.putCacheValue(key,value);
	}
	
	/*
	 * Method to remove already existing group.
	 */
	public void removeGroup(String groupId)  {
		
		
		Object group = this.cacheRegistryMap.get(groupId);
		group = null;
		
		this.cacheRegistryMap.remove(groupId);
	}
	
	public Set getCacheGroups() {
		
		return this.cacheRegistryMap.keySet();
	}
	
	public Set getCacheKeys(String groupId) {
		
		SimpleCacheService sc = (SimpleCacheService) this.cacheRegistryMap.get(groupId);
		return sc.getCacheKeys();
	}
	
	public void removeCacheKey(String groupId,String key)  {
		
		SimpleCacheService sc = (SimpleCacheService) this.cacheRegistryMap.get(groupId);
		sc.removeKey(key);
		
		if(getCacheKeys(groupId).size() == 0){
			removeGroup(groupId);
		}
	}

}
