
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.attributes;

import java.util.HashMap;


/*
 * Represents the supported types of the attributes
 */
public abstract class AttributeTypes {
    // mapping from Attribute type(index) to the string names
    public static final String[] num2String = {
            "UN_SUPPORTED", "STRING", "INTEGER", "NUMBER", "DATE", "FLAG",
            "TIMEZONE", "CURRENCY", "PASSWORD", "PERIOD", "SEQUENCE",
            "RELATIONSHIP", "BINARY", "IMAGE", "THUMBNAIL_IMAGE", "URL",
            "IMAGE_URL", "THUMBNAIL_IMAGE_URL", "STRING_ENUMERATION",
            "NUMBER_ENUMERATION", "LOOKUP_TABLE", "GROUPING"
        };

    // reverse mapping (String representation to attribute ints (index))
    public static HashMap str2Num;
    public static final int UNSUPPORTED = 0;
    public static final int STRING = 1;
    public static final int INTEGER = 2;
    public static final int NUMBER = 3;
    public static final int DATE = 4;
    public static final int FLAG = 5;
    public static final int TIMEZONE = 6;
    public static final int CURRENCY = 7;
    public static final int PASSWORD = 8;
    public static final int PERIOD = 9;
    public static final int SEQUENCE = 10;
    public static final int RELATIONSHIP = 11;
    public static final int BINARY = 12;
    public static final int IMAGE = 13;
    public static final int THUMB_NAIL_IMAGE = 14;
    public static final int URL = 15;
    public static final int IMAGE_URL = 16;
    public static final int THUMB_NAIL_IMAGE_URL = 17;
    public static final int STRING_ENUMERATION = 18;
    public static final int NUMBER_ENUMERATION = 19;
    public static final int LOOKUP_TABLE = 20;
    public static final int GROUP = 21;

    /*
     * Returns the attribute numeric rep. from the given string rep.
     *
     */
    public static int getType(String name) {
        // if the map is not populated .. 
        if (str2Num == null) {
            synchronized (AttributeTypes.class) {
                if (str2Num == null) {
                    str2Num = new HashMap();

                    for (int i = 0; i < num2String.length; ++i) {
                        str2Num.put(num2String[i], new Integer(i));
                    }
                }
            }
        }

        try {
            return ((Integer) str2Num.get(name)).intValue();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return UNSUPPORTED;
    }

    /*
     * Returns the attribute String rep. from the given numeric rep.
     *
     */
    public static String getString(int type) {
        return (type < num2String.length) ? num2String[type] : num2String[0];
    }
}
