
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.attributes;

import java.util.ArrayList;

import com.ibm.ccd.ui.dataobjects.FileUploadResource;


/*
 * Represents the Binary attribute to hold generically
 * any binary file uploads.
 * The value is represented as the FileUploadResource data type
 */
public class BinaryAttribute extends Attribute {
    // default value 
    protected FileUploadResource defaultValue;
    protected String localPath;

    public BinaryAttribute(String name, boolean editable,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);
        this.valueList = new ArrayList(getMaximumOccurrence());
        valueList.add((defaultValue != null) ? defaultValue
                                             : new FileUploadResource());
    }

    public BinaryAttribute(String name, FileUploadResource defaultValue,
        boolean editable, int minimumOccurrence, int maximumOccurrence,
        String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);
        this.defaultValue = defaultValue;
        this.valueList = new ArrayList(getMaximumOccurrence());
        valueList.add((defaultValue != null) ? defaultValue
                                             : new FileUploadResource());
    }

    public BinaryAttribute(String name, FileUploadResource defaultValue,
        boolean editable, int minimumLength, int maximumLength,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, editable, minimumLength, maximumLength, minimumOccurrence,
            maximumOccurrence, displayName);
        this.defaultValue = defaultValue;
        this.valueList = new ArrayList(getMaximumOccurrence());
        valueList.add((defaultValue != null) ? defaultValue
                                             : new FileUploadResource());
    }

    public int getType() {
        return AttributeTypes.BINARY;
    }

    public FileUploadResource getDefaultValue() {
        return defaultValue;
    }

    public void setDefaultValue(FileUploadResource defaultValue) {
        this.defaultValue = defaultValue;
    }

    public FileUploadResource getValue() {
        if ((getValueList() == null) || (getValueList().size() == 0)) {
            throw new RuntimeException("Value not set for" + getName());
        }

        return (FileUploadResource) valueList.get(0);
    }

    public FileUploadResource[] getValues() {
        if ((valueList == null) || (valueList.size() == 0)) {
            return null;
        }

        FileUploadResource[] resources = new FileUploadResource[valueList.size()];

        for (int i = 0; i < resources.length; ++i) {
            resources[i] = (FileUploadResource) valueList.get(i);
        }

        return resources;
    }

    public void setValue(FileUploadResource value) {
        if (getCurrentOccurrence() > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for" +
                getName());
        }

        if (value == null) {
            return;
        }

        if (isMultiValued()) {
            valueList.add(value);
        } else {
            valueList.add(0, value);
        }
    }

    public void setValue(int occurrence, FileUploadResource value) {
        if (value == null) {
            return;
        }

        if (occurrence > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for" +
                getName());
        }

        if (isMultiValued()) {
            valueList.add(occurrence, value);
        } else {
            valueList.add(0, value);
        }
    }

    public void setValues(FileUploadResource[] values) {
        if ((values == null) || (values.length == 0)) {
            return;
        }

        if (values.length > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            // clear the list
            valueList.clear();

            for (int i = 0; i < values.length; ++i) {
                valueList.add(values[i]);
            }
        } else {
            valueList.add(0, values[0]);
        }
    }

    public void printMe() {
        super.printMe();
        System.out.println("\t\t DefVal:" + defaultValue);
    }

    /**
     * @return
     */
    public String getLocalPath() {
        return localPath;
    }

    /**
     * @param string
     */
    public void setLocalPath(String string) {
        localPath = string;
    }
}
