/*
 * Licensed Materials: Property of IBM
 * "Restricted Materials of IBM"
 * 5724-I68.
 * (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/*
 * Created on Feb 8, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.ibm.ccd.ui.dataobjects.attributes;

import java.util.HashMap;

import com.ibm.ccd.ui.util.UIUtils;

/**
 * @author administrator
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class GroupAttribute extends Attribute {
    private Attribute[] attributes;
    private Attribute[] defaultAttributes;
    private int groupIndex = -1;
    private HashMap attributeMap;
        
    private String groupAppendPath = "";

    public GroupAttribute(
                          String name,
                          boolean editable,
                          int minimumOccurrence,
                          int maximumOccurrence,
                          Attribute[] attributes,
                          String displayName) {
        super(
              name,
              editable,
              minimumOccurrence,
              maximumOccurrence,
              displayName);
        this.attributes = attributes;
        groupIndex = 0;
		defaultAttributes = (this.isMultiValued())?cloneValue(attributes):attributes;
        groupIndex = -1;
        setValue(attributes);
        hashName = this.getHashName()+"";
        prepareAttributeMap();
    }

    private void prepareAttributeMap()
    {
        attributeMap = new HashMap();
        for (int i=0; i<attributes.length; i++)
        {
            attributeMap.put(attributes[i].getName(), new Integer(i));
        }
    }
	
    public int getType() {
        return AttributeTypes.GROUP;
    }

    /**
     * @return attributes
     */
    public Attribute[] getAttributes() {
        return attributes;
    }

    /**
     * @param attributes
     */
    public void setAttributes(Attribute[] attributes) {
        this.attributes = attributes;
    }

    public boolean hasValue(int index) {
        if (index < valueList.size()) {
            return true;
        } else {
            return false;
        }
    }
    public Attribute[] getValue() {
        if ((getValueList() == null) || (getValueList().size() == 0)) {
            throw new RuntimeException("Value not set for " + getName());
        }

        return (Attribute[]) valueList.get(0);
    }

    public Attribute[] getValue(int index) {
        if ((getValueList() == null) || (getValueList().size() == 0)) {
            throw new RuntimeException("Value not set for " + getName());
        }

        return (Attribute[]) valueList.get(index);
    }

    public Attribute getValue(int index, String name) {
        Attribute[] attributes = getValue(index);

        for (int i = 0; i < attributes.length; i++) {
            if (attributes[i].getName().equals(name)) {
                return attributes[i];
            }
        }

        return null;
    }

    public Attribute[][] getValues() {
        int groupLength = getValueList().size();
        Attribute[][] values = new Attribute[groupLength][];

        for (int i = 0; i < groupLength; i++) {
            values[i] = (Attribute[]) getValueList().get(i);
        }

        return values;
    }

    public void setValue(Attribute[] attributes) {
        if (getCurrentOccurrence() > getMaximumOccurrence()) {
            throw new RuntimeException(
                                       "Occurrence is exceeded for " + getName());
        }

        if (attributes == null) {
            return;
        }
        setParentGroup(attributes);

        if (isMultiValued()) {
            valueList.add(attributes);
        } else {
            valueList.add(0, attributes);
        }
    }

    public void setValues(Attribute[][] values) {
        if ((values == null) || (values.length == 0)) {
            return;
        }

        if (values.length > getMaximumOccurrence()) {
            throw new RuntimeException(
                                       "Occurrence is exceeded for " + getName());
        }

        if (isMultiValued()) {
            valueList.clear();

            for (int i = 0; i < values.length; ++i) {
                setValue(values[i]);
            }
        } else {
            setValue(values[0]);
        }
    }

    /**
     * @see com.ibm.ccd.ui.dataobjects.attributes.Attribute#removeValue(int)
     */
    public void removeValue(int index) {
        getValueList().remove(index);
    }

    public Attribute[] cloneValue() {
        Attribute[] clonedAttributes = new Attribute[defaultAttributes.length];

        for (int i = 0; i < defaultAttributes.length; i++) {
            clonedAttributes[i] =
                (Attribute) UIUtils.copy(defaultAttributes[i]);
            if (clonedAttributes[i] instanceof GroupAttribute) {
                ((GroupAttribute) clonedAttributes[i]).setGroupIndex(0);
                //resetGroupIndex()
            }
        }
        return clonedAttributes;
    }

    public Attribute[] cloneValue(Attribute[] attribs) {
        Attribute[] clonedAttributes = new Attribute[attribs.length];

        for (int i = 0; i < attribs.length; i++) {
            clonedAttributes[i] = (Attribute) UIUtils.copy(attribs[i]);
            if (clonedAttributes[i] instanceof GroupAttribute) {
                ((GroupAttribute) clonedAttributes[i]).setGroupIndex(0);
            }
        }
        return clonedAttributes;
    }

    /**
     * checks whether the group has subgroups
     * @return
     */
    public boolean hasNextGroup() {
        if (groupIndex < valueList.size() - 1) {
            return true;
        }

        return false;
    }

    /**
     * checks whether the group has subgroup and return its subgroup
     * @return
     */
    public Attribute[] nextGroup() {

        if (hasNextGroup()) {
            groupIndex++;
            Attribute[] attribs = this.getValue(groupIndex);
            //resetGroupIndex(attribs);
            return attribs;
        } else {
            groupIndex = 0;
            return this.getValue(groupIndex);
        }
		
    }

    public void setGroupIndex(int groupIndex) {
        this.groupIndex = groupIndex;
    }
    /**
     * @return
     */
    public int getGroupIndex() {
        if(groupIndex != -1)
            return groupIndex;
        else 
            return 0;	
    }
    
    public int getGroupIndexPos() {
    	if(groupIndex < 0) {
    		return 0;
    	} else  {
    		return groupIndex + 1;
    	}
    }
    
    private void setParentGroup(Attribute[] attributes) {
        for (int i = 0; i < attributes.length; i++) {
            attributes[i].setParentGroup(this);
        }
    }

    public void resetGroupIndex(Attribute[] attribs) {
        if (attribs == null)
            return;
        for (int i = 0; i < attribs.length; i++) {
            if (attribs[i] instanceof GroupAttribute) {
                GroupAttribute groupAttrib = (GroupAttribute) attribs[i];
                groupAttrib.setGroupIndex(0);
            }
        }
    }
	
	
    /**
     * @return
     */
    public String getGroupAppendPath() {
        return groupAppendPath;
    }

    /**
     * @param string
     */
    public void setGroupAppendPath(String string) {
        groupAppendPath = string;
    }

    /**
     * @return Returns the attributeMap.
     */
    public HashMap getAttributeMap() {
        return attributeMap;
    }

}
