
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.attributes;

import java.util.ArrayList;

import com.ibm.ccd.ui.util.UIUtils;



/*
 * Represents the integer
 */
public class IntegerAttribute extends Attribute {
	private static final String STR_EMPTY="";
	
    private String defaultValue = "";

    public IntegerAttribute(String name, boolean editable,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);
        removeValues();
        setValue(STR_EMPTY);
    }

    public IntegerAttribute(String name, String defaultValue, boolean editable,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);
        if(defaultValue.equals("null")){
        	defaultValue = "";
        }
        this.defaultValue = defaultValue;
        removeValues();
        setValue(defaultValue);
    }

    public IntegerAttribute(String name, String defaultValue, boolean editable,
        int minimumLength, int maximumLength, int minimumOccurrence,
        int maximumOccurrence, String displayName) {
        super(name, editable, minimumLength, maximumLength, minimumOccurrence,
            maximumOccurrence, displayName);
        this.defaultValue = defaultValue;
        removeValues();
        setValue(defaultValue);
    }

    public int getType() {
        return AttributeTypes.INTEGER;
    }

    public String getValue() {
        if ((getValueList() == null) || (getValueList().size() == 0)) {
            throw new RuntimeException("Value not set for " + getName());
        }

        return (String)valueList.get(0);
    }

	private String getValue(Object intObject) {
		if (intObject == null) {
			return STR_EMPTY;
		}

//		if (intObject instanceof Long) {
			return intObject + STR_EMPTY;
//		}
		// throw exception when intObject is not Long
//		throw new IllegalArgumentException();
	}
	
    public String[] getValues() {
        return getValues(valueList);
    }
	
	private String[] getValues(ArrayList arrayList) {
		if ((arrayList == null) || (arrayList.size() == 0)) {
			return null;
		}

		int arraySize = arrayList.size();
		String[] stringArray = new String[arraySize];

		for (int i = 0; i < arraySize; ++i) {
			stringArray[i] = ((arrayList.get(i) != null)
				? ((String)arrayList.get(i)): "");
		}

		return stringArray;
		
	}

    public void setValue(String value) {
        if (getCurrentOccurrence() > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }
		
		if (isMultiValued()) {
			if(!UIUtils.isEmpty(value)) {
				valueList.add(value);
			} else {
				valueList.add(defaultValue);
			}	
		} else {
			if(!UIUtils.isEmpty(value)) {
				valueList.add(0,value);
			} else {
				//when default value is empty put null reference
				//otherwise put default value
				if(UIUtils.isEmpty(defaultValue)) 
					valueList.add(0,null);
				else
					valueList.add(0,defaultValue);
			}	
		}
    }

    public void setValue(int occurrence, String value) {
        if (occurrence > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
        	if(!UIUtils.isEmpty(value)) {
            	valueList.add(occurrence, value);
        	} else {
            	//when default value is empty put null reference
            	//otherwise put default value
            	if(UIUtils.isEmpty(defaultValue)) 
	            	valueList.add(occurrence,null);
            	else
            		valueList.add(occurrence,defaultValue);
            }	
        } else {
			if(!UIUtils.isEmpty(value)) {
            	valueList.add(0, value);
			} else {
				//when default value is empty put null reference
				//otherwise put default value
				if(UIUtils.isEmpty(defaultValue)) 
					valueList.add(0,null);
				else
					valueList.add(0, defaultValue);
			}	
        }
    }

    public void setValues(String[] values) {
        if ((values == null) || (values.length == 0)) {
            return;
        }

        if (values.length > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.clear();

            for (int i = 0; i < values.length; ++i) {
				setValue(i,values[i]);
            }
        } else {
        		setValue(0,values[0]);
        }
    }

    public String getDefaultValue() {
        return defaultValue;
    }

    public void setDefaultValue(String defValue) {
        defaultValue = defValue;
    }
	
    public void printMe() {
        super.printMe();
        System.out.println("\t\tDefVal:" + defaultValue);
    }
}
