
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.attributes;

import java.util.ArrayList;

import com.ibm.ccd.ui.dataobjects.NVPair;
import com.ibm.ccd.ui.util.UIUtils;


/*
 * Represents the LookupTable as a list of name value pairs
 */
public class LookupTableAttribute extends Attribute {
    // the look up list represented as the key value pair
    // and array is used to keep the order
    private NVPair[] lookupTable;
    private String defaultValue;
    private String lookupTableName;

    public LookupTableAttribute(String name, NVPair[] lookupTable,
        boolean editable, int minimumOccurrence, int maximumOccurrence,
        String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);

        this.lookupTable = lookupTable;
        this.valueList = new ArrayList(getMaximumOccurrence());
    }

    public LookupTableAttribute(String name, NVPair[] lookupTable,
        String defaultValue, boolean editable, int minimumOccurrence,
        int maximumOccurrence, String displayName, String lookupTableName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);

        this.defaultValue = defaultValue;
        this.lookupTable = lookupTable;
        this.lookupTableName=lookupTableName;
        this.valueList = new ArrayList(getMaximumOccurrence());
    }

    public LookupTableAttribute(String name, NVPair[] lookupTable,
        String defaultValue, boolean editable, int minimumLength,
        int maximumLength, int minimumOccurrence, int maximumOccurrence,
        String displayName) {
        super(name, editable, minimumLength, maximumLength, minimumOccurrence,
            maximumOccurrence, displayName);

        this.lookupTable = lookupTable;
        this.defaultValue = defaultValue;
        this.valueList = new ArrayList(getMaximumOccurrence());
    }

    public int getType() {
        return AttributeTypes.LOOKUP_TABLE;
    }

    public String getValue() {
        if ((getValueList() == null) || (getValueList().size() == 0)) {
            throw new RuntimeException("Value not set for " + getName());
        }

        return (String) valueList.get(0);
    }

    public String[] getValues() {
        if ((valueList == null) || (valueList.size() == 0)) {
            return null;
        }

        return UIUtils.getStringArray(valueList);
    }

    public void setValue(String value) {
        if (value == null) {
            return;
        }

        if (getCurrentOccurrence() == getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.add(value);
        } else {
            valueList.add(0, value);
        }
    }

    public void setValue(int occurrence, String value) {
        if (value == null) {
            return;
        }

        if (occurrence > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.add(occurrence, value);
        } else {
            valueList.add(0, value);
        }
    }

    public void setValues(String[] values) {
        if ((values == null) || (values.length == 0)) {
            return;
        }

        if (values.length > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.clear();

            for (int i = 0; i < values.length; ++i) {
                valueList.add(values[i]);
            }
        } else {
            valueList.add(0, values[0]);
        }
    }

    public String getDefaultValue() {
    	if(defaultValue==null){
			defaultValue="";
    	}
        return defaultValue;
    }

    public void setDefaultValue(String defaultValue) {
        this.defaultValue = defaultValue;
    }

    public NVPair[] getLookupTable() {
        return lookupTable;
    }

    public void setLookupTable(NVPair[] lookupTable) {
        this.lookupTable = lookupTable;
    }

    public void printMe() {
        super.printMe();

        if (lookupTable != null) {
            for (int i = 0; i < lookupTable.length; ++i) {
                if (i == 0) {
                    System.out.print("\t\tLookupEnum : ");
                }

                System.out.print(lookupTable[i] + " ");
            }
        }

        System.out.println(" \nDefault Value:" + getDefaultValue());
    }
	/**
	 * @return
	 */
	public String getLookupTableName() {
		return lookupTableName;
	}

	/**
	 * @param string
	 */
	public void setLookupTableName(String string) {
		lookupTableName = string;
	}

}
