/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package com.ibm.ccd.ui.dataobjects.attributes;

import java.util.ArrayList;

import org.w3c.dom.Element;

import com.ibm.ccd.ui.mediator.ISoapMessageKeys;
import com.ibm.ccd.ui.mediator.XMLHelper;
import com.ibm.ccd.ui.util.UIUtils;

/*
 * Represents the Period Attribute (start and end of time period(s))
 * This attribute is modelled to represent the start and end 
 * periods by sub-attribues
 */

public class PeriodAttribute extends Attribute {

	// represents the start perdiod
	private String startPeriod = "";
	//	// represents the end period
	private String endPeriod = "";
	// Start date end date are sepearated by |

	public int getType() {
		return AttributeTypes.PERIOD;
	}

	/**
	 * @param name
	 * @param editable
	 * @param minimumOccurrence
	 * @param maximumOccurrence
	 * @param displayName
	 */
	public PeriodAttribute(
		String name,
		boolean editable,
		int minimumOccurrence,
		int maximumOccurrence,
		String displayName) {

		super(
			name,
			editable,
			minimumOccurrence,
			maximumOccurrence,
			displayName);

		valueList = new ArrayList();
		setValue(startPeriod,endPeriod);
	}

	/**
	 * @param name
	 * @param startPeriod
	 * @param endPeriod
	 * @param editable
	 * @param minimumOccurrence
	 * @param maximumOccurrence
	 * @param displayName
	 */
	public PeriodAttribute(
		String name,
		String startPeriod,
		String endPeriod,
		boolean editable,
		int minimumOccurrence,
		int maximumOccurrence,
		String displayName) {

		super(
			name,
			editable,
			minimumOccurrence,
			maximumOccurrence,
			displayName);

		this.startPeriod = startPeriod;
		this.endPeriod = endPeriod;
		valueList = new ArrayList();
		setValue(this.startPeriod,this.endPeriod);
	}

	/**
	 * @param name
	 * @param startPeriod
	 * @param endPeriod
	 * @param editable
	 * @param minimumLength
	 * @param maximumLength
	 * @param minimumOccurrence
	 * @param maximumOccurrence
	 * @param displayName
	 */
	public PeriodAttribute(
		String name,
		String startPeriod,
		String endPeriod,
		boolean editable,
		int minimumLength,
		int maximumLength,
		int minimumOccurrence,
		int maximumOccurrence,
		String displayName) {

		super(
			name,
			editable,
			minimumLength,
			maximumLength,
			minimumOccurrence,
			maximumOccurrence,
			displayName);

		this.startPeriod = startPeriod;
		this.endPeriod = endPeriod;
		setValue(this.startPeriod,this.startPeriod);
	}

	/**
	 * @param startDate
	 * @param endDate
	 */
	public void setValue(String startDate, String endDate) {
		if (getCurrentOccurrence() > getMaximumOccurrence()) {
			throw new RuntimeException(
				"Occurrence is exceeded for " + getName());
		}

		if (isMultiValued()) {

			valueList.add(startDate + "|" + endDate);
		} else {
			valueList.add(0, startDate + "|" + endDate);
		}

	}

	/**
	 * @return
	 */
	public String getValue() {
		if (getValueList() == null || getValueList().size() == 0) {
			throw new RuntimeException("Value not set for " + getName());
		}

		return (String) valueList.get(0);
	}

	/**
	 * @param index
	 * @return
	 */
	public String getStartDate(int index) {
		String periodStr = (String) valueList.get(index);
		return getStartDateString(periodStr);
	}

	/**
	 * @param index
	 * @return
	 */
	public String getEndDate(int index) {
		String periodStr = (String) valueList.get(index);
		return getEndDateString(periodStr);
	}

	/**
	 * @param period
	 * @return
	 */
	private String getStartDateString(String period) {
		if(UIUtils.isEmpty(period)){
			return "" ;
		}
		return period.substring(0, period.indexOf('|'));
	}

	/**
	 * @param period
	 * @return
	 */
	private String getEndDateString(String period) {
		if(UIUtils.isEmpty(period)){
			return "";
		}
		return period.substring(period.indexOf('|') + 1);
	}

	public void printMe() {

		super.printMe();
		if (startPeriod != null) {
			System.out.print("\t\tStart Period: " + startPeriod);
		} else {
			System.out.println("\t\tStart or End Period Not Available. ");
		}
	}

}
