
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.attributes;

import com.ibm.ccd.ui.dataobjects.NVPair;


/*
 * Represents the Realtion ship WPC attribute
 * The value relates to is modelled as a NVpair
 * name being the relation and value the related value.
 */
public class RelationshipAttribute extends Attribute {
    // relations 
    private String[] relations;
    private NVPair defaultValue;

    public RelationshipAttribute(String name, boolean editable,
        String[] relations, int minimumOccurrence, int maximumOccurrence,
        String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);

        this.relations = relations;
        valueList.clear();
        valueList.add(defaultValue);
    }

    public RelationshipAttribute(String name, String[] relations,
        NVPair defaultValue, boolean editable, int minimumOccurrence,
        int maximumOccurrence, String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);

        this.relations = relations;
        this.defaultValue = defaultValue;
        valueList.clear();
        valueList.add(defaultValue);
    }

    public RelationshipAttribute(String name, String[] relations,
        NVPair defaultValue, boolean editable, int minimumLength,
        int maximumLength, int minimumOccurrence, int maximumOccurrence,
        String displayName) {
        super(name, editable, minimumLength, maximumLength, minimumOccurrence,
            maximumOccurrence, displayName);

        this.relations = relations;
        this.defaultValue = defaultValue;
        valueList.clear();
        valueList.add(defaultValue);
    }

    public int getType() {
        return AttributeTypes.RELATIONSHIP;
    }

    public NVPair getValue() {
        if ((getValueList() == null) || (getValueList().size() == 0)) {
            throw new RuntimeException("Value not set for " + getName());
        }

        return (NVPair) valueList.get(0);
    }

    public NVPair[] getValues() {
        if ((valueList == null) || (valueList.size() == 0)) {
            return null;
        }

        int arraySize = valueList.size();
        NVPair[] nvpArray = new NVPair[arraySize];

        for (int i = 0; i < arraySize; ++i) {
            nvpArray[i] = (NVPair) valueList.get(i);
        }

        return nvpArray;
    }

    public void setValue(NVPair value) {
        if (value == null) {
            return;
        }

        if (getCurrentOccurrence() > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.add(value);
        } else {
            valueList.add(0, value);
        }
    }

    public void setValue(int occurrence, String value) {
        if (value == null) {
            return;
        }

        if (occurrence > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.add(occurrence, value);
        } else {
            valueList.add(0, value);
        }
    }

    public void setValues(NVPair[] values) {
        if ((values == null) || (values.length == 0)) {
            return;
        }

        if (values.length > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.clear();

            for (int i = 0; i < values.length; ++i) {
                valueList.add(values[i]);
            }
        } else {
            valueList.add(0, values[0]);
        }
    }

    public NVPair getDefaultValue() {
        return defaultValue;
    }

    public String[] getRelations() {
        return relations;
    }

    public void setDefaultValue(NVPair defaultValue) {
        this.defaultValue = defaultValue;
    }

    public void setRelations(String[] relations) {
        this.relations = relations;
    }

    public void printMe() {
        super.printMe();

        if (relations != null) {
            System.out.print("\t\tRealtion. Enum: ");

            for (int i = 0; i < relations.length; ++i) {
                System.out.print("" + relations[i]);
            }

            System.out.println("\n\t\tDef.Val:" + defaultValue);
        }
    }
}
