
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.attributes;

import com.ibm.ccd.ui.util.UIUtils;



/*
 * Represents the String value(s)
 * limitations - no format/pattern support is available.
 */
public class StringAttribute extends Attribute {
    private String defaultValue;

    public StringAttribute(String name, boolean editable,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);
        removeValues();
        setValue((defaultValue != null) ? defaultValue : "");
    }

    public StringAttribute(String name, String defaultValue, boolean editable,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);
        this.defaultValue = defaultValue;
        removeValues();
        setValue((defaultValue != null) ? defaultValue : "");
    }

    public StringAttribute(String name, String defaultValue, boolean editable,
        int minimumLength, int maximumLength, int minimumOccurrence,
        int maximumOccurrence, String displayName) {
        super(name, editable, minimumLength, maximumLength, minimumOccurrence,
            maximumOccurrence, displayName);
        this.defaultValue = defaultValue;
        removeValues();
        setValue((defaultValue != null) ? defaultValue : "");
    }

    public int getType() {
        return AttributeTypes.STRING;
    }

    public String getValue() {
        if ((getValueList() == null) || (getValueList().size() == 0)) {
            throw new RuntimeException("Value not set for " + getName());
        }

        return (String) valueList.get(0);
    }

    public String[] getValues() {
        return UIUtils.getStringArray(valueList);
    }

    public void setValue(String value) {
        if (getCurrentOccurrence() > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (value == null) {
            //throw new IllegalArgumentException();
            return;
        }

        if (isMultiValued()) {
            valueList.add(value);
        } else {
            valueList.add(0, value);
        }
    }

    public void setValues(String[] values) {
        if ((values == null) || (values.length == 0)) {
            return;
        }

        if (values.length > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.clear();

            for (int i = 0; i < values.length; ++i) {
                valueList.add(values[i]);
            }
        } else {
            valueList.add(0, values[0]);
        }
    }

    public void setValue(int occurrence, String value) {
        if (occurrence > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (value == null) {
            return;
        }

        if (isMultiValued()) {
            valueList.add(occurrence, value);
        } else {
            valueList.add(0, value);
        }
    }

    public String getDefaultValue() {
        return defaultValue;
    }

    public void setDefaultValue(String string) {
        defaultValue = string;
    }

    public void printMe() {
        super.printMe();
        System.out.println("\t\tDef.Val:" + defaultValue);
    }
}
