
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.attributes;

import com.ibm.ccd.ui.util.UIUtils;



/*
 * Represents the String enumeration
 * the values are modelled as an array of strings
 */
public class StringEnumAttribute extends StringAttribute {
    // string enum array
    protected String[] enumValues;

    public StringEnumAttribute(String name, String[] enumParam, boolean editable,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);
        this.enumValues = enumParam;
    }

    public StringEnumAttribute(String name, String[] enumParam, String defaultValue,
        boolean editable, int minimumOccurrence, int maximumOccurrence,
        String displayName) {
        super(name, defaultValue, editable, minimumOccurrence,
            maximumOccurrence, displayName);

        this.enumValues = enumParam;
    }

    public StringEnumAttribute(String name, String[] enumValues, String defaultValue,
        boolean editable, int minimumLength, int maximumLength,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, defaultValue, editable, minimumLength, maximumLength,
            minimumOccurrence, maximumOccurrence, displayName);

        this.enumValues = enumValues;
    }

    public int getType() {
        return AttributeTypes.STRING_ENUMERATION;
    }

    protected void checkValue(String value) {
        if ((enumValues == null) || (enumValues.length == 0)) {
            throw new IllegalArgumentException("Enum is Empty/Not defined" +
                "for attribute" + getName());
        }

        for (int i = 0; i < enumValues.length; ++i) {
            if (enumValues[i].equals(value)) {
                return;
            }
        }

        throw new IllegalArgumentException("Value Not defined in Enum" + value +
            "for attribute" + getName());
    }

    public void setValue(String value) {
        if (value == null) {
            return;
        }

        //checkValue(value);
        super.setValue(value);
    }

    public void setValue(int occurrence, String value) {
        if (value == null) {
            return;
        }

        //checkValue(value);
        super.setValue(occurrence, value);
    }

    public void setValues(String[] values) {
        if ((values == null) || (values.length == 0)) {
            return;
        }

        for (int i = 0; i < values.length; ++i) {
            if (UIUtils.isEmpty(values[i])) {
                continue;
            }

            //	checkValue(values[i]);
        }

        super.setValues(values);
    }

    public String[] getEnumValues() {
        return enumValues;
    }

    public void setEnumValues(String[] enumValues) {
        this.enumValues = enumValues;
    }

    public void printMe() {
        super.printMe();

        if (enumValues != null) {
            for (int i = 0; i < enumValues.length; ++i) {
                if (i == 0) {
                    System.out.print("\t\tEnum : ");
                }

                System.out.print(enumValues[i] + " ");
            }
        }

        System.out.println(" Def.Value:" + getDefaultValue());
    }
}
