
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.attributes;

import com.ibm.ccd.ui.dataobjects.NVPair;
import com.ibm.ccd.ui.mediator.ConfigManager;


/*
 * Represents the  Timezone attributes
 * The timezone list is configured in the timezones.xml file, which will be parsed by
 * the config.Manager.
 * limitation : No NLS support is added currently .
 * Improvement : Dependecy on config. Manger can be avioded
 */
public class TimeZoneAttribute extends LookupTableAttribute {
    protected static NVPair[] timeZoneEnum;

    public TimeZoneAttribute(String name, boolean editable,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, getTimeZoneEnum(), editable, minimumOccurrence,
            maximumOccurrence, displayName);
        valueList.add(getTimeZoneEnum()[0].getValue());
    }

    public TimeZoneAttribute(String name, String defaultValue,
        boolean editable, int minimumOccurrence, int maximumOccurrence,
        String displayName) {
        super(name, getTimeZoneEnum(), defaultValue, editable,
            minimumOccurrence, maximumOccurrence, displayName,null);
        valueList.add(defaultValue);
    }

    public TimeZoneAttribute(String name, String defaultValue,
        boolean editable, int minimumLength, int maximumLength,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, getTimeZoneEnum(), defaultValue, editable, minimumLength,
            maximumLength, minimumOccurrence, maximumOccurrence, displayName);
        valueList.add(defaultValue);
    }

    public int getType() {
        return AttributeTypes.TIMEZONE;
    }

    public static NVPair[] getTimeZoneEnum() {
        if (timeZoneEnum == null) {
            synchronized (TimeZoneAttribute.class) {
                if (timeZoneEnum == null) {
                    // get the values from the config manager
                    timeZoneEnum = ConfigManager.getInstance().getTimeZones();
                }
            }
        }

        return timeZoneEnum;
    }
}
