
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.attributes;

import java.net.MalformedURLException;
import java.net.URL;


/*
 * Represents the url value(s)
 * limitations - no format/pattern support is available.
 *                                 - real time validation is not enabled for the URL
 */
public class URLAttribute extends Attribute {
    private String defaultValue;

    public URLAttribute(String name, boolean editable, int minimumOccurrence,
        int maximumOccurrence, String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);
        removeValues();
		
		setValue(null);
    }

    public URLAttribute(String name, String defaultValue, boolean editable,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);

        this.defaultValue = defaultValue;
        removeValues();
        if (defaultValue != null) {
        	        	
            setValue(defaultValue);
        } else {
        	setValue(null);
        }
    }

    public URLAttribute(String name, String defaultValue, boolean editable,
        int minimumLength, int maximumLength, int minimumOccurrence,
        int maximumOccurrence, String displayName) {
        super(name, editable, minimumLength, maximumLength, minimumOccurrence,
            maximumOccurrence, displayName);

        this.defaultValue = defaultValue;
        removeValues();
        if (defaultValue != null) {
            setValue(defaultValue);
        } else {
        	setValue(null);
        }
    }

    public int getType() {
        return AttributeTypes.URL;
    }

    public String getValue() {
        if ((getValueList() == null) || (getValueList().size() == 0)) {
            throw new RuntimeException("Value not set for " + getName());
        }
		
        return (valueList.get(0)!=null) ? (String)valueList.get(0) : "";
    }

    public String[] getValues() {
        if ((getValueList() == null) || (getValueList().size() == 0)) {
            throw new RuntimeException("Value not set for " + getName());
        }

        int arrSize = valueList.size();
        String[] urls = new String[arrSize];

        for (int i = 0; i < arrSize; ++i) {
            urls[i] = (valueList.get(i)!=null) ? (String)valueList.get(i) : null;
        }

        return urls;
    }

    public void setValue(String value) {

        if (getCurrentOccurrence() > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.add(value);
        } else {
            valueList.add(0, value);
        }
    }

    public void setValue(int occurrence, String value) {

        if (occurrence > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded");
        }

        if (isMultiValued()) {
            valueList.add(occurrence, value);
        } else {
            valueList.add(0, value);
        }
    }

    public void setValues(String[] values) {
        if ((values == null) || (values.length == 0)) {
            return;
        }

        if (values.length > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.clear();

            for (int i = 0; i < values.length; ++i) {
                valueList.add(values[i]);
            }
        } else {
            valueList.add(0, values[0]);
        }
    }

    public String getDefaultValue() {
        return defaultValue;
    }

    public void printMe() {
        super.printMe();
        System.out.println("\t\tDef.Val:" + defaultValue);
    }
}
