
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package com.ibm.ccd.ui.dataobjects.items;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.StringTokenizer;

import org.apache.jetspeed.portlet.PortletRequest;
import org.w3c.dom.Node;

import com.ibm.ccd.ui.dataobjects.FileUploadResource;
import com.ibm.ccd.ui.dataobjects.NVPair;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.AttributeTypes;
import com.ibm.ccd.ui.dataobjects.attributes.BinaryAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.CurrencyAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.DateAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.FlagAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.GroupAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ImageAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ImageURLAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.IntegerAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.LookupTableAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberEnumAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.PasswordAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.PeriodAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.RelationshipAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringEnumAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ThumbnailImageAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ThumbnailImageURLAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.TimeZoneAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.URLAttribute;
import com.ibm.ccd.ui.dataobjects.workflow.CollaborationArea;
import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.mediator.WPCMediator;
import com.ibm.ccd.ui.portlets.massupdate.IMassUpdate;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;
import com.ibm.ccd.ui.validator.IValidatorMessage;
import com.ibm.ccd.ui.validator.ValidationHelper;

/**
 *  Represents the WPC Catalog Item/Spec
 *  The CatalogItem must have catalogID and itemID (PrimaryKey) values and composes several Attributes
 */

// TODO once the dyanamic rendereing enabled the CatalogItem should be 
// reorganized to accommodate views and configurable JSPS
public class CatalogItem implements ICatalogItem,Serializable {
	// parent category ID
	private String prentCtgyID;

	// catalogID
	private String catalogID;

	// CollabrationArea
	private CollaborationArea collabArea;

	// CatalogVersion
	private String ctgVersion;

	// Item unique indentifier, likely to be the primaryKey of the item
	private String itemID;
	private Attribute primaryKeyAttr = null;

	// List of attributes
	private Attribute[] attributes;

	// Attributes list is re-organized with map being key as the attribute name and value as the attribute
	// which helps in retreiving the attribute based on the name, rather than iterating on the array 
	private HashMap attributesMap;
	private HashMap flatAttributeMap;
	private HashMap downloadAttributeMap;
	private ArrayList flatAttributeList;

	// View JSP used to Create (displaying the create form) Item
	private String createJSP;

	//	View JSP used to display the Create success state
	private String createSuccessJSP;

	//	View JSP used to display the Create failure state
	private String createFailureJSP;

	//	View JSP used to Update (displaying the update form) Item
	private String updateJSP;

	//	View JSP used to display the Update success state
	private String updateSuccessJSP;

	//	View JSP used to display the Update failure state
	private String updateFailureJSP;

	//	View JSP used to display the list of items
	private String listJSP;

	//	View JSP used to display the Print view of the item
	private String printJSP;

	//	flag to find if item is in workflow or not in workflow 
	private boolean bworkflowItem = false;
	
	//	catalog associated with workflow item.
	// Note: Above catalogID stores collaboration name when item is in workflow
	private String associateWFCatalogID ;
	
	private List changedValueList;
	
	public CatalogItem() {
	}

	public CatalogItem(
		String parentCategory,
		String catalogID,
		String catalogVersion,
		Attribute[] attributes) {
		this(catalogID, attributes);
		setParentCategoryID(parentCategory);
		setCatalogVersion(catalogVersion);
		createJSP = "/jsps/html/itemcatalog/" + catalogID + "_Create.jsp";
		updateJSP = "/jsps/html/itemcatalog/" + catalogID + "_Create.jsp";
		printJSP = "/jsps/html/itemcatalog/" + catalogID + "_Create.jsp";
	}

	public CatalogItem(String catalogID, Attribute[] attributes) {
		this.catalogID = catalogID;
		this.attributes = attributes;
		prepareAttributesMap();
		flatAttributeMap = new HashMap();
		flatAttributeMap = prepareFlatAttributeMap(attributes);
		
		flatAttributeList = new ArrayList();
		flatAttributeList = prepareFlatAttributeList(attributes);		
	}

	public CatalogItem(
		String parentCategoryID,
		String catalogID,
		String catalogVersion,
		String itemID,
		Attribute[] attributes) {
		this.prentCtgyID = parentCategoryID;
		this.catalogID = catalogID;
		this.itemID = itemID;
		this.attributes = attributes;
		prepareAttributesMap();
		flatAttributeMap = new HashMap();
		flatAttributeMap = prepareFlatAttributeMap(attributes);
		
		flatAttributeList = new ArrayList();
		flatAttributeList = prepareFlatAttributeList(attributes);
	}

	/**
	 * @deprecated
	 */
	public String getCatalogID() {
		if ((catalogID == null) && (catalogID.equals(""))) {
			String catalogIDtemp =
				WPCMediator.getInstance().getDefaultCatalog();

			return catalogIDtemp;
		}

		return catalogID;
	}

	public void setCatalogID(String catalogID) {
		this.catalogID = catalogID;
	}

	public String getItemID() {
		return itemID;
	}

	public void setItemID(String itemID) {
		this.itemID = itemID;
	}

	public String getCreateView() {
		return createJSP;
	}

	public String getCreateSuccessView() {
		return (createSuccessJSP != null) ? createSuccessJSP : createJSP;
	}

	public String getCreateFailureView() {
		return (createFailureJSP != null) ? createFailureJSP : createJSP;
	}

	public void setCreateView(String createJSP) {
		this.createJSP = createJSP;
	}

	public void setCreateSuccessView(String createSuccessJSP) {
		this.createSuccessJSP = createSuccessJSP;
	}

	public void setCreateFailureView(String createFailureJSP) {
		this.createFailureJSP = createFailureJSP;
	}

	public String getUpdateView() {
		return updateJSP;
	}

	public String getUpdateSuccessView() {
		return (updateSuccessJSP != null) ? updateSuccessJSP : updateJSP;
	}

	public String getUpdateFailureView() {
		return (updateFailureJSP != null) ? updateFailureJSP : updateJSP;
	}

	public void setUpdateView(String updateJSP) {
		this.updateJSP = updateJSP;
	}

	public void setUpdateSuccessView(String updateSuccessJSP) {
		this.updateSuccessJSP = updateSuccessJSP;
	}

	public void setUpdateFailureView(String updateFailureJSP) {
		this.updateFailureJSP = updateFailureJSP;
	}

	public String getPrintView() {
		return printJSP;
	}

	public String getListView() {
		return listJSP;
	}

	public void setPrintView(String printJSP) {
		this.printJSP = printJSP;
	}

	public void setListView(String listJSP) {
		this.listJSP = listJSP;
	}

	public void setAttributes(Attribute[] attributes) {
		this.attributes = attributes;
		prepareAttributesMap();
	}

	public Attribute[] getAttributes() {
		return attributes;
	}

	public Attribute getAttribute(String name) {
		return (attributesMap != null)
			? (Attribute) attributesMap.get(name)
			: null;
	}

	/**
	 * The XML parsing capability is externilzed to CatalogItemParser
	 */
	public void fromXML(Node domNode) {
	}

	public void fromXML(String xml) {
	}
	/**
	 * @deprecated
	 * @see updateProperty
	 */
	public MessageMap setProperty(
		PortletRequest portletRequest,
		String specAttributeName) throws ParseException {
		if (attributes == null) {
			return null;
		}

		MessageMap messageMap = (MessageMap) portletRequest.getAttribute(IReference.MESSAGE_KEY);
		if (messageMap == null) {
			messageMap = new MessageMap();
			portletRequest.setAttribute(IReference.MESSAGE_KEY, messageMap);
		}
		Attribute attribute =
			(Attribute) flatAttributeMap.get(specAttributeName);
		updateAttribProperty(attribute,portletRequest, messageMap);
		return messageMap;
	}
	
	protected void checkMandataoryAttribute(
		Attribute attribute,
		PortletRequest portletRequest,
		MessageMap messageMap) {
		// Mandatory check is not required for Attribute 
		if (attribute.getType() == AttributeTypes.GROUP) {
			return;
		}

		//		String name = attribute.getName();
		String massEdit =
			(String) portletRequest.getAttribute(IMassUpdate.KEY_MASS_EDIT);

		// constraints has to be enabled for all the attributes
		String name = "";

		// In case of mass edit item id will be prepended to each name key
		if ((massEdit != null)
			&& (massEdit.equals(IMassUpdate.KEY_MASS_EDIT))) {
			name =
				"ATTR"
					+ itemID
					+ "_"
					+ UIUtils.replaceSlash(attribute.getName());
		} else {
			name = attribute.getName();
		}

		String[] attribRequestVals = portletRequest.getParameterValues(name);

		if (attribRequestVals == null) {
			// check if the min. occurrence is 0 or not
			if (attribute.getMinimumOccurrence() != 0) {
				// force the user to enter atleast min. occurrence values..
				messageMap.addErrorMessage("MESSAGE_CODE_110", " " + name);
			}
		} else if (
			attribRequestVals.length < attribute.getMinimumOccurrence()) {
			// the values entered is not meeting the min. requirements.
			// force the user to enter atleast min. occurrence values..
			//						messageMap.addErrorMessage("MESSAGE_CODE_110" , new Object[]{name, attribute.getMinimumOccurrence()+""});
			messageMap.addErrorMessage("MESSAGE_CODE_110", " " + name);
		} else {
			for (int i = 0; i < attribRequestVals.length; i++) {
				if (UIUtils.isEmpty(attribRequestVals[i])
					&& (i < attribute.getMinimumOccurrence())) {
					messageMap.addErrorMessage(
						"MESSAGE_CODE_110",
						" " + name + " " + (i + 1));
				}
			}
		}
	}

	/**
	 * The XML generation capability is externilzed to CatalogItemXMLGenerator
	 *
	 */
	public String toXML() {
		return null;
	}

	private void prepareAttributesMap() {
		if (attributes == null) {
			return;
		}

		attributesMap = new HashMap();

		for (int i = 0; i < attributes.length; ++i) {
			attributesMap.put(attributes[i].getName(), attributes[i]);
		}
	}

	public HashMap getAttributeMap() {
		return attributesMap;
	}

	public String getCatalogVersion() {
		return ctgVersion;
	}

	public String getParentCategoryID() {
		return prentCtgyID;
	}

	public void setCatalogVersion(String ctgVersion) {
		this.ctgVersion = ctgVersion;
	}

	public void setParentCategoryID(String parCtgyID) {
		this.prentCtgyID = parCtgyID;
	}

	public void printMe() {
		System.out.println(
			"******************** ITEM DETAILS ****************");
		System.out.println("ParentCategory::" + prentCtgyID);
		System.out.println("CatalogName::" + catalogID);
		System.out.println("ItemID::" + itemID);

		System.out.println("Printing the attributes list");

		for (int i = 0; i < attributes.length; ++i) {
			if (attributes[i] != null) {
				attributes[i].printMe();
			}
		}

		System.out.println(
			"******************** ///ITEM DETAILS ////****************");
	}

	/**
	 * returns the value of the Primary key attribute 
	 */
	public String getPrimaryKeyValue() {
		Attribute attribute = getPrimaryKeyAttr();
		
		switch(attribute.getType()) {

			case AttributeTypes.STRING :
				StringAttribute stringAttrib = (StringAttribute) attribute;
				return stringAttrib.getValue();
				
			case AttributeTypes.INTEGER :
				IntegerAttribute integerAttrib = (IntegerAttribute) attribute;
				return integerAttrib.getValue();
			
			case AttributeTypes.NUMBER :
				NumberAttribute numberAttrib = (NumberAttribute) attribute;
				return numberAttrib.getValue();
				
			case AttributeTypes.DATE :
				DateAttribute dateAttrib = (DateAttribute) attribute;
				return dateAttrib.getValue();
			
			case AttributeTypes.CURRENCY :
				CurrencyAttribute currencyAttrib = (CurrencyAttribute) attribute;
				return currencyAttrib.getValue();
				
			case AttributeTypes.PASSWORD :
				PasswordAttribute passwordAttrib = (PasswordAttribute) attribute;
				return passwordAttrib.getValue();
				
			case AttributeTypes.STRING_ENUMERATION :
				StringEnumAttribute stringenumAttrib = (StringEnumAttribute) attribute;
				return stringenumAttrib.getValue();
				
			case AttributeTypes.NUMBER_ENUMERATION :
				NumberEnumAttribute numberenumAttrib = (NumberEnumAttribute) attribute;
				return numberenumAttrib.getValue();

			default:
				// empty when attribute type is not supported
				return "";
		}
		
	}
	
	public Attribute getPrimaryKeyAttr() {
		return getPrimaryKeyAttr(attributes);
	}
	
	public Attribute getPrimaryKeyAttr(Attribute[] attrArry) {
		if (primaryKeyAttr == null) {
			return preparePrimaryKeyAttr(attrArry);
		} else {
			return primaryKeyAttr;
		}
	}

	public Attribute preparePrimaryKeyAttr(Attribute[] attrArry) {
		for (int k = 0; k < attrArry.length; k++) {
			Attribute attrTemp = attrArry[k];

			if (attrTemp.isPrimaryKey()) {
				primaryKeyAttr = attrTemp;

				break;
			}
		}

		return primaryKeyAttr;
	}

	/**
	 * prepare map of all attributes available including the map inside group attributes
	 * @param attributesParam
	 * @return
	 */
	private HashMap prepareFlatAttributeMap(Attribute[] attributesParam) {
		for (int i = 0; i < attributesParam.length; i++) {
			if (attributesParam[i] != null) {
				if (attributesParam[i].getType() == AttributeTypes.GROUP) {
					flatAttributeMap.put(attributesParam[i].getName(),
						attributesParam[i]);
					prepareFlatAttributeMap(((GroupAttribute) attributesParam[i]).getAttributes());
				} else {
					//if (attributesParam[i].isLocalized() && attributesParam[i].getLocalizedValues().size() > 0){
                    
					if (attributesParam[i].isLocalized()) {
						flatAttributeMap.put(attributesParam[i].getName(),
													attributesParam[i]);
						Attribute[] attrTempArray = new Attribute[attributesParam[i].getLocalizedValues().size()];
						for (int j = 0; j < attributesParam[i].getLocalizedValues().size(); j++) {
							attrTempArray[j] = (Attribute)attributesParam[i].getLocalizedValues().get(j);
						}
						prepareFlatAttributeMap(attrTempArray);
					} else {
						flatAttributeMap.put(attributesParam[i].getName(),
							attributesParam[i]);
					}
				}
			}
		}

		return flatAttributeMap;
	}

	/**
	 * @return
	 */
	public HashMap getFlatAttributeMap() {
		return flatAttributeMap;
	}

	/** 
	 * 
	 * @return flatAttributeList
	 */
	public ArrayList getFlatAttributeList() {
		return flatAttributeList;		
	}

	/**
	 * 
	 * @param attributes
	 * @return
	 */
	private ArrayList prepareFlatAttributeList(Attribute[] attributes) {
		
		for (int i = 0; i < attributes.length; i++) {
			if (attributes[i] != null && !attributes[i].isCategorySpecificAttribute()) {
				if (attributes[i].getType() == AttributeTypes.GROUP) {
					flatAttributeList.add(attributes[i].getName());
					prepareFlatAttributeList(((GroupAttribute) attributes[i]).getAttributes());
				} else {
					//if (attributesParam[i].isLocalized() && attributesParam[i].getLocalizedValues().size() > 0){
                    
					if (attributes[i].isLocalized()) {
						flatAttributeList.add(attributes[i].getName());
						Attribute[] attrTempArray = new Attribute[attributes[i].getLocalizedValues().size()];
						prepareFlatAttributeList(attrTempArray);
					} else {
						flatAttributeList.add(attributes[i].getName());
						
					}
				}
			}
		}

		return flatAttributeList;
	}


	/**
	 * @see com.ibm.ccd.ui.dataobjects.items.ICatalogItem#getCollaborationArea()
	 */
	public CollaborationArea getCollaborationArea() {
		return collabArea;
	}

	/**
	 * @see com.ibm.ccd.ui.dataobjects.items.ICatalogItem#setCollaborationArea(com.ibm.ccd.ui.dataobjects.workflow.CollaborationArea)
	 */
	public void setCollaborationArea(CollaborationArea collabArea) {
		this.collabArea = collabArea;
	}

	public MessageMap updateProperty(
		PortletRequest portletRequest,
		String attribName) {
		MessageMap messageMap = new MessageMap();
		Attribute attribute = (Attribute) flatAttributeMap.get(attribName);
		
		String changedValues = portletRequest.getParameter("changedValue");
		changedValueList = getChangedValueList(changedValues);
					
		updateAttribProperty(attribute, portletRequest, messageMap);

		return messageMap;
	}

	public MessageMap updateProperties(PortletRequest portletRequest) {
	//	MessageMap messageMap = new MessageMap();
		MessageMap messageMap = (MessageMap)portletRequest.getAttribute(IReference.MESSAGE_KEY);
		
		String changedValues = portletRequest.getParameter("changedValue");
		changedValueList = getChangedValueList(changedValues);
				
		if(messageMap == null){
			messageMap = new MessageMap();
		}

		for (int i = 0; i < attributes.length; ++i) {
			updateAttribProperty(attributes[i], portletRequest, messageMap);
		}

		return messageMap;
	}

	public void updateAttribProperty(
		Attribute attribute,
		PortletRequest portletRequest,
		MessageMap messageMap) {
		String name = null;
		String value = null;
		String path = attribute.getPath();
		String parameter = null;
		String massEdit = (String) portletRequest.getAttribute(IMassUpdate.KEY_MASS_EDIT);
		int attrOccurenceCount = 0;

		if ((attribute.getType() == AttributeTypes.SEQUENCE) || !attribute.isEditable()) {
			return;
		}
		
		if (path != null) {
			int pathIndex = path.lastIndexOf('_');

			if (pathIndex != -1) {
				path = path.substring(0, pathIndex);
			}
			if(!hasValueChanged(attribute) && attribute.isOptional() && !attribute.isValueChanged() ) { 
				//return when attribute value is not changed
				return;
			}
			attribute.setPath(null);
		} else {
			//Check for a Localized attribute. The parent/Localized attribute is always with a path=null, 
			//therefore recurse through the localized values that it holds
			if(attribute.isLocalized()){
				for (int i = 0; i < attribute.getLocalizedValues().size(); i++) {
					updateAttribProperty(
					(Attribute)attribute.getLocalizedValues().get(i),
					portletRequest,
					messageMap);
				}
			}
			if(massEdit == null){
				return;
			}
		}
		
		/*
		 * The request parameter name is different for Work With Item portlet and the MassEdit portlet.
		 * Since a common method is being used now to do mandatory check, store the parameter name based on
		 * the portlet from which this method got invoked.
		 */

		// In case of mass edit item id will be prepended to each name key
		if ((massEdit != null)
			&& (massEdit.equals(IMassUpdate.KEY_MASS_EDIT))) {
			parameter =
				"ATTR"
					+ itemID
					+ "_"
					+ UIUtils.replaceEmptyStr(
						UIUtils.replaceSlash(attribute.getName()));
		} else {
			parameter = path;
		}
		int currentOcc = attribute.getCurrOccurrence();
		String error = null;
		Locale locale = portletRequest.getLocale();
		name = attribute.getDisplayLabelName();

		/*
		 * Data Validation: For each attribute type that requires validation, fetch the value from the portlet request
		 * object and invoke the corresponding validateXXX method on ValidationHelper class. Set the value on the 
		 * corresponding attribute object irrespective of whether it is valid or invalid. The values will now be stored 
		 * as String. After setting the value, populate the errorMap in Attribute object by invoking the  
		 * setValidationError method. The key for this map will be the name of the HTML input text field that will be
		 * used to enter the value of this attribute. For valid values the errorMap will be set to null and for
		 * invalid values it will be set to the error message that is to be displayed to the user.
		 */
		switch (attribute.getType()) {
			case AttributeTypes.STRING :
				{
					StringAttribute stringAttribute =
						(StringAttribute) attribute;

					stringAttribute.removeValues();

					if (stringAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(parameter + "_" + i);
							value = UIUtils.unescapeForHTML(value);
							stringAttribute.setValue(value);
							stringAttribute.setValidationError(parameter + "_" + i,null);
						}
					} else {
						value = portletRequest.getParameter(parameter + "_" + "0");
						value = UIUtils.unescapeForHTML(value);
						stringAttribute.setValue(value);
						stringAttribute.setValidationError(parameter + "_" + "0",null);
					}

					break;
				}

			case AttributeTypes.PASSWORD :
				{
					PasswordAttribute passwordAttribute =
						(PasswordAttribute) attribute;
					String isPwdChanged = null;

					if (passwordAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							isPwdChanged = portletRequest.getParameter(parameter + "_" + i + "_h");
							if(!UIUtils.isEmpty(isPwdChanged) && isPwdChanged.equals("1")) {
								value = portletRequest.getParameter(parameter + "_" + i);
								value = UIUtils.unescapeForHTML(value);
								passwordAttribute.removeValue(i);
								passwordAttribute.setValue(i,value);
							}
						}
					} else {
						isPwdChanged = portletRequest.getParameter(parameter + "_0" + "_h");
						if(!UIUtils.isEmpty(isPwdChanged) && isPwdChanged.equals("1")) {
							value = portletRequest.getParameter(parameter + "_" + "0");
							value = UIUtils.unescapeForHTML(value);
							passwordAttribute.removeValue(0);							
							passwordAttribute.setValue(0,value);
						}
					}

					break;
				}

			case AttributeTypes.CURRENCY :
				{
					CurrencyAttribute currencyAttribute =
						(CurrencyAttribute) attribute;

					currencyAttribute.removeValues();
					int precision = 2;
					if (currencyAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(parameter + "_" + i);
							value = UIUtils.unescapeForHTML(value);
							error = ValidationHelper.validateCurrency(name + "#" + i,value,precision,locale,messageMap);
							if(error == null) {
								try {
									value = UIUtils.parseNumber(value,portletRequest.getLocale());
									value = UIUtils.formatNumber(value,Locale.ENGLISH);
								} catch (ParseException e) {
									e.printStackTrace();
								}	
							}
							currencyAttribute.setValidationError(parameter + "_" + i,error);
							currencyAttribute.setValue(value);
						}
					} else {
						value = portletRequest.getParameter(parameter + "_" + "0");
						value = UIUtils.unescapeForHTML(value);
						error = ValidationHelper.validateCurrency(name,value,precision,locale,messageMap);
						if(error == null) {
							try {
								value = UIUtils.parseNumber(value,portletRequest.getLocale());
								value = UIUtils.formatNumber(value,Locale.ENGLISH);
							} catch (ParseException e) {
								e.printStackTrace();
							}	
						}
						currencyAttribute.setValidationError(parameter + "_" + "0",error);
						currencyAttribute.setValue(value);
					}
					break;
				}

			case AttributeTypes.DATE :
				{
					DateAttribute dateAttribute = (DateAttribute) attribute;
					dateAttribute.removeValues();

					if (dateAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(parameter + "_" + i);
							if (!UIUtils.isEmpty(value)) {
								error = ValidationHelper.validateDate(name + "#" + i,value,locale,messageMap);
								dateAttribute.setValidationError(parameter + "_" + i,error);
								if(error == null){
									// Date is valid. Convert it to the WPC format and then store it.
									String wpcValue = UIUtils.formatWPCDate(value,locale);		
									dateAttribute.setValue(wpcValue);
								}else{
									dateAttribute.setValue(value);
								}
							}else{
								dateAttribute.setValue(value);
								dateAttribute.setValidationError(parameter + "_" + i,error);
							}
						}
					}
					else {
						value = portletRequest.getParameter(parameter + "_" + "0");
						System.err.println("value of Date entered is " + value);
						if (!UIUtils.isEmpty(value)) {
						error = ValidationHelper.validateDate(name,value,locale,messageMap);
						dateAttribute.setValidationError(parameter + "_" + "0",error);
						if(error == null){
							// Date is valid. Convert it to the WPC format and then store it.
							String wpcValue = UIUtils.formatWPCDate(value,locale);	
							dateAttribute.setValue(wpcValue);
						}else{
							dateAttribute.setValue(value);
						}
						}
					}
					break;
				}

			case AttributeTypes.INTEGER :
				{
					IntegerAttribute integerAttribute =
						(IntegerAttribute) attribute;
					integerAttribute.removeValues();

					if (integerAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(parameter + "_" + i);
							error = ValidationHelper.validateInteger(name + "#" + i,value,locale,messageMap);
							integerAttribute.setValidationError(parameter + "_" + i,error);				
							integerAttribute.setValue(value);											
						}
					} else {
						value = portletRequest.getParameter(parameter + "_" + "0");
						error = ValidationHelper.validateInteger(name,value,locale,messageMap);
						integerAttribute.setValidationError(parameter + "_" + "0",error);
						integerAttribute.setValue(value);
					}
					break;
				}

			case AttributeTypes.NUMBER :
				{
					NumberAttribute numberAttribute =
						(NumberAttribute) attribute;
					numberAttribute.removeValues();
					int precision = numberAttribute.getPrecision();
					if (numberAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(parameter + "_" + i);
							error = ValidationHelper.validateNumber(name + "#" + i,value,precision,locale,messageMap);
							if(error == null) {
								try {
									value = UIUtils.parseNumber(value,portletRequest.getLocale());
									value = UIUtils.formatNumber(value,Locale.ENGLISH);
								} catch (ParseException e) {
									e.printStackTrace();
								}	
							}
							numberAttribute.setValidationError(parameter + "_" + i,error);
							numberAttribute.setValue(value);											
						}
					} else {
						value = portletRequest.getParameter(parameter + "_" + "0");
						error = ValidationHelper.validateNumber(name,value,precision,locale,messageMap);
						if(error == null) {
							try {
								value = UIUtils.parseNumber(value,portletRequest.getLocale());
								value = UIUtils.formatNumber(value,Locale.ENGLISH);
							} catch (ParseException e) {
								e.printStackTrace();
							}	
						}
						numberAttribute.setValidationError(parameter + "_" + "0",error);
						numberAttribute.setValue(value);
					}
					break;
				}

			case AttributeTypes.FLAG :
				{
					FlagAttribute flagAttribute = (FlagAttribute) attribute;
					attribute.removeValues();

					if (flagAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(parameter + "_" + i);
							flagAttribute.setValue(value);
						}
					} else {
						value = portletRequest.getParameter(parameter + "_" + "0");
						flagAttribute.setValue(value);
					}

					break;
				}
				
			case AttributeTypes.PERIOD :
			{
				PeriodAttribute periodAttribute = (PeriodAttribute) attribute;
				periodAttribute.removeValues();
				
				if(periodAttribute.isMultiValued()) {
					for(int i=0;i<currentOcc;i++) {
						String startDate = portletRequest.getParameter(parameter + "_" + i+"_"+"V1");
						String endDate = portletRequest.getParameter(parameter + "_" + i+"_"+"V2");
						checkPeriodValidation(attribute,name + "#" + i,parameter + "_" + i,startDate,endDate,locale,messageMap);
					}
				}else {
					String startDate = portletRequest.getParameter(parameter + "_" + "0" +"_"+"V1");
					String endDate = portletRequest.getParameter(parameter + "_" + "0" +"_"+"V2");
					checkPeriodValidation(attribute,name,parameter + "_" + "0",startDate,endDate,locale,messageMap);
				}
		
				break;
			}
			case AttributeTypes.URL :
			case AttributeTypes.IMAGE_URL :
			case AttributeTypes.THUMB_NAIL_IMAGE_URL :	
				{
					URLAttribute urlAttribute = (URLAttribute) attribute;
					urlAttribute.removeValues();
					
					if (urlAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(parameter + "_" + i);
							value = UIUtils.unescapeForHTML(value);
							
								if (!UIUtils.isEmpty(value)) {
									error = ValidationHelper.validateURL(name + "#" + i,value,locale,messageMap);
									urlAttribute.setValidationError(parameter + "_" + i,error);
									urlAttribute.setValue(value);
								}else {
									urlAttribute.setValue(value);
								}
						}
					} else {
						value = portletRequest.getParameter(parameter + "_" + "0");
						if(!UIUtils.isEmpty(value)) {
							error = ValidationHelper.validateURL(name,value,locale,messageMap);
							urlAttribute.setValidationError(parameter + "_" + "0",error);
							urlAttribute.setValue(value);
							
						} else {
							urlAttribute.setValue(value);
						}
					}
					break;
				}

			case AttributeTypes.BINARY :
				{
					BinaryAttribute binaryAttrib = (BinaryAttribute) attribute;
					FileUploadResource[] files = null;

					if (binaryAttrib.isMultiValued()) {
						files = binaryAttrib.getValues();

						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(parameter + "_" + i);
							value = UIUtils.unescapeForHTML(value);
							
							if ((files[i] != null)
								&& ((files[i].getName() == null)
									|| !files[i].getName().equals(value))) {
								// set server details only when user update with new file attachment
								files[i].setDirectory("");
								files[i].setName("");
								files[i].setPassword("");
								files[i].setServer("");
								files[i].setType("");
								files[i].setUserID("");
								files[i].setName(value);
							}
						}
					} else {
						value = portletRequest.getParameter(parameter + "_" + "0");
						value = UIUtils.unescapeForHTML(value);
						
						FileUploadResource file = binaryAttrib.getValue();

						if ((file != null)
							&& ((file.getName() == null)
								|| !file.getName().equals(value))) {
							// set server details only when user update with new file attachment
							file.setDirectory("");
							file.setName("");
							file.setPassword("");
							file.setServer("");
							file.setType("");
							file.setUserID("");
							file.setName(value);
						}
					}
					break;
				}

			case AttributeTypes.IMAGE :
			case AttributeTypes.THUMB_NAIL_IMAGE :	
				{
					ImageAttribute imageAttribute = (ImageAttribute) attribute;
					FileUploadResource[] files = null;

					if (imageAttribute.isMultiValued()) {
						files = imageAttribute.getValues();

						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(parameter + "_" + i);
							value = UIUtils.unescapeForHTML(value);
							if(!UIUtils.isEmpty(value)) {
							error = ValidationHelper.validateImage(name + "#" + i,value,locale,messageMap);
							}
							imageAttribute.setValidationError(parameter + "_" + i,error);							
							if ((files[i] != null)
								&& ((files[i].getName() == null)
									|| !files[i].getName().equals(value))) {
								// set server details only when user update with new file attachment
								files[i].setDirectory("");
								files[i].setName("");
								files[i].setPassword("");
								files[i].setServer("");
								files[i].setType("");
								files[i].setUserID("");
								files[i].setName(value);
							}
						}
					} else {
						value = portletRequest.getParameter(parameter + "_" + "0");
						value = UIUtils.unescapeForHTML(value);
						
						FileUploadResource file = imageAttribute.getValue();
						
						if(!UIUtils.isEmpty(value)) {
							error = ValidationHelper.validateImage(name,value,locale,messageMap);
						}
						imageAttribute.setValidationError(parameter + "_" + "0",error);

						if ((file != null)
							&& ((file.getName() == null)
								|| !file.getName().equals(value))) {
							// set server details only when user update with new file attachment
							file.setDirectory("");
							file.setName("");
							file.setPassword("");
							file.setServer("");
							file.setType("");
							file.setUserID("");
							file.setName(value);
						}
					}

					break;
				}

			case AttributeTypes.STRING_ENUMERATION :
				{
					StringEnumAttribute stringEnumAttribute =
						(StringEnumAttribute) attribute;
					stringEnumAttribute.removeValues();

					if (stringEnumAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(parameter + "_" + i);
							value = UIUtils.unescapeForHTML(value);
							stringEnumAttribute.setValue(value);
						}
					} else {
						value = portletRequest.getParameter(parameter + "_" + "0");
						value = UIUtils.unescapeForHTML(value);
						stringEnumAttribute.setValue(value);
					}

					break;
				}

			case AttributeTypes.NUMBER_ENUMERATION :
				{
					NumberEnumAttribute numberEnumAttribute =
						(NumberEnumAttribute) attribute;
					numberEnumAttribute.removeValues();
					int precision = numberEnumAttribute.getPrecision();

					if (numberEnumAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(parameter + "_" + i);
							//error = ValidationHelper.validateNumber(name + "#" + i,value,precision,locale,messageMap);
							numberEnumAttribute.setValidationError(parameter + "_" + i,error);
							numberEnumAttribute.setValue(value);							
						}
					} else {
						value = portletRequest.getParameter(parameter + "_" + "0");
						//error = ValidationHelper.validateNumber(name,value,precision,locale,messageMap);
						numberEnumAttribute.setValidationError(parameter + "_" + "0",error);
						numberEnumAttribute.setValue(value);
					}
					break;
				}

			case AttributeTypes.TIMEZONE :
				{
					TimeZoneAttribute timeZoneAttribute =
						(TimeZoneAttribute) attribute;
					timeZoneAttribute.removeValues();

					if (timeZoneAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(parameter + "_" + i);
							timeZoneAttribute.setValue(value);
						}
					} else {
						value = portletRequest.getParameter(parameter + "_" + "0");
						timeZoneAttribute.setValue(value);
					}

					break;
				}

			case AttributeTypes.UNSUPPORTED :
				break;

			case AttributeTypes.GROUP :
				{
					GroupAttribute groupAttrib = (GroupAttribute) attribute;
					Attribute[] subGroupAttrib = groupAttrib.getAttributes();

					for (int i = 0; i < currentOcc; i++) {
						subGroupAttrib = groupAttrib.getValue(i);

						for (int j = 0; j < subGroupAttrib.length; j++) {
							updateAttribProperty(
								subGroupAttrib[j],
								portletRequest,
								messageMap);
						}
					}

					break;
				}

			case AttributeTypes.RELATIONSHIP :
				{
					String catalogName = null;
					String key = null;
					String itemId = null;
					NVPair pair = null;
					RelationshipAttribute relationshipAttribute =
						(RelationshipAttribute) attribute;
					relationshipAttribute.removeValues();
					
					System.out.println("***8** " + relationshipAttribute.isMultiValued());

					if (relationshipAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							catalogName =
								portletRequest.getParameter(
									parameter + "_" + i + "_V1");
							itemId =
								portletRequest.getParameter(
									parameter + "_" + i + "_V2");
							pair = new NVPair(catalogName, itemId);
							catalogName = UIUtils.unescapeForHTML(catalogName);
							itemId = UIUtils.unescapeForHTML(itemId);							
							relationshipAttribute.setValue(pair);
						}
					} else {
						
						if(massEdit == null) {
							catalogName =
								portletRequest.getParameter(parameter + "_" + 0 + "_V1");
							itemId =
								portletRequest.getParameter(parameter + "_" + 0 + "_V2");
								
							catalogName = UIUtils.unescapeForHTML(catalogName);
							itemId = UIUtils.unescapeForHTML(itemId);	
							pair = new NVPair(catalogName, itemId);
							relationshipAttribute.setValue(pair);

						} else {
							value =
								portletRequest.getParameter(parameter + "_" + 0 );
							/*itemId =
								portletRequest.getParameter(parameter + "_" + 0 ); */
								
							if (value != null) {
								String key1 = value.substring(0,(value.indexOf("|")));
								pair = new NVPair();
								pair.setValue(value.substring(value.indexOf("|")+1,value.length()));
								pair.setName(key1);
								relationshipAttribute.setValue(pair);
							}

						}
					}

					break;
				}

			case AttributeTypes.LOOKUP_TABLE :
				{
					LookupTableAttribute lookupTableAttribute =
						(LookupTableAttribute) attribute;
					lookupTableAttribute.removeValues();

					if (lookupTableAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(parameter + "_" + i);
							value = UIUtils.unescapeForHTML(value);
							lookupTableAttribute.setValue(value);
						}
					} else {
						value = portletRequest.getParameter(parameter + "_" + "0");
						value = UIUtils.unescapeForHTML(value);
						lookupTableAttribute.setValue(value);
					}

					break;
				}
		}
		checkMandatoryCriteria(parameter,attribute,portletRequest,messageMap);
	}

	private void checkPeriodValidation(Attribute attribute,String name,String path,String startDate,String endDate,Locale locale,MessageMap messageMap){
		PeriodAttribute periodAttribute = (PeriodAttribute)attribute;
		String startValue = "";
		String endValue = "";
		String fromError = "";
		String toError = "";
		String error = null;
		periodAttribute.setValidationError(path + "_"+"V1",null);
		periodAttribute.setValidationError(path + "_"+"V2",null);
		periodAttribute.setValidationError(path,null);
		//Check for validity of FROM DATE
		if (!UIUtils.isEmpty(startDate)) {
		fromError = ValidationHelper.validateDate(name,startDate,locale,messageMap);
		if(fromError == null){
			// Convert the date to WPC format and then store
			startValue = UIUtils.formatWPCDate(startDate,locale);
		}
		else
			periodAttribute.setValidationError(path +"_"+"V1",fromError);
		}
		//Check for validity of TO DATE
		if (!UIUtils.isEmpty(endDate)) {
		toError = ValidationHelper.validateDate(name,endDate,locale,messageMap);
		if(toError == null){
			// Convert the date to WPC format and then store
			endValue = UIUtils.formatWPCDate(endDate,locale);	
		}else{
			periodAttribute.setValidationError(path +"_"+"V2",toError);
		}
		}
	
		//If both from and to dates are valid then check for period validity
		if(fromError == null && toError == null){
			error = ValidationHelper.validatePeriod(name,startDate,endDate,locale,messageMap);
			periodAttribute.setValidationError(path,error);
			periodAttribute.setValue(startValue,endValue);
		}else{
			//isValid = false;
			error = null;
			periodAttribute.setValue(startDate,endDate);
		}
	}

	protected void checkMandatoryCriteria(String parameter, Attribute attribute, PortletRequest portletRequest, MessageMap messageMap){

		// Mandatory check is not required for Attribute 
		if (attribute.getType() == AttributeTypes.GROUP) {
			return;
		}
		String value = null;
		int currOccurence = attribute.getCurrOccurrence();
		int attrOccurenceCount = 0;
		
		if (attribute.isMultiValued()) {
			for (int i = 0; i < currOccurence; i++) {
				value = portletRequest.getParameter(parameter + "_" + i);
//				// For Number Enumeration the check for empty value is different than for other types
				if(attribute.getType() == AttributeTypes.NUMBER_ENUMERATION){
					if(!value.equals("-1"))
						attrOccurenceCount++;
				}
				else if (!UIUtils.isEmpty(value)) {
					attrOccurenceCount++;
				}
			}
			
			if(attrOccurenceCount < attribute.getMinimumOccurrence()){
				messageMap.addErrorMessage(IValidatorMessage.MSG_MIN_OCCURENCE_ERROR,attribute.getDisplayLabelName(),String.valueOf(attribute.getMinimumOccurrence()));
				Object[] values = {attribute.getDisplayLabelName(),String.valueOf(attribute.getMinimumOccurrence())};
				String error = ValidationHelper.createErrorMessage(IValidatorMessage.MSG_MIN_OCCURENCE_ERROR,portletRequest.getLocale(),values);
				for(int i=0;i < currOccurence; i++){
				attribute.setValidationError(parameter + "_" + i,error);
				}
				
			}
		}
		else{
			value = portletRequest.getParameter(parameter + "_" + "0");
			
			if(attribute.getMinimumOccurrence() != 0){
				if (UIUtils.isEmpty(value)) {
					Object[] values = {attribute.getDisplayLabelName(),String.valueOf(attribute.getMinimumOccurrence())};
					String error = ValidationHelper.createErrorMessage(IValidatorMessage.MSG_MANDATORY_ATTRIBUTE,portletRequest.getLocale(),values);
					attribute.setValidationError(parameter + "_" + "0",error);
					messageMap.addErrorMessage(IValidatorMessage.MSG_MANDATORY_ATTRIBUTE,attribute.getDisplayLabelName());
				}
				else {
					attribute.setValidationError(parameter + "_" + "0",null);
				}
			}
		}
	}
	
	public Attribute getAttributeFromhashValue(String hashValue) {
		HashMap flatAttrib = getFlatAttributeMap();
		Iterator keyIter = flatAttrib.keySet().iterator();
		Attribute attrib = null;

		while (keyIter.hasNext()) {
			attrib = (Attribute) flatAttrib.get(keyIter.next());

			if (attrib.getHashName().equals(hashValue)) {
				return attrib;
			}
		}

		return attrib;
	}
	public String getAttributeNameFromhashValue(String hashValue) {
		Attribute attrib = getAttributeFromhashValue(hashValue);
		if (attrib != null) {
			return attrib.getName();
		} else {
			return null;
		}
	}
	/**
	 *
	 * @param attribPath is GroupID/GroupOcc/AttribID/AttribOcc
	 * @return
	 */
	public Attribute getAttributeFromPath(String attribPath) {
		if (UIUtils.isEmpty(getGroupAttributeID(attribPath))) {
			String attribID = getAttributeID(attribPath);

			return getAttributeFromhashValue(attribID);
		} else {
			String groupID = getGroupAttributeID(attribPath);
			String groupOcc = getGroupAttributeOcc(attribPath);
			String attribID = getAttributeID(attribPath);
			GroupAttribute groupAttrib =
				(GroupAttribute) getAttributeFromhashValue(groupID);
			Attribute[] attribs =
				groupAttrib.getValue(Integer.parseInt(groupOcc));

			for (int i = 0; i < attribs.length; i++) {
				if (attribs[i].getName().equals(attribID)) {
					return attribs[i];
				}
			}
			return null;
		}
	}

	/**
	 * attribPath should contain both groupId and attrubuteId
	 * @param attribPath
	 * @return
	 */
	public Attribute getAttributeFromGroupPath(String attribPath)
		throws Exception {
		return getAttributeFromGroupPath(attributes, attribPath);
	}

	/**
	 * attribPath should contain both groupId and attributeId 
	 * @param attributes
	 * @param attribPath
	 * @return
	 */
	public Attribute getAttributeFromGroupPath(
		Attribute[] attribs,
		String attribPath)
		throws Exception {
		StringTokenizer tokenizer = new StringTokenizer(attribPath, "_");

		//contains nestedGroup
		String groupId = null;
		String groupOcc = null;

		String attribId = null;
		String attribOcc = null;
		String truncPath = null;
		if (tokenizer.countTokens() > 4) {
			groupId = tokenizer.nextToken(); // GroupId
			groupOcc = tokenizer.nextToken(); // GroupOcc

			truncPath = groupId + "_" + groupOcc + "_";
			truncPath = attribPath.substring(truncPath.length());

			for (int i = 0; i < attribs.length; i++) {
				if (attribs[i]
					.getName()
					.equals(
						getAttributeNameFromhashValue(groupId))) {
					GroupAttribute groupAttrib = (GroupAttribute) attribs[i];
					if (!groupAttrib.hasValue(Integer.parseInt(groupOcc))) {
						throw new Exception("Exceeds Group limit " + attribPath);
					}
					Attribute[] subAttribs =
						groupAttrib.getValue(Integer.parseInt(groupOcc));
					return getAttributeFromGroupPath(subAttribs, truncPath);
				}
			}
		} else if (tokenizer.countTokens() > 2) {
			groupId = tokenizer.nextToken(); // GroupId
			groupOcc = tokenizer.nextToken(); // GroupOcc

			attribId = tokenizer.nextToken();
			attribOcc = tokenizer.nextToken();

			for (int i = 0; i < attribs.length; i++) {
				
				if (attribs[i]
					.getName()
					.equals(getAttributeNameFromhashValue(groupId))) {
					GroupAttribute groupAttrib = (GroupAttribute) attribs[i];
					if (!groupAttrib.hasValue(Integer.parseInt(groupOcc))) {
						throw new Exception("Exceeds Group limit " + attribPath);
					}
					Attribute[] subAttribs =
						groupAttrib.getValue(Integer.parseInt(groupOcc));

					for (int j = 0; j < subAttribs.length; j++) {
						if (subAttribs[j]
							.getName()
							.equals(
								getAttributeNameFromhashValue(attribId))) {
							return subAttribs[j];
						}
					}

				}
			}
		} else {
			attribId = tokenizer.nextToken();
			attribOcc = tokenizer.nextToken();
			//Check for a Localized attribute. The parent/Localized attribute is always with a path=null, 
			//therefore recurse through the localized values that it holds

			for (int i = 0; i < attributes.length; i++) {
				
				if(attributes[i].isLocalized()){
					for (int k = 0; k < attributes[i].getLocalizedValues().size(); k++) {
						Attribute localizedAttribute = (Attribute) attributes[i].getLocalizedValues().get(k);
						if (localizedAttribute
							.getName()
							.equals(
								getAttributeNameFromhashValue(attribId))) {
							return localizedAttribute;
						}
					} // End of for loop getLocalizedvalues array
				}else{  // End isLocalized check
					if (attributes[i]
						.getName()
						.equals(
							getAttributeNameFromhashValue(attribId))) {
						return attributes[i];
					}
				}
			}// End attributes array

		}
		return null;

	}
	/**
	 * 
	 * @param attribPath is GroupID_GroupOcc_GroupID_GroupOcc_GroupID_GroupOcc
	 * @return
	 */
	public Attribute[] getAttributesFromGroupPath(String attribPath)
		throws Exception {
		return getAttributesFromGroupPath(attributes, attribPath);
	}

	/**
	 * 
	 * @param attribPath is GroupID_GroupOcc_GroupID_GroupOcc_GroupID_GroupOcc
	 * @return
	 */
	public Attribute[] getAttributesFromGroupPath(
		Attribute[] attribs,
		String attribPath)
		throws Exception {

		StringTokenizer tokenizer = new StringTokenizer(attribPath, "_");

		//contains nestedGroup
		String groupId = null;
		String groupOcc = null;
		String truncPath = null;
		if (tokenizer.countTokens() > 2) {
			groupId = tokenizer.nextToken(); // GroupId
			groupOcc = tokenizer.nextToken(); // GroupOcc
			truncPath = groupId + "_" + groupOcc + "_";
			truncPath = attribPath.substring(truncPath.length());


			for (int i = 0; i < attribs.length; i++) {
				if (attribs[i]
					.getName()
					.equals(
						getAttributeNameFromhashValue(groupId))) {
					GroupAttribute groupAttrib = (GroupAttribute) attribs[i];
					if (!groupAttrib.hasValue(Integer.parseInt(groupOcc))) {
						throw new Exception("Exceeds Group limit " + attribPath);
					}
					Attribute[] subAttribs =
						groupAttrib.getValue(Integer.parseInt(groupOcc));
					return getAttributesFromGroupPath(subAttribs, truncPath);
				}
			}
		} else {
			groupId = tokenizer.nextToken(); // GroupId
			groupOcc = tokenizer.nextToken(); // GroupOcc

			for (int i = 0; i < attribs.length; i++) {
				if (attribs[i]
					.getName()
					.equals(
						getAttributeNameFromhashValue(groupId))) {
					GroupAttribute groupAttrib = (GroupAttribute) attribs[i];
					if (!groupAttrib.hasValue(Integer.parseInt(groupOcc))) {
						throw new Exception("Exceeds Group limit " + attribPath);
					}
					Attribute[] subAttribs =
						groupAttrib.getValue(Integer.parseInt(groupOcc));

					return subAttribs;
				}
			}
		}
		return null;
	}
	
	public boolean isValidGroupAttributePath(String attribPath) {
		return true;
	}
	
	public Attribute getGroupAttribute(Attribute attribute) {
		String attribName = attribute.getName();
		int firstIndex = attribName.indexOf('_');
		int lastIndex = attribName.lastIndexOf('_');

		if (firstIndex == lastIndex) {
			return null;
		} else {
			attribName = attribName.substring(0, lastIndex);

			return getAttribute(attribName);
		}
	}

	public static String getGroupAttributeID(String attribParam) {
		int firstIndex = attribParam.indexOf('_');
		int lastIndex = attribParam.lastIndexOf('_');

		if (firstIndex == lastIndex) {
			return "";
		} else {
			return attribParam.substring(0, firstIndex);
		}
	}

	public static String getGroupAttributeOcc(String attribParam) {
		int firstIndex = attribParam.indexOf('_');
		int lastIndex = attribParam.lastIndexOf('_');

		if (firstIndex == lastIndex) {
			return "";
		} else {
			int nextIndex = attribParam.indexOf('_', firstIndex + 1);

			return attribParam.substring(firstIndex + 1, nextIndex);
		}
	}

	public static String getAttributeID(String attribParam) {
		int firstIndex = attribParam.indexOf('_');
		int lastIndex = attribParam.lastIndexOf('_');

		if (firstIndex == lastIndex) {
			return attribParam.substring(0, firstIndex);
		} else {
			int nextIndex = attribParam.indexOf('_', firstIndex + 1);

			return getAttributeID(attribParam.substring(nextIndex + 1));
		}
	}

	public static String getAttributeOcc(String attribParam) {
		int firstIndex = attribParam.indexOf('_');
		int lastIndex = attribParam.lastIndexOf('_');

		if (firstIndex == lastIndex) {
			return attribParam.substring(firstIndex + 1);
		} else {
			int nextIndex = attribParam.indexOf('_', firstIndex + 1);

			return getAttributeOcc(attribParam.substring(nextIndex + 1));
		}
	}
	
	protected boolean hasValueChanged(Attribute attribute) {
		String path = attribute.getParentPath() + "_" + attribute.getHashName();
		
		if (attribute.getType() == AttributeTypes.GROUP) {
			path = ((GroupAttribute)attribute).getGroupAppendPath();
			path = path.substring(0,path.lastIndexOf('_'));  
			if (changedValueList != null && changedValueList.contains(path)) {
				attribute.setParentGroupFlagValueChanged();
				attribute.setValueChanged(true);
			}
			return true;
		}
		
		if (changedValueList != null && changedValueList.contains(path)) {
			attribute.setParentGroupFlagValueChanged();
			attribute.setValueChanged(true);
			return true;
		} else {
			return false;
		}
	}

	protected List getChangedValueList(String changedValues) {
		List list = new ArrayList();
		StringTokenizer token = new StringTokenizer(changedValues, ",");
		while (token.hasMoreTokens()) {
			list.add(token.nextToken());
		}
		return list;
	}
	
	/**
	 * @return HashMap return the attribute Map for Item Download
	 */
	public HashMap getDownloadAttributeMap() {
		downloadAttributeMap = new HashMap();
		return getDownloadAttributeMap(attributes);
	}

	/**
	 * @return HashMap return the attribute Map for Item Download
	 */
	
	private HashMap getDownloadAttributeMap(Attribute[] attributes) {
		
		for (int i = 0; i < attributes.length; i++) 
		{
			if (attributes[i] != null) 
			{
				if (attributes[i].getType() == AttributeTypes.GROUP) 
				{
					getDownloadAttributeMap(((GroupAttribute) attributes[i]).getAttributes());
				} else 
				{
					if(attributes[i].isLocalized()) {
						Attribute[] attrTempArray = new Attribute[attributes[i].getLocalizedValues().size()];
						for (int j = 0; j < attributes[i].getLocalizedValues().size(); j++) {
							attrTempArray[j] = (Attribute)attributes[i].getLocalizedValues().get(j);
						}
						getDownloadAttributeMap(attrTempArray);
					} else 
					{
						downloadAttributeMap.put(attributes[i].getName(),attributes[i]);	
					}
				}
			}
		}

		return downloadAttributeMap;
		
	}	
	
	/**
	 * @return Returns the bworkflowItem.
	 */
	public boolean isBworkflowItem() {
		return bworkflowItem;
	}
	/**
	 * @param bworkflowItem The bworkflowItem to set.
	 */
	public void setBworkflowItem(boolean bworkflowItem) {
		this.bworkflowItem = bworkflowItem;
	}
	/**
	 * @return Returns the associateWFCatalogID.
	 */
	public String getAssociateWFCatalogID() {
		return associateWFCatalogID;
	}
	/**
	 * @param associateWFCatalogID The associateWFCatalogID to set.
	 */
	public void setAssociateWFCatalogID(String associateWFCatalogID) {
		this.associateWFCatalogID = associateWFCatalogID;
		createJSP = "/jsps/html/itemcatalog/" + associateWFCatalogID + "_Create.jsp";
		updateJSP = "/jsps/html/itemcatalog/" + associateWFCatalogID + "_Create.jsp";
		printJSP = "/jsps/html/itemcatalog/" + associateWFCatalogID + "_Create.jsp";
	}
}
