
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.items;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;

import org.w3c.dom.Element;

import com.ibm.ccd.ui.dataobjects.FileUploadResource;
import com.ibm.ccd.ui.dataobjects.NVPair;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.AttributeTypes;
import com.ibm.ccd.ui.dataobjects.attributes.BinaryAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.CurrencyAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.DateAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.FlagAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.GroupAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ImageAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ImageURLAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.IntegerAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.LookupTableAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberEnumAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.PasswordAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.PeriodAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.RelationshipAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.SequenceAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringEnumAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ThumbnailImageAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ThumbnailImageURLAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.TimeZoneAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.URLAttribute;
import com.ibm.ccd.ui.mediator.ISoapMessageKeys;
import com.ibm.ccd.ui.mediator.XMLHelper;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;


/**
 *  Provides the capabailiy of creating the item object by parsing the WPC CatalogItem details
 *  Note: This comp. has to optimized for parsing.
 */
public class CatalogItemParser extends CatalogSpecParser {
	
	
    public static final String COMP_NAME = "CatalogItemParser";

    /**
     *  Creates the CatalogItem by parsing the Catalog Spec.XML and CatalogItemdetails XML
     *  Note: CatalogSpec.XML contains mainly the metadata of the catalog, where as the
     *  CatalogItedetails.XML contains attribute values of the item.
     *
     * However the CatalogItem is modelled to represent the attributes both interms of metadata
     * and object data hence this uses CatalogSpecParser to parse the spec before populating the
     * item details.
     */
    public static ICatalogItem prepareItem(Element specElement,
        Element tradeItemElement) {
        // prepare the spec..
        ICatalogItem item = getCatalogSpecItem(specElement);

        // fill the item with attrib values
        prepareItem(item, tradeItemElement);

        return item;
    }

    public static void prepareItem(ICatalogItem item, Element tradeItemElement) {
        // fill the Item ID..
        String itemID = XMLHelper.getAttributeValue(tradeItemElement,
                ISoapMessageKeys.ID);
        item.setItemID(itemID);

        // fill the attributes now.				
        fillAttributes(item.getAttributeMap(), tradeItemElement);
    }

    ////////////////////////////////////////////////////////////
    ///// Utility methods for XML parsing //////////////////////
    ////////////////////////////////////////////////////////////
    protected static void fillAttributes(HashMap attributesMap,
        Element tradeItemElement) {
        //Element[] attributeElements = XMLHelper.getElementsInNS(tradeItemElement,ISoapMessageKeys.ATTRIBUTE);
        Element[] attribListElement = XMLHelper.getElementsInNS(tradeItemElement,
                ISoapMessageKeys.ATTRIBUTE_LIST);

        ArrayList list = new ArrayList();
        Element[] attributeElements = null;

        for (int i = 0; i < attribListElement.length; i++) {
            attributeElements = XMLHelper.getChildElementsInNS(attribListElement[i],
                    ISoapMessageKeys.ATTRIBUTE);

            for (int j = 0; j < attributeElements.length; j++) {
                list.add(attributeElements[j]);
            }
        }

        int listSize = list.size();
        attributeElements = new Element[listSize];
        attributeElements = (Element[]) list.toArray(attributeElements);

        if ((attributeElements == null) || (attributeElements.length == 0)) {
            if (IReference.COND_CMPL) {
                System.out.println(COMP_NAME +
                    ".fillAttributes(): Can't find the  Attribute elements");
            }

            return;
        }

        int size = attributeElements.length;

        for (int i = 0; i < size; i++) 
        {
            String name = XMLHelper.getAttributeValue(attributeElements[i], ISoapMessageKeys.ATTRIB_NAME);
            Attribute attribute = (Attribute) attributesMap.get(name);

            //Support for localizedAttribute
            Attribute localizedAttribute = null;
            if (null == attribute){
                String localizedName = name.substring(0,name.lastIndexOf("/"));
                attribute = (Attribute) attributesMap.get(localizedName);
                
                if (attribute != null && attribute.isLocalized()){
                    for (int j = 0; j<attribute.getLocalizedValues().size();j++){
                        if(((Attribute)attribute.getLocalizedValues().get(j)).getName().equalsIgnoreCase(name)){
                            localizedAttribute = (Attribute)attribute.getLocalizedValues().get(j);
                            attribute = localizedAttribute;
                            break;
                        }
                    }
                }
            }
            if (attribute == null && IReference.COND_CMPL) {
                System.out.println(COMP_NAME +
                                   ".fillAttributes(): Can't find the  Attribute " + name +
                                   " in the attribute map");
            }

            if (attribute == null) {
                continue;
            }

            fillAttribute(attributeElements[i], attribute);
        }
    }

    private static void fillAttribute(Element attributeElement,
        Attribute attribute) {
			String localeName = 
				attributeElement.getAttribute(ISoapMessageKeys.LOCALE_NAME);
			attribute.setLocaleName(localeName);

			String editableStr = 
				attributeElement.getAttribute(ISoapMessageKeys.EDITABLE);
			boolean editable = true;
			if ((!(UIUtils.isEmpty(editableStr))) && editableStr.equalsIgnoreCase("false")) {
				editable = false;
			}
			attribute.setEditable(editable);

        switch (attribute.getType()) {
        case AttributeTypes.STRING: {
            StringAttribute stringAttribute = (StringAttribute) attribute;

						
            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeElement);
                stringAttribute.setValues(values);
            } else {
                String value = XMLHelper.getFirstElementValueInNS(attributeElement,
                        ISoapMessageKeys.ATTRIB_VALUE);

                if (value != null) {
                	stringAttribute.removeValues();
                    stringAttribute.setValue(value);
                }
            }

            return;
        }

        case AttributeTypes.INTEGER: {
            IntegerAttribute integerAttribute = (IntegerAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeElement);
                integerAttribute.setValues(values);
            } else {
                String temp = XMLHelper.getFirstElementValueInNS(attributeElement,
                        ISoapMessageKeys.ATTRIB_VALUE);

                try {
                    if (temp != null) {
                    	integerAttribute.removeValues();
                        integerAttribute.setValue(temp);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            return;
        }

        case AttributeTypes.NUMBER: {
            NumberAttribute numberAttribute = (NumberAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeElement);
                numberAttribute.setValues(values);
            } else {
                String temp = XMLHelper.getFirstElementValueInNS(attributeElement,
                        ISoapMessageKeys.ATTRIB_VALUE);

                try {
                    if (temp != null) {
                    	numberAttribute.removeValues();
                        numberAttribute.setValue(temp);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            return;
        }

        case AttributeTypes.DATE: {
            DateAttribute dateAttribute = (DateAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeElement);
                dateAttribute.setValues(values);
            } else {
                String value = XMLHelper.getFirstElementValueInNS(attributeElement,
                        ISoapMessageKeys.ATTRIB_VALUE);

                if (value != null) {
                	dateAttribute.removeValues();
                    dateAttribute.setValue(value);
                }
            }

            return;
        }

        case AttributeTypes.FLAG: {
            FlagAttribute flagAttribute = (FlagAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeElement);
                flagAttribute.setValues(values);
            } else {
                String temp = XMLHelper.getFirstElementValueInNS(attributeElement,
                        ISoapMessageKeys.ATTRIB_VALUE);

                if (temp != null) {
                	flagAttribute.removeValues();
                    flagAttribute.setValue(temp);
                }
            }

            return;
        }

        case AttributeTypes.TIMEZONE: {
            TimeZoneAttribute timeZoneAttribute = (TimeZoneAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeElement);
                timeZoneAttribute.setValues(values);
            } else {
                String value = XMLHelper.getFirstElementValueInNS(attributeElement,
                        ISoapMessageKeys.ATTRIB_VALUE);

                if (value != null) {
                	timeZoneAttribute.removeValues();
                    timeZoneAttribute.setValue(value);
                }
            }

            return;
        }

        case AttributeTypes.CURRENCY: {
            CurrencyAttribute currencyAttribute = (CurrencyAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeElement);
                currencyAttribute.setValues(values);
            } else {
                String value = XMLHelper.getFirstElementValueInNS(attributeElement,
                        ISoapMessageKeys.ATTRIB_VALUE);

                if (value != null) {
                	currencyAttribute.removeValues();
                    currencyAttribute.setValue(value);
                }
            }

            return;
        }

        case AttributeTypes.PASSWORD: {
            PasswordAttribute passwordAttribute = (PasswordAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeElement);
                passwordAttribute.setValues(values);
            } else {
                String value = XMLHelper.getFirstElementValueInNS(attributeElement,
                        ISoapMessageKeys.ATTRIB_VALUE);

                if (value != null) {
                	passwordAttribute.removeValues();
                    passwordAttribute.setValue(value);
                }
            }

            return;
        }

		case AttributeTypes.PERIOD :
			// special element contains the nested attributes for the start and end periods
			{
				attribute.removeValues();
				PeriodAttribute periodAttribute = (PeriodAttribute) attribute;
					
				if (attribute.isMultiValued()) {
						Element[] values = getPeriodValues(attributeElement);
						for(int k = 0; k< values.length ; k++ ){
							String startDate  = XMLHelper.getAttributeValue(values[k],ISoapMessageKeys.ATTRIB_PERIOD_STARTDATE);
							String endDate  = XMLHelper.getAttributeValue(values[k],ISoapMessageKeys.ATTRIB_PERIOD_ENDDATE);
							periodAttribute.setValue(startDate , endDate);
						}
							
					} else {
						Element periodElement = XMLHelper.getFirstElementInNS(attributeElement,ISoapMessageKeys.PERIODVALUE);
								
						if (periodElement != null) {
							String startDate  = XMLHelper.getAttributeValue(periodElement,ISoapMessageKeys.ATTRIB_PERIOD_STARTDATE);
							String endDate  = XMLHelper.getAttributeValue(periodElement,ISoapMessageKeys.ATTRIB_PERIOD_ENDDATE);
							periodAttribute.removeValues();								
							periodAttribute.setValue(startDate, endDate);
						}else{
							System.out.println(COMP_NAME+" : Period element do not contain the start date,end date attributes");
						}
					}
				if (periodAttribute == null)
				{
					if(IReference.COND_CMPL)
					{
						System.out.println(COMP_NAME+" : Period element do not contain the sub-attributes");
					}
					return; 
				}
                  
				return;
			}
        case AttributeTypes.SEQUENCE: {
            SequenceAttribute sequenceAttribute = (SequenceAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeElement);
                sequenceAttribute.setValues(values);
            } else {
                String value = XMLHelper.getFirstElementValueInNS(attributeElement,
                        ISoapMessageKeys.ATTRIB_VALUE);

                if (value != null) {
                	sequenceAttribute.removeValues();
                    sequenceAttribute.setValue(value);
                }
            }

            return;
        }

        case AttributeTypes.RELATIONSHIP: {
            RelationshipAttribute relationshipAttribute = (RelationshipAttribute) attribute;
            NVPair[] values = getRelationValues(attributeElement);

            if (attribute.isMultiValued()) {
                relationshipAttribute.setValues(values);
            } else {
                if ((values != null) && (values.length > 0)) {
                	relationshipAttribute.removeValues();
                    relationshipAttribute.setValue(values[0]);
                }
            }

            return;
        }

        case AttributeTypes.BINARY: {
            BinaryAttribute binaryAttribute = (BinaryAttribute) attribute;
            FileUploadResource[] values = getFileUploadResourceValues(attributeElement);

            if (binaryAttribute.isMultiValued()) {
                binaryAttribute.setValues(values);
            } else {
                if ((values != null) && (values.length > 0)) {
                	binaryAttribute.removeValues();
                    binaryAttribute.setValue(values[0]);
                }
            }

            return;
        }

        case AttributeTypes.IMAGE: {
            ImageAttribute imageAttribute = (ImageAttribute) attribute;
            FileUploadResource[] values = getFileUploadResourceValues(attributeElement);

            if (imageAttribute.isMultiValued()) {
                imageAttribute.setValues(values);
            } else {
                if ((values != null) && (values.length > 0)) {
                	imageAttribute.removeValues();
                    imageAttribute.setValue(values[0]);
                }
            }

            return;
        }

        case AttributeTypes.THUMB_NAIL_IMAGE: {
            ThumbnailImageAttribute thumbnailImageAttribute = (ThumbnailImageAttribute) attribute;
            FileUploadResource[] values = getFileUploadResourceValues(attributeElement);

            if (thumbnailImageAttribute.isMultiValued()) {
                thumbnailImageAttribute.setValues(values);
            } else {
                if ((values != null) && (values.length > 0)) {
                	thumbnailImageAttribute.removeValues();
                    thumbnailImageAttribute.setValue(values[0]);
                }
            }

            return;
        }

        case AttributeTypes.URL: {
            URLAttribute urlAttribute = (URLAttribute) attribute;
            String[] values = getStringValues(attributeElement);

            if (urlAttribute.isMultiValued()) {
                urlAttribute.setValues(values);
            } else {
                if ((values != null) && (values.length > 0)) {
                	urlAttribute.removeValues();
                    urlAttribute.setValue(values[0]);
                }
            }

            return;
        }

        case AttributeTypes.IMAGE_URL: {
            ImageURLAttribute imageURLAttribute = (ImageURLAttribute) attribute;
            String[] values = getStringValues(attributeElement);

            if (imageURLAttribute.isMultiValued()) {
                imageURLAttribute.setValues(values);
            } else {
                if ((values != null) && (values.length > 0)) {
                	imageURLAttribute.removeValues();
                    imageURLAttribute.setValue(values[0]);
                }
            }

            return;
        }

        case AttributeTypes.THUMB_NAIL_IMAGE_URL: {
            ThumbnailImageURLAttribute thumbnailImageURLAttribute = (ThumbnailImageURLAttribute) attribute;
            String[] values = getStringValues(attributeElement);

            if (thumbnailImageURLAttribute.isMultiValued()) {
                thumbnailImageURLAttribute.setValues(values);
            } else {
                if ((values != null) && (values.length > 0)) {
                	thumbnailImageURLAttribute.removeValues();
                    thumbnailImageURLAttribute.setValue(values[0]);
                }
            }

            return;
        }

        case AttributeTypes.STRING_ENUMERATION: {
            StringEnumAttribute stringEnumAttribute = (StringEnumAttribute) attribute;
			String[] enumValues = null;
			
            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeElement);
                stringEnumAttribute.setValues(values);
            } else {
                String value = XMLHelper.getFirstElementValueInNS(attributeElement,
                        ISoapMessageKeys.ATTRIB_VALUE);

                if (value != null) {
                	stringEnumAttribute.removeValues();
                    stringEnumAttribute.setValue(value);
                }
            }
			//	must and should have only one enum						
			Element enumElement = XMLHelper.getFirstElementInNS(attributeElement,
					ISoapMessageKeys.ENUM);

			if (enumElement != null) {
				// get the allowed values.
				enumValues = getStringEnum(enumElement);
				//set the generated enum values 
				stringEnumAttribute.setEnumValues(enumValues);
			}
            return;
        }

        case AttributeTypes.NUMBER_ENUMERATION: {
            NumberEnumAttribute numberEnumAttribute = (NumberEnumAttribute) attribute;
			double[] enumValues = null;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeElement);
                numberEnumAttribute.setValues(values);
            } else {
                String temp = XMLHelper.getFirstElementValueInNS(attributeElement,
                        ISoapMessageKeys.ATTRIB_VALUE);

                try {
                    if (temp != null) {
                    	numberEnumAttribute.removeValues();
                        numberEnumAttribute.setValue(temp);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

			Element enumElement = XMLHelper.getFirstElementInNS(attributeElement,
					ISoapMessageKeys.ENUM);

			// must and should have only one enum
			if (enumElement != null) {
				// get the allowed values.
				enumValues = getNumberEnum(enumElement);
				//set the generated enum values
				numberEnumAttribute.setEnumDb(enumValues);
			}

            return;
        }

        case AttributeTypes.LOOKUP_TABLE: {
            LookupTableAttribute lookupTableAttribute = (LookupTableAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeElement);
                lookupTableAttribute.setValues(values);
            } else {
                String value = XMLHelper.getFirstElementValueInNS(attributeElement,
                        ISoapMessageKeys.ATTRIB_VALUE);

                if (value != null) {
                	lookupTableAttribute.removeValues();
                    lookupTableAttribute.setValue(value);
                }
            }

            return;
        }

        case AttributeTypes.GROUP: {
            GroupAttribute groupAttribute = (GroupAttribute) attribute;
            
            groupAttribute.setGroupIndex(-1);
            TrackAttribute trackAttrib = new TrackAttribute(groupAttribute,attributeElement); 
			
            Attribute[] attributes = null;
            Element groupAttribElement = null;
            Element groupElement[] = null;
			
            while(trackAttrib.hasNextGroup() && groupAttribute.hasNextGroup()) {
                groupAttribElement = (Element)trackAttrib.getNextGroupElement();
                groupElement = XMLHelper.getChildElementsInNS(groupAttribElement,ISoapMessageKeys.ATTRIBUTE);
                
                attributes = groupAttribute.nextGroup();
                
                if(trackAttrib.hasNextGroup()) {
                    groupAttribute.setValue(groupAttribute.cloneValue());
                }
                
                for(int i=0; i<groupElement.length; i++) 
                {
                    String name = XMLHelper.getAttributeValue(groupElement[i], ISoapMessageKeys.ATTRIB_NAME);
                    Integer pos = (Integer) groupAttribute.getAttributeMap().get(name);
                    
                    Attribute subAttribute = null;
                    if (pos == null)
                    {
                        // support localized attribute
                        String localizedName = name.substring(0,name.lastIndexOf("/"));
                        pos = (Integer) groupAttribute.getAttributeMap().get(localizedName);
                        subAttribute = attributes[pos.intValue()];
                        if (subAttribute.isLocalized())
                        {
                            for (int j = 0; j<subAttribute.getLocalizedValues().size();j++)
                            {
                                if(((Attribute)subAttribute.getLocalizedValues().get(j)).getName().equalsIgnoreCase(name))
                                {
                                    subAttribute = (Attribute)subAttribute.getLocalizedValues().get(j);
                                    break;
                                }
                            }
                        }
                    }
                    else
                    {
                        subAttribute = attributes[pos.intValue()];
                    }
                    if (subAttribute == null && IReference.COND_CMPL) 
                    {
                        System.out.println(COMP_NAME +
                                           ".fillAttributes(): Can't find the  Attribute " + name +
                                           " in the attribute map");
                    }
                    
                    if (subAttribute == null)
                    {
                        continue;
                    }
                    fillAttribute(groupElement[i], subAttribute);
                }
            }
 			
            break;
        }
			     
            //////////////
        } // end of switch	
    }

    
    public static String[] getStringValues(Element valuesElement) {
        Element[] valueElements = XMLHelper.getElementsInNS(valuesElement,
                ISoapMessageKeys.ATTRIB_VALUE);

        if (valueElements == null) {
            return null;
        }

        String[] valuesList = new String[valueElements.length];

        for (int i = 0; i < valuesList.length; ++i) {
            valuesList[i] = XMLHelper.getElementValue(valueElements[i]);
        }

        return valuesList;
    }

    /**
     * 
     * @deprecated This method is no longer used as all atributes will be stored as strings
     */
    public static URL[] getURLValues(Element valuesElement) {
        String[] urlStings = getStringValues(valuesElement);

        if ((urlStings == null) || (urlStings.length == 0)) {
            return null;
        }

        URL[] urls = new URL[urlStings.length];

        for (int i = 0; i < urlStings.length; ++i) {
            try {
                if (!UIUtils.isEmpty(urlStings[i])) {
                    urls[i] = new URL(urlStings[i]);
                }
            } catch (MalformedURLException mue) {
                mue.printStackTrace();
            }
        }

        return urls;
    }

    public static NVPair[] getRelationValues(Element valuesElement) {
        Element[] valuesElements = XMLHelper.getElementsInNS(valuesElement,
                ISoapMessageKeys.RELATION);

        if (valuesElements == null) {
            return null;
        }

        NVPair[] valuesList = new NVPair[valuesElements.length];

        for (int i = 0; i < valuesList.length; ++i) {
            String name = XMLHelper.getAttributeValue(valuesElements[i],
                    ISoapMessageKeys.RELATION_TO);
            String value = XMLHelper.getElementValue(valuesElements[i]);
            valuesList[i] = new NVPair(name, value);
        }

        return valuesList;
    }

    public static FileUploadResource[] getFileUploadResourceValues(
        Element valuesElement) {
        Element[] valuesElements = XMLHelper.getElementsInNS(valuesElement,
                ISoapMessageKeys.ATTRIB_VALUE);

        if (valuesElements == null) {
            return null;
        }

        FileUploadResource[] valuesList = new FileUploadResource[valuesElements.length];

        for (int i = 0; i < valuesList.length; ++i) {
            //valuesList[i] = CatalogSpecParser.getFileUploadResource(valuesElements[i]);
            valuesList[i] = new FileUploadResource(XMLHelper.getElementValue(
                        valuesElements[i]));
        }

        return valuesList;
    }

    /**
     * 
     * @deprecated This method is no longer used as all atributes will be stored as strings
     */
    public static String[] getIntegerValues(Element valuesElement) {
        Element[] intElements = XMLHelper.getElementsInNS(valuesElement,
                ISoapMessageKeys.ATTRIB_VALUE);

        if (intElements == null) {
            return null;
        }

        String[] valuesList = new String[intElements.length];

        for (int i = 0; i < valuesList.length; ++i) {
            try {
                valuesList[i] = XMLHelper.getElementValue(intElements[i]);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return valuesList;
    }

    /**
     * 
     * @deprecated This method is no longer used as all atributes will be stored as strings
     */
    public static String[] getNumberValues(Element valuesElement) {
        Element[] valuesElements = XMLHelper.getElementsInNS(valuesElement,
                ISoapMessageKeys.ATTRIB_VALUE);

        if (valuesElements == null) {
            return null;
        }

        String[] valuesList = new String[valuesElements.length];

        for (int i = 0; i < valuesList.length; ++i) {
            try {
            	if(!UIUtils.isEmpty(XMLHelper.getElementValue(
				valuesElements[i]))) {
					
	                valuesList[i] = XMLHelper.getElementValue(valuesElements[i]);
				}
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return valuesList;
    }
	public static Element[] getPeriodValues(Element attributeElement) {

		Element[] periodAttribues =
			XMLHelper.getElementsInNS(
				attributeElement,
				ISoapMessageKeys.PERIODVALUE);

		return periodAttribues;
	}

	static class TrackAttribute {
		GroupAttribute groupAttrib;
		Element element;
		Element[] subElements;
		int index;
		
		public TrackAttribute (GroupAttribute groupAttrib, 
				Element element) {
			this(groupAttrib,element,0);	
			}			
		public TrackAttribute (GroupAttribute groupAttrib, 
				Element element,int index) {
			this.groupAttrib = groupAttrib;
			this.element = element;
			this.index = index;
			Element values = XMLHelper.getFirstElementInNS(element, ISoapMessageKeys.ATTRIB_VALUES);
			this.subElements = XMLHelper.getChildElementsInNS(values,ISoapMessageKeys.GROUP_ATTTRIBUTE);
		}
	
		public GroupAttribute getGroup() {
			return groupAttrib;	
		}
		
		public Element getElement() {
			return element;
		}
	
		public int getIndex() {
			return index;
		}
		public void setIndex(int index) {
			this.index = index; 
		}
		
		public boolean hasNextGroup() {
			
			if(index < subElements.length)
				return true;
			else 
				return false;
		}
		public Element getNextGroupElement() {
			if(hasNextGroup()) {
				Element e =subElements[index];
				index++;
				return e;
				
			}
			return null;
		}
	}
}

	
