
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.items;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.ibm.ccd.ui.dataobjects.CategoryHierarchy;
import com.ibm.ccd.ui.dataobjects.Node;
import com.ibm.ccd.ui.mediator.ISoapMessageKeys;
import com.ibm.ccd.ui.mediator.XMLHelper;


/**
 * Provides the functionality to generate and parse xml from nodes and to nodes.
 */
public class CatalogNodeXMLHelper {
    /**
     *  Prepares the WPC specific XML request for getting the catalog children.
     */
    public static String prepareCtgChilrenRequestXML(String parentID) {
        StringBuffer requestXML = new StringBuffer();
        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.CATALOG_PARENT_ID).append(">")
                  .append(parentID).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.CATALOG_PARENT_ID).append(">");

        return requestXML.toString();
    }

    /**
     *  Prepares the WPC specific XML request for getting the catalog children.
     */
    public static String prepareCtgChilrenRequestXML(String parentCategory,
        String catalogName, String hierarchyName, String hierarychyType,
        String childrenSize) {
        StringBuffer requestXML = new StringBuffer();
        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.PARENT_CATEGORY).append(">")
                  .append(parentCategory).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.PARENT_CATEGORY).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.CATALOG_NAME).append(">")
                  .append(catalogName).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.CATALOG_NAME).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.CATEGORY_HIERARCHY)
                  .append(" " + ISoapMessageKeys.ATTRIB_NAME + "=\"")
                  .append(hierarchyName).append("\"")
                  .append(" " + ISoapMessageKeys.ATTRIB_TYPE + "=\"")
                  .append(hierarychyType).append("\"").append("/>")
                  .append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.CATALOG_CHILD_LENGTH).append(">")
                  .append(childrenSize).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.CATALOG_CHILD_LENGTH).append(">");

        return requestXML.toString();
    }

    public static CategoryHierarchy[] parseCategoryNode(Element ctgs) {
        if (ctgs == null) {
            return null;
        }

        Element[] childElements = XMLHelper.getElementsInNS(ctgs,
                ISoapMessageKeys.CATALOG);

        return getCategoryHierarychy(childElements);
    }

    private static CategoryHierarchy[] getCategoryHierarychy(Element[] catalogs) {
        if (catalogs == null) {
            return null;
        }

        CategoryHierarchy[] hierarchy = new CategoryHierarchy[catalogs.length];

        for (int i = 0; i < catalogs.length; i++) {
            Element catalogName = XMLHelper.getFirstElementInNS(catalogs[i],
                    ISoapMessageKeys.CATALOG_NAME);
            Element hier = XMLHelper.getFirstElementInNS(catalogs[i],
                    ISoapMessageKeys.CATEGORY_HIERARCHIES);

            NodeList hierList = hier.getChildNodes();
            System.out.println("catalogName " + catalogName);

            //System.out.println("Catalog value " + XMLHelper.getElementValue(catalogName));
            hierarchy[i] = new CategoryHierarchy();
            hierarchy[i].setCatalogName(XMLHelper.getElementValue(catalogName));

            String[] hierarchyNames = new String[hierList.getLength()];
            String[] hierarchyTypes = new String[hierList.getLength()];

            for (int j = 0; j < hierList.getLength(); j++) {
                if (hierList.item(j) != null) {
                    Element nodeHier = (Element) hierList.item(j);
                    hierarchyNames[j] = XMLHelper.getAttributeValue(nodeHier,
                            ISoapMessageKeys.ATTRIB_NAME);
                    hierarchyTypes[j] = XMLHelper.getAttributeValue(nodeHier,
                            ISoapMessageKeys.ATTRIB_TYPE);
                }
            }

            hierarchy[i].setHierarchy(hierarchyNames);
            hierarchy[i].setHierarchyType(hierarchyTypes);
        }

        return hierarchy;
    }

    /**
         *  Parses the XML and  Prepares the nodes
         */
    public static Node[] parseParentNode(Element ctgChildrenNode) {
        if (ctgChildrenNode == null) {
            return null;
        }

        String catalogName = XMLHelper.getFirstElementValueInNS(ctgChildrenNode,
                ISoapMessageKeys.CATALOG_NAME);
        String parentPath = XMLHelper.getFirstElementValue(ctgChildrenNode,
                ISoapMessageKeys.CATALOG_PARENT_PATH);
        Element hierarchyElement = XMLHelper.getFirstElementInNS(ctgChildrenNode,
                ISoapMessageKeys.CATEGORY_HIERARCHY);

        String hierarchyName = XMLHelper.getAttributeValue(hierarchyElement,
                ISoapMessageKeys.ATTRIB_NAME);
        String hierarchyType = XMLHelper.getAttributeValue(hierarchyElement,
                ISoapMessageKeys.ATTRIB_TYPE);

        CategoryHierarchy parentHier = new CategoryHierarchy(catalogName,
                hierarchyName, hierarchyType, parentPath);

        Element[] childElements = XMLHelper.getElementsInNS(ctgChildrenNode,
                ISoapMessageKeys.CATALOG_CHILD);

        Node[] childNodes = null;

        if (childElements != null) {
            childNodes = new Node[childElements.length];

            for (int i = 0; i < childElements.length; ++i) {
                String id = XMLHelper.getElementValue(XMLHelper.getFirstElementInNS(
                            childElements[i],
                            ISoapMessageKeys.CATALOG_CHILD_PATH));
                String dispName = XMLHelper.getElementValue(XMLHelper.getFirstElementInNS(
                            childElements[i],
                            ISoapMessageKeys.CATALOG_CHILD_DISP_NAME));
                ;

                String type = XMLHelper.getElementValue(XMLHelper.getFirstElementInNS(
                            childElements[i],
                            ISoapMessageKeys.CATALOG_CHILD_TYPE));
                ;

                childNodes[i] = new Node(id, dispName, type);
                childNodes[i].setCategoryHierarchy(parentHier);
            }
        } else {
            childNodes = new Node[0];
        }

        return childNodes;
    }
}
