
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.items;

import java.util.ArrayList;

import org.w3c.dom.Element;

import com.ibm.ccd.ui.dataobjects.FileUploadResource;
import com.ibm.ccd.ui.dataobjects.NVPair;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.AttributeTypes;
import com.ibm.ccd.ui.dataobjects.attributes.BinaryAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.CurrencyAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.DateAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.FlagAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.GroupAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ImageAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ImageURLAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.IntegerAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.LookupTableAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberEnumAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.PasswordAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.PeriodAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.RelationshipAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.SequenceAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringEnumAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ThumbnailImageAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ThumbnailImageURLAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.TimeZoneAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.URLAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.UnSupportedAttribute;
import com.ibm.ccd.ui.mediator.ISoapMessageKeys;
import com.ibm.ccd.ui.mediator.XMLHelper;
import com.ibm.ccd.ui.util.UIUtils;


/**
 *  Provides the capabailiy of creating the CatalogSpec object by parsing the WPC CatalogSpec. XML
 *  Note: This comp. has to optimized for parsing.
 */
public class CatalogSpecParser {
    public static final String COMP_NAME = "CatalogSpecParser";
    public static final String STRING_VALUE_TRUE = "true";

    /**
     *  Creates and returns the CatalogItem representing the spec. by parsing the catalogSpecXML
     */
    public static ICatalogItem getCatalogSpecItem(Element specElement) {
        String parentCategory = "";

        Element parentCategryElement = XMLHelper.getFirstElementInNS(specElement,
                ISoapMessageKeys.PARENT_CATEGORY);

        if (parentCategryElement != null) {
            parentCategory = XMLHelper.getElementValue(parentCategryElement);
        }

        String catalogID = "";
        Element catalogNameElement = XMLHelper.getFirstElementInNS(specElement,
                ISoapMessageKeys.CATALOG_NAME);

        if (catalogNameElement != null) {
            catalogID = XMLHelper.getElementValue(catalogNameElement);
        }

        String catalogVersion = "";

        //Element attribListElement = XMLHelper.getFirstElementInNS(specElement,ISoapMessageKeys.ATTRIBUTE_LIST);
        Element[] attribListElement = XMLHelper.getElementsInNS(specElement,
                ISoapMessageKeys.ATTRIBUTE_LIST);

        Attribute[] attributes = null;
        String isCategorySpecificAttrib = null;

        ArrayList list = new ArrayList();

        for (int i = 0; i < attribListElement.length; i++) {
            attributes = getCatalogAttribs(attribListElement[i]);

            isCategorySpecificAttrib = XMLHelper.getAttributeValue(attribListElement[i],
                    ISoapMessageKeys.ATTRIB_CATEGORY_SPECIFIC);
            setCategorySpecificAttribute(attributes, isCategorySpecificAttrib);

            for (int j = 0; j < attributes.length; j++) {
                list.add(attributes[j]);
            }
        }

        int size = list.size();

        attributes = new Attribute[size];
        attributes = (Attribute[]) list.toArray(attributes);

        //		if(IReference.COND_CMPL)
        //			{
        ////				System.out.println(COMP_NAME + ".getCatalogSpecItem(): Printing the attributes list::\n");
        //				for(int i=0; i< attributes.length; ++i)
        //				{
        //				   if(attributes[i] != null){ attributes[i].printMe();
        //      	 
        //				}
        //			}
        //			}
        return new CatalogItem(parentCategory, catalogID, catalogVersion,
            attributes);
    }

    /**
     *
     */
    public static Attribute[] getSpecAttributes(Element specElement) {
        String parentCategory = "";

        Element parentCategryElement = XMLHelper.getFirstElementInNS(specElement,
                ISoapMessageKeys.PARENT_CATEGORY);

        if (parentCategryElement != null) {
            parentCategory = XMLHelper.getElementValue(parentCategryElement);
        }

        String catalogID = "";
        Element catalogNameElement = XMLHelper.getFirstElementInNS(specElement,
                ISoapMessageKeys.CATALOG_NAME);

        if (catalogNameElement != null) {
            catalogID = XMLHelper.getElementValue(catalogNameElement);
        }

        String catalogVersion = "";

        //Element attribListElement = XMLHelper.getFirstElementInNS(specElement,ISoapMessageKeys.ATTRIBUTE_LIST);
        Element[] attribListElement = XMLHelper.getElementsInNS(specElement,
                ISoapMessageKeys.ATTRIBUTE_LIST);

        Attribute[] attributes = null;
        String isCategorySpecificAttrib = null;

        ArrayList list = new ArrayList();

        for (int i = 0; i < attribListElement.length; i++) {
            attributes = getCatalogAttribs(attribListElement[i]);

            isCategorySpecificAttrib = XMLHelper.getAttributeValue(attribListElement[i],
                    ISoapMessageKeys.ATTRIB_CATEGORY_SPECIFIC);
            setCategorySpecificAttribute(attributes, isCategorySpecificAttrib);

            for (int j = 0; j < attributes.length; j++) {
                list.add(attributes[j]);
            }
        }

        int size = list.size();

        attributes = new Attribute[size];
        attributes = (Attribute[]) list.toArray(attributes);

        return attributes;
    }

    ////////////////////////////////////////////////////////////////////////
    //////////////  Utility methods  to parse the XML //////////////////////
    ////////////////////////////////////////////////////////////////////////
    protected static boolean isPrimaryKeyAttribute(Element attributeElement) {
        boolean isPrimaryKey = false;
        String primaryKey = null;

        primaryKey = XMLHelper.getAttributeValue(attributeElement,
                ISoapMessageKeys.ATTRIB_PRIMARY_KEY).trim();

        /*
         * Change History : For CQ WPC00024568 
         */
        try{
        	isPrimaryKey = (!UIUtils.isEmpty(primaryKey)) ? (new Boolean(primaryKey)).booleanValue() : false;
        } catch(Exception e) {
        	e.printStackTrace();
        }
        /*
         * End Change History
         */
        
        return isPrimaryKey;
    }

    protected static void setCategorySpecificAttribute(Attribute[] attributes,
        String categorySpecAttrib) {
        if ((categorySpecAttrib != null) &&
                categorySpecAttrib.equalsIgnoreCase(STRING_VALUE_TRUE)) {
            for (int i = 0; i < attributes.length; i++) {
                attributes[i].setCategorySpecificAttribute(true);
            }
        }
    }

    protected static Attribute[] getCatalogAttribs(Element catalogSpecElement) {
        Element[] attribElements = XMLHelper.getChildElementsInNS(catalogSpecElement,
                ISoapMessageKeys.ATTRIBUTE);

        System.out.println("size sub = " + attribElements.length);

        Attribute[] attributes = new Attribute[attribElements.length];

        for (int i = 0; i < attribElements.length; i++) {
            attributes[i] = getAttribute(attribElements[i]);

            if (isPrimaryKeyAttribute(attribElements[i])) {
                attributes[i].setPrimaryKey(true);
            }
        }

        return attributes;
    }

    protected static Attribute[] getCatalogAttribs(Element catalogSpecElement,
        String attributeName) {
        Element[] attribElements = XMLHelper.getChildElementsInNS(catalogSpecElement,
                attributeName);
        System.out.println("size sub Group = " + attribElements.length);

        Attribute[] attributes = new Attribute[attribElements.length];

        for (int i = 0; i < attribElements.length; i++) {
            attributes[i] = getAttribute(attribElements[i]);

            if (isPrimaryKeyAttribute(attribElements[i])) {
                attributes[i].setPrimaryKey(true);
            }
        }

        return attributes;
    }

    protected static Attribute getAttribute(Element attributeElement) {
        int type;
        String name = null;
        boolean editable = true;
        int min_occurrence = 0;
        int max_occurrence = 1;
        int min_length = 10;
        int max_length = 1000;
        String displayName = null;
        boolean isLocalized = false;
        String localeName = "en_US'";

        String attribTypeValue = XMLHelper.getAttributeValue(attributeElement,
                ISoapMessageKeys.ATTRIB_TYPE);

        type = AttributeTypes.getType(attribTypeValue.trim());
        name = attributeElement.getAttribute(ISoapMessageKeys.ATTRIB_NAME);
        displayName = attributeElement.getAttribute(ISoapMessageKeys.DISPLAY_NAME);
        localeName = attributeElement.getAttribute(ISoapMessageKeys.LOCALE_NAME);

        try {
            min_occurrence = UIUtils.getIntValue(XMLHelper.getFirstElementValueInNS(
                        attributeElement, ISoapMessageKeys.MIN_OCCUREENCE));
        } catch (Exception e) {
        }

        try {
            max_occurrence = UIUtils.getIntValue(XMLHelper.getFirstElementValueInNS(
                        attributeElement, ISoapMessageKeys.MAX_OCCUREENCE));
        } catch (Exception e) {
        }

        try {
            min_length = UIUtils.getIntValue(XMLHelper.getFirstElementValueInNS(
                        attributeElement, ISoapMessageKeys.MIN_LENGTH));
        } catch (Exception e) {
        }

        try {
            max_length = UIUtils.getIntValue(XMLHelper.getFirstElementValueInNS(
                        attributeElement, ISoapMessageKeys.MAX_LENGTH));
        } catch (Exception e) {
        }

        try {
            editable = UIUtils.getFlagValue(XMLHelper.getFirstElementValueInNS(
                        attributeElement, ISoapMessageKeys.EDITABLE));
        } catch (Exception e) {
        }

        try {
        	String localeValue = XMLHelper.getFirstElementValueInNS(
			attributeElement, ISoapMessageKeys.IS_LOCALIZED);
            isLocalized = ("1".equalsIgnoreCase(localeValue))? true : false;
        } catch (Exception e) {
            isLocalized = false;
        }

        switch (type) {
        case AttributeTypes.STRING: {
            String defVal = XMLHelper.getFirstElementValueInNS(attributeElement,
                    ISoapMessageKeys.ATTRIB_DEF_VALUE);
            StringAttribute attr = new StringAttribute(name, defVal, editable,
                    min_length, max_length, min_occurrence, max_occurrence,
                    displayName);
            attr.setLocaleName(localeName);
            attr.setAsLocalized(isLocalized);

            if (attr.isLocalized()) {
                Attribute[] attribute = getCatalogAttribs(attributeElement,
                        ISoapMessageKeys.ATTRIBUTE);
                ArrayList al = new ArrayList();

                for (int i = 0; i < attribute.length; i++) {
                	// The locale specific attributes will borrow their displayName 
                	// from the Localized Attribute - their owner
                	attribute[i].setDisplayLabelName(attr.getDisplayLabelName());
                    al.add(i, attribute[i]);
                }

                attr.setLocalizedValues(al);
            }

            return attr;
        }

        case AttributeTypes.INTEGER: {
            String defVal = "";

            try {
                defVal = XMLHelper.getFirstElementValueInNS(
                            attributeElement, ISoapMessageKeys.ATTRIB_DEF_VALUE);
            } catch (Exception e) {
            }

            return new IntegerAttribute(name, defVal, editable, min_length,
                max_length, min_occurrence, max_occurrence, displayName);
        }

        case AttributeTypes.NUMBER: {
            String defVal = "";

            try {
                defVal = XMLHelper.getFirstElementValueInNS(
                            attributeElement, ISoapMessageKeys.ATTRIB_DEF_VALUE);
            } catch (Exception e) {
            }

            return new NumberAttribute(name, defVal, editable, min_length,
                max_length, min_occurrence, max_occurrence, displayName);
        }

        case AttributeTypes.DATE: {
            String defVal = null;
            defVal = XMLHelper.getFirstElementValueInNS(attributeElement,
                    ISoapMessageKeys.ATTRIB_DEF_VALUE);

            return new DateAttribute(name, defVal, editable, min_length,
                max_length, min_occurrence, max_occurrence, displayName);
        }

        case AttributeTypes.FLAG: {
            String defVal = XMLHelper.getFirstElementValueInNS(attributeElement,
                    ISoapMessageKeys.ATTRIB_DEF_VALUE);

            return new FlagAttribute(name, defVal, editable, min_length,
                max_length, min_occurrence, max_occurrence, displayName);
        }

        case AttributeTypes.TIMEZONE: {
            String defVal = null;

            defVal = XMLHelper.getFirstElementValueInNS(attributeElement,
                    ISoapMessageKeys.ATTRIB_DEF_VALUE);

            return new TimeZoneAttribute(name, defVal, editable, min_length,
                max_length, min_occurrence, max_occurrence, displayName);
        }

        case AttributeTypes.CURRENCY: {
            String defVal = null;
            defVal = XMLHelper.getFirstElementValueInNS(attributeElement,
                    ISoapMessageKeys.ATTRIB_DEF_VALUE);

            return new CurrencyAttribute(name, defVal, editable, min_length,
                max_length, min_occurrence, max_occurrence, displayName);
        }

        case AttributeTypes.PASSWORD: {
            String defVal = null;
            defVal = XMLHelper.getFirstElementValueInNS(attributeElement,
                    ISoapMessageKeys.ATTRIB_DEF_VALUE);

            return new PasswordAttribute(name, defVal, editable, min_length,
                max_length, min_occurrence, max_occurrence, displayName);
        }

        case AttributeTypes.PERIOD:// special element contains the nested attributes for the start and end periods
		{
			
			return new PeriodAttribute(name, "", "", editable,
				min_occurrence, max_occurrence, displayName);
		}

        case AttributeTypes.SEQUENCE: {
            String defVal = XMLHelper.getFirstElementValueInNS(attributeElement,
                    ISoapMessageKeys.ATTRIB_DEF_VALUE);

            return new SequenceAttribute(name, defVal, displayName);
        }

        case AttributeTypes.RELATIONSHIP: {
            NVPair defVal = null;
            String[] enumValues = null;

            Element enumElement = XMLHelper.getFirstElementInNS(attributeElement,
                    ISoapMessageKeys.ENUM);

            if (enumElement != null) {
                defVal = new NVPair();

                // get the default value								
                defVal.setName(XMLHelper.getAttributeValue(enumElement,
                        ISoapMessageKeys.DEF));
                defVal.setValue(XMLHelper.getFirstElementValueInNS(
                        attributeElement, ISoapMessageKeys.ATTRIB_DEF_VALUE));

                // get the allowed values.
                enumValues = getStringEnum(enumElement);
            }

            return new RelationshipAttribute(name, enumValues, defVal, editable,
                min_occurrence, max_occurrence, displayName);
        }

        case AttributeTypes.BINARY: {
            FileUploadResource defVal = null;

            Element fileElement = XMLHelper.getFirstElementInNS(attributeElement,
                    ISoapMessageKeys.FILE);

            if (fileElement != null) {
                defVal = getFileUploadResource(fileElement);
            }

            return new BinaryAttribute(name, defVal, editable, min_length,
                max_length, min_occurrence, max_occurrence, displayName);
        }

        case AttributeTypes.IMAGE: {
            FileUploadResource defVal = null;

            Element fileElement = XMLHelper.getFirstElementInNS(attributeElement,
                    ISoapMessageKeys.FILE);

            if (fileElement != null) {
                defVal = getFileUploadResource(fileElement);
            }

            return new ImageAttribute(name, defVal, editable, min_length,
                max_length, min_occurrence, max_occurrence, displayName);
        }

        case AttributeTypes.THUMB_NAIL_IMAGE: {
            FileUploadResource defVal = null;

            Element fileElement = XMLHelper.getFirstElementInNS(attributeElement,
                    ISoapMessageKeys.FILE);

            if (fileElement != null) {
                defVal = getFileUploadResource(fileElement);
            }

            return new ThumbnailImageAttribute(name, defVal, editable,
                min_length, max_length, min_occurrence, max_occurrence,
                displayName);
        }

        case AttributeTypes.URL: {
        //    URL defVal = null;
            String urlStr = XMLHelper.getFirstElementValueInNS(attributeElement,
                    ISoapMessageKeys.ATTRIB_DEF_VALUE);

//            if ((urlStr != null) && (urlStr.length() > 0)) {
//                try {
//                    defVal = new URL(urlStr);
//                } catch (MalformedURLException mfe) {
//                    mfe.printStackTrace();
//                }
//            }

            return new URLAttribute(name, urlStr, editable, min_length,
                max_length, min_occurrence, max_occurrence, displayName);
        }

        case AttributeTypes.IMAGE_URL: {
//            URL defVal = null;
            String urlStr = XMLHelper.getFirstElementValueInNS(attributeElement,
                    ISoapMessageKeys.ATTRIB_DEF_VALUE);

//            if ((urlStr != null) && (urlStr.length() > 0)) {
//                try {
//                    defVal = new URL(urlStr);
//                } catch (MalformedURLException mfe) {
//                    mfe.printStackTrace();
//                }
//            }

            return new ImageURLAttribute(name, urlStr, editable, min_length,
                max_length, min_occurrence, max_occurrence, displayName);
        }

        case AttributeTypes.THUMB_NAIL_IMAGE_URL: {
//            URL defVal = null;
            String urlStr = XMLHelper.getFirstElementValueInNS(attributeElement,
                    ISoapMessageKeys.ATTRIB_DEF_VALUE);

//            if ((urlStr != null) && (urlStr.length() > 0)) {
//                try {
//                    defVal = new URL(urlStr);
//                } catch (MalformedURLException mfe) {
//                    mfe.printStackTrace();
//                }
//            }

            return new ThumbnailImageURLAttribute(name, urlStr, editable,
                min_length, max_length, min_occurrence, max_occurrence,
                displayName);
        }

        case AttributeTypes.STRING_ENUMERATION: {
            String defVal = null;
            String[] enumValues = null;

            //	must and should have only one enum						
            Element enumElement = XMLHelper.getFirstElementInNS(attributeElement,
                    ISoapMessageKeys.ENUM);

            if (enumElement != null) {
                // get the default value
                
                defVal = enumElement.getAttribute(ISoapMessageKeys.DEF);

                // get the allowed values.
                enumValues = getStringEnum(enumElement);
            }

            return new StringEnumAttribute(name, enumValues, defVal, editable,
                min_occurrence, max_occurrence, displayName);
        }

        case AttributeTypes.NUMBER_ENUMERATION: {
            String defVal = "";
            double[] enumValues = null;

            //	must and should have only one enumValues						
            Element enumElement = XMLHelper.getFirstElementInNS(attributeElement,
                    ISoapMessageKeys.ENUM);

            // must and should have only one enum
            if (enumElement != null) {
                // get the default value
                try {
					if(!UIUtils.isEmpty(enumElement.getAttribute(ISoapMessageKeys.DEF))){
						defVal = enumElement.getAttribute(
									ISoapMessageKeys.DEF);
					}                	
                } catch (Exception e) {
                    e.printStackTrace();
                }

                // get the allowed values.
                enumValues = getNumberEnum(enumElement);
            }

            return new NumberEnumAttribute(name, enumValues, defVal, editable,
                min_occurrence, max_occurrence, displayName);
        }

        case AttributeTypes.LOOKUP_TABLE: {
            String defVal = null;
            String lookupTableName=null;
            NVPair[] nvPair = null;
			Element  tableElement=null;

            // must and should have only one enum	
            Element lookupElement = XMLHelper.getFirstElementInNS(attributeElement,
                    ISoapMessageKeys.ENTRIES);

			lookupTableName=XMLHelper.getFirstElementValueInNS(attributeElement, ISoapMessageKeys.LOOKUP_TABLE_NAME);
            if (lookupElement != null) {
                // get the default value
                defVal = XMLHelper.getAttributeValue(lookupElement,
                        ISoapMessageKeys.DEF);
				
                // get the allowed values.
                nvPair = getLookupTable(lookupElement);
            }

            return new LookupTableAttribute(name, nvPair, defVal, editable,
                min_occurrence, max_occurrence, displayName,lookupTableName);
        }

        case AttributeTypes.GROUP: {
            String defVal = null;

            Attribute[] attribute = getCatalogAttribs(attributeElement,
                    ISoapMessageKeys.ATTRIBUTE);

            return new GroupAttribute(name, editable, min_occurrence,
                max_occurrence, attribute, displayName);
        }

        default:

            //   must be unsupported type
            return new UnSupportedAttribute(name, editable, min_occurrence,
                max_occurrence, displayName);
        } // end of the switch..
    }

    protected static final FileUploadResource getFileUploadResource(
        Element fileElement) {
        String type = XMLHelper.getAttributeValue(fileElement,
                ISoapMessageKeys.TYPE);

        String fName = XMLHelper.getFirstElementValueInNS(fileElement,
                ISoapMessageKeys.NAME);
        String dir = XMLHelper.getFirstElementValueInNS(fileElement,
                ISoapMessageKeys.DIRECTORY);
        String serverName = XMLHelper.getFirstElementValueInNS(fileElement,
                ISoapMessageKeys.SERVERNAME);
        String userID = XMLHelper.getFirstElementValueInNS(fileElement,
                ISoapMessageKeys.USER_ID);
        String password = XMLHelper.getFirstElementValueInNS(fileElement,
                ISoapMessageKeys.PASSWORD);

        return new FileUploadResource(type, fName, dir, serverName, userID,
            password);
    }

    protected static final String[] getStringEnum(Element enumElement) {
        Element[] enumValueElements = XMLHelper.getElementsInNS(enumElement,
                ISoapMessageKeys.VALUE);

        if (enumValueElements == null) {
            return null;
        }

        String[] enumValueList = new String[enumValueElements.length];

        for (int i = 0; i < enumValueList.length; ++i) {
            enumValueList[i] = XMLHelper.getElementValue(enumValueElements[i]);
        }

        return enumValueList;
    }

    protected static final double[] getNumberEnum(Element enumElement) {
        Element[] enumValueElements = XMLHelper.getElementsInNS(enumElement,
                ISoapMessageKeys.VALUE);

        if (enumValueElements == null) {
            return null;
        }

        double[] enumValueList = new double[enumValueElements.length];

        for (int i = 0; i < enumValueList.length; ++i) {
            try {
                enumValueList[i] = UIUtils.getDoubleValue(XMLHelper.getElementValue(enumValueElements[i]));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return enumValueList;
    }

    private static final NVPair[] getLookupTable(Element lookupElement) {
        Element[] lookupValueElements = XMLHelper.getElementsInNS(lookupElement,
                ISoapMessageKeys.ENTRY);

        if (lookupValueElements == null) {
            return null;
        }

        NVPair[] lookValueList = new NVPair[lookupValueElements.length];

        for (int i = 0; i < lookValueList.length; ++i) {
            String key = XMLHelper.getAttributeValue(lookupValueElements[i],
                    ISoapMessageKeys.KEY);
            String value = XMLHelper.getElementValue(lookupValueElements[i]);
            lookValueList[i] = new NVPair(key, value);
        }

        return lookValueList;
    }
}
