/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
package com.ibm.ccd.ui.dataobjects.view;

import java.io.IOException;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.StringTokenizer;

import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.AttributeTypes;

/**
 * @author Senthil G Kumar
 *
 * AttributeCollection consists of collection name and list of 
 * attributes belong. There can one or more attribute collection 
 * associated with a tab.
 * 
 */

public class AttributeCollectionView implements Serializable {

	private String collectionName;

	private boolean isEditable;

	private String type;

	private String[] attributeNames;

	private Attribute[] attributes;

	/**
	 * @return attributeNames
	 */
	public String[] getAttributeNames() {
		return attributeNames;
	}

	/**
	 * @return collectionName
	 */
	public String getCollectionName() {
		return collectionName;
	}

	/**
	 * @return isEditable
	 */
	public boolean isEditable() {
		return isEditable;
	}

	/**
	 * @return type
	 */
	public String getType() {
		return type;
	}

	/**
	 * @param attribute
	 */
	public void setAttributeNames(String[] attribute) {
		attributeNames = attribute;
	}

	/**
	 * @param collection
	 */
	public void setCollectionName(String collection) {
		collectionName = collection;
	}

	/**
	 * @param editable
	 */
	public void setEditable(boolean editable) {
		isEditable = editable;
	}

	/**
	 * @param string
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * @return
	 */
	public Attribute[] getAttributes() {
		return attributes;
	}

	/**
	 * @param attributes
	 */
	public void setAttributes(Attribute[] attributes) {
		this.attributes = attributes;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return getCollectionName();
	}

	/**
	 * print the attribute names in AttributeCollection 
	 *
	 */
	public void printME() {
		for (int i = 0; i < attributeNames.length; i++) {
			System.out.print(attributeNames[i] + ", ");
		}
	}

	/**
	 * print the JSP render Attribute tag for the AttributeCollection
	 * 
	 * @param out
	 */
	public void printME(OutputStream out) {

		String val = renderAttributeCollection();
		try {
			out.write(val.getBytes("UTF-8"));
		} catch (IOException e) {
			e.printStackTrace();
		}

	}
	/**
	 * rendering logic for the attributes and the group attributes 
	 * which is nested in.
	 *  
	 * @return JSP content for AttributeCollection
	 */
	public String renderAttributeCollection() {
		StringBuffer buffer = new StringBuffer();
		String strAddtnAttrib = "";
		int count = 1;
		int curr = 1;
		for (int i = 0; i < attributeNames.length; i++) {
			
			curr = getGroupDepth(attributes[i].getName());
			if (count > curr) {
				for(int k=0;k<count-curr;k++) {
					loopIndent(buffer, curr);
					buffer.append("</wpcui:groupAttribute>");
				}
			}

			if (attributes[i].getType() != AttributeTypes.GROUP) {
				if (curr > 1)
					loopIndent(buffer, curr + 1);
				else
					loopIndent(buffer, curr);
				
				if (attributes[i].getType() == AttributeTypes.IMAGE
						|| attributes[i].getType() == AttributeTypes.BINARY
						|| attributes[i].getType() == AttributeTypes.THUMB_NAIL_IMAGE) {
					strAddtnAttrib = " preview =\"true\" ";
				} else if (attributes[i].getType() == AttributeTypes.DATE){
					strAddtnAttrib = " parentformname =\"workCatalogForm\" ";
				}

				buffer.append(
					"<wpcui:attribute attribname=\""
						+ attributes[i].getName()+ "\" "
						+ strAddtnAttrib
						+ ((!isEditable)? "editable=\"false\"/>" : "/>")
						);  

			} else {
				loopIndent(buffer, curr + 1);
				buffer.append(
					"<wpcui:groupAttribute  attribname=\""
						+ attributes[i].getName()
						+ "\">");
			}
			count = curr;
		}

		for (int i = 1; i < count; i++) {
			loopIndent(buffer, curr--);
			buffer.append("</wpcui:groupAttribute>");
		}
		return buffer.toString();
	}

	/**
	 *  diplay the specified attribute with indent
	 * 
	 * @param buffer
	 * @param curr
	 */
	private void loopIndent(StringBuffer buffer, int curr) {
		buffer.append("\n");
		for (int i = 1; i <= curr; i++) {
			buffer.append("\t");
		}
	}

	/**
	 * display the specified group attribute with indent
	 *  
	 * @param name
	 * @return
	 */
	public int getGroupDepth(String name) {
		StringTokenizer token = new StringTokenizer(name, "/");
		return token.countTokens() - 1;
	}

}
