/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package com.ibm.ccd.ui.dataobjects.view;

import java.io.IOException;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import org.w3c.dom.Element;

import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.AttributeTypes;
import com.ibm.ccd.ui.dataobjects.attributes.GroupAttribute;
import com.ibm.ccd.ui.dataobjects.items.CatalogItem;
import com.ibm.ccd.ui.mediator.XMLHelper;
import com.ibm.ccd.ui.util.UIUtils;

/**
 * CatalogViewMaster contains container name, view name and the list of 
 * tabs associated with the container.
 *  
 *  @author Senthil G Kumar
 *
 */
public class CatalogViewMaster implements Serializable {

	private String containerViewName;

	private String containerName;

	private TabView tabGroup[];
	
	private boolean isTabGrouped;

	private static CatalogViewMaster viewMaster;

	/**
	 * 
	 * @param containerViewName
	 * @param containerName
	 * @param tabGroup
	 */
	protected CatalogViewMaster(
		String containerViewName,
		String containerName,
		TabView[] tabGroup) {
		this.containerViewName = containerViewName;
		this.containerName = containerName;
		this.tabGroup = tabGroup;
	}


	/**
	 * Transforms the attribute object to catalog view  
	 * @param catalogItem
	 */
	public void transformView(CatalogItem catalogItem) {
		HashMap attributeMap = catalogItem.getFlatAttributeMap();
		AttributeCollectionView[] colView;
		String[] attribNames = null;
		Attribute[] attributes = null;
		
		if(tabGroup == null) return ;
		
		for (int i = 0; i < tabGroup.length; i++) {
			colView = tabGroup[i].getCollection();

			for (int j = 0; j < colView.length; j++) {
				attribNames = colView[j].getAttributeNames();
				attributes = new Attribute[attribNames.length];
				colView[j].setAttributes(attributes);

				for (int k = 0; k < attribNames.length; k++) {
					attributes[k] =
						(Attribute) attributeMap.get(attribNames[k]);
						if(attributes[k].getType() == AttributeTypes.GROUP) {
							GroupAttribute g = ((GroupAttribute)attributes[k]);
							g.setGroupIndex(-1);
						}
						
				}

			}
		}
	}

	/**
	 * Rearrange the attribute collection attributes in the order it found in
	 * CatalogItem  
	 * @param catalogItem
	 */
	public void rearrangeAttributes(CatalogItem catalogItem) {
		ArrayList keyList0 = catalogItem.getFlatAttributeList();
		ArrayList keyList = null;
		
		if(keyList0 == null) {
			return;
		}
		
		if(tabGroup == null) return ;
		
		keyList = (ArrayList) UIUtils.copy(keyList0);
		
		AttributeCollectionView[] colView;
		String[] attribNames = null;
	
		for (int i = 0; i < tabGroup.length; i++) {
			colView = tabGroup[i].getCollection();
	
			for (int j = 0; j < colView.length; j++) {
				attribNames = colView[j].getAttributeNames();
				ArrayList list = getArrayList(attribNames);
				ArrayList keyList1 = (ArrayList) UIUtils.copy(keyList);
	
				keyList1.retainAll(list);
				
				//keyList.removeAll(list);
				
				String[] str = new String[keyList1.size()];
				keyList1.toArray(str);
				colView[j].setAttributeNames(str);
			}
	
		}
	
	}

	/**
	 * converts the object array to Arraylist Object
	 * @param obj
	 * @return
	 */
	private ArrayList getArrayList(Object[] obj) {
		ArrayList list = new ArrayList();
		if (obj != null) {

			for (int i = 0; i < obj.length; i++) {
				list.add(obj[i]);
			}

			return list;
		} else {
			return list;
		}
	}

	/**
	 * @return containerName
	 */
	public String getContainerName() {
		return containerName;
	}

	/**
	 * @return containerViewName
	 */
	public String getContainerViewName() {
		return containerViewName;
	}

	/**
	 * @return tabGroup
	 */
	public TabView[] getTabGroup() {
		return tabGroup;
	}

	/**
	 * @param string containerName
	 */
	public void setContainerName(String string) {
		containerName = string;
	}

	/**
	 * @param string containerViewName
	 */
	public void setContainerViewName(String string) {
		containerViewName = string;
	}

	/**
	 * @param views tabviews
	 */
	public void setTabGroup(TabView[] views) {
		tabGroup = views;
	}

	/**
	 * printME prints the Container Name, ViewName and TabGroupName in the 
	 * standard output 
	 */
	public void printME() {
		System.out.println("Container View Name : " + containerViewName);
		System.out.println("Container Name      : " + containerName);

		for (int i = 0; i < tabGroup.length; i++) {
			tabGroup[i].printME();
		}
	}

	/**
	 * printME prints the Container Name, ViewName and TabGroupName in the
	 * OutputStream
	 * @param out
	 */
	public void printME(OutputStream out) {
		StringBuffer buffer = new StringBuffer();

		buffer.append("<div id=\"tabControl\" class=\"tab-container\">");
		
		if(tabGroup == null) return;
		
		if(isTabGrouped) {
			for (int i = 0; i < tabGroup.length; i++) {
				buffer.append(
					"<a href=\"#\" onClick=\"submitForm(document.workCatalogForm,'<%=WorkwithItemPortlet.createCommand(portletResponse.createURI(), tab_submit, ICatalogAction.ACTIVE_TAB, \""
						+ (i + 1)
						+ "\")%>')\" id=\"tab"
						+ (i + 1)
						+ "\" class='<%=activeTab["
						+ i
						+ "]%>'><span style=\"COLOR: #444444\">"
						+ tabGroup[i].getTabName()
						+ "</span></a>");
			}
		}

		buffer.append("\n<div id=\"tabPane\" class=\"tab-pane\">\n");

		try {
			out.write(buffer.toString().getBytes("UTF-8"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		if (isTabGrouped) {
			for (int i = 0; i < tabGroup.length; i++) {
				if (i == 1) {
					tabGroup[i].printME(out, i, true);
				} else {
					tabGroup[i].printME(out, i);
				}
			}
		} else {
			for (int i = 0; i < tabGroup.length; i++) {
				if (i == 0)
					tabGroup[i].printME(out, i, true);
				else
					tabGroup[i].printME(out, i);
			}
		}

		buffer.delete(0, buffer.length());
		buffer.append("\n</div>\n\n</div>");

		try {
			out.write(buffer.toString().getBytes("UTF-8"));
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	/**
	 * create the view JSP for the CatalogItem  
	 * @param item
	 * @param out
	 */
	public void createView(CatalogItem item, OutputStream out) {
		viewMaster.rearrangeAttributes(item);
		viewMaster.transformView(item);
		viewMaster.printME(out);
	}
	
	
	/**
	 * @return
	 */
	public boolean isTabGrouped() {
		return isTabGrouped;
	}

	/**
	 * @param b
	 */
	public void setTabGrouped(boolean b) {
		isTabGrouped = b;
	}

}
