/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package com.ibm.ccd.ui.dataobjects.view;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;

import com.ibm.ccd.ui.mediator.ISoapMessageKeys;
import com.ibm.ccd.ui.mediator.XMLHelper;

/**
 * Provides the functionality to parse the the CatalogView XML
 *   
 * @author Senthil G Kumar
 */
public class CatalogViewParser implements ICatalogViewKeys {

	/**
	 * 
	 * create and returns the CatalogViewMaster Object from the viewXML
	 * 
	 * @param viewResponseXML
	 * @return
	 */
	public static CatalogViewMaster getCatalogView(String viewResponseXML) {
		Element viewRootElement;
		viewRootElement = XMLHelper.getRootElementFromString(viewResponseXML);

		Element containerViewElement =
			XMLHelper.getFirstElementInNS(viewRootElement, VIEW_CONTAINER);

		String containerViewName =
			XMLHelper.getAttributeValue(
				containerViewElement,
				ISoapMessageKeys.NAME);

		String containerName =
			XMLHelper.getFirstElementValueInNS(
				viewRootElement,
				VIEW_CONTAINER_NAME);

		TabView[] tabGroup = getTabView(viewRootElement);

		return new CatalogViewMaster(
			containerViewName,
			containerName,
			tabGroup);
	}

	/**
	 * create and returns the CatalogViewMaster Object from XML Element
	 * @param viewRootElement
	 * @return
	 */
	public static CatalogViewMaster getCatalogView(Element viewRootElement) {
		//this.viewRootElement = viewRootElement;

		Element containerViewElement =
			XMLHelper.getFirstElementInNS(viewRootElement, VIEW_CONTAINER);

		String containerViewName =
			XMLHelper.getAttributeValue(
				containerViewElement,
				ISoapMessageKeys.NAME);

		String containerName =
			XMLHelper.getFirstElementValueInNS(
				viewRootElement,
				VIEW_CONTAINER_NAME);

		TabView[] tabGroup = null;
		CatalogViewMaster viewMaster = null;

		if (isTabGrouped(viewRootElement)) {
			tabGroup = getTabView(viewRootElement);
			viewMaster =
				new CatalogViewMaster(
					containerViewName,
					containerName,
					tabGroup);

			viewMaster.setTabGrouped(true);
		} else {
			tabGroup = getSingleView(viewRootElement);

			viewMaster =
				new CatalogViewMaster(
					containerViewName,
					containerName,
					tabGroup);

			viewMaster.setTabGrouped(false);

		}
		return viewMaster;	
	}
	
	/**
	 * create and returns the CatalogViewMaster Object from XML Element
	 * @param viewRootElement
	 * @return
	 */
	public static CatalogViewMaster getWorkFlowView(Element viewRootElement) {
		//this.viewRootElement = viewRootElement;

		Element containerViewElement =
			XMLHelper.getFirstElementInNS(viewRootElement, VIEW_CONTAINER);

		String containerViewName =
			XMLHelper.getAttributeValue(
				containerViewElement,
				ISoapMessageKeys.NAME);

		String containerName = "";
//			XMLHelper.getFirstElementValueInNS(
//				viewRootElement,
//				VIEW_CONTAINER_NAME);

		TabView[] tabGroup = null;
		CatalogViewMaster viewMaster = null;

		if (isTabGrouped(viewRootElement)) {
			tabGroup = getTabView(viewRootElement);
			viewMaster =
				new CatalogViewMaster(
					containerViewName,
					containerName,
					tabGroup);

			viewMaster.setTabGrouped(true);
		} else {
			tabGroup = getSingleView(viewRootElement);

			viewMaster =
				new CatalogViewMaster(
					containerViewName,
					containerName,
					tabGroup);

			viewMaster.setTabGrouped(false);

		}
		return viewMaster;	
	}
	
	
	/**
	 * return true if Attribute Collection is grouped under Tab
	 * @param root
	 * @return
	 */
	public static boolean isTabGrouped(Element root) {
		Element tabGroup = XMLHelper.getFirstElementInNS(root, TAB_GROUP);
		Element[] tabed = XMLHelper.getElementsInNS(tabGroup, TAB);
		if(tabed == null || tabed.length == 0) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * return one single view without tab 
	 * @param root
	 * @return
	 */
	protected static TabView[] getSingleView(Element root) {
		Element tabGroup = XMLHelper.getFirstElementInNS(root, TAB_GROUP);

		TabView[] tabView = new TabView[1];
		
		Element attribColl =
			XMLHelper.getFirstElementInNS(root, ICatalogViewKeys.ATTRIB_COLS);

		Element attribColls[] =
			XMLHelper.getChildElementsInNS(attribColl, ICatalogViewKeys.ATTRIB_LIST);
		
		// return null when there is no AttributeCollection 
		if(attribColls == null) return null;
		
		AttributeCollectionView[] collectionView =
			new AttributeCollectionView[attribColls.length];

		for (int i = 0; i < attribColls.length; i++) {
			String collectionName =
				XMLHelper.getFirstElementValueInNS(attribColls[i], ATTRIB_NAME);
			
			String isEditable =
				XMLHelper.getAttributeValue(
					attribColls[i],
					ISoapMessageKeys.EDITABLE);				

			String[] attribNames =
				getAttributeNames(root, collectionName);

			collectionView[i] = new AttributeCollectionView();
			collectionView[i].setCollectionName(collectionName);
			collectionView[i].setAttributeNames(attribNames);
			if (isEditable != null && isEditable.trim().equalsIgnoreCase("true")) {
				collectionView[i].setEditable(true);
			}
		}
		tabView[0] = new TabView(null,collectionView);
		
		return tabView;
	}
	
	/**
	 * Get TabView from the XML Element
	 * @param root
	 * @return
	 */
	protected static TabView[] getTabView(Element root) {

		Element tabGroup = XMLHelper.getFirstElementInNS(root, TAB_GROUP);

		Element[] tab = XMLHelper.getElementsInNS(tabGroup, TAB);
		
		// return null when there is no Tab associated with AttributeCollection
		if(tab == null) return null;

		String tabName;

		String attributeColName;

		TabView[] tabView = new TabView[tab.length];

		for (int i = 0; i < tab.length; i++) {

			tabName =
				XMLHelper.getAttributeValue(
					tab[i],
					ISoapMessageKeys.ATTRIB_NAME);
			AttributeCollectionView[] colView =
				getAllAttributeCollectionView(root, tab[i]);

			if(colView == null) return null;
			
			tabView[i] = new TabView(tabName, colView);

		}

		return tabView;
	}

	/**
	 * get the AttributeCollectionView object from XML Element
	 * @param tab
	 * @return
	 */
	protected static AttributeCollectionView[] getAllAttributeCollectionView(
		Element viewRootElement,
		Element tab) {
		Element[] attribColl = XMLHelper.getElementsInNS(tab, ATTRIB_COL);

		if(attribColl == null) return null;
		
		AttributeCollectionView[] attribColView =
			new AttributeCollectionView[attribColl.length];

		String collectionName;
		String isEditable;
		String type;
		String[] attributeNames;

		for (int i = 0; i < attribColl.length; i++) {
			collectionName = XMLHelper.getElementValue(attribColl[i]);
			isEditable =
				XMLHelper.getAttributeValue(
					attribColl[i],
					ISoapMessageKeys.EDITABLE);

			attribColView[i] = new AttributeCollectionView();

			attribColView[i].setCollectionName(collectionName);
			attribColView[i].setAttributeNames(
				getAttributeNames(viewRootElement, collectionName));

			if (isEditable.equalsIgnoreCase("true")) {
				attribColView[i].setEditable(true);
			}
		}
		return attribColView;
	}

	/**
	 * return the AttributeNames for the given AttributeCollection Element
	 * 
	 * @param viewRoot
	 * @param attributeColName
	 * @return
	 */
	protected static String[] getAttributeNames(
		Element viewRoot,
		String attributeColName) {

		Element attribColls =
			XMLHelper.getFirstElementInNS(viewRoot, ATTRIB_COLS);
		Element[] attribCollection =
			XMLHelper.getChildElementsInNS(attribColls, ATTRIB_LIST);

		String[] names = null;

		for (int i = 0; i < attribCollection.length; i++) {

			String attribColName =
				XMLHelper.getFirstElementValueInNS(
					attribCollection[i],
					ATTRIB_NAME);

			if (!attributeColName.equalsIgnoreCase(attribColName))
				continue;

			Element[] attributes =
				XMLHelper.getElementsInNS(attribCollection[i], ATTRIBUTE);

			names = new String[attributes.length];
			for (int j = 0; j < attributes.length; j++) {
				names[j] =
					XMLHelper.getAttributeValue(
						attributes[j],
						ISoapMessageKeys.NAME);
			}
		}

		return names;
	}

	public static List getCatalogViewNameList(Element viewRoot) {
		Element viewList =
			XMLHelper.getFirstElementInNS(viewRoot, CATALOG_VIEW_LIST);
			
		Element[] viewNames =
			XMLHelper.getElementsInNS(viewRoot, CATALOG_VIEW_NAME);

		List list = new ArrayList();
		String viewName = null;

		for (int i = 0; i < viewNames.length; i++) {
			viewName = XMLHelper.getElementValue(viewNames[i]);
			list.add(viewName);
		}

		return list;

	}

}
