
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.workflow;

import java.util.List;
import java.util.Map;

import com.ibm.ccd.ui.context.IWPCUser;
import com.ibm.ccd.ui.context.WPCUser;
import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.dataobjects.items.ICatalogItem;
import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.exception.UIException;
import com.ibm.ccd.ui.mediator.WPCMediator;
import com.ibm.ccd.ui.mediator.WPCMediatorException;
import com.ibm.ccd.ui.portlets.workflow.IWorkFlow;
import com.ibm.ccd.ui.util.UIUtils;


/**
 * WorkflowHelper class
 * All methods for processing the requests for Business Portlet
 */
public class WorkflowHelper implements IWorkflowHelper {
    /**
     * getting instance
     */
    public static IWorkflowHelper getInstance() {
        WorkflowHelper instance = new WorkflowHelper();
        IWorkflowHelper ref = (IWorkflowHelper) instance;

        return ref;
    }

    /**
     * Method for executing the request for fetching the data in workflow
     * @param workflowObj  Data object carrying all parameters for executing the request
     * @param messageMap Map for adding information
     * @param context user context object
     * @return
     */
    public void process(WorkFlowTO workflowObj, MessageMap messageMap,
        WPSContext context) {
        WPCUser wpcUser = context.getWpcUser();
        String userName = (String) wpcUser.getProperty(IWPCUser.KEY_WPC_USERNAME);
        String role = null;

        if (UIUtils.isEmpty(workflowObj.getTeamListContainer().getSelectedValue())) {
            workflowObj.getTeamListContainer().setSelectedValue(userName);
        }

        if (!((workflowObj.getTeamListContainer().getSelectedValue()).equals(
                    userName))) {
            role = workflowObj.getTeamListContainer().getSelectedValue();
        }

        String command = workflowObj.getCommand();
        IWorkFlowToOutput workflowOut = (IWorkFlowToOutput) workflowObj;
        WPCMediator mediator = WPCMediator.getInstance();

        try {
            if (command.equals(IWorkFlow.GO_TEAM_SELECT)) {
                ListboxContainer listContainer = null;
                List list = null;

                if ((workflowObj.getTeamListContainer().getSelectedValue()).equals(
                            userName)) {
                    list = mediator.getBusinessProcess(context);
                } else {
                    list = mediator.getBusinessProcess(context,
                            workflowObj.getTeamListContainer().getSelectedValue());
                }

                String selectedCollabName = null;

                if (list.size() == 0) {
                    messageMap.addInfoMessage("MESSAGE_CODE_516");
                    selectedCollabName = null;
                } else {
                    selectedCollabName = ((CollaborationArea) list.get(0)).getCollabName();
                    messageMap.addInfoMessage("MESSAGE_CODE_507");
                }

                listContainer = new ListboxContainer(list, selectedCollabName);
                workflowOut.setCollabAreaListContainer(listContainer);

                return;
            } else if (command.equals(IWorkFlow.GO_BUS_PROC_SELECT)) {
                
            	
            	workflowOut.setStepWorkflow(mediator.getSteps(context, role,
                        workflowObj.getCollabAreaListContainer()
                                   .getSelectedValue()));
                
                if (workflowObj.getStepWorkflow().getSteps().size() == 0) {
                    messageMap.addInfoMessage("MESSAGE_CODE_505");
                } else {
                    messageMap.addInfoMessage("MESSAGE_CODE_513");
                }

                return;
            } else if (command.equals(IWorkFlow.STEPNAME) ||
                    command.equals(IWorkFlow.SORT_TYPE)) {
                workflowObj.getPageNav().reset();

                Map dataMap = mediator.getStepItems(context, role,
                        workflowObj.getCollabAreaListContainer()
                                   .getSelectedValue(),
                        workflowObj.getStepWorkflow().getWorkflowName(),
                        workflowObj.getStepWorkflow().getSelectedStep()
                                   .getStepName(),  workflowObj.getPageNav(),
                        workflowObj.getAttributeList());

                workflowOut.setItemList((List) dataMap.get(
                        IWorkFlow.KEY_MAP_GET_ITEMS_DATASET1));

                int itemCount = (new Integer(dataMap.get(
                            IWorkFlow.KEY_MAP_GET_ITEMS_DATASET2).toString())).intValue();
                workflowObj.getStepWorkflow().getSelectedStep().setItemCount(itemCount);
                workflowObj.setCatalogAssociated((String) dataMap
                        .get(IWorkFlow.KEY_MAP_GET_ITEMS_DATASET3));

                if (itemCount < workflowObj.getPageNav().getItemsPerPage()) {
                    workflowObj.getPageNav().setEndIndex(itemCount);
                } else {
                    workflowObj.getPageNav().setEndIndex(workflowObj.getPageNav()
                                                                    .getItemsPerPage());
                }

                if (workflowObj.getItemList().size() == 0) {
                    messageMap.addInfoMessage("MESSAGE_CODE_506");
                }

                return;

                //			}else if(command.equals(IWorkFlow.SORT_COLUMN)) {
                //				workflowObj.getPageNav().reset();
                //				workflowOut.setItemList(mediator.getStepItems(context,role,
                //					workflowObj.getCollabAreaListContainer().getSelectedValue(),workflowObj.getStepWorkflow().getWorkflowName(),
                //					workflowObj.getStepWorkflow().getSelectedStep().getStepName(),workflowObj.getPageNav(),,workflowObj.getAttributeList()));
                //				return;
            } else if (command.equals(IWorkFlow.CATEGORYIDX)) {
                workflowObj.getPageNav().reset();

                Map dataMap = mediator.getStepItems(context, role,
                        workflowObj.getCollabAreaListContainer()
                                   .getSelectedValue(),
                        workflowObj.getStepWorkflow().getWorkflowName(),
                        workflowObj.getStepWorkflow().getSelectedStep()
                                   .getStepName(),workflowObj.getCategoryId(),
                                   workflowObj.getPageNav(),
                        workflowObj.getAttributeList());

                workflowOut.setItemList((List) dataMap.get(
                        IWorkFlow.KEY_MAP_GET_ITEMS_DATASET1));

                int itemCount = (new Integer(dataMap.get(
                            IWorkFlow.KEY_MAP_GET_ITEMS_DATASET2).toString())).intValue();
                workflowObj.getStepWorkflow().getSelectedStep().setItemCount(itemCount);
                workflowObj.setCatalogAssociated((String) dataMap
                        .get(IWorkFlow.KEY_MAP_GET_ITEMS_DATASET3));

                if (itemCount < workflowObj.getPageNav().getItemsPerPage()) {
                    workflowObj.getPageNav().setEndIndex(itemCount);
                } else {
                    workflowObj.getPageNav().setEndIndex(workflowObj.getPageNav()
                                                                    .getItemsPerPage());
                }

                if (workflowObj.getItemList().size() == 0) {
                    messageMap.addInfoMessage("MESSAGE_CODE_506");
                }

            } else if (command.equals(IWorkFlow.PREV_PAGE)) {
                int newendIndex = 0;
                int noofItemsLastPage = workflowObj.getPageNav().getEndIndex() -
                    workflowObj.getPageNav().getStartIndex();

                int newstartIndex = workflowObj.getPageNav().getStartIndex() -
                    workflowObj.getPageNav().getItemsPerPage();
                workflowObj.getPageNav().setStartIndex(newstartIndex);

                if (noofItemsLastPage < workflowObj.getPageNav()
                                                       .getItemsPerPage()) {
                    newendIndex = workflowObj.getPageNav().getEndIndex() -
                        noofItemsLastPage;
                } else {
                    newendIndex = workflowObj.getPageNav().getEndIndex() -
                        workflowObj.getPageNav().getItemsPerPage();
                }

                workflowObj.getPageNav().setEndIndex(newendIndex);

                Map dataMap = mediator.getStepItems(context, role,
                        workflowObj.getCollabAreaListContainer()
                                   .getSelectedValue(),
                        workflowObj.getStepWorkflow().getWorkflowName(),
                        workflowObj.getStepWorkflow().getSelectedStep()
                                   .getStepName(),workflowObj.getCategoryId(), 
                                   workflowObj.getPageNav(),
                        workflowObj.getAttributeList());

                workflowOut.setItemList((List) dataMap.get(
                        IWorkFlow.KEY_MAP_GET_ITEMS_DATASET1));

                int itemCount = (new Integer(dataMap.get(
                            IWorkFlow.KEY_MAP_GET_ITEMS_DATASET2).toString())).intValue();
                workflowObj.getStepWorkflow().getSelectedStep().setItemCount(itemCount);
                workflowObj.setCatalogAssociated((String) dataMap
                        .get(IWorkFlow.KEY_MAP_GET_ITEMS_DATASET3));
               
                return;
            } else if (command.equals(IWorkFlow.NEXT_PAGE)) {
                int newstartIndex = workflowObj.getPageNav().getStartIndex() +
                    workflowObj.getPageNav().getItemsPerPage();
                workflowObj.getPageNav().setStartIndex(newstartIndex);

                int newendIndex = workflowObj.getPageNav().getEndIndex() +
                    workflowObj.getPageNav().getItemsPerPage();

                if (newendIndex > workflowObj.getStepWorkflow().getSelectedStep()
                                                 .getItemCount()) {
                    newendIndex = workflowObj.getStepWorkflow().getSelectedStep()
                                             .getItemCount();
                }

                workflowObj.getPageNav().setEndIndex(newendIndex);

                Map dataMap = mediator.getStepItems(context, role,
                        workflowObj.getCollabAreaListContainer()
                                   .getSelectedValue(),
                        workflowObj.getStepWorkflow().getWorkflowName(),
                        workflowObj.getStepWorkflow().getSelectedStep()
                                   .getStepName(),workflowObj.getCategoryId(),
                                   workflowObj.getPageNav(),
                        workflowObj.getAttributeList());

                workflowOut.setItemList((List) dataMap.get(
                        IWorkFlow.KEY_MAP_GET_ITEMS_DATASET1));

                int itemCount = (new Integer(dataMap.get(
                            IWorkFlow.KEY_MAP_GET_ITEMS_DATASET2).toString())).intValue();
                workflowObj.getStepWorkflow().getSelectedStep().setItemCount(itemCount);
                workflowObj.setCatalogAssociated((String) dataMap
                        .get(IWorkFlow.KEY_MAP_GET_ITEMS_DATASET3));
              
                return;
            } else if (command.equals(IWorkFlow.FIRST_PAGE)) {
                int newstartIndex = IWorkFlow.START_ITEM_INDEX;
                workflowObj.getPageNav().setStartIndex(newstartIndex);

                int newendIndex = workflowObj.getPageNav().getItemsPerPage();
                workflowObj.getPageNav().setEndIndex(newendIndex);

                Map dataMap = mediator.getStepItems(context, role,
                        workflowObj.getCollabAreaListContainer()
                                   .getSelectedValue(),
                        workflowObj.getStepWorkflow().getWorkflowName(),
                        workflowObj.getStepWorkflow().getSelectedStep()
                                   .getStepName(), workflowObj.getCategoryId(),
                                   workflowObj.getPageNav(),
                        workflowObj.getAttributeList());
                workflowOut.setItemList((List) dataMap.get(
                        IWorkFlow.KEY_MAP_GET_ITEMS_DATASET1));

                int itemCount = (new Integer(dataMap.get(
                            IWorkFlow.KEY_MAP_GET_ITEMS_DATASET2).toString())).intValue();
                workflowObj.getStepWorkflow().getSelectedStep().setItemCount(itemCount);
                workflowObj.setCatalogAssociated((String) dataMap
                        .get(IWorkFlow.KEY_MAP_GET_ITEMS_DATASET3));
               
                return;
            } else if (command.equals(IWorkFlow.LAST_PAGE)) {
                int newstartIndex = 0;
                int newendIndex = 0;

                int itemsOnLastPage = workflowObj.getStepWorkflow()
                                                 .getSelectedStep()
                                                 .getItemCount() % workflowObj.getPageNav()
                                                                              .getItemsPerPage();
                int startIndex = 0;

                if (itemsOnLastPage == 0) {
                    startIndex = workflowObj.getStepWorkflow().getSelectedStep()
                                            .getItemCount() -
                        workflowObj.getPageNav().getItemsPerPage();
                } else {
                    startIndex = workflowObj.getStepWorkflow().getSelectedStep()
                                            .getItemCount() - itemsOnLastPage;
                }

                newstartIndex = startIndex;
                newendIndex = workflowObj.getStepWorkflow().getSelectedStep()
                                         .getItemCount();

                workflowObj.getPageNav().setStartIndex(newstartIndex);
                workflowObj.getPageNav().setEndIndex(newendIndex);

                Map dataMap = mediator.getStepItems(context, role,
                        workflowObj.getCollabAreaListContainer()
                                   .getSelectedValue(),
                        workflowObj.getStepWorkflow().getWorkflowName(),
                        workflowObj.getStepWorkflow().getSelectedStep()
                                   .getStepName(), workflowObj.getCategoryId(),
                                   workflowObj.getPageNav(),
                        workflowObj.getAttributeList());
                workflowOut.setItemList((List) dataMap.get(
                        IWorkFlow.KEY_MAP_GET_ITEMS_DATASET1));

                int itemCount = (new Integer(dataMap.get(
                            IWorkFlow.KEY_MAP_GET_ITEMS_DATASET2).toString())).intValue();
                workflowObj.getStepWorkflow().getSelectedStep().setItemCount(itemCount);
                workflowObj.setCatalogAssociated((String) dataMap
                        .get(IWorkFlow.KEY_MAP_GET_ITEMS_DATASET3));
//                workflowObj.setCategoryInfoListFrom((Map) dataMap
//                		.get(IWorkFlow.KEY_MAP_GET_ITEMS_DATASET4));
                
                return;
            }
        } catch (WPCMediatorException e) {
            e.printStackTrace();
      
            messageMap.addErrorMessage("MESSAGE_CODE_106");
        }
    }

    /**
     * Method for executing the request for the actions done on items
     * @param workflowObj  Data object carrying all parameters for executing the request
     * @param messageMap Map for adding information
     * @param context user context object
     * @return
     */
    public void processAction(WorkFlowTO workflowObj, MessageMap messageMap,
        WPSContext context) throws WPCMediatorException {
        String command = workflowObj.getCommand();
        String role = workflowObj.getTeamListContainer().getSelectedValue();
        WPCMediator mediator = WPCMediator.getInstance();
        mediator.processAction(context, workflowObj);
    }

    /**
     * Method for executing the request for user roles
     * @param workflowObj  Data object carrying all parameters for executing the request
     * @param messageMap Map for adding information
     * @param context user context object
     * @return
     */
    public void getUserRoles(WorkFlowTO workflowObj, MessageMap messageMap,
        WPSContext context) throws WPCMediatorException {
        String command = workflowObj.getCommand();
        IWorkFlowToOutput workflowOut = (IWorkFlowToOutput) workflowObj;
        List list = null;
        list = WPCMediator.getInstance().getUserRoles(context);
        list.add(0, context.getWpcUser().getProperty(IWPCUser.KEY_WPC_USERNAME));

        // Get User option as selected
        workflowOut.setTeamListContainer(new ListboxContainer(list,
                (String) list.get(0)));
    }

    /**
     * Getting attributes from the selected catalog
     * @param context
     * @param catalogId
     * @return
     * @throws UIException
     */
    public Map getAttributes(WPSContext context, String catalogId,
        MessageMap messageMap) {
        List attributesList = null;
        ICatalogItem catalogItem = null;
        Map attributesMap = null;

        try {
            WPCMediator mediator = WPCMediator.getInstance();

            catalogItem = mediator.getCatalogItem("", catalogId, context);
            attributesMap = catalogItem.getFlatAttributeMap();
        } catch (WPCMediatorException e) {
            e.printStackTrace();
            messageMap.addErrorMessage("MESSAGE_CODE_501");
        }

        return attributesMap;
    }
}
