
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.workflow;

import java.util.Iterator;
import java.util.List;

import com.ibm.ccd.ui.mediator.ISoapMessageKeys;
import com.ibm.ccd.ui.mediator.XMLHelper;
import com.ibm.ccd.ui.util.UIUtils;


/**
 * class WorkflowXMLGenerator
 *
 * Generates the request xmls for sending the WPC request
 */
public class WorkflowXMLGenerator {
    /**
     * preparing xmls for the workflow role
     * @param roleName
     * @return
     */
    public static String prepareXMLWorkflowsRole(String roleName) {
        StringBuffer requestXML = new StringBuffer();
        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_ROLENAME).append(">")
                  .append(roleName).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.WORKFLOW_ROLENAME).append(">");

        return requestXML.toString();
    }

    /**
     * preparing xmls for the workflow details
     * @param workflowName
     * @return
     */
    public static String prepareXMLWorkflowsColDetails(String workflowName,
        String role) {
        StringBuffer requestXML = new StringBuffer();

        if (!UIUtils.isEmpty(role)) {
            requestXML.append(prepareXMLWorkflowsRole(role));
        }

        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_COLL_AREA).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG).append(ISoapMessageKeys.NAME)
                  .append(">").append(workflowName)
                  .append(XMLHelper.WPC_NS_REF_END).append(ISoapMessageKeys.NAME)
                  .append(">").append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.WORKFLOW_COLL_AREA).append(">");

        return requestXML.toString();
    }

    /**
     * preparing xmls for the getting items steps
     * @param roleName
     * @param collabName
     * @param workflowName
     * @param stepName
     * @param pageNav
     * @return
     */
    public static String prepareXMLStepItems(String roleName,
        String collabName, String workflowName, String stepName,String categoryFilter,
        PageNavigation pageNav, List attributeList) {
        StringBuffer requestXML = new StringBuffer();
        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_COLL_AREA)
                  .append(" " + ISoapMessageKeys.WORKFLOW_DELWHEN_EMPTY)
                  .append("=\"" + "false" + "\"").append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG).append(ISoapMessageKeys.NAME)
                  .append(">").append(collabName)
                  .append(XMLHelper.WPC_NS_REF_END).append(ISoapMessageKeys.NAME)
                  .append(">").append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_NAME).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG).append(ISoapMessageKeys.NAME)
                  .append(">").append(workflowName)
                  .append(XMLHelper.WPC_NS_REF_END).append(ISoapMessageKeys.NAME)
                  .append(">").append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_STEP).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_STEPNAME).append(">")
                  .append(stepName).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.WORKFLOW_STEPNAME).append(">");

        if (!UIUtils.isEmpty(pageNav.getSortedColumn())) {
            requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                      .append(ISoapMessageKeys.WORKFLOW_SORTBYCOL).append(" ")
                      .append(ISoapMessageKeys.WORKFLOW_SORTBY).append("=\"")
                      .append(pageNav.getSortType()).append("\"").append(">")
                      .append(pageNav.getSortedColumn())
                      .append(XMLHelper.WPC_NS_REF_END)
                      .append(ISoapMessageKeys.WORKFLOW_SORTBYCOL).append(">");
        }

        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_LISTITEMS).append(" ")
                  .append(ISoapMessageKeys.WORKFLOW_STARTINDEX).append("=\"")
                  .append(pageNav.getStartIndex()).append("\"").append(" ")
                  .append(ISoapMessageKeys.WORKFLOW_ENDINDEX).append("=\"")
                  .append(pageNav.getEndIndex()).append("\"").append("/>")
           		  .append(XMLHelper.WPC_NS_REF_BEG)
           		  .append(ISoapMessageKeys.WORKFLOW_CATEGORY_PKLIST).append(">")
           		  .append(categoryFilter)
           		  .append(XMLHelper.WPC_NS_REF_END)
           		  .append(ISoapMessageKeys.WORKFLOW_CATEGORY_PKLIST).append(">")
           		  .append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.WORKFLOW_STEP).append(">")
                  .append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.WORKFLOW_NAME).append(">")
                  .append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.WORKFLOW_COLL_AREA).append(">");

        // attribute list needs to be fetched with get items
        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.ATTRIBUTELIST).append(">");

        for (Iterator iter = attributeList.iterator(); iter.hasNext();) {
            String attributeName = (String) iter.next();

            //			GTIN ATTRIBUTE ADDED HERE...
            requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                      .append(ISoapMessageKeys.ATTRIBUTE).append(" ")
                      .append(ISoapMessageKeys.NAME).append("=\"")
                      .append(attributeName).append("\"").append("/>");
        }

        requestXML.append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.ATTRIBUTELIST).append(">");

        return requestXML.toString();
    }

    /**
     * preparing xmls for the action on items
     * @param strArr
     * @param collabName
     * @param workflowName
     * @param stepName
     * @param nextStep
     * @param action
     * @return
     */
    public static String prepareXMLItemAction(String[] strArr,
        String collabName, String workflowName, String stepName,
        String nextStep, String action) {
        StringBuffer requestXML = new StringBuffer();
        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_COLL_AREA)
                  .append(" " + ISoapMessageKeys.WORKFLOW_DELWHEN_EMPTY)
                  .append("=\"" + "false" + "\"").append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG).append(ISoapMessageKeys.NAME)
                  .append(">").append(collabName)
                  .append(XMLHelper.WPC_NS_REF_END).append(ISoapMessageKeys.NAME)
                  .append(">").append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_NAME).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG).append(ISoapMessageKeys.NAME)
                  .append(">").append(workflowName)
                  .append(XMLHelper.WPC_NS_REF_END).append(ISoapMessageKeys.NAME)
                  .append(">").append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_STEP).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_STEPNAME).append(">")
                  .append(stepName).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.WORKFLOW_STEPNAME).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_COLLABAREA_ACTION)
                  .append(" " + ISoapMessageKeys.WORKFLOW_ACTION).append("=\"")
                  .append(action).append("\"");

        if (action.equals(ISoapMessageKeys.WORKFLOW_ACTION_MOVE)) {
            requestXML.append(" " + ISoapMessageKeys.WORKFLOW_STEP_EXITVALUE)
                      .append("=\"").append(nextStep).append("\"");
        }

        requestXML.append(">");

        for (int i = 0; i < strArr.length; i++) {
            requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                      .append(ISoapMessageKeys.WORKFLOW_TRADEITEM)
                      .append(" " + ISoapMessageKeys.ID)
                      .append("=\"" + strArr[i]).append("\"").append("/>");
        }

        requestXML.append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.WORKFLOW_COLLABAREA_ACTION)
                  .append(">").append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.WORKFLOW_STEP).append(">")
                  .append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.WORKFLOW_NAME).append(">")
                  .append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.WORKFLOW_COLL_AREA).append(">");

        return requestXML.toString();
    }
}
