
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.workflow;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Element;

import com.ibm.ccd.ui.mediator.ISoapMessageKeys;
import com.ibm.ccd.ui.mediator.XMLHelper;
import com.ibm.ccd.ui.portlets.workflow.IWorkFlow;


/**
 * WorkflowXMLParser class
 * Parses all the xmls received WPC Response
 */
public class WorkflowXMLParser {
    /**
     * Getting extracting the user roles list
     * @param element
     * @return
     */
    public static List getUserRolesList(Element[] element) {
        List roleList = extractValuesList(element);

        return roleList;
    }

    /**
     * Extracting Values from any element
     * @param element
     * @return
     */
    private static List extractValuesList(Element[] element) {
        List elementlist = new ArrayList();

        for (int i = 0; i < element.length; i++) {
            String elementValue = XMLHelper.getElementValue(element[i]);
            elementlist.add(elementValue);
        }

        return elementlist;
    }

    /**
     * Getting the business processnames list
     * @param collAreaListElement
     * @return
     */
    public static List getBusinessProcessNamesList(Element collAreaListElement) {
        List bpList = new ArrayList();

        if (collAreaListElement != null) {
            Element[] element = XMLHelper.getElementsInNS(collAreaListElement,
                    ISoapMessageKeys.WORKFLOW_COLL_AREA);

            for (int i = 0; i < element.length; i++) {
                Element nameElement = XMLHelper.getFirstElementInNS(element[i],
                        ISoapMessageKeys.NAME);
                String elementValue = XMLHelper.getElementValue(nameElement);
                bpList.add(new CollaborationArea(elementValue));
            }
        }

        return bpList;
    }

    /**
     * Getting the steps list
     * @param workflowElement
     * @return
     */
    public static Workflow getStepsList(Element bodyElement) {
        List stepList = new ArrayList();

        String containerName = XMLHelper.getFirstElementValueInNS(bodyElement,
                ISoapMessageKeys.WORKFLOW_COLLAB_CONTAINER); 
        
        Element workflowElement = XMLHelper.getFirstElementInNS(bodyElement,
                ISoapMessageKeys.WORKFLOW_NAME);
        
        String workflowName = XMLHelper.getFirstElementValueInNS(workflowElement,
                ISoapMessageKeys.NAME);
        Element[] element = XMLHelper.getElementsInNS(workflowElement,
                ISoapMessageKeys.WORKFLOW_STEP);
        

        
        for (int i = 0; i < element.length; i++) {
            Element step = XMLHelper.getFirstElementInNS(element[i],
                    ISoapMessageKeys.WORKFLOW_STEPNAME);
            String stepName = XMLHelper.getFirstElementValueInNS(element[i],
                    ISoapMessageKeys.WORKFLOW_STEPNAME);
            String itemCount = XMLHelper.getAttributeValue(step,
                    ISoapMessageKeys.WORKFLOW_STEP_ITEMCOUNT);
            String reservetoEditFlg = XMLHelper.getFirstElementValueInNS(element[i],
                    ISoapMessageKeys.WORKFLOW_STEP_RESERVETOEDIT);
            
            Element exitValues = XMLHelper.getFirstElementInNS(element[i],
                    ISoapMessageKeys.WORKFLOW_STEP_EXITVALUES);
            Element[] exitValue = XMLHelper.getElementsInNS(exitValues,
                    ISoapMessageKeys.WORKFLOW_STEP_EXITVALUE);
            String[] value = new String[exitValue.length];

            for (int j = 0; j < exitValue.length; j++) {
                value[j] = XMLHelper.getElementValue(exitValue[j]);
            }

            Steps stepsTemp = new Steps(stepName,
                    (new Integer(itemCount)).intValue());
            stepsTemp.setNextStepExitVal(value);
            stepsTemp.setEditStepflg(reservetoEditFlg);
            stepList.add(stepsTemp);
        }

        Workflow workflow = new Workflow();
        workflow.setWorkflowName(workflowName);
        workflow.setSteps(stepList);
        workflow.setContainerName(containerName);

        return workflow;
    }

    /**
     * Getting items map
     * @param element
     * @return
     */
    public static Map getItemsMap(Element element) {
        List itemList = new ArrayList();
        Map dataMap = new HashMap();
        Map categoryInfoMap = new HashMap();
        String stepItemCount = "0";
        String containerName = XMLHelper.getFirstElementValueInNS(element,
                ISoapMessageKeys.WORKFLOW_COLLAB_CONTAINER);        
        Element workflowElement = XMLHelper.getFirstElementInNS(element,
                ISoapMessageKeys.WORKFLOW_NAME);
        Element stepElement = XMLHelper.getFirstElementInNS(workflowElement,
                ISoapMessageKeys.WORKFLOW_STEP);
        Element stepNameElement = XMLHelper.getFirstElementInNS(stepElement,
                ISoapMessageKeys.WORKFLOW_STEPNAME);
        stepItemCount = XMLHelper.getAttributeValue(stepNameElement,
                ISoapMessageKeys.WORKFLOW_STEP_ITEMCOUNT);

        Element checkOutEntries = XMLHelper.getFirstElementInNS(stepElement,
                ISoapMessageKeys.WORKFLOW_CHECKOUTENTRIES);

        Element[] recordElement = XMLHelper.getElementsInNS(checkOutEntries,
                ISoapMessageKeys.WORKFLOW_CHECKOUTENTRY);

        if (recordElement != null) {
            for (int i = 0; i < recordElement.length; i++) {
                Map attrMap = new HashMap();
                Element itemElement = XMLHelper.getFirstElementInNS(recordElement[i],
                        ISoapMessageKeys.TRADE_ITEM);
                String itemId = "";
                String strDesc = "";
                Element attrListElement = XMLHelper.getFirstElementInNS(recordElement[i],
                        ISoapMessageKeys.ATTRIBUTE_LIST);
                Element[] attrElement = XMLHelper.getElementsInNS(attrListElement,
                        ISoapMessageKeys.ATTRIBUTE);

                for (int j = 0; j < attrElement.length; j++) {
                    Element valuesElement = XMLHelper.getFirstElementInNS(attrElement[j],
                            ISoapMessageKeys.ATTRIB_VALUES);
                    Element valueElement = XMLHelper.getFirstElementInNS(valuesElement,
                            ISoapMessageKeys.ATTRIB_VALUE);
                    
                    String attrName = XMLHelper.getAttributeValue(attrElement[j],
                            ISoapMessageKeys.NAME);
                    
                    attrMap.put(attrName,
                        XMLHelper.getElementValue(valueElement));
                }

                Element reservedElement = XMLHelper.getFirstElementInNS(recordElement[i],
                        ISoapMessageKeys.WORKFLOW_RESERVEDBY);
                String reservedValue = "";

                if (reservedElement != null) {
                    reservedValue = XMLHelper.getElementValue(reservedElement);
                }

                Element startDateElement = XMLHelper.getFirstElementInNS(recordElement[i],
                        ISoapMessageKeys.WORKFLOW_STARTDATE);
                String startDateValue = XMLHelper.getElementValue(startDateElement);

                Element endDateElement = XMLHelper.getFirstElementInNS(recordElement[i],
                        ISoapMessageKeys.WORKFLOW_ENDDATE);
                String endDateValue = "";

                if (endDateElement != null) {
                    endDateValue = XMLHelper.getElementValue(endDateElement);
                }

                CatalogItemView temp = new CatalogItemView(itemId);

                temp.setProductDesc(strDesc);
                temp.setReservedBy(reservedValue);
                temp.setStartDate(startDateValue);
                temp.setEndDate(endDateValue);
                temp.setAttributeMap(attrMap);
                itemList.add(temp);

            }
        }
 
        dataMap.put(IWorkFlow.KEY_MAP_GET_ITEMS_DATASET1, itemList);
        dataMap.put(IWorkFlow.KEY_MAP_GET_ITEMS_DATASET2, stepItemCount);
        dataMap.put(IWorkFlow.KEY_MAP_GET_ITEMS_DATASET3, containerName);
        
        return dataMap;
    }
}
