
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.exception;

import java.io.Serializable;
import java.util.HashMap;


public class MessageMap implements Serializable {
    protected HashMap messageMap;

    public MessageMap() {
        messageMap = new HashMap(5);
    }

    private void addMessage(String messageType, String key, Object[] values) {
        MessageList msgList = (MessageList) messageMap.get(messageType);

        if (msgList == null) {
            msgList = new MessageList(messageType);
            messageMap.put(messageType, msgList);
        }

        // add the message to the message list
        msgList.add(new Message(messageType, key, values));
    }
    
    
	public void copyMessageMap(MessageMap srcMessageMap){
		
		messageMap.put(MessageTypes.FATAL,srcMessageMap.getFatalList());
		messageMap.put(MessageTypes.ERROR,srcMessageMap.getErrorErrorList());
		messageMap.put(MessageTypes.INFO,srcMessageMap.getInfoMessageList());
		messageMap.put(MessageTypes.DEBUG,srcMessageMap.getDebugMessageList());
		messageMap.put(MessageTypes.WARNING,srcMessageMap.getWarningMessageList());
		
	}
    /////////////////////////////////
    // DEBUG MESSAGES    ////////////
    /////////////////////////////////
    public void addDebugMessage(String key) {
        addDebugMessage(key, null);
    }

    public void addDebugMessage(String key, Object value0) {
        addDebugMessage(key, new Object[] { value0 });
    }

    public void addDebugMessage(String key, Object value0, Object value1) {
        addDebugMessage(key, new Object[] { value0, value1 });
    }

    public void addDebugMessage(String key, Object value0, Object value1,
        Object value2) {
        addDebugMessage(key, new Object[] { value0, value1, value2 });
    }

    public void addDebugMessage(String key, Object[] values) {
        addMessage(MessageTypes.DEBUG, key, values);
    }

    //////////////////////////////////
    // INFO MESSAGES     ////////////
    /////////////////////////////////
    public void addInfoMessage(String key) {
        addInfoMessage(key, null);
    }

    public void addInfoMessage(String key, Object value0) {
        addInfoMessage(key, new Object[] { value0 });
    }

    public void addInfoMessage(String key, Object value0, Object value1) {
        addInfoMessage(key, new Object[] { value0, value1 });
    }

    public void addInfoMessage(String key, Object value0, Object value1,
        Object value2) {
        addInfoMessage(key, new Object[] { value0, value1, value2 });
    }

    public void addInfoMessage(String key, Object[] values) {
        addMessage(MessageTypes.INFO, key, values);
    }

    /////////////////////////////////
    // WARNING MESSAGES  ////////////
    /////////////////////////////////
    public void addWarningMessage(String key) {
        addWarningMessage(key, null);
    }

    public void addWarningMessage(String key, Object value0) {
        addWarningMessage(key, new Object[] { value0 });
    }

    public void addWarningMessage(String key, Object value0, Object value1) {
        addWarningMessage(key, new Object[] { value0, value1 });
    }

    public void addWarningMessage(String key, Object value0, Object value1,
        Object value2) {
        addWarningMessage(key, new Object[] { value0, value1, value2 });
    }

    public void addWarningMessage(String key, Object[] values) {
        addMessage(MessageTypes.WARNING, key, values);
    }

    /////////////////////////////////
    // ERROR MESSAGES  ////////////
    /////////////////////////////////
    public void addErrorMessage(String key) {
        addErrorMessage(key, null);
    }

    public void addErrorMessage(String key, Object value0) {
        addErrorMessage(key, new Object[] { value0 });
    }

    public void addErrorMessage(String key, Object value0, Object value1) {
        addErrorMessage(key, new Object[] { value0, value1 });
    }

    public void addErrorMessage(String key, Object value0, Object value1,
        Object value2) {
        addErrorMessage(key, new Object[] { value0, value1, value2 });
    }

    public void addErrorMessage(String key, Object[] values) {
        addMessage(MessageTypes.ERROR, key, values);
    }

    /////////////////////////////////
    // FATAL MESSAGES  ////////////
    /////////////////////////////////
    public void addFatalMessage(String key) {
        addFatalMessage(key, null);
    }

    public void addFatalMessage(String key, Object value0) {
        addFatalMessage(key, new Object[] { value0 });
    }

    public void addFatalMessage(String key, Object value0, Object value1) {
        addFatalMessage(key, new Object[] { value0, value1 });
    }

    public void addFatalMessage(String key, Object value0, Object value1,
        Object value2) {
        addFatalMessage(key, new Object[] { value0, value1, value2 });
    }

    public void addFatalMessage(String key, Object[] values) {
        addMessage(MessageTypes.FATAL, key, values);
    }

    public void clear() {
        messageMap.clear();
    }

    public boolean empty() {
        return messageMap.size() == 0;
    }

    public int size() {
        return messageMap.size();
    }

    public MessageList getDebugMessageList() {
        return (MessageList) messageMap.get(MessageTypes.DEBUG);
    }

    public MessageList getInfoMessageList() {
        return (MessageList) messageMap.get(MessageTypes.INFO);
    }

    public MessageList getWarningMessageList() {
        return (MessageList) messageMap.get(MessageTypes.WARNING);
    }

    public MessageList getErrorErrorList() {
        return (MessageList) messageMap.get(MessageTypes.ERROR);
    }

    public MessageList getFatalList() {
        return (MessageList) messageMap.get(MessageTypes.FATAL);
    }

    public HashMap getMessageMap() {
        return messageMap;
    }

    public static void main(String[] args) {
        MessageMap map = new MessageMap();
        map.addDebugMessage("My Debug1");
        map.addInfoMessage("MyInfo", "1");
        map.addErrorMessage("MY Error", "0", "1");
    }
}
