
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.mediator;

import org.apache.axis.components.uuid.SimpleUUIDGen;
import org.apache.axis.components.uuid.UUIDGen;


/**
 *  Provides the functionality of generation unique Identities
 *  over space and time using  AXIS UUIDGen framework.
 */
public class UniqueIdGenerator {
    // singleton instance of this
    private static UniqueIdGenerator uniqueIdGenerator;

    // uses axis uuid generator    
    private UUIDGen uniqueGenerator;

    /**
    * Private constructor.
    */
    private UniqueIdGenerator() {
    }

    /**
     * Initializes the instance.
     */
    private void init() {
        uniqueGenerator = new SimpleUUIDGen();
    }

    /**
     * Returns the UniqueIdGenerator by using Lazyloading approach
     * @return UniqueIdGenerator singleton instance.
     */
    public static UniqueIdGenerator getInstance() {
        if (uniqueIdGenerator == null) {
            synchronized (UniqueIdGenerator.class) {
                if (uniqueIdGenerator == null) {
                    uniqueIdGenerator = new UniqueIdGenerator();
                    uniqueIdGenerator.init();
                }
            }
        }

        return uniqueIdGenerator;
    }

    public String getUniqueId() {
        return uniqueGenerator.nextUUID();
    }
}
