
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package com.ibm.ccd.ui.mediator;


/**
 *  Represents the exception thrown by the WPC both at system level
 *  and application level
 */
public class WPCMediatorException extends Exception {
    // Application level error occurred, which is a part of WPC XML notification
    public static final int APPLICAION_ERROR = 1;

    // represents the system level errors, Axis fault etc.
    public static final int SYSTEM_ERROR = 2;

    // error type 
    private int type;

    // encompassed exception in case of Systemlevel exceptions.
    private Throwable throwable;

    // notification details in case of Application error
    private WPCNotificationBean notificationBean;

    /**
     *  Constructs the WPC application level exception
     */
    public WPCMediatorException(int type, WPCNotificationBean notificationBean) {
        this.type = type;
        this.notificationBean = notificationBean;
    }

    /**
      *  Constructs the WPC System level exception
      */
    public WPCMediatorException(int type, Throwable throwable) {
        this.type = type;
        this.throwable = throwable;
    }

    public WPCNotificationBean getNotificationBean() {
        return notificationBean;
    }

    public Throwable getThrowable() {
        return throwable;
    }

    public int getType() {
        return type;
    }

    public void setNotificationBean(WPCNotificationBean bean) {
        notificationBean = bean;
    }

    public void setThrowable(Throwable throwable) {
        this.throwable = throwable;
    }

    public void setType(int type) {
        this.type = type;
    }
}
