
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.mediator;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * This servlet initalizes the ConfigManager ( which depends on ServletContext to resolve config file path)
 * and WPCMediator and keeps their references ( not allowing to garbage collect)
 * Note : Ensure this as start-up on load (deployment config. in web.xml)
 */
public class WPCMediatorInitializer extends HttpServlet {
    // WPC Mediator Identifier
    public static final String WPC_MEDIATOR = "WPC_MED";

    // represents the initialization status of the mediator 
    private boolean initated = false;

    public void init() throws ServletException {
        super.init();

        if (getServletContext().getAttribute(WPC_MEDIATOR) == null) {
            //				load the config. manager		        
            getServletContext().log("WPCMediatorInitializer.init(): Initializing the ConfigManager");
            ConfigManager.getInstance().init(getServletContext());
            getServletContext().log("WPCMediatorInitializer.init(): ConfigManager initiated succesfully");

            // Config. Manager intialized successfully, now init. the mediator.
            getServletContext().log("WPCMediatorInitializer.init():Initializing the WPCMediator");

            WPCMediator mediator = WPCMediator.getInstance();
            getServletContext().setAttribute(WPC_MEDIATOR, mediator);
            getServletContext().log("WPCMediatorInitializer.init(): WPCMediator initiated succesfully");
            initated = true;
        } else {
            getServletContext().log("WPCMediatorInitializer.init(): Mediator already Initialized");
        }
    }

    /**
         * Just prints the status of the Mediator initialization
         * @see javax.servlet.http.HttpServlet#void (javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
         */
    public void doGet(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException {
        resp.setContentType("text/html");

        PrintWriter out = resp.getWriter();
        out.print("WPC Mediator initialized :" + initated);
        out.close();
    }

    /**
     * @see javax.servlet.http.HttpServlet#doPost(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public void doPost(HttpServletRequest req, HttpServletResponse res)
        throws ServletException, IOException {
        doGet(req, res);
    }
}
