
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.mediator;

import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.List;

import javax.xml.rpc.ServiceException;

import org.apache.axis.AxisFault;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.ibm.ccd.ui.context.WPCUser;
import com.ibm.ccd.ui.dataobjects.Attachment;
import com.ibm.ccd.ui.dataobjects.FileDownloadResource;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;


public class WSHelper implements ISoapMessageKeys {
    public static final String COMP_NAME = "WebSerivcesHelper";

    // namespace length.
    private static final int wpcNSIndex = (ISoapMessageKeys.WPC_NS_URI_REF +
        ":").length();

    // soap action URLS 
    private static HashMap soapActionURLS;

    // default company ..
    public static final String DEF_WPC_COMPANY = "ibm";

    // default WPC user ID
    public static final String DEF_WPC_USER_ID = "portaluser";

    //	default WPC password
    public static final String DEF_WPC_PASSWORD = "password";

    // unique ID generator
    private UniqueIdGenerator uniqueIdGenerator;

    public WSHelper() {
        uniqueIdGenerator = UniqueIdGenerator.getInstance();
        soapActionURLS = ConfigManager.getInstance().getSoapActionURLs();
    }

    private static String getSoapURL(String action) {
        System.out.println("----- Web Service invoked : "+ action +"-----");
        return (String) soapActionURLS.get(action);
    }

    public Element getCatalogs(WPCUser user) throws WPCMediatorException {
        String requestXML = XMLHelper.prepareWPCRequest("", REQ_GET_CATALOGS,
                SYNCHRONOUS, user);

        SoapResponse soapResponse = invokeSOAP(getSoapURL(
			ISoapMessageKeys.REQ_GET_CATALOGS), requestXML, 
			(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);           

        String responseXML = soapResponse.getDataXML();

        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        // anaylize the result element.
        WPCNotificationBean notificationBean = WSHelper.getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
            return XMLHelper.getFirstElementInNS(responseEnvelopElement,
                ISoapMessageKeys.CATALOGS);
        }

        throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
            notificationBean);
    }

    public Element getCatalogChildren(String ctgChildrenReqXML, WPCUser user)
        throws WPCMediatorException {
        String requestXML = XMLHelper.prepareWPCRequest(ctgChildrenReqXML,
                ISoapMessageKeys.REQ_CATALOG_CHILDREN,
                ISoapMessageKeys.SYNCHRONOUS, user);

        /*String responseXML =

                                        "<wpc:envelope communicationVersion=\"5.2\" xmlns:wpc=\"http://ibm.com/wpc/\">"
                                        +"<wpc:wpcHeader><wpc:companyName>ibm</wpc:companyName><wpc:userID>portaluser</wpc:userID><wpc:password>password</wpc:password>"
                                        +"<wpc:messageIdentifier>1104756468</wpc:messageIdentifier><wpc:timestamp>2005-01-03</wpc:timestamp></wpc:wpcHeader>"
                                        +"<wpc:wpcBody><wpc:wpcResponse><wpc:wpcEntityIdentifier><wpc:wpcMessageIdentifier>A2473730-5D85-11D9-BF8A-BC1A417CAB44</wpc:wpcMessageIdentifier></wpc:wpcEntityIdentifier>"
                                        +"<wpc:wpcNotification type=\"SUCCESS\"><wpc:notificationInfo><wpc:notificationCode>000</wpc:notificationCode><wpc:notificationMessage>SUCCESS</wpc:notificationMessage>"
                                        +"<wpc:wpcTradeItemPK/></wpc:notificationInfo></wpc:wpcNotification>"
                                        +"<wpc:wpcChildren><wpc:wpcChild><wpc:wpcChildID>parentID/parentID/11234</wpc:wpcChildID><wpc:wpcChildDisplayName>IBM InfoPrint</wpc:wpcChildDisplayName><wpc:wpcChildType>CATEGORY / CATALOG / ITEM</wpc:wpcChildType></wpc:wpcChild><wpc:wpcChild><wpc:wpcChildID>parentID/parentID/12345</wpc:wpcChildID><wpc:wpcChildDisplayName>IBM InfoPrint 4322</wpc:wpcChildDisplayName><wpc:wpcChildType>CATEGORY / CATALOG / ITEM</wpc:wpcChildType></wpc:wpcChild></wpc:wpcChildren>"
                                        +"</wpc:wpcResponse></wpc:wpcBody></wpc:envelope>"; */
        SoapResponse soapResponse = invokeSOAP(getSoapURL(
				ISoapMessageKeys.REQ_CATALOG_CHILDREN), requestXML,
				(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));

		setWPCSessionID(soapResponse,user);

        String responseXML = soapResponse.getDataXML();

        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        // anaylize the result element.
        WPCNotificationBean notificationBean = WSHelper.getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
            return XMLHelper.getFirstElementInNS(responseEnvelopElement,
                ISoapMessageKeys.CATALOG_QUERY_CHILDREN);
        }

        throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
            notificationBean);
    }
    
	public Element getSearchSpecXML(String catalogReqXML, WPCUser user)
			throws WPCMediatorException {
			String requestXML = XMLHelper.prepareWPCRequest(catalogReqXML,
					ISoapMessageKeys.REQ_SEARCH_ITEMSKELETON,
					ISoapMessageKeys.SYNCHRONOUS, user);

			SoapResponse soapResponse = invokeSOAP(getSoapURL(
						ISoapMessageKeys.REQ_SEARCH_ITEMSKELETON), requestXML,
						(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
			setWPCSessionID(soapResponse,user);

			String responseXML = soapResponse.getDataXML();
			Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

			// anaylize the result element.
			WPCNotificationBean notificationBean = WSHelper.getNotificationDetails(responseEnvelopElement);

			if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
				return XMLHelper.getFirstElementInNS(responseEnvelopElement,
					ISoapMessageKeys.CATALOG_SPEC);
			}

			throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
				notificationBean);
		}
		
	public Element getSearchResultSpecXML(String catalogReqXML, WPCUser user)
				throws WPCMediatorException {
				String requestXML = XMLHelper.prepareWPCRequest(catalogReqXML,
						ISoapMessageKeys.REQ_SEARCH_ITEMSKELETON,
						ISoapMessageKeys.SYNCHRONOUS, user);

				SoapResponse soapResponse = invokeSOAP(getSoapURL(
							ISoapMessageKeys.REQ_SEARCH_ITEMSKELETON), requestXML,
							(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
				setWPCSessionID(soapResponse,user);
												
				String responseXML = soapResponse.getDataXML();
				Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

				// anaylize the result element.
				WPCNotificationBean notificationBean = WSHelper.getNotificationDetails(responseEnvelopElement);

				if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
					return XMLHelper.getFirstElementInNS(responseEnvelopElement,
						ISoapMessageKeys.CATALOG_SPEC);
				}

				throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
					notificationBean);
			}

    public Element getCatalogSpecXML(String catalogReqXML, WPCUser user)
        throws WPCMediatorException {
        String requestXML = XMLHelper.prepareWPCRequest(catalogReqXML,
                ISoapMessageKeys.REQ_CATALOG_SPEC,
                ISoapMessageKeys.SYNCHRONOUS, user);

        SoapResponse soapResponse = invokeSOAP(getSoapURL(
				ISoapMessageKeys.REQ_CATALOG_SPEC), requestXML,
				(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);
                    
        String responseXML = soapResponse.getDataXML();
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        // anaylize the result element.
        WPCNotificationBean notificationBean = WSHelper.getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
            return XMLHelper.getFirstElementInNS(responseEnvelopElement,
                ISoapMessageKeys.CATALOG_SPEC);
        }

        throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
            notificationBean);
    }

    public Element getLookupTableSpecXML(String lookupTableSpecReqXML,
        WPCUser user) throws WPCMediatorException {
        String requestXML = XMLHelper.prepareWPCRequest(lookupTableSpecReqXML,
                ISoapMessageKeys.REQ_LOOKUP_TABLE_SPEC,
                ISoapMessageKeys.SYNCHRONOUS, user);

        SoapResponse soapResponse = invokeSOAP(getSoapURL(
					ISoapMessageKeys.REQ_LOOKUP_TABLE_SPEC), requestXML,
					(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);                    

        String responseXML = soapResponse.getDataXML();
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        // anaylize the result element.
        WPCNotificationBean notificationBean = WSHelper.getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
            return XMLHelper.getFirstElementInNS(responseEnvelopElement,
                ISoapMessageKeys.CATALOG_SPEC);
        }

        throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
            notificationBean);
    }

    public Element getCatalogItemXML(String itemReqXML, WPCUser user)
        throws WPCMediatorException {
        String requestXML = XMLHelper.prepareWPCRequest(itemReqXML,
                ISoapMessageKeys.REQ_ITEM_DETAILS,
                ISoapMessageKeys.SYNCHRONOUS, user);
        SoapResponse soapResponse = invokeSOAP(getSoapURL(
				ISoapMessageKeys.REQ_ITEM_DETAILS), requestXML,
				(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);
                    
        String responseXML = soapResponse.getDataXML();
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        // anaylize the result element.
        WPCNotificationBean notificationBean = WSHelper.getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
            return XMLHelper.getFirstElementInNS(responseEnvelopElement,
                ISoapMessageKeys.TRADE_ITEM);
        }

        throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
            notificationBean);
    }

	/**
	 * 
	 * @param viewReqXML
	 * @param user
	 * @return
	 * @throws WPCMediatorException
	 */
	public Element getCatalogViewListXML(String viewReqXML, WPCUser user)
		throws WPCMediatorException {
		String requestXML =
			XMLHelper.prepareWPCRequest(
				viewReqXML,
				REQ_CATALOG_VIEW_LIST,
				SYNCHRONOUS,
				user);

		SoapResponse soapResponse =
			invokeSOAP(getSoapURL(REQ_CATALOG_VIEW_LIST), requestXML,
			(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);			
		String responseXML = soapResponse.getDataXML();
		Element responseEnvelopElement =
			XMLHelper.getRootElementFromString(responseXML);

		// analyze the result
		WPCNotificationBean notificationBean =
			WSHelper.getNotificationDetails(responseEnvelopElement);

		if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
			return responseEnvelopElement;
		}

		throw new WPCMediatorException(
			WPCMediatorException.APPLICAION_ERROR,
			notificationBean);
	}

	
	/**
	 * 
	 * @param viewReqXML
	 * @param user
	 * @return
	 * @throws WPCMediatorException
	 */
	public Element getCatalogViewXML(String viewReqXML, WPCUser user)
		throws WPCMediatorException {
		String requestXML =
			XMLHelper.prepareWPCRequest(
				viewReqXML,
				REQ_CATALOG_VIEW,
				SYNCHRONOUS,
				user);

		SoapResponse soapResponse =
			invokeSOAP(getSoapURL(REQ_CATALOG_VIEW), requestXML,
			(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);
		
		String responseXML = soapResponse.getDataXML();
		Element responseEnvelopElement =
			XMLHelper.getRootElementFromString(responseXML);

		// analyze the result
		WPCNotificationBean notificationBean =
			WSHelper.getNotificationDetails(responseEnvelopElement);

		if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
			return responseEnvelopElement;
		}

		throw new WPCMediatorException(
			WPCMediatorException.APPLICAION_ERROR,
			notificationBean);
	}

    public String createItem(String catalogItemAddXML, WPCUser user)
        throws WPCMediatorException {
        String requestXML = XMLHelper.prepareWPCRequest(catalogItemAddXML,
                ISoapMessageKeys.REQ_ITEM_ADD, ISoapMessageKeys.SYNCHRONOUS,
                user);

        SoapResponse soapResponse = invokeSOAP(getSoapURL(
                    ISoapMessageKeys.REQ_ITEM_ADD), requestXML,
					(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);

        String responseXML = soapResponse.getDataXML();

        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        // analyze the result
        WPCNotificationBean notificationBean = WSHelper.getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
            return (String) notificationBean.getDetails().get(ISoapMessageKeys.TRADE_ITEM_PK);
        }

        throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
            notificationBean);
    }

    public void updateItem(String catalogItemUpdateXML, WPCUser user)
        throws WPCMediatorException {
        String requestXML = XMLHelper.prepareWPCRequest(catalogItemUpdateXML,
                ISoapMessageKeys.REQ_ITEM_MODIFY, ISoapMessageKeys.SYNCHRONOUS,
                user);

        SoapResponse soapResponse = invokeSOAP(getSoapURL(
                    ISoapMessageKeys.REQ_ITEM_MODIFY), requestXML,
					(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);
		
        String responseXML = soapResponse.getDataXML();
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        // anyalize the result
        WPCNotificationBean notificationBean = WSHelper.getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() != IMediatorResultState.SUCCESS) {
            throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
                notificationBean);
        }
    }

    public void deleteItem(String itemDeleteReqXML, WPCUser user)
        throws WPCMediatorException {
        String requestXML = XMLHelper.prepareWPCRequest(itemDeleteReqXML,
                ISoapMessageKeys.REQ_ITEM_DELETE, ISoapMessageKeys.SYNCHRONOUS,
                user);

        SoapResponse soapResponse = invokeSOAP(getSoapURL(
                    ISoapMessageKeys.REQ_ITEM_DELETE), requestXML,
					(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);			
        String responseXML = soapResponse.getDataXML();
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        // anyalize the result
        WPCNotificationBean notificationBean = WSHelper.getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() != IMediatorResultState.SUCCESS) {
            throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
                notificationBean);
        }
    }

    public Element getCatalogItemCloneXML(String itemReqXML, WPCUser user)
        throws WPCMediatorException {
        String requestXML = XMLHelper.prepareWPCRequest(itemReqXML,
                ISoapMessageKeys.REQ_ITEM_CLONE, ISoapMessageKeys.SYNCHRONOUS,
                user);

        SoapResponse soapResponse = invokeSOAP(getSoapURL(
                    ISoapMessageKeys.REQ_ITEM_CLONE), requestXML,
					(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);
		                    
        String responseXML = soapResponse.getDataXML();
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        // anaylize the result element.
        WPCNotificationBean notificationBean = WSHelper.getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
            return XMLHelper.getFirstElementInNS(responseEnvelopElement,
                ISoapMessageKeys.TRADE_ITEM);
        }

        throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
            notificationBean);
    }

    public String getSearchCriteriaSpecXML(String partialXML, WPCUser user,
        long startIndex, long endIndex) throws WPCMediatorException {
        String catalogId = null;

        catalogId = UIUtils.getCatalogName(user);

        String tempRequestXML = XMLHelper.prepareSearchCriteriaXML(partialXML,
                catalogId,startIndex,endIndex);
        String requestXML = XMLHelper.prepareWPCRequest(tempRequestXML,
                ISoapMessageKeys.REQ_ITEM_SEARCH, ISoapMessageKeys.SYNCHRONOUS,
                user);

		
		SoapResponse soapResponse = invokeSOAP(getSoapURL(
					ISoapMessageKeys.REQ_ITEM_SEARCH), requestXML, 
					(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		
		setWPCSessionID(soapResponse,user);	

		return soapResponse.getDataXML();	
    }

    /**
     * Searching Items for Multi Edit.
     * @param partialXML
     * @param user
     * @param maxResults
     * @param catalogName
     * @return
     * @throws WPCMediatorException
     */
    public String getSearchCriteriaSpecXMLForMultiEdit(String partialXML, WPCUser user,
        String maxResults,String catalogName) throws WPCMediatorException {
        String catalogId = null;

        catalogId = catalogName;

        String tempRequestXML = XMLHelper.prepareSearchCriteriaXMLForMultiEdit(partialXML,
                maxResults, catalogId);
 
        String requestXML = XMLHelper.prepareWPCRequest(tempRequestXML.toString(),
                ISoapMessageKeys.REQ_ITEM_SEARCH, ISoapMessageKeys.SYNCHRONOUS,
                user);

		SoapResponse soapResponse = invokeSOAP(getSoapURL(
					ISoapMessageKeys.REQ_ITEM_SEARCH), requestXML, 
					(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
        
		setWPCSessionID(soapResponse,user);
		
		String responseXML = soapResponse.getDataXML();

        
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        // anaylize the result element.
        WPCNotificationBean notificationBean = WSHelper.getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
            return responseXML;
        }

        throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
            notificationBean);

    }
    public String getLookupTableSearchCriteriaSpecXML(String partialXML,
        WPCUser user, long startIndex, long endIndex) throws WPCMediatorException {
    	String lookupId = null;

        lookupId = UIUtils.getLookupTableName(user);
        String tempRequestXML = XMLHelper.prepareSearchCriteriaXML(partialXML,
                lookupId,startIndex,endIndex);
        String requestXML = XMLHelper.prepareWPCRequest(tempRequestXML,
                ISoapMessageKeys.REQ_LOOKUP_TABLE_SEARCH,
                ISoapMessageKeys.SYNCHRONOUS, user);

		SoapResponse soapResponse = invokeSOAP(getSoapURL(
					ISoapMessageKeys.REQ_LOOKUP_TABLE_SEARCH), requestXML, 
					(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		
		setWPCSessionID(soapResponse,user);			

		return soapResponse.getDataXML();

    }

    /**
     * Fileupload support
     */
    public String uploadFile(String fileUploadRequestXML,
        Attachment attachment, WPCUser user) throws WPCMediatorException {
        String documentPath = null;

        String uploadFileRequestXML = XMLHelper.prepareFileUploadRequestXML(attachment.getUploadedFileName());

        String requestXML = XMLHelper.prepareWPCRequest(uploadFileRequestXML,
                ISoapMessageKeys.REQ_FILE_UPLOAD, ISoapMessageKeys.SYNCHRONOUS,
                user);

        SoapResponse soapResponse = invokeSOAP(getSoapURL(
					ISoapMessageKeys.REQ_FILE_UPLOAD), requestXML, attachment,
					(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);

        String responseXML = soapResponse.getDataXML();
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        // anyalize the result
        WPCNotificationBean notificationBean = WSHelper.getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() != IMediatorResultState.SUCCESS) {
            throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
                notificationBean);
        }

        /**
          * get the document path from the response XML
         */
        if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
            documentPath = XMLHelper.getFirstElementValueInNS(responseEnvelopElement,
                    "wpcFileDocStorePath");
        }

        return documentPath;
    }

    public String processUploadedFile(String documentPath, String collabName,
        WPCUser user) throws WPCMediatorException {
        String requestXML = XMLHelper.prepareFileImportRequestXML(documentPath,
                collabName);
        String completeXML = XMLHelper.prepareWPCRequest(requestXML,
                ISoapMessageKeys.REQ_FILE_IMPORT,
                ISoapMessageKeys.ASYNCHRONOUS, user);

        SoapResponse soapResponse = invokeSOAP(getSoapURL(
					ISoapMessageKeys.REQ_FILE_IMPORT), completeXML,(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);

        String responseXML = soapResponse.getDataXML();
        WPCNotificationBean notificationBean = createTransactionNotification(responseXML);

        throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
            notificationBean);
    }

    /*
            <wpc:getTransactionIDResponse xmlns:wpc="http://ibm.com/wpc/">
            <wpc:transactionID>26</wpc:transactionID>
            </wpc:getTransactionIDResponse>
            */
    public static final WPCNotificationBean createTransactionNotification(
        String asyncResponseElement) {
        WPCNotificationBean notificationBean = new WPCNotificationBean();

        Element responseElement = XMLHelper.getRootElementFromString(asyncResponseElement);

        notificationBean.setType(ISoapMessageKeys.PROCESSING);
        notificationBean.setCode(IMediatorResultState.PROCESING);

        // get the info element.
        NodeList infNodeList = responseElement.getChildNodes();

        for (int i = 0; i < infNodeList.getLength(); i++) {
            Node node = infNodeList.item(i);

            if ((node != null) && node instanceof Element) {
                Element outputElement = ((Element) node);

                if (outputElement.getTagName().substring(wpcNSIndex).equals(ISoapMessageKeys.NOTIFICATION_MESSAGE)) {
                    notificationBean.setMessage(XMLHelper.getElementValue(
                            outputElement));

                    continue;
                }

                notificationBean.addDetail(outputElement.getTagName().substring((ISoapMessageKeys.WPC_NS_URI_REF +
                        ":").length()), XMLHelper.getElementValue(outputElement));
            }
        }

        return notificationBean;
    }

    /*
     *
     <wpc:wpcNotification type="SUCCESS">-
            <wpc:notificationInfo>
            <wpc:notificationCode>000</wpc:notificationCode>
            <wpc:notificationMessage>SUCCESS</wpc:notificationMessage>
            <wpc:wpcTradeItemPK>205</wpc:wpcTradeItemPK>
            </wpc:notificationInfo>
            </wpc:wpcNotification>
     */
    public static final WPCNotificationBean getNotificationDetails(
        Element envelopElement) {
        WPCNotificationBean notificationBean = new WPCNotificationBean();

        Element notificatonElement = XMLHelper.getFirstElementInNS(envelopElement,
                ISoapMessageKeys.NOTIFICATION);

        //get the type
        String type = XMLHelper.getAttributeValue(notificatonElement,
                ISoapMessageKeys.TYPE);
        notificationBean.setType(type);

        // get the info element.
        Element notficationInfElement = XMLHelper.getFirstElementInNS(notificatonElement,
                ISoapMessageKeys.NOTIFICATION_INFO);
        NodeList infNodeList = notficationInfElement.getChildNodes();

        for (int i = 0; i < infNodeList.getLength(); i++) {
            Node node = infNodeList.item(i);

            if ((node != null) && node instanceof Element) {
                Element outputElement = ((Element) node);

                if (outputElement.getTagName().substring(wpcNSIndex).equals(ISoapMessageKeys.NOTIFICATION_CODE)) {
                    notificationBean.setCode(UIUtils.getIntValue(
                            XMLHelper.getElementValue(outputElement)));

                    continue;
                }

                if (outputElement.getTagName().substring(wpcNSIndex).equals(ISoapMessageKeys.NOTIFICATION_MESSAGE)) {
                    notificationBean.setMessage(XMLHelper.getElementValue(
                            outputElement));

                    continue;
                }

                notificationBean.addDetail(outputElement.getTagName().substring((ISoapMessageKeys.WPC_NS_URI_REF +
                        ":").length()), XMLHelper.getElementValue(outputElement));
            }
        }

        return notificationBean;
    }

    public SoapResponse invokeSOAP(String url, String requestXML,
        Attachment attachment) throws WPCMediatorException {
        return callSoapService(url, requestXML, attachment);
    }

	public SoapResponse invokeSOAP(String url, String requestXML,
		Attachment attachment, String wpcSessionID) throws WPCMediatorException {
		return callSoapService(url, requestXML, attachment);
	}

    public SoapResponse invokeSOAP(String url, String requestXML)
        throws WPCMediatorException {
        return callSoapService(url, requestXML, null);
    }

	public SoapResponse invokeSOAP(String url, String requestXML, String wpcSessionID)
		throws WPCMediatorException {
		return callSoapService(url, requestXML, null,wpcSessionID);
	}

    private SoapResponse callSoapService(String url, String requestXML,
        Attachment attachment) throws WPCMediatorException {
        if (IReference.COND_CMPL) {
            System.out.println("\n  --------  Request XML to WPC ---------");
            System.out.println(requestXML);
            System.out.println("\n  -------- 000 ---------");
        }

        SoapResponse soapResponse = null;

        try {
            soapResponse = SoapInvoker.invokeSoapWithMOM(url, requestXML,
                    attachment);
        } catch (MalformedURLException mue) {
            mue.printStackTrace();
            throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR,
                mue);
        } catch (ServiceException sve) {
            sve.printStackTrace();
            throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR,
                sve);
        } catch (AxisFault af) {
            af.printStackTrace();
            throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR, af);
        }

        if (IReference.COND_CMPL) {
            System.out.println("\n  --------  Response XML from WPC ---------");
            System.out.println(soapResponse.getDataXML());
            System.out.println("\n  -------- $$$ ---------");
        }

        // if the response element contains any notification error/processing throw the WPCMediatorException				
        return soapResponse;
    }

	private SoapResponse callSoapService(String url, String requestXML,
		Attachment attachment, String wpcSessionID) throws WPCMediatorException {
		if (IReference.COND_CMPL) {
			System.out.println("\n  --------  Request XML to WPC ---------  SessionID : " + wpcSessionID);
			System.out.println(requestXML);
			System.out.println("\n  -------- 000 ---------");
		}

		SoapResponse soapResponse = null;

		try {
			soapResponse = SoapInvoker.invokeSoapWithMOM(url, requestXML,
					attachment,wpcSessionID);
		} catch (MalformedURLException mue) {
			mue.printStackTrace();
			throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR,
				mue);
		} catch (ServiceException sve) {
			sve.printStackTrace();
			throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR,
				sve);
		} catch (AxisFault af) {
			af.printStackTrace();
			throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR, af);
		}

		if (IReference.COND_CMPL) {
			System.out.println("\n  --------  Response XML from WPC --------- Session ID : " + soapResponse.getWpcAxisSessionID());
			System.out.println(soapResponse.getDataXML());
			System.out.println("\n  -------- $$$ ---------" );
		}

		// if the response element contains any notification error/processing throw the WPCMediatorException				
		return soapResponse;
	}

    /**
     * @param transmap
     * @return
     * @throws WPCMediatorException
     */
    public String getTransactionsStatus(String commanSepkeys, WPCUser user)
        throws WPCMediatorException {
        String responseXml = processTransactionStatus(commanSepkeys, user);

        return responseXml;
    }

    private String processTransactionStatus(String documentPath, WPCUser user)
        throws WPCMediatorException {
        String transactionStatus = null;

        String requestXML = XMLHelper.prepareFileTransactionStatusRequestXML(documentPath);
        String completeXML = XMLHelper.prepareWPCRequest(requestXML,
                ISoapMessageKeys.REQ_FILE_QUERY_TRANSACTION_STATUS,
                ISoapMessageKeys.SYNCHRONOUS, user);

        SoapResponse soapResponse = invokeSOAP(getSoapURL(
                    ISoapMessageKeys.REQ_FILE_QUERY_TRANSACTION_STATUS),
                completeXML,(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));

        String responseXML = soapResponse.getDataXML();
		setWPCSessionID(soapResponse,user);
		
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        WPCNotificationBean notificationBean = getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() != IMediatorResultState.SUCCESS) {
            throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
                notificationBean);
        }

        /**
          * get the document path from the response XML for transactions
         */
        if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
            transactionStatus = XMLHelper.getFirstElementValueInNS(responseEnvelopElement,
                    TRANSACTION_RESPONSE_RESULT_STRING);
        }

        return transactionStatus;
    }

    /**
     * Used for fetching Transaction details and attachment
     * @param transactionId
     * @param user
     * @return
     * @throws WPCMediatorException
     */
    public List getTransactionsDetails(String transactionId, WPCUser user)
        throws WPCMediatorException {
        String transactionStatus = null;
        String requestXML = XMLHelper.prepareFileTransactionStatusRequestXML(transactionId);
        String completeXML = XMLHelper.prepareWPCRequest(requestXML,
                ISoapMessageKeys.REQ_FILE_QUERY_TRANSACTION_DETAILS,
                ISoapMessageKeys.SYNCHRONOUS, user);

        SoapResponse soapResponse = invokeSOAP(getSoapURL(
                    ISoapMessageKeys.REQ_FILE_QUERY_TRANSACTION_DETAILS),
                completeXML,(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);                

        String responseXML = soapResponse.getDataXML();
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        WPCNotificationBean notificationBean = getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() != IMediatorResultState.SUCCESS) {
            throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
                notificationBean);
        }

        /**
          * get the document path from the response XML for transactions
         */
        List listAttachments = null;

        if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
            String transId = XMLHelper.getFirstElementValueInNS(responseEnvelopElement,
                    TRANSACTION_RESPONSE_RESULT_STRING);
            System.out.println("Response element transId = " + transId);
        }

        listAttachments = soapResponse.getAttachmentParts();

        return listAttachments;
    }

    /**
     * Used for fetching Transaction details and attachment
     * @param transactionId
     * @param user
     * @return
     * @throws WPCMediatorException
     */
    public Object getImageObject(FileDownloadResource resourceObj, WPCUser user)
        throws WPCMediatorException {
        String requestXML = XMLHelper.prepareFileImageObjectRequestXML(resourceObj);
        String completeXML = XMLHelper.prepareWPCRequest(requestXML,
                ISoapMessageKeys.REQ_FILE_GET_IMAGE_OBJECT,
                ISoapMessageKeys.SYNCHRONOUS, user);

        SoapResponse soapResponse = invokeSOAP(getSoapURL(
					ISoapMessageKeys.REQ_FILE_GET_IMAGE_OBJECT), completeXML,
					(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);

        String responseXML = soapResponse.getDataXML();
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        WPCNotificationBean notificationBean = getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() != IMediatorResultState.SUCCESS) {
            throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
                notificationBean);
        }

        /**
          * get the document path from the response XML for transactions
         */
        List listImageObjects = null;

        if (notificationBean.getCode() == IMediatorResultState.SUCCESS) {
            String gtinId = XMLHelper.getFirstElementValueInNS(responseEnvelopElement,
                    REQ_FILE_GET_IMAGE_OBJECT);
            System.out.println("Response element = " + gtinId);
        }

        listImageObjects = soapResponse.getAttachmentParts();
        System.out.println("Response listImageObjects= " +
            listImageObjects.size());

        return listImageObjects.get(0);
    }

    public String downloadItemDetails(String requestXML, WPCUser wpcUser)
        throws WPCMediatorException {
        String completeXML = XMLHelper.prepareWPCRequest(requestXML,
                ISoapMessageKeys.REQ_ITEMS_FILE_DOWNLOAD,
                ISoapMessageKeys.ASYNCHRONOUS, wpcUser);

        SoapResponse soapResponse = invokeSOAP(getSoapURL(
					ISoapMessageKeys.REQ_ITEMS_FILE_DOWNLOAD), completeXML, 
					(String)wpcUser.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,wpcUser);
                    
        String responseXML = soapResponse.getDataXML();

        WPCNotificationBean notificationBean = createTransactionNotification(responseXML);

        throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
            notificationBean);
    }

    /**
     * @param user
     * @return
     * @throws WPCMediatorException
     */
    public Element[] getUserRole(WPCUser user) throws WPCMediatorException {
        String requestXML = XMLHelper.prepareWPCRequest("",
                ISoapMessageKeys.REQ_USER_ROLES, ISoapMessageKeys.SYNCHRONOUS,
                user);
		SoapResponse soapResponse = invokeSOAP(getSoapURL(
					ISoapMessageKeys.REQ_USER_ROLES), requestXML,
					(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);

        String responseXML = soapResponse.getDataXML();
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        WPCNotificationBean notificationBean = getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() != IMediatorResultState.SUCCESS) {
            throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
                notificationBean);
        }

        Element userListElement = XMLHelper.getFirstElementInNS(responseEnvelopElement,
                ISoapMessageKeys.WORKFLOW_USERLIST);
        Element userElement = XMLHelper.getFirstElementInNS(userListElement,
                ISoapMessageKeys.WORKFLOW_USER);
        Element roleListElement = XMLHelper.getFirstElement(userElement,
                ISoapMessageKeys.WORKFLOW_ROLELIST);
        Element[] element = XMLHelper.getElementsInNS(roleListElement,
                ISoapMessageKeys.WORKFLOW_ROLENAME);

        return element;
    }

    /**
     * @param user
     * @param reqXML
     * @return
     * @throws WPCMediatorException
     */
    public Element getBusinessProcess(WPCUser user, String reqXML)
        throws WPCMediatorException {
        String requestXML = XMLHelper.prepareWPCRequest(reqXML,
                ISoapMessageKeys.REQ_GET_WORKFLOWS,
                ISoapMessageKeys.SYNCHRONOUS, user);
        SoapResponse soapResponse = invokeSOAP(getSoapURL(
					ISoapMessageKeys.REQ_GET_WORKFLOWS), requestXML, 
					(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);

        String responseXML = soapResponse.getDataXML();
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        WPCNotificationBean notificationBean = getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() != IMediatorResultState.SUCCESS) {
            throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
                notificationBean);
        }

        Element collListElement = XMLHelper.getFirstElementInNS(responseEnvelopElement,
                ISoapMessageKeys.WORKFLOW_COLL_LIST);

        return collListElement;
    }

    public Element getSteps(WPCUser user, String reqXML)
        throws WPCMediatorException {
        String requestXML = XMLHelper.prepareWPCRequest(reqXML,
                ISoapMessageKeys.REQ_GET_STEPS, ISoapMessageKeys.SYNCHRONOUS,
                user);
        SoapResponse soapResponse = invokeSOAP(getSoapURL(
				ISoapMessageKeys.REQ_GET_STEPS), requestXML, 
				(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);

        String responseXML = soapResponse.getDataXML();
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        WPCNotificationBean notificationBean = getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() != IMediatorResultState.SUCCESS) {
            throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
                notificationBean);
        }

        Element collaborationElement = XMLHelper.getFirstElementInNS(responseEnvelopElement,
                ISoapMessageKeys.WORKFLOW_COLL_AREA);
        Element workflowElement = XMLHelper.getFirstElementInNS(collaborationElement,
                ISoapMessageKeys.WORKFLOW_NAME);
        Element workflowElement1 = XMLHelper.getFirstElementInNS(collaborationElement,
                ISoapMessageKeys.WORKFLOW_COLL_AREA);

        
        return collaborationElement;
    }

    public Element getStepItems(WPCUser user, String reqXML)
        throws WPCMediatorException {
        String requestXML = XMLHelper.prepareWPCRequest(reqXML,
                ISoapMessageKeys.REQ_GET_ITEMS, ISoapMessageKeys.SYNCHRONOUS,
                user);

        SoapResponse soapResponse = invokeSOAP(getSoapURL(
					ISoapMessageKeys.REQ_GET_ITEMS), requestXML, 
					(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);            

        String responseXML = soapResponse.getDataXML();
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);

        WPCNotificationBean notificationBean = getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() != IMediatorResultState.SUCCESS) {
            throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
                notificationBean);
        }

        Element collaborationListElement = XMLHelper.getFirstElementInNS(responseEnvelopElement,
                ISoapMessageKeys.WORKFLOW_COLL_LIST);
        Element collaborationElement = XMLHelper.getFirstElementInNS(collaborationListElement,
                ISoapMessageKeys.WORKFLOW_COLL_AREA);

        return collaborationElement;
    }

    public void processAction(WPCUser user, String reqXML)
        throws WPCMediatorException {
        String requestXML = XMLHelper.prepareWPCRequest(reqXML,
                ISoapMessageKeys.REQ_COLLAB_AREA_ACTION,
                ISoapMessageKeys.SYNCHRONOUS, user);
        SoapResponse soapResponse = invokeSOAP(getSoapURL(
					ISoapMessageKeys.REQ_COLLAB_AREA_ACTION), requestXML, 
					(String)user.getProperty(WPCUser.WPC_AXIS_SESSIONID));
		setWPCSessionID(soapResponse,user);

        String responseXML = soapResponse.getDataXML();
        Element responseEnvelopElement = XMLHelper.getRootElementFromString(responseXML);
        WPCNotificationBean notificationBean = getNotificationDetails(responseEnvelopElement);

        if (notificationBean.getCode() != IMediatorResultState.SUCCESS) {
            throw new WPCMediatorException(WPCMediatorException.APPLICAION_ERROR,
                notificationBean);
        }
    }

	//set WPC Axis generated sessionID to WPCUser    
	private void setWPCSessionID(SoapResponse soapResponse, WPCUser user) {
		String resSessionID = soapResponse.getWpcAxisSessionID();
		
		if(resSessionID != null) {
			//set the response sessionID in WPC User Property  
			if(! resSessionID.equals(user.getProperty(WPCUser.WPC_AXIS_SESSIONID)))
				user.setProperty(WPCUser.WPC_AXIS_SESSIONID,soapResponse.getWpcAxisSessionID());
		}	
    
	}
}
