
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.mediator;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.StringReader;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Date;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.CharacterData;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import com.ibm.ccd.ui.cache.CacheConstants;
import com.ibm.ccd.ui.cache.CacheFactory;
import com.ibm.ccd.ui.cache.CacheManager;
import com.ibm.ccd.ui.cache.CacheNotEnabledException;
import com.ibm.ccd.ui.cache.ICacheFactory;
import com.ibm.ccd.ui.context.IWPCUser;
import com.ibm.ccd.ui.context.WPCUser;
import com.ibm.ccd.ui.dataobjects.FileDownloadResource;
import com.ibm.ccd.ui.util.UIUtils;


public class XMLHelper {
    public static final String XML_HEADER = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>";
    private static DocumentBuilderFactory factory;
    public static final String WPC_NS_REF_BEG = "<" +
        ISoapMessageKeys.WPC_NS_URI_REF + ":";
    public static final String WPC_NS_REF_END = "</" +
        ISoapMessageKeys.WPC_NS_URI_REF + ":";
    public static final String WPC_ENV_BEG = WPC_NS_REF_BEG +
        ISoapMessageKeys.ENVELOPE + " xmlns:" +
        ISoapMessageKeys.WPC_NS_URI_REF + "='" + ISoapMessageKeys.WPC_NS_URI +
        "'  communicationVersion='" + ISoapMessageKeys.WPC_COMM_VERSION + "'>";
    public static final String WPC_ENV_END = WPC_NS_REF_END +
        ISoapMessageKeys.ENVELOPE + ">";

    static {
        //Initiate DocumentBuilderFactory
        factory = DocumentBuilderFactory.newInstance();

        // To get a validating parser
        factory.setValidating(false);

        // To get one that understands namespaces
        factory.setNamespaceAware(true);

        // set the CDATA sections into the Text Nodes
        factory.setCoalescing(true);
    }

    public static Element getRootElementFromString(String xmlStrIn) {
        return parseString(xmlStrIn).getDocumentElement();
    }

    public static Element getRootElementFromFile(String xmlFile) {
        return parseFromFile(xmlFile).getDocumentElement();
    }

    /**
     * Parse the XML String and create Document
     * @param fileName
     * @return Document
     */
    public static Document parseString(String xmlStr) {
        Document document = null;

        try {
            // Get DocumentBuilder
            DocumentBuilder builder = factory.newDocumentBuilder();

            // Parse and load into memory the Document
            document = builder.parse(new InputSource(new StringReader(xmlStr)));

            return document;
        } catch (SAXParseException spe) {
            // Error generated by the parser
            handleSAXParseException(spe);
        } catch (SAXException sxe) {
            // Error generated during parsing
            handleSAXException(sxe);
        } catch (ParserConfigurationException pce) {
            // Parser with specified options can't be built
            pce.printStackTrace();
        } catch (IOException ioe) {
            // I/O error
            ioe.printStackTrace();
        }

        return null;
    }

    /**
     * Parse the XML file and create Document
     * @param fileName
     * @return Document
     */
    public static Document parseFromFile(String fileName) {
        Document document = null;

        try {
            // Get DocumentBuilder
            DocumentBuilder builder = factory.newDocumentBuilder();

            // Parse and load into memory the Document
            document = builder.parse(new File(fileName));

            return document;
        } catch (SAXParseException spe) {
            // Error generated by the parser
            handleSAXParseException(spe);
        } catch (SAXException sxe) {
            // Error generated during parsing
            handleSAXException(sxe);
        } catch (ParserConfigurationException pce) {
            // Parser with specified options can't be built
            pce.printStackTrace();
        } catch (IOException ioe) {
            // I/O error
            ioe.printStackTrace();
        }

        return null;
    }

    public static String getElementValue(Element element) {
        try {
            return ((CharacterData) element.getFirstChild()).getData();
        } catch (NullPointerException ne) {
            return "";
        }
    }

    public static String getAttributeValue(Element element, String name) {
        return element.getAttribute(name);

        //return element.getAttributeNS("http://ibm.com/wpc/", name);
    }

    public static String getFirstElementValue(Element node, String elementName) {
        NodeList elementList = node.getElementsByTagName(elementName);

        Element element = null;
        String value = null;

        if (elementList.getLength() >= 1) {
            if (elementList.item(0) instanceof Element) {
                return getElementValue((Element) elementList.item(0));
            }
        }

        return null;
    }

    public static Element getFirstElement(Element inputElement, String name) {
        NodeList nodeList = getNodeList(inputElement, name);
        int size = nodeList.getLength();

        if (size == 0) {
            return null;
        }

        for (int i = 0; i < size; i++) {
            Node node = nodeList.item(i);

            if ((node != null) && node instanceof Element) {
                return (Element) node;
            }
        }

        return null;
    }

    public static Element[] getElements(Element inputElement, String name) {
        NodeList nodeList = getNodeList(inputElement, name);
        int size = nodeList.getLength();

        if (size == 0) {
            return null;
        }

        ArrayList elementList = new ArrayList(size);

        for (int i = 0; i < size; i++) {
            Node node = nodeList.item(i);

            if ((node != null) && node instanceof Element) {
                Element outputElement = ((Element) node);
                elementList.add(outputElement);
            }
        }

        Element[] elements = new Element[elementList.size()];

        for (int i = 0; i < elements.length; ++i) {
            elements[i] = (Element) elementList.get(i);
        }

        return elements;
    }

    public static NodeList getNodeList(Element element, String name) {
        return element.getElementsByTagName(name);
    }

    ////////////////////////////////////////////////////
    /// Utils which works with the WPC NameSpace  //////
    ////////////////////////////////////////////////////
    public static String getFirstElementValueInNS(Element node,
        String elementName) {
        //NodeList elementList = node.getElementsByTagName(elementName);
        NodeList elementList = node.getElementsByTagNameNS(ISoapMessageKeys.WPC_NS_URI,
                elementName);
        Element element = null;
        String value = null;

        if (elementList.getLength() >= 1) {
            element = (Element) elementList.item(0);

            CharacterData charDataElement = (CharacterData) element.getFirstChild();

            if (charDataElement == null) {
                return null;
            }

            value = charDataElement.getData();

            if (value != null) {
                value = value.trim();
            }
        }

        return value;
    }

    public static Element getFirstElementInNS(Element inputElement, String name) {
        NodeList nodeList = getNodeListInNS(inputElement, name);
        int size = nodeList.getLength();

        if (size == 0) {
            return null;
        }

        for (int i = 0; i < size; i++) {
            Node node = nodeList.item(i);

            if ((node != null) && node instanceof Element) {
                return (Element) node;
            }
        }

        return null;
    }

    public static Element[] getChildElementsInNS(Element inputElement,
        String name) {
        NodeList nodeList = XMLHelper.getChildNodeListInNS(inputElement, name);

        int size = nodeList.getLength();

        if (size == 0) {
            return null;
        }

        ArrayList elementList = new ArrayList(size);

        for (int i = 0; i < size; i++) {
            Node node = nodeList.item(i);

            if ((node != null) && node instanceof Element &&
                    node.getNodeName().equalsIgnoreCase(ISoapMessageKeys.WPC_NS_URI_REF +
                        ":" + name)) {
                Element outputElement = ((Element) node);
                elementList.add(outputElement);
            }
        }

        Element[] elements = new Element[elementList.size()];

        for (int i = 0; i < elements.length; ++i) {
            elements[i] = (Element) elementList.get(i);
        }

        return elements;
    }

    public static NodeList getChildNodeListInNS(Element parentElement,
        String name) {
        NodeList nodeList = parentElement.getChildNodes();

        return nodeList;
    }

    public static Element[] getElementsInNS(Element inputElement, String name) {
        NodeList nodeList = getNodeListInNS(inputElement, name);
        int size = nodeList.getLength();

        if (size == 0) {
            return null;
        }

        ArrayList elementList = new ArrayList(size);

        for (int i = 0; i < size; i++) {
            Node node = nodeList.item(i);

            if ((node != null) && node instanceof Element) {
                Element outputElement = ((Element) node);
                elementList.add(outputElement);
            }
        }

        Element[] elements = new Element[elementList.size()];

        for (int i = 0; i < elements.length; ++i) {
            elements[i] = (Element) elementList.get(i);
        }

        return elements;
    }

    public static NodeList getNodeListInNS(Element element, String name) {
        return element.getElementsByTagNameNS(ISoapMessageKeys.WPC_NS_URI, name);
    }

    /**
     * This method writes a DOM document to a file
     * @param filename
     * @param document
     */
    public static void writeXML2File(String filename, Document document) {
        try {
            // Prepare the DOM document for writing
            Source source = new DOMSource(document);

            // Prepare the output file
            File file = new File(filename);
            Result result = new StreamResult(file);

            // Write the DOM document to the file
            // Get Transformer
            Transformer xformer = TransformerFactory.newInstance()
                                                    .newTransformer();

            // Write to a file
            xformer.transform(source, result);
        } catch (TransformerConfigurationException e) {
            System.out.println("TransformerConfigurationException: " + e);
        } catch (TransformerException e) {
            System.out.println("TransformerException: " + e);
        }
    }

    /**
     * Count Elements in Document by Tag Name
     * @param tag
     * @param document
     * @return number elements by Tag Name
     */
    public static int countByTagName(String tag, Document document) {
        NodeList list = document.getElementsByTagName(tag);

        return list.getLength();
    }

    private static final void handleSAXParseException(SAXParseException spe) {
        System.out.println("\n** Parsing error , line " + spe.getLineNumber() +
            ", uri " + spe.getSystemId());
        System.out.println(" " + spe.getMessage());

        // Use the contained exception, if any
        Exception x = spe;

        if (spe.getException() != null) {
            x = spe.getException();
        }

        x.printStackTrace();
    }

    private static final void handleSAXException(SAXException sxe) {
        //			Error generated during parsing
        Exception x = sxe;

        if (sxe.getException() != null) {
            x = sxe.getException();
        }

        x.printStackTrace();
    }

    /** Prints the specified node, then prints all of its children. */
    public static void printDOM(Node node) {
        int type = node.getNodeType();

        switch (type) {
        // print the document element
        case Node.DOCUMENT_NODE: {
            System.out.println(XML_HEADER);
            printDOM(((Document) node).getDocumentElement());

            break;
        }

        // print element with attributes
        case Node.ELEMENT_NODE: {
            System.out.print("<");
            System.out.print(node.getNodeName());

            NamedNodeMap attrs = node.getAttributes();

            for (int i = 0; i < attrs.getLength(); i++) {
                Node attr = attrs.item(i);
                System.out.print(" " + attr.getNodeName().trim() + "=\"" +
                    attr.getNodeValue().trim() + "\"");
            }

            System.out.println(">");

            NodeList children = node.getChildNodes();

            if (children != null) {
                int len = children.getLength();

                for (int i = 0; i < len; i++)
                    printDOM(children.item(i));
            }

            System.out.print("\n</");
            System.out.print(node.getNodeName());

            for (int i = 0; i < attrs.getLength(); i++) {
                Node attr = attrs.item(i);
                System.out.print(" " + attr.getNodeName().trim() + "=\"" +
                    attr.getNodeValue().trim() + "\"");
            }

            System.out.println(">");

            break;
        }

        // handle entity reference nodes
        case Node.ENTITY_REFERENCE_NODE: {
            System.out.print("&");
            System.out.print(node.getNodeName().trim());
            System.out.print(";");

            break;
        }

        // print cdata sections
        case Node.CDATA_SECTION_NODE: {
            System.out.print("");

            break;
        }

        //	   print text
        case Node.TEXT_NODE: {
            System.out.print(node.getNodeValue().trim());

            break;
        }

        //	   print processing instruction
        case Node.PROCESSING_INSTRUCTION_NODE: {
            System.out.print("");

            break;
        }
        }

        if (type == Node.ELEMENT_NODE) {
            System.out.println();
           System.out.print("");
        }
    }

    /*
             *
             String soapMessage = "<wpc:envelope xmlns:wpc='http://ibm.com/wpc/'  communicationVersion='5.2'>" +
                    "<wpc:WPCResponseType>synchronous</wpc:WPCResponseType>" +
                    "<wpc:wpcHeader>" +
                            "<wpc:companyName>String</wpc:companyName>" +
                            "<wpc:userID>String</wpc:userID>" +
                            "<wpc:password>String</wpc:password>" +
                            "<wpc:messageIdentifier>String</wpc:messageIdentifier>" +
                            "<wpc:timestamp>1967-08-13</wpc:timestamp>" +
                            "<wpc:supplierId>1234567890123</wpc:supplierId>" +
                    "</wpc:wpcHeader>" +
                    "<wpc:wpcBody>" +
                    "<wpc:wpcCommand type='ITEMSKELETON' mode='SYNC'>" +
                                    "<wpc:wpcCatalogName>CTG_PORTAL</wpc:wpcCatalogName>" +
                            "</wpc:wpcCommand>" +
                    "</wpc:wpcBody>" +
            "</wpc:envelope>";
             */
    public static String prepareWPCHeader(String company, String userID,
        String password, String msgIdentifier, String timestamp,
        String supplierGLN, String wpcUsername, String userPreferredLanguage,
        String userRestrictedLanguage) {

        CacheManager cacheManager = null;
        String tempStrWPCHeader = null;
       	/*
       	 * Returning the cached header xml for the user.If not available then generate the xml and 
       	 * store in cache.
       	 */
        ICacheFactory cacheFactory = CacheFactory.getInstance();
        tempStrWPCHeader = (String)cacheFactory.getCachedObject(CacheConstants.USERS_TEMPLATE_DYNAMIC,userID);
		if (tempStrWPCHeader != null) {
			return tempStrWPCHeader;
		}else{
			tempStrWPCHeader = (String)cacheFactory.getCachedObject(CacheConstants.TEMPLATES,CacheConstants.TEMPLATE_SOAP_HEADER);
		}
        	
		/*
    	 * <wpc:companyName>{0}</wpc:companyName><wpc:userID>{1}</wpc:userID><wpc:wpcUserID>{2}</wpc:wpcUserID>
    	 * <wpc:password>{3}</wpc:password><wpc:messageIdentifier>{4}</wpc:messageIdentifier><wpc:timestamp>{5}</wpc:timestamp>
    	 * <wpc:supplierId>{6}</wpc:supplierId><wpc:localeForDisplay>{7}</wpc:localeForDisplay><wpc:localeRestriction>{8}</wpc:localeRestriction>
    	 */
		
    	String[] arrArgument = {company,userID,wpcUsername,
							password,msgIdentifier,timestamp,
							supplierGLN,userPreferredLanguage,userRestrictedLanguage};
		
		tempStrWPCHeader = MessageFormat.format(tempStrWPCHeader,arrArgument);

		try {
   			cacheManager = CacheManager.getInstance();
	    } catch (CacheNotEnabledException e1) {
		}
		// Dumping the header in cache.Happens only first time for a particular user.
		if (CacheManager.isCacheEnabled() && cacheManager!= null) {
    		cacheManager.putInCache(CacheConstants.USERS_TEMPLATE_DYNAMIC,userID,tempStrWPCHeader);
		}
        return tempStrWPCHeader;
    }

    /*
      "<wpc:envelope xmlns:wpc='http://ibm.com/wpc/'  communicationVersion='5.2'>" +
     *
     */
    public static String prepareWPCRequest(String reqXML, String command,
        String mode, WPCUser user) {
        String company = null;
        String userID = null;
        String password = null;
        String supplierGLN = null;
        String wpcUsername = null;
        userID = (String) user.getProperty(IWPCUser.KEY_USER_ID);
        password = (String) user.getProperty(IWPCUser.KEY_PASSWORD);
        company = (String) user.getProperty(IWPCUser.KEY_COMPANY);
        supplierGLN = (String) user.getProperty(IWPCUser.KEY_SUPPLIER_GLN);
        wpcUsername = (String) user.getProperty(IWPCUser.KEY_WPC_USERNAME);

        String userPreferredLanguage = (String) user.getProperty(IWPCUser.KEY_PREFERRED_LANGUAGE);
        String userRestrictedLanguage = (String) user.getProperty(IWPCUser.KEY_RESTRICTED_LANGUAGE);

        StringBuffer stringBuffer = new StringBuffer();
        String msgID = UniqueIdGenerator.getInstance().getUniqueId();
        String date = UIUtils.getDateString(new Date());
        String responseModeType = "<wpc:WPCResponseType>synchronous</wpc:WPCResponseType>";

        if (ISoapMessageKeys.ASYNCHRONOUS.equalsIgnoreCase(mode)) {
            responseModeType = "<wpc:WPCResponseType>asynchronous</wpc:WPCResponseType>";
        }

        stringBuffer.append(WPC_ENV_BEG).append(responseModeType)
                    .append(XMLHelper.prepareWPCHeader(company, userID,
                password, msgID, date, supplierGLN, wpcUsername,
                userPreferredLanguage, userRestrictedLanguage))
                    .append(WPC_NS_REF_BEG).append(ISoapMessageKeys.BODY)
                    .append(">").append(WPC_NS_REF_BEG)
                    .append(ISoapMessageKeys.COMMAND).append(" ")
                    .append(ISoapMessageKeys.TYPE).append("=\"").append(command)
                    .append("\" ").append(ISoapMessageKeys.MODE).append("=\"")
                    .append(mode).append("\" ").append(">").append(reqXML)
                    .append(WPC_NS_REF_END).append(ISoapMessageKeys.COMMAND)
                    .append(">").append(WPC_NS_REF_END)
                    .append(ISoapMessageKeys.BODY).append(">").append(WPC_ENV_END);

        return stringBuffer.toString();
    }

	public static String prepareSearchCriteriaXML(String partialXML, String catalogId, long startIndex, long endIndex) {
		StringBuffer partialRequestXML = new StringBuffer();
		partialRequestXML.append("<")
						 .append(ISoapMessageKeys.SEARCH_CATALOG_NAME)
						 .append(">").append(catalogId).append("</")
						 .append(ISoapMessageKeys.SEARCH_CATALOG_NAME)
						 .append(">")
						 .append("<").append(ISoapMessageKeys.SEARCH_START_INDEX).append(">")
						 .append(startIndex)
						 .append("</").append(ISoapMessageKeys.SEARCH_START_INDEX).append(">")
						 .append("<").append(ISoapMessageKeys.SEARCH_END_INDEX).append(">")
						 .append(endIndex)
						 .append("</").append(ISoapMessageKeys.SEARCH_END_INDEX).append(">")
						 .append(partialXML);

		return partialRequestXML.toString();
	}
  
    public static String prepareSearchCriteriaXMLForMultiEdit(String partialXML,
            String resultCount, String catalogId) {
            StringBuffer partialRequestXML = new StringBuffer();
            partialRequestXML.append("<")
                             .append(ISoapMessageKeys.SEARCH_CATALOG_NAME)
                             .append(">").append(catalogId).append("</")
                             .append(ISoapMessageKeys.SEARCH_CATALOG_NAME)
                             .append(">").append("<")
                             .append(ISoapMessageKeys.SEARCH_RESULT_COUNT_NAME)
                             .append(">").append(resultCount).append("</")
                             .append(ISoapMessageKeys.SEARCH_RESULT_COUNT_NAME)
                             .append(">")
                             .append(partialXML);

            return partialRequestXML.toString();
        }
    public static String prepareFileImportRequestXML(String documentPath,
        String collabName) {
        StringBuffer requestXML = new StringBuffer();

        if (!UIUtils.isEmpty(collabName)) {
            requestXML.append(WPC_NS_REF_BEG)
                      .append(ISoapMessageKeys.CATALOG_NAME).append(">")
                      .append(collabName).append(WPC_NS_REF_END)
                      .append(ISoapMessageKeys.CATALOG_NAME).append(">");
        }

        requestXML.append(WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.FILE_DOCSTORE_PATH_STRING).append(">")
                  .append(documentPath).append(WPC_NS_REF_END)
                  .append(ISoapMessageKeys.FILE_DOCSTORE_PATH_STRING).append(">");

        return requestXML.toString();
    }

    public static String prepareFileUploadRequestXML(String filename) {
        StringBuffer requestXML = new StringBuffer();
        requestXML.append(WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.DATA_CONTAINER_STRING).append(">")
                  .append(WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.ATTACHMENTS_DATA_CONTAINER_STRING)
                  .append(">").append(WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.ATTACHMENTS_FILE_NAME_STRING)
                  .append(">").append(filename).append(WPC_NS_REF_END)
                  .append(ISoapMessageKeys.ATTACHMENTS_FILE_NAME_STRING)
                  .append(">").append(WPC_NS_REF_END)
                  .append(ISoapMessageKeys.ATTACHMENTS_DATA_CONTAINER_STRING)
                  .append(">").append(WPC_NS_REF_END)
                  .append(ISoapMessageKeys.DATA_CONTAINER_STRING).append(">");

        return requestXML.toString();
    }

    public static String prepareFileTransactionStatusRequestXML(
        String arrTransId) {
        StringBuffer requestXML = new StringBuffer();
        requestXML.append(WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.TRANSACTION_RESPONSE_STRING)
                  .append(arrTransId).append(WPC_NS_REF_END).append(ISoapMessageKeys.TRANSACTION_RESPONSE_STRING);
        ;

        return requestXML.toString();
    }

    //	"<wpc:wpcParentCategory>3</wpc:wpcParentCategory>" +
    //		   "<wpc:wpcCatalogName>CTG_PORTAL_MASTER</wpc:wpcCatalogName>" +
    //		   "<wpc:wpcHierarchyName>Hier_Portal</wpc:wpcHierarchyName>" +
    //			 "<wpc:wpcTradeItem ID=\"89\">" + 
    //				"<wpc:attribute-list>" +
    //				   "<wpc:attribute name=\"Master Item Spec\testImage\" type=\"Thumbnail image\"/>" +
    //				"</wpc:attribute-list>" + 
    //			 "</wpc:wpcTradeItem>" +
    public static String prepareFileImageObjectRequestXML(
        FileDownloadResource resourceObj) {
        StringBuffer requestXML = new StringBuffer();

        requestXML.append(WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.PARENT_CATEGORY).append(">")
                  .append(resourceObj.getCategoryID()).append(WPC_NS_REF_END)
                  .append(ISoapMessageKeys.PARENT_CATEGORY).append(">")
                  .append(WPC_NS_REF_BEG).append(ISoapMessageKeys.CATALOG_NAME)
                  .append(">").append(resourceObj.getCatalogID())
                  .append(WPC_NS_REF_END).append(ISoapMessageKeys.CATALOG_NAME)
                  .append(">").append(WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.CATEGORY_HIERARCHY).append("  ")
                  .append(ISoapMessageKeys.TYPE).append("=\"").append("PRIMARY")
                  .append("\"").append("  ").append(ISoapMessageKeys.ATTRIB_NAME)
                  .append("=\"").append(resourceObj.getHierarchyID())
                  .append("\"").append("/>").append(WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.TRADE_ITEM).append("  ")
                  .append(ISoapMessageKeys.ID).append("=\"")
                  .append(resourceObj.getItemPrimaryKey()).append("\"")
                  .append(">").append(WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.ATTRIBUTE_LIST).append(">")
                  .append(WPC_NS_REF_BEG).append(ISoapMessageKeys.ATTRIBUTE)
                  .append("  ").append(ISoapMessageKeys.ATTRIB_NAME)
                  .append("=\"").append(resourceObj.getAttributeName())
                  .append("\"").append("  ").append(ISoapMessageKeys.TYPE)
                  .append("=\"").append(resourceObj.getItemAttributeTypeName())
                  .append("\"").append("/>").append(WPC_NS_REF_END)
                  .append(ISoapMessageKeys.ATTRIBUTE_LIST).append(">")
                  .append(WPC_NS_REF_END).append(ISoapMessageKeys.TRADE_ITEM)
                  .append(">");

        return requestXML.toString();
    }

    public static String readFile(String fileName) {
        byte[] byt = new byte[1024];

        try {
            FileInputStream finput = new FileInputStream(new File(fileName));
            DataInputStream in = new DataInputStream(finput);
            int length = in.available();

            byt = new byte[length];
            in.read(byt);

            return new String(byt, "UTF-8");
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }

    public static void main(String[] args) {
        //		System.out.println(XMLHelper.prepareFileImportRequestXML("mydocpath"));
        //String str = "C:/VGProject\Project work\WPS_521_releaseWork\Sugu XML and HDD\GET_ROLES_FOR_USER_Response.xml";
        String str = "C:/GET_ROLES_FOR_USER_Response.xml";

        //		String string  = readFile(str);
        Element rootElement = getRootElementFromFile(str);

        //String  strVal = getFirstElementValue(rootElement,"wpc:wpcRoleList");
        Element element = getFirstElement(rootElement, "wpc:wpcRoleList");
        Element[] eleArr = getElements(element, "wpc:wpcRole");
        int i = 0;

        while (i < eleArr.length) {
            printDOM(eleArr[i]);
            i++;
        }

        //printDOM(element); 
    }
}
