
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.portlets;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;

import javax.servlet.http.HttpSession;

import org.apache.jetspeed.portlet.PortletAdapter;
import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletLog;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletURI;
import org.apache.jetspeed.portlet.UnavailableException;
import org.apache.jetspeed.portlet.User;

import com.ibm.ccd.ui.alert.framework.IAlertsDataSource;
import com.ibm.ccd.ui.alert.framework.ITransactionStatus;
import com.ibm.ccd.ui.alert.framework.TransactionExchangeService;
import com.ibm.ccd.ui.cache.CacheException;
import com.ibm.ccd.ui.cache.CacheManager;
import com.ibm.ccd.ui.cache.CacheNotEnabledException;
import com.ibm.ccd.ui.context.IWPCUser;
import com.ibm.ccd.ui.context.WPCUser;
import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.mediator.ConfigManager;
import com.ibm.ccd.ui.mediator.IMediatorResultState;
import com.ibm.ccd.ui.mediator.WPCMediator;
import com.ibm.ccd.ui.mediator.WPCMediatorException;
import com.ibm.ccd.ui.mediator.WPCNotificationBean;
import com.ibm.ccd.ui.portlets.alert.ITransactionDisplay;
import com.ibm.ccd.ui.portlets.helper.PortletHelper;
import com.ibm.ccd.ui.services.user.UserBean;
import com.ibm.ccd.ui.services.user.UserManager;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;



/**
 *
 * BasePortlet is abstract Class of all Portlets. All common methods should
 * reside here.
 */
public abstract class BasePortlet extends PortletAdapter {
    protected static PortletLog log = null;
    protected static IAlertsDataSource alertDataSource;

    /**
     * @see org.apache.jetspeed.portlet.Portlet#init(PortletConfig)
     */
    public void init(PortletConfig portletConfig) throws UnavailableException {
        super.init(portletConfig);
        log = getPortletLog();
        log.debug(" init ->");

        //		get the Portlet Context
        PortletContext pc = portletConfig.getContext();

        try {
            alertDataSource = TransactionExchangeService.getInstance(pc);
        } catch (Exception ex) {
            log.error(ex.toString());
            log.error("alertDataSourcenot found");
            ex.printStackTrace();
        }
    }

    /**
     * getMessageMap:Method returns MessageMap after setting it in Request
     * object
     *
     * @param portletRequest
     *                 PortletRequest
     */
    public MessageMap getMessageMap(PortletRequest portletRequest) {
        MessageMap mmap = null;
        mmap = (MessageMap) portletRequest.getAttribute(IReference.MESSAGE_KEY);

        if (mmap == null) {
            mmap = new MessageMap();
            portletRequest.setAttribute(IReference.MESSAGE_KEY, mmap);
        }

        return mmap;
    }

    /**
     * setBusinessFunction: All Portlets should override this method to set the
     * business function in session variable() (SESSION_BUS_FUNCT)
     *
     * @param request
     *                 PortletRequest
     *
     */
    protected abstract void setBusinessFunction(PortletRequest portletRequest);

    /**
     * @param portletRequest
     * @param key
     * @return
     */
    public String getTextLabel(PortletRequest portletRequest, String key) {
        String retStr = "";
        String businessfunction = (String) portletRequest.getSession()
                                                         .getAttribute(IReference.SESSION_BUS_FUNCT);
        String actionString = (String) portletRequest.getSession().getAttribute(IReference.SESSION_ACTION);
        retStr = UIUtils.getTextLabel(getPortletConfig().getContext(),
                portletRequest,
                businessfunction + "." + actionString + "." + key);

        /**
         * If the key created by prefixing the business function with
         * name(businessfunction+name) is not found then only name as key is
         * searched in resource bundle.
         */
        if (retStr.equals("")) {
            retStr = UIUtils.getTextLabel(getPortletConfig().getContext(),
                    portletRequest, businessfunction + "." + key);
        }

        if (retStr.equals("")) {
            retStr = UIUtils.getTextLabel(getPortletConfig().getContext(),
                    portletRequest, key);
        }

        if (retStr.equals("")) {
            log.warn(UIUtils.getTextMessages(getPortletConfig().getContext(),
                    portletRequest, "MESSAGE_CODE_54") + ":" + key);
        }

        return retStr;
    }

    /**
     * @param portletRequest
     * @param key
     * @return
     */
    public String getTextMessages(PortletRequest portletRequest, String key) {
        String retStr = UIUtils.getTextMessages(getPortletConfig().getContext(),
                portletRequest, key);

        return retStr;
    }



    /**
     * @param portletRequest
     * @param transactionId
     */
    protected void addTransaction(String transactionId, WPSContext context) {
        try {
            addTransaction(transactionId, "", context);
        } catch (Exception e) {
            e.printStackTrace();
        }

        log.debug(" addTransaction <- ");
    }

    /**
     * For adding transaction
     *
     * @param transactionId
     * @param description
     * @param userId
     */
    protected void addTransaction(String transactionId, String description,
        WPSContext context) {
        try {
            alertDataSource.addTransaction(transactionId, description, null,
                null, String.valueOf(ITransactionStatus.IN_PROGRESS), context);
        } catch (Exception e) {
            e.printStackTrace();
        }

        log.debug(" addTransaction <- ");
    }

    public static String createCommand(PortletURI portletURI,
        String actionString) {
        if (portletURI != null) {
            portletURI.addAction(actionString);

            return portletURI.toString();
        } else {
            return null;
        }
    }

    protected void handleException(WPCMediatorException mediatorException,
        PortletRequest request) {
        if (mediatorException == null) {
            return;
        }

        MessageMap messageMap = null;

        if (request.getAttribute(IReference.MESSAGE_KEY) != null) {
            messageMap = (MessageMap) request.getAttribute(IReference.MESSAGE_KEY);
        } else {
            try {
                messageMap = getMessageMap(request);
            } catch (Exception e) {
                e.printStackTrace();

                return;
            }
        }

        switch (mediatorException.getType()) {
        case WPCMediatorException.APPLICAION_ERROR:

            WPCUser user = null;
            WPSContext context = null;
            WPCNotificationBean notificationBean = mediatorException.getNotificationBean();

            if (notificationBean.getCode() == IMediatorResultState.PROCESING) {
                String transactionID = notificationBean.getTrasactionID();

                if (UIUtils.isEmpty(transactionID)) {
                    // TODO throw error or populate the messagemap.
                    return;
                }

                String description = (String) request.getAttribute(ITransactionDisplay.TRANSACTION_DESCRIPTION_KEY);

                if (description == null) {
                    description = "";
                }

                context = getWPSContext(request);
                addTransaction(transactionID, description, context);
                messageMap.addInfoMessage("MESSAGE_CODE_107",
                    "TransactionID:" + transactionID);
            } else // application error occurred. handle it
             {
                // indiacate as an application error
                messageMap.addErrorMessage("MESSAGE_CODE_107",
                    notificationBean.getMessage());

                HashMap errorDetails = notificationBean.getDetails();
                Iterator keyIterator = errorDetails.keySet().iterator();

                while (keyIterator.hasNext()) {
                    String key = (String) keyIterator.next();

                    if (errorDetails.get(key) != null) {
                        messageMap.addErrorMessage("MESSAGE_CODE_107",
                            key + ":" + errorDetails.get(key));
                    }
                }
            }

            break;

        case WPCMediatorException.SYSTEM_ERROR:default:

            //indiacate the sys. exception
            messageMap.addErrorMessage("MESSAGE_CODE_106");

            Throwable throwable = mediatorException.getThrowable();

            if (throwable != null) {
                StringWriter stringWriter = new StringWriter();
                throwable.printStackTrace(new PrintWriter(stringWriter));
                messageMap.addInfoMessage("MESSAGE_CODE_107",
                    stringWriter.toString());

                return;
            } else {
                // TODO check by any chance the notification is set here..
            }
        }
    }

    /**
     * This method retrives <code>WPCUser</code> from
     * <code>PortletSession</code>
     *
     * @return WPCUser contains wpc login info
     */
	protected WPSContext getWPSContext(PortletRequest request) {
/*	//	PortletContext portletContext = null;
		WPSContext context = null;
	//	String userID = null;
		
		 User user = request.getUser();
	//	 userID = user.getUserID();
		/*
		 * commented the code for portletContext since the caching has been implemented for getting the context
		 */
	//	portletContext = getPortletConfig().getContext();

		/*long lastLogin = user.getLastLoginTime();
		CacheManager cManager = null;
		try{
			cManager = CacheManager.getInstance();
		} catch(CacheNotEnabledException e){
			System.out.println(e);
		}
		String userKey = "USER_CONTEXT";
		String key = (String) user.getAttribute(IWPCUser.KEY_USER_ID);
		key = key + "_" + lastLogin;
		System.out.println("initial key value is " + key);

		if (cManager != null && cManager.isCached(userKey, key)) {

			context = (WPSContext) cManager.getFromCache(userKey, key);
			System.out.println("inside the not null context loop" + context);
		} else {

			context = createContext(request);
			System.out.println("inside the else context" + context);
			cManager.putInCache(userKey, key, context);
		}

		return context; */
		
		PortletHelper helper = new PortletHelper(getPortletConfig());
		
		WPSContext context = helper.getWPSContext(request);
		return context;
	}

    protected WPSContext createContext(PortletRequest request) {
        /*HttpSession httpSession = null;
        User portalUser = null;
        WPCUser wpcUser = null;
        WPSContext context = null;
        Enumeration userAttributes = null;
        WPCMediator mediator = null;
        String catalogId = null;
        String catalogNode=null;
        //retrive data from User object
        portalUser = request.getUser();

        String strUserID = (String)portalUser.getAttribute("uid");
        Properties ldapProp = ConfigManager.getInstance().getLdapProp();
        
        /*if (ConfigManager.getInstance().isLdapEnabled()) {
            UserManager userManager = UserManager.getInstance();
            UserBean userBean = new UserBean();
            userBean.setUserName(strUserID);
            //   Retrieve User's details from the LDAP
            UserBean userDetails = null;
            userDetails = userManager.validateLoginUser(userBean);

            wpcUser=new WPCUser();
            wpcUser.setProperty(IWPCUser.KEY_USER_ID, userDetails.getUserName());
            wpcUser.setProperty(IWPCUser.KEY_PASSWORD, userDetails.getPassword());
            wpcUser.setProperty(IWPCUser.KEY_COMPANY, userDetails.getCompany());
            wpcUser.setProperty(IWPCUser.KEY_SUPPLIER_GLN, userDetails.getSupplierGLN());
            wpcUser.setProperty(IWPCUser.KEY_WPC_USERNAME, userDetails.getWpcUserName());
    		wpcUser.setProperty(IWPCUser.KEY_PREFERRED_LANGUAGE, userDetails.getPreferredLanguage());
    		wpcUser.setProperty(IWPCUser.KEY_RESTRICTED_LANGUAGE, userDetails.getRestrictedLanguage());
			
		} else{
			  wpcUser=new WPCUser();
	            wpcUser.setProperty(IWPCUser.KEY_USER_ID, ldapProp.get(IWPCUser.KEY_USER_ID));
	            wpcUser.setProperty(IWPCUser.KEY_PASSWORD, ldapProp.get(IWPCUser.KEY_PASSWORD));
	            wpcUser.setProperty(IWPCUser.KEY_COMPANY, ldapProp.get(IWPCUser.KEY_COMPANY));
	            wpcUser.setProperty(IWPCUser.KEY_SUPPLIER_GLN, ldapProp.get(IWPCUser.KEY_SUPPLIER_GLN));
	            wpcUser.setProperty(IWPCUser.KEY_WPC_USERNAME, ldapProp.get(IWPCUser.KEY_WPC_USERNAME));
	    		wpcUser.setProperty(IWPCUser.KEY_PREFERRED_LANGUAGE, ldapProp.get(IWPCUser.KEY_PREFERRED_LANGUAGE));
	    		wpcUser.setProperty(IWPCUser.KEY_RESTRICTED_LANGUAGE, ldapProp.get(IWPCUser.KEY_RESTRICTED_LANGUAGE));
		}*/
		
        
//        // BEGIN REMOVE
        /*wpcUser = new WPCUser();
        wpcUser.setProperty(IWPCUser.KEY_USER_ID, "admin");
        wpcUser.setProperty(IWPCUser.KEY_WPC_USERNAME, "Admin");
		wpcUser.setProperty(IWPCUser.KEY_SUPPLIER_GLN, "C4");
		wpcUser.setProperty(IWPCUser.KEY_PREFERRED_LANGUAGE, "fr_FR");
		wpcUser.setProperty(IWPCUser.KEY_RESTRICTED_LANGUAGE, "fr_FR");	
        // END REMOVE
        mediator = WPCMediator.getInstance();
        catalogId = mediator.getDefaultCatalog();
        /*
         * Set the catalog for a user. If there was a catalog already set for a user, then re-use it. 
         * This is required because the context object is created for every call.
         */
      /*  PortletContext portletContext = getPortletConfig().getContext();
        WPSContext wpsContext = ((WPSContext) portletContext.getAttribute(strUserID));
        if (wpsContext != null) {
            String currentCatalogID = (String)wpsContext.getWpcUser().getProperty(IWPCUser.KEY_WPC_CATALOGUE_NAME);
            catalogId = (currentCatalogID != null)? currentCatalogID : mediator.getDefaultCatalog();
            String currentCatalogNode = (String)wpsContext.getWpcUser().getProperty(IWPCUser.KEY_WPC_CATALOGUE_NODE);
            if(currentCatalogNode!=null){
            catalogNode=currentCatalogNode;    
            }
         
         }
         
         wpcUser.setProperty(IWPCUser.KEY_WPC_CATALOGUE_NAME, catalogId);
         wpcUser.setProperty(IWPCUser.KEY_WPC_CATALOGUE_NODE, catalogNode);

        //set the user in context
        context = new WPSContext();
        context.setWpcUser(wpcUser);

        //return the context
        return context; */
    	PortletHelper helper = new PortletHelper(getPortletConfig());
    	WPSContext context = helper.createContext(request);
    	return context;
    }

    public static String createCommand(PortletURI portletURI,
        String actionString, String paramkey, String paramValue) {
        if (portletURI != null) {
            portletURI.addAction(actionString);
            portletURI.addParameter(paramkey, paramValue);

            return portletURI.toString();
        } else {
            return null;
        }
    }

    /**
     * Always returns ItemArray with one or multiple item ids.
     *
     * @param request
     * @param strKey
     * @return
     */
    protected String[] getReqParamArray(PortletRequest request, String strKey) {
        String[] itemArray = null;

        itemArray = request.getParameterValues(strKey);

        if (itemArray == null) {
            itemArray = new String[1];

            String item_0 = request.getParameter(strKey); //"ITEMID"

            if (item_0 != null) {
                itemArray[0] = item_0;
            } else {
                itemArray = new String[0];
            }
        }

        return itemArray;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.jetspeed.portlet.PortletAdapter#doHelp(org.apache.jetspeed.portlet.PortletRequest,
     *         org.apache.jetspeed.portlet.PortletResponse)
     */
    public void doHelp(PortletRequest portletRequest,
        PortletResponse portletResponse) throws PortletException, IOException {
        super.doHelp(portletRequest, portletResponse);

        String helpJSP = getClass().toString() + "_" +
            portletRequest.getMode() + "_.jsp";
        getPortletConfig().getContext().include(helpJSP, portletRequest,
            portletResponse);
    }
}
