
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/**
 * TransactionDisplayPortlet Porlet is used for fetching the Transactions from WPC .
 * and displaying the transactions list
 *
 */
package com.ibm.ccd.ui.portlets.alert;

import java.io.IOException;
import java.util.List;

import javax.xml.soap.SOAPException;

import org.apache.axis.attachments.AttachmentPart;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletSession;
import org.apache.jetspeed.portlet.PortletURI;
import org.apache.jetspeed.portlet.event.ActionEvent;
import org.apache.jetspeed.portlet.event.ActionListener;

import com.ibm.ccd.ui.alert.framework.ITransaction;
import com.ibm.ccd.ui.alert.framework.ITransactions;
import com.ibm.ccd.ui.context.IWPCUser;
import com.ibm.ccd.ui.context.WPCUser;
import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.mediator.WPCMediator;
import com.ibm.ccd.ui.mediator.WPCMediatorException;
import com.ibm.ccd.ui.portlets.BasePortlet;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;
import com.ibm.ccd.ui.util.ValidationUtil;


public class TransactionDisplayPortlet extends BasePortlet
    implements ActionListener, IReference {
    public static final String TRANSACTIONLIST_JSP = "/jsps/html/alert/transactionslist.";
    public static final String TRANSACTIONDETAILS_JSP = "/jsps/html/alert/transactiondetails.";
    public static final String VIEW_BEAN = "com.ibm.ccd.ui.portlets.alert.TransactionDisplayViewBean";

    /**
     * Returns the file extension for the JSP file
     * @param request PortletRequest
     * @param response PortletResponse
     * @return JSP extension
     */
    public void doView(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        log.debug(" Transdisplay doView  -> ");
        
        WPSContext context = null;
        WPCUser user = null;
        
        String JSP2RENDER = TRANSACTIONLIST_JSP;
        PortletSession session = request.getPortletSession();
		context = getWPSContext(request);
		
		String userName = (String) context.getWpcUser().getProperty(IWPCUser.KEY_USER_ID);;		

        MessageMap mmap = getMessageMap(request);
        setBusinessFunction(request);

        TransactionDisplayViewBean viewBean = null;

        try {
            // Invoke the JSP to render
            ITransactions transactionslist = alertDataSource.getTransactions(context);
            viewBean = (TransactionDisplayViewBean) session.getAttribute(ITransactionDisplay.TRANSACTION_VIEWBEAN);

            if (viewBean == null) {
                viewBean = new TransactionDisplayViewBean();
            }

            session.setAttribute(ITransactionDisplay.TRANSACTION_VIEWBEAN,
                viewBean);

            String command = viewBean.getCommand();

            if ((command != null) &&
                    (command.equals(
                        ITransactionDisplay.TRANSACTION_VIEW_DETAILS))) {
                JSP2RENDER = "";

                String transId = viewBean.getTransactionIdDisplayed();

                ITransaction transaction = transactionslist.getTransaction(transId);
                viewBean.setTransaction(transaction);

                String attachFilePath = "/" +
                    ITransactionDisplay.TRANSACTION_ATTACHMENT_DIR + "/" +
                    transaction.getAttachmentFilePath();
                viewBean.setAttachmentpath(attachFilePath);

                if (!(transaction.getXlsAttachmentFilePath().equals("") ||
                        transaction.getXlsAttachmentFilePath().equals(ITransactionDisplay.TRANSACTION_NO_ATTACH) ||
                        transaction.getXlsAttachmentFilePath().equals("null"))) {
                    String attachXlsFilePath = "/" +
                        ITransactionDisplay.TRANSACTION_ATTACHMENT_DIR + "/" +
                        transaction.getXlsAttachmentFilePath();
                    viewBean.setAttachmentXLSpath(attachXlsFilePath);
                }

                mmap.addInfoMessage("MESSAGE_CODE_306");
                JSP2RENDER = TRANSACTIONDETAILS_JSP;
            } else {
                if ((ValidationUtil.isNullOrEmpty(userName))) {
                    log.error(UIUtils.getTextMessages(
                            getPortletConfig().getContext(), request,
                            "MESSAGE_CODE_61"));
                    throw new Exception();
                }

                // Set actionURI in the view mode bean
                PortletURI formActionURI = response.createURI();

                //				viewBean =(TransactionDisplayViewBean) session.getAttribute(ITransactionDisplay.TRANSACTION_VIEWBEAN);
                //				if(viewBean == null){
                //					viewBean = new TransactionDisplayViewBean();
                //				}
                viewBean.setTransSet(transactionslist);

                if (transactionslist.getTransactionsSize() == 0) {
                    mmap.addInfoMessage("MESSAGE_CODE_305");
                } else {
                    mmap.addInfoMessage("MESSAGE_CODE_131");
                }
            }

            request.setAttribute(VIEW_BEAN, viewBean);

            getPortletConfig().getContext().include(JSP2RENDER +
                getJspExtension(request), request, response);
        } catch (Exception er) {
            er.printStackTrace();
        }

        log.debug("  doView <-  ");
    }

    /**
     * Returns the file extension for the JSP file
     * @param request PortletRequest
     * @return JSP extension
     */
    private static String getJspExtension(PortletRequest request) {
        String markupName = request.getClient().getMarkupName();

        return "jsp";
    }

    /**
     * Method  setBusinessFunction: This method is overridden and used for setting business function
     * in sesssion object.That is used for prepending keys of resource bundle.
     * @param request PortletRequest
     */
    protected void setBusinessFunction(PortletRequest portletRequest) {
        PortletSession session = portletRequest.getPortletSession();
        session.setAttribute(IReference.SESSION_BUS_FUNCT, "TRANSACTION_DISP");
        session.setAttribute(IReference.SESSION_ACTION, "VIEW");
    }

    /* (non-Javadoc)
     * @see org.apache.jetspeed.portlet.event.ActionListener#actionPerformed(org.apache.jetspeed.portlet.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent event) throws PortletException {
        log.debug(" actionPerformed ->");

        PortletRequest request = event.getRequest();
        WPSContext context = null;
        PortletSession session = request.getPortletSession();
        
		context = getWPSContext(request);
        

        Object attachmentObjxml = null;
        Object attachmentObjxls = null;

        String actionString = event.getActionString();

        //log.debug(" transactionId->"+transactionId);
        List attachmentList = null;

        if (actionString.equals(ITransactionDisplay.TRANSACTION_DETAILS)) {
            String transactionId = request.getParameter(ITransactionDisplay.TRANSACTION_REQ_TRANSID_KEY);

            if (transactionId == null) {
                return;
            }

            TransactionDisplayViewBean viewBean = (TransactionDisplayViewBean) session.getAttribute(ITransactionDisplay.TRANSACTION_VIEWBEAN);
            viewBean.setCommand(ITransactionDisplay.TRANSACTION_VIEW_DETAILS);
            viewBean.setTransactionIdDisplayed(transactionId);

            ITransactions transactionslist = alertDataSource.getTransactions(context);
            ITransaction transactionbean = transactionslist.getTransaction(transactionId);

            if (transactionbean.getAttachmentFilePath().equals("") ||
                    transactionbean.getXlsAttachmentFilePath().equals("")) {
                try {
                    attachmentList = WPCMediator.getInstance()
                                                .getTransactionsDetails(transactionId.trim(),
                            context.getWpcUser());
                } catch (WPCMediatorException e) {
                    getMessageMap(request).addErrorMessage("MESSAGE_CODE_307");
                    e.printStackTrace();
                    throw new PortletException();
                } catch (Exception e) {
                    getMessageMap(request).addErrorMessage("MESSAGE_CODE_106");
                    e.printStackTrace();
                    throw new PortletException();
                }

                try {
                    AttachmentPart attachPart1 = (AttachmentPart) attachmentList.get(0);
                    attachmentObjxml = attachPart1.getContent();

                    AttachmentPart attachPart2 = null;

                    if (attachmentList.size() > 1) {
                        attachPart2 = (AttachmentPart) attachmentList.get(1);
                        attachmentObjxls = attachPart2.getContent();
                    }

                    //							request.setAttribute("STATUS","DOWNLOADED");
                } catch (SOAPException e1) {
                    e1.printStackTrace();
                    throw new PortletException();
                }

                if (transactionslist.getTransaction(transactionId)
                                        .getAttachmentFilePath().equals("")) {
                    alertDataSource.saveAttachmentFile(transactionslist.getTransaction(
                            transactionId), attachmentObjxml,
                        ITransactionDisplay.ATTACHMENT_FILE_EXTENSION_XML);
                }

                if (transactionslist.getTransaction(transactionId)
                                        .getXlsAttachmentFilePath().equals("")) {
                    alertDataSource.saveAttachmentFile(transactionslist.getTransaction(
                            transactionId), attachmentObjxls,
                        ITransactionDisplay.ATTACHMENT_FILE_EXTENSION_XLS);
                }
            }

            //				else if (transactionbean.getAttachmentFilePath().equals(ITransactionDisplay.TRANSACTION_NO_ATTACH)){
            //					transactionbean.setAttachmentFilePath(ITransactionDisplay.TRANSACTION_NO_ATTACH);
            ////					transactionbean.setXlsAttachmentFilePath(ITransactionDisplay.TRANSACTION_NO_ATTACH);					
            //					alertDataSource.saveAttachmentFile(transactionslist.getTransaction(transactionId),attachmentObj);
            //				}
        } // End action string
        else {
            session.removeAttribute(ITransactionDisplay.TRANSACTION_VIEWBEAN);
        }
    } // function
}
