
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.portlets.catalog;

import java.util.Enumeration;


/**
 * <p>
 * This class Constructs a logical tree structure
 * which represents Catalog, Categories and Items.
 * The item is the leaf in this catalog tree
 * </p>
 */
public class CatalogTree {
    public static final int ROOT = 0;
    public static final int CATALOG = 1;
    public static final int CATEGORY = 2;
    public static final int ITEM = 3;
    public static final int HIERARCHY = 4;
    public static final int UNASSIGNED = 5;

    /** root of the Catalog Tree */
    private ItemNodeCatalog root;

    /**
     * Creates a Catalog Tree with root node
     *
     */
    public CatalogTree() {
        root = new ItemNodeCatalog("0", "0", ROOT, "root");
        root.setExpanded(true);
    }

    /**
     *
     * @param start search for the item from this node
     * @param id search by item id
     * @return itemNode return itemNode from start of the tree otherwise null
     */
    public ItemNodeCatalog searchNode(ItemNodeCatalog start, String id) {
        ItemNodeCatalog itemFound = null;
        Enumeration enumValues = start.breadthFirstEnumeration();

        while (enumValues.hasMoreElements()) {
            itemFound = (ItemNodeCatalog) enumValues.nextElement();

            if (!itemFound.getId().equals(id)) {
                continue;
            } else {
                return itemFound;
            }
        }

        return null;
    }

    public ItemNodeCatalog searchNode(ItemNodeCatalog start,
        ItemNodeCatalog search) {
        ItemNodeCatalog itemFound = null;
        Enumeration enumValues = start.breadthFirstEnumeration();

        while (enumValues.hasMoreElements()) {
            itemFound = (ItemNodeCatalog) enumValues.nextElement();

            if (itemFound != search) {
                continue;
            } else {
                return itemFound;
            }
        }

        return null;
    }

    /**
     * Return the root node of the tree
     * @return root
     */
    public ItemNodeCatalog getRoot() {
        return root;
    }

    public void add(ItemNodeCatalog child) {
        root.add(child);
    }

    /**
     * Add child node to this parent node
     * @param parent
     * @param child
     */
    public void addChild(ItemNodeCatalog parent, ItemNodeCatalog child) {
        parent.add(child);
    }

    /**
     * Removes this node from its Parent Node
     * @param child
     */
    public void removeChild(ItemNodeCatalog child) {
        if (child.getParent() != null) {
            ItemNodeCatalog parent = (ItemNodeCatalog) child.getParent();
            parent.getChildren().remove(child);
        }
    }
}
