
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.portlets.catalog;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import com.ibm.ccd.ui.dataobjects.CategoryHierarchy;
import com.ibm.ccd.ui.dataobjects.items.ICatalogItem;


/**
 * <p>
 * ItemNodeCatalog is a logical node in the tree data Structure
 * </p>
 *
 */
public class ItemNodeCatalog {
    private ItemNodeCatalog parent;
    private String id;
    private String wpcID;
    private int type;
    private String displayName;
    private boolean isExpanded;
    private boolean isSelected;
    private boolean isLeaf;
    private ICatalogItem iCatalogItem;
    private CategoryHierarchy category;
    private Object object;
    private List children;

    public ItemNodeCatalog(String id) {
        this.id = id;
    }

    public ItemNodeCatalog(String id, int type, String displayName) {
        this.id = id;
        this.type = type;
        this.displayName = displayName;
    }

    public ItemNodeCatalog(String id, String wpcID, int type, String displayName) {
        this.id = id;
        this.wpcID = wpcID;
        this.type = type;
        this.displayName = displayName;
    }

    /**
     * @return displayName of the node
     */
    public String getDisplayName() {
        return displayName;
    }

    /**
     * @return id of the node
     */
    public String getId() {
        return id;
    }

    /**
     * @return state of the node ie., expanded or collapsed
     */
    public boolean isExpanded() {
        return isExpanded;
    }

    /**
     * @return state of the node ie., selecte
     */
    public boolean isSelected() {
        return isSelected;
    }

    /**
     * @return type of the node Catalog, Category or Item
     */
    public int getType() {
        return type;
    }

    /**
     * @param displayName
     */
    public void setDisplayName(String displayName) {
        this.displayName = displayName;
    }

    /**
     * @param id
     */
    public void setId(String id) {
        this.id = id;
    }

    /**
     * @param expanded
     */
    public void setExpanded(boolean b) {
        isExpanded = b;
    }

    /**
     * @param selected
     */
    public void setSelected(boolean b) {
        isSelected = b;
    }

    /**
     * @param type
     */
    public void setType(int type) {
        this.type = type;
    }

    /**
     * @return
     */
    public ICatalogItem getICatalogItem() {
        return iCatalogItem;
    }

    /**
     * @param item
     */
    public void setICatalogItem(ICatalogItem item) {
        iCatalogItem = item;
    }

    /**
     * @return
     */
    public boolean isLeaf() {
        if (children != null) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * @return
     */
    public ItemNodeCatalog getParent() {
        return parent;
    }

    /**
     * @param b
     */
    private void setLeaf(boolean b) {
        isLeaf = b;
    }

    /**
            * @param catalog
            */
    public void setParent(ItemNodeCatalog catalog) {
        parent = catalog;
    }

    /**
     * @return
     */
    public List getChildren() {
        if (children == null) {
            children = new ArrayList();
        }

        return children;
    }

    /**
     * @param list
     */
    public void setChildren(List list) {
        children = list;
    }

    /**
     * @return
     */
    public Object getObject() {
        return object;
    }

    /**
     * @param object
     */
    public void setObject(Object object) {
        this.object = object;
    }

    public void add(ItemNodeCatalog child) {
        if (children == null) {
            children = new ArrayList();
        }

        if (!childExist(child.getId())) {
            child.setParent(this);
            children.add(child);
        }
    }

    protected boolean childExist(String id) {
        Iterator iter = children.iterator();
        ItemNodeCatalog node = null;

        while (iter.hasNext()) {
            node = (ItemNodeCatalog) iter.next();

            if (!node.getId().equals(id)) {
                continue;
            } else {
                return true;
            }
        }

        return false;
    }

    public ItemNodeCatalog search(ItemNodeCatalog node, String id) {
        List list = (List) node.getChildren();
        Iterator iter = list.iterator();
        ItemNodeCatalog child = null;

        while (iter.hasNext()) {
            child = (ItemNodeCatalog) iter.next();

            if (child.getId() == id) {
                return child;
            }

            search(child, id);
        }

        return null;
    }

    public Enumeration traverse(ItemNodeCatalog node) {
        Vector v = null;

        ItemNodeCatalog child = null;

        if (!node.isLeaf()) {
            v = new Vector();

            Enumeration enumValues = (Enumeration) node.children();

            while (enumValues.hasMoreElements()) {
                child = (ItemNodeCatalog) enumValues.nextElement();

                //System.out.println(child);
                v.add(child);
                traverse(child);
            }
        } else {
            v.add(child);
        }

        //System.out.println("Elements " + v);
        return v.elements();
    }

    public String getPathFromRoot() {
        return getPathFromRoot("/");
    }

    public String getPathFromRoot(String separator) {
        if (separator == null) {
            separator = "/";
        }

        List chain = new ArrayList();

        //chain.add(String.valueOf(getId()));
        chain.add(String.valueOf(getWpcID()));

        ItemNodeCatalog parent = getParent();

        while (parent != null) {
            //chain.add(String.valueOf(parent.getId()));
            chain.add(String.valueOf(parent.getWpcID()));
            parent = parent.getParent();
        }

        StringBuffer sb = new StringBuffer();

        for (int i = chain.size() - 1; i >= 0; i--) {
            sb.append(separator).append(chain.get(i));
        }

        return sb.toString();
    }

    public String getPathFromLevel(String separator, int level) {
        if (separator == null) {
            separator = "/";
        }

        if (level < 0) {
            level = 0;
        }

        List chain = new ArrayList();

        //chain.add(String.valueOf(getId()));
        chain.add(getWpcID());

        ItemNodeCatalog parent = getParent();

        while (parent != null) {
            //chain.add(String.valueOf(parent.getId()));
            chain.add(parent.getWpcID());
            parent = parent.getParent();
        }

        StringBuffer sb = new StringBuffer();

        for (int i = chain.size() - 1 - level; i >= 0; i--) {
            sb.append(separator).append(chain.get(i));
        }

        return sb.toString();
    }

    public ItemNodeCatalog[] getPath() {
        int level = getLevel();
        ItemNodeCatalog[] chain = new ItemNodeCatalog[level + 2];

        ItemNodeCatalog current = this;

        while (current != null) {
            chain[level--] = current;

            //System.out.print(current+"/");
            current = current.getParent();
        }

        return chain;
    }

    public int getLevel() {
        int i = 0;
        ItemNodeCatalog current = this;

        while ((current = current.getParent()) != null) {
            i++;
        }

        return i;
    }

    public ItemNodeCatalog getRoot() {
        ItemNodeCatalog current = this;
        ItemNodeCatalog parent = getParent();

        while (parent != null) {
            current = parent;
            parent = parent.getParent();
        }

        return current;
    }

    public Enumeration depthFirstEnumeration() {
        ItemNodeCatalog current = this;
        Enumeration enumValues = new PostTraverseNode().traverseDepth(current);

        return enumValues;
    }

    public Enumeration breadthFirstEnumeration() {
        ItemNodeCatalog current = this;
        Enumeration enumValues = new PostTraverseNode().traverseBreadth(current);

        return enumValues;
    }

    public Enumeration children() {
        if (children != null) {
            Vector v = new Vector(children);

            return v.elements();
        }

        return null;
    }

    public void remove(ItemNodeCatalog node) {
        children.remove(node);
    }

    /**
     * @return
     */
    public CategoryHierarchy getCategory() {
        return category;
    }

    /**
     * @param hierarchy
     */
    public void setCategory(CategoryHierarchy hierarchy) {
        category = hierarchy;
    }

    /**
     * @return
     */
    public String getWpcID() {
        return wpcID.substring(wpcID.lastIndexOf('/') + 1);
    }

    /**
     * @param wpcID
     */
    public void setWpcID(String wpcID) {
        this.wpcID = wpcID;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return (this.wpcID != null) ? this.wpcID : "";
    }

    private class PostTraverseNode {
        Vector v = new Vector();

        public Enumeration traverseDepth(ItemNodeCatalog node) {
            ItemNodeCatalog child = null;

            if (!node.isLeaf()) {
                Enumeration enumValues = (Enumeration) node.children();

                while (enumValues.hasMoreElements()) {
                    child = (ItemNodeCatalog) enumValues.nextElement();
                    v.add(child);
                    traverse(child);
                }
            }

            return v.elements();
        }

        public Enumeration traverseBreadth(ItemNodeCatalog node) {
            Vector v1 = new Vector();
            ItemNodeCatalog child = null;

            if (!node.isLeaf()) {
                v1.addAll(node.children);
                v.addAll(node.children);

                Iterator iter = v1.iterator();

                while (iter.hasNext()) {
                    child = (ItemNodeCatalog) iter.next();
                    traverseBreadth(child);
                }
            } else {
                v1.add(node);
            }

            return v.elements();
        }
    }
}
