
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * Created on Nov 24, 2004
 */
package com.ibm.ccd.ui.portlets.fileupload;

import java.io.Serializable;
import java.util.HashMap;

import com.ibm.ccd.ui.context.WPSContext;


/**
 * This class is a data transfer object. Used for confg passing data to <code>FileUploadUtil</code>
 */
public class FileUploadBean implements Serializable {
    /**
     * Contains the WPSContext
     */
    private WPSContext context = null;

    /**
     * contains the form action URI
     */
    private String formActionURI = null;

    /**
     * contains the file transfer protocol
     */
    private String protocol = null;

    /**
     * contains FTP host IP address
     */
    private String ftpHost = null;

    /**
     * Contains FTP port
     */
    private String ftpPort = null;

    /**
     * contains the FTP host user name
     */
    private String ftpUser = null;

    /**
     * contains password of FTP user
     */
    private String ftpPassword = null;

    /**
     * contains ftp folder anme.
     */
    private String ftpFolder = null;

    /**
     * contains proxy server URL
     */
    private String proxyURL = null;

    /**
     * contains the proxy port
     */
    private String proxyPort = null;

    /**
     * contais the proxy server user name
     */
    private String proxyUser = null;

    /**
     * contains password for proxy server user
     */
    private String proxyPassword = null;

    /**
     * contains name of file to be uploaded
     */
    private String fileName = null;

    /**
     * contains content type of file
     */
    private String contentType = null;

    /**
     * contains the docstore path of file in WPC
     */
    private String documentPath = null;

    /**
     * contains the docstore path of file in WPC
     */
    private String collabName = null;

    /**
     * returns FTp folder anme
     * @return String
     */
    public String getFtpFolder() {
        return ftpFolder;
    }

    /**
     * returns FTP host name
     * @return String
     */
    public String getFtpHost() {
        return ftpHost;
    }

    /**
     * returns FTp user password
     * @return String
     */
    public String getFtpPassword() {
        return ftpPassword;
    }

    /**
     * returns FTP port
     * @return String
     */
    public String getFtpPort() {
        return ftpPort;
    }

    /**
     * returns FTP user
     * @return String
     */
    public String getFtpUser() {
        return ftpUser;
    }

    /**
     * returns file transfer protocol
     * @return String
     */
    public String getProtocol() {
        return protocol;
    }

    /**
     *  sets the FTp folder name
     * @param string
     */
    public void setFtpFolder(String string) {
        ftpFolder = string;
    }

    /**
     * Sets the FTp host name
     * @param string
     */
    public void setFtpHost(String string) {
        ftpHost = string;
    }

    /**
     * Sets the FTp password
     * @param string
     */
    public void setFtpPassword(String string) {
        ftpPassword = string;
    }

    /**
     * Sets the FTP port
     * @param string
     */
    public void setFtpPort(String string) {
        ftpPort = string;
    }

    /**
     * Sets the FTP user name
     * @param string
     */
    public void setFtpUser(String string) {
        ftpUser = string;
    }

    /**
     * Sets the file transfer protocol
     * @param string
     */
    public void setProtocol(String string) {
        protocol = string;
    }

    /**
     * returns proxy server password
     * @return String
     */
    public String getProxyPassword() {
        return proxyPassword;
    }

    /**
     * returns proxy server port
     * @return String
     */
    public String getProxyPort() {
        return proxyPort;
    }

    /**
     * returns proxy server URL
     * @return String
     */
    public String getProxyURL() {
        return proxyURL;
    }

    /**
     * returns proxy server user name
     * @return String
     */
    public String getProxyUser() {
        return proxyUser;
    }

    /**
     * sets the proxy server password
     * @param string
     */
    public void setProxyPassword(String string) {
        proxyPassword = string;
    }

    /**
     * sets the proxy server port
     * @param string
     */
    public void setProxyPort(String string) {
        proxyPort = string;
    }

    /**
     * sets the proxy server URL
     * @param string
     */
    public void setProxyURL(String string) {
        proxyURL = string;
    }

    /**
     * sets the proxy server user name
     * @param string
     */
    public void setProxyUser(String string) {
        proxyUser = string;
    }

    /**
     * returns the form action URI
     * @return String
     */
    public String getFormActionURI() {
        return formActionURI;
    }

    /**
     * sets the form action URI
     * @param string
     */
    public void setFormActionURI(String string) {
        formActionURI = string;
    }

    /**
     * returns the file name
     * @return String
     */
    public String getFileName() {
        return fileName;
    }

    /**
     * sets the file name
     * @param string
     */
    public void setFileName(String string) {
        fileName = string;
    }

    /**
     * returns <code>HashMap</code> with key/ value pair of supported protocols for file transfer.
     * @return HashMap
     */
    public HashMap getProtocolMap() {
        HashMap protocolMap = null;
        protocolMap = new HashMap();
        protocolMap.put(IFileUpload.PROTOCOL_FTP, IFileUpload.PROTOCOL_FTP);
        protocolMap.put(IFileUpload.PROTOCOL_SOAP, IFileUpload.PROTOCOL_SOAP);

        return protocolMap;
    }

    /**
     * returns the content type of file
     * @return
     */
    public String getContentType() {
        return contentType;
    }

    /**
     * sets the content tye of file.
     * @param string
     */
    public void setContentType(String string) {
        contentType = string;
    }

    /**
     * returns the file path in WPC docstore
     * @return String
     */
    public String getDocumentPath() {
        return documentPath;
    }

    /**
     * sets the  WPC docstore file path
     * @param string
     */
    public void setDocumentPath(String string) {
        documentPath = string;
    }

    /**
     * @return
     */
    public WPSContext getContext() {
        return context;
    }

    /**
     * @param context
     */
    public void setContext(WPSContext context) {
        this.context = context;
    }

    /**
     * @return
     */
    public String getCollabName() {
        return collabName;
    }

    /**
     * @param string
     */
    public void setCollabName(String string) {
        collabName = string;
    }
}
