
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * Created on Nov 30, 2004
 *
 */
package com.ibm.ccd.ui.portlets.fileupload;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.Authenticator;
import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Properties;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletSession;
import org.apache.jetspeed.portlet.PortletSettings;
import org.apache.jetspeed.portlet.PortletURI;
import org.apache.jetspeed.portlet.UnavailableException;
import org.apache.jetspeed.portlet.event.ActionEvent;
import org.apache.jetspeed.portlet.event.ActionListener;

import sun.net.ftp.FtpClient;

import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.exception.UIException;
import com.ibm.ccd.ui.mediator.WPCMediatorException;
import com.ibm.ccd.ui.portlets.BasePortlet;
import com.ibm.ccd.ui.portlets.alert.ITransactionDisplay;
import com.ibm.ccd.ui.util.FileUploadUtil;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.SimpleAuthenticator;
import com.ibm.ccd.ui.util.UIUtils;
import com.ibm.ccd.ui.util.ValidationUtil;


/**
  * This is a Portlet class for File upload function
  */
public class FileUploadPortlet extends BasePortlet implements ActionListener {
    /**
     * This method is called by container while doing initialisation.
     * @param  portletConfig
     * @throws UnavailableException
     */
    public void init(PortletConfig portletConfig) throws UnavailableException {
        super.init(portletConfig);
    }

    /**
     * This method is called by the container in view mode to display view to user
     *@param request
     *@param response
     * @throws PortletException, IOException
     */
    public void doView(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        PortletURI formActionURI = null;
        PortletSession portletSession = null;
        MessageMap messageMap = null;
        FileUploadBean viewBean = null;

        try {
            portletSession = request.getPortletSession();
            setBusinessFunction(request);
            portletSession.setAttribute(IReference.SESSION_ACTION,
                IFileUpload.BUSINESS_ACTION_VIEW);

            //add info message
            messageMap = getMessageMap(request);
            messageMap.addInfoMessage(IFileUpload.MSG_VIEW_INFO_MESSAGE);

            //		Make a view mode bean
            viewBean = new FileUploadBean();
            request.setAttribute(IFileUpload.FILE_UPLOAD_BEAN, viewBean);

            //		Set actionURI in the view mode bean
            formActionURI = response.createURI();
            formActionURI.addAction(IFileUpload.FORM_ACTION_FILE_UPLOAD);
            viewBean.setFormActionURI(formActionURI.toString());

            getPortletConfig().getContext().include(IFileUpload.PATH_FILE_UPLOAD_VIEW_JSP,
                request, response);
        } catch (Exception exception) {
            throw new PortletException(exception);
        }
    }

    /**
     * This method is called by container when a user sends a http request to server
     * @param event
     * @throws PortletException
     */
    public void actionPerformed(ActionEvent event) throws PortletException {
        String actionString = null;
        String fileName = null;
        String contentType = null;
        PortletRequest request = null;
        InputStream inputStream = null;
        File file = null;
        MessageMap messageMap = null;

        try {
            actionString = event.getActionString();

            // Add action string handler here
            request = event.getRequest();
            messageMap = getMessageMap(request);

            if (actionString.equals(IFileUpload.FORM_ACTION_FILE_UPLOAD)) {
                try {
                    MultipartRequest multi = new MultipartRequest(request);
                    Enumeration files = multi.getFileNames();

                    while (files.hasMoreElements()) {
                        String name = (String) files.nextElement();
                        fileName = multi.getFilesystemName(name);
                        contentType = multi.getContentType(fileName);
                        file = multi.getFile(name);

                        if (file == null) {
                            messageMap.addErrorMessage(IFileUpload.MSG_FILED_MANDATORY,
                                getTextLabel(request,
                                    IFileUpload.PROPERTY_FILE_NAME));

                            break;
                        }

                        inputStream = new FileInputStream(file);
                        uploadFile(fileName, contentType, inputStream, request);
                    }
                } catch (IOException e) {
                    messageMap.addErrorMessage(IFileUpload.MSG_FILE_NOT_FOUND);
                    e.printStackTrace();
                    throw new PortletException(e);
                } catch (UIException e) {
                    messageMap.addErrorMessage(IFileUpload.MSG_FILE_UPLOAD_FAILED);
                    e.printStackTrace();
                    throw new PortletException(e);
                }
            }

            //if in config mode
            if (actionString.equals(IFileUpload.FORM_ACTION_CONFIG)) {
                try {
                    saveConfiguration(request);
                    messageMap.addInfoMessage(IFileUpload.MSG_FILE_UPLOAD_CONFIG_SAVED);

                    //messageMap.addInfoMessage(IFileUpload.MSG_VIEW_LINK);
                } catch (Exception e) {
                    e.printStackTrace();
                    throw new PortletException(e);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new PortletException(e);
        }
    }

    /**
     * This method is called to upload the file. It instantiates the <code>fileUploadUtil</code>
     * and starts the upload. The upload takes place in a seperate thread.
     * @param fileName name of the file
     * @param inputStream byte stream of file
     * @throws Exception
     */
    private String uploadFile(String fileName, String contentType,
        InputStream inputStream, PortletRequest request)
        throws UIException, Exception {
        String uploadResponse = null;
        FileUploadUtil fileUploadUtil = null;
        FileUploadBean fileUploadBean = null;
        String ftpHost = null;
        String user = null;
        String ftpPassword = null;
        String ftpPort = null;
        String ftpFolder = null;
        String alertMessage = null;
        MessageMap messageMap = null;
        WPSContext context = null;
        PortletSettings portletSettings = this.getPortletSettings();

        //read general config settings
        String protocol = portletSettings.getAttribute(IFileUpload.PROPERTY_PROTOCOL);

        if (ValidationUtil.isNullOrEmpty(protocol)) {
            protocol = IFileUpload.PROTOCOL_SOAP;
        }

        //read FTP config settings	
        ftpHost = portletSettings.getAttribute(IFileUpload.PROPERTY_FTP_HOST);
        user = portletSettings.getAttribute(IFileUpload.PROPERTY_FTP_USER);
        ftpPassword = portletSettings.getAttribute(IFileUpload.PROPERTY_FTP_PASSWORD);
        ftpPort = portletSettings.getAttribute(IFileUpload.PROPERTY_FTP_PORT);
        ftpFolder = portletSettings.getAttribute(IFileUpload.PROPERTY_FTP_FOLDER);

        //put validation logic at later stage if required
        context = getWPSContext(request);

        fileUploadBean = new FileUploadBean();
        fileUploadBean.setContext(context);
        fileUploadBean.setProtocol(protocol);
        fileUploadBean.setFtpHost(ftpHost);
        fileUploadBean.setFtpUser(user);
        fileUploadBean.setFtpPassword(ftpPassword);
        fileUploadBean.setFtpPort(ftpPort);
        fileUploadBean.setFtpFolder(ftpFolder);
        fileUploadBean.setFileName(fileName);
        fileUploadBean.setContentType(contentType);

        //set message for alerts portlet
        fileUploadUtil = new FileUploadUtil(inputStream, fileUploadBean);

        try {
            uploadResponse = fileUploadUtil.uploadFile();
        } catch (WPCMediatorException we) {
            Object[] values = new Object[2];
            values[0] = IFileUpload.QUOTES + fileName + IFileUpload.QUOTES;
            values[1] = IFileUpload.QUOTES + fileUploadBean.getDocumentPath() +
                IFileUpload.QUOTES;
            alertMessage = getAlertMessage(request,
                    IFileUpload.MSG_TRANSACTION_DESCRIPTION, values);
            request.setAttribute(ITransactionDisplay.TRANSACTION_DESCRIPTION_KEY,
                alertMessage);
            handleException(we, request);
            messageMap = getMessageMap(request);

            //		set success message
            messageMap.addInfoMessage(IFileUpload.MSG_TRANSACTION_DESCRIPTION,
                values);
        }

        return uploadResponse;
    }

    /**
     * This method is called by container when administrator clicks on the configure button on toolbar.
     * @param request , response
     * @throws PortletException
     * @throws  IOException
     */
    public void doConfigure(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        PortletURI formActionURI = null;
        PortletSession portletSession = null;
        PortletSettings portletSettings = null;
        FileUploadBean configBean = null;
        portletSession = request.getPortletSession();
        setBusinessFunction(request);
        portletSession.setAttribute(IReference.SESSION_ACTION,
            IFileUpload.BUSINESS_ACTION_CONFIG);
        portletSettings = request.getPortletSettings();

        configBean = new FileUploadBean();
        request.setAttribute(IFileUpload.FILE_UPLOAD_BEAN, configBean);

        //		Set actionURI in the view mode bean
        formActionURI = response.createURI();
        formActionURI.addAction(IFileUpload.FORM_ACTION_CONFIG);
        configBean.setFormActionURI(formActionURI.toString());

        //read general confoguration parameters
        configBean.setProtocol(portletSettings.getAttribute(
                IFileUpload.PROPERTY_PROTOCOL));
        configBean.setProxyURL(portletSettings.getAttribute(
                IFileUpload.PROPERTY_PROXY_URL));
        configBean.setProxyPort(portletSettings.getAttribute(
                IFileUpload.PROPERTY_PROXY_PORT));
        configBean.setProxyUser(portletSettings.getAttribute(
                IFileUpload.PROPERTY_PROXY_USER));
        configBean.setProxyPassword(portletSettings.getAttribute(
                IFileUpload.PROPERTY_PROXY_PASSWORD));

        //read FTP confoguration parameters
        configBean.setFtpHost(portletSettings.getAttribute(
                IFileUpload.PROPERTY_FTP_HOST));
        configBean.setFtpPort(portletSettings.getAttribute(
                IFileUpload.PROPERTY_FTP_PORT));
        configBean.setFtpUser(portletSettings.getAttribute(
                IFileUpload.PROPERTY_FTP_USER));
        configBean.setFtpPassword(portletSettings.getAttribute(
                IFileUpload.PROPERTY_FTP_PASSWORD));
        configBean.setFtpFolder(portletSettings.getAttribute(
                IFileUpload.PROPERTY_FTP_FOLDER));

        getPortletConfig().getContext().include(IFileUpload.PATH_FILE_UPLOAD_CONFIG_JSP,
            request, response);
    }

    /**
     * This method saves the configuratin settings to <code>PortletSettings</code>
     * and proxy settings to System properties.
     * @param request
     * @throws Exception
     */
    private void saveConfiguration(PortletRequest request)
        throws Exception {
        PortletSettings portletSettings = null;
        MessageMap messageMap = null;
        String protocol = null;
        String proxyURL = null;
        String proxyPort = null;
        String proxyUser = null;
        String proxyPassword = null;
        String ftpHost = null;
        String ftpUser = null;
        String ftpPassword = null;
        String ftpPort = null;
        String ftpFolder = null;

        messageMap = getMessageMap(request);

        //read general config parameters
        protocol = request.getParameter(IFileUpload.PROPERTY_PROTOCOL);
        proxyURL = request.getParameter(IFileUpload.PROPERTY_PROXY_URL);
        proxyPort = request.getParameter(IFileUpload.PROPERTY_PROXY_PORT);
        proxyUser = request.getParameter(IFileUpload.PROPERTY_PROXY_USER);
        proxyPassword = request.getParameter(IFileUpload.PROPERTY_PROXY_PASSWORD);

        //read FTP config parameters
        ftpHost = request.getParameter(IFileUpload.PROPERTY_FTP_HOST);
        ftpUser = request.getParameter(IFileUpload.PROPERTY_FTP_USER);
        ftpPassword = request.getParameter(IFileUpload.PROPERTY_FTP_PASSWORD);
        ftpPort = request.getParameter(IFileUpload.PROPERTY_FTP_PORT);
        ftpFolder = request.getParameter(IFileUpload.PROPERTY_FTP_FOLDER);

        if (UIUtils.isEmpty(protocol)) {
            messageMap.addErrorMessage(IFileUpload.MSG_FILED_MANDATORY,
                getTextLabel(request, IFileUpload.PROPERTY_PROTOCOL));
            messageMap.addInfoMessage(IFileUpload.MSG_VIEW_LINK);
        }

        if (UIUtils.isEmpty(ftpHost)) {
            messageMap.addErrorMessage(IFileUpload.MSG_FILED_MANDATORY,
                getTextLabel(request, IFileUpload.PROPERTY_PROXY_FTP_HOST));
            messageMap.addInfoMessage(IFileUpload.MSG_VIEW_LINK);
        }

        //test the FTP Connection
        testFTPConnectin(ftpHost, ftpPort, ftpUser, ftpPassword, ftpFolder,
            request);

        // testing of proxy connection is still not done.
        //save the proxy settings configuration
        saveProxySettings(proxyURL, proxyPort, proxyUser, proxyPassword);

        portletSettings = this.getPortletSettings();

        //save general portlet config settings
        portletSettings.setAttribute(IFileUpload.PROPERTY_PROTOCOL, protocol);
        portletSettings.setAttribute(IFileUpload.PROPERTY_PROXY_URL, proxyURL);
        portletSettings.setAttribute(IFileUpload.PROPERTY_PROXY_PORT, proxyPort);
        portletSettings.setAttribute(IFileUpload.PROPERTY_PROXY_USER, proxyUser);
        portletSettings.setAttribute(IFileUpload.PROPERTY_PROXY_PASSWORD,
            proxyPassword);

        //save FTP Config portlet settings 
        portletSettings.setAttribute(IFileUpload.PROPERTY_FTP_HOST, ftpHost);
        portletSettings.setAttribute(IFileUpload.PROPERTY_FTP_PORT, ftpPort);
        portletSettings.setAttribute(IFileUpload.PROPERTY_FTP_USER, ftpUser);
        portletSettings.setAttribute(IFileUpload.PROPERTY_FTP_PASSWORD,
            ftpPassword);
        portletSettings.setAttribute(IFileUpload.PROPERTY_FTP_FOLDER, ftpFolder);
        portletSettings.store();

        //save proxy settings to the Sysrem.properties files
    }

    /**
     * This method saves the proxy FTP data to system files
     * @param proxy
     * @param port
     * @param user
     * @param password
     */
    private void saveProxySettings(String proxy, String port, String user,
        String password) {
        Properties systemProperties = null;
        Authenticator.setDefault(new SimpleAuthenticator(user, password));
        systemProperties = System.getProperties();
        systemProperties.setProperty(IFileUpload.PROPERTY_PROXY_FTP_HOST, proxy);
        systemProperties.setProperty(IFileUpload.PROPERTY_PROXY_FTP_PORT, port);
    }

    /**
     * Test the FTP connection
     * @param host
     * @param port
     * @param user
     * @param password
     * @param folder
     * @param request
     * @throws Exception
     */
    private void testFTPConnectin(String host, String port, String user,
        String password, String folder, PortletRequest request)
        throws Exception {
        MessageMap messageMap = null;
        FtpClient client = new FtpClient();

        try {
            client.openServer(host.trim() + IFileUpload.SEPERATOR_SPACE +
                port.trim());
        } catch (Exception e) {
            messageMap = getMessageMap(request);
            messageMap.addErrorMessage(IFileUpload.MSG_INVALID_FTP_HOST);
            throw e;
        }

        if (ValidationUtil.isNullOrEmpty(user)) {
            user = IFileUpload.DEFAULT_FTP_USER;
            password = IFileUpload.DEFAULT_FTP_PASSWPORD;
        }

        try {
            client.login(user, password);
        } catch (Exception e) {
            messageMap = getMessageMap(request);
            messageMap.addErrorMessage(IFileUpload.MSG_INVALID_FTP_USER);
            throw e;
        }

        //set tupe to binary
        client.binary();

        //change dir
        if (!ValidationUtil.isNullOrEmpty(folder)) {
            try {
                client.cd(folder);
            } catch (Exception e) {
                messageMap = getMessageMap(request);
                messageMap.addErrorMessage(IFileUpload.MSG_INVALID_FTP_FOLDER);
                throw e;
            }
        }
    }

    /**
     * Handles the Portlet help events
     * @param request
     * @param  response
     * @throws PortletException
     * @throws IOException
     */
    public void doHelp(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        getPortletConfig().getContext().include(IFileUpload.PATH_FILE_UPLOAD_HELP_JSP,
            request, response);
    }

    /**
     * This method sets the business function in session.
     * @param request
     */
    protected void setBusinessFunction(PortletRequest request) {
        PortletSession portletSession = null;
        portletSession = request.getPortletSession();
        portletSession.setAttribute(IReference.SESSION_BUS_FUNCT,
            IFileUpload.BUSINESS_FUNCTION_FILE_UPLOAD);
    }

    /**
     * This method returns the message used for alert portlet
     */
    private String getAlertMessage(PortletRequest request, String key,
        Object[] obj) {
        String message = null;
        String formattedMessage = null;
        message = getTextMessages(request, key);

        MessageFormat msgFormat = new MessageFormat(message);
        formattedMessage = msgFormat.format(obj);

        return formattedMessage;
    }
}
