
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * Created on Nov 26, 2004
  */
package com.ibm.ccd.ui.portlets.fileupload;

import java.io.IOException;

import javax.servlet.ServletInputStream;


/**
 *          A class to aid in reading multipart/form-data from a ServletInputStream.
 *          It keeps track of how many bytes have been read and detects when the
 *          Content-Length limit has been reached. This is necessary because some
 *          servlet engines are slow to notice the end of stream
 */
public class MultipartInputStreamHandler {
    /**
     * contains the  <code>ServletInputStream</code>
     */
    private ServletInputStream in;

    /**
     * Indiactes the boundary in request
     */
    private String boundary;

    /**
     *  Indicates totalExpected
     */
    private int totalExpected;

    /**
     * indicates  totalRead
     */
    private int totalRead = 0;

    /**
     * byte array
     */
    private byte[] buf = new byte[8 * 1024];

    /**
     * Constructor
     * @param pInputStream
     * @param pBoundary
     * @param pTotalExpected
     */
    public MultipartInputStreamHandler(ServletInputStream pInputStream,
        String pBoundary, int pTotalExpected) {
        this.in = pInputStream;
        this.boundary = pBoundary;
        this.totalExpected = pTotalExpected;
    }

    /**
     * reads a line from <code>ServletInputStream</code>
     * @return String
     * @throws IOException
     */
    public String readLine() throws IOException {
        StringBuffer sbuf = new StringBuffer();
        int result;
        String line;

        do {
            result = this.readLine(buf, 0, buf.length);

            // this.readLine() does +=
            if (result != -1) {
                sbuf.append(new String(buf, 0, result, "UTF-8"));
            }
        } while (result == buf.length); // loop only if the buffer was filled

        if (sbuf.length() == 0) {
            return null; // nothing read, must be at the end of stream
        }

        sbuf.setLength(sbuf.length() - 2); // cut off the trailing \r\n

        return sbuf.toString();
    }

    /**
     * reads the line
     * @param b
     * @param off
     * @param len
     * @return
     * @throws IOException
     */
    public int readLine(byte[] b, int off, int len) throws IOException {
        if (totalRead >= totalExpected) {
            return -1;
        } else {
            int result = in.readLine(b, off, len);

            if (result > 0) {
                totalRead += result;
            }

            return result;
        }
    }
}
