
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * Created on Dec 8, 2004
 *
 */
package com.ibm.ccd.ui.portlets.items;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;

import org.apache.jetspeed.portlet.AccessDeniedException;
import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletSession;
import org.apache.jetspeed.portlet.PortletSettings;
import org.apache.jetspeed.portlet.UnavailableException;
import org.apache.jetspeed.portlet.event.ActionEvent;
import org.apache.jetspeed.portlet.event.ActionListener;
import org.apache.jetspeed.portlet.event.MessageEvent;
import org.apache.jetspeed.portlet.event.MessageListener;

import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.dataobjects.CategoryHierarchy;
import com.ibm.ccd.ui.dataobjects.FileUploadResource;
import com.ibm.ccd.ui.dataobjects.NVPair;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.AttributeTypes;
import com.ibm.ccd.ui.dataobjects.attributes.BinaryAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.CurrencyAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.DateAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.FlagAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ImageAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ImageURLAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.IntegerAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.LookupTableAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberEnumAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.PasswordAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.RelationshipAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringEnumAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ThumbnailImageAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ThumbnailImageURLAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.TimeZoneAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.URLAttribute;
import com.ibm.ccd.ui.dataobjects.items.CatalogItem;
import com.ibm.ccd.ui.dataobjects.items.ICatalogItem;
import com.ibm.ccd.ui.dataobjects.view.CatalogViewMaster;
import com.ibm.ccd.ui.dataobjects.view.ViewJSPBuilder;
import com.ibm.ccd.ui.dataobjects.workflow.CollaborationArea;
import com.ibm.ccd.ui.dataobjects.workflow.Steps;
import com.ibm.ccd.ui.dataobjects.workflow.Workflow;
import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.mediator.WPCMediator;
import com.ibm.ccd.ui.mediator.WPCMediatorException;
import com.ibm.ccd.ui.portlets.BasePortlet;
import com.ibm.ccd.ui.portlets.GenericPortletMessage;
import com.ibm.ccd.ui.portlets.catalog.ICatalogAction;
import com.ibm.ccd.ui.portlets.workflow.IWorkFlow;
import com.ibm.ccd.ui.tags.element.IHTMLRenderer;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;
import com.ibm.ccd.ui.validator.IValidatorMessage;


/**
 * WorkwithItem Portlet indent to create, modify, clone and delete an
 *
 * @author Senthil
 */
public class WorkwithItemPortlet extends BasePortlet implements ActionListener,
    MessageListener, ICatalogAction {
    /**
     * @see com.ibm.ccd.ui.portlets.BasePortlet#init(org.apache.jetspeed.portlet.PortletConfig)
     */
    public void init(PortletConfig portletConfig) throws UnavailableException {
        super.init(portletConfig);
    }

    /**
     * @see org.apache.jetspeed.portlet.PortletAdapter#doView(org.apache.jetspeed.portlet.PortletRequest, org.apache.jetspeed.portlet.PortletResponse)
     */
    public void doView(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        PortletSession session = request.getPortletSession();
        PortletSettings settings = request.getPortletSettings();
        
        setBusinessFunction(request);

        /*
         * Check for Itemto be displayed from SearchResult.
         * Special case for page navigation
         */

        if (SEARCH.equals(request.getParameter(COMMAND))) {
            displaySingleItemView(request);
        }
        else if(SEARCH_VIEW_MODE.equals(request.getParameter(COMMAND))){
			displaySingleItemView(request);
			request.getPortletSession().setAttribute(COMMAND,ICatalogAction.SEARCH_VIEW_MODE);
        }
         else if (WORKFLOW.equals(request.getParameter(COMMAND))) {
            displaySingleItemViewForWorkFlow(request);
        }

        String jsp2Render = null;

        if (session.getAttribute(JSP2RENDER) != null) {
            jsp2Render = (String) session.getAttribute(JSP2RENDER);
        }

        String command = null;

        if (session.getAttribute(COMMAND) != null) {
           command = (String) request.getPortletSession().getAttribute(COMMAND);
        }

        if (jsp2Render == null) {
            jsp2Render = MESSAGEPAGE;
            setMessageHeader(request, null);
        } else {
            setMessageHeader(request, command);
        }
		
		if(!MESSAGEPAGE.equals(jsp2Render)) {
			String catalogName = (String)session.getAttribute(CATALOG);
			String viewName = (String) settings.getAttribute(catalogName);
			
			String cmd = (String) session.getAttribute(COMMAND);
			String cmd_prev = (String) session.getAttribute(COMMAND_PREV);
			
			if (WORKFLOW.equals(request.getParameter(COMMAND)) || WORKFLOW.equals(cmd) 
			|| WORKFLOW.equals(cmd_prev)) {
			    CatalogItem item = (CatalogItem)session.getAttribute(ITEM);
			    viewName = ((Steps)item.getCollaborationArea().getSteps().get(0)).getStepName();
			}
			
			jsp2Render = displayCatalogView(request,catalogName,viewName);
			session.setAttribute(CATALOG_VIEW,viewName);			
		}
		
        System.out.println("Command ------> " + command +
            " JSP2Render  ------> " + jsp2Render );

        getPortletConfig().getContext().include(jsp2Render, request, response);

        
    }
	
	/**
	 * @see org.apache.jetspeed.portlet.PortletAdapter#doConfigure(org.apache.jetspeed.portlet.PortletRequest, org.apache.jetspeed.portlet.PortletResponse)
	 */
	public void doConfigure(PortletRequest request, PortletResponse response)
		throws PortletException, IOException {
			String jsp2Render= "/jsps/html/itemcatalog/ItemViewConfig.jsp";
			
			PortletSession session = request.getPortletSession();
			PortletSettings settings = request.getPortletSettings();
			
			String command = (String) session.getAttribute(COMMAND);
			String command_pre = (String) session.getAttribute(COMMAND_PREV);
			
			if(WORKFLOW.equals(command) || WORKFLOW.equals(command_pre)) {
				jsp2Render = MESSAGEPAGE;
				MessageMap map = getMessageMap(request);
				map.clear();
				map.addInfoMessage("MESSAGE_CODE_144");
				map.addInfoMessage("MESSAGE_CODE_145",session.getAttribute(CATALOG_VIEW));
				
				getPortletConfig().getContext().include(jsp2Render, request, response);
				return;
			}
			
			String catalogName = (String) session.getAttribute(CATALOG);

			if(session.getAttribute(CATALOG) != null ) {
				getCatalogViewList(request,catalogName);
				setMessageHeader(request,"MESSAGE_CODE_146");
			}else {
				jsp2Render = MESSAGEPAGE;
				setMessageHeader(request,"MESSAGE_CODE_144");
			}
			
			getPortletConfig().getContext().include(jsp2Render, request, response);						
	}

	public String displayCatalogView(PortletRequest request, String catalogName, String viewName) {
		String filePath = "/jsps/html/itemcatalog/view";
		String jsp2Render = filePath + "/Item.jsp";
		PortletSession session = request.getPortletSession();	
		
		String realPath = getPortletConfig().getContext().getRealPath("/") + filePath;
			
		String includePage = null;
			
		if(viewName == null){
			setMessageHeader(request,CATALOG_CONFIG);
			jsp2Render = MESSAGEPAGE;
		}else {
			String locale = request.getLocale().toString();			
			includePage = ViewJSPBuilder.getFileName(filePath,catalogName,viewName,locale);
				
			if(!ViewJSPBuilder.checkFileExists(realPath,catalogName,viewName,locale) ||
			session.getAttribute(CATALOG_CONFIG_CLEAR_VIEW) != null) {
				//create JSP Builder
			    includePage = getIncludePage(request,realPath);
			    session.removeAttribute(CATALOG_CONFIG_CLEAR_VIEW);
			}
			session.setAttribute("includePage",includePage); 
		}
		
		return jsp2Render;
	}

	public String getIncludePage(PortletRequest request, String realPath) {
		PortletSession session = request.getPortletSession();
		PortletSettings settings = request.getPortletSettings();

		String catalogName = (String)session.getAttribute(CATALOG);
		String viewName = (String) settings.getAttribute(catalogName);
		String includePage = null;

		WPSContext context = getWPSContext(request);
		CatalogViewMaster viewMaster = null;
		
		String cmd = (String)session.getAttribute(COMMAND);

		try {
		    if (WORKFLOW.equals(request.getParameter(COMMAND)) || WORKFLOW.equals(cmd)) {
		        CatalogItem item = (CatalogItem)session.getAttribute(ITEM);
		        catalogName = item.getCollaborationArea().getCollabName();
		        viewName = ((Steps)item.getCollaborationArea().getSteps().get(0)).getStepName();
		        String workFlowName = item.getCollaborationArea().getWorkflowName(); 
		        viewMaster = WPCMediator.getInstance().getWorkFlowView(workFlowName,viewName,context);
		    }else{
		        viewMaster = WPCMediator.getInstance().getCatalogView(catalogName,viewName,context);    
		    }

		} catch (WPCMediatorException e) {
			MessageMap map = null;
			map = getMessageMap(request);
			request.setAttribute(IReference.MESSAGE_KEY, map);

			handleException(e, request);
			session.removeAttribute(JSP2RENDER);
			return null;
		}
		
		CatalogItem item = (CatalogItem)session.getAttribute(ITEM);
		
		String locale = request.getLocale().toString();
		try {
			ViewJSPBuilder.createFile(item,viewMaster,realPath,catalogName,viewName,locale);
		} catch (Exception e1) {
			e1.printStackTrace();
		}
		return ViewJSPBuilder.getFileName("/jsps/html/itemcatalog/view",catalogName,viewName,locale);
	}
	
	/**
	 * 
	 * @param request
	 */
	public void getCatalogViewList(PortletRequest request,String catalogName) {
		PortletSession session = request.getPortletSession();
		PortletSettings settings = request.getPortletSettings();
		
		if(catalogName == null)
		 catalogName = (String) session.getAttribute(CATALOG);

		WPSContext context = getWPSContext(request);
		List viewList = null;
		try {
			viewList =
				WPCMediator.getInstance().getCatalogViewList(
					catalogName,
					context);
		} catch (WPCMediatorException e) {
			e.printStackTrace();
		}
		
		session.setAttribute(CATALOG_VIEW_LIST,viewList);
		findCatalogViewName (request, catalogName);
	}

	public void findCatalogViewName (PortletRequest request, String catalogName) {
		PortletSession session = request.getPortletSession();
		PortletSettings settings = request.getPortletSettings();
		
		String viewName = settings.getAttribute(catalogName);
		if(viewName != null) {
			ArrayList list = (ArrayList)session.getAttribute(CATALOG_VIEW_LIST);
			
			if(list != null && !list.isEmpty() && list.contains(viewName)) {
				session.setAttribute(CATALOG_VIEW_NAME,viewName);	
			} 
		}
	}
	

	/**
	 * @param request
	 * @param jsp2Render
	 * @return
	 */
	private String checkValidJSP(PortletRequest request, String jsp2Render) {
		
		String filePath = getPortletConfig().getContext().getRealPath("/") +
			jsp2Render;
		File jspPath = new File(filePath);
		
		
		if (!jspPath.exists()) {
			String catalogID = (String)request.getPortletSession().getAttribute(CATALOG);			
			// jsp does not exist...
			getMessageMap(request).addErrorMessage("MESSAGE_CODE_142",catalogID);
			jsp2Render = MESSAGEPAGE;
		  }
		return jsp2Render;
	}

    /**
     * @see org.apache.jetspeed.portlet.event.ActionListener#actionPerformed(org.apache.jetspeed.portlet.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent event) throws PortletException {
        PortletRequest request = event.getRequest();
        PortletSession session = request.getPortletSession();

        String actionString = event.getActionString();
        String command = (String) session.getAttribute(COMMAND);

        CatalogItem item = (CatalogItem) session.getAttribute(ITEM);

        if (OK.equals(actionString)) {
            if (CREATE.equals(command)) {
                createItem(request);
            }

            if (UPDATE.equals(command) || WORKFLOW.equals(command)) {
                updateItem(request);
            }

            if (CLONE.equals(command)) {
                cloneItem(request);
            }

            if (DELETE.equals(command)) {
                deleteItem(request);
            }

            return;
        }

        if (CANCEL.equals(actionString)) {
            if (CREATE.equals(command) || VIEW.equals(command) ||
                    CLONE.equals(command) || WORKFLOW.equals(command)) {
                try {
                    getMessageMap(request).clear();
                } catch (Exception e) {
                    e.printStackTrace();
                }

                session.setAttribute(JSP2RENDER, null);
				session.removeAttribute(CATALOG);
                return;
            } else {
                session.setAttribute(JSP2RENDER, item.getUpdateView());
                session.setAttribute(COMMAND, VIEW);
            }

            return;
        }

        if (PRINT.equals(actionString)) {
            session.setAttribute(JSP2RENDER, item.getPrintView());

            return;
        }

        if (RETURN_URL.equals(actionString)) {
            session.setAttribute(JSP2RENDER, item.getUpdateView());

            return;
        }

        /**
         * Action triggered in View mode of Work with Item Portlet
         * */
        if (VIEW.equals(command)) {
            if (CREATE.equals(actionString)) {
                session.setAttribute(COMMAND, actionString);
                displayItemCreateView(request);

                return;
            }

            if (UPDATE.equals(actionString)) {
                session.setAttribute(COMMAND, actionString);

                //displayItemUpdateView(request);
                session.setAttribute(JSP2RENDER, item.getUpdateView());

                return;
            }

            if (CLONE.equals(actionString)) {
                session.setAttribute(COMMAND, actionString);
                displayItemCloneView(request);

                return;
            }

            if (DELETE.equals(actionString)) {
                session.setAttribute(COMMAND, actionString);
                session.setAttribute(JSP2RENDER, item.getUpdateView());

                return;
            }
        }

        WorkwithItemHandler itemHandler = new WorkwithItemHandler();

        if (IHTMLRenderer.ATTRIB_ADD_EVENT.equals(actionString)) {
            updateModifiedItem(request);
            itemHandler.invokeAttributeAdd(request);

            return;
        } else if (IHTMLRenderer.ATTRIB_REMOVE_EVENT.equals(actionString)) {
			updateModifiedItem(request);
            itemHandler.invokeAttributeRemove(request);
            return;
        } 
        
        if(SWITCH_TAB.equals(actionString)) {
        	updateModifiedItem(request);
        		session.setAttribute(ACTIVE_TAB,request.getParameter(ACTIVE_TAB));	
        }else if(SWITCH_TAB_NO_SUBMIT.equals(actionString)) {
			session.setAttribute(ACTIVE_TAB,request.getParameter(ACTIVE_TAB));
        }
        
        if(CATALOG_CONFIG_SUBMIT.equals(actionString)) {
        	String selectedView = request.getParameter(CATALOG_VIEW_LIST);
        	
        	PortletSettings settings = request.getPortletSettings();
        	
        	String catalogName = (String) session.getAttribute(CATALOG);
        	
        	session.setAttribute(CATALOG_CONFIG_CLEAR_VIEW,CATALOG_CONFIG_CLEAR_VIEW);
        	
        	try {
				settings.setAttribute(catalogName,selectedView);
				settings.store();
			} catch (AccessDeniedException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
        	
        	session.setAttribute(CATALOG_VIEW_NAME,selectedView);
        }
        
    }

    /**
     * @see org.apache.jetspeed.portlet.event.MessageListener#messageReceived(org.apache.jetspeed.portlet.event.MessageEvent)
     */
    public void messageReceived(MessageEvent event) throws PortletException {
        WPSContext context = null;
        PortletRequest request = event.getRequest();
        PortletSession session = request.getPortletSession();
        context = getWPSContext(request);
        GenericPortletMessage message = (GenericPortletMessage) event.getMessage();

        // return if the message is not with name WORKWITHITEMPORTLET
        if ((message.getMessage() != null) &&
                !WORKWITHITEMPORTLET.equals(message.getMessage())) {
            return;
        }
		
		session.removeAttribute(COMMAND_PREV);
		
        String command = (String) message.get(COMMAND);
        session.setAttribute(COMMAND, command);

        String catalogID = null;
        String parentID = "";
        String itemID = null;
        CategoryHierarchy hierarchy = null;

        setMessageHeader(request, command);
		
		//remove the activeTab value from session
		session.removeAttribute(ACTIVE_TAB);
		
		
		
        if (CREATE.equals(command)) {
            catalogID = (String) message.get(CATALOG);
            parentID = (String) message.get(PARENTID);
            itemID = (String) message.get(ITEMID);
            hierarchy = (CategoryHierarchy) message.get(CATEGORY_HIERARCHY);

            session.setAttribute(CATALOG, catalogID);
            session.setAttribute(PARENTID, parentID);
            session.setAttribute(ITEMID, itemID);
            session.setAttribute(CATEGORY_HIERARCHY, hierarchy);
            displayItemCreateView(request);

            return;
        }

        if (VIEW.equals(command)) {
            catalogID = (String) message.get(CATALOG);
            parentID = (String) message.get(PARENTID);
            itemID = (String) message.get(ITEMID);
            hierarchy = (CategoryHierarchy) message.get(CATEGORY_HIERARCHY);

            session.setAttribute(CATALOG, catalogID);
            session.setAttribute(PARENTID, parentID);
            session.setAttribute(ITEMID, itemID);
            session.setAttribute(CATEGORY_HIERARCHY, hierarchy);

            displayItemUpdateView(request);

            return;
        }

        if (UPDATE.equals(command)) {
            catalogID = (String) message.get(CATALOG);
            parentID = (String) message.get(PARENTID);
            itemID = (String) message.get(ITEMID);
            hierarchy = (CategoryHierarchy) message.get(CATEGORY_HIERARCHY);

            session.setAttribute(CATALOG, catalogID);
            session.setAttribute(PARENTID, parentID);
            session.setAttribute(ITEMID, itemID);
            session.setAttribute(CATEGORY_HIERARCHY, hierarchy);

            displayItemUpdateView(request);

            return;
        }

        if (CLONE.equals(command)) {
            catalogID = (String) message.get(CATALOG);
            parentID = (String) message.get(PARENTID);
            itemID = (String) message.get(ITEMID);
            hierarchy = (CategoryHierarchy) message.get(CATEGORY_HIERARCHY);

            session.setAttribute(CATALOG, catalogID);
            session.setAttribute(PARENTID, parentID);
            session.setAttribute(ITEMID, itemID);
            session.setAttribute(CATEGORY_HIERARCHY, hierarchy);

            displayItemCloneView(request);

            return;
        }

        if (DELETE.equals(command)) {
            catalogID = (String) message.get(CATALOG);
            parentID = (String) message.get(PARENTID);
            itemID = (String) message.get(ITEMID);
            hierarchy = (CategoryHierarchy) message.get(CATEGORY_HIERARCHY);

            session.setAttribute(CATALOG, catalogID);
            session.setAttribute(PARENTID, parentID);
            session.setAttribute(ITEMID, itemID);
            session.setAttribute(CATEGORY_HIERARCHY, hierarchy);

            displayItemUpdateView(request);

            return;
        }

        if (SEARCH.equals(command)) {
            catalogID = (String) message.get(CATALOG);
            parentID = (String) message.get(PARENTID);

            List itemIDs = (List) message.get(ITEMID);
            ICatalogItem item = null;

            try {
                if ((itemIDs != null) && (itemIDs.size() == 1)) {
                    item = (ICatalogItem) getMediator().getCatalogItem(parentID,
                            catalogID, (String) itemIDs.get(0), context);
                } else {
                    return;
                }
            } catch (WPCMediatorException we) {
                handleException(we, request);

                request.getPortletSession().setAttribute(JSP2RENDER, null);

                return;
            }

            session.setAttribute(JSP2RENDER, item.getUpdateView());
            session.setAttribute(COMMAND, VIEW);
            session.setAttribute(CATALOG, catalogID);
            session.setAttribute(PARENTID, parentID);
            session.setAttribute(ITEMID, itemIDs.get(0));
            session.setAttribute(ITEM, item);

            return;
        }
    }

    public void createItem(PortletRequest request) {
        /** Create Logical Item Node */
        PortletSession session = request.getPortletSession();

        CatalogItem item = (CatalogItem) session.getAttribute(ITEM);

        // set properties and check for the errors
        MessageMap map = null;

        try {
            map = item.updateProperties(request);
        } catch (Exception e) {
            e.printStackTrace();
        }

        if ((map != null) && (map.size() > 0)) {
            // validation error occured route to the CreateView JSP again .
            request.setAttribute(IReference.MESSAGE_KEY, map);
            session.setAttribute(JSP2RENDER, item.getCreateFailureView());

            return;
        }

        //Remove the error message attributes from portlet context
        getPortletConfig().getContext().removeAttribute(IValidatorMessage.ATTRIBUTE_LIST);
        getPortletConfig().getContext().removeAttribute(IValidatorMessage.ERROR_MESSAGE_LIST);

        // create the item by invoking the mediator
        String itemPKID = null;

        try {
            WPSContext context = getWPSContext(request);
            itemPKID = getMediator().createItem(item, context);
            item.setItemID(itemPKID);

            CatalogItem catalogItem = (CatalogItem) item;

            GenericPortletMessage createMessage = new GenericPortletMessage(ITEMCATALOGPORTLET);
            createMessage.put(ITEMID, itemPKID);
            createMessage.put(COMMAND, CREATE);
            sendMessage(ITEMCATALOGPORTLET, createMessage);

            session.setAttribute(JSP2RENDER, item.getUpdateView());
            session.setAttribute(COMMAND, VIEW);
        } catch (WPCMediatorException me) {
            try {
                map = getMessageMap(request);
                request.setAttribute(IReference.MESSAGE_KEY, map);
            } catch (Exception e) {
                e.printStackTrace();
            }

            handleException(me, request);

            // set the error JSP and then return
            if (me.getType() == WPCMediatorException.SYSTEM_ERROR) {
                session.removeAttribute(JSP2RENDER);
            } else {
                session.setAttribute(JSP2RENDER, item.getCreateFailureView());
            }

            return;
        }
    }

    public void updateItem(PortletRequest request) {
        PortletSession session = request.getPortletSession();

        // retrieve the catalogItem from the session
        CatalogItem item = (CatalogItem) session.getAttribute(ITEM);

        // handle the error when the item is not found in the session
        if (item == null) {
            if (getPortletLog().isErrorEnabled()) {
                getPortletLog().error("Not able to find the catalog item");
            }

            // set the error JSP and then return
            request.setAttribute(JSP2RENDER, MESSAGEPAGE);

            return;
        }

        // set properties and check for the errors
        MessageMap map = null;

        try {
            map = item.updateProperties(request);
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }

        if ((map != null) && (map.size() > 0)) {
            // validation error occured route to the CreateView JSP again .
            request.setAttribute(IReference.MESSAGE_KEY, map);
            request.setAttribute(JSP2RENDER, item.getUpdateFailureView());

            return;
        }

        // create the item by invoking the mediator
        try {
            WPSContext context = getWPSContext(request);
            getMediator().updateItem(item, context);
        } catch (WPCMediatorException me) {
            try {
                map = getMessageMap(request);
                request.setAttribute(IReference.MESSAGE_KEY, map);
            } catch (Exception e) {
                e.printStackTrace();
            }

            handleException(me, request);

            // set the error JSP and then return
            if (me.getType() == WPCMediatorException.SYSTEM_ERROR) {
                session.removeAttribute(JSP2RENDER);
            } else {
                session.setAttribute(JSP2RENDER, item.getCreateFailureView());
            }

            if (getPortletLog().isErrorEnabled()) {
                getPortletLog().error("Not able to update the catalog item");
            }

            return;
        }

        session.setAttribute(ITEM, item);
        session.setAttribute(JSP2RENDER, item.getUpdateSuccessView());
        session.setAttribute(COMMAND, VIEW);
    }

    public void cloneItem(PortletRequest request) {
        PortletSession session = request.getPortletSession();
        ICatalogItem item = (ICatalogItem) session.getAttribute(ITEM);

        MessageMap map = null;

        try {
            map = item.updateProperties(request);
        } catch (Exception e) {
            e.printStackTrace();
        }

        if ((map != null) && (map.size() > 0)) {
            // validation error occured route to the CreateView JSP again .
            request.setAttribute(IReference.MESSAGE_KEY, map);
            session.setAttribute(JSP2RENDER, item.getCreateFailureView());

            return;
        }

        String itemPKID = null;

        try {
            WPSContext context = getWPSContext(request);
            itemPKID = getMediator().cloneItem(item, context);
            item.setItemID(itemPKID);
        } catch (WPCMediatorException me) {
            try {
                map = getMessageMap(request);
                request.setAttribute(IReference.MESSAGE_KEY, map);
            } catch (Exception e) {
                e.printStackTrace();
            }

            handleException(me, request);

            // set the error JSP and then return
            if (me.getType() == WPCMediatorException.SYSTEM_ERROR) {
                session.removeAttribute(JSP2RENDER);
            } else {
                session.setAttribute(JSP2RENDER, item.getCreateFailureView());
            }

            return;
        } 

        session.setAttribute(JSP2RENDER, item.getUpdateView());
        session.setAttribute(COMMAND, VIEW);

        GenericPortletMessage cloneMessage = new GenericPortletMessage(ITEMCATALOGPORTLET);
        cloneMessage.put(ITEMID, item.getItemID());
        cloneMessage.put(COMMAND, CLONE);
        sendMessage(ITEMCATALOGPORTLET, cloneMessage);
    }

    public void deleteItem(PortletRequest request) {
        PortletSession session = request.getPortletSession();
        ICatalogItem item = (ICatalogItem) session.getAttribute(ITEM);
        MessageMap messageMap = null;

        // delete the item by invoking the mediator
        try {
            WPSContext context = getWPSContext(request);
            getMediator().deleteItem(item, context);
        } catch (WPCMediatorException me) {
            try {
                messageMap = getMessageMap(request);
                request.setAttribute(IReference.MESSAGE_KEY, messageMap);
            } catch (Exception e) {
                e.printStackTrace();
            }

            handleException(me, request);

            // set the error JSP and then return
            if (me.getType() == WPCMediatorException.SYSTEM_ERROR) {
                session.removeAttribute(JSP2RENDER);
            } else {
                session.setAttribute(JSP2RENDER, item.getCreateFailureView());
            }

            if (getPortletLog().isErrorEnabled()) {
                getPortletLog().error("Not able to Delete item from catalog ");
            }
        }

        try {
            messageMap = getMessageMap(request);
            messageMap.clear();
            messageMap.addInfoMessage("MESSAGE_CODE_104");
            session.setAttribute(JSP2RENDER, null);

            GenericPortletMessage deleteMessage = new GenericPortletMessage(ITEMCATALOGPORTLET);
            deleteMessage.put(ITEMID, item.getItemID());
            deleteMessage.put(COMMAND, DELETE);
            sendMessage(ITEMCATALOGPORTLET, deleteMessage);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void updateModifiedItem(PortletRequest request) {
        PortletSession session = request.getPortletSession();

        // retrieve the catalogItem from the session
        CatalogItem item = (CatalogItem) session.getAttribute(ITEM);

        // handle the error when the item is not found in the session
        if (item == null) {
            if (getPortletLog().isErrorEnabled()) {
                getPortletLog().error("Not able to find the catalog item");
            }

            // set the error JSP and then return
            request.setAttribute(JSP2RENDER, MESSAGEPAGE);

            return;
        }

        // set properties and check for the errors
        MessageMap map = null;

        try {
            map = item.updateProperties(request);
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }

        if ((map != null) && (map.size() > 0)) {
            // validation error occured route to the CreateView JSP again .
            request.setAttribute(IReference.MESSAGE_KEY, map);
            request.setAttribute(JSP2RENDER, item.getUpdateFailureView());

            return;
        }
    }
	
    public void displayItemCreateView(PortletRequest request) {
        ICatalogItem item = null;
        PortletSession session = request.getPortletSession();

        String catalogID = (String) session.getAttribute(CATALOG);
        String parentID = (String) session.getAttribute(PARENTID);

        try {
            WPSContext context = getWPSContext(request);
            item = getMediator().getCatalogItem(parentID, catalogID, context);

            //Reset ParentCategory because Item Spec is cached for a particular parentID 			
            item.setParentCategoryID(parentID);
        } catch (WPCMediatorException me) {
            if (getPortletLog().isErrorEnabled()) {
                getPortletLog().error("Not able to find the catalog item");
            }

            // set the error JSP and then return
            handleException(me, request);
            session.setAttribute(JSP2RENDER, null);

            return;
        }

        session.setAttribute(ITEM, item);
        session.setAttribute(JSP2RENDER, item.getCreateView());
    }

    public void displayItemUpdateView(PortletRequest request) {
        ICatalogItem item = null;
        PortletSession session = request.getPortletSession();

        String catalogID = (String) session.getAttribute(CATALOG);
        String parentID = (String) session.getAttribute(PARENTID);
        log.debug(" catalogID->" + catalogID);
        log.debug(" parentID->" + parentID);
        log.debug(" ITEM->" + session.getAttribute(ITEM));
        log.debug(" ITEMID->" + session.getAttribute(ITEMID));

        try {
            WPSContext context = getWPSContext(request);

            if (session.getAttribute(ITEMID) != null) {
                String itemID = (String) session.getAttribute(ITEMID);
                item = getMediator().getCatalogItem(parentID, catalogID,
                        itemID, context);
            } else if (session.getAttribute(ITEM) != null) {
                item = (ICatalogItem) session.getAttribute(ITEM);
                item = getMediator().getCatalogItem(parentID, catalogID,
                        item.getItemID(), context);
            }
            /*
             * Code for image loading is removed from here.Now images a
             * binary attachments are loaded lazily on d
             */ 
        } catch (WPCMediatorException me) {
            if (getPortletLog().isErrorEnabled()) {
                getPortletLog().error("Not able to find the catalog item");
            }

            handleException(me, request);
            session.setAttribute(JSP2RENDER, null);

            return;
        }

        log.debug(" displayItemUpdateView  item.getUpdateView()->" +
            item.getUpdateView());
        session.setAttribute(ITEM, item);

        // set the error JSP and then return
        session.setAttribute(JSP2RENDER, item.getUpdateView());
    }

   public void displayItemCloneView(PortletRequest request) {
        PortletSession session = request.getPortletSession();

        ICatalogItem item = null;
        ICatalogItem clonedItem = null;
        WPSContext context = getWPSContext(request);

        String catalogID = (String) session.getAttribute(CATALOG);
        String parentID = (String) session.getAttribute(PARENTID);

        try {
            if (session.getAttribute(ITEMID) != null) {
                String itemID = (String) session.getAttribute(ITEMID);
                clonedItem = getMediator().getCatalogItemClone(parentID,
                        catalogID, itemID, context);
            } else if (session.getAttribute(ITEM) != null) {
                item = (ICatalogItem) session.getAttribute(ITEM);
                clonedItem = getMediator().getCatalogItemClone(parentID,
                        catalogID, item.getItemID(), context);
            }
        } catch (WPCMediatorException we) {
            handleException(we, request);
            session.setAttribute(JSP2RENDER, null);

            return;
        }

        session.setAttribute(ITEM, clonedItem);
        session.setAttribute(JSP2RENDER, clonedItem.getUpdateView());
    }

    public void displaySingleItemView(PortletRequest request) {
        String catalogID = request.getParameter(CATALOG);
        String parentID = request.getParameter(PARENTID);

        if (parentID == null) {
            parentID = "";
        }

        // Commented pls remove this 
        String itemID = request.getParameter(ITEMID);
        log.debug(" catalogID->" + catalogID);
        log.debug(" parentID->" + parentID);
        log.debug(" itemID->" + itemID);

        if (!((itemID != null) && (itemID.length() > 0) &&
                (itemID.indexOf("-") == -1))) {
            return;
        }

        ICatalogItem item = null;
        PortletSession session = request.getPortletSession();
        WPSContext context = getWPSContext(request);

        try {
            if ((itemID != null) && (itemID.length() > 0)) {
                item = (ICatalogItem) getMediator().getCatalogItem(parentID,
                        catalogID, itemID, context);
            } else {
                return;
            }
        } catch (WPCMediatorException we) {
            handleException(we, request);
            request.getPortletSession().setAttribute(JSP2RENDER, null);

            return;
        }
        /*
         * Code for image loading is removed from here.Now images a
         * binary attachments are loaded lazily on d
         */ 
        session.setAttribute(JSP2RENDER, item.getUpdateView());
        session.setAttribute(COMMAND, VIEW);
        session.setAttribute(CATALOG, catalogID);
        session.setAttribute(PARENTID, parentID);
        session.setAttribute(ITEMID, itemID);
        session.setAttribute(ITEM, item);

        return;
    }

    public void displaySingleItemViewForWorkFlow(PortletRequest request) {
        String catalogID = request.getParameter(CATALOG);
        String parentID = request.getParameter(PARENTID);

        if (parentID == null) {
            parentID = "";
        }

        // Commented pls remove this 
        String itemID = request.getParameter(ITEMID);

        String collabName = catalogID;
        String workflowName = request.getParameter(WORKFLOWNAME);
        String stepName = request.getParameter(STEPNAME);
        String catAssociated = request.getParameter(IWorkFlow.REQ_PARAM_KEY_CAT_ASSOCIATED);

        Steps step = new Steps(stepName, 1);
        Workflow workFlow = new Workflow();
        ArrayList stepList = new ArrayList();
        stepList.add(step);
        workFlow.setSteps(stepList);
        workFlow.setWorkflowName(workflowName);

        CollaborationArea collabArea = new CollaborationArea(collabName);
        ArrayList workflowList = new ArrayList();
        workflowList.add(workFlow);
        collabArea.setWorkflows(workflowList);

        log.debug(" catalogID->" + catalogID);
        log.debug(" parentID->" + parentID);
        log.debug(" itemID->" + itemID);
		
        if (!((itemID != null) && (itemID.length() > 0) &&
                (itemID.indexOf("-") == -1))) {
            return;
        }

        ICatalogItem item = null;
        PortletSession session = request.getPortletSession();
        WPSContext context = getWPSContext(request);

        try {
            if ((itemID != null) && (itemID.length() > 0)) {
                
                item = (ICatalogItem) getMediator().getCatalogItem(parentID,
                        catalogID, itemID, context);

                // set CollaborationArea when the item navigates from WorkFlow
                item.setCollaborationArea(collabArea);
                item.setAssociateWFCatalogID(catAssociated);
                item.setBworkflowItem(true);
                
            } else {
                return;
            }

        } catch (WPCMediatorException we) {
            handleException(we, request);
            request.getPortletSession().setAttribute(JSP2RENDER, null);

            return;
        }
        
		String jsp2Render = item.getUpdateView();
        session.setAttribute(COMMAND, WORKFLOW);
        session.setAttribute(COMMAND_PREV,WORKFLOW);
        session.setAttribute(CATALOG, catalogID);
        session.setAttribute(JSP2RENDER, jsp2Render);
        session.setAttribute(PARENTID, parentID);
        session.setAttribute(ITEMID, itemID);
        session.setAttribute(ITEM, item);

        return;
    }

    private WPCMediator getMediator() {
        WPCMediator mediator = null;

        if (mediator == null) {
            mediator = WPCMediator.getInstance();
        }

        return mediator;
    }

    /**
     * @see com.ibm.ccd.ui.portlets.BasePortlet#setBusinessFunction(org.apache.jetspeed.portlet.PortletRequest)
     */
    protected void setBusinessFunction(PortletRequest request) {
        PortletSession session = request.getPortletSession();

        session.setAttribute(IReference.SESSION_BUS_FUNCT, "WORK_WITH_ITEM");
        session.setAttribute(IReference.SESSION_ACTION, "VIEW");
    }
    
    public static String getStringValue(String itemID,Attribute attribute, Locale locale, int index) {
		
    	switch(attribute.getType()){
		case AttributeTypes.NUMBER:
		case AttributeTypes.CURRENCY:
		{
			NumberAttribute numberAttrib = (NumberAttribute) attribute;

		if (numberAttrib.getCurrentOccurrence() > 0) {
			if (numberAttrib.isMultiValued()) {
				String[] values = numberAttrib.getValues();
				index = (index < values.length) ? index : 0;
				if(attribute.getValidationError(itemID)!=null)
					return values[index];
				else
					return UIUtils.formatNumber(values[index],locale);
			} else {
				if(attribute.getValidationError(itemID)!=null)
					return numberAttrib.getValue();
				else
					return UIUtils.formatNumber(String.valueOf(numberAttrib.getValue()),locale);	
			}
		} else {
			return UIUtils.formatNumber(String.valueOf(numberAttrib.getDefaultValue()),locale);
		}
		
		}
		case AttributeTypes.DATE:
		{
			DateAttribute dateAttrib = (DateAttribute) attribute;

        if (dateAttrib.getCurrentOccurrence() > 0) {
            if (dateAttrib.isMultiValued()) {
                String[] values = dateAttrib.getValues();
                if(attribute.getValidationError(itemID)!=null)
                	return (values[index] != null) ? values[index] : "";
                else
                	return (values[index] != null) ? UIUtils.formatWPCDate(values[index],locale) : "";
            } else {
            	if(attribute.getValidationError(itemID)!=null)
            		return (dateAttrib.getValue() != null) ? dateAttrib.getValue() : "";
            	else
            		return (dateAttrib.getValue() != null) ? UIUtils.formatWPCDate(dateAttrib.getValue(),locale) : "";
            }
        } else {
            return (dateAttrib.getDefaultValue() != null)
            ? dateAttrib.getDefaultValue() : "";
        }
        
		}
        default:
        	return "";
			
		}//end of switch
	}
    
    public static String getStringValue(Attribute attribute, int index) {
        switch (attribute.getType()) {
        case AttributeTypes.STRING:

            StringAttribute stringAttrib = (StringAttribute) attribute;

            if (stringAttrib.getCurrentOccurrence() > 0) {
                if (stringAttrib.isMultiValued()) {
                    String[] values = stringAttrib.getValues();

                    if ((values != null) || (values.length > 0)) {
                        return values[index];
                    } else {
                        return "";
                    }
                } else {
                    return (stringAttrib.getValue() != null)
                    ? stringAttrib.getValue() : "";
                }
            } else {
                return (stringAttrib.getDefaultValue() != null)
                ? stringAttrib.getDefaultValue() : "";
            }

        case AttributeTypes.BINARY:

            BinaryAttribute binaryAttrib = (BinaryAttribute) attribute;

            if (binaryAttrib.getCurrentOccurrence() > 0) {
                binaryAttrib.printMe();

                if (binaryAttrib.isMultiValued()) {
                    FileUploadResource[] file = binaryAttrib.getValues();

                    if ((file != null) && (file.length > 0)) {
                        index = (index < file.length) ? index : 0;

                        return (file[index].getName() != null)
                        ? file[index].getName() : "";
                    } else {
                        return "";
                    }
                } else {
                    return (binaryAttrib.getValue().getName() != null)
                    ? binaryAttrib.getValue().getName() : "";
                }
            } else {
                //return binaryAttrib.getDefaultValue().getName();
                return "";
            }

        case AttributeTypes.CURRENCY:

            CurrencyAttribute currencyAttrib = (CurrencyAttribute) attribute;

            if (currencyAttrib.getCurrentOccurrence() > 0) {
                if (currencyAttrib.isMultiValued()) {
                    String[] values = currencyAttrib.getValues();
                    index = (index < values.length) ? index : 0;

                    return (values[index] != null) ? values[index] : "";
                } else {
                    return (currencyAttrib.getValue() != null)
                    ? currencyAttrib.getValue() : "";
                }
            } else {
                return (currencyAttrib.getDefaultValue() != null)
                ? currencyAttrib.getDefaultValue() : "";
            }

        case AttributeTypes.INTEGER:

            IntegerAttribute integerAttrib = (IntegerAttribute) attribute;

            if (integerAttrib.getCurrentOccurrence() > 0) {
                if (integerAttrib.isMultiValued()) {
                    String [] values = integerAttrib.getValues();
                    index = (index < values.length) ? index : 0;

                    return values[index] + "";
                } else {
                    return integerAttrib.getValue() + "";
                }
            } else {
                return integerAttrib.getDefaultValue() + "";
            }

        case AttributeTypes.NUMBER:

            NumberAttribute numberAttrib = (NumberAttribute) attribute;

            if (numberAttrib.getCurrentOccurrence() > 0) {
                if (numberAttrib.isMultiValued()) {
                    String[] values = numberAttrib.getValues();
                    index = (index < values.length) ? index : 0;

                    return values[index] + "";
                } else {
                    return numberAttrib.getValue() + "";
                }
            } else {
                return "";
            }

        case AttributeTypes.DATE:

            DateAttribute dateAttrib = (DateAttribute) attribute;

            if (dateAttrib.getCurrentOccurrence() > 0) {
                if (dateAttrib.isMultiValued()) {
                    String[] values = dateAttrib.getValues();

                    return (values[index] != null) ? values[index] : "";
                } else {
                    return (dateAttrib.getValue() != null)
                    ? dateAttrib.getValue() : "";
                }
            } else {
                return (dateAttrib.getDefaultValue() != null)
                ? dateAttrib.getDefaultValue() : "";
            }

        case AttributeTypes.FLAG:

            FlagAttribute flagAttrib = (FlagAttribute) attribute;

            if (flagAttrib.getCurrentOccurrence() > 0) {
                String[] fvalues = flagAttrib.getValues();

                if (flagAttrib.isMultiValued()) {
                    return (fvalues[index].equalsIgnoreCase("on")) ? "true"
                                                                   : fvalues[index];
                } else {
                    if (fvalues[0] == null) {
                        return "false";
                    } else {
                        return (fvalues[0].equalsIgnoreCase("on")) ? "true"
                                                                   : fvalues[index];
                    }
                }
            } else {
                if (flagAttrib.getDefaultValue() != null) {
                    return (flagAttrib.getDefaultValue().equalsIgnoreCase("on"))
                    ? "true" : flagAttrib.getDefaultValue();
                } else {
                    return "false";
                }
            }

        case AttributeTypes.TIMEZONE:

            TimeZoneAttribute tzoneAttrib = (TimeZoneAttribute) attribute;

            if (tzoneAttrib.getCurrentOccurrence() > 0) {
                if (tzoneAttrib.isMultiValued()) {
                    String[] values = tzoneAttrib.getValues();

                    return (values[index] != null) ? values[index] : "";
                } else {
                    return (tzoneAttrib.getValue() != null)
                    ? tzoneAttrib.getValue() : "";
                }
            } else {
                return (tzoneAttrib.getDefaultValue() != null)
                ? tzoneAttrib.getDefaultValue() : "";
            }

        case AttributeTypes.PASSWORD:

            PasswordAttribute pwdAttrib = (PasswordAttribute) attribute;

            if (pwdAttrib.getCurrentOccurrence() > 0) {
                if (pwdAttrib.isMultiValued()) {
                    String[] values = pwdAttrib.getValues();

                    return (values[index] != null) ? values[index] : "";
                } else {
                    return (pwdAttrib.getValue() != null)
                    ? pwdAttrib.getValue() : "";
                }
            } else {
                return (pwdAttrib.getDefaultValue() != null)
                ? pwdAttrib.getDefaultValue() : "";
            }

        case AttributeTypes.SEQUENCE:
            return "";

        case AttributeTypes.RELATIONSHIP:

            RelationshipAttribute relationAttrib = (RelationshipAttribute) attribute;

            if (relationAttrib.getCurrentOccurrence() > 0) {
                NVPair[] rvalues = null;

                if (relationAttrib.isMultiValued()) {
                    rvalues = relationAttrib.getValues();

                    if (rvalues[index] != null) {
                        return ((rvalues[index].getName() != null)
                        ? rvalues[index].getName() : "") + "|" +
                        ((rvalues[index].getValue() != null)
                        ? rvalues[index].getValue() : "");
                    }

                    //return rvalues[index].getName()+"|"+(String)rvalues[index].getValue();
                } else {
                    NVPair rvalue = null;
                    rvalue = relationAttrib.getValue();

                    return ((rvalue.getName() != null) ? rvalue.getName() : "") +
                    "|" +
                    ((rvalue.getValue() != null) ? rvalue.getValue() : "");

                    //return rvalue.getName()+"|"+(String)rvalue.getValue();
                }
            } else {
                return "";
            }

        case AttributeTypes.IMAGE:

            ImageAttribute imageAttrib = (ImageAttribute) attribute;

            if (imageAttrib.getCurrentOccurrence() > 0) {
                if (imageAttrib.isMultiValued()) {
                    FileUploadResource[] files = imageAttrib.getValues();

                    return (files[index].getName() != null)
                    ? files[index].getName() : "";
                } else {
                    FileUploadResource file = imageAttrib.getValue();

                    return (file.getName() != null) ? file.getName() : "";
                }
            } else {
                return "";
            }

        case AttributeTypes.THUMB_NAIL_IMAGE:

            ThumbnailImageAttribute thumbImgAttrib = (ThumbnailImageAttribute) attribute;

            if (thumbImgAttrib.getCurrentOccurrence() > 0) {
                if (thumbImgAttrib.isMultiValued()) {
                    FileUploadResource[] files = thumbImgAttrib.getValues();

                    return (files[index].getName() != null)
                    ? files[index].getName() : "";
                } else {
                    return (thumbImgAttrib.getValue().getName() != null)
                    ? thumbImgAttrib.getValue().getName() : "";
                }
            } else {
                //return thumbImgAttrib.getDefaultValue().getName();
                return "";
            }

        case AttributeTypes.URL:

            URLAttribute urlAttrib = (URLAttribute) attribute;

            if (urlAttrib.getCurrentOccurrence() > 0) {
                if (urlAttrib.isMultiValued()) {
                    String[] url = urlAttrib.getValues();

                    return (url[index] != null) ? url[index].toString() : "";
                } else {
                    return (urlAttrib.getValue() != null)
                    ? urlAttrib.getValue().toString() : "";
                }
            } else {
                return "";
            }

        case AttributeTypes.IMAGE_URL:

            ImageURLAttribute imgURLAttrib = (ImageURLAttribute) attribute;

            if (imgURLAttrib.getCurrentOccurrence() > 0) {
                if (imgURLAttrib.isMultiValued()) {
                    String[] urls = imgURLAttrib.getValues();

                    return (urls[index] != null) ? urls[index].toString() : "";
                } else {
                    return (imgURLAttrib.getValue() != null)
                    ? imgURLAttrib.getValue().toString() : "";
                }
            } else {
                return "";
            }

        case AttributeTypes.THUMB_NAIL_IMAGE_URL:

            ThumbnailImageURLAttribute thumbImgURLAttrib = (ThumbnailImageURLAttribute) attribute;

            if (thumbImgURLAttrib.getCurrentOccurrence() > 0) {
                if (thumbImgURLAttrib.isMultiValued()) {
                    String[] urls = thumbImgURLAttrib.getValues();

                    return (urls[index] != null) ? urls[index].toString() : "";
                } else {
                    return (thumbImgURLAttrib.getValue() != null)
                    ? thumbImgURLAttrib.getValue().toString() : "";
                }
            } else {
                return "";
            }

        case AttributeTypes.STRING_ENUMERATION:

            StringEnumAttribute stringEnumAttrib = (StringEnumAttribute) attribute;

            if (stringEnumAttrib.getCurrentOccurrence() > 0) {
                if (stringEnumAttrib.isMultiValued()) {
                    String[] values = stringEnumAttrib.getValues();

                    return (values[index] != null) ? values[index] : "";
                } else {
                    return (stringEnumAttrib.getValue() != null)
                    ? stringEnumAttrib.getValue() : "";
                }
            } else {
                return (stringEnumAttrib.getDefaultValue() != null)
                ? stringEnumAttrib.getDefaultValue() : "";
            }

        case AttributeTypes.NUMBER_ENUMERATION:

            NumberEnumAttribute numberEnumAttrib = (NumberEnumAttribute) attribute;

            if (numberEnumAttrib.getCurrentOccurrence() > 0) {
                if (numberEnumAttrib.isMultiValued()) {
                    String[] values = numberEnumAttrib.getValues();

                    return values[index] + "";
                } else {
                    return numberEnumAttrib.getValue() + "";
                }
            } else {
                return numberEnumAttrib.getDefaultValue() + "";
            }

        case AttributeTypes.LOOKUP_TABLE:

            LookupTableAttribute lookupAttrib = (LookupTableAttribute) attribute;

            if (lookupAttrib.getCurrentOccurrence() > 0) {
                if (lookupAttrib.isMultiValued()) {
                    String[] values = lookupAttrib.getValues();

                    return values[index];
                } else {
                    return lookupAttrib.getValue();
                }
            } else {
                return lookupAttrib.getDefaultValue();
            }

        case AttributeTypes.UNSUPPORTED:}

        return null;
    }

    public static HashMap getCollection(Attribute attribute) {
        HashMap map = new HashMap();

        switch (attribute.getType()) {
        case AttributeTypes.STRING_ENUMERATION:

            StringEnumAttribute stringEnum = (StringEnumAttribute) attribute;
            String[] values = stringEnum.getEnumValues();

            for (int i = 0; i < values.length; i++) {
                map.put(values[i], values[i]);
            }

            return map;

        case AttributeTypes.NUMBER_ENUMERATION:

            NumberEnumAttribute numberEnum = (NumberEnumAttribute) attribute;
            double[] dvalues = numberEnum.getEnumDb();

            for (int i = 0; i < dvalues.length; i++) {
                map.put(dvalues[i] + "", dvalues[i] + "");
            }

            return map;

        case AttributeTypes.LOOKUP_TABLE:

            LookupTableAttribute lookup = (LookupTableAttribute) attribute;
            NVPair[] pair = lookup.getLookupTable();

            for (int i = 0; i < pair.length; i++) {
                map.put(pair[i].getName(), pair[i].getValue());
            }

            return map;

        case AttributeTypes.RELATIONSHIP:

            RelationshipAttribute relationAttrib = (RelationshipAttribute) attribute;
            String[] rvalues = relationAttrib.getRelations();

            for (int i = 0; i < rvalues.length; i++) {
                map.put(rvalues[i], rvalues[i]);
            }

            return map;

        case AttributeTypes.TIMEZONE:

            NVPair[] tzPair = TimeZoneAttribute.getTimeZoneEnum();

            for (int i = 0; i < tzPair.length; i++) {
                map.put(tzPair[i].getName(), tzPair[i].getValue());
            }

            return map;
        }

        return null;
    }

    public static String getSelectedValue(Attribute attribute, String key) {
        switch (attribute.getType()) {
        case AttributeTypes.TIMEZONE:

            TimeZoneAttribute tzoneAttrib = (TimeZoneAttribute) attribute;
            NVPair[] nvPair = TimeZoneAttribute.getTimeZoneEnum();

            for (int i = 0; i < nvPair.length; i++) {
                if (nvPair[i].getName().equals(key)) {
                    return (String) nvPair[i].getValue();
                }
            }

        case AttributeTypes.LOOKUP_TABLE:

            LookupTableAttribute lookupAttrib = (LookupTableAttribute) attribute;
            NVPair[] lookupPair = lookupAttrib.getLookupTable();

            for (int i = 0; i < lookupPair.length; i++) {
                if (lookupPair[i].getName().equals(key)) {
                    return (String) lookupPair[i].getValue();
                }
            }
        }

        return "";
    }

    public static boolean isMandatory(Attribute attribute) {
        if (attribute != null) {
            if (attribute.getMinimumOccurrence() != 0) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    protected void setMessageHeader(PortletRequest request, String command) {
        MessageMap map = null;

        try {
            map = getMessageMap(request);
        } catch (Exception e) {
            e.printStackTrace();

            return;
        }

        if (command != null) {
            if (CREATE.equals(command)) {
                map.addInfoMessage("MESSAGE_CODE_132");

                return;
            }

            if (VIEW.equals(command)) {
                map.addInfoMessage("MESSAGE_CODE_133");

                return;
            }

            if (UPDATE.equals(command) || WORKFLOW.equals(command)) {
                map.addInfoMessage("MESSAGE_CODE_134");

                return;
            }

            if (CLONE.equals(command)) {
                map.addInfoMessage("MESSAGE_CODE_135");

                return;
            }

            if (DELETE.equals(command)) {
                map.addInfoMessage("MESSAGE_CODE_136");

                return;
            }
            
			if(CATALOG_CONFIG.equals(command)) {
				map.clear();
				map.addInfoMessage("MESSAGE_CODE_143");
				return;
			}
			
			map.clear();
			map.addInfoMessage(command);
            
        } else {
            map.addInfoMessage("MESSAGE_CODE_137");
        }
    }

    public void sendMessage(String portlet, GenericPortletMessage message) {
        try {
            getPortletConfig().getContext().send(null, message);
        } catch (AccessDeniedException e) {
            e.printStackTrace();
        }
    }
}
