
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.portlets.lookuptablesearch;

import java.io.IOException;

import org.apache.jetspeed.portlet.Portlet;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletWindow;

import com.ibm.ccd.ui.portlets.lookuptablesearch.cmd.LookupTableGetFirstResultPageCmd;
import com.ibm.ccd.ui.portlets.lookuptablesearch.cmd.LookupTableGetLastResultPageCmd;
import com.ibm.ccd.ui.portlets.lookuptablesearch.cmd.LookupTableGetNextResultPageCmd;
import com.ibm.ccd.ui.portlets.lookuptablesearch.cmd.LookupTableGetPrvResultPageCmd;
import com.ibm.ccd.ui.portlets.lookuptablesearch.cmd.LookupTableSearchFromHistoryCmd;
import com.ibm.ccd.ui.portlets.lookuptablesearch.cmd.LookupTableUpdateSearchCriteriaCmd;
import com.ibm.ccd.ui.portlets.lookuptablesearch.cmd.PerformLookupTableSearchCmd;
import com.ibm.ccd.ui.portlets.lookuptablesearch.cmd.PopulateLookupTableViewDataStoreCmd;
import com.ibm.ccd.ui.portlets.search.core.BaseSearchPortlet;
import com.ibm.ccd.ui.portlets.search.core.IBaseSearch;
import com.ibm.ccd.ui.portlets.search.core.ICollectionViewBean;
import com.ibm.ccd.ui.portlets.search.core.cmd.ISearchCommand;


public class LookupTableSearchPortlet extends BaseSearchPortlet {
    protected void render(PortletRequest request, PortletResponse response,
        Portlet.Mode mode) throws PortletException, IOException {
        String jspName = null;

        if(!((PortletWindow.State.SOLO.toString()).equals(request.getWindow().getWindowState().toString()) )){
			getMessageMap(request).addInfoMessage("MESSAGE_CODE_140");
		} 

        if (Portlet.Mode.VIEW.equals(mode)) {
            jspName = ILookupTableSearch.PATH_SEARCH_VIEW_JSP;
        }

        if (Portlet.Mode.EDIT.equals(mode)) {
            jspName = ILookupTableSearch.PATH_SEARCH_EDIT_JSP;
        }

        if (Portlet.Mode.CONFIGURE.equals(mode)) {
            jspName = ILookupTableSearch.PATH_SEARCH_CONFIG_JSP;
        }

        if (Portlet.Mode.HELP.equals(mode)) {
        }

        includeJSP(jspName, request, response);
    }

    protected ISearchCommand getDataStore(Portlet.Mode mode) {
        ISearchCommand command = null;

        if (Portlet.Mode.VIEW.equals(mode)) {
            command = new PopulateLookupTableViewDataStoreCmd();
        } else {
            command = super.getDataStore(mode);
        }

        return command;
    }

    protected ISearchCommand getAction(String actionString) {
        ISearchCommand command = null;

        if (IBaseSearch.FORM_ACTION_PERFORM_SEARCH.equals(actionString)) {
            command = new PerformLookupTableSearchCmd();
        } 
        else if(IBaseSearch.FORM_ACTION_SAVED_SEARCH.equals(actionString)){
			command = new LookupTableSearchFromHistoryCmd();
        }
        
		else if (IBaseSearch.FORM_ACTION_EDIT_SEARCH_CRITERIA.equals(actionString)) {
			command = new LookupTableUpdateSearchCriteriaCmd();
		}
		
		else if (ICollectionViewBean.GET_NEXT_RESULTS.equals(actionString)) {
			 command = new LookupTableGetNextResultPageCmd();
		 }

		else  if (ICollectionViewBean.GET_FIRST_RESULTS.equals(actionString)) {
				command = new LookupTableGetFirstResultPageCmd();
		}

		else if (ICollectionViewBean.GET_LAST_RESULTS.equals(actionString)) {
			  command = new LookupTableGetLastResultPageCmd();
		  }

		else if (ICollectionViewBean.GET_PREVIOUS_RESULTS.equals(actionString)) {
		   command = new LookupTableGetPrvResultPageCmd();
		}
		
        else {
            command = super.getAction(actionString);
        }

        return command;
    }
}
