/*
 * Licensed Materials: Property of IBM
 * "Restricted Materials of IBM"
 * 5724-I68.
 * (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/*
 * Created on Jan 03, 2006
 *
 */
package com.ibm.ccd.ui.portlets.massupdate;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

import org.w3c.dom.Element;

import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.dataobjects.FileUploadResource;
import com.ibm.ccd.ui.dataobjects.NVPair;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.AttributeTypes;
import com.ibm.ccd.ui.dataobjects.attributes.BinaryAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.CurrencyAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.DateAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.FlagAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ImageAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ImageURLAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.IntegerAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.LookupTableAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberEnumAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.PasswordAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.RelationshipAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.SequenceAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringEnumAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ThumbnailImageAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ThumbnailImageURLAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.TimeZoneAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.URLAttribute;
import com.ibm.ccd.ui.dataobjects.items.ICatalogItem;
import com.ibm.ccd.ui.exception.UIException;
import com.ibm.ccd.ui.mediator.ISoapMessageKeys;
import com.ibm.ccd.ui.mediator.WPCMediator;
import com.ibm.ccd.ui.mediator.WPCMediatorException;
import com.ibm.ccd.ui.portlets.search.core.IBaseSearch;
import com.ibm.ccd.ui.portlets.search.core.SearchBean;
import com.ibm.ccd.ui.portlets.search.core.SearchCriteriaBean;
import com.ibm.ccd.ui.portlets.search.core.SearchInputBean;
import com.ibm.ccd.ui.portlets.search.core.SearchResultBean;
import com.ibm.ccd.ui.taglib.dataobjects.ResultData;
import com.ibm.ccd.ui.taglib.dataobjects.ResultRowData;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;

/**
 *  
 */
public class MassUpdateHelper {

    private String maxItemRows = null;

    private WPSContext wpsContext = null;

    private SearchCriteriaBean searchCriteriaBean = null;

    /**
     *  
     */
    public MassUpdateHelper() {
        super();
    }

    /**
     * Main method getting CatalogItems from WPC.
     * It does search get Items and populate CatalogItems objects and return Map.  
     * @param massUpdateBean
     * @param itemListConv
     * @param catalogID
     * @param context
     * @return
     * @throws WPCMediatorException
     */
    public Map getCatalogItems(MassUpdateBean massUpdateBean,
            List itemListConv, String catalogID, WPSContext context)
            throws WPCMediatorException {
        HashMap catListMap = new HashMap();
        List retList = new ArrayList();
        Set unSucc = new HashSet();

        WPCMediatorException wpcExcep = null;
        ICatalogItem catalogItem = null;

        /*
         * Getting the Item skeleton.
         */
        catalogItem = massUpdateBean.getCatalogItemSkeleton();
        /*
         * Preparing Search request xml
         */

        SearchInputBean searchInputBean = null;
        searchInputBean = getSearchInput(itemListConv, massUpdateBean,
                catalogID);
        try {
            // Performing Search to fetch Items
            SearchBean searchBean = performSearch(searchInputBean,catalogID);
            
            // Parsing CSV to Result Data
            ResultData resultData = searchBean.getSearchResults();

            // 0 is for traversal size,Not used in Multi edit portlet
            SearchResultBean resultBean = new SearchResultBean(resultData, 0);

            retList = fillAttributes(catalogItem, resultBean);
        } catch (UIException e1) {
            e1.printStackTrace();
        } catch (WPCMediatorException e2) {
            e2.printStackTrace();
            throw e2;
        }

        catListMap.put(IReference.MASSUPDATE_MAP_LIST_KEY, retList);
        catListMap.put(IReference.MASSUPDATE_MAP_MESSAGE_KEY, unSucc);

        //return the catalog item Map
        return catListMap;
    }

    /**
     * Creating close and filling data in attribute objects.   
     * @param catalogItem
     * @param resultBean
     * @return
     */
    private List fillAttributes(ICatalogItem catalogItem,
            SearchResultBean resultBean) {
        ArrayList retList = new ArrayList();
        ResultData resultdata = resultBean.getResultData();
        List rowlist = resultdata.getResultRowData();
        /*
         * Iterating through searched items
         */
        for (int i = 0; i < rowlist.size(); i++) {
            ICatalogItem clonedItem = (ICatalogItem) UIUtils.copy(catalogItem);
            Map cloneItemMap = clonedItem.getFlatAttributeMap();

            ResultRowData rowData = (ResultRowData) rowlist.get(i);
            Map elementMap = rowData.getRowElements();
            /*
             * Todo Primary key pass
             */
            String pkAttributeVal = (String) elementMap.get(clonedItem
                    .getPrimaryKeyAttr(clonedItem.getAttributes()).getName());
            clonedItem.setItemID(pkAttributeVal.trim());
            Set keySet = elementMap.keySet();
            Iterator elementIterator = keySet.iterator();
            /*
             * Iterating through attribute
             */
            while (elementIterator.hasNext()) {
                String attributeName = (String) elementIterator.next();
                Attribute attribute = (Attribute) cloneItemMap
                        .get(attributeName);
                String elementData = (String) elementMap.get(attributeName);
                fillAttribute(elementData, attribute);
            }
            retList.add(clonedItem);
        }

        return retList;
    }

    /**
     * Filling the attributeData in attribute object passed.
     * @param attributeData
     * @param attribute
     */
    private static void fillAttribute(String attributeData, Attribute attribute) {
        attributeData = attributeData.trim();
        switch (attribute.getType()) {
        case AttributeTypes.STRING: {
            StringAttribute stringAttribute = (StringAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeData);
                stringAttribute.setValues(values);
            } else {
                String value = attributeData;

                if (value != null) {
                    stringAttribute.setValue(value);
                }
            }

            return;
        }

        case AttributeTypes.INTEGER: {
            IntegerAttribute integerAttribute = (IntegerAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeData);
                integerAttribute.setValues(values);
            } else {
                try {
                    if (attributeData != null) {
                        integerAttribute.setValue(attributeData);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            return;
        }

        case AttributeTypes.NUMBER: {
            NumberAttribute numberAttribute = (NumberAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeData);
                numberAttribute.setValues(values);
            } else {
                try {
                    if (attributeData != null) {
                        numberAttribute.setValue(attributeData);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            return;
        }

        case AttributeTypes.DATE: {
            DateAttribute dateAttribute = (DateAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeData);
                dateAttribute.setValues(values);
            } else {
                String value = attributeData;
                if (value != null) {
                    dateAttribute.setValue(value);
                }
            }

            return;
        }

        case AttributeTypes.FLAG: {
            FlagAttribute flagAttribute = (FlagAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeData);
                flagAttribute.setValues(values);
            } else {
                String value = attributeData;
                if (attributeData != null) {
                    flagAttribute.setValue(value);
                }
            }

            return;
        }

        case AttributeTypes.TIMEZONE: {
            TimeZoneAttribute timeZoneAttribute = (TimeZoneAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeData);
                timeZoneAttribute.setValues(values);
            } else {
                String value = attributeData;
                if (value != null) {
                    timeZoneAttribute.setValue(value);
                }
            }

            return;
        }

        case AttributeTypes.CURRENCY: {
            CurrencyAttribute currencyAttribute = (CurrencyAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeData);
                currencyAttribute.setValues(values);
            } else {
                String value = attributeData;

                if (value != null) {
                    currencyAttribute.setValue(value);
                }
            }

            return;
        }

        case AttributeTypes.PASSWORD: {
            PasswordAttribute passwordAttribute = (PasswordAttribute) attribute;

            if (attribute.isMultiValued()) {
                StringTokenizer tokenizer = new StringTokenizer(attributeData,
                        "|");
                String[] values = new String[tokenizer.countTokens()];

                for (int i = 0; i < values.length; i++) {
                    values[i] = "*****";
                }
                passwordAttribute.setValues(values);
            } else {
                String value = "*****";
                if (value != null) {
                    passwordAttribute.setValue(value);
                }
            }

            return;
        }

        case AttributeTypes.PERIOD: {
            // special element contains the nested attributes for the start and end
            // periods.
            return;
        }
        case AttributeTypes.SEQUENCE: {
            SequenceAttribute sequenceAttribute = (SequenceAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeData);
                sequenceAttribute.setValues(values);
            } else {
                String value = attributeData;

                if (value != null) {
                    sequenceAttribute.setValue(value);
                }
            }

            return;
        }

        case AttributeTypes.RELATIONSHIP: {
            RelationshipAttribute relationshipAttribute = (RelationshipAttribute) attribute;
            NVPair[] values = getRelationValues(attributeData);

            if (attribute.isMultiValued()) {
                relationshipAttribute.setValues(values);
            } else {
                if ((values != null) && (values.length > 0)) {
                    relationshipAttribute.setValue(values[0]);
                }
            }

            return;
        }

        case AttributeTypes.BINARY: {
            BinaryAttribute binaryAttribute = (BinaryAttribute) attribute;
            FileUploadResource[] values = getFileUploadResourceValues(attributeData);

            if (binaryAttribute.isMultiValued()) {
                binaryAttribute.setValues(values);
            } else {
                if ((values != null) && (values.length > 0)) {
                    binaryAttribute.setValue(values[0]);
                }
            }

            return;
        }

        case AttributeTypes.IMAGE: {
            ImageAttribute imageAttribute = (ImageAttribute) attribute;
            FileUploadResource[] values = getFileUploadResourceValues(attributeData);

            if (imageAttribute.isMultiValued()) {
                imageAttribute.setValues(values);
            } else {
                if ((values != null) && (values.length > 0)) {
                    imageAttribute.setValue(values[0]);
                }
            }

            return;
        }

        case AttributeTypes.THUMB_NAIL_IMAGE: {
            ThumbnailImageAttribute thumbnailImageAttribute = (ThumbnailImageAttribute) attribute;
            FileUploadResource[] values = getFileUploadResourceValues(attributeData);

            if (thumbnailImageAttribute.isMultiValued()) {
                thumbnailImageAttribute.setValues(values);
            } else {
                if ((values != null) && (values.length > 0)) {
                    thumbnailImageAttribute.setValue(values[0]);
                }
            }

            return;
        }

        case AttributeTypes.URL: {
            URLAttribute urlAttribute = (URLAttribute) attribute;
            String[] values = getStringValues(attributeData);

            if (urlAttribute.isMultiValued()) {
                urlAttribute.setValues(values);
            } else {
                if ((values != null) && (values.length > 0)) {
                    urlAttribute.setValue(values[0]);
                }
            }

            return;
        }

        case AttributeTypes.IMAGE_URL: {
            ImageURLAttribute imageURLAttribute = (ImageURLAttribute) attribute;
            String[] values = getStringValues(attributeData);

            if (imageURLAttribute.isMultiValued()) {
                imageURLAttribute.setValues(values);
            } else {
                if ((values != null) && (values.length > 0)) {
                    imageURLAttribute.setValue(values[0]);
                }
            }

            return;
        }

        case AttributeTypes.THUMB_NAIL_IMAGE_URL: {
            ThumbnailImageURLAttribute thumbnailImageURLAttribute = (ThumbnailImageURLAttribute) attribute;
            String[] values = getStringValues(attributeData);

            if (thumbnailImageURLAttribute.isMultiValued()) {
                thumbnailImageURLAttribute.setValues(values);
            } else {
                if ((values != null) && (values.length > 0)) {
                    thumbnailImageURLAttribute.setValue(values[0]);
                }
            }

            return;
        }

        case AttributeTypes.STRING_ENUMERATION: {
            StringEnumAttribute stringEnumAttribute = (StringEnumAttribute) attribute;
            String[] enumValues = null;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeData);
                stringEnumAttribute.setValues(values);
            } else {
                String value = attributeData;

                if (value != null) {
                    stringEnumAttribute.setValue(value);
                }
            }
            //no enum is received and populated for mass edit
            return;
        }

        case AttributeTypes.NUMBER_ENUMERATION: {
            NumberEnumAttribute numberEnumAttribute = (NumberEnumAttribute) attribute;
            double[] enumValues = null;
            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeData);
                numberEnumAttribute.setValues(values);
            } else {
                String temp = attributeData;
                try {
                    if (temp != null) {
//                        double value = UIUtils.getDoubleValue(temp);
                        numberEnumAttribute.setValue(temp);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            //no enumValues is received and populated in case Mass Update
            return;
        }

        case AttributeTypes.LOOKUP_TABLE: {
            LookupTableAttribute lookupTableAttribute = (LookupTableAttribute) attribute;

            if (attribute.isMultiValued()) {
                String[] values = getStringValues(attributeData);
                lookupTableAttribute.setValues(values);
            } else {
                String value = attributeData;
                if (value != null) {
                    lookupTableAttribute.setValue(value);
                }
            }
            return;
        }

        case AttributeTypes.GROUP: {
            System.out.println("GROUP ATTRIBUTES*****************************");
            // Group attributes are not iterated or displayed.
            break;
        }

        } // end of switch
    }

    /**
     * Performs search updates search Bean with result.
     * @param searchInputBean
     * @throws UIException
     * @throws WPCMediatorException
     */
    public SearchBean performSearch(SearchInputBean searchInputBean,String collabName)
            throws UIException, WPCMediatorException {
        String searchReqXML = null;

        //get limit for results from settings
        searchReqXML = searchInputBean.getSearchXML();
        SearchBean searchBean = new SearchBean();
        searchBean.setReqXML(searchReqXML);
        searchBean.setMaxResults(maxItemRows);
        searchBean.setCollabName(collabName);

        WPCMediator mediator = WPCMediator.getInstance();
        mediator.performSearchForMultiEdit(searchBean, wpsContext);

        return searchBean;
    }

    /**
     * Creates search Input Request XML
     * @param itemIdList
     * @param massUpdateBean
     * @param catalogId
     * @return
     * @throws WPCMediatorException
     */
    private SearchInputBean getSearchInput(List itemIdList,
            MassUpdateBean massUpdateBean, String catalogId)
            throws WPCMediatorException {
        SearchInputBean searchInputBean = null;
        StringBuffer reqXML = null;
        String pkAttrName = massUpdateBean.getPrimaryKeyName();
        
        int elementCount = 0;
        int total = 0;
        int predicateIndex = 0;
        int index = 0;
        String operatorDisplayName = null;
        String displayName = null;
        String labelAnd = null;
        int wpcAttributeType;

        searchInputBean = new SearchInputBean();
        reqXML = new StringBuffer();

        reqXML.append("<").append(ISoapMessageKeys.SEARCH_CRITERIA_NAME)
                .append(">");
        reqXML.append(IBaseSearch.WPC_SEARCH_OPERATOR_OR_START_TAG);
        total = itemIdList.size();

        // **** start search element****

        for (Iterator iter = itemIdList.iterator(); iter.hasNext();) {
            String itemIdValue = (String) iter.next();

            if ((elementCount % 2) == 0) {
                reqXML.append(IBaseSearch.WPC_SEARCH_OPERATOR_OR_START_TAG);
            }

            reqXML.append(IBaseSearch.WPC_SEARCH_ELEMENT_START_TAG);

            //set elementName name
            reqXML.append(IBaseSearch.WPC_SEARCH_ATTRIBUTE_START_TAG);
            reqXML.append(pkAttrName);
            reqXML.append(IBaseSearch.WPC_SEARCH_ATTRIBUTE_END_TAG);

            // set attribute value
            reqXML.append(IBaseSearch.WPC_SEARCH_VALUE_START_TAG);
            reqXML.append(itemIdValue);
            reqXML.append(IBaseSearch.WPC_SEARCH_VALUE_END_TAG);

            // set the predicate
            reqXML.append(IBaseSearch.WPC_SEARCH_PREDICATE_START_TAG);
            reqXML.append("0");
            reqXML.append(IBaseSearch.WPC_SEARCH_PREDICATE_END_TAG);

            //set operator
            reqXML.append(IBaseSearch.WPC_SEARCH_ELEMENT_OPERATOR_START_TAG);
            reqXML.append("=");
            reqXML.append(IBaseSearch.WPC_SEARCH_ELEMENT_OPERATOR_END_TAG);
            reqXML.append(IBaseSearch.WPC_SEARCH_ELEMENT_END_TAG);

            if (((elementCount % 2) == 1) || (total == (elementCount + 1))) {
                reqXML.append(IBaseSearch.WPC_SEARCH_OPERATOR_END_TAG);
            }
            elementCount++;
        }
        //****END search element****

        reqXML.append(IBaseSearch.WPC_SEARCH_OPERATOR_END_TAG);

        reqXML.append("</").append(ISoapMessageKeys.SEARCH_CRITERIA_NAME)
                .append(">");
        /*
         * Attributelist node is added here...
         *  
         */
        reqXML.append("<" + ISoapMessageKeys.WPC_NS_URI_REF).append(":")
                .append(ISoapMessageKeys.ATTRIBUTELIST).append(" ").append(
                        ISoapMessageKeys.WORKFLOW_SPECNAME).append("=\"")
                .append(catalogId).append("\"").append(" ").append(
                        ISoapMessageKeys.ATTRIB_CATEGORY_SPECIFIC)
                .append("=\"").append("false").append("\"").append(">");

        for (Iterator iter = massUpdateBean.getItemAttributeList().iterator(); iter
                .hasNext();) {
            ItemAttribute itemAttribute = (ItemAttribute) iter.next();
            if (itemAttribute.getIsDisplayable()) {
                reqXML.append("<").append(ISoapMessageKeys.WPC_NS_URI_REF)
                        .append(":").append(ISoapMessageKeys.ATTRIBUTE).append(
                                " ").append(ISoapMessageKeys.ATTRIB_NAME)
                        .append("=\"").append(itemAttribute.getName()).append(
                                "\"").append("/>");
            }

        }
        reqXML.append("</" + ISoapMessageKeys.WPC_NS_URI_REF).append(":")
                .append(ISoapMessageKeys.ATTRIBUTELIST).append(">");

        searchInputBean.setSearchXML(reqXML.toString());
        return searchInputBean;
    }

    /**
     * @return
     */
    public WPSContext getWpsContext() {
        return wpsContext;
    }

    /**
     * @param context
     */
    public void setWpsContext(WPSContext context) {
        wpsContext = context;
    }

    /**
     * @return
     */
    public SearchCriteriaBean getSearchCriteriaBean() {
        return searchCriteriaBean;
    }

    /**
     * @param bean
     */
    public void setSearchCriteriaBean(SearchCriteriaBean bean) {
        searchCriteriaBean = bean;
    }

    /**
     * @param attributeData
     * @return
     */
    public static FileUploadResource[] getFileUploadResourceValues(
            String attributeData) {

        if (attributeData == null) {
            return null;
        }

        StringTokenizer tokenizer = new StringTokenizer(attributeData, "|");
        FileUploadResource[] values = new FileUploadResource[tokenizer
                .countTokens()];

        for (int i = 0; tokenizer.hasMoreTokens(); i++) {
            String data = tokenizer.nextToken();
            values[i] = new FileUploadResource(data);
        }
        return values;
    }

    /**
     * @param attributeData
     * @return
     */
    public static int[] getIntegerValues(String attributeData) {

        if (attributeData == null) {
            return null;
        }
        StringTokenizer tokenizer = new StringTokenizer(attributeData, "|");
        int[] values = new int[tokenizer.countTokens()];
        for (int i = 0; tokenizer.hasMoreTokens(); i++) {
            String data = tokenizer.nextToken();
            try {
                values[i] = Integer.parseInt(data.trim());
            } catch (NumberFormatException e) {
                e.printStackTrace();
            }
        }
        return values;
    }

    /**
     * @param attributeData
     * @return
     */
    public static double[] getNumberValues(String attributeData) {

        if (attributeData == null) {
            return null;
        }
        StringTokenizer tokenizer = new StringTokenizer(attributeData, "|");
        double[] values = new double[tokenizer.countTokens()];

        for (int i = 0; tokenizer.hasMoreTokens(); i++) {
            String data = tokenizer.nextToken();
            if (!UIUtils.isEmpty(data)) {
                values[i] = Double.parseDouble(data.trim());
            }
        }
        return values;
    }

    public static Element[] getPeriodValues(String attributeData) {
        return null;
    }

    /**
     * @param attributeData
     * @return
     */
    public static String[] getStringValues(String attributeData) {
        if (attributeData == null) {
            return null;
        }
        StringTokenizer tokenizer = new StringTokenizer(attributeData, "|");
        String[] values = new String[tokenizer.countTokens()];
        for (int i = 0; tokenizer.hasMoreTokens(); i++) {
            String data = tokenizer.nextToken();
            values[i] = data;
        }
        return values;
    }

    /**
     * @param attributeData
     * @return
     */
    public static NVPair[] getRelationValues(String attributeData) {

        if (attributeData == null) {
            return null;
        }
        StringTokenizer tokenizer = new StringTokenizer(attributeData, "|");
        NVPair[] valuesList = new NVPair[tokenizer.countTokens()];

        for (int i = 0; tokenizer.hasMoreTokens(); ++i) {
            String nameValue = tokenizer.nextToken();
            if (!UIUtils.isEmpty(nameValue)) {
                String name = nameValue.substring(0, nameValue.indexOf('^'));
                String value = nameValue.substring(nameValue.indexOf('^') + 1,
                        nameValue.length());
                ;
                valuesList[i] = new NVPair(name, value);
            }
        }

        return valuesList;
    }

    /**
     * @param attributeData
     * @return
     */
    public static URL[] getURLValues(String attributeData) {
        String[] urlStings = getStringValues(attributeData);

        if ((urlStings == null) || (urlStings.length == 0)) {
            return null;
        }

        URL[] urls = new URL[urlStings.length];

        for (int i = 0; i < urlStings.length; ++i) {
            try {
                if (!UIUtils.isEmpty(urlStings[i])) {
                    urls[i] = new URL(urlStings[i]);
                }
            } catch (MalformedURLException mue) {
                mue.printStackTrace();
            }
        }

        return urls;
    }

    /**
     * @return Returns the maxItemRows.
     */
    public String getMaxItemRows() {
        return maxItemRows;
    }

    /**
     * @param maxItemRows The maxItemRows to set.
     */
    public void setMaxItemRows(String maxItemRows) {
        this.maxItemRows = maxItemRows;
    }
}
