/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * Created on Jan 5, 2005
*/
package com.ibm.ccd.ui.portlets.massupdate;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.text.MessageFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

import org.apache.jetspeed.portlet.AccessDeniedException;
import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletData;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletSession;
import org.apache.jetspeed.portlet.PortletSettings;
import org.apache.jetspeed.portlet.PortletURI;
import org.apache.jetspeed.portlet.UnavailableException;
import org.apache.jetspeed.portlet.event.ActionEvent;
import org.apache.jetspeed.portlet.event.ActionListener;

import com.ibm.ccd.ui.context.IWPCUser;
import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.AttributeTypes;
import com.ibm.ccd.ui.dataobjects.attributes.StringAttribute;
import com.ibm.ccd.ui.dataobjects.items.ICatalogItem;
import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.exception.UIException;
import com.ibm.ccd.ui.mediator.WPCMediator;
import com.ibm.ccd.ui.mediator.WPCMediatorException;
import com.ibm.ccd.ui.portlets.BasePortlet;
import com.ibm.ccd.ui.portlets.alert.ITransactionDisplay;
import com.ibm.ccd.ui.portlets.catalog.ICatalogAction;
import com.ibm.ccd.ui.portlets.fileupload.FileUploadBean;
import com.ibm.ccd.ui.portlets.fileupload.IFileUpload;
import com.ibm.ccd.ui.portlets.items.WorkwithItemPortlet;
import com.ibm.ccd.ui.portlets.quicksearch.IQuickSearch;
import com.ibm.ccd.ui.util.FileUploadUtil;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;


/**
 *
 */
public class MassUpdatePortlet extends BasePortlet implements ActionListener {
    /**
     * Set th business function
     */
    protected void setBusinessFunction(PortletRequest request) {
        PortletSession portletSession = null;
        portletSession = request.getPortletSession();
        portletSession.setAttribute(IReference.SESSION_BUS_FUNCT,
            IMassUpdate.BUSINESS_FUNCTION_MASS_UPDATE);
    }

    /**
     * This method is called by container while doing initialisation.
     * @param  portletConfig
     * @throws UnavailableException
     */
    public void init(PortletConfig portletConfig) throws UnavailableException {
        super.init(portletConfig);
    }

    /**
     * This method is called by the container in view mode to display view to user
     *@param request
     *@param response
     * @throws PortletException, IOException
     */
    public void doView(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        log.debug(" Mass Update doView ->");
        PortletSession portletSession = null;
        PortletURI formActionURI = null;
        PortletURI formSaveUpdateURI = null;
        PortletURI formDeleteURI = null;
        MassUpdateBean massUpdateBean = null;
        String jsp2Render = IMassUpdate.VIEW_JSP_PATH;

        //set the business function
        setBusinessFunction(request);
        portletSession = request.getPortletSession();
        portletSession.setAttribute(IReference.SESSION_ACTION,
            IMassUpdate.BUSINESS_ACTION_VIEW);

        /*         Code for populating the bean for displayable attributes and store in session.
         */

        //check the bean in session , if null populate and set in session
        String sourcePortlet = request.getParameter(ICatalogAction.COMMAND);
        try {
            if (ICatalogAction.SEARCH.equals(sourcePortlet)) {
                massUpdateBean = getSearchedItemIds(request);
        	}else {
                massUpdateBean = (MassUpdateBean) portletSession.getAttribute(IMassUpdate.KEY_MASS_UDATE_BEAN);
            }
            
            String catalogId = UIUtils.getCatalogName(getWPSContext(request).getWpcUser());
            if( (massUpdateBean != null) && (massUpdateBean.getItemsSetforWorkflow()!=null)){
                catalogId = massUpdateBean.getItemsSetforWorkflow(); 
            }
			
			// If current catalog name is changed in from the one stored by mass edit.  
			if((massUpdateBean != null ) && !(massUpdateBean.getPrimaryCatalogName().equals(catalogId))){
				massUpdateBean = null ;
			}
			
            if (massUpdateBean == null) {
                massUpdateBean = new MassUpdateBean();
                populateMassUpdateBean(request, massUpdateBean);
            }
        } catch (UIException e) {
            e.printStackTrace();
            throw new PortletException(e);
        }

        setHeaderAndMode(massUpdateBean, getMessageMap(request));

        /*
         *  Code for displaying the item ids and selected attributes
         */
        List allAttrList = massUpdateBean.getItemAttributeList();
        massUpdateBean.setItemAttributeListForHeader(getHeaderAttributeList(
                allAttrList));

        Map displayableitems = massUpdateBean.getItemMap();
        if (displayableitems == null && massUpdateBean.getPortletRequestAttributes() != null) {
            try {
                massUpdateBean = getSearchedItemIds(request);
                displayableitems = massUpdateBean.getItemMap();
            } catch (PortletException e1) {
                e1.printStackTrace();
            } catch (UIException e1) {
                e1.printStackTrace();
            }
        }
        if (displayableitems == null) {
            displayableitems = new HashMap();
        }
        massUpdateBean.setItemMap(displayableitems);

        formSaveUpdateURI = response.createURI();
        formSaveUpdateURI.addAction(IMassUpdate.VIEW_FORM_SAVE_ACTION_URI);
        massUpdateBean.setFormSaveUpdateURI(formSaveUpdateURI.toString());

        formDeleteURI = response.createURI();
        formDeleteURI.addAction(IMassUpdate.VIEW_FORM_DELETE_ACTION_URI);
        massUpdateBean.setFormDeleteURI(formDeleteURI.toString());
        request.setAttribute(IMassUpdate.KEY_ITEMSLIST, massUpdateBean);

        //	Set actionURI in the view mode bean
        formActionURI = response.createURI();
        formActionURI.addAction(IMassUpdate.VIEW_FORM_ACTION_URI);

        MessageMap msgMap = getMessageMap(request);
        
        if ((massUpdateBean.getMode().equals(IMassUpdate.VIEW)) &&
                (massUpdateBean.getItemMap().size() == 0)) {
            jsp2Render = ICatalogAction.MESSAGEPAGE;
            msgMap.addInfoMessage("MESSAGE_CODE_360");
        } else if ((massUpdateBean.getMode().equals(IMassUpdate.UPDATE)) &&
                (massUpdateBean.getItemMap().size() == 0)) {
            jsp2Render = ICatalogAction.MESSAGEPAGE;
            msgMap.addInfoMessage("MESSAGE_CODE_360");
        } else if ((massUpdateBean.getMode().equals(IMassUpdate.VIEW)) &&
                (massUpdateBean.getItemMap().size() > 0)) {
            msgMap.addInfoMessage("MESSAGE_CODE_363");
        }
        //Changed
        WPSContext context = getWPSContext(request);
        portletSession.setAttribute("WPS_CONTEXT",context);
        
        // Invoke the JSP to render        		
        getPortletConfig().getContext().include(jsp2Render, request, response);
    }

    /**
     * This method sets initial header messages and Mode.
     * @param massUpdateBean
     * @param msgMap
     */
    private void setHeaderAndMode(MassUpdateBean massUpdateBean,
        MessageMap msgMap) {
        String strMode = massUpdateBean.getMode();

        if (strMode == null) {
            if (!((massUpdateBean.getItemMap() == null) ||
                    (massUpdateBean.getItemMap().size() == 0))) {
                msgMap.addInfoMessage("MESSAGE_CODE_361");
            }
        } else if (strMode.equals(IMassUpdate.UPDATE)) {
            if (!((massUpdateBean.getItemMap() == null) ||
                    (massUpdateBean.getItemMap().size() == 0))) {
                msgMap.addInfoMessage("MESSAGE_CODE_361");
            }
        } else if (strMode.equals(IMassUpdate.DELETE)) {
            msgMap.addInfoMessage("MESSAGE_CODE_362");
        }

        if (strMode == null) {
            massUpdateBean.setMode(IMassUpdate.UPDATE);
        }
    }

    /**
     * Method returns iterates through all items and returns Map of items
     * with key as item id and value as ItemViewBean Object(that holds item Object)
     * @param catitemsObjlist
     * @return
     */
    private HashMap getDisplayItemsMap(List catitemsObjlist) {
        HashMap displayableitems = new HashMap();

        if ((catitemsObjlist != null) && (catitemsObjlist.size() != 0)) {
            Iterator iteratorCatItems = catitemsObjlist.iterator();

            // list of items that needs to be displayed  
            while (iteratorCatItems.hasNext()) {
                // loop for each items fetched from wpc					
                ICatalogItem catItem = (ICatalogItem) iteratorCatItems.next();

                //	Map attrMap = catItem.getAttributeMap();
                ItemViewBean itemViewObj = new ItemViewBean();
                itemViewObj.setCatalogItem(catItem);

                Attribute[] attrArry = catItem.getAttributes();
                Attribute primaryKeyAttr = catItem.getPrimaryKeyAttr(attrArry);
                StringAttribute strAttr = (StringAttribute) primaryKeyAttr;

                itemViewObj.setItemId(strAttr.getValue());
                displayableitems.put(strAttr.getValue(), itemViewObj);
            }
        }

        return displayableitems;
    }

    /**
     * Method returns header attribute list displayed on Mass edit portlet
     * @param allAttrList
     * @return
     */
    private List getHeaderAttributeList(List allAttrList) {
        log.debug(" getHeaderAttributeList ->");

        Iterator iteratorAll = allAttrList.iterator();
        List displayablelist = new ArrayList();

        while (iteratorAll.hasNext()) {
            ItemAttribute itemAttribute = (ItemAttribute) iteratorAll.next();

            if (itemAttribute.getIsDisplayable()) {
                displayablelist.add(itemAttribute);
                itemAttribute.setEditable(getEditableAttribute(
                        itemAttribute.getAttribute()));
            }
        }

        return displayablelist;
    }

    /**
     * Method decides which attributes are editable in bulk edit screen
     * @param attribute
     * @return
     */
    public boolean getEditableAttribute(Attribute attribute) {
		return UIUtils.getEditableAttributeCheckforMass(attribute);
    }



    /**
     * Handles the Edit mode event
     */
    public void doEdit(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        MessageMap messageMap = null;
        PortletData portletData = null;
        PortletSession portletSession = null;
        PortletSettings portletSettings = null;
        PortletURI formActionURI = null;
        MassUpdateBean massUpdateBean = null;
        String mxAttributes = null;

        portletSession = request.getPortletSession();

        //set the business function
        setBusinessFunction(request);

        //set the business action
        portletSession.setAttribute(IReference.SESSION_ACTION,
            IMassUpdate.BUSINESS_ACTION_EDIT);

        //check the bean in session , if null populate and set in session
        massUpdateBean = (MassUpdateBean) portletSession.getAttribute(IMassUpdate.KEY_MASS_UDATE_BEAN);

        try {
            if (massUpdateBean == null) {
                massUpdateBean = new MassUpdateBean();
                populateMassUpdateBean(request, massUpdateBean);
            }
        } catch (UIException e) {
            e.printStackTrace();
            throw new PortletException(e);
        }

        try {
            messageMap = getMessageMap(request);
        } catch (Exception e) {
            e.printStackTrace();
            throw new PortletException(e);
        }

        portletSettings = request.getPortletSettings();
        mxAttributes = portletSettings.getAttribute(IMassUpdate.KEY_MAX_ATTRIBUTES);
        messageMap.addInfoMessage(IMassUpdate.MSG_MAX_ALLOWED_ATTRIBUTES,
            mxAttributes);

        //	Set actionURI in the view mode bean
        formActionURI = response.createURI();
        formActionURI.addAction(IMassUpdate.EDIT_FORM_ACTION_UPDATE);
        massUpdateBean.setFormActionURI(formActionURI.toString());
        //		Invoke the JSP to render        		
        getPortletConfig().getContext().include(IMassUpdate.EDIT_JSP_PATH,
            request, response);
    }

    /**
     * Handles the Portlet help events
     * @param PortletRequest , PortletResponse
     */
    public void doHelp(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        setBusinessFunction(request);

        getPortletConfig().getContext().include(IMassUpdate.HELP_JSP_PATH,
            request, response);
    }

    /**
     * @param request
     */
    private void saveConfig(PortletRequest request)
        throws UIException, Exception {
        PortletSettings portletSettings = null;
        MessageMap messageMap = null;
        String sMaxAttributes = null;
        String sMaxRows = null;
        int iMaxRows = 0;
        int isMaxAttributes = 0;
        String colName = null;

        sMaxAttributes = request.getParameter(IMassUpdate.KEY_MAX_ATTRIBUTES);
        sMaxRows = request.getParameter(IMassUpdate.KEY_MAX_ROWS);

        messageMap = getMessageMap(request);
        
        //parse strings to int for validation
        isMaxAttributes = getValidatedIntValue(sMaxAttributes,
                IMassUpdate.KEY_MAX_ATTRIBUTES, request);
        iMaxRows = getValidatedIntValue(sMaxRows, IMassUpdate.KEY_MAX_ROWS, request);

        // store the value in portletSettings object
        portletSettings = request.getPortletSettings();
        portletSettings.setAttribute(IMassUpdate.KEY_MAX_ATTRIBUTES,
            sMaxAttributes);

        portletSettings.setAttribute(IMassUpdate.KEY_MAX_ROWS, sMaxRows);

        portletSettings.store();
        messageMap.addInfoMessage(IMassUpdate.MSG_UPDATE_SUCCESS);
    }

    /**
     * This method is called by container when administrator clicks on the configure button on toolbar.
     * @param request , response
     * @throws PortletException
     * @throws  IOException
     */
    public void doConfigure(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        PortletURI formActionURI = null;
        PortletSession portletSession = null;
        PortletSettings portletSettings = null;
        MessageMap messageMap = null;
        MassUpdateBean massUpdateBean = null;
        String sMaxAttributes = null;
        String sMaxRows = null;

        portletSession = request.getPortletSession();

        //		set the business function
        setBusinessFunction(request);

        //set the business action
        portletSession.setAttribute(IReference.SESSION_ACTION,
            IMassUpdate.BUSINESS_ACTION_CONFIG);

        //check the bean in session , if null populate and set in session
        massUpdateBean = (MassUpdateBean) portletSession.getAttribute(IMassUpdate.KEY_MASS_UDATE_BEAN);

        try {
            if (massUpdateBean == null) {
                massUpdateBean = new MassUpdateBean();
                populateMassUpdateBean(request, massUpdateBean);
            }

            messageMap = getMessageMap(request);
        } catch (UIException e) {
            e.printStackTrace();
            throw new PortletException(e);
        } catch (Exception e) {
            e.printStackTrace();
            throw new PortletException(e);
        }

        //set max attributes value in bean
        portletSettings = request.getPortletSettings();
        sMaxAttributes = portletSettings.getAttribute(IMassUpdate.KEY_MAX_ATTRIBUTES);
        sMaxRows = portletSettings.getAttribute(IMassUpdate.KEY_MAX_ROWS);

        //set values in data bean
        massUpdateBean.setMaxAttributes(sMaxAttributes);
        massUpdateBean.setMaxRows(sMaxRows);

        //add info message
        messageMap.addInfoMessage(IMassUpdate.MSG_MAX_ATTRIBUTES_AND_ROWS);

        // create form action URI
        formActionURI = response.createURI();
        formActionURI.addAction(IMassUpdate.CONFIG_FORM_ACTION_UPDATE);
        massUpdateBean.setFormActionURI(formActionURI.toString());

        getPortletConfig().getContext().include(IMassUpdate.CONFIG_JSP_PATH,
            request, response);
    }

    /**
     * Handles all action events
     */
    public void actionPerformed(ActionEvent event) throws PortletException {
        log.debug(" actionPerformed ->");

        MessageMap messageMap = null;
        PortletRequest request = null;
        PortletSession portletSession = null;
        MassUpdateBean massUpdateBean = null;
        String actionString = null;

        // initialize		
        request = event.getRequest();
        portletSession = request.getPortletSession();

        actionString = event.getActionString();
        massUpdateBean = (MassUpdateBean) portletSession.getAttribute(IMassUpdate.KEY_MASS_UDATE_BEAN);

        //process event
        try {
            messageMap = getMessageMap(request);

            if (actionString.equals(IMassUpdate.EDIT_FORM_ACTION_UPDATE)) {
                //save new edit settings
                updateItemAttributes(request, massUpdateBean);
                messageMap.addInfoMessage(IMassUpdate.MSG_UPDATE_SUCCESS);
            }

            if (actionString.equals(IMassUpdate.CONFIG_FORM_ACTION_UPDATE)) {
                //save new config settings
                saveConfig(request);

                //bean set to null so that in next mode it is repopulated according to new settings
                portletSession.setAttribute(IMassUpdate.KEY_MASS_UDATE_BEAN,
                    null);
            }

            if (actionString.equals(IMassUpdate.CANCEL)) {
                //save new edit settings
                massUpdateBean.setMode(IMassUpdate.VIEW);
            } else if (actionString.equals(IMassUpdate.VIEW)) {
                //save new edit settings
                massUpdateBean.setMode(IMassUpdate.VIEW);
            } else if (actionString.equals(IMassUpdate.DELETE)) {
                massUpdateBean.setMode(IMassUpdate.DELETE);
                getMessageMap(request).addInfoMessage("MESSAGE_CODE_362");
            } else if (actionString.equals(IMassUpdate.UPDATE)) {
                massUpdateBean.setMode(IMassUpdate.UPDATE);

                if (!((massUpdateBean.getItemMap() == null) ||
                        (massUpdateBean.getItemMap().size() == 0))) {
                    getMessageMap(request).addInfoMessage("MESSAGE_CODE_361");
                }
            }

            /* added for update and delete bult Item attributes */
            if (actionString.equals(IMassUpdate.VIEW_FORM_SAVE_ACTION_URI)) {
                //save new edit settings
                updateSelectedItems(request, massUpdateBean);
            } else if (actionString.equals(
                        IMassUpdate.VIEW_FORM_DELETE_ACTION_URI)) {
                // save new edit settings
                deleteSelectedItems(request, massUpdateBean);
            }
        } catch (UIException e) {
            e.printStackTrace();
            throw new PortletException(e);
        } catch (Exception e) {
            e.printStackTrace();
            throw new PortletException(e);
        }
    }

    /**
     * Method saves all changes done by all the displayed attributes of Item.
     * It generates the formatted comma separated csv file to send it to wpc.
     * All attributes are listed in one row for each item starting with primary key.
     * @param request
     * @param massUpdateBean
     * @throws Exception
     */
    public void updateSelectedItems(PortletRequest request,
        MassUpdateBean massUpdateBean) throws Exception {
        log.debug(" updateSelectedItems ->");

        String[] itemArray = getReqParamArray(request, "ITEMID");
        HashSet succSet = new HashSet();
        MessageMap messageMap = getMessageMap(request);
        boolean updateFlg = false;
        PortletSession session = request.getPortletSession();
        WPSContext context = getWPSContext(request);
        List itemattrlist = massUpdateBean.getItemAttributeList();

        Map catItemMap = massUpdateBean.getItemMap();
        String csvString = "";
        StringBuffer strBuff = createHeader(itemattrlist);
        Set errorAttributes = new HashSet();        
        // iterating through each items
        for (int i = 0; i < itemArray.length; i++) {
            String itemChk = request.getParameter("CHKID_" +
                    itemArray[i].trim());

            if ((itemChk != null) && (itemChk.equals("on"))) {
                ItemViewBean itemViewObj = (ItemViewBean) catItemMap.get(itemArray[i]);
                strBuff.append(UIUtils.escapeForCSV(itemViewObj.getItemId()));

                // tracking ids for display messages
                succSet.add(itemViewObj.getItemId());

                ICatalogItem item = itemViewObj.getCatalogItem();

                // item  displayable attribute list 
                Iterator attrList = itemattrlist.iterator();

                while (attrList.hasNext()) {
                    ItemAttribute itemAttribute = (ItemAttribute) attrList.next();

                    Attribute attr = itemAttribute.getAttribute();

                    if (itemAttribute.getIsDisplayable() &&
                            (itemAttribute.isEditable())) {
                        String attrName = attr.getName();
                        Attribute attrtemp = (Attribute) item.getFlatAttributeMap()
                                                             .get(attrName);

                        if (attrtemp.isPrimaryKey()) {
                            continue;
                        }

                        updateFlg = true;

                        // update items
                        request.setAttribute(IMassUpdate.KEY_MASS_EDIT,
                            IMassUpdate.KEY_MASS_EDIT);
                        try {
                        // This will set request object value to Attribute object for setAttribProperty 
                        messageMap = item.setProperty(request, attrName);
                        } catch (ParseException e) {
                            errorAttributes.add(attrName);
                            e.printStackTrace();
                        }
                        strBuff.append(",");
                        strBuff.append(UIUtils.escapeForCSV(WorkwithItemPortlet.getStringValue(
                                attrtemp, 0)));
                    }
                }

                ItemViewBean itemViewObjtemp = new ItemViewBean();
                itemViewObjtemp.setItemId(itemArray[i]);
                itemViewObjtemp.setCatalogItem(item);
                massUpdateBean.getItemMap().put(itemArray[i], itemViewObjtemp);

                if (i != (itemArray.length - 1)) {
                    strBuff.append("\n");
                }
            } // end of if for checkbox of item
        } // end of for loop for items
        
        if ((messageMap != null) && (messageMap.size() > 0)) {
        	updateFlg = false;
        }
        
        if((errorAttributes != null) && (errorAttributes.size() != 0)){
            messageMap.addErrorMessage("MESSAGE_CODE_367",UIUtils.getCommaSeparatedId(errorAttributes));
            updateFlg = false;            
        }
        // update only if items are checked and updated.
        if (updateFlg) {
            processItems(context, request, strBuff.toString(), succSet);
        }

        log.debug(" updateSelectedItems <- ");
    }

    /**
     * This method creates Header for CSV string to send to WPC.
     * that consists attribute names starting with GTIN.First byte is '.'
     * @param itemattrlist
     * @return StringBuffer
     */
    private StringBuffer createHeader(List itemattrlist) {
        StringBuffer strBuff = new StringBuffer();
        StringBuffer strBufftemp = new StringBuffer();

        Iterator iterator = itemattrlist.iterator();

        while (iterator.hasNext()) {
            ItemAttribute itemAttribute = (ItemAttribute) iterator.next();
            String attrName = itemAttribute.getName();

            if (itemAttribute.getAttribute().isPrimaryKey()) {
                strBuff.append(UIUtils.escapeForCSV(attrName));
                strBuff.append(",");
            } else if (itemAttribute.getIsDisplayable() &&
                    (itemAttribute.isEditable())) {
                strBufftemp.append(UIUtils.escapeForCSV(attrName));
                strBufftemp.append(",");
            }
        }

        strBufftemp = strBufftemp.deleteCharAt(strBufftemp.length() - 1);
        strBuff.append(strBufftemp.toString());
        strBuff.append("\n");

        return strBuff;
    }

    /**
     * Method used for deleting selected Items.It deletes all the checked Items
     * in Mass Edit Portlet
     * @param request
     * @param massUpdateBean
     * @throws Exception
     */
    public void deleteSelectedItems(PortletRequest request,
        MassUpdateBean massUpdateBean) throws Exception {
        log.debug(" deleteSelectedItems ->");

        MessageMap messagemap = getMessageMap(request);
        String[] itemArray = getReqParamArray(request, "ITEMID");
        Map catItemMap = massUpdateBean.getItemMap();
        HashSet succSet = new HashSet();
        HashSet unSuccSet = new HashSet();
        StringBuffer unsuccMsg = new StringBuffer();
        WPSContext context = getWPSContext(request);

        for (int i = 0; i < itemArray.length; i++) {
            String itemChk = request.getParameter("CHKID_" +
                    itemArray[i].trim());

            if ((itemChk != null) && (itemChk.equals("on"))) {
                if (itemArray[i] != null) {
                    try {
                        ItemViewBean itemViewObj = (ItemViewBean) catItemMap.get(itemArray[i]);
                        ICatalogItem item = itemViewObj.getCatalogItem();
                        getMediator().deleteItem(item, context);
                        succSet.add(itemArray[i]);
                        catItemMap.remove(itemArray[i]);
                    } catch (WPCMediatorException e) {
                        e.printStackTrace();
                        unSuccSet.add(itemArray[i]);
                    }
                } else {
                    return;
                }
            }
        }

        //		Error Message logging..
        if (unSuccSet.size() > 0) {
            if (succSet.size() > 0) {
                messagemap.addErrorMessage("MESSAGE_CODE_355",
                    UIUtils.getCommaSeparatedId(succSet),
                    UIUtils.getCommaSeparatedId(unSuccSet));
            } else {
                messagemap.addErrorMessage("MESSAGE_CODE_354",
                    UIUtils.getCommaSeparatedId(unSuccSet));
            }
        } else {
            messagemap.addInfoMessage("MESSAGE_CODE_353",
                UIUtils.getCommaSeparatedId(succSet));
        }
    }

    /**
     * This method updates isDisplayable field of ItemAttribute
     * @param request
     * @param massUpdateBean
     */
    private void updateItemAttributes(PortletRequest request,
        MassUpdateBean massUpdateBean) {
        ArrayList itemAttributeList = null;
        Iterator itemAttributeItr = null;
        ItemAttribute itemAttribute = null;
        String reqAttributeName = null;
        String itemAttributeName = null;
        // Added later 
        StringBuffer selectedAttributes = null;
        int listBoxIndex = 0;
        String catalogId = null;

        String[] attrArray = getReqParamArray(request, "ATTR_CHKBOX");
        List selectedAttrList = UIUtils.getStringArrayList(attrArray);
        
        if(!(selectedAttrList.contains(massUpdateBean.getPrimaryKeyName()))){
            getMessageMap(request).addErrorMessage("MESSAGE_CODE_366",massUpdateBean.getPrimaryKeyLable());
            return;
        }
        itemAttributeList = massUpdateBean.getItemAttributeList();
        itemAttributeItr = itemAttributeList.iterator();
         

        Iterator parameterIter = selectedAttrList.iterator();
        catalogId = massUpdateBean.getPrimaryCatalogName();
        
        // Checking if user has checked more than max attribute configured
        PortletSession portletSession = request.getPortletSession();        
        PortletSettings portletSettings = request.getPortletSettings();
        String sMaxAttributes = portletSettings.getAttribute(IMassUpdate.KEY_MAX_ATTRIBUTES);
        int iMaxAttribute = Integer.parseInt(sMaxAttributes);
        int attributeIndex = 0;
        
        if (selectedAttrList.size() > iMaxAttribute) {
            getMessageMap(request).addErrorMessage("MESSAGE_CODE_352",
                sMaxAttributes);
            return;
        }
        
        selectedAttributes = new StringBuffer();
        
        //set all itemAttributes isDisplayable to false
        while (itemAttributeItr.hasNext()) {
            itemAttribute = (ItemAttribute) itemAttributeItr.next();
            itemAttribute.setIsDisplayable(false);
            itemAttribute.setSelected(false);
        }

        //loop through parameters
        while (parameterIter.hasNext()) {
            reqAttributeName = (String) parameterIter.next();
            reqAttributeName = reqAttributeName.trim();
            itemAttributeItr = itemAttributeList.iterator();
            while (itemAttributeItr.hasNext()) {
                itemAttribute = (ItemAttribute) itemAttributeItr.next();
                itemAttributeName = itemAttribute.getName();
                
                if (itemAttributeName.equals(reqAttributeName)) {
                    itemAttribute.setIsDisplayable(true);
                    itemAttribute.setSelected(true);
                    itemAttribute.setListBoxIndex(attributeIndex++);
                    // add to setting string
                    selectedAttributes.append(itemAttributeName);
                    selectedAttributes.append(IQuickSearch.CONFIG_INDEX_TOKENIZER);
                    selectedAttributes.append(attributeIndex);
                    selectedAttributes.append(IQuickSearch.CONFIG_ATTRIBUTE_TOKENIZER);

                }
            }
        }
        ArrayList sortedSelectedAttrs = (ArrayList)orderAttributes(massUpdateBean.getItemAttributeList(),selectedAttrList.size());

        massUpdateBean.setItemAttributeList(sortedSelectedAttrs);
        catalogId = massUpdateBean.getPrimaryCatalogName();
        // Since attribute list is reset.Items needs to be fetched from wpc again 
        massUpdateBean.setItemMap(null);
        PortletData portletData = request.getData();
        try {
            portletData.setAttribute(catalogId,
                selectedAttributes.toString());
            portletData.store();
        } catch (AccessDeniedException e) {
            e.printStackTrace();
        } catch (IOException e1) {
            e1.printStackTrace();
        } 
            
    }

    /**
     * This is a method that fetches and populate all attributes of Item  
     * skeleton and store in session.This method must be executed before    
     * rendering any mode of Mass Update portlet.
     * Sets the <code>Attribute</code> list in the session
     */
    private void populateMassUpdateBean(PortletRequest request,
        MassUpdateBean bean) throws UIException {
        log.debug(" populateMassUpdateBean ->");

        PortletSession portletSession = null;
        PortletSettings portletSettings = null;
        ArrayList attributesList = null;
        Map attributesMap = null;
        ArrayList itemAttributeList = null;
        ItemAttribute itemAttribute = null;
        int count = 0;
        String sMaxAttributes = null;
        int iMaxAttribute = 0;

        WPSContext context = getWPSContext(request);
        String catalogID = (String) request.getParameter(ICatalogAction.CATALOG);
        
        if (catalogID == null) {
            catalogID = UIUtils.getCatalogName(getWPSContext(request)
                                                   .getWpcUser());
        }

        bean.setPrimaryCatalogName(catalogID);
       
        // call mediator and retrives Attribute array
        ICatalogItem catalogItem = getAttributes(context, catalogID);
        
        ICatalogItem catalogItemOrig =  (ICatalogItem)UIUtils.copy(catalogItem);

        bean.setCatalogItemSkeleton(catalogItemOrig);
        
        attributesMap = catalogItem.getFlatAttributeMap();
        //get maximum allowed atributes from portlet settings
        portletSettings = request.getPortletSettings();
        sMaxAttributes = portletSettings.getAttribute(IMassUpdate.KEY_MAX_ATTRIBUTES);
        iMaxAttribute = Integer.parseInt(sMaxAttributes);
        // Adjustment for adding primary key separately
        iMaxAttribute = iMaxAttribute - 1 ;

        //Populate the list of ItemAttributes
        itemAttributeList = new ArrayList();
        bean.setItemAttributeList(itemAttributeList);
        
        int attributeIndex = 1;
        
        // getting Map of Stored Attributes from PortletSettings 
        Map storedAttributeMap = getStoredAttributeMap(request.getData(), catalogID);
       
        // Merges stored attributes Map with the current attribute map   
        attributeIndex = getMergedAttributeList(bean, attributesMap, count ,iMaxAttribute, 
                context, attributeIndex,storedAttributeMap);
        count = bean.getItemAttributeList().size(); 
        
        getMergedAttributeList(bean, attributesMap, count, iMaxAttribute, 
                context,attributeIndex,attributesMap);
        
        //Primary key should always appear in the list  
        addPrimarykey(attributesMap, itemAttributeList, bean.getPrimaryKeyName());

        // set the ArrayList in bean
        bean.setItemAttributeList(itemAttributeList);
        //set the bean in session
        portletSession = request.getPortletSession();
        portletSession.setAttribute(IMassUpdate.KEY_MASS_UDATE_BEAN, bean);
    }

    /**
     * Gets merged Attribute list.It merges the attribute list that is stored in
     * portletData.It fetches maximum configured no. attribute count and adjust 
     * attributelist.It includes adds/remove attributes in list based on need.
     * Also checks for locale specific attribute.   
     * @param bean
     * @param attributesMap
     * @param countObj
     * @param iMaxAttribute
     * @param context
     * @param attributeIndexObj
     * @param iterAttSpecSet
     */
    private int getMergedAttributeList(MassUpdateBean bean, Map attributesMap,  
            int count, int iMaxAttribute, WPSContext context,int attributeIndex,Map iterableMap) {
        
    	
        Set set = iterableMap.keySet();
        Iterator iterAttSpecSet = set.iterator();
        ArrayList itemAttributeList = bean.getItemAttributeList();
        
        boolean bRemove = false;
        Attribute pkAttribute = null ;
        
        if(attributesMap != iterableMap){
            bRemove= true;
        }
        
        while (iterAttSpecSet.hasNext()) {
        	
            Attribute attributeLoopTemp = (Attribute) attributesMap.get((String) iterAttSpecSet.next());
            
			String locale =
				(String) context.getWpcUser().getProperty(
					IWPCUser.KEY_RESTRICTED_LANGUAGE);
			
			/*
			 * Added Null check.
			 * For Fix : WPC00028798
			 */
			if(attributeLoopTemp == null)
				continue;
				
			if(attributeLoopTemp.getType() == AttributeTypes.GROUP || 
				!UIUtils.isAttributeDisplayableForLocale(
					attributeLoopTemp,locale)){
				continue;
			}
            // added for displaying the primary key attribute Name in jsp
            if (attributeLoopTemp.isPrimaryKey()) {
                bean.setPrimaryKeyAttribute(attributeLoopTemp);
                continue;
            }
            ItemAttribute itemAttribute = new ItemAttribute();
            itemAttribute.setAttribute(attributeLoopTemp);
            itemAttribute.setName(attributeLoopTemp.getName());
            
            if (count < iMaxAttribute) {
                itemAttribute.setIsDisplayable(true);
                itemAttribute.setSelected(true);
                itemAttribute.setListBoxIndex(attributeIndex++);
            } else {
                itemAttribute.setIsDisplayable(false);
                itemAttribute.setSelected(false);
            }
            
			boolean displayableAttrCountReduced = true;
			if (itemAttribute.getIsDisplayable()) {
				if (UIUtils
					.isAttributeDisplayableForLocale(
						itemAttribute.getAttribute(),
						locale)) {
					displayableAttrCountReduced = true;
				} else {
					itemAttribute.setIsDisplayable(false);
				    itemAttribute.setSelected(false);					
					displayableAttrCountReduced = false;
				}
			}
			if (!displayableAttrCountReduced) {
				count--;
			}
            count++;
			itemAttributeList.add(itemAttribute);
			
			/*
			 * If no attribute is added for display, then display primary key.
			 * For Fix : WPC00028798
			 */
			if(itemAttributeList.size() == 0){
				
			   Iterator attributeMapIter = attributesMap.values().iterator();
			   while(attributeMapIter.hasNext()){
			   		
			   		Attribute tempAttr = (Attribute) attributeMapIter.next();
			   		if(tempAttr.isPrimaryKey()){
			   			itemAttributeList.add(tempAttr);
			   			break;
			   		}
			   }
			}
			
			// To avoid duplication of attributes in two separate calls to this method.
			// The attributes which are added in first method call will be removed from flat attribute Map.
			
			if(bRemove){
			    attributesMap.remove(itemAttribute.getName());
			}
        }
        return attributeIndex;
    }

    /**
     * @param portletSettings
     * @param catalogID
     */
    private Map getStoredAttributeMap(PortletData portletData, String catalogID) {

        // extracting attribute list from portlet data 
        String selectedAttributes = (String)portletData.getAttribute(catalogID);
        HashMap storedAttributesMap = new HashMap();
        
        if(UIUtils.isEmpty(selectedAttributes)){
            return storedAttributesMap;
        }
        
        StringTokenizer tokenizer = new StringTokenizer(selectedAttributes,
                IQuickSearch.CONFIG_ATTRIBUTE_TOKENIZER);
        while (tokenizer.hasMoreTokens()) {
            String stringToken = tokenizer.nextToken();
            int index = stringToken.indexOf(IQuickSearch.CONFIG_INDEX_TOKENIZER);
            String attributeName = stringToken.substring(0, index);
            Integer listBoxIndex = Integer.valueOf(stringToken.substring(index + 1));
            storedAttributesMap.put(attributeName, listBoxIndex);
        }

        return storedAttributesMap;
    }

    /**
     * @param attributesMap
     * @param itemAttributeList
     * @param primarykeyName
     */
    private void addPrimarykey(Map attributesMap, List itemAttributeList, String primarykeyName) {
        Attribute pkAttribute = (Attribute) attributesMap.get(primarykeyName);
        ItemAttribute itemAttributePk = new ItemAttribute();
        itemAttributePk.setAttribute(pkAttribute);
        itemAttributePk.setName(pkAttribute.getName());
        itemAttributePk.setIsDisplayable(true);
        itemAttributePk.setSelected(true);
        // Listbox Index is 0 
        itemAttributePk.setListBoxIndex(0);
        itemAttributeList.add(itemAttributePk);
    }

    /**
     * @param context
     * @param catalogId
     * @return
     * @throws UIException
     */
    private ICatalogItem getAttributes(WPSContext context, String catalogId)
        throws UIException {
        ArrayList attributesList = null;
        ICatalogItem catalogItem = null;
        Map attributesMap = null;

        try {
            WPCMediator mediator = WPCMediator.getInstance();

            catalogItem = mediator.getCatalogItem("", catalogId, context);

        } catch (WPCMediatorException e) {
            e.printStackTrace();
            throw new UIException(e);
        }

        return catalogItem;
    }
    /**
     * @param request
     */

    /**
     * @param request
     */
   /**
     * This method parses the string and returns the int value of string
     * @param sPaam
     * @param key
     * @param request
     * @return int
     * @throws UIException
     * @throws Exception
     */
    private int getValidatedIntValue(String sPaam, String key, PortletRequest request)
        throws UIException, Exception {
        MessageMap messageMap = null;
        int returnValue = 0;
        String colName = null;

        try {
            messageMap = getMessageMap(request);
            returnValue = Integer.parseInt(sPaam);
            if (returnValue <= 0) {
                messageMap.addErrorMessage("MESSAGE_CODE_517");
                throw new UIException();
            }
        } catch (NumberFormatException exception) {
            exception.printStackTrace();
            colName = getTextLabel(request, key);
            messageMap.addErrorMessage(IMassUpdate.MSG_INVALID_NUMBER_VALUE,
                colName);
            throw new UIException(exception);
        }

        return returnValue;
    }

    /**
     * Method called when user comes to Mass edit screen and Edit mode from Search.
     * This fetches all items passed from WPC and sets it in MassUpdatebean.
     * @param request
     * @return
     * @throws UIException
     * @throws PortletException
     */
    private MassUpdateBean getSearchedItemIds(PortletRequest request)
        throws UIException, PortletException {
        log.debug(" getSearchedItemIds ->");
        List catalogItemsList = null;
        Map catalogItemsRetMap = null;
        MassUpdateBean massUpdateBean = null;
        PortletSession session;

        session = request.getPortletSession();
        Map inputParamsMap = setInputParams(request);

        String itemIDStr = (String)inputParamsMap.get(ICatalogAction.ITEMID);
        String catalogID = (String)inputParamsMap.get(ICatalogAction.CATALOG);
        String itemSetforWorkflow = null;
        if (catalogID == null) {
            catalogID = (String)inputParamsMap.get(IMassUpdate.REQ_PARAM_KEY_CAT_WORKFLOW);
        }

        itemSetforWorkflow = (String)inputParamsMap.get(IMassUpdate.REQ_PARAM_KEY_CAT_WORKFLOW);
        
        List itemListConv = UIUtils.tokenizeString(itemIDStr, "-");

        if ((itemListConv == null) ||
                ((catalogID == null) || (itemListConv.size() == 1) ||
                catalogID.equals(""))) {
            session.setAttribute(IMassUpdate.KEY_MASS_UDATE_BEAN, null);

            return massUpdateBean;
        }

        PortletSettings portletSettings = request.getPortletSettings();
        String mxRows = portletSettings.getAttribute(IMassUpdate.KEY_MAX_ROWS);
        WPSContext context = getWPSContext(request);

        if ((mxRows == null) || mxRows.equals("")) {
            log.debug(
                "The attribute for Maximum items to be displayed is null or invalid");
            throw new PortletException();
        }

        massUpdateBean = (MassUpdateBean) session.getAttribute(IMassUpdate.KEY_MASS_UDATE_BEAN);

        if ((massUpdateBean == null) || !(massUpdateBean.getPrimaryCatalogName().equals(catalogID))) {
            massUpdateBean = new MassUpdateBean();
            populateMassUpdateBean(request, massUpdateBean);
        }
        try {
            int catalogItemsListSize = (itemListConv.size() > Integer.parseInt(mxRows))
                ? Integer.parseInt(mxRows) : itemListConv.size();
            itemListConv = itemListConv.subList(0, catalogItemsListSize);
            /*
             * invoking the MassUpdate Helper  
             */

            MassUpdateHelper helper = new MassUpdateHelper();
            	helper.setWpsContext(getWPSContext(request));
            	helper.setMaxItemRows(mxRows);
            	
           	catalogItemsRetMap = helper.getCatalogItems(massUpdateBean,itemListConv,catalogID, context);
            catalogItemsList = (List) catalogItemsRetMap.get(IReference.MASSUPDATE_MAP_LIST_KEY);

        } catch (WPCMediatorException we) {
            we.printStackTrace();
            getMessageMap(request).addErrorMessage("MESSAGE_CODE_106");
            handleException(we, request);

            return massUpdateBean;
        }

        massUpdateBean.setMode(IMassUpdate.UPDATE);
        massUpdateBean.setItemMap(getDisplayItemsMap(catalogItemsList));
        massUpdateBean.setCollabName(catalogID);
        massUpdateBean.setItemsSetforWorkflow(itemSetforWorkflow);
        massUpdateBean.setPortletRequestAttributes(inputParamsMap);
        return massUpdateBean;
    }
    /**
     * Setting Input params in Map stored in session.Input params will be 
     * used to fetch Items when switching from Edit mode to view mode. 
     * @param request
     * @return
     */
    private Map setInputParams(PortletRequest request) {
        /*
         * All portlet request parameters are stored back session
         * to ensure the request can be made again if User switches from edit mode
         * to view mode.
         */
        MassUpdateBean massUpdateBean = (MassUpdateBean) request.getPortletSession().getAttribute(IMassUpdate.KEY_MASS_UDATE_BEAN);
        
        if((massUpdateBean != null) && (massUpdateBean.getItemMap() == null)){
            return massUpdateBean.getPortletRequestAttributes();
        }else{
            String itemIDStr = request.getParameter(ICatalogAction.ITEMID);
            String catalogID = request.getParameter(ICatalogAction.CATALOG);
            String itemSetforWorkflow = request.getParameter(IMassUpdate.REQ_PARAM_KEY_CAT_WORKFLOW);
            Map map= new HashMap();
            map.put(ICatalogAction.ITEMID,itemIDStr);
            map.put(ICatalogAction.CATALOG,catalogID);
            map.put(IMassUpdate.REQ_PARAM_KEY_CAT_WORKFLOW,itemSetforWorkflow);
            return map;
        }
    }

    /**
     * Returns instance of Mediator
     * @return
     */
    private WPCMediator getMediator() {
        WPCMediator mediator = null;

        if (mediator == null) {
            mediator = WPCMediator.getInstance();
        }

        return mediator;
    }

    /**
     * This method initiates the processing of items updation.Basically,this method creates comma
     * separated string to send it to WPC for mass update.
     * @param context
     * @param request
     * @param csvString
     */
    private void processItems(WPSContext context, PortletRequest request,
        String csvString, Set itemIdset) {
        log.debug(" processItems ->");

        MessageMap messageMap = getMessageMap(request);
        MassUpdateBean massUpdateBean = (MassUpdateBean) request.getPortletSession()
                                                                .getAttribute(IMassUpdate.KEY_MASS_UDATE_BEAN);

        String fileName = "massedit_file.csv";

        FileUploadBean fileUploadBean = new FileUploadBean();

        fileUploadBean.setContext(context);
        fileUploadBean.setProtocol(IFileUpload.PROTOCOL_SOAP);
        fileUploadBean.setFileName(fileName);
        fileUploadBean.setCollabName(massUpdateBean.getCollabName());

        //set message for alerts portlet
        ByteArrayInputStream byteInputStream = null;

        try {
            byteInputStream = new ByteArrayInputStream(csvString.getBytes(
                        "UTF-8"));
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        FileUploadUtil fileUploadUtil = new FileUploadUtil(byteInputStream,
                fileUploadBean);
        massUpdateBean.getCollabName();

        try {
            String uploadResponse = fileUploadUtil.uploadFile();
        } catch (WPCMediatorException we) {
            String commaSepitemId = UIUtils.getCommaSeparatedId(itemIdset);
            Object[] values = new Object[1];
            values[0] = commaSepitemId.replace(',', ' ');

            String alertMessage = getAlertMessage(request, "MESSAGE_CODE_364",
                    values);

            request.setAttribute(ITransactionDisplay.TRANSACTION_DESCRIPTION_KEY,
                alertMessage);

            handleException(we, request);
            messageMap = getMessageMap(request);
            String commaSepitemIdMsg = commaSepitemId.trim().replaceAll(",",", ");
            //	set success message
            messageMap.addInfoMessage("MESSAGE_CODE_364",commaSepitemIdMsg);
        }

        log.debug(" processItems <- ");
    }

    /**
     * Ordrs the list according to user selection
     */
    
    protected List orderAttributes(List itemAttrList,int selectedAttributeSize) {
        ItemAttribute itemAttribute = null;
        int count = 0;
        int index = -1;
        int listBoxindex = -1;

        ItemAttribute selectedAttributesArr[] = new ItemAttribute[selectedAttributeSize];
        int emptyAttrPositionsinListArr[]  = new int[selectedAttributeSize];
        
        for(int k = 0,i = 0; i < itemAttrList.size(); i++) {
            itemAttribute = (ItemAttribute) itemAttrList.get(i);
            if (itemAttribute.getAttribute().isPrimaryKey()) {
                String primaryKey = itemAttribute.getName();
            }

            if (itemAttribute.isSelected()) {
                listBoxindex = itemAttribute.getListBoxIndex();

                if (listBoxindex >= 0) {
                    // Selected Attributes are copied in array selectedAttributesArr 
                    // in sorted order
                    selectedAttributesArr[listBoxindex] = itemAttribute;
                    
                    // The positions of selected Attributes in the list is set to null.Those 
                    // postions are made empty
                    itemAttrList.set(i,null);
                    
                    // The empty positions of selected Attributes in the list is 
                    // stored in the array selectedAttrPositionsinListArr 
                    emptyAttrPositionsinListArr[k++]= i;
                }
            }
        }
        // filling the empty positions in attributes objects from sorted order Array selectedAttributesArr  
        for (int j = 0; j < emptyAttrPositionsinListArr.length; j++) {
            itemAttrList.set(emptyAttrPositionsinListArr[j],selectedAttributesArr[j]);
        }
        return itemAttrList;
    }

    /**
     * This method returns the message used for alert portlet
     */
    private String getAlertMessage(PortletRequest request, String key,
        Object[] obj) {
        String message = null;
        String formattedMessage = null;
        message = getTextMessages(request, key);

        MessageFormat msgFormat = new MessageFormat(message);
        formattedMessage = msgFormat.format(obj);

        return formattedMessage;
    }
}
