
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.portlets.quicksearch;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.StringTokenizer;

import org.apache.jetspeed.portlet.AccessDeniedException;
import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletSession;
import org.apache.jetspeed.portlet.PortletSettings;
import org.apache.jetspeed.portlet.PortletURI;
import org.apache.jetspeed.portlet.UnavailableException;
import org.apache.jetspeed.portlet.event.ActionEvent;
import org.apache.jetspeed.portlet.event.ActionListener;

import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.exception.UIException;
import com.ibm.ccd.ui.mediator.WPCMediator;
import com.ibm.ccd.ui.mediator.WPCMediatorException;
import com.ibm.ccd.ui.portlets.BasePortlet;
import com.ibm.ccd.ui.portlets.GenericPortletMessage;
import com.ibm.ccd.ui.portlets.search.core.IBaseSearch;
import com.ibm.ccd.ui.portlets.search.core.SearchBean;
import com.ibm.ccd.ui.portlets.search.core.SearchCriteriaBean;
import com.ibm.ccd.ui.taglib.dataobjects.BaseSearchElement;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;
import com.ibm.ccd.ui.util.ValidationUtil;


/**
 *
 *
 */
public class QuickSearchPortlet extends BasePortlet implements ActionListener {
    /**
     *  contains portlet context
     */
    private PortletContext context = null;

    /**
     * @see org.apache.jetspeed.portlet.Portlet#init(PortletConfig)
     */
    public void init(PortletConfig portletConfig) throws UnavailableException {
        super.init(portletConfig);
        context = getPortletConfig().getContext();
    }

    /**
     * @see org.apache.jetspeed.portlet.PortletAdapter#doView(PortletRequest, PortletResponse)
     */
    public void doView(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        SearchCriteriaBean criteriaBean = null;
        MessageMap messageMap = getMessageMap(request);

        try {
            setSearchCriteriaBean(request);
        } catch (UIException exception) {
            exception.printStackTrace();
            messageMap.addErrorMessage(IBaseSearch.MSG_APPLICATION_EXCEPTION);
        }

        //	Invoke the JSP to render        		
        getPortletConfig().getContext().include(IQuickSearch.PATH_SEARCH_VIEW_JSP,
            request, response);
    }

    /**
     * @param request
     */
    private void setAttributeDisplay(PortletRequest request) {
        PortletSettings portletSettings = null;
        String selectedAttributes = null;
        StringTokenizer tokenizer = null;
        String stringToken = null;
        String attributeName = null;
        ArrayList attributeList = null;
        SearchCriteriaBean criteriaBean = null;
        BaseSearchElement element = null;
        String elementName = null;
        HashMap attributeMap = null;
        int index = -1;
        int count=0;
        Integer listBoxIndex = null;
        Object value = null;
        String catalogId=null;

		portletSettings = request.getPortletSettings();
		
        criteriaBean = (SearchCriteriaBean) request.getPortletSession()
                                                   .getAttribute(IQuickSearch.KEY_SEARCH_CRITERIA_BEAN);
        attributeList = criteriaBean.getAttributeList();
		catalogId=criteriaBean.getCatalogId();
        selectedAttributes = portletSettings.getAttribute(catalogId);

		if(!ValidationUtil.isNullOrEmpty(selectedAttributes)){
			for (int i = 0; i < attributeList.size(); i++) {
				element = (BaseSearchElement) attributeList.get(i);
				element.setIsDisplayable(false);
			}
       	tokenizer = new StringTokenizer(selectedAttributes,IQuickSearch.CONFIG_ATTRIBUTE_TOKENIZER);
       	attributeMap = new HashMap();

       	while (tokenizer.hasMoreTokens()) {
             stringToken = tokenizer.nextToken();
             index = stringToken.indexOf(IQuickSearch.CONFIG_INDEX_TOKENIZER);
              attributeName = stringToken.substring(0, index);
              listBoxIndex = Integer.valueOf(stringToken.substring(index + 1));
              attributeMap.put(attributeName, listBoxIndex);
      }

            for (int i = 0; i < attributeList.size(); i++) {
                element = (BaseSearchElement) attributeList.get(i);
                elementName = element.getName();
                value = attributeMap.get(elementName);

                if (value != null) {
                    listBoxIndex = (Integer) value;
                    element.setIsDisplayable(true);
                    element.setListBoxIndex(listBoxIndex.intValue());
                    count++;
                }
            }
            
			//If attribute is removed form spec re assign the index
			  if(count<attributeMap.size()){
				  int j=0;
				  for (int i = 0; i < attributeList.size(); i++) {
					  element = (BaseSearchElement) attributeList.get(i);
					  if(element.getIsDisplayable()){
						 element.setListBoxIndex(j++);
					  }
				  }
			 }
        }
    }

    /**
     * @param request
     */
    private void setSearchCriteriaBean(PortletRequest request)
        throws UIException {
        WPCMediator mediator = null;
        PortletSession session = null;
        SearchCriteriaBean searchCriteriaBean = null;
        SearchBean searchBean = null;
        BaseSearchElement searchElement = null;
        WPSContext wpsContext = null;
        ArrayList criteriaAttributes = null;
		String catalogID = null;
		String currentCatalogId=null;
		boolean removeCache=false;

        session = (PortletSession) request.getSession();
		wpsContext = getWPSContext(request);
		searchCriteriaBean = (SearchCriteriaBean) session.getAttribute(IQuickSearch.KEY_SEARCH_CRITERIA_BEAN);
//		Checkif the SearchCriteriaBean is cached for diffrent catalog than current catalog. 
			 currentCatalogId=UIUtils.getCatalogName(wpsContext.getWpcUser());
			 if(searchCriteriaBean!=null){
				 catalogID=searchCriteriaBean.getCatalogId();	
				 if(!catalogID.equals(currentCatalogId)){
					 removeCache=true;
				 }
			 }

			if (searchCriteriaBean == null || removeCache ) {
            //get attribute list
            mediator = WPCMediator.getInstance();
            searchBean = new SearchBean();
            searchBean.setContext(context);
            searchBean.setPortletRequest(request);
            searchBean.setWpsContext(wpsContext);

            try {
                criteriaAttributes = searchBean.getSearchCriteriaElements();

                //	set the values
				searchCriteriaBean = new SearchCriteriaBean();
				searchCriteriaBean.setAttributeList(criteriaAttributes);
				searchCriteriaBean.setCatalogId(currentCatalogId);
                session.setAttribute(IQuickSearch.KEY_SEARCH_CRITERIA_BEAN,
				searchCriteriaBean);

                //set display flag
                setAttributeDisplay(request);
            } catch (WPCMediatorException e) {
                e.printStackTrace();
                handleException(e, request);
            }
        }
    }

    /**
     * @see org.apache.jetspeed.portlet.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent event) throws PortletException {
        PortletRequest request = null;
        MessageMap messageMap = null;

        String action = null;
        request = event.getRequest();
        action = event.getActionString();
        messageMap = getMessageMap(request);

        if (IQuickSearch.FORM_ACTION_CONFIG.equals(action)) {
            try {
                updateCriteria(request);
                messageMap.addInfoMessage(IBaseSearch.MSG_CONFIG_UPDATE_SUCCESS);
            } catch (UIException e) {
                e.printStackTrace();
                messageMap.addErrorMessage(IQuickSearch.MSG_APPLICATION_EXCEPTION);
            }
        }
    }

    /**
     * @param request
     */
    private void updateCriteria(PortletRequest request)
        throws UIException {
        PortletSession portletSession = null;
        PortletSettings portletSettings = null;
        SearchCriteriaBean criteriaBean = null;
        BaseSearchElement searchElement = null;
        ArrayList criteriaList = null;
        String[] criteriaAttributes = null;
        String attributeName = null;
        String elementName = null;
        StringBuffer selectedAttributes = null;
        int listBoxIndex = 0;
        String catalogId=null;
		int size = 0;
		
        portletSettings = request.getPortletSettings();
		portletSession = request.getPortletSession();
        
        criteriaAttributes = request.getParameterValues(IQuickSearch.ELEMENT_SEARCH_CRITERIA);
        criteriaBean = (SearchCriteriaBean) portletSession.getAttribute(IQuickSearch.KEY_SEARCH_CRITERIA_BEAN);
        criteriaList = criteriaBean.getAttributeList();
        size = criteriaList.size();
		catalogId=criteriaBean.getCatalogId();
		
        //set display flag to false
        for (int i = 0; i < size; i++) {
            searchElement = (BaseSearchElement) criteriaList.get(i);
            searchElement.setIsDisplayable(false);
        }

        selectedAttributes = new StringBuffer();

        //update search criteria
        size = criteriaList.size();

        for (int i = 0; i < criteriaAttributes.length; i++) {
            attributeName = criteriaAttributes[i];

            for (int j = 0; j < size; j++) {
                searchElement = (BaseSearchElement) criteriaList.get(j);
                elementName = searchElement.getName();

                if (elementName.equals(attributeName)) {
                    searchElement.setIsDisplayable(true);
                    searchElement.setListBoxIndex(i);

                    //add to setting string
                    selectedAttributes.append(attributeName);
                    selectedAttributes.append(IQuickSearch.CONFIG_INDEX_TOKENIZER);
                    selectedAttributes.append(i);
                    selectedAttributes.append(IQuickSearch.CONFIG_ATTRIBUTE_TOKENIZER);

                    break;
                }
            }
        }

        try {
            portletSettings.setAttribute(catalogId,
                selectedAttributes.toString());
            portletSettings.store();
			portletSession.setAttribute(IQuickSearch.KEY_SEARCH_CRITERIA_BEAN,
                null);
        } catch (AccessDeniedException e) {
            e.printStackTrace();
            throw new UIException(e);
        } catch (IOException e1) {
            e1.printStackTrace();
            throw new UIException(e1);
        }
    }

    /**
     * Handles the Config mode event
     */
    public void doConfigure(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        PortletURI formActionURI = null;
        MessageMap messageMap = null;
        QuickSearchConfigBean configBean = null;
        messageMap = getMessageMap(request);

        try {
            setSearchCriteriaBean(request);
            formActionURI = response.createURI();
            formActionURI.addAction(IQuickSearch.FORM_ACTION_CONFIG);
            configBean = new QuickSearchConfigBean();
            configBean.setFormActionURI(formActionURI.toString());
            request.setAttribute(IQuickSearch.KEY_SEARCH_CONFIG_BEAN, configBean);
        } catch (UIException exception) {
            exception.printStackTrace();
            messageMap.addErrorMessage(IBaseSearch.MSG_APPLICATION_EXCEPTION);
        }

        //add info message
        messageMap.addInfoMessage(IBaseSearch.MSG_SELECT_QUICK_SEARCH);

        //Invoke the JSP to render        		
        getPortletConfig().getContext().include(IQuickSearch.PATH_SEARCH_CONFIG_JSP,
            request, response);
    }

    /**
     * sets the business function in <code>PortletSession</code> object
     */
    protected void setBusinessFunction(PortletRequest request) {
        PortletSession portletSession = null;
        portletSession = request.getPortletSession();
        portletSession.setAttribute(IReference.SESSION_BUS_FUNCT,
            IQuickSearch.BUSINESS_FUNCTION_QUICK_SEARCH);
    }

    /**
     * Method
     * @param portlet
     * @param GenericPortletMessage
     *
     */
    public void sendMessage(String portlet, GenericPortletMessage message)
        throws AccessDeniedException {
        context.send(null, message);
    }

    /**
     * Handles the Portlet help events
     * @param PortletRequest , PortletResponse
     */
    public void doHelp(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        getPortletConfig().getContext().include(IQuickSearch.PATH_SEARCH_HELP_JSP,
            request, response);
    }
}
