
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.portlets.search.core;

import java.io.IOException;

import org.apache.jetspeed.portlet.Portlet;
import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletSession;
import org.apache.jetspeed.portlet.UnavailableException;
import org.apache.jetspeed.portlet.event.ActionEvent;
import org.apache.jetspeed.portlet.event.ActionListener;

import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.exception.UIException;
import com.ibm.ccd.ui.mediator.WPCMediatorException;
import com.ibm.ccd.ui.portlets.BasePortlet;
import com.ibm.ccd.ui.portlets.search.core.cmd.GetFirstResultPageCmd;
import com.ibm.ccd.ui.portlets.search.core.cmd.GetLastResultPageCmd;
import com.ibm.ccd.ui.portlets.search.core.cmd.GetNextResultPageCmd;
import com.ibm.ccd.ui.portlets.search.core.cmd.GetPrvResultPageCmd;
import com.ibm.ccd.ui.portlets.search.core.cmd.ISearchCommand;
import com.ibm.ccd.ui.portlets.search.core.cmd.PerformSearchCmd;
import com.ibm.ccd.ui.portlets.search.core.cmd.PopulateConfigDataStoreCmd;
import com.ibm.ccd.ui.portlets.search.core.cmd.PopulateEditDataStoreCmd;
import com.ibm.ccd.ui.portlets.search.core.cmd.PopulateViewDataStoreCmd;
import com.ibm.ccd.ui.portlets.search.core.cmd.SearchFromHistoryCmd;
import com.ibm.ccd.ui.portlets.search.core.cmd.UpdateConfigSettingsCmd;
import com.ibm.ccd.ui.portlets.search.core.cmd.UpdateSearchCriteriaCmd;
import com.ibm.ccd.ui.util.IReference;


/**
 *
 *
 */
public abstract class BaseSearchPortlet extends BasePortlet
    implements ActionListener {
    /**
     *  contains portlet context
     */
    private PortletContext context = null;

    /**
     * @see org.apache.jetspeed.portlet.Portlet#init(PortletConfig)
     */
    public void init(PortletConfig portletConfig) throws UnavailableException {
        super.init(portletConfig);
        context = getPortletConfig().getContext();
    }

    /**
     * @see org.apache.jetspeed.portlet.PortletAdapter#doView(PortletRequest, PortletResponse)
     */
    public void doView(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        PortletSession portletSession = null;
        ISearchCommand command = null;
        Portlet.Mode mode = Portlet.Mode.VIEW;
        portletSession = request.getPortletSession();

        //set business function 
        setBusinessFunction(request);
        portletSession.setAttribute(IReference.SESSION_ACTION,
            IBaseSearch.BUSINESS_ACTION_VIEW);

        //execute command
        command = getDataStore(mode);

        executeCommand(command, request, response);

        //Invoke the JSP to render        		
        render(request, response, mode);
    }

    protected abstract void render(PortletRequest request,
        PortletResponse response, Portlet.Mode mode)
        throws PortletException, IOException;

    protected ISearchCommand getDataStore(Portlet.Mode mode) {
        ISearchCommand command = null;

        if (Portlet.Mode.VIEW.equals(mode)) {
            command = new PopulateViewDataStoreCmd();
        }

        if (Portlet.Mode.EDIT.equals(mode)) {
            command = new PopulateEditDataStoreCmd();
        }

        if (Portlet.Mode.CONFIGURE.equals(mode)) {
            command = new PopulateConfigDataStoreCmd();
        }

        if (Portlet.Mode.HELP.equals(mode)) {
        }

        return command;
    }

    /**
     * @see org.apache.jetspeed.portlet.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent event) throws PortletException {
        PortletRequest request = null;
        PortletSession portletSession = null;
        WPSContext wpsContext = null;
        String actionString = null;
        ISearchCommand command = null;

        actionString = event.getActionString();
        request = event.getRequest();
        portletSession = request.getPortletSession();
        wpsContext = getWPSContext(request);

        //used in Jsp to switch to the result tab						
        request.setAttribute("stabNo", "2");

        command = getAction(actionString);

        //execute the command
        executeCommand(command, request);
    }

    /**
     * @param command
     * @param request
     */
    protected void executeCommand(ISearchCommand command, PortletRequest request) {
        executeCommand(command, request, null);
    }

    /**
     * Handles the Edit mode event
     */
    public void doEdit(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        ISearchCommand command = null;
        PortletSession portletSession = null;
        Portlet.Mode mode = Portlet.Mode.EDIT;
        portletSession = request.getPortletSession();

        //set business function 
        setBusinessFunction(request);
        portletSession.setAttribute(IReference.SESSION_ACTION,
            IBaseSearch.BUSINESS_ACTION_EDIT);

        //execute command
        command = getDataStore(mode);
        executeCommand(command, request, response);

        // Invoke the JSP to render        		
        render(request, response, mode);
    }

    /**
     * Handles the Config mode event
     */
    public void doConfigure(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        ISearchCommand command = null;
        PortletSession portletSession = null;
        MessageMap messageMap = null;
        Portlet.Mode mode = Portlet.Mode.CONFIGURE;

        setBusinessFunction(request);
        portletSession = request.getPortletSession();
        portletSession.setAttribute(IReference.SESSION_ACTION,
            IBaseSearch.BUSINESS_ACTION_CONFIG);
        command = getDataStore(mode);
        executeCommand(command, request, response);

        //Invoke the JSP to render        		
        render(request, response, mode);
    }

    /**
     * sets the business function in <code>PortletSession</code> object
     */
    protected void setBusinessFunction(PortletRequest request) {
        PortletSession portletSession = null;
        portletSession = request.getPortletSession();
        portletSession.setAttribute(IReference.SESSION_BUS_FUNCT,
            IBaseSearch.BUSINESS_FUNCTION_SEARCH);
    }

    /**
             * Handles the Portlet help events
             * @param PortletRequest , PortletResponse
             */
    public void doHelp(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        Portlet.Mode mode = Portlet.Mode.HELP;
        render(request, response, mode);
    }

    protected void executeCommand(ISearchCommand command,
        PortletRequest request, PortletResponse response) {
        MessageMap messageMap = getMessageMap(request);
        command.setWpsContext(getWPSContext(request));
        command.setPortletContext(getPortletConfig().getContext());
        command.setPortletName(getPortletConfig().getServletName());

        try {
            command.execute(request, response);
        } catch (UIException e) {
			e.printStackTrace();
			if(e.getMessageMap()==null){
				messageMap.addErrorMessage(IBaseSearch.MSG_APPLICATION_EXCEPTION);	
			}else{
				MessageMap exceptionMessage=e.getMessageMap();
				messageMap.copyMessageMap(exceptionMessage);
			}
            return;
        } catch (WPCMediatorException e) {
            handleException(e, request);
        }
    }

    protected void includeJSP(String jspName, PortletRequest request,
        PortletResponse response) throws PortletException, IOException {
        //	Invoke the JSP to render        		
        getPortletConfig().getContext().include(jspName, request, response);
    }

    protected ISearchCommand getAction(String actionString) {
        ISearchCommand command = null;

        if (IBaseSearch.FORM_ACTION_PERFORM_SEARCH.equals(actionString)) {
            command = new PerformSearchCmd();
        }

        if (IBaseSearch.FORM_ACTION_EDIT_SEARCH_CRITERIA.equals(actionString)) {
            command = new UpdateSearchCriteriaCmd();
        }

        if (IBaseSearch.FORM_ACTION_CONFIG.equals(actionString)) {
            command = new UpdateConfigSettingsCmd();
        }

        if (IBaseSearch.FORM_ACTION_SAVED_SEARCH.equals(actionString)) {
            command = new SearchFromHistoryCmd();
        }

        if (ICollectionViewBean.GET_NEXT_RESULTS.equals(actionString)) {
            command = new GetNextResultPageCmd();
        }

        if (ICollectionViewBean.GET_FIRST_RESULTS.equals(actionString)) {
            command = new GetFirstResultPageCmd();
        }

        if (ICollectionViewBean.GET_LAST_RESULTS.equals(actionString)) {
            command = new GetLastResultPageCmd();
        }

        if (ICollectionViewBean.GET_PREVIOUS_RESULTS.equals(actionString)) {
            command = new GetPrvResultPageCmd();
        }

        return command;
    }
}
