
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.portlets.search.core;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.StringTokenizer;

import com.ibm.ccd.ui.exception.UIException;
import com.ibm.ccd.ui.taglib.dataobjects.ResultData;
import com.ibm.ccd.ui.taglib.dataobjects.ResultMetaData;
import com.ibm.ccd.ui.taglib.dataobjects.ResultRowData;
import com.ibm.ccd.ui.util.ValidationUtil;


/**
 * This class is used to parse the CSV text with specified delimiters.
 * The delimiters are are user defined.
 */
public class CSVParser {
    private static final String SPACE = " "; //$NON-NLS-1$
    ArrayList metaDataList = null;

    /** Construct a CSV parser, with the default separator (`~~~~~~'). */
    /**
     * This method parses the CSV text
     *
     * @param xmlString is parsed to get the desired CSV text
     * @return ResultData object
     * @throws UIException
     */
    public ResultData parse(String xmlString) throws UIException {
        SearchResultParser searchResultParser = null;
        ResultData resultData = null;
        ArrayList dataList = null;
        String metaData = null;
        String data = null;
        String sSize=null;
        long lSize=0;

        searchResultParser = new SearchResultParser();
        searchResultParser.parse(xmlString);
        metaData = searchResultParser.getMetaData();
        data = searchResultParser.getData();

        if ((metaData != null) && (data != null)) {
            resultData = new ResultData();

            //set metat data list
            metaDataList = getMetaDataList(metaData);
            resultData.setResultMetaData(metaDataList);

            //set data list
            dataList = getDataList(data);
            resultData.setResultRowData(dataList);
			sSize=searchResultParser.getTotalResultSize().toString();
			if(((!ValidationUtil.isNullOrEmpty(sSize)))){
				if(!"0".equals(sSize.trim())){
					lSize=Long.parseLong(sSize);
					resultData.setTotalResultSize(lSize);	
				}
				
			}
        }
		
        return resultData;
    }

    private ArrayList getDataList(String data) {
        ArrayList dataList = null;
        ResultRowData resultRowData = null;
        String[] rows = null;
        String row = null;

        rows = data.split(ICollectionViewBean.ROW_DELIMITER); /* NOI18N */
        dataList = new ArrayList();
        if(rows!=null){
        	for (int i=0;i<rows.length;i++) { /* NOI18N */
        		row = rows[i]; /* NOI18N */
        		resultRowData = constructResultRow(row);
        		dataList.add(resultRowData);
        	}
        }
        return dataList;
    }

    private ResultRowData constructResultRow(String row) {
        ResultMetaData resultMetaData = null;
        ResultRowData resultRowData = null;
        HashMap map = new HashMap();
        String[] columns = null;
        String columnName = null;
        String columnValue = null;
        int index = 0;

        columns = row.split(ICollectionViewBean.COL_DELIMITER); /* NOI18N */
        if(columns!=null){	
        	for (int i=0;i<columns.length;i++) { /* NOI18N */
        		resultMetaData = (ResultMetaData) metaDataList.get(index);
        		columnName = resultMetaData.getName();
        		columnValue = columns[i]; /* NOI18N */

        		if (!ValidationUtil.isNullOrEmpty(columnValue)) {
        			map.put(columnName, columnValue);
        		} else {
        			map.put(columnName, SPACE);
        		}
        		index++;
        	}
        }

        resultRowData = new ResultRowData();
        resultRowData.setRowReference(null);
        resultRowData.setRowElements(map);

        return resultRowData;
    }

    /**
     * This method returns the metaDataList for the ResultData
     *
     * @param metaDataString used for creating the table headings.
     * @return metaDataList
     */
    public ArrayList getMetaDataList(String metaData) {
        ResultMetaData resultMetaData = null;
        String [] st=null;
        st=metaData.split(ICollectionViewBean.COL_DELIMITER);
        ArrayList metaDataList = new ArrayList();
        
        if(st!=null){	
        	for(int i=0 ; i<st.length;i++) { /* NOI18N */
        		resultMetaData = new ResultMetaData();
        		resultMetaData.setName(st[i]); /* NOI18N */
        		resultMetaData.setType(1);
        		resultMetaData.setSortable(false);
        		resultMetaData.setDisplayable(true);
        		metaDataList.add(resultMetaData);
        	}
        }
        return metaDataList;
    }
}
