
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * Created on Jan 13, 2005
 */
package com.ibm.ccd.ui.portlets.search.core;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;

import com.ibm.ccd.ui.exception.UIException;
import com.ibm.ccd.ui.util.ValidationUtil;


/**
 * This class provides the implementaion for interface <code>ISaveSearch</code>
 * Data is stored in file by using object serialisation
 */
public class FileSavedSearch implements ISaveSearch {
    public static final String SAVE_SEARCH_FILENAME_PREFIX = "SearchHistory_";
    public static final String SAVE_SEARCH_FILENAME_SUFFIX = ".ser";

    /**
     * contains a unique identifier for user.
     * used for persisting user data
     */
    private String userId = null;

    /**
     * contains the folder path to store files in case of file saved search
     */
    private String path = null;

    /**
     * constructor
     */
    public FileSavedSearch() {
        super();
    }

    public void saveSearch(ArrayList searchData) throws UIException {
        String fileName = null;
        String completePath = null;

        if (ValidationUtil.isNullOrEmpty(userId)) {
            throw new UIException("User Id not found");
        }

        fileName = userId + SAVE_SEARCH_FILENAME_SUFFIX;
        completePath = path + File.separator + fileName;

        try {
            FileOutputStream fileOut = new FileOutputStream(completePath);
            ObjectOutputStream out = new ObjectOutputStream(fileOut);

            out.writeObject(searchData);
            out.close();
            fileOut.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            throw new UIException(e);
        } catch (IOException e) {
            e.printStackTrace();
            throw new UIException(e);
        }
    }

    public ArrayList loadSearch() throws UIException {
        ArrayList savedSearch = null;
        FileInputStream fileIn = null;
        String fileName = null;
        String completePath = null;

        File file = null;

        if (ValidationUtil.isNullOrEmpty(userId)) {
            throw new UIException("User Id not found");
        }

        fileName = userId + SAVE_SEARCH_FILENAME_SUFFIX;
        completePath = path + File.separator + fileName;

        try {
            fileIn = new FileInputStream(completePath);
        } catch (FileNotFoundException e) {
            try {
                file = new File(completePath);
                file.createNewFile();
                fileIn = new FileInputStream(completePath);
            } catch (IOException e1) {
                e1.printStackTrace();
                throw new UIException(e1);
            }
        }

        try {
            ObjectInputStream in = new ObjectInputStream(fileIn);
            savedSearch = (ArrayList) in.readObject();
            in.close();
            fileIn.close();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
            throw new UIException(e);
        } catch (IOException e) {
            e.printStackTrace();
        }

        return savedSearch;
    }

    /**
     * @return
     */
    public String getUserId() {
        return userId;
    }

    /**
     * @param string
     */
    public void setUserId(String string) {
        userId = string;
    }

    /**
             * @return the folder path
             */
    public String getFolder() {
        return path;
    }

    /**
     * Set the folder name which stores all files
     */
    public void setFolder(String folder) {
        path = folder;
    }
}
