
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * Created on Dec 2, 2004
 */
package com.ibm.ccd.ui.portlets.search.core;

import java.util.ArrayList;

import com.ibm.ccd.ui.exception.UIException;
import com.ibm.ccd.ui.taglib.dataobjects.ResultData;


/**
 * This interface lists the methods for search criteria and search results
 */
public interface IBaseSearch {
    /**
     * Indicates the UI control of type text
     */
    String TYPE_TEXT = "TEXT";

    /**
     * Indicates the UI control of type select
     */
    String TYPE_SELECT = "SELECT";

    /**
     *Indicates the form acotion for serach edit jsp
     */
    String FORM_ACTION_EDIT_SEARCH_CRITERIA = "FormActionEditSearchCriteria";

    /**
     *Indicates the form acotion for serach view jsp
     */
    String FORM_ACTION_VIEW = "FormActionView";

    /**
     * Indicates the form acotion for serach view jsp
     */
    String FORM_ACTION_CONFIG = "FormActionConfig";

    /**
     * Indicates the key for storing <code>SearchCriteriaViewBean</code>
     */
    String KEY_SEARCH_CRITERIA_BEAN = "SearchCriteriaBean";

    /**
     * Indicates the business function <i>search criteria</i>
     */
    String BUSINESS_FUNCTION_SEARCH = "SEARCH";

    /**
     * indiccates the application mode
     */
    String BUSINESS_ACTION_VIEW = "VIEW";

    /**
     * indiccates the application mode
     */
    String BUSINESS_ACTION_EDIT = "EDIT";

    /**
     * indiccates the application mode
     */
    String BUSINESS_ACTION_CONFIG = "CONFIG";

    /**
     * Indicates the key for storing max no of columns allowed for search
     */
    String MAX_SEARCH_COLS = "MAX_SEARCH_COLS";

    /**
     *Indicates constant for max search results to be returned
     */
    String MAX_SEARCH_RESULTS = "MAX_SEARCH_RESULTS";
	String MAX_TRAVERSAL_SIZE="MAX_TRAVERSAL_SIZE";

    //	Section Added by Mahesh begins

    /**
     *Indicates the form action for serach result edit jsp
     */
    String SEARCH_RESULT_FORM_ACTION_EDIT = "SearchResultFormActionEdit";

    /**
     *Indicates the form action for serach result view jsp
     */
    String SEARCH_RESULT_FORM_ACTION_VIEW = "SearchResultFormActionView";

    /**
     * Indicates the key for storing <code>SearchResultsViewBean</code>
     */
    String KEY_SEARCH_RESULTS_BEAN = "SearchResultBean";

    /**
     * Indicates the form action
     */
    String FORM_ACTION_PERFORM_SEARCH = "performSearch";

    /**
     * Indicates the form action
     */
    String FORM_ACTION_WORK_WITH_ITEM = "work_with_item";

    /**
     * indicates the form action for saved search
     */
    String FORM_ACTION_SAVED_SEARCH = "SAVED_SEARCH";

    /**
     * indicates the form action for quick search
     */
    String FORM_ACTION_QUICK_SEARCH = "QUICK_SEARCH";
    
    /**
     * key used as parameter name in urlGenration tag
     */
	String KEY_ACTION="ACTION";

    /**
     *  key for filed name fleld
     */
    String LABEL_SCROLL_SIZE = "SCROLL_SIZE";

    /**
     * key for button label
     */
    String LABEL_CONFIG_SUBMIT = "UPDATE";

    /**
     * Indicates the key for storing <code>SearchConfigBean</code>
     */
    String KEY_SEARCH_CONFIG_BEAN = "SEARCH_CONFIG_BEAN";

    /**
     *  Indicates the message code
     */
    String MSG_INVALID_NUMBER_VALUE = "MESSAGE_CODE_111";

    /**
     * Indicates message code for info on edit page
     */
    String MSG_EDIT_INFO = "MESSAGE_CODE_151";

    /**
     * Indicates message code for data updation success
     */
    String MSG_EDIT_UPDATE_SUCCESS = "MESSAGE_CODE_155";

    /**
     * Indicates message code for data updation success
     */
    String MSG_CONFIG_UPDATE_SUCCESS = "MESSAGE_CODE_156";

    /**
     * Indicates message code for max fetched results
     */
    String MSG_MAX_FETCHED_RESULTS = "MESSAGE_CODE_157";

    /**
     * Indicates the message code for application exception message
     */
    String MSG_APPLICATION_EXCEPTION = "MESSAGE_CODE_105";

    /**
     * Indicates the message code for application exception message
     */
    String MSG_SYSTEM_EXCEPTION = "MESSAGE_CODE_106";

    /**
     * Indicates the message code for processing failur
     */
    String MSG_TRY_LATER = "MESSAGE_CODE_100";

    /**
     * Indicates key for search history messgae
     */
    String MSG_SEARCH_HISTORY = "MESSAGE_CODE_153";

    /**
     * Indicates key for search history messgae
     */
    String MSG_SEARCH_CRITERIA = "MESSAGE_CODE_154";

    /**
     * Displays message that no search criteria is entered by user
     */
    String MSG_NO_SEARCH_CRITERIA = "MESSAGE_CODE_158";

    /**
     * indicates the message user has not selected any search critria
     */
    String MSG_SEARCH_ATTRIBUTES_NOT_SELECTED = "MESSAGE_CODE_159";

    /**
     *Indicates the message to slect search criteria
     */
    String MSG_SELECT_QUICK_SEARCH = "MESSAGE_CODE_160";
	String MSG_INVALID_TRAVERSAL_SIZE="MESSAGE_CODE_161";

    /**
     * Indicates prefix for search result column 
     */
    String PREFIX_SEARCH_RESULT = "SEARCH_RESULT_";
    
    /**
     * Indicates prefix for search criteria column
     */
	String PREFIX_SEARCH_CRITERIA = "SEARCH_CRITERIA_";
	
    /**
     * Key for storing traversal size
     */
    String KEY_TRAVERSAL_SIZE = "TRAVERSAL_SIZE";

    /**
     * Indicates the default traversal size
     */
    String DEFAULT_TRAVERSAL_SIZE = "10";

    /**
     *  start of search tag
     */
    String WPC_SEARCH_ELEMENT_START_TAG = "<wpc:wpcSearchElement>";

    /**
     *  end of search tag
     */
    String WPC_SEARCH_ELEMENT_END_TAG = "</wpc:wpcSearchElement>";

    /**
     *  search Attribute start tag
     */
    String WPC_SEARCH_ATTRIBUTE_START_TAG = "<wpc:wpcAttributePath>";

    /**
     *   search Attribute end tag
     */
    String WPC_SEARCH_ATTRIBUTE_END_TAG = "</wpc:wpcAttributePath>";

    /**
     *   search value start tag
     */
    String WPC_SEARCH_VALUE_START_TAG = "<wpc:wpcAttributeValue>";

    /**
     *   search value end tag
     */
    String WPC_SEARCH_VALUE_END_TAG = "</wpc:wpcAttributeValue>";

    /**
     *   search element operator start tag
     */
    String WPC_SEARCH_ELEMENT_OPERATOR_START_TAG = "<wpc:wpcSearchElementOperator>";

    /**
     *   search element operator end tag
     */
    String WPC_SEARCH_ELEMENT_OPERATOR_END_TAG = "</wpc:wpcSearchElementOperator>";

    /**
     *   search value start tag
     */
    String WPC_SEARCH_PREDICATE_START_TAG = "<wpc:wpcAttributeNotPredicate>";

    /**
     *   search opperator end tag
     */
    String WPC_SEARCH_PREDICATE_END_TAG = "</wpc:wpcAttributeNotPredicate>";

    /**
     *   search operator start tag
     */
    String WPC_SEARCH_OPERATOR_START_TAG = "<wpc:wpcSearchOperator  name=\"AND\"> ";
   
    /**
     *   search operator start tag
     */    
    String WPC_SEARCH_OPERATOR_OR_START_TAG = "<wpc:wpcSearchOperator  name=\"OR\"> ";

    /**
     *   search operator end tag
     */
    String WPC_SEARCH_OPERATOR_END_TAG = "</wpc:wpcSearchOperator>";

    /**
     *   search logical operator AND
     */
    String AND_OPERATOR = "AND";

    /**
     * indicates path of help jsp
     */
    String WORK_WITH_ITEM_PORLET_NAME = "WorkwithItemPortlet";

    /**
     * Indicates the name of text operator
     */
    String OP_BEGINSWITH = "BEGINSWITH";

    /**
     * Indicates the name of text operator
     */
    String OP_BEGINSWITH_NOT = "BEGINSWITH_NOT";

    /**
     * Indicates the name of text operator
     */
    String OP_CONTAINS = "CONTAINS";

    /**
     * Indicates the name of text operator
     */
    String OP_CONTAINS_NOT = "CONTAINS_NOT";

    /**
     *Indicates the name of text operator
     */
    String OP_ENDSWITH = "ENDSWITH";

    /**
     *Indicates the name of text operator
     */
    String OP_ENDSWITH_NOT = "ENDSWITH_NOT";

    /**
     *Indicates the name of text operator
     */
    String OP_ISEMPTY = "ISEMPTY";

    /**
     *Indicates the name of text operator
     */
    String OP_ISEMPTY_NOT = "ISEMPTY_NOT";

    /**
     *Indicates the name of text operator
     */
    String OP_EXACTMATCH = "EXACTMATCH";

    /**
     *Indicates the name of text operator
     */
    String OP_EXACTMATCH_NOT = "EXACTMATCH_NOT";
    
    /**
     * Indicates possible value of flag attribute
     */
	String FLAG_TRUE="TRUE";
	
	/**
	 * Indicates possible value of flag attribute
	 */
	String FLAG_FALSE="FALSE";
		

    /**
     * Indicates the prefix for keys in properties files
     */
    String PREFIX_LOCALISATION = "SEARCH.VIEW.";

    /**
     * Indicates the default text operator
     */
    String DEFAULT_TEXT_OPERATOR = OP_CONTAINS;

    /**
     * Indicates the default text operator
     */
    String DEFAULT_NUMERIC_OPERATOR = "=";

    /**
     * Indicates the default boolean operator
     */
    String DEFAULT_BOOLEAN_OPERATOR = OP_EXACTMATCH;

    /**
     *Indicates the default date operator
     */
    String DEFAULT_DATE_OPERATOR = OP_EXACTMATCH;

    /**
     * indicates the key for saved search in <code>PortletRequest</code> Object
     */
    String KEY_SAVED_SEARCH_KEY = "SAVED_SEARCH_KEY";

    /**
     * Indicats tyhe name of class for file implementation of nterface <code>Isavesearch</code>
     */
    String CLASS_NAME_FILE_SAVED_SEARCH = "com.ibm.ccd.ui.portlets.search.core.FileSavedSearch";

    /**
     * Indicates the key for storing max saved searchs limit in <code>PortletSettings</code>
     */
    String KEY_MAX_SAVED_SEARCHS = "MAX_SAVED_SEARCHS";

    /**
     * Indicates the seperator used in search URL String
     */
    String URL_STRING_SEPERATOR = ", ";

    /**
     * indcates the key for search ID in <code>PortletRequest</code> object
     */
    String KEY_SEARCH_ID = "ID";

    /**
     * Indicates the constant for retriing label
     */
    String MAX_SEARCH_HISTORY = "MAX_SEARCH_HISTORY";

    /**
     * name of folder for file saved search history
     */
    String HISTORY_FOLDER_NAME = "SearchHistory_";

    /**
     * relative path of foler from servlet real path
     */
	String RELATIVE_FOLDER_PATH = "/WEB-INF/";

    /**
     * indicates the lable for "And"
     */
    String LABEL_VIEW_AND = "SEARCH.VIEW.AND";

    /**
     * label for download button on result tab
     */
    String LABEL_DOWNLOAD = "SEARCH.VIEW.DOWNLOAD";

    /**
     * Name of HTML SELECT contol for search Criteria
     */
    String KEY_SELECT_SEARCH_CRITERIA = "searchCriteria";

    /**
     * Name of HTML SELECT contol for search Result
     */
    String KEY_SELECT_SEARCH_RESULT = "searchResult";
    
    /**
     * Indicates the key for 'None'
     */
	String LABEL_KEY_NONE="DROPDOWN_VALUE_NONE";

    /**
     * This methods returns the list of <code>BasicSearchElements<code> containig the
     * search criteria.
     * @return ArraList
     */
    public ArrayList getSearchCriteriaElements() throws Exception;

    public ResultData getSearchResults() throws UIException;

    public String getReqXML();

    public void setReqXML(String string);
}
