
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.portlets.search.core;

import java.util.ArrayList;


/**
 * The Interface every view bean implements for collection display
 * functionality
 * This interface is to be implemented if the view bean has to display a
 * collection of data. This would mean that the View bean understand how to
 * provide the data that is required for displaying a collection, and also work
 * with the actions associated with the collection
 */
public interface ICollectionViewBean {
    /**
     * The action name for performing the sort.
     */
    String SORT = "SORT"; //$NON-NLS-1$

    /**
     * Sorting order is ascending
     */
    String ASCENDING = "ASCENDING"; //$NON-NLS-1$

    /**
     * Sorting order is descending
     */
    String DESCENDING = "DESCENDING"; //$NON-NLS-1$

    /**
     * Action to get the next set of values in the collection
     */
    String GET_NEXT_RESULTS = "get_next_results"; //$NON-NLS-1$

    /**
     * Action to get the previous set of values in the collection
     */
    String GET_PREVIOUS_RESULTS = "get_previous_results"; //$NON-NLS-1$

    /**
     * Action to get the first set of values in the collection
     */
    String GET_FIRST_RESULTS = "get_first_results"; //$NON-NLS-1$

    /**
     * Action to get the last set of values in the collection
     */
    String GET_LAST_RESULTS = "get_last_results"; //$NON-NLS-1$

    /**
     * Marks the column as containing only text values
     */
    int TEXT = 1;

    /**
     * Marks the column contents as ones which have links
     */
    int LINK = 2;

    /**
     * The desired delimiter to which separates the values.
     */
    String COL_DELIMITER = "~~~~~"; //$NON-NLS-1$

    /**
     * The desired delimiter to which separates the lines.
     */
    String ROW_DELIMITER = "&&&"; //$NON-NLS-1$

    /**
     * The desired keystring for "Showing".
     */
    String SHOWING = "SEARCH.RESULT.VIEW.SHOWING"; //$NON-NLS-1$

    /**
     * The desired keystring for "to".
     */
    String TO = "SEARCH.RESULT.VIEW.TO"; //$NON-NLS-1$

    /**
     * The desired keystring for "of".
     */
    String OF = "SEARCH.RESULT.VIEW.OF"; //$NON-NLS-1$	

    /**
     * The desired keystring for "Last".
     */
    String LAST = "SEARCH.RESULT.VIEW.LAST"; //$NON-NLS-1$	

    /**
     * The desired keystring for "First".
     */
    String FIRST = "SEARCH.RESULT.VIEW.FIRST"; //$NON-NLS-1$	

    /**
     * The desired keystring for "Next".
     */
    String NEXT = "SEARCH.RESULT.VIEW.NEXT"; //$NON-NLS-1$	

    /**
     * The desired keystring for "Previous".
     */
    String PREVIOUS = "SEARCH.RESULT.VIEW.PREVIOUS"; //$NON-NLS-1$		

    /**
     * The desired keystring for "Work with Item(s)".
     */
    String WORK_WITH_ITEM = "SEARCH.VIEW.WORK_WITH_ITEM"; //$NON-NLS-1$	

    /**
     * The desired keystring for "0 records found".
     */
    String ZERO_RECORDS_FOUND = "MESSAGE_CODE_152"; //$NON-NLS-1$			

    /**
     * Action to display the collection
     */
    String COLLECTION_FORM_DISPLAY = "collection_form_display"; //$NON-NLS-1$

    /**
     * Returns the Results Metadata
     *
     * @return Array of Result MetaData Elements
     */
    ArrayList getResultsMetaData();

    /**
     * Returns the availabe ResultRows associated with the current Result
     * @return      Array of ResultRows
     */
    ArrayList getResultsRows();

    /**
     * Returns the availability of Next Results for the Current Result
     * @return      true , if next results available <br>
     * false , if next results are not available
     */
    boolean hasNextResults();

    /**
     * Returns the availability of Previous results for the Current Result
     * @return      true , if next results available <br>
     * false , if next results are not available
     */
    boolean hasPreviousResults();

    /**
     * Returns the Total Results size
     * @return     total results size, int
     */
    int getTotalResultsSize();

    /**
     * Returns the  current displayed Results size
     * @return     size of the Current displayed Result Set, int
     */

    /**
     * Returns the  start index
     * @return     int, the start index
     */
    int getStartIndex();

    /**
     * Returns the  end index
     * @return     int, the end index
     */
    int getEndIndex();

    /**
     * Returns the name  associated with the current  element
     * @return String, the name  of the result set element
     **/
}
