
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * Created on Dec 2, 2004
 */
package com.ibm.ccd.ui.portlets.search.core;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletRequest;
import org.w3c.dom.Node;

import com.ibm.ccd.ui.context.IWPCUser;
import com.ibm.ccd.ui.context.WPCUser;
import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.dataobjects.IDataBean;
import com.ibm.ccd.ui.dataobjects.NVPair;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.AttributeTypes;
import com.ibm.ccd.ui.dataobjects.attributes.GroupAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.LookupTableAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberEnumAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringEnumAttribute;
import com.ibm.ccd.ui.dataobjects.items.ICatalogItem;
import com.ibm.ccd.ui.exception.MessageList;
import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.exception.UIException;
import com.ibm.ccd.ui.mediator.WPCMediator;
import com.ibm.ccd.ui.mediator.WPCMediatorException;
import com.ibm.ccd.ui.taglib.dataobjects.BaseSearchElement;
import com.ibm.ccd.ui.taglib.dataobjects.ResultData;
import com.ibm.ccd.ui.taglib.dataobjects.ResultMetaData;
import com.ibm.ccd.ui.taglib.dataobjects.SelectSearchElement;
import com.ibm.ccd.ui.taglib.dataobjects.TextSearchElement;
import com.ibm.ccd.ui.util.UIUtils;


/**
 * This class provides implementation for the <code>IBaseSearch</code> interface.
  */
public class SearchBean implements IDataBean, IBaseSearch, Serializable {
	/**
	 * Contains flattened attributes
	 */
	HashMap flatAttributeMap=new HashMap();
	
    /**
     * contains operators for numeric types
     */
    private static HashMap numericOperators = null;

    /**
     * contains operators for text types
     */
    private static HashMap textOperators = null;

    /**
     * contains operators for date type
     */
    private static HashMap flagOperators = null;

    /**
     * contains operators for boolean type
     */
    private static HashMap stringEnumOperators = null;

    /**
     * contains the boolean values
     */
    private static NVPair [] booleanValues = null;

    /**
     * Contains the <code>WPCUser</code> Object
     */
    private WPSContext wpsContext = null;

    /**
     *Indicates the the business function which will consume this bean e.g search criteria
     */
    private String businessFunction = null;

    /**
     *  contains the <code>PortletRequest</code> Object
     */
    private PortletRequest portletRequest = null;

    /**
     * contains <code>PortletContext</code>
     */
    private PortletContext context = null;

    /**
     * contains the request xml
     */
    private String reqXML = null;

    /**
     * contains the response xml
     */
    private String resXML = null;

    /**
     * contains the actual response xml
     */
    private String responseXML = null;

    /**
     * contains the search result limit
     */
    private String maxResults = null;
    private long startIndex=0;
    
    private long endIndex=0;
    private String collabName = null; 
    public SearchBean() {
    }

    public String toXML() {
        return reqXML;
    }

    /**
     * Mediator calls thsi method and sets the xml
     */
    public void fromXML(String xml) {
        resXML = xml;
    }

    /**
     * sets the <code>PortletRequest</code> Object
     */
    public MessageMap setProperties(PortletRequest portletRequest) {
        this.portletRequest = portletRequest;

        return null;
    }

    /**
     * Validation
     */
    public MessageList validate(PortletRequest request, String action) {
        return null;
    }

    /**
     *
     */
    public void fromXML(Node domNode) {
    }

    /**
     * This method returns the list of <code>BasicSearchElement</code>
     *
     */
    public ArrayList getSearchCriteriaElements()
        throws UIException, WPCMediatorException {
        String name = null;
        int type = 0;
        Attribute[] attributes = null;
        ArrayList searchCriteriaElements = null;
        Attribute attribute = null;
        BaseSearchElement searchElement = null;
        NVPair[] possibleValues = null;
        BaseSearchElement childElement = null;
        boolean isMandatory = false;
        NVPair[] pairs = null;
        String displayName=null;
        //initialise text operators for localisation support
        initOperators();
		
        String locale =(String) wpsContext.getWpcUser().getProperty(IWPCUser.KEY_RESTRICTED_LANGUAGE);
        // get catlog specs
		attributes = getSearchAttributes();
        searchCriteriaElements = new ArrayList();

        for (int i = 0; i < attributes.length; i++) {
            attribute = (Attribute) attributes[i];
            if(!UIUtils.isAttributeDisplayableForLocale(attribute,locale)){
                continue;
            }
            
            name = attribute.getName();
            displayName=attribute.getDisplayLabelName();
            type = attribute.getType();
            isMandatory = attribute.isPrimaryKey();

            switch (type) {
            case AttributeTypes.STRING:
            case AttributeTypes.THUMB_NAIL_IMAGE_URL:

            case AttributeTypes.URL:

            case AttributeTypes.BINARY:

            case AttributeTypes.THUMB_NAIL_IMAGE:

            case AttributeTypes.IMAGE:

            case AttributeTypes.IMAGE_URL:
                searchElement = (BaseSearchElement)new TextSearchElement(name, "",
                                                                         new String(IBaseSearch.DEFAULT_TEXT_OPERATOR),
                                                                         textOperators, false, true);
                searchElement.setMandatory(isMandatory);
                searchElement.setDisplayName(displayName);
                searchElement.setWpcAttributeType(type);
                searchElement.setWpcAttribute(attribute);
                searchCriteriaElements.add(searchElement);

                break;
            /*    
            case AttributeTypes.RELATIONSHIP:
                searchElement = (BaseSearchElement)new TextSearchElement(name, "",new String(IBaseSearch.DEFAULT_TEXT_OPERATOR),
                                                                         stringEnumOperators, false, true);
                searchElement.setMandatory(isMandatory);
                searchElement.setDisplayName(displayName);
                searchElement.setWpcAttributeType(type);
                searchElement.setWpcAttribute(attribute);
                searchCriteriaElements.add(searchElement);
                break;				 
              */  
            case AttributeTypes.DATE:
                searchElement = (BaseSearchElement)new TextSearchElement(name, "",
                                                                         new String(IBaseSearch.DEFAULT_DATE_OPERATOR),
                                                                         numericOperators, false, true);
                searchElement.setMandatory(isMandatory);
                searchElement.setDisplayName(displayName);
                searchElement.setWpcAttributeType(type);
                addChildElement(searchElement);
                searchElement.setWpcAttribute(attribute);
                searchCriteriaElements.add(searchElement);

                break;

            case AttributeTypes.INTEGER:
            
            case AttributeTypes.NUMBER:
            
            case AttributeTypes.SEQUENCE:
            
            case AttributeTypes.PERIOD:
            
            case AttributeTypes.CURRENCY:
            
                searchElement = (BaseSearchElement)new TextSearchElement(name, "",
                                                                         new String(IBaseSearch.DEFAULT_NUMERIC_OPERATOR),
                                                                         numericOperators, false, true);
                searchElement.setMandatory(isMandatory);
                searchElement.setDisplayName(displayName);
                searchElement.setWpcAttributeType(type);
                addChildElement(searchElement);
                searchElement.setWpcAttribute(attribute);
                searchCriteriaElements.add(searchElement);

                break;

            case AttributeTypes.STRING_ENUMERATION:

                StringEnumAttribute strEnum = (StringEnumAttribute) attribute;
                possibleValues = getMap(strEnum.getEnumValues());
                if (possibleValues==null||possibleValues.length == 0)
                {
                    searchElement = (BaseSearchElement)new TextSearchElement(name, "",
                                                                             new String(IBaseSearch.DEFAULT_TEXT_OPERATOR),
                                                                             textOperators, false, true);
                }
                else
                {
                    searchElement = (BaseSearchElement)new SelectSearchElement(name, 1,
                                                                               possibleValues,
                                                                               new String(IBaseSearch.DEFAULT_TEXT_OPERATOR),
                                                                               stringEnumOperators, false, true);
                }
                searchElement.setMandatory(isMandatory);
                searchElement.setDisplayName(displayName);
                searchElement.setWpcAttributeType(type);
                searchElement.setWpcAttribute(attribute);
                searchCriteriaElements.add(searchElement);

                break;

            case AttributeTypes.NUMBER_ENUMERATION:

                NumberEnumAttribute numEnum = (NumberEnumAttribute) attribute;
                possibleValues = getMap(numEnum.getEnumDb());
                if (possibleValues == null||possibleValues.length == 0)
                {
                    searchElement = (BaseSearchElement)new TextSearchElement(name, "",
                                                                             new String(IBaseSearch.DEFAULT_NUMERIC_OPERATOR),
                                                                             numericOperators, false, true);
                }
                else
                {
                    searchElement = (BaseSearchElement)new SelectSearchElement(name, 1,
                                                                               possibleValues,
                                                                               new String(IBaseSearch.DEFAULT_NUMERIC_OPERATOR),
                                                                               numericOperators, false, true);
                }
                searchElement.setMandatory(isMandatory);
                searchElement.setDisplayName(displayName);
                searchElement.setWpcAttributeType(type);
                searchElement.setWpcAttribute(attribute);
                addChildElement(searchElement);
                searchCriteriaElements.add(searchElement);

                break;

            case AttributeTypes.FLAG:
                searchElement = (BaseSearchElement)new SelectSearchElement(name, 1, booleanValues,
                                                                           new String(IBaseSearch.DEFAULT_BOOLEAN_OPERATOR),
                                                                           flagOperators, false, true);
                searchElement.setMandatory(isMandatory);
                searchElement.setDisplayName(displayName);
                searchElement.setWpcAttributeType(type);
                searchElement.setWpcAttribute(attribute);
                searchCriteriaElements.add(searchElement);

                break;

            case AttributeTypes.TIMEZONE:
            
            case AttributeTypes.LOOKUP_TABLE:

                LookupTableAttribute lookupTable = (LookupTableAttribute) attribute;
                pairs = lookupTable.getLookupTable();

                if ((pairs != null) && (pairs.length > 0)) {
                    possibleValues = getMap(lookupTable);
                    searchElement = (BaseSearchElement)new SelectSearchElement(name, 1,
                                                                               possibleValues,
                                                                               new String(IBaseSearch.DEFAULT_TEXT_OPERATOR),
                                                                               stringEnumOperators, false, true);
                    searchElement.setMandatory(isMandatory);
                    searchElement.setDisplayName(displayName);
                    searchElement.setWpcAttributeType(type);
                    searchElement.setWpcAttribute(attribute);
                    searchCriteriaElements.add(searchElement);
                } else {
                    searchElement = (BaseSearchElement)new TextSearchElement(name, "",
                                                                             new String(IBaseSearch.DEFAULT_TEXT_OPERATOR),
                                                                             stringEnumOperators, false, true);
                    searchElement.setMandatory(isMandatory);
                    searchElement.setDisplayName(displayName);
                    searchElement.setWpcAttributeType(type);
                    searchElement.setWpcAttribute(attribute);
                    searchCriteriaElements.add(searchElement);
                }

                break;
				 
            }
        }

        return searchCriteriaElements;
    }

    /**
     * @param lookupTable
     * @return
     */
    private NVPair [] getMap(LookupTableAttribute lookupTable) {
        NVPair pair = null;
        NVPair[] lookup = lookupTable.getLookupTable();
		NVPair[] result= new NVPair[lookup.length+1];
		String labelNone=UIUtils.getTextLabel(getContext(), getPortletRequest(),LABEL_KEY_NONE);
		
		if(lookupTable.getType()==AttributeTypes.LOOKUP_TABLE){
			result[0]= new NVPair("",labelNone);
			for(int i=0; i<lookup.length;i++){
				result[i+1]=new NVPair(lookup[i].getName(),lookup[i].getName());
			}					
		}
		else if(lookupTable.getType()==AttributeTypes.TIMEZONE){
			result[0]= new NVPair("",labelNone);
			for(int i=0; i<lookup.length;i++){
				result[i+1]=lookup[i];
			}		
		}
		
		
        return result;
    }

    /**
     * @param textlement
     */
    private void addChildElement(BaseSearchElement searchElement)
        throws UIException {
        BaseSearchElement childElement = null;
        String name = null;

        try {
            childElement = (BaseSearchElement) searchElement.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
            throw new UIException();
        }
        name = childElement.getName();
        childElement.setName(name + "_TO");
        childElement.setOperatorID(childElement.getName() + "_OP");
        searchElement.setChildElement(childElement);
    }

    /**
     *
     * @return ArrayList of <code>Attribute</code> Objects
     */
    public Attribute[] getSearchAttributes()
        throws UIException, WPCMediatorException {
        ArrayList attributesList = null;
        ICatalogItem catalogItem = null;
        Attribute[] attributes = null;
		Attribute[] flatAttributes = null;
		HashMap flatAttributeMap=null;
        WPCUser user = null;
        String catalogId = null;
		Collection attrCollection = null;
		Iterator attrItr = null;
		int size = 0;
        user = wpsContext.getWpcUser();
        // Use the cached catalogId
        catalogId=UIUtils.getCatalogName(user);

        WPCMediator mediator = WPCMediator.getInstance();
		attributes=mediator.getSearchAttributes(catalogId,getWpsContext());
		flatAttributeMap=prepareFlatAttributeMap(attributes);
		attrCollection = flatAttributeMap.values();
		size = attrCollection.size();
		attrItr = attrCollection.iterator();
		flatAttributes = new Attribute[size];
		
		for (int i = 0; i < size; i++) {
			flatAttributes[i] = (Attribute) attrItr.next();
		}
		
        return flatAttributes;
    }
    
	public Attribute[] getSearchResultAttributes()
		   throws UIException, WPCMediatorException {
		   ArrayList attributesList = null;
		   ICatalogItem catalogItem = null;
		   Attribute[] attributes = null;
		   Attribute[] flatAttributes = null;
		   HashMap flatAttributeMap=null;
		   WPCUser user = null;
		   String catalogId = null;
		   Collection attrCollection = null;
			Iterator attrItr = null;
			int size = 0;
			
		   user = wpsContext.getWpcUser();
		   // Use the cached catalogId
	       catalogId=UIUtils.getCatalogName(user);
		   WPCMediator mediator = WPCMediator.getInstance();
		   attributes=mediator.getSearchResultAttributes(catalogId,getWpsContext());
		   flatAttributeMap=prepareFlatAttributeMap(attributes);
			attrCollection = flatAttributeMap.values();
			size = attrCollection.size();
			attrItr = attrCollection.iterator();
			flatAttributes = new Attribute[size];
			
		   	for (int i = 0; i < size; i++) {
			   flatAttributes[i] = (Attribute) attrItr.next();
			 }
			 
		   return flatAttributes;
	   }

    // Added by Mahesh ends

    /**
     * This method returns the ResultData which holds the search results
     *  @return ResultData
     */
    public ResultData getSearchResults() throws UIException {
        ResultData resultData = new ResultData();
        CSVParser csv = new CSVParser();
        String resXML = getResponseXML();

        if (!resXML.toString().equals("")) {
            resultData = csv.parse(resXML);
        }

        if ((resultData != null) && (resultData.getResultRowData().size() > 0)) {
            return resultData;
        } else {
            return null;
        }
    }

    private void initOperators() {
        String opBeginsWith = null;
        String opBeginsWithNot = null;
        String opContains = null;
        String opContainsNot = null;
        String opEndsWith = null;
        String opEndsWithNot = null;
        String opExactMatch = null;
        String opExactMatchNot = null;
        String opIsEmpty = null;
        String opIsEmptyNot = null;
        String flagTrue=null;
		String flagFalse=null;
        String key = null;
		String labelNone=null;

        booleanValues = new NVPair[3];
        
		key = PREFIX_LOCALISATION + IBaseSearch.FLAG_FALSE;
		flagFalse=UIUtils.getTextLabel(getContext(), getPortletRequest(),key);
		key = PREFIX_LOCALISATION + IBaseSearch.FLAG_TRUE;
		flagTrue=UIUtils.getTextLabel(getContext(), getPortletRequest(),key);
		labelNone=UIUtils.getTextLabel(getContext(), getPortletRequest(),LABEL_KEY_NONE);
		booleanValues[0]=new NVPair ("", labelNone);
        booleanValues[1]=new NVPair (flagFalse, flagFalse);
        booleanValues[2]= new NVPair(flagTrue, flagTrue);

        //load numeric operators
        numericOperators = new HashMap();

        numericOperators.put("=", new String("="));
        numericOperators.put(">=", new String(">="));
        numericOperators.put("&lt;=", new String("&lt;="));
        numericOperators.put("&lt;", new String("&lt;"));
        numericOperators.put(">", new String(">"));
        numericOperators.put("!=", new String("!="));

        //	load text operators
        textOperators = new HashMap();
        stringEnumOperators = new HashMap();
		flagOperators = new HashMap();

        //begins with
        key = PREFIX_LOCALISATION + IBaseSearch.OP_BEGINSWITH;
        opBeginsWith = UIUtils.getTextLabel(getContext(), getPortletRequest(),
                key);
        textOperators.put(opBeginsWith, IBaseSearch.OP_BEGINSWITH);
        key = PREFIX_LOCALISATION + IBaseSearch.OP_BEGINSWITH_NOT;
        opBeginsWithNot = UIUtils.getTextLabel(getContext(),
                getPortletRequest(), key);
        textOperators.put(opBeginsWithNot, IBaseSearch.OP_BEGINSWITH_NOT);

        //contains
        key = PREFIX_LOCALISATION + IBaseSearch.OP_CONTAINS;
        opContains = UIUtils.getTextLabel(getContext(), getPortletRequest(), key);
        textOperators.put(opContains, IBaseSearch.OP_CONTAINS);
        key = PREFIX_LOCALISATION + IBaseSearch.OP_CONTAINS_NOT;
        opContainsNot = UIUtils.getTextLabel(getContext(), getPortletRequest(),
                key);
        textOperators.put(opContainsNot, IBaseSearch.OP_CONTAINS_NOT);

        //ends with	
        key = PREFIX_LOCALISATION + IBaseSearch.OP_ENDSWITH;
        opEndsWith = UIUtils.getTextLabel(getContext(), getPortletRequest(), key);
        textOperators.put(opEndsWith, IBaseSearch.OP_ENDSWITH);
        key = PREFIX_LOCALISATION + IBaseSearch.OP_ENDSWITH_NOT;
        opEndsWithNot = UIUtils.getTextLabel(getContext(), getPortletRequest(),
                key);
        textOperators.put(opEndsWithNot, IBaseSearch.OP_ENDSWITH_NOT);

        //exact match
        key = PREFIX_LOCALISATION + IBaseSearch.OP_EXACTMATCH;
        opExactMatch = UIUtils.getTextLabel(getContext(), getPortletRequest(),
                key);
        textOperators.put(opExactMatch, "=");
        stringEnumOperators.put(opExactMatch, "=");
		flagOperators.put(opExactMatch, "=");

        key = PREFIX_LOCALISATION + IBaseSearch.OP_EXACTMATCH_NOT;
        opExactMatchNot = UIUtils.getTextLabel(getContext(),
                getPortletRequest(), key);
        textOperators.put(opExactMatchNot, "!=");
        stringEnumOperators.put(opExactMatchNot, "!=");
		flagOperators.put(opExactMatchNot, "!=");

        //is emplty
        key = PREFIX_LOCALISATION + IBaseSearch.OP_ISEMPTY;
        opIsEmpty = UIUtils.getTextLabel(getContext(), getPortletRequest(), key);
        textOperators.put(opIsEmpty, IBaseSearch.OP_ISEMPTY);
        stringEnumOperators.put(opIsEmpty, IBaseSearch.OP_ISEMPTY);

        key = PREFIX_LOCALISATION + IBaseSearch.OP_ISEMPTY_NOT;
        opIsEmptyNot = UIUtils.getTextLabel(getContext(), getPortletRequest(),
                key);
        textOperators.put(opIsEmptyNot, IBaseSearch.OP_ISEMPTY_NOT);
        stringEnumOperators.put(opIsEmptyNot, IBaseSearch.OP_ISEMPTY_NOT);
        
    }

    private NVPair[] getMap(Object[] obj) {
		NVPair[] pair=null;
        if (obj != null) {
			pair= new NVPair[obj.length+1];
			String labelNone=UIUtils.getTextLabel(getContext(), getPortletRequest(),LABEL_KEY_NONE);
			pair[0]= new NVPair("",labelNone);
		
    	    for (int i = 0; i < obj.length; i++) {
				pair[i+1]= new NVPair(obj[i].toString(),obj[i].toString());
        	}
        }
        return pair;
    }

    private NVPair[] getMap(double[] obj) {
		NVPair[] pair=null;
		if (obj != null) {
			pair= new NVPair[obj.length+1];
			String labelNone=UIUtils.getTextLabel(getContext(), getPortletRequest(),LABEL_KEY_NONE);
			pair[0]= new NVPair("",labelNone);
		
	        for (int i = 0; i < obj.length; i++) {
				Double value = new Double(obj[i]);
				pair[i+1]= new NVPair(value.toString(),value);
			
    	    }
		}
        return pair;
    }

    /**
     * @return
     */
    public String getReqXML() {
        return reqXML;
    }

    /**
     * @param string
     */
    public void setReqXML(String string) {
        reqXML = string;
    }

    /**
     * @return
     */
    public String getResponseXML() {
        return responseXML;
    }

    /**
     * @param string
     */
    public void setResponseXML(String string) {
        responseXML = string;
    }

    /**
     * @return
     */
    public PortletRequest getPortletRequest() {
        return portletRequest;
    }

    /**
     * @param request
     */
    public void setPortletRequest(PortletRequest request) {
        portletRequest = request;
    }

    /**
     * @return
     */
    public PortletContext getContext() {
        return context;
    }

    /**
     * @param context
     */
    public void setContext(PortletContext context) {
        this.context = context;
    }

    /**
     * @return
     */
    public WPSContext getWpsContext() {
        return wpsContext;
    }

    /**
     * @param context
     */
    public void setWpsContext(WPSContext context) {
        wpsContext = context;
    }

    /**
     * @return
     */
    public String getMaxResults() {
        return maxResults;
    }

    /**
     * @param string
     */
    public void setMaxResults(String string) {
        maxResults = string;
    }

    /* (non-Javadoc)
     * @see com.ibm.ccd.ui.dataobjects.IDataBean#updateProperties(org.apache.jetspeed.portlet.PortletRequest)
     */
    public MessageMap updateProperties(PortletRequest portletRequest) {
        // TODO Auto-generated method stub
        return null;
    }

	/**
	 * @return
	 */
	public ArrayList getSearchResultElements()throws UIException, WPCMediatorException {
		Attribute [] attributes=null;
		Attribute attribute=null; 
		ArrayList metaDataList=null;
		ResultMetaData resultMetaData=null;
		String name=null;
		String displayName=null;
		boolean isMandatory=false;
		int attrType=0;
		
		String locale =(String) wpsContext.getWpcUser().getProperty(IWPCUser.KEY_RESTRICTED_LANGUAGE);
		// get specs
		attributes = getSearchResultAttributes();
		metaDataList= new ArrayList();
		for (int i = 0; i < attributes.length; i++) {
			attribute = attributes[i];
			attrType=attribute.getType();
			if(attrType==AttributeTypes.PASSWORD||attrType==AttributeTypes.GROUP
			||attrType==AttributeTypes.UNSUPPORTED ){
				continue;
			}
			else if(!UIUtils.isAttributeDisplayableForLocale(attribute,locale)){
				continue;
			}
            
			name = attribute.getName();
			displayName=attribute.getDisplayLabelName();
			isMandatory = attribute.isPrimaryKey();
			resultMetaData= new ResultMetaData();
			resultMetaData.setName(name);
			resultMetaData.setDisplayName(displayName);
			resultMetaData.setMandatory(isMandatory);
			if(isMandatory){
				resultMetaData.setDisplayable(true);
			}
			metaDataList.add(resultMetaData);
		}
		return metaDataList;
	}
	
	/**
		 * prepare map of all attributes available including the map inside group attributes
		 * @param attributesParam
		 * @return
		 */
		public HashMap prepareFlatAttributeMap(Attribute[] attributesParam) {
			
			for (int i = 0; i < attributesParam.length; i++) {
				if (attributesParam[i] != null) {
					if (attributesParam[i].getType() == AttributeTypes.GROUP) {
						flatAttributeMap.put(attributesParam[i].getName(),
							attributesParam[i]);
						prepareFlatAttributeMap(((GroupAttribute) attributesParam[i]).getAttributes());
					} else {
						//if (attributesParam[i].isLocalized() && attributesParam[i].getLocalizedValues().size() > 0){
                    
						if (attributesParam[i].isLocalized()) {
							flatAttributeMap.put(attributesParam[i].getName(),
														attributesParam[i]);
							Attribute[] attrTempArray = new Attribute[attributesParam[i].getLocalizedValues().size()];
							for (int j = 0; j < attributesParam[i].getLocalizedValues().size(); j++) {
								attrTempArray[j] = (Attribute)attributesParam[i].getLocalizedValues().get(j);
							}
							prepareFlatAttributeMap(attrTempArray);
						} else {
							flatAttributeMap.put(attributesParam[i].getName(),
								attributesParam[i]);
						}
					}
				}
			}
			
			
			return flatAttributeMap;
		}
	public long getEndIndex() {
		return endIndex;
	}

	/**
	 * @return
	 */
	public long getStartIndex() {
		return startIndex;
	}

	/**
	 * @param i
	 */
	public void setEndIndex(long i) {
		endIndex = i;
	}

	/**
	 * @param i
	 */
	public void setStartIndex(long i) {
		startIndex = i;
	}
	/**
	 * @return Returns the collabName.
	 */
	public String getCollabName() {
		return collabName;
	}
	/**
	 * @param collabName The collabName to set.
	 */
	public void setCollabName(String collabName) {
		this.collabName = collabName;
	}
}
