
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * Created on May 18, 2005
 *
 */
package com.ibm.ccd.ui.portlets.search.core;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletData;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletSession;
import org.apache.jetspeed.portlet.PortletSettings;

import com.ibm.ccd.ui.context.WPCUser;
import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.dataobjects.attributes.AttributeTypes;
import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.exception.UIException;
import com.ibm.ccd.ui.mediator.WPCMediator;
import com.ibm.ccd.ui.mediator.WPCMediatorException;
import com.ibm.ccd.ui.portlets.search.ISearch;
import com.ibm.ccd.ui.taglib.dataobjects.BaseSearchElement;
import com.ibm.ccd.ui.taglib.dataobjects.ResultData;
import com.ibm.ccd.ui.taglib.dataobjects.ResultMetaData;
import com.ibm.ccd.ui.util.UIUtils;
import com.ibm.ccd.ui.util.ValidationUtil;


/**
 *
 */
public class SearchHelper {
    private PortletContext portletContext = null;
    private PortletRequest request = null;
    private WPSContext wpsContext = null;
    private SearchCriteriaBean searchCriteriaBean = null;
    private String portletName = null;
    private SearchBean searchBean = null;

    /**
     *
     */
    public SearchHelper() {
        super();
    }

    public void setRequestValues() {
        ArrayList searchElements = searchCriteriaBean.getAttributeList();
        BaseSearchElement element = null;
        BaseSearchElement childElement = null;
        Enumeration parameterNames = null;
        String elementName = null;
        String parameter = null;
        String value = null;
        Iterator searchElementsItr = null;

        searchElementsItr = searchElements.iterator();

        while (searchElementsItr.hasNext()) {
            element = (BaseSearchElement) searchElementsItr.next();

            if (element.getIsDisplayable()) {
                setReqData(request, element);

                if (element.getChildElement() != null) {
                    childElement = element.getChildElement();
                    setReqData(request, childElement);
                }
            }
        }
    }

    /**
     * @param childElement
     */
    private void setReqData(PortletRequest request, BaseSearchElement element) {
        String elementName = null;
        String value = null;
        String operatorName = null;
        String operatorValue = null;
        elementName = element.getName();
        value = request.getParameter(elementName);
        //use escape char for spl xml char
		value=value.replaceAll("&","&amp;");
        element.setSelectedValue(value);
        operatorName = elementName + "_OP";
        operatorValue = request.getParameter(operatorName);
        element.setDefaultOperatorID(operatorValue);
    }

    public void saveSearch(SearchInputBean searchInputBean)
        throws UIException {
        ArrayList searchData = null;
        ISaveSearch savedSearch = null;
        PortletSession portletSession = null;
        PortletSettings portletSettings = null;
        SearchCriteriaBean searchCriteriaBean = null;
        String sMaxSavedsearchs = null;
        int iMaxSavedSearchs = 0;
        String userId = null;
        String prvSearchMsg = null;
        int count = 0;
        WPCUser wpcUser = null;
        String catalogId = null;

        //get user Id
        userId = request.getUser().getUserID();

        //get max saved searchs limit
        portletSettings = request.getPortletSettings();
        sMaxSavedsearchs = portletSettings.getAttribute(IBaseSearch.MAX_SEARCH_HISTORY);
        iMaxSavedSearchs = Integer.parseInt(sMaxSavedsearchs);

        //load saved searchs
        savedSearch = (ISaveSearch) SearchUtil.getClass(IBaseSearch.CLASS_NAME_FILE_SAVED_SEARCH);
        savedSearch.setUserId(userId);

        wpcUser = wpsContext.getWpcUser();
        catalogId = UIUtils.getCatalogName(wpcUser);
        savedSearch.setFolder(SearchUtil.getHistoryFolder(portletContext,
                portletName, catalogId));
        searchData = savedSearch.loadSearch();

        //		modify hashtable
        if (searchData == null) {
            searchData = new ArrayList();
        }

        count = searchData.size();

        if (count >= iMaxSavedSearchs) {
            for (int i = 0; count >= iMaxSavedSearchs; --count) {
                searchData.remove(i);
            }
        }

        searchData.add(searchInputBean);

        //save new search XML
        savedSearch.saveSearch(searchData);

        //update prev searchs in session
        portletSession = request.getPortletSession();
        searchCriteriaBean = (SearchCriteriaBean) portletSession.getAttribute(IBaseSearch.KEY_SEARCH_CRITERIA_BEAN);

        //update criteria bean		
        searchCriteriaBean.setPreviousSeach(searchData);
    }

    /**
     * @param searchInputBean
     */
    public void performSearch(SearchInputBean searchInputBean)
        throws UIException, WPCMediatorException {
        String searchReqXML = null;
        String searchCriteria = null;
        PortletSettings portletSettings = null;
        SearchResultBean searchResultBean=null;
		StringBuffer viewAttributes=null;
		ResultMetaData resultMetaData=null;
		ArrayList resultAttributes=null;
		String sTraversalSize=null;
		
		sTraversalSize=(String) request.getData().getAttribute(IBaseSearch.KEY_TRAVERSAL_SIZE);
         if(ValidationUtil.isNullOrEmpty(sTraversalSize)){
			sTraversalSize=IBaseSearch.DEFAULT_TRAVERSAL_SIZE;
         }
		resultAttributes=searchCriteriaBean.getMetaDataList();
		searchResultBean=(SearchResultBean) request.getPortletSession().getAttribute(IBaseSearch.KEY_SEARCH_RESULTS_BEAN);
		int traversalSize=Integer.parseInt(sTraversalSize);

//		requested attributes in search result		
		viewAttributes= new StringBuffer();
		viewAttributes.append("<wpc:wpcAttributeList>");
		for(int j=0;j<resultAttributes.size();j++){
			resultMetaData=(ResultMetaData) resultAttributes.get(j);
			if(resultMetaData.isDisplayable()){
				viewAttributes.append("<wpc:attribute name=\"");
				viewAttributes.append(resultMetaData.getName());
				viewAttributes.append("\"/>");
			}
		}
		viewAttributes.append("</wpc:wpcAttributeList>");
		
		searchReqXML=searchInputBean.getSearchXML() + viewAttributes.toString();
		
		if(searchResultBean!=null){
			searchBean.setStartIndex(searchResultBean.getStartIndex());
			searchBean.setEndIndex(searchResultBean.getEndIndex());
		}else{
			searchBean.setStartIndex(1);
			searchBean.setEndIndex(traversalSize);
		}
       	searchBean.setReqXML(searchReqXML);
        WPCMediator mediator = WPCMediator.getInstance();
        mediator.performSearch(searchBean, wpsContext);

        //	set SR bean in session
        setSearchResult(searchBean, searchInputBean);
    }

    public void performLookupTableSearch(SearchInputBean searchInputBean)
        throws UIException, WPCMediatorException {
        String searchReqXML = null;
        String searchCriteria = null;
        PortletSettings portletSettings = null;
		SearchResultBean searchResultBean=null;
		ArrayList resultAttributes=null;
		ResultMetaData resultMetaData=null;
		StringBuffer viewAttributes=null;
		String sTraversalSize=null;
		
		sTraversalSize=(String) request.getData().getAttribute(IBaseSearch.KEY_TRAVERSAL_SIZE);
		 if(ValidationUtil.isNullOrEmpty(sTraversalSize)){
				sTraversalSize=IBaseSearch.DEFAULT_TRAVERSAL_SIZE;
		}
	   resultAttributes=searchCriteriaBean.getMetaDataList(); 
	   searchResultBean=(SearchResultBean) request.getPortletSession().getAttribute(IBaseSearch.KEY_SEARCH_RESULTS_BEAN);
	   int traversalSize=Integer.parseInt(sTraversalSize);
	   
//	   requested attributes in search result		
	   viewAttributes= new StringBuffer();
	   viewAttributes.append("<wpc:wpcAttributeList>");
	   for(int j=0;j<resultAttributes.size();j++){
		   resultMetaData=(ResultMetaData) resultAttributes.get(j);
		   if(resultMetaData.isDisplayable()){
			   viewAttributes.append("<wpc:attribute name=\"");
			   viewAttributes.append(resultMetaData.getName());
			   viewAttributes.append("\"/>");
		   }
	   }
	   viewAttributes.append("</wpc:wpcAttributeList>");
		
	   searchReqXML=searchInputBean.getSearchXML() + viewAttributes.toString();
	   
	   if(searchResultBean!=null){
		   searchBean.setStartIndex(searchResultBean.getStartIndex());
		   searchBean.setEndIndex(searchResultBean.getEndIndex());
	   }else{
		   searchBean.setStartIndex(1);
		   searchBean.setEndIndex(traversalSize);
	   }
        searchReqXML = searchInputBean.getSearchXML();
        searchBean.setReqXML(searchReqXML);

        WPCMediator mediator = WPCMediator.getInstance();
        mediator.performLookupTableSearch(searchBean, wpsContext);

        //	set SR bean in session
        setSearchResult(searchBean, searchInputBean);
    }

    /**
     * @param searchCriteriaBean
     * @return
     */
    public SearchInputBean getSearchInput()throws UIException {
        SearchInputBean searchInputBean = null;
        StringBuffer reqXML = null;
        StringBuffer searchURL = null;
        ArrayList attributeList = null;
        Iterator attributeIterator = null;
        BaseSearchElement element = null;
        BaseSearchElement childElement = null;
        String operator = null;
        String value = null;
        String elementName = null;
        int elementCount = 0;
        int total = 0;
        int predicateIndex = 0;
        int index = 0;
        ArrayList userSearchCriteriaList = null;
        Iterator userSearchCriteriaItr = null;
        String operatorDisplayName = null;
        String displayName = null;
        String labelAnd = null;
        int wpcAttributeType;

        labelAnd = UIUtils.getTextLabel(portletContext, request,
                IBaseSearch.LABEL_VIEW_AND);

        searchInputBean = new SearchInputBean();
        reqXML = new StringBuffer();
        searchURL = new StringBuffer();
        attributeList = searchCriteriaBean.getAttributeList();
		
        attributeIterator = attributeList.iterator();
        userSearchCriteriaList = new ArrayList();

        while (attributeIterator.hasNext()) {
            element = (BaseSearchElement) attributeIterator.next();

            if (element.getIsDisplayable() && element.isSearchable()) {
                userSearchCriteriaList.add(element);
                addToBuffer(element, searchURL);

                if (element.getChildElement() != null) {
                    childElement = element.getChildElement();

                    if (!ValidationUtil.isNullOrEmpty(
                                childElement.getSelectedValue())) {
                        userSearchCriteriaList.add(childElement);
                        total++;
                        searchURL.append(" " + labelAnd + " ");
                        addToBuffer(childElement, searchURL);
                    }
                }

                searchURL.append(IBaseSearch.URL_STRING_SEPERATOR);
                total++;
            }
        }

        //set search URL ( displayed in history )
        //remove the last char ','
        searchInputBean.setURL(searchURL.substring(0, searchURL.length() - 2));

        userSearchCriteriaItr = userSearchCriteriaList.iterator();
		reqXML.append("<wpc:wpcSearchCriteria>");
        reqXML.append(IBaseSearch.WPC_SEARCH_OPERATOR_START_TAG);

        while (userSearchCriteriaItr.hasNext()) {
            element = (BaseSearchElement) userSearchCriteriaItr.next();
            elementName = element.getName();
            wpcAttributeType = element.getWpcAttributeType();
            index = elementName.lastIndexOf("_TO");

            if (index > 0) {
                elementName = elementName.substring(0, index);
            }

            operator = element.getDefaultOperator();                        
            value =UIUtils.unescapeForHTML(element.getSelectedValue());
            value = "<![CDATA["+value+"]]>";
            
			
			//validation for number type
			if (AttributeTypes.NUMBER == wpcAttributeType) {
				try {
					Double.parseDouble(value);	
				} catch (NumberFormatException e) {
					MessageMap messageMap= new MessageMap();
					messageMap.addErrorMessage(IBaseSearch.MSG_INVALID_NUMBER_VALUE,elementName);
					throw new UIException(messageMap);
				}
			}

            //this method handles local for number values
            value = SearchUtil.getFormattedValue(value, wpcAttributeType,
                    request.getLocale());

            if ((elementCount % 2) == 0) {
                reqXML.append(IBaseSearch.WPC_SEARCH_OPERATOR_START_TAG);
            }

            // **** sart search element****
            reqXML.append(IBaseSearch.WPC_SEARCH_ELEMENT_START_TAG);

            //set elementName name
            reqXML.append(IBaseSearch.WPC_SEARCH_ATTRIBUTE_START_TAG);
            reqXML.append(elementName);
            reqXML.append(IBaseSearch.WPC_SEARCH_ATTRIBUTE_END_TAG);

            // set attribute value
            reqXML.append(IBaseSearch.WPC_SEARCH_VALUE_START_TAG);
            reqXML.append(value);
            reqXML.append(IBaseSearch.WPC_SEARCH_VALUE_END_TAG);

            // set the predicate
            reqXML.append(IBaseSearch.WPC_SEARCH_PREDICATE_START_TAG);

            if (operator.indexOf("_NOT") != -1) {
                predicateIndex = operator.indexOf("_NOT");
                operator = operator.substring(0, predicateIndex);
                reqXML.append("1");
            } else {
                reqXML.append("0");
            }

            reqXML.append(IBaseSearch.WPC_SEARCH_PREDICATE_END_TAG);

            //set operator
            reqXML.append(IBaseSearch.WPC_SEARCH_ELEMENT_OPERATOR_START_TAG);
            reqXML.append(operator);
            reqXML.append(IBaseSearch.WPC_SEARCH_ELEMENT_OPERATOR_END_TAG);

            //****END search element****
            reqXML.append(IBaseSearch.WPC_SEARCH_ELEMENT_END_TAG);

            if (((elementCount % 2) == 1) || (total == (elementCount + 1))) {
                reqXML.append(IBaseSearch.WPC_SEARCH_OPERATOR_END_TAG);
            }

            elementCount++;
        }

        reqXML.append(IBaseSearch.WPC_SEARCH_OPERATOR_END_TAG);
		reqXML.append("</wpc:wpcSearchCriteria>");
	
        searchInputBean.setSearchXML(reqXML.toString());

        return searchInputBean;
    }
    
	protected String replaceSpecialCharacters(String str){
		if(str == null)
			return str;
		if(str.equals("&nbsp;"))
			return "&amp;nbsp;";
		else if(str.equals("&gt;"))
			return "&amp;gt;";
		else if(str.equals("&lt;"))
			return "&amp;lt;";
			return str;
	}

    /**
     * @param element
     */
    private void addToBuffer(BaseSearchElement element, StringBuffer searchURL) {
        String displayName = null;
        String operator = null;
        String operatorDisplayName = null;
        String value = "";
        int index = 0;
        
		displayName = element.getDisplayName();
        //	write to search URL buffer
        operator = element.getDefaultOperator();
        operatorDisplayName = SearchUtil.getOperatorDisplayName(element);
        if (operator.indexOf("EMPTY") < 0) {
            value = element.getSelectedDisplayValue();
        }
        searchURL.append(displayName + " " + operatorDisplayName + " " + value);
    }

    private final void setSearchResult(SearchBean searchBean,
        SearchInputBean inputBean) throws UIException {
        SearchResultBean searchResultBean = null;
        PortletSession portletSession = null;
        PortletData portletData = null;
        String traversalSize = null;
        ResultData resultData = null;
        PortletSettings portletSettings = null;
        String maxCols = null;
        String maxResults = null;
        int iMaxresults = 0;
        Object[] values = new Object[1];
        String message = null;
        String searchCriteria = null;
		long totalResultSize=0;
		
        portletData = request.getData();
        portletSettings = request.getPortletSettings();
		portletSession = request.getPortletSession();
        //get traversal size & max cols
        maxCols = portletSettings.getAttribute(IBaseSearch.MAX_SEARCH_COLS);
        traversalSize = (String) portletData.getAttribute(IBaseSearch.KEY_TRAVERSAL_SIZE);

        if (ValidationUtil.isNullOrEmpty(traversalSize)) {
            traversalSize = IBaseSearch.DEFAULT_TRAVERSAL_SIZE;
        }

        resultData = searchBean.getSearchResults();
		searchResultBean = (SearchResultBean) request.getPortletSession().getAttribute(IBaseSearch.KEY_SEARCH_RESULTS_BEAN);
		if(resultData!=null){
				totalResultSize=resultData.getTotalResultSize();
		}
		if(searchResultBean==null){
			searchResultBean = new SearchResultBean(resultData,	Integer.parseInt(traversalSize),totalResultSize);	
		}else{
			searchResultBean.setResultData(resultData);
			searchResultBean.setTraversalSize(Integer.parseInt(traversalSize));
			searchResultBean.setTotalResultSize(totalResultSize);
		}
        
        searchResultBean.setMaxCols(maxCols);

        //message formatting is done here since this is not part of uimessage tag
        searchCriteria = inputBean.getURL();
        values[0] = searchCriteria;
        message = getFormattedMessage(IBaseSearch.MSG_SEARCH_CRITERIA, values);
        searchResultBean.setSearchCriteria(message);
        
        if(resultData==null){
			message = UIUtils.getTextMessages(portletContext, request,
			ICollectionViewBean.ZERO_RECORDS_FOUND);
			searchResultBean.setResultMsg(message);
			searchResultBean.initialize();
        }
		
		//set the bean in session	
		portletSession.setAttribute(IBaseSearch.KEY_SEARCH_RESULTS_BEAN,searchResultBean);       
    }

    /**
     * This method returns the message used for alert portlet
     */
    public String getFormattedMessage(String key, Object[] obj) {
        String message = null;
        String formattedMessage = null;
        message = UIUtils.getTextMessages(portletContext, request, key);

        MessageFormat msgFormat = new MessageFormat(message);
        formattedMessage = msgFormat.format(obj);

        return formattedMessage;
    }

    /**
     * @return
     */
    public PortletContext getPortletContext() {
        return portletContext;
    }

    /**
     * @return
     */
    public PortletRequest getRequest() {
        return request;
    }

    /**
     * @return
     */
    public WPSContext getWpsContext() {
        return wpsContext;
    }

    /**
     * @param context
     */
    public void setPortletContext(PortletContext context) {
        portletContext = context;
    }

    /**
     * @param request
     */
    public void setRequest(PortletRequest request) {
        this.request = request;
    }

    /**
     * @param context
     */
    public void setWpsContext(WPSContext context) {
        wpsContext = context;
    }

    public int setQuickSearchDisplay() {
        String operatorValue = null;
        String elementValue = null;
        BaseSearchElement element = null;
        ArrayList list = null;
        String operator = null;
        int size = 0;
        int count = 0;
        String elementName = null;
        list = searchCriteriaBean.getAttributeList();
        size = list.size();

        for (int i = 0; i < size; i++) {
            element = (BaseSearchElement) list.get(i);
            elementName = element.getName();
            operator = elementName + "_OP";
            operatorValue = request.getParameter(operator);
            elementValue = request.getParameter(elementName);
			element.setSearchable(false);
			
            if (!ValidationUtil.isNullOrEmpty(operatorValue)) {
                if (!ValidationUtil.isNullOrEmpty(elementValue) ||
                        (operatorValue.indexOf("EMPTY") > 0)) {
                    element.setIsDisplayable(true);
					element.setSearchable(true);
                    count++;
                }
            }
        }

        return count;
    }

    /**
     * @return
     */
    public SearchCriteriaBean getSearchCriteriaBean() {
        return searchCriteriaBean;
    }

    /**
     * @param bean
     */
    public void setSearchCriteriaBean(SearchCriteriaBean bean) {
        searchCriteriaBean = bean;
    }

    public void populateSearchCriteria()
        throws UIException, WPCMediatorException {
        PortletSettings portletSettings = null;
        PortletSession portletSession = null;
		PortletData portletData = null;
        ArrayList attributeList = null;
        Iterator attrItr = null;
        BaseSearchElement element = null;
        String sMaxCol = null;
        int iMaxCol = 0;
        ArrayList metaDataList = null;
        ResultMetaData resultMetaData = null;
        PortletContext context = null;
        ISaveSearch history = null;
        ArrayList previousSearch = null;
        String sMaxSearchHistory = null;
		String traversalSize = null;
        Object[] values = new Object[1];
        String searchHistoryMsg = null;
        WPSContext wpsContext = null;
        SearchHelper searchHelper = null;
        String catalogId = null;
        int listBoxIndex = -1;

        portletSettings = request.getPortletSettings();
        sMaxCol = (String) portletSettings.getAttribute(IBaseSearch.MAX_SEARCH_COLS);
        sMaxSearchHistory = portletSettings.getAttribute(IBaseSearch.MAX_SEARCH_HISTORY);
        portletSession = request.getPortletSession();

        // set portletrequest and context for getting localised text operators
        context = getPortletContext();
        searchBean.setPortletRequest(request);
        searchBean.setContext(context);
        wpsContext = getWpsContext();
        searchBean.setWpsContext(wpsContext);

        try {
            attributeList = searchBean.getSearchCriteriaElements();
            metaDataList=searchBean.getSearchResultElements();
        } catch (WPCMediatorException e) {
            e.printStackTrace();
            throw e;
        }
        
        searchCriteriaBean.setAttributeList(attributeList);
        searchCriteriaBean.setMetaDataList(metaDataList);
        
		//set traversl size
		 portletData = request.getData();
		 traversalSize = (String) portletData.getAttribute(IBaseSearch.KEY_TRAVERSAL_SIZE);

		 if (ValidationUtil.isNullOrEmpty(traversalSize)) {
			  traversalSize = IBaseSearch.DEFAULT_TRAVERSAL_SIZE;
		 }

        searchCriteriaBean.setTraversalSize(traversalSize);
        searchHelper = new SearchHelper();
        searchHelper.setPortletContext(context);
        searchHelper.setRequest(request);

        //load search history
        history = (ISaveSearch) SearchUtil.getClass(IBaseSearch.CLASS_NAME_FILE_SAVED_SEARCH);
        history.setUserId(request.getUser().getUserID());
        catalogId = UIUtils.getCatalogName(wpsContext.getWpcUser());
        history.setFolder(SearchUtil.getHistoryFolder(portletContext,
                portletName, catalogId));
        previousSearch = history.loadSearch();

        //	remove search hostory if it is larger than config setting
        //this case is possibe when portal admin reduces the seach history size
        updateSearchHistory(portletSettings, previousSearch);
        searchCriteriaBean.setPreviousSeach(previousSearch);

        //	set the message
        values[0] = sMaxSearchHistory;
        searchHistoryMsg = searchHelper.getFormattedMessage(IBaseSearch.MSG_SEARCH_HISTORY,
                values);

        searchCriteriaBean.setPreviousSearchMsg(searchHistoryMsg);
        portletSession.setAttribute(IBaseSearch.KEY_SEARCH_CRITERIA_BEAN,
            searchCriteriaBean);
            
        if(previousSearch!=null){
			portletSession.setAttribute(IBaseSearch.KEY_SEARCH_ID,(previousSearch.size()-1)+ "");
        }else{
			portletSession.setAttribute(IBaseSearch.KEY_SEARCH_ID,"0");	
        }
		
    }

    /**
             * @param previousSearch
             */
    private void updateSearchHistory(PortletSettings portletSettings,
        ArrayList previousSearch) {
        // remove search hostory if it is larger than config setting
        //this case is possibe when portal admin reduces the seach history size
        String sMaxSavedSearchs = null;
        int iMaxSavedSearchs = 0;
        int count = 0;

        if (!ValidationUtil.isNullOrEmpty(previousSearch)) {
            sMaxSavedSearchs = portletSettings.getAttribute(IBaseSearch.MAX_SEARCH_HISTORY);
            iMaxSavedSearchs = Integer.parseInt(sMaxSavedSearchs);
            count = previousSearch.size();

            if (count > iMaxSavedSearchs) {
                for (int i = 0; count > iMaxSavedSearchs; --count) {
                    previousSearch.remove(i);
                }
            }
        }
    }

    /**
     * @return
     */
    public String getPortletName() {
        return portletName;
    }

    /**
     * @param string
     */
    public void setPortletName(String string) {
        portletName = string;
    }

    /**
     * @return
     */
    public SearchBean getSearchBean() {
        return searchBean;
    }

    /**
     * @param bean
     */
    public void setSearchBean(SearchBean bean) {
        searchBean = bean;
    }

	
	public void displayAllAttributes() {
		SearchCriteriaBean searchCriteriaBean=null;
		PortletSession portletSession=null;
		ArrayList attributeList=null;
		ArrayList metaDataList=null;
		BaseSearchElement baseSearchElement=null;
		ResultMetaData resultMetaData=null;
		
		portletSession=request.getPortletSession(); 
		searchCriteriaBean = (SearchCriteriaBean) portletSession.getAttribute(IBaseSearch.KEY_SEARCH_CRITERIA_BEAN);	
		attributeList=searchCriteriaBean.getAttributeList();
		metaDataList=searchCriteriaBean.getMetaDataList();
		
		for(int i=0;i<attributeList.size();i++){
			baseSearchElement=(BaseSearchElement) attributeList.get(i);
			baseSearchElement.setIsDisplayable(true);
		}
		
		for(int i=0;i<metaDataList.size();i++){
			resultMetaData=(ResultMetaData) metaDataList.get(i);
			resultMetaData.setDisplayable(true);
		}
	}
	
	public void displayUserSetAttributes() {
		PortletSession portletSession=null;
		PortletData portletData=null;
		ArrayList attributeList=null;
		ArrayList metaDataList=null;
		BaseSearchElement element=null;
		String elementName1 = null;
		String elementName2 = null;
		ResultMetaData resultMetaData=null;
		String catalogId=null;		
		String tokenizedCriteria=null;
		String tokenizedResult=null;
		StringTokenizer tokenizer = null;
		HashMap attributeMap = null;
		String stringToken = null;
		int index = -1;
		int count=0;
		String attributeName = null;
		Integer listBoxIndex = null;
		Object value = null;
		
		portletSession=request.getPortletSession(); 
		searchCriteriaBean = (SearchCriteriaBean) portletSession.getAttribute(IBaseSearch.KEY_SEARCH_CRITERIA_BEAN);
		catalogId=searchCriteriaBean.getCatalogId();
		portletData=request.getData();
		tokenizedCriteria=(String) portletData.getAttribute(IBaseSearch.PREFIX_SEARCH_CRITERIA + catalogId );
		tokenizedResult=(String) portletData.getAttribute(IBaseSearch.PREFIX_SEARCH_RESULT+ catalogId );
		attributeList=searchCriteriaBean.getAttributeList();
		metaDataList=searchCriteriaBean.getMetaDataList();
		
		//set search citeria display
		if(!ValidationUtil.isNullOrEmpty(tokenizedCriteria)){
		
		for (int i = 0; i < attributeList.size(); i++) {
			element = (BaseSearchElement) attributeList.get(i);
			element.setIsDisplayable(false);
		}
		
			tokenizer = new StringTokenizer(tokenizedCriteria,ISearch.ATTRIBUTE_NAME_TOKENIZER);
			attributeMap = new HashMap();

			while (tokenizer.hasMoreTokens()) {
				stringToken = tokenizer.nextToken();
				index = stringToken.indexOf(ISearch.ATTRIBUTE_INDEX_TOKENIZER);
				attributeName = stringToken.substring(0, index);
				listBoxIndex = Integer.valueOf(stringToken.substring(index + 1));
				attributeMap.put(attributeName, listBoxIndex);
			}

			for (int i = 0; i < attributeList.size(); i++) {
				element = (BaseSearchElement) attributeList.get(i);
				attributeName = element.getName();
				value = attributeMap.get(attributeName);

				if (value != null) {
					listBoxIndex = (Integer) value;
					element.setIsDisplayable(true);
					element.setListBoxIndex(listBoxIndex.intValue());
					count++;
				}
			}
			
			//If attribute is removed form spec re assign the index
	 		if(count<attributeMap.size()){
	 			int j=0;
				for (int i = 0; i < attributeList.size(); i++) {
					element = (BaseSearchElement) attributeList.get(i);
					if(element.getIsDisplayable()){
						element.setListBoxIndex(j++);
					}
				}
	 		}
	 		
		}
		
		//set search result display
		if(!ValidationUtil.isNullOrEmpty(tokenizedResult)){
				for (int i = 0; i < metaDataList.size(); i++) {
					resultMetaData = (ResultMetaData) metaDataList.get(i);
					resultMetaData.setDisplayable(false);
				}
		
					tokenizer = new StringTokenizer(tokenizedResult,ISearch.ATTRIBUTE_NAME_TOKENIZER);
					attributeMap = new HashMap();

					while (tokenizer.hasMoreTokens()) {
						stringToken = tokenizer.nextToken();
						index = stringToken.indexOf(ISearch.ATTRIBUTE_INDEX_TOKENIZER);
						attributeName = stringToken.substring(0, index);
						listBoxIndex = Integer.valueOf(stringToken.substring(index + 1));
						attributeMap.put(attributeName, listBoxIndex);
					}
					count=0;
					for (int i = 0; i < metaDataList.size(); i++) {
						resultMetaData = (ResultMetaData) metaDataList.get(i);
						attributeName = resultMetaData.getName();
						value = attributeMap.get(attributeName);

						if (value != null) {
							listBoxIndex = (Integer) value;
							resultMetaData.setDisplayable(true);
							resultMetaData.setListBoxIndex(listBoxIndex.intValue());
							count++;
						}
					}
					
			//If attribute is removed form spec re assign the index
			  if(count<attributeMap.size()){
				int j=0;
				for (int i = 0; i < metaDataList.size(); i++) {
					  resultMetaData = (ResultMetaData) metaDataList.get(i);
					  if(resultMetaData.isDisplayable()){
						resultMetaData.setListBoxIndex(j++);
					  }
				  }
			 }
	 
		  }
		}
	public int filterSearchCriteria() {
		String operatorValue = null;
		String elementValue = null;
		BaseSearchElement element = null;
		ArrayList list = null;
		String operator = null;
		int size = 0;
		int count = 0;
		String elementName = null;
		list = searchCriteriaBean.getAttributeList();
		size = list.size();

		for (int i = 0; i < size; i++) {
			element = (BaseSearchElement) list.get(i);
			elementName = element.getName();
			operator = elementName + "_OP";
			operatorValue = request.getParameter(operator);
			elementValue = request.getParameter(elementName);
			element.setSearchable(false);
			if (!ValidationUtil.isNullOrEmpty(operatorValue)) {
				if (!ValidationUtil.isNullOrEmpty(elementValue) ||
					(operatorValue.indexOf("EMPTY") > 0)) {
							element.setSearchable(true);
							count++;
						}
					}
				}

		return count;
	}
	
}
