
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * Created on Dec 2, 2004
 */
package com.ibm.ccd.ui.portlets.search.core;

import java.io.Serializable;
import java.util.ArrayList;

import com.ibm.ccd.ui.taglib.dataobjects.ResultData;


/**
 * This class provides implementation for the <code>IBaseSearch</code> interface.
 */
public class SearchResultBean implements  Serializable {
    //public static final String DEFAULT_DELIMITER = "&";
    //public static String str = "1,Book,Java Book,100,1000&2,Book,XML Book,101,1001&3,Book,Java2 Book,103,1003&4,Book,XSL Book,104,1004&5,Book,XUL Book,105,1005";

    /**
     * Contains the search criteria f
     */
    String searchCriteria = null;

    /**
     * Indicates the form action URI
     */
    private String formActionURI = null;

    /**
     * Indicates the next action URI
     */
    private String nextLinkURI = null;

    /**
     * Indicates the last action URI
     */
    private String lastLinkURI = null;

    /**
     * Indicates the first action URI
     */
    private String firstLinkURI = null;

    /**
     * Indicates the previous action URI
     */
    private String previousLinkURI = null;
    

    /**
     * collection for result data
     */
    private ResultData resultData = null;
    

    /**
     * Identifier for  start Index
     */
    private long startIndex = 0;

    /**
     * Identifier for end Index
     */
    private long endIndex = 0;

    /**
     * Identifer for Traversal size
     */
    private int traversalSize;

    /**
    * Identifier for end Index
    */
    private long totalResultSize = 0;

    /**
     * Indicates the work with item status
     */
    private String workWithItemURI = null;

    /**
     * Indicates max displayable cols from config settings
     *
     */
    private String maxCols = null;

    /**
     * info message to user when max allowed results are displayed
     */
    private String resultMsg;

    /**
     * Default Constructor
     */
    public SearchResultBean() {
        //super();
    }

    /**
     * Provide a minimum info of the results to work with, and the traversal size
     * @param ResultData represemtimg the collection to work with
     * @param int representing the traversal size
     */
    public SearchResultBean(ResultData data, int aTraversalSize,long count){
        if (data != null) {
            this.resultData = data;
			this.totalResultSize=count;
            if (aTraversalSize > 0) {
                this.traversalSize = aTraversalSize;
                initialize();
            }
        }
    }
    
    public SearchResultBean(ResultData data,int traversalSize){
		this.resultData = data;
		this.traversalSize = traversalSize;
    }

    /**
     * Initialize the helper with default values
     */
    public void initialize() {
       startIndex=0;
       if(resultData!=null){
       	   if(totalResultSize>traversalSize){
       	   		endIndex=traversalSize;
       	   }else{
       	   	endIndex=totalResultSize;
       	   }
       }
    }

    /**
     * @see ICollectionViewBean#getResultsMetaData()
     * @return ArrayList
     */
    public ArrayList getResultsMetaData() {
        return getResultData().getResultMetaData();
    }

    /**
     * @see ICollectionViewBean#getResultsRows()
     * @return ArrayList
     */
    public ArrayList getResultsRows() {
        ArrayList rows = new ArrayList(getResultData().getResultRowData() );
        return rows;
    }

    /**
     * @see ICollectionViewBean#hasNextResults()
     * @return  boolean
     */
    public boolean hasNextResults() {
        return ( endIndex < totalResultSize);
    }

    /**
     * @see ICollectionViewBean#hasPreviousResults()
     * @return  boolean
     */
    public boolean hasPreviousResults() {
        return ((startIndex - traversalSize) >= 0);
    }

   

    /**
     * @see ICollectionViewBean#getStartIndex()
     * @return int representing start size
     */
    public long getStartIndex() {
        return  (startIndex + 1);
    }

    /**
     * @see ICollectionViewBean#getEndIndex()
     * @return int representing end index
     */
    public long getEndIndex() {
    	long value=0;
    	if(endIndex >totalResultSize){
			value=totalResultSize;
    	}else{
			value=endIndex;
    	}
        return value;
    }

    /**
     * @see IViewBean#isMandatory(String)
     * @param String
     * @return boolean
     */
    public boolean isMandatory(String fieldID) {
        return false;
    }

    /**
     * @see IViewBean#isVisible(String)
     * @param String
     * @return boolean
     */
    public boolean isVisible(String fieldID) {
        return false;
    }



    /**
     * Gets the resultData
     * @return Returns a ResultData
     */
    public ResultData getResultData() {
        return resultData;
    }

    /**
     * Sets the resultData
     * @param resultData The resultData to set
     */
    public void setResultData(ResultData resultData) {
        this.resultData = resultData;

        //initialize();
    }

    

    /**
     * Sets the endIndex
     * @param endIndex The endIndex to set
     */
    public void setEndIndex(long endIndex) {
        this.endIndex = endIndex;
    }

    /**
     * Sets the startIndex
     * @param startIndex The startIndex to set
     */
    public void setStartIndex(long startIndex) {
        this.startIndex = startIndex;
    }

    /**
     * Gets the traversalSize
     * @return Returns a int
     */
    public int getTraversalSize() {
        return traversalSize;
    }

    /**
     * Sets the traversalSize
     * @param traversalSize The traversalSize to set
     */
    public void setTraversalSize(int traversalSize) {
        this.traversalSize = traversalSize;
    }

    /**
     * @return
     */
    public String getFormActionURI() {
        return formActionURI;
    }

    /**
     * @param string
     */
    public void setFormActionURI(String string) {
        formActionURI = string;
    }

    /**
     * @return
     */
    public String getNextActionURI() {
        return nextLinkURI;
    }

    /**
     * @param string
     */
    public void setNextActionURI(String string) {
        nextLinkURI = string;
    }

    /**
     * @return
     */
    public String getPreviousLinkURI() {
        return previousLinkURI;
    }

    /**
     * @param string
     */
    public void setPreviousLinkURI(String string) {
        previousLinkURI = string;
    }

    /**
     * @return
     */
    public String getFirstActionURI() {
        return firstLinkURI;
    }

    /**
     * @param string
     */
    public void setFirstActionURI(String string) {
        firstLinkURI = string;
    }

    /**
     * @return
     */
    public String getLastActionURI() {
        return lastLinkURI;
    }

    /**
     * @param string
     */
    public void setLastActionURI(String string) {
        lastLinkURI = string;
    }
   

    /**
     * @return
     */
    public String getWorkWithItemURI() {
        return workWithItemURI;
    }

    /**
     * @param string
     */
    public void setWorkWithItemURI(String string) {
        workWithItemURI = string;
    }

    /**
     * @return
     */
    public String getMaxCols() {
        return maxCols;
    }

    /**
     * @param string
     */
    public void setMaxCols(String string) {
        maxCols = string;
    }

    /**
     * @return
     */
    public String getSearchCriteria() {
        return searchCriteria;
    }

    /**
     * @param string
     */
    public void setSearchCriteria(String string) {
        searchCriteria = string;
    }

    /**
     * @return
     */
    public String getResultMsg() {
        return resultMsg;
    }

    /**
     * @param string
     */
    public void setResultMsg(String string) {
        resultMsg = string;
    }
	/**
	 * @return
	 */
	public long getTotalResultSize() {
		return totalResultSize;
	}

	/**
	 * @param i
	 */
	public void setTotalResultSize(long i) {
		totalResultSize = i;
	}

	public void firstPage(){
		startIndex=0;
		endIndex=startIndex+ traversalSize;	
	}
	
	public void previousPage(){
		startIndex=startIndex-traversalSize;
		if(endIndex%traversalSize!=0){
			endIndex=endIndex- (endIndex%traversalSize);
		}else{
			endIndex=endIndex-traversalSize;	
		}
		
	}
	
	public void nextPage(){
		startIndex=startIndex+ traversalSize;
		endIndex=endIndex+traversalSize;
		if(endIndex>=totalResultSize){
			endIndex=totalResultSize;
		}
	}
	
	public void lastPage(){
		endIndex=totalResultSize;
		if(endIndex%traversalSize!=0){
			startIndex=endIndex- (endIndex%traversalSize);
		}else{
			startIndex=endIndex-traversalSize;
		}
	}
}
