
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.portlets.search.core;

import java.io.IOException;
import java.io.StringReader;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.ibm.ccd.ui.exception.UIException;
import com.ibm.ccd.ui.mediator.ISoapMessageKeys;


/**
 * A helper class for search result xml parsing using the SAX Parser
 *
 */
public class SearchResultParser extends DefaultHandler {
   
    /**
     * Represents an empty string.
     */
    private static final String EMPTY = ""; //$NON-NLS-1$

    /**
     * Contains CSV from xml
     */
    private StringBuffer csv = null;

    /**
     * Conatains the column names
     */
    private String metaData = null;

    /**
     * Contains the data
     */
    private String data = null;

    /**
     * Contains the event name.
     */
    private String eventName = null;
    
    /**
     * Total result size of search
     */
	private StringBuffer totalResultSize=null;

    /**
     * Receives the startElement event
     * @param namespaceURI string
     * @param localName string
     * @param qName string
     * @param atts Attributes object
     * @throws org.xml.sax.SAXException
     */
    public void startElement(String namespaceURI, String localName,
        String qName, Attributes atts) throws SAXException {
        eventName = qName;
    }

    /**
     * This method processes the event based on conditions.
     * @param ch character array
     * @param start starting index of the char[]
     * @param length lengthof the char[]
     * @throws org.xml.sax.SAXException
     */
    public void characters(char[] ch, int start, int length)
        throws SAXException {
        if (ISoapMessageKeys.SEARCH_CSV_RESULTS_NAME.equals(eventName)) {
            String temp = null;
            temp = new String(ch, start, length);

            if (csv == null) {
                csv = new StringBuffer(temp);
            } else {
                csv.append(temp);
            }
        }
        
        else if(ISoapMessageKeys.SEARCH_ROW_COUNT_NAME.equals(eventName)){
			String temp = null;
			temp = new String(ch, start, length);
			
			if (totalResultSize == null) {
				totalResultSize = new StringBuffer(temp);
			} else {
				totalResultSize.append(temp);
			}
        }
    }

    /**
     * This method parses the XML
     * @param xmlString string
     * @throws UIException
     */
    public void parse(String xmlString) throws UIException {
        SAXParser saxParser;

        // Use the default (non-validating) parser
        SAXParserFactory factory = SAXParserFactory.newInstance();

        // Specifies that the parser produced by this code will provide support for XML namespaces.
        factory.setNamespaceAware(true);

        // Specifies that the parser produced by this code will validate documents as they are parsed.
        factory.setValidating(false);

        // Parse the input 
        try {
            saxParser = factory.newSAXParser();
            saxParser.parse(new InputSource(new StringReader(xmlString)), this);
            splitCsv();
        } catch (SAXException saxException) {
            throw new UIException(saxException);
        } catch (ParserConfigurationException exception) {
            throw new UIException(exception);
        } catch (IOException ioException) {
            throw new UIException(ioException);
        }
    }

    /**
     * This method returns the meta data string which forms the table headings.
     * @param dataString data string
     * @return getMetaDataString the metaData string
     */
    public String getMetaData() {
        return metaData;
    }

    /**
     * This method returns the data string which forms the table values.
     * @param dataString data string
     * @return data the data string
     */
    public String getData() {
        return data;
    }

    /**
     * Returns the event name
     * @return eventName
     */
    public String getEventName() {
        return eventName;
    }

    /**
     * Set the event name
     * @param string
     */
    public void setEventName(String string) {
        eventName = string;
    }

    private void splitCsv() {
        String csvStr = null;

        if (csv != null) {
            csvStr = csv.toString();

            //get first row from CSV and assign to meta data
            String delimiter = ICollectionViewBean.ROW_DELIMITER;
            int index = csvStr.indexOf(delimiter);
            metaData = csv.substring(0, index);

            //get data string from CSV
            index = index + delimiter.length();
            data = csv.substring(index, csv.length());
        }
    }
	/**
	 * @return
	 */
	public StringBuffer getTotalResultSize() {
		return totalResultSize;
	}

	/**
	 * @param string
	 */
	public void setTotalResultSize(StringBuffer data) {
		totalResultSize = data;
	}

}
