
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * Created on Jul 31, 2005
 */
package com.ibm.ccd.ui.portlets.search.core.cmd;

import java.util.ArrayList;

import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletSession;

import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.exception.UIException;
import com.ibm.ccd.ui.mediator.WPCMediatorException;
import com.ibm.ccd.ui.portlets.search.core.IBaseSearch;
import com.ibm.ccd.ui.portlets.search.core.SearchBean;
import com.ibm.ccd.ui.portlets.search.core.SearchCriteriaBean;
import com.ibm.ccd.ui.portlets.search.core.SearchHelper;
import com.ibm.ccd.ui.portlets.search.core.SearchInputBean;
import com.ibm.ccd.ui.portlets.search.core.SearchUtil;
import com.ibm.ccd.ui.taglib.dataobjects.BaseSearchElement;
import com.ibm.ccd.ui.util.UIUtils;


public class QuickSearchCmd implements ISearchCommand {
	
	protected PortletContext portletContext = null;
	protected WPSContext wpsContext = null;
    protected String portletName = null;
	protected SearchHelper helper=null;

    public QuickSearchCmd() {
        super();
    }

    public void execute(PortletRequest request, PortletResponse response)
        throws UIException, WPCMediatorException {
        SearchCriteriaBean criteriaBean = null;
        PortletSession session = null;
        SearchInputBean inputBean = null;
        ArrayList attributeList = null;
		ArrayList metaDataList=null;
        ArrayList quickSearchAttrList = null;
        BaseSearchElement element = null;
        MessageMap messageMap = null;
        String currentCatalogId=null;
        int count = 0;
		SearchBean searchBean=null;
		
        messageMap = SearchUtil.getMessageMap(request);
        session = request.getPortletSession();
		
        criteriaBean = (SearchCriteriaBean) session.getAttribute(IBaseSearch.KEY_SEARCH_CRITERIA_BEAN);
		session.setAttribute(IBaseSearch.KEY_SEARCH_RESULTS_BEAN,null);
        attributeList = criteriaBean.getAttributeList();
        metaDataList =criteriaBean.getMetaDataList();
		
		currentCatalogId=UIUtils.getCatalogName(wpsContext.getWpcUser());
        //clone the list to avoid unsetting advanced search settings
        quickSearchAttrList = new ArrayList();

        for (int i = 0; i < attributeList.size(); i++) {
            element = (BaseSearchElement) attributeList.get(i);

            try {
                element = (BaseSearchElement) element.clone();
                element.setIsDisplayable(false);
            } catch (CloneNotSupportedException e1) {
                e1.printStackTrace();
            }

            quickSearchAttrList.add(element);
        }
		
		criteriaBean = new SearchCriteriaBean();
		searchBean= new SearchBean();
		helper = new SearchHelper();
		
		criteriaBean.setAttributeList(quickSearchAttrList);
		criteriaBean.setMetaDataList(metaDataList);
		criteriaBean.setCatalogId(currentCatalogId);
		
		helper.setSearchBean(searchBean);
        helper.setPortletContext(portletContext);
        helper.setRequest(request);
        helper.setWpsContext(wpsContext);
        helper.setSearchCriteriaBean(criteriaBean);
        helper.setPortletName(portletName);
        
        count = helper.setQuickSearchDisplay();

        if (count == 0) {
            messageMap.addInfoMessage(IBaseSearch.MSG_NO_SEARCH_CRITERIA);
            request.setAttribute("stabNo", "1");

            return;
        }

        helper.setRequestValues();
        inputBean = helper.getSearchInput();
        search(inputBean);
        helper.saveSearch(inputBean);
        request.setAttribute("stabNo", "2");
    }
    
	protected void search(SearchInputBean searchInputBean)
		   throws UIException, WPCMediatorException {
		   helper.performSearch(searchInputBean);
	  }

    public WPSContext getWpsContext() {
        return wpsContext;
    }

    public void setWpsContext(WPSContext wpsContext) {
        this.wpsContext = wpsContext;
    }

    public PortletContext getPortletContext() {
        return portletContext;
    }

    public void setPortletContext(PortletContext portletContext) {
        this.portletContext = portletContext;
    }

    public void setPortletName(String portletName) {
        this.portletName = portletName;
    }

    public String getPortletName() {
        return portletName;
    }
}
