
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * This package consists of classes pertaining to User Registration
 * related functionality
 */
package com.ibm.ccd.ui.portlets.user;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Vector;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletSession;
import org.apache.jetspeed.portlet.PortletSettings;
import org.apache.jetspeed.portlet.PortletURI;
import org.apache.jetspeed.portlet.UnavailableException;
import org.apache.jetspeed.portlet.event.ActionEvent;
import org.apache.jetspeed.portlet.event.ActionListener;

import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.portlets.BasePortlet;
import com.ibm.ccd.ui.services.user.UserBean;
import com.ibm.ccd.ui.services.user.UserManager;
import com.ibm.ccd.ui.util.IReference;


/**
 * LdapNavigationPortlet is used for Directory Server Navigation.
 */
public class UserApprovalPortlet extends BasePortlet implements ActionListener {
    /**  JSP file name to be rendered on the view mode
     */
    public static final String VIEW_JSP = "/jsps/html/userRegistration/UserApprovalPortletView.";

    /** Bean name for the view mode request
     */
    public static final String VIEW_BEAN = "userapproval.UserApprovalPortletViewBean";

    /** Parameter name for Business function attribute
     */
    public static final String USER_APPROVAL = "USER_APPROVAL";

    /** Parameter name for IBM Tivoli Directory Server Web Admin tool
     */
    public static String LDAP_WEB_TOOL_URL = "";

    //"http://9.182.74.51:9080/IDSWebApp/IDSjsp/Login.jsp";

    /** Parameter name for Ldap not configured
     */
    public static final String LDAP_CONFIG_ERROR = "MESSAGE_CODE_255";

    /** Parameter name for Ldap Configuration Info
     */
    public static final String LDAP_CONFIG_INFO = "LDAP_CONFIG_INFO";

    /** Parameter name for Ldap not Configured Info
     */
    public static final String LDAP_NOT_CONFIGURED = "LDAP_NOT_CONFIGURED";

    /** Parameter name for Ldap not Configured Info
     */
    public static final String LDAP_CONFIGURED = "LDAP_CONFIGURED";

    /** Parameter name for Ldap not Configured Info
     */
    public static final String LDAP_USER_LIST = "LDAP_USER_LIST";

    /** Parameter name for User(s) Approve button
     */
    public static final String APPROVE_USER = "userapproval.UserApprovalPortletApprove";

    /** Variables for User related functionality
     */
    boolean isLdapConfigured = false;
    private UserManager userManager = null;

    /** Initialize String  array's for User(s)
     */
    String[] usersArray = null;

    // Added for wpcUserName which needs to be added for WPC
    String[] wpcUsrNameArray = null;
    String[] presentCheckedUsers = null;
    String[] prevDisapprovedUsers = null;
    String[] prevApprovedUsers = null;

    /**
     * @see org.apache.jetspeed.portlet.Portlet#init(PortletConfig)
     */
    public void init(PortletConfig portletConfig) throws UnavailableException {
        super.init(portletConfig);
    }

    /**
     * @see org.apache.jetspeed.portlet.Portlet#init(PortletConfig)
     */
    public void initConcrete(PortletSettings arg) throws UnavailableException {
        if (!isLdapConfigured) {
            userManager = UserManager.getInstance();

            if (userManager != null) {
                isLdapConfigured = true;
            }
        }
    }

    /**
     * @see com.ibm.ccd.ui.portlets.BasePortlet#setBusinessFunction(org.apache.jetspeed.portlet.PortletRequest)
     */
    protected void setBusinessFunction(PortletRequest portletRequest) {
        PortletSession session = portletRequest.getPortletSession();

        if (session.getAttribute(IReference.SESSION_BUS_FUNCT) == null) {
            session.setAttribute(IReference.SESSION_BUS_FUNCT, USER_APPROVAL);
        }
    }

    /**
     * @see org.apache.jetspeed.portlet.PortletAdapter#doView(PortletRequest, PortletResponse)
     */
    public void doView(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        /** Set the Message map for LDAP Acess Info
         */
        MessageMap msgMap = null;

        try {
            msgMap = getMessageMap(request);
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        if (!isLdapConfigured) {
            msgMap.addErrorMessage(LDAP_CONFIG_ERROR);
            request.setAttribute(LDAP_CONFIG_INFO, LDAP_NOT_CONFIGURED);
        } else {
            request.setAttribute(LDAP_CONFIG_INFO, LDAP_CONFIGURED);
        }

        // Initialize the Business function
        setBusinessFunction(request);

        // Make a view mode bean
        UserApprovalPortletViewBean viewBean = new UserApprovalPortletViewBean();
        request.setAttribute(VIEW_BEAN, viewBean);

        // Set actionURI in the view mode bean
        PortletURI formActionURI = response.createURI();
        formActionURI.addAction(APPROVE_USER);
        viewBean.setFormActionURI(formActionURI.toString());

        //Acess the User Manager & get the Portal User List
        Vector userList = null;
        userList = userManager.getSupplierPortalUserList();
        System.out.println("UserApprovalPortlet :: doView : Users List ===>> " +
            userList);
        request.setAttribute(LDAP_USER_LIST, userList);

        // Invoke the JSP to render        		
        getPortletConfig().getContext().include(VIEW_JSP +
            getJspExtension(request), request, response);
    }

    /**
     * @see org.apache.jetspeed.portlet.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent event) throws PortletException {
        //	Add action string handler here
        PortletRequest request = event.getRequest();

        MessageMap msgMap = null;

        try {
            msgMap = getMessageMap(request);
        } catch (Exception e) {
            e.printStackTrace();
        }

        // ActionEvent handler
        String actionString = event.getActionString();

        // Trigger the User Approval Ation
        if (APPROVE_USER.equals(actionString)) {
            try {
                approveSelectedUsers(request);
            } catch (Exception e) {
                msgMap.addErrorMessage("MESSAGE_CODE_112",
                    "An Exception has occured in User Approval");
                e.printStackTrace();
            }
        }
    }

    /**
     * Returns the file extension for the JSP file
     *
     * @param request PortletRequest
     * @return JSP extension
     */
    private static String getJspExtension(PortletRequest request) {
        String markupName = request.getClient().getMarkupName();

        return "jsp";
    }

    /** approveSelectedUsers: Does the Approval and Disapproval of User(s)
     * @param request
     * @returns None
     */
    public void approveSelectedUsers(PortletRequest request)
        throws Exception {
        // Extract the array of User(s) 
        usersArray = request.getParameterValues("USERS");
        wpcUsrNameArray = request.getParameterValues("WPC_USER_NAME");
        prevApprovedUsers = request.getParameterValues("DEFAULT_CHECKED_STATE");
        prevDisapprovedUsers = request.getParameterValues(
                "DEFAULT_UNCHECKED_STATE");
        presentCheckedUsers = request.getParameterValues("CHK_USER_ID");

        // Initialize the User(s) ArrayList for Approval & Disapproval 				
        ArrayList approveUsersList = new ArrayList();
        ArrayList disapproveUsersList = new ArrayList();

        // Loop through the user(s) to find whether Approve/Disapprove 		
        for (int iUsers = 0; iUsers < usersArray.length; iUsers++) {
            boolean blnPrevApproved = false;
            boolean blnPrevDisapproved = false;
            boolean blnPresentCheckdState = false;

            //	Create the User object
            UserBean usrBean = new UserBean();
            usrBean.setUserName(usersArray[iUsers].trim());
            usrBean.setWpcUserName(wpcUsrNameArray[iUsers].trim());

            // Previously Checked & Presently Unchecked User(s)
            blnPrevApproved = previousApprovedUser(request,
                    usersArray[iUsers].trim());

            if (blnPrevApproved) {
                blnPresentCheckdState = presentUserStatus(request,
                        usersArray[iUsers].trim());

                if (!blnPresentCheckdState) {
                    disapproveUsersList.add(usersArray[iUsers].trim());
                } else {
                    approveUsersList.add(usrBean);
                }
            }

            // Previously Unchecked & Presently Checked User(s)
            blnPrevDisapproved = previousDisapprovedUser(request,
                    usersArray[iUsers].trim());

            if (blnPrevDisapproved) {
                blnPresentCheckdState = presentUserStatus(request,
                        usersArray[iUsers].trim());

                if (blnPresentCheckdState) {
                    //approveUsersList.add(usersArray[iUsers].trim());
                    approveUsersList.add(usrBean);
                }
            }
        } //end of for Loop

        System.out.println(
            "UserApprovalPortlet :: approveSelectedUsers : Final Approve ===>> " +
            approveUsersList);
        System.out.println(
            "UserApprovalPortlet :: approveSelectedUsers : Final DisApprove ===>> " +
            disapproveUsersList);

        //	Send the List of User(s) to be Approved and Disapproved by the Administrator
        userManager.approveSupplierPortalUsers(approveUsersList);
        userManager.disapproveSupplierPortalUsers(disapproveUsersList);
    }

    /**previousApprovedUser: Checks the present User Status and if Checked returns true else returns false
     * @param request
     * @param  userId
     * @return blnPrevApproved
     */
    private boolean previousApprovedUser(PortletRequest request, String userId) {
        boolean blnPrevApproved = false;

        // Start of Loop 
        for (int i = 0; i < prevApprovedUsers.length; i++) {
            if ((prevApprovedUsers[i].trim()).equals(userId)) {
                blnPrevApproved = true;

                return blnPrevApproved;
            }
        } //End of Loop

        return blnPrevApproved;
    }

    /**previousDisapprovedUser: Checks the present User Status and if Checked returns true else returns false
     * @param request
     * @param userId
     * @return blnPrevDisapproved
     */
    private boolean previousDisapprovedUser(PortletRequest request,
        String userId) {
        boolean blnPrevDisapproved = false;

        // Start of Loop 
        for (int i = 0; i < prevDisapprovedUsers.length; i++) {
            if ((prevDisapprovedUsers[i].trim()).equals(userId)) {
                blnPrevDisapproved = true;

                return blnPrevDisapproved;
            }
        } //End of Loop

        return blnPrevDisapproved;
    }

    /**presentUserStatus: Checks the present User Status and if Checked returns true else returns false
     * @param request
     * @param userId
     * @return blnPresentStaus
     */
    private boolean presentUserStatus(PortletRequest request, String userId) {
        boolean blnPresentStaus = false;

        //	Start of Loop 
        for (int i = 0; i < presentCheckedUsers.length; i++) {
            if ((presentCheckedUsers[i].trim()).equals(userId)) {
                blnPresentStaus = true;

                return blnPresentStaus;
            }
        } //End of Loop

        return blnPresentStaus;
    }
}
