
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * This package consists of classes pertaining to User Registration
 * related functionality
 */
package com.ibm.ccd.ui.portlets.user;

import java.io.IOException;
import java.util.Vector;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletSession;
import org.apache.jetspeed.portlet.PortletSettings;
import org.apache.jetspeed.portlet.PortletURI;
import org.apache.jetspeed.portlet.UnavailableException;
import org.apache.jetspeed.portlet.event.ActionEvent;
import org.apache.jetspeed.portlet.event.ActionListener;

import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.portlets.BasePortlet;
import com.ibm.ccd.ui.services.user.UserBean;
import com.ibm.ccd.ui.services.user.UserManager;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;


/**
 * User Registration Portlet which does New User Registration Functionality.
 * 
 * @deprecated Use SelfCare Portlet provided by Websphere Portal instead.
 */
public class UserRegistrationPortlet extends BasePortlet
    implements ActionListener {
    /** JSP file name to be rendered on the view mode
     */
    public static final String VIEW_JSP = "/jsps/html/userRegistration/UserRegistrationPortletView.";

    /** JSP file name to be rendered on the help mode
     */
    public static final String HELP_JSP = "/jsps/html/userRegistration/UserRegistrationPortletHelp.";

    /** Bean name for the view mode request
     */
    public static final String VIEW_BEAN = "userregistration.UserRegistrationPortletViewBean";

    /** Bean name for the portlet session
     */
    public static final String SESSION_BEAN = "userregistration.UserRegistrationPortletSessionBean";

    /** Action name for the orderId entry form
     */
    public static final String FORM_ACTION = "userregistration.UserRegistrationPortletFormAction";

    /** Bean name for the Capturing user details in the Registration field.
     */
    public static final String USER_BEAN = "userregistration.UserBean";

    /** Parameter name to get GLN value
     */
    public static final String GLN = "GLNTEXT";

    /** Parameter name to get User Name
     */
    public static final String USERNAME = "USERNAMETEXT";

    /** Parameter name to get Password
     */
    public static final String PASSWORD = "PASSWORDTEXT";

    /** Parameter name to get Confirm Password
     */
    public static final String CONFIRM_PASSWORD = "CONFIRM_PASSWORDTEXT";

    /** Parameter name to get company
     */
    public static final String COMPANY = "COMPANYTEXT";

    /** Parameter name to get eMail
     */
    public static final String EMAIL = "EMAILTEXT";

    /** Parameter name for general Register button
     */
    public static final String REGISTER_USER = "userregistration.UserRegistrationPortletRegister";

    /** Parameter name for Business function attribute
     */
    public static final String USER_REGISTRATION = "USER_REGISTRATION";

    /** Parameter name for User GLN length
     */
    public static final int USER_GLN_LENGTH = 13;

    /** Parameter name for Ldap not configured
     */
    public static final String LDAP_CONFIG_ERROR = "MESSAGE_CODE_255";

    /** Parameter name for Ldap Configuration Info
     */
    public static final String LDAP_CONFIG_INFO = "LDAP_CONFIG_INFO";

    /** Parameter name for Ldap not Configured Info
     */
    public static final String LDAP_NOT_CONFIGURED = "LDAP_NOT_CONFIGURED";

    /** Parameter name for Ldap not Configured Info
     */
    public static final String LDAP_CONFIGURED = "LDAP_CONFIGURED";

    /** Variables for User related functionality
     */
    boolean isLdapConfigured = false;
    private UserManager userManager = null;

    /**
     * @see org.apache.jetspeed.portlet.Portlet#init(PortletConfig)
     */
    public void init(PortletConfig portletConfig) throws UnavailableException {
        super.init(portletConfig);
    }

    /**
     * @see org.apache.jetspeed.portlet.Portlet#init(PortletConfig)
     */
    public void initConcrete(PortletSettings arg) throws UnavailableException {
        if (!isLdapConfigured) {
            userManager = UserManager.getInstance();

            if (userManager != null) {
                isLdapConfigured = true;
            }
        }
    }

    /**
     * @see com.ibm.ccd.ui.portlets.BasePortlet#setBusinessFunction(org.apache.jetspeed.portlet.PortletRequest)
     */
    protected void setBusinessFunction(PortletRequest portletRequest) {
        PortletSession session = portletRequest.getPortletSession();

        if (session.getAttribute(IReference.SESSION_BUS_FUNCT) == null) {
            session.setAttribute(IReference.SESSION_BUS_FUNCT, USER_REGISTRATION);
        }
    }

    /**
     * @see org.apache.jetspeed.portlet.PortletAdapter#doView(PortletRequest, PortletResponse)
     */
    public void doView(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        UserBean resultUsrBean = new UserBean();
        MessageMap msgMap = null;

        try {
            msgMap = getMessageMap(request);

            if (!isLdapConfigured) {
                msgMap.addErrorMessage(LDAP_CONFIG_ERROR);
                request.setAttribute(LDAP_CONFIG_INFO, LDAP_NOT_CONFIGURED);
            } else {
                request.setAttribute(LDAP_CONFIG_INFO, LDAP_CONFIGURED);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        // Initialize the Business function
        setBusinessFunction(request);

        // Check if portlet session exists
        UserRegistrationPortletSessionBean sessionBean = getSessionBean(request);

        if (sessionBean == null) {
            return;
        }

        // Make a view mode bean
        UserRegistrationPortletViewBean viewBean = new UserRegistrationPortletViewBean();
        request.setAttribute(VIEW_BEAN, viewBean);

        // Set actionURI in the view mode bean
        PortletURI formActionURI = response.createURI();
        formActionURI.addAction(REGISTER_USER);
        viewBean.setFormActionURI(formActionURI.toString());

        if (request.getAttribute(USER_BEAN) == null) {
            request.setAttribute(USER_BEAN, resultUsrBean);
        }

        // Invoke the JSP to render        		
        getPortletConfig().getContext().include(VIEW_JSP +
            getJspExtension(request), request, response);
    }

    /**
     * @see org.apache.jetspeed.portlet.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent event) throws PortletException {
        //	Add action string handler here
        PortletRequest request = event.getRequest();
        UserRegistrationPortletSessionBean sessionBean = getSessionBean(request);
        MessageMap msgMap = null;

        try {
            msgMap = getMessageMap(request);
        } catch (Exception e) {
            e.printStackTrace();
        }

        // Initialize the Business function
        // This method is called mainly for Anonymus User Login  
        setBusinessFunction(request);

        // ActionEvent handler
        String actionString = event.getActionString();

        // set the user bean with the required params
        // By default UserStatus is set to false and 
        // WPCRole is set as wpsAdmin 
        UserBean userBean = new UserBean();
        userBean.setSupplierGLN(request.getParameter(GLN));
        userBean.setUserName(request.getParameter(USERNAME));
        userBean.setPassword(request.getParameter(PASSWORD));
        userBean.setConfirm_password(request.getParameter(CONFIRM_PASSWORD));
        userBean.setMail(request.getParameter(EMAIL));
        userBean.setCompany(request.getParameter(COMPANY));

        if (REGISTER_USER.equals(actionString)) {
            //	set the values for result user Bean before registration
            UserBean resultUsrBean = new UserBean();
            resultUsrBean.setSupplierGLN(request.getParameter(GLN));
            resultUsrBean.setUserName(request.getParameter(USERNAME));
            resultUsrBean.setPassword(request.getParameter(PASSWORD));
            resultUsrBean.setConfirm_password(request.getParameter(
                    CONFIRM_PASSWORD));
            resultUsrBean.setMail(request.getParameter(EMAIL));
            resultUsrBean.setCompany(request.getParameter(COMPANY));

            // Check for Null or Blank fields
            if (validateUserFields(userBean, request)) {
                //	Check for GLN which should be numeric
                if (validateUserGLNField(userBean, request)) {
                    //	Check for Passwords fields
                    if (validateUserPasswordFields(userBean, request)) {
                        resultUsrBean = userManager.validateLoginUser(userBean);

                        if (!UIUtils.isEmpty(resultUsrBean.getSuccessMessage())) {
                            //	Supplier Portal User added successfully
                            msgMap.addInfoMessage("MESSAGE_CODE_256");
                        }

                        if (!UIUtils.isEmpty(resultUsrBean.getErrorMessage())) {
                            if (resultUsrBean.getErrorMessage().equals("MESSAGE_CODE_253")) {
                                //	User Already exists
                                msgMap.addErrorMessage("MESSAGE_CODE_253");
                            } else {
                                //	User Registration error
                                msgMap.addErrorMessage(resultUsrBean.getErrorMessage());
                            }
                        }
                    } else {
                        resultUsrBean.setPassword("");
                        resultUsrBean.setConfirm_password("");
                    }
                } else {
                    resultUsrBean.setSupplierGLN("");
                }
            }

            request.setAttribute(USER_BEAN, resultUsrBean);
        }
    }

    /**
     * Get SessionBean.
     *
     * @param request PortletRequest
     * @return UserRegistrationPortletSessionBean
     */
    private UserRegistrationPortletSessionBean getSessionBean(
        PortletRequest request) {
        PortletSession session = request.getPortletSession();

        if (session == null) {
            return null;
        }

        UserRegistrationPortletSessionBean sessionBean = (UserRegistrationPortletSessionBean) session.getAttribute(SESSION_BEAN);

        if (sessionBean == null) {
            sessionBean = new UserRegistrationPortletSessionBean();
            session.setAttribute(SESSION_BEAN, sessionBean);
        }

        return sessionBean;
    }

    /**
     * Returns the file extension for the JSP file
     *
     * @param request PortletRequest
     * @return JSP extension
     */
    private static String getJspExtension(PortletRequest request) {
        String markupName = request.getClient().getMarkupName();

        return "jsp";
    }

    /**
     * @param userBean
     * @return
     */
    private boolean validateUserFields(UserBean userBean, PortletRequest request) {
        boolean isUserValid = true;
        MessageMap msgMap = null;

        try {
            msgMap = getMessageMap(request);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
        }

        Vector errorLabels = new Vector();

        if (UIUtils.isEmpty(userBean.getSupplierGLN())) {
            errorLabels.add(getTextLabel(request, GLN));
        }

        if (UIUtils.isEmpty(userBean.getUserName())) {
            errorLabels.add(getTextLabel(request, USERNAME));
        }

        if (UIUtils.isEmpty(userBean.getMail())) {
            errorLabels.add(getTextLabel(request, EMAIL));
        }

        if (UIUtils.isEmpty(userBean.getPassword())) {
            errorLabels.add(getTextLabel(request, PASSWORD));
        }

        if (UIUtils.isEmpty(userBean.getConfirm_password())) {
            errorLabels.add(getTextLabel(request, CONFIRM_PASSWORD));
        }

        if (UIUtils.isEmpty(userBean.getCompany())) {
            errorLabels.add(getTextLabel(request, COMPANY));
        }

        for (int iErrors = 0; iErrors < errorLabels.size(); iErrors++) {
            isUserValid = false;
            msgMap.addErrorMessage("MESSAGE_CODE_110", errorLabels.get(iErrors));
        }

        return isUserValid;
    }

    /** Validates the Supplier GLN field for Numeric type and Maxlength 13
     * @param userBean
     * @return
     */
    private boolean validateUserGLNField(UserBean userBean,
        PortletRequest request) {
        boolean isSupplierGLNValid = true;
        MessageMap msgMap = null;

        try {
            msgMap = getMessageMap(request);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
        }

        try {
            if ((userBean.getSupplierGLN().length() > USER_GLN_LENGTH) ||
                    (userBean.getSupplierGLN().length() < USER_GLN_LENGTH)) {
                isSupplierGLNValid = false;
            } else {
                long lnGLN = Long.parseLong(userBean.getSupplierGLN().trim());
                isSupplierGLNValid = true;
            }
        } catch (NumberFormatException nfe) {
            isSupplierGLNValid = false;
        }

        if (userBean.getSupplierGLN().indexOf('+') != -1) {
            isSupplierGLNValid = false;
        }

        if (userBean.getSupplierGLN().indexOf('-') != -1) {
            isSupplierGLNValid = false;
        }

        if (userBean.getSupplierGLN().indexOf('.') != -1) {
            isSupplierGLNValid = false;
        }

        if (!isSupplierGLNValid) {
            // Error Message Supplier User GLN is Invalid, Enter 13 digit Number  
            msgMap.addErrorMessage("MESSAGE_CODE_251");
        }

        return isSupplierGLNValid;
    }

    /** Checks for Passwords fields to be identical.
     * @param userBean
     * @return
     */
    private boolean validateUserPasswordFields(UserBean userBean,
        PortletRequest request) {
        boolean isPasswordsValid = true;
        MessageMap msgMap = null;

        try {
            msgMap = getMessageMap(request);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
        }

        if (!userBean.getPassword().equals(userBean.getConfirm_password())) {
            isPasswordsValid = false;
            msgMap.addErrorMessage("MESSAGE_CODE_252");
        }

        return isPasswordsValid;
    }
}
