
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.portlets.workflow;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

import org.apache.jetspeed.portlet.AccessDeniedException;
import org.apache.jetspeed.portlet.PortletData;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletSession;
import org.apache.jetspeed.portlet.PortletSettings;
import org.apache.jetspeed.portlet.event.ActionEvent;
import org.apache.jetspeed.portlet.event.ActionListener;

import com.ibm.ccd.ui.context.IWPCUser;
import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.AttributeTypes;
import com.ibm.ccd.ui.dataobjects.workflow.CategoryInfo;
import com.ibm.ccd.ui.dataobjects.workflow.IWorkFlowToInput;
import com.ibm.ccd.ui.dataobjects.workflow.ListboxContainer;
import com.ibm.ccd.ui.dataobjects.workflow.PageNavigation;
import com.ibm.ccd.ui.dataobjects.workflow.WorkFlowTO;
import com.ibm.ccd.ui.dataobjects.workflow.WorkflowHelper;
import com.ibm.ccd.ui.mediator.ConfigManager;
import com.ibm.ccd.ui.mediator.WPCMediatorException;
import com.ibm.ccd.ui.portlets.BasePortlet;
import com.ibm.ccd.ui.portlets.massupdate.ItemAttribute;
import com.ibm.ccd.ui.portlets.quicksearch.IQuickSearch;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;


public class BusinessProcessPortlet extends BasePortlet
    implements ActionListener {
    /**
     * Setting business function
     * @param request
     */
    protected void setBusinessFunction(PortletRequest portletRequest) {
        PortletSession portletSession = null;
        portletSession = portletRequest.getPortletSession();
        portletSession.setAttribute(IReference.SESSION_BUS_FUNCT,
            IWorkFlow.BUSINESS_FUNCTION_WORKFLOW);
    }
    /**
     * Invoked in view mode
     * @param response
     * @param request
     */
    public void doView(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        PortletSession session = request.getPortletSession();
        setBusinessFunction(request);

        WorkflowBean workflowBean = (WorkflowBean) session.getAttribute(IWorkFlow.SESSION_DATABEAN);
        
        if (workflowBean == null) {
            workflowBean = new WorkflowBean();

            try {
                loadWorkFlowBean(workflowBean, request);
            } catch (RuntimeException e) {
                e.printStackTrace();
                getMessageMap(request).addErrorMessage("MESSAGE_CODE_100");
            }
        } else {
            if (workflowBean.getItemList() != null) {
                getMessageMap(request).addInfoMessage("MESSAGE_CODE_514");
            }
        }

        if (request.getAttribute(IWorkFlow.REQ_PARAM_RESERVE_EDIT) == null) {
            workflowBean.setReserveEditBean(null);
        }

        request.setAttribute(IWorkFlow.REQ_ATTR_BEAN_KEY, workflowBean);
        session.setAttribute(IWorkFlow.SESSION_DATABEAN, workflowBean);
        //session.setAttribute("SESSION_DATA_WF",workflowBean);

        //	Invoke the JSP to render
        String jsp2Render = "/jsps/html/workflow/workflowsearch.jsp";

        getPortletConfig().getContext().include(jsp2Render, request, response);
    }

    /**
     * @param workflowBean
     * @param request
     */
    public void loadWorkFlowBean(WorkflowBean workflowBean,
        PortletRequest request) {
        WorkFlowTO workflowTo = new WorkFlowTO();
        WPCMediatorException error = null;

        if (workflowBean.getTeamList() == null) {
            try {
                WorkflowHelper.getInstance().getUserRoles(workflowTo,
                    getMessageMap(request), getWPSContext(request));
            } catch (WPCMediatorException e) {
                handleException(e, request);
                e.printStackTrace();
                error = e;
            }

            workflowBean.setTeamList(workflowTo.getTeamListContainer());
        }

        if (error == null) {
            workflowTo.setCommand(IWorkFlow.GO_TEAM_SELECT);
            WorkflowHelper.getInstance().process(workflowTo,
                getMessageMap(request), getWPSContext(request));
            workflowBean.setWorkflowList(workflowTo.getCollabAreaListContainer());
        }
    }

    public void actionPerformed(ActionEvent event) throws PortletException {
        log.debug(" actionPerformed ->");

        PortletRequest request = event.getRequest();
        PortletSession session = request.getPortletSession();
        WorkflowBean workflowSessionBean = (WorkflowBean) session.getAttribute(IWorkFlow.SESSION_DATABEAN);
        String actionString = event.getActionString();

        if (actionString.equals(IWorkFlow.CONFIG_ITEMSPAGE)) {
            updateConfigAttribute(request);

            //			workflowSessionBean.setWorkflow(null);
            workflowSessionBean.setItemList(null);
            workflowSessionBean.setPageNav(null);

            return;
        }

        if (actionString.equals(IWorkFlow.EDIT_ITEMSPAGE)) {
            updateEditAttribute(request, workflowSessionBean);

            //			workflowSessionBean.setWorkflow(null);
            workflowSessionBean.setItemList(null);
            workflowSessionBean.setPageNav(null);

            return;
        }

        WorkFlowTO workflowTo = new WorkFlowTO();

        extractReqParams(request, actionString, workflowTo);
    }

    /**
     * Updating attributes in Edit mode
     * @param request
     * @param workflowSessionBean
     */
    private void updateEditAttribute(PortletRequest request,
        WorkflowBean workflowSessionBean) {
        String[] attrArray = getReqParamArray(request, "ATTR_CHKBOX");
        List selectedAttrList = UIUtils.getStringArrayList(attrArray);
        
        List itemAttributeList = workflowSessionBean.getDispAttributeList();
        Iterator itemAttributeItr = itemAttributeList.iterator();
        

        // Checking if user has checked more than max attribute configured  
        int iMaxAttribute = getSettingsAttribute(IWorkFlow.PORTLET_PARAM_MAX_ATTRIBUTES,
                request);

        if (selectedAttrList.size() > iMaxAttribute) {
            getMessageMap(request).addErrorMessage("MESSAGE_CODE_352",
                String.valueOf(getSettingsAttribute(
                      IWorkFlow.PORTLET_PARAM_MAX_ATTRIBUTES, request)));
            return;
        }
        
        if(!(selectedAttrList.contains(workflowSessionBean.getPrimaryKeyName()))){
            getMessageMap(request).addErrorMessage("MESSAGE_CODE_366",workflowSessionBean.getPrimaryKeyLable());
            return;
        }
        	// Only primary key is always at the top.
			String firstattributeTemp = (String) selectedAttrList.get(0);
			if (!firstattributeTemp.equals(workflowSessionBean.getPrimaryKeyName())) {
	        	for (int i = 0; i < selectedAttrList.size(); i++) {
	        		String attributeTemp = (String) selectedAttrList.get(i);
	        		if (attributeTemp.equals(workflowSessionBean.getPrimaryKeyName())) {
	        			selectedAttrList.remove(i);
	        			selectedAttrList.add(0,workflowSessionBean.getPrimaryKeyName());
	        			break;
					}
				}
				
			}

		Iterator parameterIter = selectedAttrList.iterator();
        //set all itemAttributes isDisplayable to false
        while (itemAttributeItr.hasNext()) {
            ItemAttribute itemAttribute = (ItemAttribute) itemAttributeItr.next();
            itemAttribute.setIsDisplayable(false);
            itemAttribute.setSelected(false);
            itemAttribute.setListBoxIndex(0);
        }

        //loop through parameters
        List dispAttributeList = new ArrayList();
        int attributeIndex = 0;
        StringBuffer selectedAttributes = new StringBuffer();
        while (parameterIter.hasNext()) {
            String reqAttributeName = (String) parameterIter.next();
            reqAttributeName = reqAttributeName.trim();

            //dispAttributeList.add(reqAttributeName);	
            itemAttributeItr = itemAttributeList.iterator();

            while (itemAttributeItr.hasNext()) {
                ItemAttribute itemAttribute = (ItemAttribute) itemAttributeItr.next();
                String itemAttributeName = itemAttribute.getName();

                if (itemAttributeName.equals(reqAttributeName)) {
                    itemAttribute.setIsDisplayable(true);
                    itemAttribute.setSelected(true);
                    itemAttribute.setListBoxIndex(attributeIndex++);

                    // add to setting string
                    selectedAttributes.append(itemAttributeName);
                    selectedAttributes.append(IQuickSearch.CONFIG_INDEX_TOKENIZER);
                    selectedAttributes.append(attributeIndex);
                    selectedAttributes.append(IQuickSearch.CONFIG_ATTRIBUTE_TOKENIZER);
                }
            }
        }
        
        ArrayList sortedSelectedAttrs = (ArrayList)orderAttributes(workflowSessionBean.getDispAttributeList(),selectedAttrList.size());
        workflowSessionBean.setDispAttributeList(itemAttributeList);
        String catalogId = workflowSessionBean.getWorkflowList().getSelectedValue();
        PortletData portletData = request.getData();
        try {
            portletData.setAttribute(catalogId,
                selectedAttributes.toString());
            portletData.store();
        } catch (AccessDeniedException e) {
            e.printStackTrace();
        } catch (IOException e1) {
            e1.printStackTrace();
        }        
        // Saving Category edit configuration data 
        // CategoryPath~~category name##
        // 1/2/4~~Category1##1/2/5~~DigitalCamera
        String[] selCategoryArray = getReqParamArray(request, IWorkFlow.REQ_PARAM_KEY_CATEGORY_SELECTED);
        StringBuffer strBuffer = new StringBuffer();
        for (int i = 0; i < selCategoryArray.length; i++) {
            int index = selCategoryArray[i].indexOf(IWorkFlow.DELIMITER_FOR_CATPATH_AND_LABEL_LIST);
            String catpath = selCategoryArray[i].substring(0,index);
            String catlabel = selCategoryArray[i].substring(index+2);
            StringBuffer strBufferTemp = new StringBuffer();
            
            // Adding category id
            strBufferTemp.append(catpath);;
            strBufferTemp.append(IWorkFlow.DELIMITER_FOR_CATPATH_AND_LABEL);
            // Adding category label            
            strBufferTemp.append(catlabel);
            strBufferTemp.append(IWorkFlow.DELIMITER_FOR_CATEGORY_INFO);
//          TODO Need validation to check for added category exist in current category list
            
            strBuffer.append(strBufferTemp);
        }
        System.out.println("strBuffer="+strBuffer + "key="+workflowSessionBean.getCatalogAssociated()+"_CATEGORY_LIST");
        try {
            portletData.setAttribute(workflowSessionBean.getCatalogAssociated()+IWorkFlow.REQ_EDIT_CATEGORY_LIST_KEY,
                    strBuffer.toString());
            portletData.store();
        } catch (AccessDeniedException e) {
            e.printStackTrace();
        } catch (IOException e1) {
            e1.printStackTrace();
        }        
        
        
        
    }

    /**
     * Updating the attributes in Config mode
     * @param request
     */
    public void updateConfigAttribute(PortletRequest request) {
        String strMaxRows = request.getParameter("ITEMS_PER_PAGE");
        String strMaxAttribute = request.getParameter("MAX_ATTRIBUTES");
        PortletSettings portletSettings = request.getPortletSettings();

        try {
            if ((strMaxRows != null) || (strMaxAttribute != null)) {
                if ((Integer.parseInt(strMaxRows) <= 0) ||
                        (Integer.parseInt(strMaxAttribute) <= 0)) {
                    throw new NumberFormatException();
                }

                new Integer(strMaxRows);
                portletSettings.setAttribute(IWorkFlow.PORTLET_PARAM_ITEMPERPAGE,
                    strMaxRows);
                new Integer(strMaxAttribute);
                portletSettings.setAttribute(IWorkFlow.PORTLET_PARAM_MAX_ATTRIBUTES,
                    strMaxAttribute);
                portletSettings.store();
                
                PortletSession session = request.getPortletSession();
                WorkflowBean workflowSessionBean = (WorkflowBean) session.getAttribute(IWorkFlow.SESSION_DATABEAN);
                if (workflowSessionBean.getWorkflow() != null) {
                    // initialising the attributes list 
                    populateAttributeList(request);
                }
                getMessageMap(request).addInfoMessage("MESSAGE_CODE_508");
            }
        } catch (NumberFormatException e) {
            e.printStackTrace();
            getMessageMap(request).addErrorMessage("MESSAGE_CODE_517");
        } catch (AccessDeniedException e1) {
            e1.printStackTrace();
        } catch (IOException e2) {
            e2.printStackTrace();
        }

        // max items per page
    }

    /**
     * Extracting the params and initiating the action or fetches for Workflow Elements
     * @param request
     * @param actionString
     * @param workflowTo
     */
    private void extractReqParams(PortletRequest request, String actionString,
        WorkFlowTO workflowTo) {
        log.debug(" extractReqParams ->");
        
        PortletSession session = request.getPortletSession();
        WorkflowBean workflowSessionBean = (WorkflowBean) session.getAttribute(IWorkFlow.SESSION_DATABEAN);
        workflowSessionBean.setReserveEditBean(null);
        workflowTo.setCommand(actionString);
        IWorkFlowToInput workflowToIn = (IWorkFlowToInput) workflowTo;

        if (actionString.equals(IWorkFlow.GO_TEAM_SELECT)) {
            String teamlstSel = request.getParameter("teamlist");
            ListboxContainer teamlist = workflowSessionBean.getTeamList();

            teamlist.setSelectedValue(teamlstSel);
            populateWorkflowTo(workflowTo, workflowSessionBean, request);

            workflowSessionBean.setWorkflowList(workflowToIn.getCollabAreaListContainer());
            workflowSessionBean.setWorkflow(null);
            workflowSessionBean.setItemList(null);
            workflowSessionBean.setPageNav(null);
            
            return;   
        } else if (actionString.equals(IWorkFlow.GO_BUS_PROC_SELECT)) {
            String workflowSel = request.getParameter("workflowlst");
            workflowSessionBean.getWorkflowList().setSelectedValue(workflowSel);

            populateWorkflowTo(workflowTo, workflowSessionBean, request);

            setSingleItemPageLink(workflowSessionBean, request);
            workflowSessionBean.setWorkflow(workflowToIn.getStepWorkflow());
            workflowSessionBean.setCatalogAssociated(workflowToIn.getStepWorkflow().getContainerName());
            workflowSessionBean.setCategoryInfoList(getCategoryFilters(workflowSessionBean.getCatalogAssociated(),request));
            workflowSessionBean.setCategorySelIdx(-1); 
            workflowSessionBean.setItemList(null);
            workflowSessionBean.setPageNav(null);
            if (workflowSessionBean.getWorkflow() != null) {
                // initialising the attributes list 
                populateAttributeList(request);
            }
            return;
        } else if (actionString.equals(IWorkFlow.STEPNAME)) {
            String stepName = request.getParameter("STEPNAME");
            String workflowId = request.getParameter("WORKFLOW_ID");
            String collabAreaId = request.getParameter("COLLABAREA_ID");
  
            workflowSessionBean.getWorkflow().setSelectedStep(workflowSessionBean.getWorkflow()
                                                                                 .findStep(stepName));

            if (workflowSessionBean.getPageNav() == null) {
                PageNavigation pageNav = new PageNavigation();
                pageNav.setItemsPerPage(getSettingsAttribute(
                        IWorkFlow.PORTLET_PARAM_ITEMPERPAGE, request));
                pageNav.setEndIndex(getSettingsAttribute(
                        IWorkFlow.PORTLET_PARAM_ITEMPERPAGE, request));

                workflowSessionBean.setPageNav(pageNav);
            }
            
            populateWorkflowTo(workflowTo, workflowSessionBean, request);
            setSingleItemPageLink(workflowSessionBean, request);

            workflowSessionBean.setPageNav(workflowTo.getPageNav());
            workflowSessionBean.setItemList(workflowTo.getItemList());
            workflowSessionBean.setCatalogAssociated(workflowTo
                    .getCatalogAssociated());
            
          workflowSessionBean.setCategoryInfoList(getCategoryFilters(workflowTo.getCatalogAssociated(),request));
           // to switch the tab to detail
            return;

        } else if (actionString.equals(IWorkFlow.CATEGORYIDX)) {

        	String categoryIdxAsString = request.getParameter("catFilter");
        	
        	int categoryIdx = -1;
        	try {
        		categoryIdx = (new Integer(categoryIdxAsString)).intValue();
        	} catch (NumberFormatException nfx) {
        	    nfx.printStackTrace();
        		// intentionally do nothing - keep initialization value
        	}
            workflowSessionBean.setCategorySelIdx(categoryIdx);
            populateWorkflowTo(workflowTo, workflowSessionBean, request);
            setSingleItemPageLink(workflowSessionBean, request);

            workflowSessionBean.setPageNav(workflowTo.getPageNav());
            workflowSessionBean.setItemList(workflowTo.getItemList());
            workflowSessionBean.setCatalogAssociated(workflowTo
                    .getCatalogAssociated());

            return;

        } else if (actionString.equals(IWorkFlow.SORT_TYPE)) {
            String sortedColumn = request.getParameter(IWorkFlow.SORT_COLUMN);
            workflowSessionBean.getPageNav().setSortedColumn(sortedColumn);

            setSingleItemPageLink(workflowSessionBean, request);

            String sortType = "";

            if (workflowSessionBean.getPageNav().getSortType().equals(IWorkFlow.ASCENDING)) {
                sortType = IWorkFlow.DESCENDING;
            } else {
                sortType = IWorkFlow.ASCENDING;
            }

            workflowSessionBean.getPageNav().setSortType(sortType);

            populateWorkflowTo(workflowTo, workflowSessionBean, request);

            workflowSessionBean.setPageNav(workflowTo.getPageNav());
            workflowSessionBean.setItemList(workflowTo.getItemList());
            workflowSessionBean.setCatalogAssociated(workflowTo
                    .getCatalogAssociated());
            return;
        } else if (actionString.equals(IWorkFlow.PAGE_NAV)) {
            String pageNavVal = request.getParameter(IWorkFlow.REQ_PAGE_NAV_KEY);
            workflowTo.setCommand(pageNavVal);

            int mxItemCount = getSettingsAttribute(IWorkFlow.PORTLET_PARAM_ITEMPERPAGE,
                    request);
            workflowSessionBean.getPageNav().setItemsPerPage(mxItemCount);

            setSingleItemPageLink(workflowSessionBean, request);
            populateWorkflowTo(workflowTo, workflowSessionBean, request);

            setSingleItemPageLink(workflowSessionBean, request);
            workflowSessionBean.setPageNav(workflowTo.getPageNav());
            workflowSessionBean.setItemList(workflowTo.getItemList());
            workflowSessionBean.setCatalogAssociated(workflowTo
                    .getCatalogAssociated());

            //	to switch the tab to detail
            return;
        } else if (actionString.equals(IWorkFlow.RESERVE_ITEM) ||
                actionString.equals(IWorkFlow.UNRESERVE_ITEM) ||
                actionString.equals(IWorkFlow.RESERVE_EDIT_ITEM) ||
                actionString.equals(IWorkFlow.NEXT_STEP_ITEM)) {
            String nextStepSelected = request.getParameter("NEXT_STEP_LIST");
            String[] strItemArr = getReqParamArray(request, "CHK_ITEM");

            if ((strItemArr == null) || (strItemArr.length == 0)) {
                getMessageMap(request).addInfoMessage("MESSAGE_CODE_514");

                return;
            }

            workflowTo.setArrSelectedItem(strItemArr);
            workflowTo.setCommand(actionString);
            workflowTo.setNextStepListContainer(new ListboxContainer(
                    nextStepSelected));

            try {
                workflowTo.setTeamListContainer(workflowSessionBean.getTeamList());
                workflowTo.setCollabAreaListContainer(workflowSessionBean.getWorkflowList());
                workflowTo.setStepWorkflow(workflowSessionBean.getWorkflow());
                workflowTo.setPageNav(workflowSessionBean.getPageNav());
                workflowTo.setItemList(workflowSessionBean.getItemList());

                WorkflowHelper.getInstance().processAction(workflowTo,
                    getMessageMap(request), getWPSContext(request));
                workflowTo.getPageNav().reset();

                if (actionString.equals(IWorkFlow.RESERVE_EDIT_ITEM)) {
                    String action2PageParam = "";
                    Map action2page = ConfigManager.getInstance()
                                                   .getPortletActionToPageMappings();
                    boolean bsingleEdit = false;

                    if (strItemArr.length > 1) {
                        action2PageParam = (String) action2page.get(
                                "ITEMCATALOG.MASSUPDATE");
                    } else {
                        action2PageParam = (String) action2page.get(
                                "ITEMCATALOG.WORKWITHITEM");
                        bsingleEdit = true;
                    }

                    String selectedItemIdsParam = replaceChar(strItemArr, "-");

                    ReserveEditDataBean reserveEditBean = new ReserveEditDataBean(selectedItemIdsParam,
                            UIUtils.getCatalogName(getWPSContext(request)
                                                       .getWpcUser()),
                            action2PageParam);
                    reserveEditBean.setNextPageSingle(bsingleEdit);
                    workflowSessionBean.setReserveEditBean(reserveEditBean);
                    workflowSessionBean.setItemList(null);
                    
                    request.setAttribute(IWorkFlow.REQ_PARAM_RESERVE_EDIT,
                        IWorkFlow.REQ_PARAM_RESERVE_EDIT);

                 //   return;
                }
                
             /*   String workflowSel = request.getParameter("workflowlst");
                workflowSessionBean.getWorkflowList().setSelectedValue(workflowSel);
                //request.(IWorkFlow.GO_BUS_PROC_SELECT);
                populateWorkflowTo(workflowTo, workflowSessionBean, request);

                setSingleItemPageLink(workflowSessionBean, request);
                workflowSessionBean.setWorkflow(workflowToIn.getStepWorkflow());
                workflowSessionBean.setItemList(null);
                workflowSessionBean.setPageNav(null);
                if (workflowSessionBean.getWorkflow() != null) {
                    // initialising the attributes list 
                    populateAttributeList(request);
                }
                workflowSessionBean.setCatalogAssociated(workflowTo
                        .getCatalogAssociated());
                //session.setAttribute("SESSION_DATA_WF",workflowSessionBean);
                //return;
*/
                if (actionString.equals(IWorkFlow.RESERVE_ITEM)) {
                    getMessageMap(request).addInfoMessage("MESSAGE_CODE_509",
                        UIUtils.getCommaSeparatedId(strItemArr));
                } else if (actionString.equals(IWorkFlow.UNRESERVE_ITEM)) {
                    getMessageMap(request).addInfoMessage("MESSAGE_CODE_510",
                        UIUtils.getCommaSeparatedId(strItemArr));
                } else if (actionString.equals(IWorkFlow.NEXT_STEP_ITEM)) {
                    getMessageMap(request).addInfoMessage("MESSAGE_CODE_511",
                        UIUtils.getCommaSeparatedId(strItemArr));
                }
               
            } catch (WPCMediatorException we) {
                we.printStackTrace();
                handleException(we, request);
            }

            setSingleItemPageLink(workflowSessionBean, request);
            workflowTo.setCommand(IWorkFlow.STEPNAME);
            populateWorkflowTo(workflowTo, workflowSessionBean, request);

            workflowSessionBean.setPageNav(workflowTo.getPageNav());
            workflowSessionBean.setItemList(workflowTo.getItemList());
            workflowSessionBean.setCatalogAssociated(workflowTo
                    .getCatalogAssociated());
            workflowSessionBean.setCategorySelIdx(-1);
            //	to switch the tab to detail
            return;
        }
    }

    /**
     * replacing character in array with "-"
     * @param values
     * @param replaceStr
     * @return
     */
    private String replaceChar(String[] values, String replaceStr) {
        StringBuffer buffer = new StringBuffer();

        if (values != null) {
            int size = values.length;

            if (size > 1) {
                for (int i = 0; i < (values.length - 1); i++) {
                    buffer.append(values[i] + replaceStr);
                }

                buffer.append(values[size - 1]);
            } else if (size == 1) {
                buffer.append(values[0]);
            }
        }

        return buffer.toString();
    }

    /**
     * Getting attribte from PortletSettings object
     * @param request
     * @return
     */
    private int getSettingsAttribute(String attributeName,
        PortletRequest request) {
        PortletSettings portletSettings = request.getPortletSettings();
        String mxItems = portletSettings.getAttribute(attributeName);
        int mxCount = 1;

        if (!UIUtils.isEmpty(mxItems)) {
            try {
                mxCount = new Integer(mxItems).intValue();
            } catch (NumberFormatException e) {
                e.printStackTrace();
            }
        }

        return mxCount;
    }

    /**
     * Creating the item Page link
     * @param workflowSessionBean
     * @return
     */
    private void setSingleItemPageLink(WorkflowBean workflowSessionBean,
        PortletRequest request) {
        HashMap action2page = null;

        if (workflowSessionBean.getLinktoWorkItem() == null) {
            action2page = ConfigManager.getInstance()
                                       .getPortletActionToPageMappings();

            String actionPagetoSingleItemEdit = (String) action2page.get(
                    "ITEMCATALOG.WORKWITHITEM");
            workflowSessionBean.setLinktoWorkItem(actionPagetoSingleItemEdit);
            workflowSessionBean.setPrimaryCatalog(UIUtils.getCatalogName(
                    getWPSContext(request).getWpcUser()));
        }
    }

    /**
     * Populating workflowTO object from workflowsession bean before initiating the action
     * @param workflowTo
     * @param workflowSessionBean
     * @param request
     */
    private WorkFlowTO populateWorkflowTo(WorkFlowTO workflowTo,
        WorkflowBean workflowSessionBean, PortletRequest request) {
        workflowTo.setTeamListContainer(workflowSessionBean.getTeamList());
        workflowTo.setCollabAreaListContainer(workflowSessionBean.getWorkflowList());
        workflowTo.setStepWorkflow(workflowSessionBean.getWorkflow());
        workflowTo.setPageNav(workflowSessionBean.getPageNav());
        workflowTo.setItemList(workflowSessionBean.getItemList());
        workflowTo.setAttributeList(filterAttributes(
                workflowSessionBean.getDispAttributeList()));
        workflowTo.setCategoryInfoList(workflowSessionBean.getCategoryInfoList());   
        workflowTo.setCategorySelIdx(workflowSessionBean.getCategorySelIdx());
        
        WorkflowHelper.getInstance().process(workflowTo,
            getMessageMap(request), getWPSContext(request));
        
        return workflowTo;
    }

    /**
     * Attributes are filtered only displayable are passed
     * @param attributeList
     * @return
     */
    private List filterAttributes(List attributeList) {
        if (attributeList == null) {
            return attributeList;
        }

        List filteredList = filteredList = new ArrayList();

        for (Iterator iter = attributeList.iterator(); iter.hasNext();) {
            ItemAttribute element = (ItemAttribute) iter.next();

            if (element.getIsDisplayable()) {
                filteredList.add(element.getName());
            }
        }

        return filteredList;
    }

    /**
     * invoked for configure mode
     * @param request
     * @return response
     */
    public void doConfigure(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        // max items per page
        String jsp2Render = "/jsps/html/workflow/workflowconfig.jsp";
        request.setAttribute(IWorkFlow.REQ_PARAM_MXITEMS,
            String.valueOf(getSettingsAttribute(
                    IWorkFlow.PORTLET_PARAM_ITEMPERPAGE, request)));

        request.setAttribute(IWorkFlow.REQ_PARAM_MXATTR,
            String.valueOf(getSettingsAttribute(
                    IWorkFlow.PORTLET_PARAM_MAX_ATTRIBUTES, request)));

        getMessageMap(request).addInfoMessage("MESSAGE_CODE_512");
        getPortletConfig().getContext().include(jsp2Render, request, response);
    }

    /**
     * invoked for Edit mode
     * @param request
     * @return response
     */
    public void doEdit(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        String jsp2Render = "/jsps/html/workflow/workflowedit.jsp";
        PortletSession session = request.getPortletSession();
        WorkflowBean workflowSessionBean = (WorkflowBean) session.getAttribute(IWorkFlow.SESSION_DATABEAN);
        
        if (workflowSessionBean.getDispAttributeList() == null) {
            populateAttributeList(request);
        }

        if (workflowSessionBean.getDispAttributeList() != null) {
            getMessageMap(request).addInfoMessage("MESSAGE_CODE_352",
                String.valueOf(getSettingsAttribute(
                      IWorkFlow.PORTLET_PARAM_MAX_ATTRIBUTES, request)));
            getMessageMap(request).addInfoMessage("MESSAGE_CODE_518");
        }
        
        request.setAttribute("REQ_EDIT_CAT_FILTER_KEY",
                getCategoryFilters(workflowSessionBean.getCatalogAssociated(),request));
        
        request.setAttribute(IWorkFlow.REQ_EDIT_ATTR_LIST_KEY,
            workflowSessionBean.getDispAttributeList());
        getPortletConfig().getContext().include(jsp2Render, request, response);
    }


    /**
     * Retrieves  the categogy list from portletData and return the list of CategoryInfo object
     * @param catalogAssociated
     * @param request
     * @return
     */
    private List getCategoryFilters(String catalogAssociated, PortletRequest request) {
        // getCategoryFilters()
        // catalogName should be included in name
        String categoryNameSep = (String)request.getData().getAttribute(catalogAssociated + IWorkFlow.REQ_EDIT_CATEGORY_LIST_KEY);
        List categoryList = new ArrayList();
        if (categoryNameSep == null) {
            return categoryList;
        }
        
        StringTokenizer tokenizer = new StringTokenizer(categoryNameSep,
                IWorkFlow.DELIMITER_FOR_CATEGORY_INFO);
        // Format for storing the data
        // CategoryPath~~category name##
        // 1/2/4~~Category1##1/2/5~~DigitalCamera 
        while (tokenizer.hasMoreTokens()) {
            String stringToken = tokenizer.nextToken();
            int index = stringToken
                    .indexOf(IWorkFlow.DELIMITER_FOR_CATPATH_AND_LABEL);
            String categoryPath = stringToken.substring(0, index);
            String categoryLabel	= stringToken.substring(index + 2);
            categoryList.add(new CategoryInfo(categoryPath,categoryLabel,0));
        }
        return categoryList;
    }
    /**
     * Invoked first time when to keep attribute list of item spec after selecting the workflow
     * @param request
     * @return
     */
    private void populateAttributeList(PortletRequest request) {
        PortletSession session = request.getPortletSession();
        WorkflowBean workflowSessionBean = (WorkflowBean) session.getAttribute(IWorkFlow.SESSION_DATABEAN);
        List itemAttributeList = new ArrayList();
        
        WPSContext context = getWPSContext(request);

        // reservedby,Start date,End date
        Map attributesMap = null;
        if ((workflowSessionBean.getWorkflowList() != null) &&
                (workflowSessionBean.getWorkflowList().getSelectedValue() != null)) {
            attributesMap = WorkflowHelper.getInstance().getAttributes(context,
                    workflowSessionBean.getWorkflowList().getSelectedValue(),
                    getMessageMap(request));
        } else {
            getMessageMap(request).addWarningMessage("MESSAGE_CODE_515");
            return;
        }
        // populate the list of ItemAttributes
        Set set = attributesMap.keySet();
        Iterator iterAttSpecSet = set.iterator();
        int count = 0;
        // Adjustment for adding primary key separately        
        int maxAttribute = getSettingsAttribute(IWorkFlow.PORTLET_PARAM_MAX_ATTRIBUTES, request) - 1 ;
        
        int attributeIndex = 1;
		String locale =
			(String) context.getWpcUser().getProperty(
				IWPCUser.KEY_RESTRICTED_LANGUAGE);
				workflowSessionBean.setDispAttributeList(itemAttributeList);
	
        // getting Map of Stored Attributes from PortletSettings 
        Map storedAttributeMap = getStoredAttributeMap(request.getData(), workflowSessionBean.getWorkflowList().getSelectedValue());
        if ((storedAttributeMap.size()-1) < maxAttribute ) {
        	maxAttribute = storedAttributeMap.size()-1;
		}
        attributeIndex = getMergedAttributeList(workflowSessionBean, attributesMap, count ,maxAttribute, 
                context, attributeIndex,storedAttributeMap);
        count = workflowSessionBean.getDispAttributeList().size(); 
        
        attributeIndex = getMergedAttributeList(workflowSessionBean, attributesMap, count, maxAttribute, 
                context,attributeIndex,attributesMap);
        
        //Primary key should always appear 
        addPrimarykey(attributesMap, itemAttributeList, workflowSessionBean.getPrimaryKeyName());
        orderAttributes(itemAttributeList,attributeIndex );
        workflowSessionBean.setDispAttributeList(itemAttributeList);

        //set the arrayList in bean
        return;
    }
    /**
     * Construct the attribute list and merge with previously stored attributes   
     * @param bean
     * @param attributesMap
     * @param countObj
     * @param iMaxAttribute
     * @param context
     * @param attributeIndexObj
     * @param iterAttSpecSet
     */
    private int getMergedAttributeList(WorkflowBean bean, Map attributesMap,  
            int count, int iMaxAttribute, WPSContext context,int attributeIndex,Map iterableMap) {
        
        Set set = iterableMap.keySet();
        Iterator iterAttSpecSet = set.iterator();
        
        List itemAttributeList = bean.getDispAttributeList();
        boolean bRemove = false;
        boolean indexAvailable= false;
        Attribute pkAttribute = null ;
        
        if(attributesMap != iterableMap){
            bRemove= true;
            indexAvailable= true;
        }
        
        while (iterAttSpecSet.hasNext()) {
        	String attributeName = (String) iterAttSpecSet.next();
            Attribute attributeLoopTemp = (Attribute) attributesMap.get(attributeName);
            String locale =
				(String) context.getWpcUser().getProperty(
					IWPCUser.KEY_RESTRICTED_LANGUAGE);            
			if(attributeLoopTemp.getType() == AttributeTypes.GROUP || 
				!UIUtils.isAttributeDisplayableForLocale(
					attributeLoopTemp,locale)){
				continue;
			}
            // added for displaying the primary key attribute Name in jsp
            if (attributeLoopTemp.isPrimaryKey()) {
                bean.setAttributePk(attributeLoopTemp);
                continue;
            }
            ItemAttribute itemAttribute = new ItemAttribute();
            itemAttribute.setAttribute(attributeLoopTemp);
            itemAttribute.setName(attributeLoopTemp.getName());
            
            if (count < iMaxAttribute) {
                itemAttribute.setIsDisplayable(true);
                itemAttribute.setSelected(true);
                Integer storedAttribIndex = null;
                int attributePosition = 0;
                // Attribute Position value is only available when iterableMap indexAvailable is true. 
                if (indexAvailable) {
                	storedAttribIndex = (Integer)iterableMap.get(attributeName);
                	attributePosition = storedAttribIndex.intValue();
                	if (attributePosition <= (iMaxAttribute+1)) {
                    	// attribute index stored in db starts from 1 so 0 adjustment
                	itemAttribute.setListBoxIndex(--attributePosition);
                	attributeIndex++;
					}else{
		                itemAttribute.setIsDisplayable(false);
		                itemAttribute.setSelected(false);
		                --count;
					}
				}
                else{
                	itemAttribute.setListBoxIndex(attributeIndex++);
				}
            } else {
                itemAttribute.setIsDisplayable(false);
                itemAttribute.setSelected(false);
            }
            
			boolean displayableAttrCountReduced = true;
			if (itemAttribute.getIsDisplayable()) {
				if (UIUtils
					.isAttributeDisplayableForLocale(
						itemAttribute.getAttribute(),
						locale)) {
					displayableAttrCountReduced = true;
				} else {
					itemAttribute.setIsDisplayable(false);
				    itemAttribute.setSelected(false);					
					displayableAttrCountReduced = false;
				}
			}
			if (!displayableAttrCountReduced) {
				count--;
			}
            count++;
			itemAttributeList.add(itemAttribute);
			
			// To avoid duplication of attributes in two separate calls to this method,
			// the attributes which are added in first method call will be removed from flat attribute Map.
			
			if(bRemove){
			    attributesMap.remove(itemAttribute.getName());
			}
        }
        return attributeIndex;
    }

    /**
     * Adds the primary key attribute in the list
     * @param attributesMap
     * @param itemAttributeList
     * @param primarykeyName
     */
    private void addPrimarykey(Map attributesMap, List itemAttributeList, String primarykeyName) {
        Attribute pkAttribute = (Attribute) attributesMap.get(primarykeyName);
        ItemAttribute itemAttributePk = new ItemAttribute();
        itemAttributePk.setAttribute(pkAttribute);
        itemAttributePk.setName(pkAttribute.getName());
        itemAttributePk.setIsDisplayable(true);
        itemAttributePk.setSelected(true);
        // Listbox Index is 0 
        itemAttributePk.setListBoxIndex(0);
        itemAttributeList.add(0,itemAttributePk);
    }
    /**
     * Attributes are sorted to order the attributes as configured in Edit mode.
     * @param itemAttrList
     * @param selectedAttributeSize
     * @return
     */
    protected List orderAttributes(List itemAttrList,int selectedAttributeSize) {
        ItemAttribute itemAttribute = null;
        int count = 0;
        int index = -1;
        int listBoxindex = -1;
        
        ItemAttribute selectedAttributesArr[] = new ItemAttribute[selectedAttributeSize];
        int emptyAttrPositionsinListArr[]  = new int[selectedAttributeSize];
        
        for(int k = 0,i = 0; i < itemAttrList.size(); i++) {
            itemAttribute = (ItemAttribute) itemAttrList.get(i);
            if (itemAttribute.getAttribute().isPrimaryKey()) {
                String primaryKey = itemAttribute.getName();
            }

            if (itemAttribute.isSelected()) {
                listBoxindex = itemAttribute.getListBoxIndex();

                    if (listBoxindex >= 0 && listBoxindex < selectedAttributeSize) {
                    // Selected Attributes are copied in array selectedAttributesArr 
                    // in sorted order
                    selectedAttributesArr[listBoxindex] = itemAttribute;
                    
                    // The positions of selected Attributes in the list is set to null.Those 
                    // postions are made empty
                    itemAttrList.set(i,null);
                    
                    // The empty positions of selected Attributes in the list is 
                    // stored in the array selectedAttrPositionsinListArr 
                    emptyAttrPositionsinListArr[k++]= i;
                }
            }
        }
        // filling the empty positions in attributes objects from sorted order Array selectedAttributesArr  
        for (int j = 0; j < emptyAttrPositionsinListArr.length; j++) {
            itemAttrList.set(emptyAttrPositionsinListArr[j],selectedAttributesArr[j]);
        }
        return itemAttrList;
    }
    /**
     * @param portletSettings
     * @param catalogID
     */
    private Map getStoredAttributeMap(PortletData portletData, String catalogID) {
        // extracting attribute list from portlet data
        String selectedAttributes = (String) portletData
                .getAttribute(catalogID);
        HashMap storedAttributesMap = new HashMap();

        if (UIUtils.isEmpty(selectedAttributes)) {
            return storedAttributesMap;
        }

        StringTokenizer tokenizer = new StringTokenizer(selectedAttributes,
                IQuickSearch.CONFIG_ATTRIBUTE_TOKENIZER);
        while (tokenizer.hasMoreTokens()) {
            String stringToken = tokenizer.nextToken();
            int index = stringToken
                    .indexOf(IQuickSearch.CONFIG_INDEX_TOKENIZER);
            String attributeName = stringToken.substring(0, index);
            Integer listBoxIndex = Integer.valueOf(stringToken
                    .substring(index + 1));
            storedAttributesMap.put(attributeName, listBoxIndex);
        }
        return storedAttributesMap;
    }
    
}
