
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.portlets.wpcversion;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.UnavailableException;
import org.apache.jetspeed.portlet.event.ActionEvent;
import org.apache.jetspeed.portlet.event.ActionListener;

import com.ibm.ccd.ui.portlets.BasePortlet;


/**
 *
 * A sample portlet based on PortletAdapter
 *
 */
public class VersionInfoPortlet extends BasePortlet implements ActionListener {
    /** JSP file name to be rendered on the view mode
    */
    public static final String VIEW_JSP = "/jsps/html/wpcversion/VersionInfoView.";

    /** Bean name for the Capturing user details of the WPC Portal version.
     */
    public static final String VERSION_INFO_BEAN = "wpcversion.VersionInfoBean";

    /**
     * @see org.apache.jetspeed.portlet.Portlet#init(PortletConfig)
     */
    public void init(PortletConfig portletConfig) throws UnavailableException {
        super.init(portletConfig);
    }

    /**
     * @see org.apache.jetspeed.portlet.PortletAdapter#doView(PortletRequest, PortletResponse)
     */
    public void doView(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        WpcVersionBean versionBean = new WpcVersionBean();

        String versionInfoFile = "/WEB-INF/versionInfo.properties".replace('/',
                File.separatorChar);
        Properties wpcVersionProps = new Properties();

        try {
            wpcVersionProps.load(new FileInputStream(
                    new File(getPortletConfig().getContext().getRealPath(versionInfoFile))));
        } catch (Exception e) {
            e.printStackTrace();
        }

        //	properties loaded successfully , now read them
        versionBean.setWpcPortalReleaseVersion(wpcVersionProps.getProperty(
                IWPCVersion.WPC_PORTAL_RELEASE_VERSION));
        versionBean.setWpcBuildlevel(wpcVersionProps.getProperty(
                IWPCVersion.WPC_PORTAL_BUILD_LEVEL));

        if (request.getAttribute(VERSION_INFO_BEAN) == null) {
            request.setAttribute(VERSION_INFO_BEAN, versionBean);
        }

        // Invoke the JSP to render        		
        getPortletConfig().getContext().include(VIEW_JSP +
            getJspExtension(request), request, response);
    }

    /**
     * @see com.ibm.ccd.ui.portlets.BasePortlet#setBusinessFunction(org.apache.jetspeed.portlet.PortletRequest)
     */
    protected void setBusinessFunction(PortletRequest portletRequest) {
    }

    /**
     * @see org.apache.jetspeed.portlet.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent event) throws PortletException {
    }

    /**
     * Returns the file extension for the JSP file
     *
     * @param request PortletRequest
     * @return JSP extension
     */
    private static String getJspExtension(PortletRequest request) {
        String markupName = request.getClient().getMarkupName();

        return "jsp";
    }
}
