
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * This package consists of classes pertaining to Supplier Portal User,
 * Login and LDAP access related functionality.
 */
package com.ibm.ccd.ui.services.user;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.Properties;
import java.util.Vector;

import javax.naming.CommunicationException;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;


/**
 * LDAP Helper class for User Registration and Customized Portal Login
 */
public class LdapHelper {
    /** Singleton Instance variable for LdapHelper
     */
    private static LdapHelper ldapHelper;

    /** Instance variables for Ldap Configuration
     */
    private static String sLDAPHost = "";
    private static String sLDAPPort = "";
    private static String sLDAPServerURL = "";
    private static String sLDAPRoot = "";
    private static String sLDAPRootPassword = "";
    private static String sLDAPSuffix = "";
    private static String sLDAPUserSuffix = "";

    /** Instance variables for Ldap Web Tool Port
     */
    private static String sLDAPWebToolPort = "";

    /** Instance variables for Constructing Ldap User object
     */

    //	private static final String sABSTRACT = "abstract";
    private static java.util.Properties env;
    private static final int STR_BUFF_SZ = 100;
    private static Attribute USER_OBJ_CLASSES;
    private static DirContext dirCtx;

    /**
     * Private constructor for LdapHelper
     */
    private LdapHelper() {
    }

    /** Singleton method for LdapHelper
     * @return LdapHelper
     */
    public static LdapHelper getInstance() {
        if (ldapHelper == null) {
            synchronized (LdapHelper.class) {
                if (ldapHelper == null) {
                    ldapHelper = new LdapHelper();

                    // load the properties from wpconfig.properties location <WPS_ROOT>/config
                    // ideally this class is deployed as a jar in <WPS_ROOT>/shared/app/ dir.
                    // so the rel. path could be ../../config
                    String configFile = System.getProperty(IUser.WPS_INSTALL_DIR) +
                        "/config/wpconfig.properties".replace('/',
                            File.separatorChar);
                    Properties ldapProps = new Properties();

                    try {
                        ldapProps.load(new FileInputStream(new File(configFile)));
                    } catch (Exception e) {
                    	System.out.println("Exception while loading wpconfig file" + configFile);
                        e.printStackTrace();

                        return null;
                    }

                    // properties loaded successfully , now read them
                    sLDAPSuffix = ldapProps.getProperty(IUser.LDAP_SUFFIX);
                    sLDAPUserSuffix = ldapProps.getProperty(IUser.LDAP_USERSUFFIX);
                    sLDAPHost = ldapProps.getProperty(IUser.LDAP_HOSTNAME);
                    sLDAPPort = ldapProps.getProperty(IUser.LDAP_PORT);
                    sLDAPServerURL = sLDAPHost + ":" + sLDAPPort;
                    sLDAPRoot = ldapProps.getProperty(IUser.LDAP_ADMIN_UID);
                    sLDAPRootPassword = ldapProps.getProperty(IUser.LDAP_ADMIN_PWD);

                    // properties added for Web Tool
                    sLDAPWebToolPort = ldapProps.getProperty(IUser.LDAP_WEB_TOOLPORT);

                    // now create object classes;
                    USER_OBJ_CLASSES = new BasicAttribute(IUser.OBJECTCLASS);
                    USER_OBJ_CLASSES.add(IUser.INETORGPERSON);
                    USER_OBJ_CLASSES.add(IUser.ORGANIZATIONALPERSON);
                    USER_OBJ_CLASSES.add(IUser.PERSON);
                    USER_OBJ_CLASSES.add(IUser.TOP);

                    env = new Properties();
                    env.put(IUser.LDAP_FACTORY_PACKAGE, IUser.LDAP_JNDI);
                    env.put(IUser.LDAP_FACTORY_INITIAL,
                        IUser.LDAP_CONTEXT_FACTORY);
                    env.put(IUser.LDAP_NAMING_URL, "ldap://" + sLDAPServerURL);

                    env.put(IUser.LDAP_SECURITY_USERNAME, sLDAPRoot);
                    env.put(IUser.LDAP_SECURITY_PASSWORD, sLDAPRootPassword);

                    try {
                        dirCtx = new InitialDirContext(env);
                    } catch (NamingException ne) {
                    	System.out.println("NamingException in ldapHelper getInstance");
                        ne.printStackTrace();

                        return null;
                    }
                }
            }
        }

        return ldapHelper;
    }

    /** Create the DN for the USer
     * @param userId
     * @return UserDN
     */
    private final String createUserDN(String userId) {
        StringBuffer dn = new StringBuffer(STR_BUFF_SZ);

        dn.append("uid=");
        dn.append(userId);
        dn.append(",");
        dn.append(sLDAPUserSuffix);
        dn.append(",");
        dn.append(sLDAPSuffix);

        return dn.toString();
    }

    /** Get the Context for the LDAP
     * @return DirContext
     * @throws NamingException
     * @throws CommunicationException
     */
    private final DirContext getDirContext()
        throws NamingException, CommunicationException {
        DirContext ctx = null;
        ctx = new InitialDirContext(getEnvironment());

        return ctx;
    }

    /**
    * @return env (Properties file which has the entries for LDAP Connection)
    */
    private final Properties getEnvironment() {
        return env;
    }

    /** Add the User to LDAP
    * @param userBean
    * @throws NamingException
    */

    /*


    // Users will now be added using the Portal's Registration portlet.

    public final void addUser(UserBean userBean) throws NamingException {
            String dn = createUserDN(userBean.getUserName());

            Attributes atrs = new BasicAttributes();
            atrs.put(USER_OBJ_CLASSES);

            Attribute cn = new BasicAttribute(IUser.USER_CN, userBean.getUserName());
            Attribute sn = new BasicAttribute(IUser.USER_SN, userBean.getUserName());
            Attribute SupplierGLN =
                    new BasicAttribute(IUser.SUPPLIER_GLN, userBean.getSupplierGLN());
            Attribute uid = new BasicAttribute(IUser.USER_ID, userBean.getUserName());
            Attribute userpassword =
                    new BasicAttribute(IUser.USER_PASSWORD, userBean.getPassword());
            Attribute givenName =
                    new BasicAttribute(IUser.USER_GIVENNAME, userBean.getUserName());
            Attribute userstatus = new BasicAttribute(IUser.USER_STATUS, IUser.USER_STATUS_FALSE);
            Attribute companyName =
                    new BasicAttribute(IUser.USER_COMPANY, userBean.getCompany());
            Attribute wpcrole =
                    new BasicAttribute(IUser.USER_WPCROLE, userBean.getWpcrole());
    //                Attribute sABSTRACT =
    //                        new BasicAttribute(IUser.ABSTRACT, userBean.getUserName());
            Attribute mail = new BasicAttribute(IUser.MAIL, userBean.getMail());
            Attribute wpcUserName =
                    new BasicAttribute(IUser.WPC_USER_NAME, userBean.getWpcUserName());

            atrs.put(cn);
            atrs.put(sn);
            atrs.put(SupplierGLN);
            atrs.put(uid);
            atrs.put(userpassword);
            atrs.put(givenName);
            atrs.put(userstatus);
            atrs.put(companyName);
            atrs.put(wpcrole);
    //                atrs.put(sABSTRACT);
            atrs.put(mail);
            atrs.put(wpcUserName);

            DirContext ctx = getDirContext();
            ctx.createSubcontext(dn, atrs);

    }*/

    /** Get the details of the Logged in user
     * @param userBean
     * @return
     * @throws NamingException
     */
    public final UserBean getUserDetails(UserBean userBean)
        throws NamingException {
        UserBean usrBean = new UserBean();
        String[] attrIDs = {
                IUser.USER_ID, IUser.USER_PASSWORD, IUser.USER_STATUS,
                IUser.USER_COMPANY, IUser.SUPPLIER_GLN, IUser.WPC_USER_NAME,
                IUser.PREFERRED_LANGUAGE, IUser.RESTRICTED_LANGUAGE,IUser.MAIL
            };
        
        try{
        String userDN = createUserDN(userBean.getUserName());

        String userName = null;
        String userStatus = null;
        String company = null;
        String password = null;
        String SupplierGLN = null;
        String wpcUserName = null;
		String wpcPreferredLanguage = null;
		String wpcRestrictededLanguage = null;

        DirContext ctx = getDirContext();
        Attributes attrs = ctx.getAttributes(userDN, attrIDs);

        /** Capture the User credentials and store it in the UserBean object
         */
        for (NamingEnumeration ae = attrs.getAll(); ae.hasMore();) {
            Attribute attr = (Attribute) ae.next();

            if (attr.getID().equalsIgnoreCase(IUser.USER_ID)) {
                for (NamingEnumeration ve = attr.getAll(); ve.hasMore();)
                    userName = (String) ve.next();

                usrBean.setUserName(userName);
            } else if (attr.getID().equalsIgnoreCase(IUser.USER_STATUS)) {
                for (NamingEnumeration ve = attr.getAll(); ve.hasMore();) {
                    userStatus = (String) ve.next();

                    if (userStatus.equalsIgnoreCase(IUser.USER_STATUS_FALSE)) {
                        usrBean.setUserStatus(IUser.USER_STATUS_FALSE);
                    } else {
                        usrBean.setUserStatus(IUser.USER_STATUS_TRUE);
                    }
                }
            } else if (attr.getID().equalsIgnoreCase(IUser.USER_COMPANY)) {
                for (NamingEnumeration ve = attr.getAll(); ve.hasMore();)
                    company = (String) ve.next();

                usrBean.setCompany(company);
            } else if (attr.getID().equalsIgnoreCase(IUser.USER_PASSWORD)) {
                for (NamingEnumeration ve = attr.getAll(); ve.hasMore();)
                    password = new String((byte[]) ve.next());

                usrBean.setPassword(password);
            } else if (attr.getID().equalsIgnoreCase(IUser.SUPPLIER_GLN)) {
                for (NamingEnumeration ve = attr.getAll(); ve.hasMore();)
                    SupplierGLN = (String) ve.next();

                usrBean.setSupplierGLN(SupplierGLN);
            } else if (attr.getID().equalsIgnoreCase(IUser.WPC_USER_NAME)) {
                for (NamingEnumeration ve = attr.getAll(); ve.hasMore();)
                    wpcUserName = (String) ve.next();

                usrBean.setWpcUserName(wpcUserName);
            }else if (attr.getID().equalsIgnoreCase(IUser.PREFERRED_LANGUAGE)) {
				for (NamingEnumeration ve = attr.getAll(); ve.hasMore();)
				wpcPreferredLanguage = (String) ve.next();
	
				usrBean.setPreferredLanguage(wpcPreferredLanguage);
			}else if (attr.getID().equalsIgnoreCase(IUser.RESTRICTED_LANGUAGE)) {
				for (NamingEnumeration ve = attr.getAll(); ve.hasMore();)
					wpcRestrictededLanguage = (String) ve.next();
		
					usrBean.setRestrictedLanguage(wpcRestrictededLanguage);
				}
        }
        }catch(Exception e){
        	System.out.println("Exception in getUserDetails " + e);
        	e.printStackTrace(System.out);
        }
      
        return usrBean;
    }

    /** Get the List of Portal Users
     * @param userBean
     * @return
     * @throws NamingException
     */
    public final Vector getUsersList(String filter) throws NamingException {
        String[] attrIDs = {
                IUser.USER_ID, IUser.SUPPLIER_GLN, IUser.USER_STATUS,
                IUser.USER_COMPANY, IUser.USER_WPCROLE, IUser.WPC_USER_NAME
            };
        SearchControls controls = new SearchControls();
        controls.setSearchScope(SearchControls.SUBTREE_SCOPE);
        controls.setReturningAttributes(attrIDs);

        Vector usersList = new Vector();

        DirContext ctx = getDirContext();
        NamingEnumeration result = ctx.search(sLDAPUserSuffix + "," +
                sLDAPSuffix, filter, controls);

        while (result.hasMore()) {
            SearchResult sr = (SearchResult) result.next();
            Attributes attrs = sr.getAttributes();

            /** Capture the User credentials and store it in the UserBean object
             */

            //	Create the User object
            UserBean usrBean = new UserBean();

            for (NamingEnumeration ae = attrs.getAll(); ae.hasMore();) {
                Attribute attr = (Attribute) ae.next();

                if (attr.getID().equalsIgnoreCase(IUser.USER_ID)) {
                    for (NamingEnumeration ve = attr.getAll(); ve.hasMore();)
                        usrBean.setUserName((String) ve.next());
                } else if (attr.getID().equalsIgnoreCase(IUser.SUPPLIER_GLN)) {
                    for (NamingEnumeration ve = attr.getAll(); ve.hasMore();)
                        usrBean.setSupplierGLN((String) ve.next());
                } else if (attr.getID().equalsIgnoreCase(IUser.USER_STATUS)) {
                    for (NamingEnumeration ve = attr.getAll(); ve.hasMore();) {
                        String userStatus = (String) ve.next();

                        if (userStatus.equalsIgnoreCase(IUser.USER_STATUS_FALSE)) {
                            usrBean.setUserStatus(IUser.USER_STATUS_FALSE);
                        } else {
                            usrBean.setUserStatus(IUser.USER_STATUS_TRUE);
                        }
                    }
                } else if (attr.getID().equalsIgnoreCase(IUser.USER_COMPANY)) {
                    for (NamingEnumeration ve = attr.getAll(); ve.hasMore();)
                        usrBean.setCompany((String) ve.next());
                } else if (attr.getID().equalsIgnoreCase(IUser.USER_WPCROLE)) {
                    for (NamingEnumeration ve = attr.getAll(); ve.hasMore();)
                        usrBean.setWpcrole((String) ve.next());
                } else if (attr.getID().equalsIgnoreCase(IUser.WPC_USER_NAME)) {
                    for (NamingEnumeration ve = attr.getAll(); ve.hasMore();)
                        usrBean.setWpcUserName((String) ve.next());
                }
            }

                usersList.addElement(usrBean);
        }

        return usersList;
    }

    /** approveUsers Portal Users
     * @param userIds
     * @return
     * @throws NamingException
     */
    public final void approveUsers(ArrayList users) throws NamingException {
        //	iterating through each User(s)
        for (int i = 0; i < users.size(); i++) {
            UserBean usrBean = (UserBean) users.get(i);
            updateUserStatus(usrBean);
        } // end of for loop for User(s)
    }

    /** disapproveUsers Portal Users
     * @param userIds
     * @return
     * @throws NamingException
     */
    public final void disapproveUsers(ArrayList userIds)
        throws NamingException {
        //	iterating through each User(s)
        for (int i = 0; i < userIds.size(); i++) {
            String userId = (String) userIds.get(i);
            disapproveUserStatus(userId);
        } // end of for loop for User(s)
    }

    /** updateUserStatus Portal Users
     * @param userId
     * @return
     * @throws NamingException
     */
    private final void updateUserStatus(UserBean usrBean)
        throws NamingException {
        if (usrBean.getUserName() == null) {
            throw new IllegalArgumentException();
        }

        String userDN = createUserDN(usrBean.getUserName());

        ModificationItem[] modAttrs = new ModificationItem[2];

        modAttrs[0] = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,
                new BasicAttribute(IUser.USER_STATUS, IUser.USER_STATUS_TRUE));
        modAttrs[1] = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,
                new BasicAttribute(IUser.WPC_USER_NAME, usrBean.getWpcUserName()));

        DirContext ctx = getDirContext();
        System.out.println("LdapHelper :: updateUserStatus ===>> " + modAttrs);
        ctx.modifyAttributes(userDN, modAttrs);
    }

    /** disapproveUserStatus
     * @param userId
     * @return
     * @throws NamingException
     */
    private final void disapproveUserStatus(String userId)
        throws NamingException {
        if (userId == null) {
            throw new IllegalArgumentException();
        }

        String userDN = createUserDN(userId);
        ModificationItem[] modAttr = new ModificationItem[1];
        modAttr[0] = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,
                new BasicAttribute(IUser.USER_STATUS, IUser.USER_STATUS_FALSE));

        DirContext ctx = getDirContext();
        ctx.modifyAttributes(userDN, modAttr);
    }
}
