
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * This package consists of classes pertaining to Supplier Portal User,
 * Login and LDAP access related functionality.
 */
package com.ibm.ccd.ui.services.user;

import java.io.IOException;

import com.ibm.portal.WpsException;
import com.ibm.wps.auth.ErrorBean;
import com.ibm.wps.engine.Problem;
import com.ibm.wps.engine.RunData;


/**
 * LoginUserAuth is a Customized class  for Login to Supplier Portal
 */
public class LoginUserAuth extends com.ibm.wps.engine.commands.LoginUserAuth {
    // --- PRIVATE MEMBERS --- //
    private final static int MY_ERROR_CODE = USER_DEFINED_ERROR + 1;

    // --- PRIVATE MEMBERS: Defined for Supplier Portal Login Cusomization
    private final static int USER_INACTIVE_ERROR_CODE = USER_DEFINED_ERROR + 1;
    private final static int USER_INVALID_COMPANY_ERROR_CODE = USER_DEFINED_ERROR +
        2;
    private final static int LDAP_ERROR_CODE = USER_DEFINED_ERROR + 3;

    /** Singleton Instance variable for UserManager
     */
    private UserManager userManager = null;

    /* (non-Javadoc)
     * @see com.ibm.wps.engine.commands.LoginUser#doPreLogin(com.ibm.wps.engine.RunData, java.lang.String, java.lang.String)
     */
    protected void doPreLogin(RunData aRunData, String aUserID, String aPassword)
        throws WpsException {
        // <do whatever you want prior to the user authentiaction>
        // (throw an exception if the login shall fail)
        // []
        // </do whatever you want prior to the user authentiaction>
        // call parent class  does nothing, but just to be sure
        super.doPreLogin(aRunData, aUserID, aPassword);
    }

    /* (non-Javadoc)
     * @see com.ibm.wps.engine.commands.LoginUser#doAuthenticate(com.ibm.wps.engine.RunData, java.lang.String, java.lang.String)
     */
    protected ErrorBean doAuthenticate(RunData aRunData, String aUserID,
        String aPassword) {
        try {
            // <do customized authentiaction>

            /** Extract the Company from the Login page.
             */
            String company = aRunData.getQueryData().getString("company");

            /** Instantiate the UserBean object from details captured in Login page
             */
            UserBean usrBean = new UserBean();
            usrBean.setUserName(aUserID);
            usrBean.setPassword(aPassword);
            usrBean.setCompany(company);

            userManager = UserManager.getInstance();

            if(userManager == null){
            	System.out.println("UserManager is null");
            }
            /** Construct the result UserBean object and which will contain the User credentials from LDAP
             */
            else {
            UserBean resUsrBean = null;
            resUsrBean = userManager.validateLoginUser(usrBean);

            /** compare the results with the User credentials and display it to the user
             */
            if (!"true".equals(resUsrBean.getUserStatus())) {
                return new ErrorBean(USER_INACTIVE_ERROR_CODE,
                    new Exception("This User is InActive, Please contact your System Administrator"));
            }

            if (!resUsrBean.getCompany().equals(company)) {
                return new ErrorBean(USER_INVALID_COMPANY_ERROR_CODE,
                    new Exception("This User doesn't belong to this company, Please verify"));
            }
           }
        } catch (Exception e) {
            System.out.println(
                "LoginUserAuth :: doAuthenticate : Exception Occured ---->" +
                e.getMessage());
            e.printStackTrace(System.out);

            return new ErrorBean(LDAP_ERROR_CODE, e);
        }

        return super.doAuthenticate(aRunData, aUserID, aPassword);
    }

    /* (non-Javadoc)
     * @see com.ibm.wps.engine.commands.LoginUser#onAuthenticationError(com.ibm.wps.engine.RunData, com.ibm.wps.auth.ErrorBean)
     */
    protected void onAuthenticationError(RunData aRunData, ErrorBean aErrorBean) {
        if (aErrorBean.getErrorCode() >= USER_DEFINED_ERROR) {
            /** Do the Error handling code for LDAP & USER
             */
            if (aErrorBean.getErrorCode() == LDAP_ERROR_CODE) {
                aErrorBean.setCause("ldapNotConfigured");

                Problem ldapNotConfigured = new Problem("ldap.not.configured");
                aRunData.setProblem(ldapNotConfigured);

                try {
                    // Redirect to the Login Screen whenvever LDAP/Session Timeout error occurs 
                    aRunData.setRedirectURL("/wps/portal/!ut/p/.cmd/li");

                    //aRunData.setRedirectURL("/wps/portal/!ut/p/.scr/Login");
                    aRunData.sendRedirect();
                } catch (IOException ie) {
                    ie.printStackTrace();
                }
            }

            if (aErrorBean.getErrorCode() == USER_INACTIVE_ERROR_CODE) {
                aErrorBean.setCause("userInactive");

                Problem prbmInactive = new Problem("login.invalid.userStatus");
                aRunData.setProblem(prbmInactive);
            }

            if (aErrorBean.getErrorCode() == USER_INVALID_COMPANY_ERROR_CODE) {
                aErrorBean.setCause("userInvalidCompany");

                Problem prbmInactive = new Problem("login.invalid.company");
                aRunData.setProblem(prbmInactive);
            }
        }

        // call the parent class to handle the other errors
        // The Login screen will be rendered in case of user defined error codes
        super.onAuthenticationError(aRunData, aErrorBean);
    }

    /* (non-Javadoc)
     * @see com.ibm.wps.engine.commands.LoginUser#doPostLogin(com.ibm.wps.engine.RunData, java.lang.String, java.lang.String)
     */
    protected void doPostLogin(RunData aRunData, String aUserID,
        String aPassword) throws WpsException {
        // <do whatever you want after a successful user authentiaction>
        // []
        // </do whatever you want after a successful user authentiaction>
        // call parent class  needed for resume session functionality
        super.doPostLogin(aRunData, aUserID, aPassword);
    }
}
