
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * This package consists of classes pertaining to Supplier Portal User,
 * Login and LDAP access related functionality.
 */
package com.ibm.ccd.ui.services.user;

import java.util.ArrayList;
import java.util.Vector;

import javax.naming.NamingException;


/**
 * UserManager Intermediate class interacts b/w the User Registration portlet and and the Ldap related functions.
 */
public class UserManager {
    /** Singleton Instance variable for UserManager
     */
    private static UserManager userManager;

    /** Singleton Instance variable for LdapHelper
     */
    private static LdapHelper ldapHelper;

    // Private constructor for UserManager
    private UserManager() {
    }

    /** Singleton class UserManager
     * @return UserManager
     */
    public static UserManager getInstance() {
        if (userManager == null) {
            synchronized (UserManager.class) {
                if (userManager == null) {
                    userManager = new UserManager();
                    ldapHelper = LdapHelper.getInstance();

                    if (ldapHelper == null) {
                        return null;
                    }
                }
            }
        }

        return userManager;
    }

    /** This method authenticates the User credentials against LDAP.
     * @param userBean
     * @return
     */
    public UserBean validateLoginUser(UserBean userBean) {
        UserBean resUsrBean = null;

        try {
            if (ldapHelper == null) {
                ldapHelper = LdapHelper.getInstance();
            }

            resUsrBean = ldapHelper.getUserDetails(userBean);
        } catch (Exception ne) {
            resUsrBean.setErrorMessage(ne.getMessage());
            ne.printStackTrace();
        }

        return resUsrBean;
    }

    /** This method Register's the user with the Supplier Portal.
     * @param userBean
     * @return
     */

    /*


    //User Registration will now be done using Portal's Registration Portlet.

    public UserBean registerSupplierUser(UserBean userBean) {
            UserBean resUsrBean = new UserBean();
            resUsrBean.setSupplierGLN(userBean.getSupplierGLN());
            resUsrBean.setUserName(userBean.getUserName());
            resUsrBean.setPassword(userBean.getPassword());
            resUsrBean.setConfirm_password(userBean.getConfirm_password());
            resUsrBean.setUserStatus(userBean.getUserStatus());
            resUsrBean.setWpcrole(userBean.getWpcrole());
            resUsrBean.setMail(userBean.getMail());
            resUsrBean.setCompany(userBean.getCompany());
            resUsrBean.setWpcUserName(userBean.getWpcUserName());

            try {

                    ldapHelper.addUser(userBean);
                    // Successfully added the Supplier Portal user
                    resUsrBean.setSuccessMessage("MESSAGE_CODE_256");

            } catch (InvalidNameException ine) {
                    // User Registration error has occured, Contact System Administrator.
                    resUsrBean.setErrorMessage("MESSAGE_CODE_254");
                    ine.printStackTrace();
            } catch (NameAlreadyBoundException nabe) {
                    // Successfully added the Supplier Portal user
                    resUsrBean.setErrorMessage("MESSAGE_CODE_253");
                    nabe.printStackTrace();
            } catch (InvalidAttributeValueException iave) {
                    resUsrBean.setErrorMessage("MESSAGE_CODE_254");
                    iave.printStackTrace();
            } catch (NamingException ne) {
                    resUsrBean.setErrorMessage("MESSAGE_CODE_254");
                    ne.printStackTrace();
            }
            return resUsrBean;
    }*/

    /** This method returns the List of Users of Supplier Portal.
     * @param userBean
     * @return
     */
    public Vector getSupplierPortalUserList() {
        Vector resUserList = null;
        String filter = "(&(uid=*))";

        try {
            resUserList = ldapHelper.getUsersList(filter);
        } catch (NamingException ne) {
            ne.printStackTrace();
        }

        return resUserList;
    }

    /** This method Approves the Users of Supplier Portal.
     * @param users
     * @return None
     */
    public void approveSupplierPortalUsers(ArrayList users) {
        try {
            ldapHelper.approveUsers(users);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /** This method Disapproves the Users of Supplier Portal.
     * @param userIds
     * @return None
     */
    public void disapproveSupplierPortalUsers(ArrayList userIds) {
        try {
            ldapHelper.disapproveUsers(userIds);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
