
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.taglib.dataobjects;

import java.util.ArrayList;


/**
* @purpose Represents the Results of an action like Search, Lookup
* @desc    This class provides the detail information about the  results  data like <br>
*          MetaData    : The result elements name, type etc. <br>
*          ControlData : Start,End indexes, Current and TotalSize of the results <br>
*          ResultData  :  The current result set associated
* @author  IBMIndia
* @version 1.0
**/
public class ResultData {
   /**
    *  Total result size associated with the the current search
    */
    private long totalResultSize;

  
    /**
    *  MetaData associated with the the current search
    **/
    private ArrayList resultMetaData;

    /**
    *   Result rows and data associated with the the current search
    **/
    private ArrayList resultRowData;

    /**
    * @desc  Constructs an empty ResutlData
    * @since Iteration 2
    **/
    public ResultData() {
        this.resultMetaData = new ArrayList(5);
        this.resultRowData = new ArrayList(5);
    }

    /**
    * @desc  Constructs  ResutlData with the specified metadata and resultrows
    * @param ArrayList
    * @param ArrayList
    * @sicne Iteration 2
    **/
    public ResultData(ArrayList aResultMetaData, ArrayList aResultRowData) {
        this.resultMetaData = aResultMetaData;
        this.resultRowData = aResultRowData;
    }

    /**
    * @desc  Returns the MetaData associated with the current results
    * @return ArrayList of Metadata elements
    * @since Iteration 2
    **/
    public ArrayList getResultMetaData() {
        return resultMetaData;
    }

    /**
    * @desc  Returns the MetaData associated with the current results
    * @param int
    * @return ArrayList of Metadata elements
    * @since Iteration 2
    **/
    public ResultMetaData getResultMetaData(int index) {
        return ((resultMetaData != null) && (resultMetaData.size() > 0))
        ? (ResultMetaData) resultMetaData.get(index) : null;
    }

    /**
    * @desc  Returns the ResultData associated with the current results
    * @return ArrayList , Array of Result Rows
    * @since Iteration 2
    **/
    public ArrayList getResultRowData() {
        return resultRowData;
    }

    /**
    * @desc  Returns the ResultRow at the specidied index
    * @param index of the ResultRows
    * @return ResultRowData at the given index
    * @since Iteration 2
    **/
    public ResultRowData getResultRowData(int index) {
        return ((resultRowData != null) && (resultRowData.size() > 0))
        ? (ResultRowData) resultRowData.get(index) : null;
    }

    /**
     * @desc  Sets the metadata  for the current  results
     * @param  ArrayList of MetadataElements
     * @since Itearation 2
     **/
    public void setResultMetaData(ArrayList aResultMetaData) {
        this.resultMetaData = aResultMetaData;
    }

   
    /**
     * @desc  Sets the resultdata  for the current results
     * @param ArrayList of Search Result Row Elements
     * @since Iteration 2
     **/
    public void setResultRowData(ArrayList aResultRowData) {
        this.resultRowData = aResultRowData;
    }


    /**
    * @desc  Returns the total result size  associated with the current results
    * @return int, Total result set size
    * @since Iteration 2
    **/
    public long getTotalResultSize() {
        return totalResultSize;
    }


    /**
     * @desc  Sets the total size of the  current search.
     * @param int , total result size
     * @since Iteration 2
     **/
    public void setTotalResultSize(long totalResutltSize) {
        this.totalResultSize = totalResutltSize;
    }

}
