
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.taglib.dataobjects;

import com.ibm.ccd.ui.tags.ISelectListTag;


/**
* @purpose Represents the Metdadata associated with the  Results
* @desc    This class provides the information about  result attributes <br>
*           viz. name,  type, visibility, sortable
*           This information represents the column data in the result set
* @author  IBMIndia
* @version 1.0
**/
public class ResultMetaData implements ISelectListTag {
    /**
     * Name of the result element
     **/
    private String name;
    private int type = 1;
    private int listBoxIndex = -1;
    private String displayName=null;

    /**
     * Sortability of the result element
     **/
    private boolean sortable;

    /**
     * The currently sorted property of this element
     */
    private boolean currentlySorted;

    /**
     * Visibility of the result element
     **/
    private boolean displayable = false;

    /**
     * TRUE if attribute is primary key
     */
    private boolean isMandatory = false;

    /**
     * @desc  Constructs an empty ResultMetaData
     * @since Iteration 2
     **/
    public ResultMetaData() {
    }

    /**
     * @desc  Constructs  ResultMetaData with the specified name and sortable flag
     * @param name, Name of the ELement
     * @param type of the elemement
     *
     */
    public ResultMetaData(String name, int type) {
        this.name = name;
        this.type = type;
    }

    /**
     * @desc  Constructs  ResultMetaData with the specified name and sortable flag
     * @param name, Name of the ELement
     * @param int type of the elemement
     * @param boolean
     * @param boolean
     */
    public ResultMetaData(String name, int type, boolean sortable,
        boolean displayable) {
        this.name = name;
        this.type = type;
        this.sortable = sortable;
        this.displayable = displayable;
    }

    /**
     * @desc  Returns the name  associated with the current  element
     * @return String, the name  of the result set element
     */
    public String getName() {
        return name;
    }

    /**
     * @desc  Returns the type of the result element
     * @return int, Element Type
     * @see    ISearchResultElementTypesEnum
     */
    public int getType() {
        return type;
    }

    /**
     * @desc  Sets the type of the result element
     * @param int, Element Type
     * @see    ISearchResultElementTypesEnum
     */
    public void setType(int aType) {
        this.type = aType;
    }

    /**
     * @desc  Specifies visibility of this element
     * @return true, If the element can be displayed <br>
     *         false, If the element can not be displayed <br>
     */
    public boolean isDisplayable() {
        return displayable;
    }

    /**
     * @desc  Specifies sortability of the result set based on this element
     * @return true, If the result set can be sorted based on this  <br>
     *         false, If the result set cann't be sorted based on this  <br>
     */
    public boolean isSortable() {
        return sortable;
    }

    /**
     * @desc  Returns the action to be done to select the result Object
     * @return String, the Select action
     */

    //public String getActionForElements() {
    //return actionForColumnElements;
    //}

    /**
     * @desc  Specifies visibility of this element
     * @param true, If the element can be displayed <br>
     *         false, If the element can not be displayed <br>
     */
    public void setDisplayable(boolean aDisplayable) {
        this.displayable = aDisplayable;
    }

    /**
     * @desc  Sets the name  associated with the current  element
     * @param String, the name  of the result set element
     */
    public void setName(String aName) {
        this.name = aName;
    }

    /**
     * @desc  Specifies sortability of the result set based on this element
     * @param true, If the result set can be sorted based on this  <br>
     *         false, If the result set cann't be sorted based on this  <br>
     */
    public void setSortable(boolean sortable) {
        this.sortable = sortable;
    }

    /**
     * Gets the currentlySorted
     * @return Returns a boolean
     */
    public boolean getCurrentlySorted() {
        return currentlySorted;
    }

    /**
     * Sets the currentlySorted
     * @param currentlySorted The currentlySorted to set
     */
    public void setCurrentlySorted(boolean currentlySorted) {
        this.currentlySorted = currentlySorted;
    }

    public boolean isSelected() {
        return isDisplayable();
    }

    public String getDisplayName() {
       return displayName;
    }

    public int getListBoxIndex() {
        return listBoxIndex;
    }

    /**
     * @param i
     */
    public void setListBoxIndex(int i) {
        listBoxIndex = i;
    }

    /**
     * @return
     */
    public boolean isMandatory() {
        return isMandatory;
    }

    /**
     * @param b
     */
    public void setMandatory(boolean b) {
        isMandatory = b;
    }
	/**
	 * @param string
	 */
	public void setDisplayName(String string) {
		displayName = string;
	}

}
