
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.taglib.dataobjects;

import java.util.HashMap;


/**
* @purpose Represents the ResultSet (A Row or ResultSet containtaing cells of result elements) associated with the Results
* @desc    This class provides the information about  result sets <br>
*          Each search result represents a row in the results and provides the information
*          of result elements (cells/colums) as a hashMap and reference of the result object
* @author  IBMIndia
* @version 1.0
**/
public class ResultRowData {
    /**
     *  The Reference Object  representing the result row
     **/
    private Object rowReference;

    /**
     *  The Search result dataElements
     **/
    private HashMap rowElements;

    /**
    * @desc  Constructs an empty ResultRowData
    * @since Iteration 2
    **/
    public ResultRowData() {
        this(null, new HashMap(5));
    }

    /**
    * @desc  Constructs the  ResultRowData with the specified referencebean and rowElements map
    * @param aRowReference - The reference of the row
    * @param aRowElements - The  elements(column elements) HashMap of this row
    * @since Iteration 2
    **/
    public ResultRowData(Object aRowReference, HashMap aRowElements) {
        this.rowReference = aRowReference;
        this.rowElements = aRowElements;
    }

    /**
    * @desc  Returns the ReferenceObject associated with the current result row
    * @return Object, the refernced object of the row
    * @since Iteration 2
    **/
    public Object getRowReference() {
        return rowReference;
    }

    /**
    * @desc  Returns the HashMap of search result elemetns of this row
    * @return HashMap, being key as  name and value as the result value of the element
    * @since Iteration 2
    **/
    public HashMap getRowElements() {
        return rowElements;
    }

    /**
    * @desc   Returns Value of the given element name
    * @param  elementName, The name of the element
    * @return Cell Element , The result value of the element
    * @note    If there is no element for this name , then  returns null
    **/
    public Object getElementValue(String elementName) {
        return (rowElements != null) ? rowElements.get(elementName) : null;
    }

    /**
    * @desc  Sets the ReferenceObject   associated with the current  result row
    * @param ReferenceObject, the refernce  for this result row
    **/
    public void setRowReference(Object aRowReference) {
        this.rowReference = aRowReference;
    }

    /**
     * @desc Adds the element to rowdata
     * @param name, Element Name of type String
     * @param value , Element value of type String
     */
    public void addElement(String name, Object value) {
        if (rowElements == null) {
            rowElements = new HashMap(5);
        }

        rowElements.put(name, value);
    }

    /**
     * @desc Removes  the element from rowdata
     * @param name, Element Name of type String
     */
    public void removeElement(String name) {
        rowElements.remove(name);
    }

    /**
    * @desc  Sets the HashMap of search result elements of this row
    * @param HashMap, being key as  name and value as the result value of the element
    **/
    public void setRowElements(HashMap aRowElements) {
        this.rowElements = aRowElements;
    }
}
