
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.taglib.dataobjects;

import java.util.HashMap;


/**
* @purpose SearchAttribute which represents the Text  Objects
* @author  IBM India
* @version 1.0
* @since   Iteration 2
**/
public class TextSearchElement extends BaseSearchElement implements Cloneable {
    /**
     * Identifier for default value
     */
    private String defValue;

    /**
     * Identifier for value
     */
    private String value;

    /**
     * @desc  Constructs an empty TextSearchElement
     **/
    public TextSearchElement() {
        super(ISearchElementTypesEnum.TEXT);
    }

    /**
     * @desc  Constructs the TextSearchElement for the specified name, oeprator, operatormap and madnadatory flag
     * @param String
     * @param String
     * @param Integer
     * @param HashMap
     * @param boolean
     * @param boolean
     **/
    public TextSearchElement(String aAttributeName, String aDefaultValue,
        String aDefaultOperator, HashMap aOperatorsMap, boolean aIsMandatory,
        boolean aIsDefaultable) {
        super(ISearchElementTypesEnum.TEXT, aAttributeName, aDefaultOperator,
            aOperatorsMap, aIsMandatory, aIsDefaultable);
        this.defValue = aDefaultValue;
    }

    /**
     * @desc  Constructs the TextSearchElement for the specified name,value. oeprator, operatormap and madnadatory flag
     * @param String
     * @param String
     * @param Integer
     * @param HashMap
     **/
    public TextSearchElement(String aAttributeName, String aDefaultValue,
        String aDefaultOperator, HashMap aOperatorsMap) {
        this(aAttributeName, aDefaultValue, aDefaultOperator, aOperatorsMap,
            true, false);
    }

    /**
     *@desc Returns the defautl text value associated with this
     *@return String, The default Value
     **/
    public String getDefaultValue() {
        return defValue;
    }

    /**
     *@desc Returns the assgined text value associated with this
     *@return String, The Assigned Value
     **/
    public String getValue() {
        return value;
    }

    /**
     *@desc Sets the defautl text value
     *@param String, The default Value
     **/
    public void setDefaultValue(String aDefaultValue) {
        this.defValue = aDefaultValue;
    }

    /**
     *@desc Sets the assgined text value
     *@param String, The Assigned Value
     **/
    public void setValue(String aValue) {
        this.value = aValue;
    }

    /* (non-Javadoc)
     * @see com.ibm.ccd.ui.taglib.dataobjects.BaseSearchElement#clone()
     */
    public Object clone() throws CloneNotSupportedException {
        return super.clone();
    }
}
