
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/**
 * This is Base class for all html Fields.All html Tag Classes should extend this class.
 * Tags attributes
 * name
 * id
 * value
 * mandatory
 * style
 * size
 * alt
 * enabled
 */
package com.ibm.ccd.ui.tags;

import java.io.IOException;
import java.util.Locale;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.TagSupport;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletLog;
import org.apache.jetspeed.portlet.PortletRequest;

import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;


public abstract class BaseUITag extends TagSupport implements ITagConstants,
    IReference {
    /* Variables for set by Custom tag */

    /**
     * attributes for the name field of html element
     */
    protected String name;

    /**
     * attributes for the id field of html element
     */
    protected String id = "";

    /**
     * attributes for the value field of html element
     */
    protected String value = "";

    /**
     * attributes for the style field of html element
     */
    protected String style = DEFAULT_EDIT_STYLE;

    /**
     * attributes for the alt field of html element.The default value is blank string.
     * This is for tool tip.
     */
    protected String alt = "";

    /**
     * attributes for the mandatory field of html element.The default value is false.
     */
    protected String mandatory = "false";

    /**
     * attributes for setting the size of html element.Optional attribute
     */
    private String size = String.valueOf(DEFAULT_FIELD_SIZE);

    /* Variables used by Tag classes */

    /**
     * attribute UILabel is used to for localized label for html field
     */
    protected UILabel uiLabel = null;

    /**
     * attribute used for log
     */
    protected PortletLog log = null;

    /**
     * attribute stores the PortletContext
     */
    protected PortletContext portletContext = null;

    /**
     * At times the tag needs to be generated without the label,
     * this attribute helps control the label generation
     */
    protected boolean labelRequired = true;

    /**
     * This attribute is true when the Tag Class is invoked from
     * Emitter caller class and not from customized tag.so pageContext will not
     * be available.portletContext will be available.
     */
    protected boolean fromHtmlEmitter = false;

    /**
     * Locales are passed from Request object.The default is locale is US.
     */
    protected Locale locale = Locale.US;

    /**
     * attribute enabled can have value true or false.Html field can enabled or disabled.
     */
    protected String enabled = "true";

    /**
     * this JspWriter to place the output
     */
    protected JspWriter out;

    /**
     * BaseUITag : Constructor
     */
    public BaseUITag() {
        super();
    }

    protected void initialize() {
        portletContext = ((PortletConfig) pageContext.getAttribute(VAR_PORTLET_CONFIG)).getContext();
        log = portletContext.getLog();

        /* Initialising the locale with locale request */
        locale = pageContext.getRequest().getLocale();
    }

    /**
     * @param inputParam
     * @param defaultValue
     * @return
     */
    protected int validate(String inputParam, int defaultValue)
        throws Exception {
        log.debug("validate ->");

        int retValue = 0;

        try {
            if (inputParam == null) {
                retValue = defaultValue;
            } else {
                retValue = Integer.parseInt(inputParam);
            }
        } catch (NumberFormatException numex) {
            log.warn(UIUtils.getTextMessages(portletContext, locale,
                    "MESSAGE_CODE_51") + " " + numex);
            retValue = defaultValue;
        } catch (Exception ex) {
            retValue = defaultValue;
            log.warn(UIUtils.getTextMessages(portletContext, locale,
                    "MESSAGE_CODE_52") + " " + ex);
        }

        return retValue;
    }

    /**
    * doStartTag is called by the JSP container when the tag is encountered
    */
    public int doStartTag() {
        try {
            out = pageContext.getOut();

            //	log is initialized ..
            initialize();

            if (getName() != null) {
                out.println("<TR align=\"CENTER\">");

                if (pageContext != null) {
                    fromHtmlEmitter = false;
                    uiLabel = new UILabel(pageContext);
                } else {
                    fromHtmlEmitter = true;
                    uiLabel = new UILabel(portletContext);
                }

                displayTag();
                out.println("</TR>");
            } else {
                log.warn(UIUtils.getTextMessages(portletContext, locale,
                        "MESSAGE_CODE_59"));
            }
        } catch (Exception ex) {
            log.error(UIUtils.getTextMessages(portletContext, locale,
                    "MESSAGE_CODE_53") + " " + ex);
            ex.printStackTrace();

            // throw new Error();
        }

        log.debug("doStartTag end <- ");

        return SKIP_BODY;
    }

    /**
     * @desc  - constructs the html component structure. The display takes care of
     * displaying the label, and the field which corresponds to this tag.
     * @throws java.io.JspException
      */
    protected void displayTag() throws Exception {
        log.debug(" displayTag ->");
        displayLabel();
        displayField();
    }

    /**
     * @desc Displays the label for this html component. The label is displayed if a
     * label is required. By default all tags require a label, the ones that do not
     * want a label return false for the isLabelRequired method.
     * @throws javax.servlet.jsp.JspException
     */
    protected void displayLabel() throws Exception {
        log.debug(" displayLabel ->");

        if (!isLabelRequired()) {
            return;
        }

        try {
            out.print("<TD valign=\"top\" align =\"RIGHT\" CLASS=\"" +
                DEFAULT_LABEL_STYLE + "\" nowrap>");
            displayForMandatory();
            out.print(uiLabel.getValue(name) + " :");
            out.print("</TD>");
        } catch (Exception ex) {
            //			out.print(name + " :");
            //			out.print("</TD>");
            log.warn(UIUtils.getTextMessages(portletContext, locale,
                    "MESSAGE_CODE_57") + " " + ex);

            //ex.printStackTrace();			
            //throw new JspTagException();
        }

        log.debug(" displayLabel <- ");
    }

    /**
     * @desc Renders the html content which the tag is expected to display. The field
     * display takes care of checking if the field needs to be visible, and if it
     * needs to be enabled.
     * @throws javax.servlet.jsp.JspException
     */
    protected void displayField() throws Exception {
        renderField();
    }

    /**
     * @param portletRequest
     * @param key
     * @return
     */
    public String getTextLabel(PortletRequest portletRequest, String key) {
        String retStr = UIUtils.getTextLabel(portletContext, portletRequest, key);

        return retStr;
    }

    /**
     * @param portletRequest
     * @param key
     * @return
     */
    public String getTextMessages(PortletRequest portletRequest, String key) {
        String retStr = UIUtils.getTextMessages(portletContext, portletRequest,
                key);

        return retStr;
    }

    /**
     * Method for generating the html field.Tag Subclasses needs overwrite this method
     */
    protected abstract void renderField() throws Exception;

    /**
     * @desc performs the mandatory field check. If the field is mandatory, then the
     * default look and feel is to print a red asterisk against the label name
     * @throws javax.servlet.jsp.JspException
     */
    protected void displayForMandatory() throws JspException {
        try {
            if (isMandatory()) {
                out.print("<SPAN CLASS=\"wpcFormAsterisk\"> * </SPAN>");
            }
        } catch (IOException ex) {
            throw new JspTagException(ex.getMessage());
        }
    }

    /**
     * doEndTag is called by the JSP container when the tag is closed
     */
    public int doEndTag() {
        return 0;
    }

    /**
     * @return
     */
    public String getValue() {
        return value;
    }

    /**
     * @param string
     */
    public void setValue(String string) {
        value = string;
    }

    /**
     * @return
     */
    public String getName() {
        return name;
    }

    /**
     * @param string
     */
    public void setName(String string) {
        name = string;
    }

    /**
     * @param string
     */
    public String getId() {
        return id;
    }

    /**
     * @param string
     */
    public void setId(String string) {
        id = string;
    }

    /**
     * @return
     */
    public PortletLog getLog() {
        return log;
    }

    /**
     * @return
     */
    public PortletContext getPortletContext() {
        return portletContext;
    }

    /**
     * @param log
     */
    public void setLog(PortletLog log) {
        this.log = log;
    }

    /**
     * @param context
     */
    public void setPortletContext(PortletContext context) {
        portletContext = context;
    }

    /**
     * @return
     */
    public String getStyle() {
        return style;
    }

    /**
     * @param string
     */
    public void setStyle(String string) {
        style = string;
    }

    /**
     * @return
     */
    public boolean isLabelRequired() {
        return labelRequired;
    }

    /**
     * @return
     */
    public UILabel getUiLabel() {
        return uiLabel;
    }

    /**
     * @param b
     */
    public void setLabelRequired(boolean b) {
        labelRequired = b;
    }

    /**
     * @param label
     */
    public void setUiLabel(UILabel label) {
        uiLabel = label;
    }

    /**
     * This method returns true if Tag class is invoked from Emitter Class
     * @return boolean fromHtmlEmitter
     */
    protected boolean isFromHtmlEmitter() {
        return fromHtmlEmitter;
    }

    /**
     * @return
     */
    public Locale getLocale() {
        return locale;
    }

    /**
     * @param locale
     */
    public void setLocale(Locale locale) {
        this.locale = locale;
    }

    /**
     * @return
     */
    public String getMandatory() {
        return mandatory;
    }

    /**
     * @return
     */
    protected boolean isMandatory() {
        if ((mandatory.equalsIgnoreCase("true")) ||
                (mandatory.equalsIgnoreCase("yes"))) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * @param string
     */
    public void setMandatory(String string) {
        mandatory = string;
    }

    /**
     * @return
     */
    protected boolean isEnabled() {
        if ((enabled.equalsIgnoreCase("true")) ||
                (enabled.equalsIgnoreCase("yes"))) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * @param param
     */
    public void setEnabled(String param) {
        enabled = param;
    }

    /**
     * attribute for getting the size of html elements.Some html element may not use it.
     * @return
     */
    public String getSize() {
        return size;
    }

    /**
     * attribute for setting the size of html elements.
     */
    public void setSize(String string) {
        size = string;
    }

    /**
     * @return
     */
    public String getEnabled() {
        return enabled;
    }

    /**
     * @return
     */
    public String getAlt() {
        return alt;
    }

    /**
     * @param string
     */
    public void setAlt(String string) {
        alt = string;
    }
}
