
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/**
 * This class is the tag implementation for the DropDown input.
 * Tags attributes
 * collection: Collection attribute needs a HashMap key/value pairs Key will
 *                     appear as id of dropdown options and value will appears label.
 * multiselect: attribute is boolean type of flag.If sets to true then
 *                          It is multiselect listbox appears.Default is false.
 * Inherited from Base tag
 * name
 * id
 * value
 * mandatory
 * style
 * size
 * alt
 * enabled
 */
package com.ibm.ccd.ui.tags;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import com.ibm.ccd.ui.util.UIUtils;


public class UIDropDownTag extends BaseUITag {
    private String multiselect = "false";

    /**
     * This represents the selected value in the drop down list box
     */
    private HashMap collection;

    /*
     * This method renders HTML Dropdown field to response object
     */
    protected void renderField() throws Exception {
        log.debug(" renderField ->");

        try {
            StringBuffer htmlCode = new StringBuffer("");
            htmlCode.append("<TD ALIGN =\"LEFT\">");

            HashMap hshMap = getCollection();
            String listBoxSize = "";

            if (isMultiselect()) {
                listBoxSize = "" + DEFAULT_LISTBOX_SIZE;
            } else {
                listBoxSize = "1";
            }

            if (isEnabled()) {
                if (value == null) {
                    value = "";
                }

                htmlCode.append("<SELECT NAME=\"" + name + "\"");
                htmlCode.append(" id=\"" + id + "\"");
                htmlCode.append(" alt=\"" + getAlt() + "\"");
                htmlCode.append(" class=\"" + getStyle() + "\"");
                htmlCode.append(" size=\"" + listBoxSize + "\"");
                htmlCode.append(" >");

                // Not to display select option if listbox is multiselect options 
                if (!isMultiselect()) {
                    htmlCode.append(" <option value=\"\" ");

                    if (getValue().equals("")) {
                        htmlCode.append(" SELECTED ");
                    }

                    htmlCode.append(" >");
                    htmlCode.append(UIUtils.getTextLabel(portletContext,
                            locale, "DEFAULT_DROPDOWN_OPTION"));
                    htmlCode.append(" </option>");
                }

                if (hshMap != null) {
                    Set set = hshMap.keySet();
                    Iterator iterator = set.iterator();

                    while (iterator.hasNext()) {
                        String valueString = (String) iterator.next();
                        String optionLabel = (String) hshMap.get(valueString);
                        htmlCode.append(" <option value=\"" + valueString +
                            "\"");

                        if (value.equals(valueString)) {
                            htmlCode.append(" SELECTED ");
                        }

                        htmlCode.append(" >");
                        htmlCode.append(optionLabel + " </option>");
                    }
                }

                htmlCode.append(" </SELECT>");
            } else {
                if ((hshMap != null) && (hshMap.size() != 0)) {
                    if (value != null) {
                        if (hshMap.containsKey(value)) {
                            htmlCode.append(" " + hshMap.get(value));
                        } else {
                            htmlCode.append("");
                        }
                    } else {
                        htmlCode.append("");
                    }
                } else {
                    htmlCode.append("");
                }
            }

            htmlCode.append("</TD>");
            out.print(htmlCode.toString());
            log.debug(" renderField <- ");
        } catch (IOException e) {
            log.error(UIUtils.getTextMessages(portletContext, locale,
                    "MESSAGE_CODE_57"));
            e.printStackTrace();
            throw e;
        }
    }

    /**
     * @return
     */
    public HashMap getCollection() {
        return collection;
    }

    /**
     * @param string
     */
    public void setCollection(HashMap hashMap) {
        collection = hashMap;
    }

    /**
     * @return
     */
    public String getMultiselect() {
        return multiselect;
    }

    /**
     * @return
     */
    public boolean isMultiselect() {
        if ((multiselect.equalsIgnoreCase("true")) ||
                (multiselect.equalsIgnoreCase("yes"))) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * @param string
     */
    public void setMultiselect(String string) {
        multiselect = string;
    }
}
