
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/**
  * This class returns localized Label to Custom Tag classes.
  * This also adds Business function string to the key.
  */
package com.ibm.ccd.ui.tags;

import java.util.Locale;
import java.util.MissingResourceException;

import javax.servlet.jsp.PageContext;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletLog;
import org.apache.jetspeed.portlet.PortletRequest;

import com.ibm.ccd.ui.util.IReference;


public class UILabel implements ITagConstants, IReference {
    /**
     * attribute stores the actual localized label value
     */
    private String value;

    /**
     * attribute stores the PortletContext of calling of class.This
     * will be initialized only when call is from emitter
     */
    private PortletContext portletContext = null;

    /**
     * attribute stores the pageContext of calling jsp.This will be
     * initialized only if this tag class is invoked from customized tag.
     */
    private PageContext pageContext = null;

    /**
     * attributes holds the business function stored retrieved from session.
     * That is used for creating the key for fetching of resource bundle data.
     *
     * businessfunction+actionString+key or
     * businessfunction+key or
     * key
     */
    private String businessfunction = "";

    /**
     * attributes holds the action stored retrieved from session.
     * That is used for creating the key for adding the action while fetching from resource bundle .
     */
    private String actionString = "";

    /**
     * attribute is for logging
     */
    private PortletLog log = null;

    /**
     * attribute stores Locale
     */
    private Locale locale = Locale.US; // Default locale

    /**
     * UILabel : Constructor
     */
    public UILabel(PageContext pageContextParam) {
        pageContext = pageContextParam;

        PortletConfig portletconfig = (PortletConfig) pageContext.getAttribute(VAR_PORTLET_CONFIG);
        portletContext = portletconfig.getContext();

        log = portletContext.getLog();
        businessfunction = (String) ((PortletRequest) pageContext.getAttribute(VAR_PORTLET_REQUEST)).getSession()
                                     .getAttribute(SESSION_BUS_FUNCT);
        actionString = (String) ((PortletRequest) pageContext.getAttribute(VAR_PORTLET_REQUEST)).getSession()
                                 .getAttribute(SESSION_ACTION);

        locale = pageContext.getRequest().getLocale();
    }

    /**
     * UILabel : Constructor
     * @param portletContextParam
     */
    public UILabel(PortletContext portletContextParam) {
        portletContext = portletContextParam;
        log = portletContext.getLog();
    }

    /**
     * This is main method used for getting the localized label
     * of Html element.
     * @return String
     */
    public String getValue(String name) throws Exception {
        log.debug("  UILabel ->getValue ");

        if (pageContext == null) {
            // locale specific code.Business function code should be added in key.   
            value = portletContext.getText(BUNDLE_WPC_UI_LABELS, name, locale);
        } else {
            try {
                value = portletContext.getText(BUNDLE_WPC_UI_LABELS,
                        businessfunction + "." + actionString + "." + name,
                        locale);
            } catch (MissingResourceException m) {
                /**
                 * If the key created by prefixing the business function with name(businessfunction+name)
                 * is not found then only name as key is searched in resource bundle.
                 */
                try {
                    value = portletContext.getText(BUNDLE_WPC_UI_LABELS,
                            businessfunction + "." + name,
                            pageContext.getRequest().getLocale());
                } catch (MissingResourceException m1) {
                    try {
                        value = portletContext.getText(BUNDLE_WPC_UI_LABELS,
                                name, pageContext.getRequest().getLocale());
                    } catch (MissingResourceException m2) {
                        value = name.substring((name.lastIndexOf('/') == -1)
                                ? 0 : (name.lastIndexOf('/') + 1));
                    }
                }
            }
        }

        log.debug(" UILabel <- getValue ");

        return value;
    }

    /**
     * @return
     */
    public Locale getLocale() {
        return locale;
    }

    /**
     * @param locale
     */
    public void setLocale(Locale locale) {
        this.locale = locale;
    }
}
