
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/**
 *  This class is the tag implementation for the Message Tag.
 *
 */
package com.ibm.ccd.ui.tags;

import java.io.IOException;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.Set;
import java.util.Vector;

import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.TagSupport;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletLog;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;

import com.ibm.ccd.ui.context.IWPCUser;
import com.ibm.ccd.ui.exception.Message;
import com.ibm.ccd.ui.exception.MessageList;
import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.exception.MessageTypes;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;
import com.ibm.ccd.ui.validator.IValidatorMessage;


public class UIMessageTag extends TagSupport implements ITagConstants,
    IReference {
    /**
     * this JspWriter to place the output
     */
    protected JspWriter out;

    /**
     * attribute used for log
     */
    PortletLog log = null;

    /**
     * Locales are passed from Request object.The default is locale is US.
     */
    protected Locale locale = Locale.US;

    /**
     * PortletResponse required for encoding the image url image url
     */
    private PortletResponse portletResponse = null;
    
    /**
     * PortletResponse required for encoding the image url image url
     */
    private PortletRequest portletRequest = null;

    /**
     * Initializing the variables
     */
    PortletContext portletContext = null;

    /**
     * initialization method
     */
    protected void initialize() {
        portletContext = ((PortletConfig) pageContext.getAttribute(VAR_PORTLET_CONFIG)).getContext();
        log = portletContext.getLog();

        /* Initialising the locale with locale request */
        locale = pageContext.getRequest().getLocale();
    }

    /**
    * doStartTag is called by the JSP container when the tag is encountered
    */
    public int doStartTag() {
        try {
            out = pageContext.getOut();
            portletResponse = (PortletResponse) pageContext.getAttribute(VAR_PORTLET_RESPONSE);
            portletRequest = (PortletRequest) pageContext.getAttribute(VAR_PORTLET_REQUEST);

            //	log is initialized ..
            initialize();

            //out.println("<DIV CLASS=\"wpsLabelText\" color=\"#CC0033\">");
            out.println("<TABLE>");
            processMessage();
            out.println("</TABLE>");
        } catch (Exception ex) {
            log.error(UIUtils.getTextMessages(portletContext, locale,
                    "MESSAGE_CODE_53") + " ");
            ex.printStackTrace();

            //	throw new Error();
        }

        log.debug("doStartTag end <- ");

        return SKIP_BODY;
    }

    /**
      * This method renders HTML process Error message and send it response object
      */
    protected void processMessage() throws Exception {
        log.debug(" processMessage ->");

        try {
            PortletRequest portletRequest = (PortletRequest) pageContext.getAttribute(VAR_PORTLET_REQUEST);

            MessageMap messageMap = (MessageMap) portletRequest.getAttribute(MESSAGE_KEY);

            if (messageMap != null) {
                if ((messageMap.getFatalList() != null) &&
                        (messageMap.getFatalList().size() != 0)) {
                    extractText(messageMap.getFatalList(), MessageTypes.FATAL);
                } else if ((messageMap.getErrorErrorList() != null) &&
                        (messageMap.getErrorErrorList().size() != 0)) {
                    extractText(messageMap.getErrorErrorList(),
                        MessageTypes.ERROR);
                } else if ((messageMap.getWarningMessageList() != null) &&
                        (messageMap.getWarningMessageList().size() != 0)) {
                    extractText(messageMap.getWarningMessageList(),
                        MessageTypes.WARNING);
                } else if ((messageMap.getInfoMessageList() != null) &&
                        (messageMap.getInfoMessageList().size() != 0)) {
                    extractText(messageMap.getInfoMessageList(),
                        MessageTypes.INFO);
                } else if ((messageMap.getDebugMessageList() != null) &&
                        (messageMap.getDebugMessageList().size() != 0)) {
                    extractText(messageMap.getDebugMessageList(),
                        MessageTypes.DEBUG);
                }
            }
        } catch (Exception e) {
            log.warn(UIUtils.getTextMessages(portletContext, locale,
                    "MESSAGE_CODE_60") + " ");
            e.printStackTrace();
            throw e;
        }

        log.debug(" processMessage <- ");
    }

    /**
     * extractText Method extracts the Error messages from key, formats and returns
     * text
     * @param msgList
     * @throws IOException
     */
    private void extractText(MessageList msgList, String type)
        throws Exception {
        log.debug(" extractText ->");

        StringBuffer displayText = null;
        StringBuffer attributeList = new StringBuffer();
        StringBuffer errorMessageList = new StringBuffer();
        int displayCount = 0;
        
        try {
            HashMap msghshMap = removeRepeatedMsgs(msgList);
            displayText = new StringBuffer();

            Set set = msghshMap.keySet();
            Iterator iterator = set.iterator();

            while (iterator.hasNext()) {
                String messagekey = (String) iterator.next();
                Vector vecReplacements = (Vector) msghshMap.get(messagekey);

                for (int i = 0; i < vecReplacements.size(); i++,displayCount++) {
                    String singleErrmsg = "";
                    Object[] valueArrays = (Object[]) vecReplacements.get(i);

                    displayText.append("<TR>");

                    try {
                        String propText = portletContext.getText(BUNDLE_WPC_UI_MESSAGES,
                                messagekey, locale);
                        String errorMsg = replacePlaceHolder(propText,valueArrays);
                        if(!errorMsg.equals(propText)){
                        	attributeList.append((String)valueArrays[0] + "~");
                        	errorMessageList.append(errorMsg + "~");
                        }
                        singleErrmsg = getImageString(type);
                        singleErrmsg = singleErrmsg +
                            "<TD VALIGN=\"CENTER\"  class=\"wpsInfoRow\">";
                        singleErrmsg = singleErrmsg +
                            replacePlaceHolder(propText, valueArrays);
                        singleErrmsg = singleErrmsg + "</TD>";
                    } catch (MissingResourceException e1) {
                        log.warn(UIUtils.getTextMessages(portletContext,
                                locale, "MESSAGE_CODE_54") + " " + messagekey);
                    }
                    if(displayCount < 2){
                        displayText = displayText.append(singleErrmsg);
                    }
                    displayText.append("</TR>");
                }
            }
            if(type.equals(MessageTypes.ERROR)){
            String key = (String) portletRequest.getUser().getAttribute(IWPCUser.KEY_USER_ID);
            portletContext.setAttribute(IValidatorMessage.ATTRIBUTE_LIST + key,attributeList.toString());
            portletContext.setAttribute(IValidatorMessage.ERROR_MESSAGE_LIST + key,errorMessageList.toString());

            String tooltip = portletContext.getText(BUNDLE_WPC_UI_MESSAGES,IValidatorMessage.MSG_TOOLTIP,locale);
            displayText.append("<TR>");
            // Show a max of 2
            if(displayCount > 2){
                displayText.append("<TD>");
                displayText.append("<IMG SRC=\"");
                displayText.append(portletResponse.encodeURL(ERROR_IMAGE_PATH));
                displayText.append("\" WIDTH=\"14\" HEIGHT=\"14\" BORDER=0 >");
                displayText.append("</TD>");
                displayText.append("<TD VALIGN=\"CENTER\"  class=\"wpsInfoRow\">");
                displayText.append("<A HREF=\"#\" title=\"");
                displayText.append(tooltip);
                displayText.append("\" onClick=\"displayErrors()\">");
                displayText.append(portletContext.getText(BUNDLE_WPC_UI_MESSAGES,IValidatorMessage.MSG_MORE_ERRORS,locale));
                displayText.append("</A>");
                displayText.append("</TD>");
            }
            displayText.append("</TR>");
            }	
        } catch (Exception e) {
            log.warn(UIUtils.getTextMessages(portletContext, locale,"MESSAGE_CODE_60"));
            e.printStackTrace();
            throw e;
        }

        out.println(displayText);
        log.debug(" extractText <- ");
    }

    /**
     * replacePlaceHolder method for replacing the place Holders.
     * @param strParam
     * @param values
     * @return
     */
    private String replacePlaceHolder(String strParam, Object[] values)
        throws Exception {
    	if(values == null){
        	return strParam;
        }
    	MessageFormat msgFormat = new MessageFormat(strParam);
        return msgFormat.format(values);
    }

    /**
     * This method return Image path for all types messages
     * @param type
     * @return
     */
    private String getImageString(String type) {
        log.debug(" getImageString ->");

        StringBuffer imageName = new StringBuffer();
        imageName.append("<TD VALIGN=\"TOP\"><IMG SRC=\"");

        if (type.equals(MessageTypes.FATAL)) {
            imageName.append(portletResponse.encodeURL(FATAL_IMAGE_PATH));
        } else if (type.equals(MessageTypes.ERROR)) {
            imageName.append(portletResponse.encodeURL(ERROR_IMAGE_PATH));
        } else if (type.equals(MessageTypes.WARNING)) {
            imageName.append(portletResponse.encodeURL(WARN_IMAGE_PATH));
        } else if (type.equals(MessageTypes.INFO)) {
            imageName.append(portletResponse.encodeURL(INFO_IMAGE_PATH));
        } else if (type.equals(MessageTypes.DEBUG)) {
            imageName.append(portletResponse.encodeURL(DEBUG_IMAGE_PATH));
        }

        imageName.append("\" WIDTH=\"14\" HEIGHT=\"14\" BORDER=0></TD>");
        log.debug(" getImageString <- ");

        return imageName.toString();
    }

    /**
     * removeRepeatedMsgs:Method repeated messages displayed for different controls.
     * This methods find out such messages in list and merge it to one.
     * @param msgList
     * @return
     */
    private HashMap removeRepeatedMsgs(MessageList msgList) {
        log.debug(" removeRepeatedMsgs ->");

        HashMap hshMap = new HashMap();
        Iterator iterator = msgList.get();
        String singleErrmsg = "";

        while (iterator.hasNext()) {
            Message msgInner = (Message) iterator.next();
            String messagekey = msgInner.getKey();

            if (!(hshMap.containsKey(messagekey))) {
                // New message
                Vector vecReplacements = new Vector();
                vecReplacements.add(msgInner.getValues());
                hshMap.put(messagekey, vecReplacements);
            } else {
                // Repeating msg
                Object[] valueArraysInner = msgInner.getValues();

                if (valueArraysInner != null) {
                    // check Multiple replacements..
                    if (valueArraysInner.length > 1) {
                        Vector vecReplacements = (Vector) hshMap.get(messagekey);
                        vecReplacements.add(valueArraysInner);
                        hshMap.put(messagekey, vecReplacements);
                    } else {
                        // for single replacements..
                        Vector vecReplacements = (Vector) hshMap.get(messagekey);
                        Object[] values = (Object[]) vecReplacements.get(0);
                        values[0] = values[0] + "," + valueArraysInner[0];
                        vecReplacements.setElementAt(values, 0);
                        hshMap.put(messagekey, vecReplacements);
                    }
                } // end if null check valueArray (when no place holders and repeatation)
            } // else of if when key already available in hashmap  
        } // End of outer while						

        // manipulate the string to append all same error associated to same field 
        log.debug(" removeRepeatedMsgs <- ");

        return hshMap;
    }
}
