
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/**
 * This class is the tag implementation for the Number field input.It will validate the numeric check js.
 * Tag Attributes:
 * currency :  Set it as true if number is currency.If this is true than field will appear currency.
 * numberMax
 * numberMin
 * Note:Text Tag attributes are inherited.
 */
package com.ibm.ccd.ui.tags;

import java.io.IOException;

import com.ibm.ccd.ui.util.UIUtils;


public class UINumberTag extends UITextTag {
    /**
     * attribute for decimalPrecision of Number field
     */
    private String decimalPrecision = "2";

    /**
     * attribute for making the number field as Currency field
     */
    private String currency = "false";

    /**
     * attribute for range of number min
     */
    private String numberMin = "";

    /**
     * attribute for range of Number max
     */
    private String numberMax = "";

    /**
     * attribute for default Currency
     */
    private String currencyLabel = DEFAULT_CURRENCY;

    /**
     * Constructor
     */
    public UINumberTag() {
        super();
    }

    /*
     * This method renders HTML text field to response object
     */
    protected void renderField() throws Exception {
        log.debug(" renderField ->");

        try {
            StringBuffer htmlCode = new StringBuffer("");

            //StringBuffer htmlField = new StringBuffer("");
            htmlCode.append("<TD ALIGN =\"LEFT\">");

            if (isCurrency()) {
                htmlCode.append(currencyLabel);
            }

            if (isEnabled()) {
                htmlCode.append("<input type=\"text\"");
                htmlCode.append(" name=\"" + name + "\"");

                if ((value != null) && !("null".equalsIgnoreCase(value))) {
                    htmlCode.append(" value=\"" + value + "\"");
                }

                htmlCode.append(" id=\"" + id + "\"");
                htmlCode.append(" alt=\"" + getAlt() + "\"");

                try {
                    htmlCode.append(" class=\"" + getStyle() + "\"");
                    htmlCode.append(" size=\"" + getSize() + "\"");
                    htmlCode.append(" maxlength=\"" + getMaxValue() + "\"");
                } catch (Exception eh) {
                    log.warn(UIUtils.getTextMessages(portletContext, locale,
                            "MESSAGE_CODE_55") + " ");
                    log.warn(" " + eh);
                }

                // Adding javascript in tag
                /* Currently javascript is disabled */
                //				if (value == null)
                //				{
                //					value = ""; 
                //				}
                //				htmlCode.append(addJavaScript()+">");
                htmlCode.append(">");
            } else {
                if ((value != null) && !("null".equalsIgnoreCase(value))) {
                    htmlCode.append(" " + value);
                }
            }

            htmlCode.append("</TD>");
            out.print(htmlCode.toString());
            log.debug(" renderField <- ");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * This method adds javascript to html text field.It needs to include
     * wpscarmel.js.
     * @return
     */
    private String addJavaScript() {
        log.debug(" addJavaScript ->");

        int mkArrayCount = 0; // no validations added yet
        StringBuffer sbuffArray = new StringBuffer();
        StringBuffer htmlErrorCode = new StringBuffer();
        StringBuffer htmlErrorMsg = new StringBuffer();
        StringBuffer htmlParams = new StringBuffer();
        StringBuffer retStrbuffer = new StringBuffer();

        sbuffArray.append("  new Array(");

        try {
            // Adding javascript validations
            mkArrayCount = mandatoryCheck(mkArrayCount, htmlErrorCode,
                    htmlErrorMsg, htmlParams);
            mkArrayCount = numberCheck(mkArrayCount, htmlErrorCode,
                    htmlErrorMsg, htmlParams);
            mkArrayCount = rangeCheck(mkArrayCount, htmlErrorCode,
                    htmlErrorMsg, htmlParams);

            if (isCurrency()) {
                // Default value
                if (getValue().equals("")) {
                    setValue("0.0");
                }
            }

            // Constructing js validation method string		
            if (mkArrayCount > 1) {
                htmlErrorCode = new StringBuffer().append(sbuffArray)
                                                  .append(htmlErrorCode).append(")");
                htmlErrorMsg = new StringBuffer().append(sbuffArray)
                                                 .append(htmlErrorMsg).append(")");
                htmlParams = new StringBuffer().append(sbuffArray)
                                               .append(htmlParams).append(")");
            }

            if (!(htmlErrorCode.equals(""))) {
                retStrbuffer.append(" onBlur=\"javascript:validate(" +
                    htmlErrorCode + ",this," + htmlErrorMsg);
                retStrbuffer.append("," + htmlParams + ")\" "); // End of validate method			
            }
        } catch (Exception e1) {
            log.warn(UIUtils.getTextMessages(portletContext, locale,
                    "MESSAGE_CODE_56") + " ");
            e1.printStackTrace();
        }

        log.debug(" addJavaScript <- ");

        return retStrbuffer.toString();
    }

    /**
     * @param mkArrayCount
     * @param htmlErrorCode
     * @param htmlErrorMsg
     * @param htmlParams
     * @return
     */
    protected int rangeCheck(int mkArrayCount, StringBuffer htmlErrorCode,
        StringBuffer htmlErrorMsg, StringBuffer htmlParams) {
        log.debug(" rangeCheck ->");

        if ((getNumberMin() != null) && (!(getNumberMin().equals("")))) {
            commanSeparation(mkArrayCount, htmlErrorCode, htmlErrorMsg,
                htmlParams);
            htmlErrorCode.append(JS_MINRANGECHK);
            htmlErrorMsg.append("'" +
                portletContext.getText(BUNDLE_WPC_UI_JSMESSAGES,
                    "JS_ERROR_TAG_MINIMUM_NUMBER", locale) + "'");
            htmlParams.append("'" + getNumberMin() + "'");
            mkArrayCount++;
        }

        if ((getNumberMax() != null) && (!(getNumberMax().equals("")))) {
            commanSeparation(mkArrayCount, htmlErrorCode, htmlErrorMsg,
                htmlParams);
            htmlErrorCode.append(JS_MAXRANGECHK);
            htmlErrorMsg.append("'" +
                portletContext.getText(BUNDLE_WPC_UI_JSMESSAGES,
                    "JS_ERROR_TAG_MAXIMUM_NUMBER", locale) + "'");
            htmlParams.append("'" + getNumberMax() + "'");
            mkArrayCount++;
        }

        log.debug(" rangeCheck <- ");

        return mkArrayCount;
    }

    /**
     * @return
     */
    public boolean isCurrency() {
        if (getCurrency().equalsIgnoreCase("true")) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * @return
     */
    public String getCurrency() {
        return currency;
    }

    /**
     * @param string
     */
    public void setCurrency(String string) {
        currency = string;
    }

    /**
     * @return
     */
    public String getNumberMax() {
        return numberMax;
    }

    /**
     * @return
     */
    public String getNumberMin() {
        return numberMin;
    }

    /**
     * @param string
     */
    public void setNumberMax(String string) {
        numberMax = string;
    }

    /**
     * @param string
     */
    public void setNumberMin(String string) {
        numberMin = string;
    }

    /**
     * @return
     */
    public String getDecimalPrecision() {
        return decimalPrecision;
    }

    /**
     * @param string
     */
    public void setDecimalPrecision(String string) {
        decimalPrecision = string;
    }

    /**
     * @param mkArrayCount
     * @param htmlErrorCode
     * @param htmlErrorMsg
     * @param htmlParam
     * @return
     */
    protected int numberCheck(int mkArrayCount, StringBuffer htmlErrorCode,
        StringBuffer htmlErrorMsg, StringBuffer htmlParam) {
        log.debug(" numberCheck ->");

        commanSeparation(mkArrayCount, htmlErrorCode, htmlErrorMsg, htmlParam);
        htmlErrorCode.append("3"); // 
        htmlErrorMsg.append("'" +
            portletContext.getText(BUNDLE_WPC_UI_JSMESSAGES,
                "JS_ERROR_TAG_NUMBER", locale) + "'");
        htmlParam.append("'0'"); //
        mkArrayCount++;
        log.debug(" numberCheck <- ");

        return mkArrayCount;
    }
}
