
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.tags;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.TagSupport;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletSession;

import com.ibm.ccd.ui.dataobjects.NVPair;
import com.ibm.ccd.ui.dataobjects.attributes.AttributeTypes;
import com.ibm.ccd.ui.dataobjects.attributes.LookupTableAttribute;
import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.portlets.search.core.IBaseSearch;
import com.ibm.ccd.ui.portlets.search.core.SearchCriteriaBean;
import com.ibm.ccd.ui.portlets.search.core.SearchUtil;
import com.ibm.ccd.ui.taglib.dataobjects.BaseSearchElement;
import com.ibm.ccd.ui.taglib.dataobjects.ISearchElementTypesEnum;
import com.ibm.ccd.ui.taglib.dataobjects.SelectSearchElement;
import com.ibm.ccd.ui.taglib.dataobjects.TextSearchElement;
import com.ibm.ccd.ui.util.UIUtils;


/**
 * @purpose To Generate the Search Criteria HTML.
 * @desc    Thi class generates the HTMl for the SearcH Criteria Form.
 * It renders the actual HTML compnent depending on the type of SearchCriteriaElement.
 * @since   Iteration 2
 */
public class UISearchCriteriaTag extends TagSupport {
    /** to write the HTML output.
     */
    protected JspWriter out;

    /** the scope object.
     */
	protected PortletRequest request;

    /** the search atributes.
    */
	protected ArrayList searchAttributes;
	
	protected BaseSearchElement[] orderedElements = null;

    /**
     * contains <code>PortletContext</code> object used for getting locale specific labels
     */
    PortletContext portletContext = null;

    /**
     * @purpose This method is the first method that gets called for Custom Tags.
     * @desc The doStartTag() method initializes output stream.and calls the generate SearchTable.
     *                  This method initializes the  SearchCriteriaElements to be used by the generateSearchTable().
     * @throws javax.servlet.jsp.JspException
     * @return int
     * @since Iteration 2
     */
    public int doStartTag() throws JspException {
        try {
            PortletSession portletSession = null;
            out = pageContext.getOut();
            request = (PortletRequest) pageContext.getAttribute(ITagConstants.VAR_PORTLET_REQUEST);
            portletSession = request.getPortletSession();
            portletContext = ((PortletConfig) pageContext.getAttribute(ITagConstants.VAR_PORTLET_CONFIG)).getContext();

            SearchCriteriaBean bean = (SearchCriteriaBean) portletSession.getAttribute(IBaseSearch.KEY_SEARCH_CRITERIA_BEAN);

            if (bean == null) {
                //put message
                return SKIP_BODY;
            }

            searchAttributes = bean.getAttributeList();
            orderSearchElements();
            generateSearchTable();
        } catch (Exception ex) {
            throw new JspTagException(ex.getMessage());
        }

        return SKIP_BODY;
    }

    /**
     * Ordrs the list according to user selection
     */
    private void orderSearchElements() {
        BaseSearchElement element = null;
        int count = 0;
        int index = -1;
        int listBoxindex = -1;

        for (int i = 0; i < searchAttributes.size(); i++) {
            element = (BaseSearchElement) searchAttributes.get(i);

            if (element.getIsDisplayable()) {
                count++;
            }
        }

        orderedElements = new BaseSearchElement[count];

        for (int i = 0; i < searchAttributes.size(); i++) {
            element = (BaseSearchElement) searchAttributes.get(i);

            if (element.getIsDisplayable()) {
                listBoxindex = element.getListBoxIndex();

                if (listBoxindex >= 0) {
                    orderedElements[listBoxindex] = element;
                } else {
                    orderedElements[++index] = element;
                }
            }
        }
    }

    /**
     * @purpose This method generates the Search Form Table.
     * @desc
     *                  The generateSearchTable() checks if the Search Criteria Element
     *                  is a simple Text Type, or a Dropdown or a Lookup.Depending upon the
     *                  Search Criteria Element it generates the appropriate html.
     * @throws javax.servlet.jsp.JspException

     * @since Iteration 2
     */
    public void generateSearchTable() throws IOException {
        boolean flag = true;
        BaseSearchElement childElement = null;
        boolean hasChild = false;
        String labelAnd = null;
        MessageMap messageMap = null;

        int size = orderedElements.length;
        out.println(
            "<TABLE WIDTH=\"100%\" CELLPADDING=\"4\" CELLSPACING=\"0\" BORDER=\"0\">");

        for (int i = 0; i < size; i++) {
            hasChild = false;

            BaseSearchElement baseSearchElement = orderedElements[i];

            if (!baseSearchElement.getIsDisplayable()) {
                continue;
            }

            if (flag) {
                flag = false;
                out.println("<TR ALIGN=\"LEFT\" CLASS=\"wpcGridOddRow\" > ");
            } else {
                out.println("<TR ALIGN=\"LEFT\" CLASS=\"wpcGridEvenRow\" > ");
                flag = true;
            }

            out.print("<TD ALIGN=\"RIGHT\" CLASS=\"wpsLabelText\">");

            if (baseSearchElement.isMandatory()) {
                out.print("<SPAN CLASS=\"wpcFormAsterisk\"> * </SPAN>");
            }

            if (baseSearchElement.getChildElement() != null) {
                hasChild = true;
            }

            displaySearchElement(baseSearchElement, hasChild);

            //get locale specific value of "And"
            labelAnd = UIUtils.getTextLabel(portletContext, request,
                    IBaseSearch.LABEL_VIEW_AND);

            if (baseSearchElement.getChildElement() != null) {
                out.println(" <TD ALIGN=\"CENTER\"CLASS=\"wpsLabelText\">" +
                    labelAnd +
                    "</TD><TD ALIGN=\"RIGHT\" CLASS=\"wpsLabelText\">");
                childElement = baseSearchElement.getChildElement();
                displaySearchElement(childElement, hasChild);
            }

            out.println("</TR>");
        } // end of for

        if (orderedElements.length > 0) {
            displaySearchButton();
        } else {
            messageMap = SearchUtil.getMessageMap(request);
            messageMap.addWarningMessage(IBaseSearch.MSG_SEARCH_ATTRIBUTES_NOT_SELECTED);
        }

        out.println("</TABLE>");
    }

    /**
     *
     */
    private void displaySearchButton() throws IOException {
        String label = null;
        label = UIUtils.getTextLabel(portletContext, request,
                "SEARCH.VIEW.PERFORM_SEARCH");
        out.println(" <TR><TD>&nbsp;</TD><TD colspan=\"2\" align=\"left\">");

        out.println(
            " <INPUT  type=\"submit\" class=\"wpsButtonText\"  value=\"" +
            label + "\" />");
        out.println("</TD></TR>");
    }

    private void displaySearchElement(BaseSearchElement searchElement,
        boolean isChild) throws IOException {
        String selectedOperator = null;
        String displayName = null;
        String html_id=null;
        
		html_id=getHtmlId(searchElement);
        displayName = searchElement.getDisplayName();
        out.print(displayName);
        out.println(" :</TD>");

        HashMap operatorsMap = searchElement.getOperatorsMap();

        if ((operatorsMap != null) && (operatorsMap.size() > 0)) {
            Set operatorMapSet = operatorsMap.keySet();
            Iterator keyIterator = operatorMapSet.iterator();

            String operatorName = null;
            String operatorValue = null;
            selectedOperator = searchElement.getDefaultOperator();

            if (isChild) {
                out.print("<TD ALIGN=\"LEFT\"><SELECT NAME=\"");
            } else {
                out.print("<TD ALIGN=\"LEFT\" COLSPAN=\"4\" ><SELECT NAME=\"");
            }

            out.print(searchElement.getOperatorID());
            out.println("\" CLASS=\"wpcGridSelectMenu\">");

            while (keyIterator.hasNext()) {
                operatorName = (String) keyIterator.next();
                operatorValue = (String) operatorsMap.get(operatorName);
                out.print("<OPTION   VALUE=\"");
                out.print(operatorValue);
                out.print("\"");

                if ((selectedOperator != null) &&
                        operatorValue.equals(selectedOperator.trim())) {
                    out.print(" selected");
                }

                out.print(" >");
                out.print(operatorName);
                out.print("</OPTION>");
            }

            out.println("</SELECT>");

            //If data type requires range 
        } // end of if 

        int attribType = searchElement.getType();

        switch (attribType) {
        case ISearchElementTypesEnum.TEXT:

            TextSearchElement textAttribute = (TextSearchElement) searchElement;
			out.print(" <INPUT ID=\"" + html_id + "\" ");
            out.print(" NAME=\"");

            out.print(textAttribute.getName());
            out.print("\" VALUE=\"");
            String textValue = null;
            textValue = textAttribute.getSelectedValue();

            if (textValue == null) {
                if (textAttribute.isDefaultable()) {
                    textValue = textAttribute.getDefaultValue();
                }
            }

            if (textValue == null) {
                textValue = "";
            }

            out.print(textValue);
            out.print("\" SIZE=\"30\" CLASS=\"wpsEditField\">");
            
            if(textAttribute.getWpcAttributeType()==AttributeTypes.LOOKUP_TABLE){
				renderLookupTableSearchButton(textAttribute);	
            }
			out.print("</TD>");

            break;

        case ISearchElementTypesEnum.SELECT:

            SelectSearchElement selectAttribute = (SelectSearchElement) searchElement;
            NVPair [] possibleValues = selectAttribute.getPossibleValues();
			out.print("<SELECT ID=\"" + html_id + "\" ");
            out.print(" NAME=\"" + selectAttribute.getName() + "\" >");
            String selectedValue = selectAttribute.getSelectedValue();

			for(int i=0;i<possibleValues.length;i++){
                String value = UIUtils.escapeForHTML(possibleValues[i].getName().toString());
                String displayValue=UIUtils.escapeForHTML(possibleValues[i].getValue().toString());
                out.print(" <OPTION");
                out.print(" VALUE=\"" + value + "\" ");

                if (value.equals(selectedValue)) {
                    out.print("SELECTED");
                }

                out.print(">");
                out.print(displayValue);
                out.print("</OPTION>");
            }

            out.print("</SELECT>");
			if(selectAttribute.getWpcAttributeType()==AttributeTypes.LOOKUP_TABLE){
				renderLookupTableSearchButton(selectAttribute);	
			}
            out.print("</TD>");

            break;
        } // end switch
    }
    
	protected void renderLookupTableSearchButton(BaseSearchElement searchElement) throws IOException{
		LookupTableAttribute lookupTableAttribute=null;
		String lookupTableName=null;
		String attributeName=null;
		String html_id=getHtmlId(searchElement);
		PortletResponse portletResponse=(PortletResponse) pageContext.getAttribute(ITagConstants.VAR_PORTLET_RESPONSE);
		//get lookuptable name and attribute names. used for invoking LookupTableSearch Portlet
		lookupTableAttribute=(LookupTableAttribute)searchElement.getWpcAttribute();
		lookupTableName=lookupTableAttribute.getLookupTableName();
		attributeName=searchElement.getName();
		
		String ImagePath=portletResponse.encodeURL("/images/Search.gif");	
		out.print("<a href=\" # \" onClick=\"openLookupTable(\'" + lookupTableName + "\',\'" + html_id+  "\')\"><img SRC=\"" +
		ImagePath + " \" border='0'></a>");
    }
    
    private String getHtmlId(BaseSearchElement searchElement){
    	return "_" + searchElement.hashCode();
    }
}
