
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.tags;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.TagSupport;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletSession;

import com.ibm.ccd.ui.dataobjects.NVPair;
import com.ibm.ccd.ui.portlets.search.core.IBaseSearch;
import com.ibm.ccd.ui.portlets.search.core.ICollectionViewBean;
import com.ibm.ccd.ui.portlets.search.core.SearchCriteriaBean;
import com.ibm.ccd.ui.portlets.search.core.SearchResultBean;
import com.ibm.ccd.ui.taglib.dataobjects.BaseSearchElement;
import com.ibm.ccd.ui.taglib.dataobjects.ISearchElementTypesEnum;
import com.ibm.ccd.ui.taglib.dataobjects.ResultData;
import com.ibm.ccd.ui.taglib.dataobjects.ResultMetaData;
import com.ibm.ccd.ui.taglib.dataobjects.ResultRowData;
import com.ibm.ccd.ui.taglib.dataobjects.SelectSearchElement;
import com.ibm.ccd.ui.util.UIUtils;
import com.ibm.ccd.ui.util.ValidationUtil;


/**
* @purpose This class is a Tag used to display contents of a Collection.
* @desc It iterates through a Collection and displays it's content in Tabular Format.
* If a field is sortable, then the particular column in the table is sortable.
* If it is a link, then the field is rendered as a HTML Link.
* If the collection has to be lazily loaded, then the collection is traversed
* using some links for traversal.
*/
public class UISearchResultTag extends TagSupport {
    public static final String SEARCH_UTIL_PORTLET = "SearchUtilPortlet";

    /**
     * Identifier JspWriter
     */
    protected JspWriter out;

    /**
     * Identifier HttpServelet Request
     */
    private HttpServletRequest request;

    /**
     * Identifier ArrayList
     */
    protected ArrayList metaDataList;

    /**
     * Ordered <code>ResultMetaData</code> array
     */
    ResultMetaData[] resultMetaDataArray = null;

    /**
     * list of result data elements
     */
    protected ArrayList resultDataList;

    /**
     * Identifier ResultRowData[]
     */
    protected ResultRowData[] rowData;

    /** the View Bean
     */
    protected SearchResultBean resultsBean;

    /**
     * list of result data elements
     */
    protected ResultData resultData;

    /**
     * max displayable cols
     */
    protected int maxCols = 0;

    /**
     * Initializing the PortletContext
     */
    protected PortletContext portletContext = null;

    /**
     * Initializing the PortletContext
     */
    protected PortletRequest portletRequest = null;

    /**
     * Contains the list of BaseSearchElement
     */
    protected ArrayList searchElements = null;

    /**
     *  Attribute name of Primary key
     */
    protected String primaryKey = null;

    /**
     *@desc This method is the first method to be called.
     * It is used to initialize the objects.
     * @throws javax.servlet.jsp.JspException
     * @since Iteration 2
     *@return an integer which signifies that tags body should not be processed.
     */
    public int doStartTag() throws JspException {
        PortletSession session = null;
        SearchCriteriaBean bean = null;
        PortletConfig portletConfig = null;

        try {
            portletConfig = (PortletConfig) pageContext.getAttribute(ITagConstants.VAR_PORTLET_CONFIG);
            portletContext = portletConfig.getContext();
            portletRequest = (PortletRequest) pageContext.getAttribute(ITagConstants.VAR_PORTLET_REQUEST);
            session = portletRequest.getPortletSession();
            bean = (SearchCriteriaBean) session.getAttribute(IBaseSearch.KEY_SEARCH_CRITERIA_BEAN);
            searchElements = bean.getAttributeList();
            metaDataList = bean.getMetaDataList();

            //order the result columns
            orderResultMetaData();
            out = pageContext.getOut();
            request = (HttpServletRequest) pageContext.getRequest();
            resultsBean = (SearchResultBean) request.getSession().getAttribute(IBaseSearch.KEY_SEARCH_RESULTS_BEAN);

            if (resultsBean == null) {
                //put message
                return SKIP_BODY;
            }

            resultData = resultsBean.getResultData();

            if (resultData == null) {
                return SKIP_BODY;
            }

            maxCols = Integer.parseInt(resultsBean.getMaxCols());
            resultDataList = resultData.getResultRowData();

            if (metaDataList != null) {
                constructResultTable();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new JspTagException(ex.getMessage());
        }

        return SKIP_BODY;
    }

    /**
     *@desc This method generates the HTML for the collection Form
     * @throws java.io.IOException
     * @since Iteration 2
     */
    protected void constructResultTable() throws IOException {
        out.print(
            "<TABLE WIDTH=\"100%\" CELLPADDING=\"4\" CELLSPACING=\"0\" BORDER=\"0\">");
        constructHeadings();
        constructRows();
        out.print("</TABLE>");
        constructNavigationLinks();
    }

    /**
     *@desc This method generates the HTML for the collection headings
     *  @throws java.io.IOException
     * @since Iteration 2
     */
    protected void constructHeadings() throws IOException {
        out.print("<TR CLASS=\"wpcGridColumnHeadings\">");

        int size = resultMetaDataArray.length;
        String textValue = "";

        displayTitle();

        for (int i = 0; i < size; ++i) {
            ResultMetaData resultMetaData = (ResultMetaData) resultMetaDataArray[i];
            String style = "CLASS=\"wpcGridColumnHeadingUnsortable\">";
            String linkBegin = "";
            String linkEnd = "";
            String arrow = "";

            if (resultMetaData.isDisplayable()) {
                out.print("<TD CLASS=\"wpcGridColumnHeading\">");

                if (resultMetaData.isSortable()) {
                    out.print("CLASS=\"wpcGridColumnHeading\">");

                    /** if currently sorted. Since this support does not
                     * exist right now, the first column is currrently sorted by default.
                     */
                    if (i == 0) {
                        linkBegin = "<A HREF=\"javaScript:void(0);\"";
                        linkEnd = "</A>";

                        //By default the first column is the only one which has the sorted link
                        style = "CLASS=\"wpcGridColumnHeadingLinkSorted\">";
                        arrow = "&nbsp;<A HREF=\"javaScript:void(0);\"><IMG SRC=\"images/ascending.gif\" WIDTH=\"7\" HEIGHT=\"8\" ALT=\"Click here to reverse the sort order.\" ID=\"SortOrderPointer\"></A>";
                    } else {
                        linkBegin = "<A HREF=\"javaScript:void(0);\"";
                        linkEnd = "</A>";
                        style = "\">";
                    }
                }

                out.print(linkBegin);

                String displayName = resultMetaData.getDisplayName();
                out.print(displayName);
                out.print(linkEnd);
                out.print(arrow);
                out.print("</TD>");
            }
        } // end of for

        out.print("</TR>");
    }

    protected void displayTitle() throws IOException {
        out.print("<TD CLASS=\"wpcGridColumnHeadings\"> ");
        out.print(
            "<INPUT TYPE='checkbox' name ='itemPrimaryKey' value='' onClick=\"javascript:switchAll(this.form,this.name,this.checked)\")/>");
        out.print("</TD>");
    }

    /**
    *@desc This method generates the HTML for the collection rows
    *  @throws java.io.IOException
    * @since Iteration 2
    */
    protected void constructRows() throws IOException {
        boolean flag = true;
        ResultMetaData metaData = null;
        ArrayList resultRows = resultsBean.getResultsRows();

        if (resultRows.isEmpty()) {
            return;
        }

        rowData = new ResultRowData[resultRows.size()];

        Iterator iterator = resultRows.iterator();
        int j = 0;

        while (iterator.hasNext() && (j < resultRows.size())) {
            ResultRowData resultRowData = (ResultRowData) iterator.next();
            rowData[j] = resultRowData;
            j++;
        }

        for (int z = 0; z < rowData.length; ++z) {
            if (flag) {
                out.print("<TR class=\"wpcGridOddRow\">");
                flag = false;
            } else {
                out.print("<TR class=\"wpcGridEvenRow\">");
                flag = true;
            }

            HashMap dataElements = rowData[z].getRowElements();
            displayPrimaryKey(dataElements);

            int size = resultMetaDataArray.length;

            for (int k = 0; k < size; ++k) {
                metaData = resultMetaDataArray[k];
                out.print("<TD CLASS=\"wpcTightRightPaddedNonwrappingCell\">");

                String colName = metaData.getName();
                Object textValueObject = dataElements.get(colName);
                String textValue = "";

                if (textValueObject != null) {
                    textValue = UIUtils.escapeForHTML(textValueObject.toString());
                }

                switch (metaData.getType()) {
                case ICollectionViewBean.LINK: {
                    out.print("<A href=\"javascript:openSelectedObject('" +
                        "EMPTY_ACTION FOR ELEMENTS" + "',");
                    out.print("'" + rowData[z].getRowReference() + "',");
                    out.print("'" + "EMPTY_OPENMODE" + "'");
                    out.print(");\">");

                    String spaceIndicator = (textValue.trim().length() > 0)
                        ? textValue : ("<" + textValue + ">");
                    out.print(spaceIndicator);
                    out.print("</A>");

                    break;
                }

                default:

                    String displayValue = null;
                    displayValue = getDisplayValue(colName, UIUtils.unescapeForHTML(textValue));
                    
                    // Make the primary key a hyper link. 
                    // Clicking on the link should show the Item details in the Work With Item portlet

                    if(!colName.equals(primaryKey))
                    	out.print(displayValue);
                    else {
                        out.print("<A HREF=\"#\" name='pkey' value='"+ displayValue.trim() +"' onClick=\"displayItem('"+ displayValue.trim() +"')\">");
                        out.print(displayValue);
                        out.print("</A>");
                    }
                }

                out.print("</TD>");
            }

            out.print("</TR>");
        }
    }

    protected void displayPrimaryKey(HashMap dataElements)
        throws IOException {
        String textValue = "";
        String inputType = "checkbox";

        Object textValueObject = dataElements.get(primaryKey);

        if (textValueObject != null) {
            textValue = UIUtils.escapeForHTML(textValueObject.toString());
        }

        out.print("<TD CLASS=\"wpcGridCheckboxCell\">");
        out.print("<INPUT TYPE=\"" + inputType +
            "\" name ='itemPrimaryKey' value='" + textValue + "' />");
        out.print("</TD>");
    }

    /**
     * @param colName
     * @param textValue
     * @return
     */
    protected String getDisplayValue(String colName, String textValue) {
        BaseSearchElement element = null;
        SelectSearchElement selectElement = null;
        NVPair[] possibleValues = null;
        String displayValue = null;
        int type = 0;

        if (!ValidationUtil.isNullOrEmpty(textValue)) {
            element = getSearchElement(colName);
            type = element.getType();

            if (type == ISearchElementTypesEnum.TEXT) {
                displayValue = textValue;
            }

            if (type == ISearchElementTypesEnum.SELECT) {
                selectElement = (SelectSearchElement) element;
                possibleValues = selectElement.getPossibleValues();
				for (int i=0;i<possibleValues.length;i++){
				   if(textValue.equals(possibleValues[i].getName())){
					   displayValue=possibleValues[i].getValue().toString();            		
					   break;
				   }
			    }
			    if(ValidationUtil.isNullOrEmpty(displayValue)){
					displayValue=textValue;
			    }
            }
        } else {
            displayValue = "";
        }

        return displayValue;
    }

    /**
     * @param colName
     * @return
     */
    private BaseSearchElement getSearchElement(String colName) {
        BaseSearchElement element = null;
        String elementName = null;

        for (int i = 0; i < searchElements.size(); i++) {
            element = (BaseSearchElement) searchElements.get(i);
            elementName = element.getName();

            if (elementName.equals(colName)) {
                break;
            }
        }

        return element;
    }

    /**
     *@desc This method generates the HTML for the collection headings
     *  @throws java.io.IOException
     * @since Iteration 2
     */
    protected void constructNavigationLinks() throws IOException {
        out.print("<TABLE><TR><TD></TD></TR></TABLE>");
        out.print(
            "<TABLE WIDTH=\"100%\" CELLPADDING=\"4\" CELLSPACING=\"0\" BORDER=\"0\">");
        out.print("<TR>");
        out.print("<TD  CLASS=\"wpcGridSummaryAndNavigationCellDark\">");
        out.print(UIUtils.getTextLabel(portletContext, portletRequest,
                ICollectionViewBean.SHOWING) + "   ");
        out.print(resultsBean.getStartIndex());
        out.print("   " +
            UIUtils.getTextLabel(portletContext, portletRequest,
                ICollectionViewBean.TO) + "   ");
        out.print(resultsBean.getEndIndex());

        out.print("   " +
            UIUtils.getTextLabel(portletContext, portletRequest,
                ICollectionViewBean.OF) + "   ");
        out.print(resultsBean.getTotalResultSize());

        renderFirstLink();
        renderPreviousLink();
        renderNextLink();
        renderLastLink();
        out.print("</TD></TR></TABLE>");
    }

    /**
     * Constructs a link for the navigation linkss
     * @param String action, for which the link needs to be constructed
     * @return String representing the link
     */
    protected String constructLink(String action) throws java.io.IOException {
        StringBuffer link = new StringBuffer("");
        link.append("<A HREF=").append(action);

        return link.toString();
    }

    /**
    * @desc To get the Last Link
    * @throws IOException
    */
    private void renderLastLink() throws java.io.IOException {
        if (resultsBean.hasNextResults()) {
            out.print(constructLink(resultsBean.getLastActionURI()));
            out.print(" CLASS=\"wpcPageLink\"> ");
            out.print(UIUtils.getTextLabel(portletContext, portletRequest,
                    ICollectionViewBean.LAST));
            out.print("</A> ");
        } else {
            out.print("<SPAN CLASS=\"wpcPageLinkDeactivated\"> ");
            out.print(UIUtils.getTextLabel(portletContext, portletRequest,
                    ICollectionViewBean.LAST));
            out.print("</SPAN> ");
        }
    }

    /**
     *  print the first navigation link
     *  @desc To get the Last Link
     * @throws IOException
     */
    private void renderFirstLink() throws java.io.IOException {
        out.print(" | ");

        if (resultsBean.hasPreviousResults()) {
            out.print("<SPAN CLASS=\"PageLinkActivated\">");
            out.print(constructLink(resultsBean.getFirstActionURI()));
            out.print(" CLASS=\"wpcPageLink\"> ");
            out.print(UIUtils.getTextLabel(portletContext, portletRequest,
                    ICollectionViewBean.FIRST));
            out.print("</A> ");
            out.print("</SPAN>");
        } else {
            out.print("<SPAN CLASS=\"wpcPageLinkDeactivated\"> ");
            out.print(UIUtils.getTextLabel(portletContext, portletRequest,
                    ICollectionViewBean.FIRST));
            out.print("</SPAN> ");
        }

        out.print(" | ");
    }

    /**
     * print the previous navigation link
     * @desc To get the Last Link
     * @throws IOException
     */
    private void renderNextLink() throws java.io.IOException {
        if (resultsBean.hasNextResults()) {
            out.print(constructLink(resultsBean.getNextActionURI()));
            out.print(" CLASS=\"wpcPageLink\"> ");
            out.print(UIUtils.getTextLabel(portletContext, portletRequest,
                    ICollectionViewBean.NEXT));
            out.print("</A> ");
        } else {
            out.print("<SPAN CLASS=\"wpcPageLinkDeactivated\"> ");
            out.print(UIUtils.getTextLabel(portletContext, portletRequest,
                    ICollectionViewBean.NEXT));
            out.print("</SPAN> ");
        }

        out.print(" | ");
    }

    /**
     * print the next navigation link
     * @desc To get the Last Link
     * @throws IOException
     */
    private void renderPreviousLink() throws java.io.IOException {
        if (resultsBean.hasPreviousResults()) {
            out.print(constructLink(resultsBean.getPreviousLinkURI()));
            out.print(" CLASS=\"wpcPageLink\"> ");
            out.print(UIUtils.getTextLabel(portletContext, portletRequest,
                    ICollectionViewBean.PREVIOUS));
            out.print("</A> ");
            out.print("</SPAN>");
        } else {
            out.print("<SPAN CLASS=\"wpcPageLinkDeactivated\"> ");
            out.print(UIUtils.getTextLabel(portletContext, portletRequest,
                    ICollectionViewBean.PREVIOUS));
            out.print("</SPAN> ");
        }

        out.print(" | ");
    }

    /**
             * Ordrs the list according to user selection
             */
    protected void orderResultMetaData() {
        ResultMetaData resultMetaData = null;
        int count = 0;
        int index = -1;
        int listBoxindex = -1;

        for (int i = 0; i < metaDataList.size(); i++) {
            resultMetaData = (ResultMetaData) metaDataList.get(i);

            if (resultMetaData.isDisplayable()) {
                count++;
            }
        }

        resultMetaDataArray = new ResultMetaData[count];

        for (int i = 0; i < metaDataList.size(); i++) {
            resultMetaData = (ResultMetaData) metaDataList.get(i);

            if (resultMetaData.isMandatory()) {
                primaryKey = resultMetaData.getName();
            }

            if (resultMetaData.isDisplayable()) {
                listBoxindex = resultMetaData.getListBoxIndex();

                if (listBoxindex >= 0) {
                    resultMetaDataArray[listBoxindex] = resultMetaData;
                } else {
                    resultMetaDataArray[++index] = resultMetaData;
                }
            }
        }
    }
}
