
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/**
 *
 * This class is the tag implementation for the Textbox input and password input.Javascript is disabled.
 * Tag attributes
 * minValue : minimum value to be added.Valid only for javascript.
 * maxValue : Maximum length of textbox.
 * type : normal or password
 * Inherited from Base tag
 * name
 * id
 * value
 * mandatory
 * style
 * size
 * alt
 * enabled
 */
package com.ibm.ccd.ui.tags;

import java.io.IOException;

import com.ibm.ccd.ui.util.UIUtils;


public class UITextTag extends BaseUITag {
    /**
     * attribute for text input field
     */
    public static final String UITEXT_NORMAL = "normal";

    /**
     * attribute for text input password field
     */
    public static final String UITEXT_PASSWORD = "password";

    /**
     * attribute for minValue
     */
    private String minValue = "";

    /**
     * attribute for maxlength of text field
     */
    private String maxValue = String.valueOf(DEFAULT_TEXT_MAXLENGTH);

    /**
     * attribute to decide whether text input is password or simple text input
     */
    private String type = UITEXT_NORMAL;

    /**
     * Constructor
     */
    public UITextTag() {
        super();
    }

    /*
     * This method renders HTML text field to response object
     */
    protected void renderField() throws Exception {
        log.debug(" renderField ->");

        try {
            StringBuffer htmlCode = new StringBuffer("");

            //StringBuffer htmlField = new StringBuffer("");
            htmlCode.append("<TD width=\"100%\" ALIGN =\"LEFT\">&nbsp;");

            if (isEnabled()) {
                htmlCode.append("<input type=\"");

                if ((type == null) || UITEXT_NORMAL.equalsIgnoreCase(type)) {
                    htmlCode.append("text\"");
                } else if (UITEXT_PASSWORD.equalsIgnoreCase(type)) {
                    htmlCode.append("password\"");
                }

                htmlCode.append(" name=\"" + name + "\"");

                if ((value != null) && !("null".equalsIgnoreCase(value))) {
                    htmlCode.append(" value=\"" + value + "\"");
                }

                htmlCode.append(" id=\"" + id + "\"");
                htmlCode.append(" alt=\"" + getAlt() + "\"");

                try {
                    htmlCode.append(" class=\"" + getStyle() + "\"");
                    htmlCode.append(" size=\"" + getSize() + "\"");
                    htmlCode.append(" maxlength=\"" + maxValue + "\"");
                } catch (Exception eh) {
                    log.warn(UIUtils.getTextMessages(portletContext, locale,
                            "MESSAGE_CODE_55"));
                    eh.printStackTrace();
                }

                // Adding javascript in tag
                /* Currently javascript is disabled */
                //		htmlCode.append(addJavaScript()+">");
                htmlCode.append(">");
            } else {
                if (getType().equals(UITEXT_PASSWORD)) {
                    htmlCode.append("*********");
                } else if ((value != null) &&
                        !("null".equalsIgnoreCase(value))) {
                    htmlCode.append(" " + value);
                }
            }

            htmlCode.append("</TD>");
            out.print(htmlCode.toString());
            log.debug(" renderField <- ");
        } catch (IOException e) {
            log.error(UIUtils.getTextMessages(portletContext, locale,
                    "MESSAGE_CODE_57"));
            e.printStackTrace();
        }
    }

    /**
     * This method adds javascript to html text field.It needs to include
     * wpscarmel.js.
     * @return
     */
    private String addJavaScript() {
        log.debug(" addJavaScript ->");

        int mkArrayCount = 0; // no validations added yet
        StringBuffer sbuffArray = new StringBuffer();
        StringBuffer htmlErrorCode = new StringBuffer();
        StringBuffer htmlErrorMsg = new StringBuffer();
        StringBuffer retStrbuffer = new StringBuffer();
        StringBuffer htmlParams = new StringBuffer();

        sbuffArray.append("  new Array(");

        try {
            // Adding javascript validations
            mkArrayCount = mandatoryCheck(mkArrayCount, htmlErrorCode,
                    htmlErrorMsg, htmlParams);

            minValueCheck(mkArrayCount, htmlErrorCode, htmlErrorMsg, htmlParams);

            // Constructing js validation method string		
            if (mkArrayCount > 1) {
                htmlErrorCode.append(sbuffArray).append(htmlErrorCode).append(")");
                htmlErrorMsg.append(sbuffArray).append(htmlErrorMsg).append(")");
                htmlParams.append(sbuffArray).append(htmlErrorMsg).append(")");
            }

            if (!(htmlErrorCode.toString().equals(""))) {
                retStrbuffer.append(" onBlur=\"javascript:validate(" +
                    htmlErrorCode + ",this," + htmlErrorMsg);
                retStrbuffer.append("," + htmlParams + ")\" "); // End of validate method				
            }
        } catch (Exception e1) {
            log.warn(UIUtils.getTextMessages(portletContext, locale,
                    "MESSAGE_CODE_56"));
            e1.printStackTrace();
        }

        log.debug(" addJavaScript <- ");

        return retStrbuffer.toString();
    }

    protected int minValueCheck(int mkArrayCount, StringBuffer htmlErrorCode,
        StringBuffer htmlErrorMsg, StringBuffer htmlParams)
        throws Exception {
        if ((minValue != null) && (!(minValue.equals(""))) &&
                (validate(minValue, 0) != 0)) {
            commanSeparation(mkArrayCount, htmlErrorCode, htmlErrorMsg,
                htmlParams);
            htmlErrorCode.append(JS_MINVALUECHK);
            htmlErrorMsg.append("'" +
                portletContext.getText(BUNDLE_WPC_UI_JSMESSAGES,
                    "JS_ERROR_TAG_MINIMUM_TEXT", locale) + "'");
            htmlParams.append("'" + getMinValue() + "'");
            mkArrayCount++;
        }

        return mkArrayCount;
    }

    protected void commanSeparation(int mkArrayCount,
        StringBuffer htmlErrorCode, StringBuffer htmlErrorMsg,
        StringBuffer htmlParams) {
        if (mkArrayCount != 0) {
            htmlErrorCode.append(",");
            htmlErrorMsg.append(",");
            htmlParams.append(",");
        }
    }

    protected int mandatoryCheck(int mkArrayCount, StringBuffer htmlErrorCode,
        StringBuffer htmlErrorMsg, StringBuffer htmlParams) {
        if (isMandatory()) {
            htmlErrorCode.append(JS_MANDATORYCHK); // 
            htmlErrorMsg.append("'" +
                portletContext.getText(BUNDLE_WPC_UI_JSMESSAGES,
                    "JS_ERROR_TAG_MANDATORY", locale) + "'");
            htmlParams.append("'0'");
            mkArrayCount++;
        }

        return mkArrayCount;
    }

    /**
     * @return
     */
    public String getMaxValue() {
        return maxValue;
    }

    /**
     * @return
     */
    public String getMinValue() {
        return minValue;
    }

    /**
     * @param string
     */
    public void setMaxValue(String string) {
        maxValue = string;
    }

    /**
     * @param string
     */
    public void setMinValue(String string) {
        minValue = string;
    }

    /**
     * @return
     */
    public String getType() {
        return type;
    }

    /**
     * @param string
     */
    public void setType(String string) {
        type = string;
    }
}
