
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.tags.element;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletURI;

import com.ibm.ccd.ui.dataobjects.FileUploadResource;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.BinaryAttribute;
import com.ibm.ccd.ui.util.UIUtils;


public class BinaryAttachElement extends ImageAttachElement {

    public BinaryAttachElement(Attribute attribute, PortletURI portletURI) {
        super(attribute, portletURI);
    }

    /**
     * @see com.ibm.ccd.wpc.tags.BaseElement#renderElement()
     */
    public String renderElement() {
        StringBuffer buffer = new StringBuffer();

        if (attribute.isMultiValued()) {
            for (int i = 0; i < attribute.getCurrOccurrence(); i++) {
                buffer.append(displayLabel());
                buffer.append(displayField(i, getStringValue(attribute, i)));
            }
        } else {
            buffer.append(displayLabel());
            buffer.append(displayField(0, getStringValue(attribute, 0)));
        }

        return buffer.toString();
    }

    /**
     * @param id
     * @param value
     * @return
     */
    public String displayField(long id, String value) {
    	String value1 = UIUtils.escapeForHTML(value);
        StringBuffer buffer = new StringBuffer();
        addHTMLProperty(HTMLVALUE, value1);

        String path = getAttributeID(String.valueOf(id));
		String path1 = path.substring(0,path.lastIndexOf('_'));
		addHTMLProperty(JS_EVENT_ONCHANGE,"validateText('"+path1+"')");
        attribute.setPath(path);
        addHTMLProperty(HTMLID, path);
        addHTMLProperty(HTMLNAME, path);
        buffer.append(HTML_ATTRIBUTE_START);

        if (isEditable()) {
            PortletRequest portletRequest = (PortletRequest) getPageContext()
                                                                 .getAttribute("portletRequest");
            PortletContext portletContext = ((PortletConfig) getPageContext()
                                                                 .getAttribute("portletConfig")).getContext();

            addScriptOpenWin(buffer, portletRequest, portletContext);

            buffer.append(HTML_SPACE).append(HTML_TEXT)
                  .append(getHTMLValue(HTMLID)).append(getHTMLValue(HTMLNAME))
                  .append(getHTMLValue(HTMLVALUE));
                  if(showLabel) {
                  	buffer.append(getHTMLValue(JS_EVENT_ONCHANGE));
                  }
                  buffer.append(getHTMLValue(HTMLSTYLECLASS)).append(HTML_CLOSE_TAG);

            addAttachButton(id, buffer, portletRequest, portletContext);
            addPreviewButton(id, buffer, portletRequest, portletContext, value1);

            if (attribute.isMultiValued()) {
                buffer.append(displayPlusMinus(id));
            }
        } else {
			if(attribute.isMultiValued()) {
				buffer.append(displayOccurrenceNumber(id));	
			}
            buffer.append(value1);
        }

        buffer.append(HTML_ATTRIBUTE_END);

        return buffer.toString();
    }

    /**
     * @param attribute
     * @param index
     * @return
     */
    protected String getStringValue(Attribute attribute, int index) {
        BinaryAttribute binaryAttrib = (BinaryAttribute) attribute;

        if (binaryAttrib.getCurrentOccurrence() > 0) {
            binaryAttrib.printMe();

            if (binaryAttrib.isMultiValued()) {
                FileUploadResource[] file = binaryAttrib.getValues();

                if ((file != null) && (file.length > 0)) {
                    index = (index < file.length) ? index : 0;

                    return (file[index].getName() != null)
                    ? file[index].getName() : "";
                } else {
                    return "";
                }
            } else {
                return (binaryAttrib.getValue().getName() != null)
                ? binaryAttrib.getValue().getName() : "";
            }
        } else {
            //return binaryAttrib.getDefaultValue().getName();
            return "";
        }
    }

}
