
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package com.ibm.ccd.ui.tags.element;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;

import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletURI;

import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.DateAttribute;
import com.ibm.ccd.ui.util.UIUtils;

public class DateElement extends BaseElement {

	// For Date Element only 
	private String parentformname = null; 

	public DateElement(Attribute attribute, PortletURI portletURI) {
		super(attribute, portletURI);
	}

	/**
	 * @see com.ibm.ccd.wpc.tags.BaseElement#displayField(long, String)
	 */
	public String displayField(long id, Object value) {
		StringBuffer buffer = new StringBuffer();
		addHTMLProperty(HTMLVALUE, (String) value);

		String path = getAttributeID(String.valueOf(id));
		attribute.setPath(path);
		addHTMLProperty(HTMLID, path);

		addHTMLProperty(HTMLNAME, path);

		buffer.append(HTML_ATTRIBUTE_START);

		if (isEditable()) {
			renderCalendarScript(buffer, path, String.valueOf(value));

			if (attribute.isMultiValued()) {
				buffer.append(displayPlusMinus(id));
			}
		} else {
			if(attribute.isMultiValued()) {
				buffer.append(displayOccurrenceNumber(id));	
			}
			buffer.append(parseWPCDate((String) value));			
		}

		buffer.append(HTML_ATTRIBUTE_END);

		return buffer.toString();
	}

	public String renderElement() {
		StringBuffer buffer = new StringBuffer();

		if (attribute.isMultiValued()) {
			for (int i = 0; i < attribute.getCurrOccurrence(); i++) {
				String path = getAttributeID(String.valueOf(i));
				String error = attribute.getValidationError(path);
				if(error != null){
					buffer.append(displayError(error));
				}
				else{
				buffer.append(displayLabel());
				}
				buffer.append(displayField(i, getStringValue(attribute, i)));
			}
		} else {
			String path = getAttributeID(String.valueOf(0));
			String error = attribute.getValidationError(path);
			if(error != null){
			buffer.append(displayError(error));
			}
			else{
			buffer.append(displayLabel());
			}
			buffer.append(displayField(0, getStringValue(attribute, 0)));
		}

		return buffer.toString();
	}

	/**
	 * @param htmlCode
	 * @param calendarId
	 * @param value
	 */
	protected void renderCalendarScript(
		StringBuffer htmlCode,
		String calendarId,
		String value) {
		StringBuffer buffer = new StringBuffer("");
		String calendarId1 = calendarId;

		calendarId = calendarId.substring(calendarId.lastIndexOf('/') + 1);

		for (int i = 0; i < calendarId.length(); i++) {
			if (Character.isSpaceChar(calendarId.charAt(i))) {
				continue;
			}

			if (calendarId.charAt(i) != '/') {
				buffer.append(calendarId.charAt(i));
			}
		}
		
		int occ = 0;
		int occ_Index = calendarId.lastIndexOf('_');
		if(occ_Index != -1) {
			occ = Integer.parseInt(calendarId.substring(occ_Index+1)); 
		}
		
		String encodeurl =
			(
				(PortletResponse) pageContext.getAttribute(
					"portletResponse")).encodeURL(
				"/");

		PortletRequest portletRequest =
			(PortletRequest) pageContext.getAttribute("portletRequest");

		String fieldValue = "";
		
		if(attribute.getValueList() != null && attribute.getValueList().size() > occ) {
			Object obj = attribute.getValueList().get(occ);
			if(obj != null) {
				fieldValue = (String) obj;	
			}
		}

		htmlCode.append("<script language=\"JavaScript\"> ");
		
		if (!UIUtils.isEmpty(fieldValue)) {
			htmlCode.append(
				"document.write(placeCalendar('"
					+ buffer
					+ "','"
					+ getParentformname()
					+ "','"
					+ encodeurl
					+ "','"
					+ calendarId1
					+ "','"
					+ parseWPCDate(fieldValue)
					+ "'));");
		} else {
			htmlCode.append(
				"document.write(placeCalendar('"
					+ buffer
					+ "','"
					+ getParentformname()
					+ "','"
					+ encodeurl
					+ "','"
					+ calendarId1
					+ "','"
					+ fieldValue
					+ "'));");
		}
		htmlCode.append("</script>");

		htmlCode.append(
			"<input type=\"hidden\" id=\""
				+ calendarId1
				+ "\" name=\""
				+ calendarId1
				+ "\" value=\""
				+ parseWPCDate(fieldValue)
				+ "\" >");
		htmlCode.append("<DIV id=\"editPopup"+buffer +"\" style='position: absolute; z-index:111; width:410; border:1px solid #a4a4a4; background-color:ffffff; visibility: hidden;'>"
		+ "<TABLE width = \"100%\" cellSpacing = '0' cellPadding = '0'>"
		+ "<TR><TD  id = \"td1"+buffer +"\"></TD></TR></TABLE></DIV> ");
	}

	/**
	 * @param attribute
	 * @param index
	 * @return
	 */
	protected String getStringValue(Attribute attribute, int index) {
		DateAttribute dateAttrib = (DateAttribute) attribute;

		if (dateAttrib.getCurrentOccurrence() > 0) {
			if (dateAttrib.isMultiValued()) {
				String[] values = dateAttrib.getValues();

				return (values[index] != null) ? values[index] : "";
			} else {
				return (dateAttrib.getValue() != null)
					? dateAttrib.getValue()
					: "";
			}
		} else {
			return (dateAttrib.getDefaultValue() != null)
				? dateAttrib.getDefaultValue()
				: "";
		}
	}

	/**
	 * Converts WPCDate String format yyyy-mm-dd HH:MM:SS to custom format 
	 * specified in wpclabels.properties  
	 * @param date_str
	 * @return
	 */
	protected String parseWPCDate(String date_str) {
		//Convert the date from WPC format to the custom format to be displayed to the user
		if(date_str.equals(""))
			return date_str;
		PortletRequest portletRequest =	(PortletRequest) pageContext.getAttribute("portletRequest");
		Locale locale = portletRequest.getLocale();

		ResourceBundle bundle =	ResourceBundle.getBundle("com.ibm.ccd.ui.nls.labels.wpclabels",locale);
		String customFormat = bundle.getString("CALENDAR.DATE.FORMAT");
		customFormat = customFormat.replace('S','s');
		customFormat = customFormat.replaceAll("mm","MM");
		customFormat = customFormat.replaceAll(":MM",":mm");
		String wpcFormat = bundle.getString("WPC.DATE.FORMAT");
		wpcFormat = wpcFormat.replace('S','s');
		wpcFormat = wpcFormat.replaceAll("mm","MM");
		wpcFormat = wpcFormat.replaceAll(":MM",":mm");		
		SimpleDateFormat customDateFormat = new SimpleDateFormat(customFormat,locale);
		SimpleDateFormat wpcDateFormat = new SimpleDateFormat(wpcFormat,locale);
		
		try{
			Date date = null;
			date = wpcDateFormat.parse(date_str);
			date_str = customDateFormat.format(date);
			System.err.println("Date after conversion is " + date_str);
			return date_str;
		}catch(ParseException pe){
			System.err.println("ParseException " + pe);
			return date_str;
		}
	}

	/**
	 * searches the original string for the search String and replaces 
	 * it with replace String.   
	 * @param original
	 * @param searchStr
	 * @param replaceStr
	 * @return
	 */
	public String replace(
		String original,
		String searchStr,
		String replaceStr) {

		StringBuffer buffer = new StringBuffer(original);
		int index = 0;

		while ((index = original.indexOf(searchStr, index)) != -1) {
			buffer.replace(index, index + searchStr.length(), replaceStr);
			index++;
		}
		return buffer.toString();
	}

	/**
	 * pad Zero if the number is less than 9
	 * @param num
	 * @return
	 */
	public String padZero(int num) {
		return (num < 10) ? "0" + num : num + "";
	}

	/**
	 * @return Returns the parentformname.
	 */
	public String getParentformname() {
		return parentformname;
	}
	/**
	 * @param parentformname The parentformname to set.
	 */
	public void setParentformname(String parentformname) {
		this.parentformname = parentformname;
	}
}
