
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.tags.element;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletURI;

import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.FlagAttribute;
import com.ibm.ccd.ui.util.UIUtils;


public class FlagElement extends TextElement {
	private PortletContext portletContext = null;
	private PortletRequest portletRequest = null;
	
    public FlagElement(Attribute attribute, PortletURI portletURI) {
        super(attribute, portletURI);
    }

	/**
	  * @see com.ibm.ccd.wpc.tags.BaseElement#renderElement()
	  */
	 public String renderElement() {
		 portletRequest = (PortletRequest) pageContext.getAttribute(
				 "portletRequest");
		 portletContext = ((PortletConfig) pageContext.getAttribute(
				 "portletConfig")).getContext();

		 StringBuffer buffer = new StringBuffer();

		 if (attribute.isMultiValued()) {
			 for (int i = 0; i < attribute.getCurrOccurrence(); i++) {
				 String path = getAttributeID(String.valueOf(i));
				 String error = attribute.getValidationError(path);
				 if(error != null){
					 if(isShowLabel())
						 buffer.append(displayError(error));
				 }
				 else{
					 if(isShowLabel())
						 buffer.append(displayLabel());
				 }
				 buffer.append(displayField(i, getStringValue(attribute, i)));
			 }
		 } else {
			 String path = getAttributeID(String.valueOf(0));
			 String error = attribute.getValidationError(path);
			 if(error != null){
				 if(isShowLabel())
					 buffer.append(displayError(error));
			 }
			 else{
				 if(isShowLabel())
					 buffer.append(displayLabel());
			 }
			 buffer.append(displayField(0, getStringValue(attribute, 0)));
		 }

		 return buffer.toString();
	 }

	 /**
	  * @param id
	  * @param value
	  * @return
	  */
	 public String displayField(long id, Object value) {
		 String value1 = UIUtils.escapeForHTML((String)value);
		 StringBuffer buffer = new StringBuffer();
		 addHTMLProperty(HTMLVALUE, value1);

		 String path = getAttributeID(String.valueOf(id));
		 String path1 = path.substring(0,path.lastIndexOf('_'));
		 addHTMLProperty(JS_EVENT_ONCHANGE,"validateText('"+path1+"')");   
		 attribute.setPath(path);
		 addHTMLProperty(HTMLID,path);
		 addHTMLProperty(HTMLNAME,path);
		 buffer.append(HTML_ATTRIBUTE_START);

		 if (isEditable()) {
			 FlagAttribute flagAttribute = (FlagAttribute) attribute; 
			 String[] stringArr = flagAttribute.getEnumValues();
			 buffer.append(HTML_SPACE).append(HTML_SELECT)
				   .append(getHTMLValue(HTMLNAME)).append(getHTMLValue(HTMLID))
				   .append(getHTMLValue(HTMLSTYLECLASS));
				   if(isShowLabel())
					 buffer.append(getHTMLValue(JS_EVENT_ONCHANGE));
				   buffer.append(HTML_CLOSE_TAG)
				   .append(generateHTMLOption(stringArr, value1)).append(HTML_SELECT_CLOSE);

			 if (attribute.isMultiValued()) {
				 if(isShowLabel())
					 buffer.append(displayPlusMinus(id));
			 }
		 } else {
			 if(attribute.isMultiValued()) {
				 buffer.append(displayOccurrenceNumber(id));	
			 }        	
			 buffer.append(UIUtils.escapeForHTML(getDisplayValue(attribute, Integer.parseInt("0"+id))));
		 }

		 buffer.append(HTML_ATTRIBUTE_END);

		 return buffer.toString();
	 }

	 /**
	  * generates the HTML options for HTML Select tag
	  *
	  * @param values
	  * @param selectedValue
	  * @return
	  */
	 protected String generateHTMLOption(String[] values, String selectedValue) {
		 StringBuffer buffer = new StringBuffer();

		 if (values != null) {

			 //Displays the HTML select option values 
			 for (int i = 0; i < values.length; i++) {
				 String value = UIUtils.escapeForHTML(values[i].trim());
				 String value1 = null; 
				 addHTMLProperty(HTMLVALUE, value);

				 if("".equals(value)) {
					value1 = UIUtils.getTextLabel(portletContext, portletRequest,"DROPDOWN_VALUE_NONE");
				 }else if("false".equals(value)) {
				 	value1 = UIUtils.getTextLabel(portletContext, portletRequest,"DROPDOWN_VALUE_FALSE");
				 }else if("true".equals(value)) {
					value1 = UIUtils.getTextLabel(portletContext, portletRequest,"DROPDOWN_VALUE_TRUE");
				 }	
				 
				 if (value.equals(selectedValue)) {
					 buffer.append(HTML_OPTION_OPEN)
						   .append(HTMLVALUE + "\"" + value + "\" ")
						   .append(HTML_SELECTED + " " + HTML_CLOSE_TAG).append(value1)
						   .append(HTML_OPTION_CLOSE);
				 } else {
					 buffer.append(HTML_OPTION_OPEN)
							.append(HTMLVALUE + "\"" + value + "\" ")
							.append(HTML_CLOSE_TAG).append(value1)
							.append(HTML_OPTION_CLOSE);
				 }
			 }
		 } 

		 return buffer.toString();
	 }

    /**
     * @param attribute
     * @param index
     * @return
     */
    protected String getStringValue(Attribute attribute, int index) {
        FlagAttribute flagAttrib = (FlagAttribute) attribute;

        if (flagAttrib.getCurrOccurrence() > 0) {
            String[] fvalues = flagAttrib.getValues();

            if (flagAttrib.isMultiValued()) {
                return (fvalues[index]);
            } else {
                if (fvalues[0] == null) {
                    return "";
                } else {
                    return (fvalues[0]);
                }
            }
        } else {
            if (flagAttrib.getDefaultValue() != null) {
                return (flagAttrib.getDefaultValue());
            } else {
                return "";
            }
        }
    }
    
    protected String getDisplayValue(Attribute attribute, int index) {
    	String value = getStringValue(attribute, index);
    	
		if("false".equals(value)) {
		   return UIUtils.getTextLabel(portletContext, portletRequest,"DROPDOWN_VALUE_FALSE");
		}else if("true".equals(value)) {
		   return UIUtils.getTextLabel(portletContext, portletRequest,"DROPDOWN_VALUE_TRUE");
		}else {
		   return UIUtils.getTextLabel(portletContext, portletRequest,"DROPDOWN_VALUE_NONE");
		}	
    }
}
