
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.tags.element;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletSession;
import org.apache.jetspeed.portlet.PortletURI;

import com.ibm.ccd.ui.context.IWPCUser;
import com.ibm.ccd.ui.context.WPCUser;
import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.dataobjects.FileUploadResource;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.ImageAttribute;
import com.ibm.ccd.ui.util.UIUtils;


public class ImageAttachElement extends TextElement {
	/*
	 *  attribute added for preview button on binaryattachment element in attribute tag 
	 */	
	private String preview ="false";
    public ImageAttachElement(Attribute attribute, PortletURI portletURI) {
        super(attribute, portletURI);
    }

    /**
     * @see com.ibm.ccd.wpc.tags.BaseElement#renderElement()
     */
    public String renderElement() {
        StringBuffer buffer = new StringBuffer();

        if (attribute.isMultiValued()) {
            for (int i = 0; i < attribute.getCurrOccurrence(); i++) {
            	String path = getAttributeID(String.valueOf(i));
				String error = attribute.getValidationError(path);
				if(error != null){
					if(isShowLabel())
						buffer.append(displayError(error));
				}
				else{
					if(isShowLabel())
						buffer.append(displayLabel());
				}
                buffer.append(displayField(i, getStringValue(attribute, i)));
            }
        } else {
        	String path = getAttributeID(String.valueOf(0));
			String error = attribute.getValidationError(path);
			if(error != null){
				if(isShowLabel())
				buffer.append(displayError(error));
			}
			else{
				if(isShowLabel())
					buffer.append(displayLabel());
			}
            buffer.append(displayField(0, getStringValue(attribute, 0)));
        }

        return buffer.toString();
    }

    /**
     * @param id
     * @param value
     * @return
     */
    public String displayField(long id, String value) {
    	String value1 = UIUtils.escapeForHTML(value);
        StringBuffer buffer = new StringBuffer();
        addHTMLProperty(HTMLVALUE, value1);

        String path = getAttributeID(String.valueOf(id));
		String path1 = path.substring(0,path.lastIndexOf('_'));
		addHTMLProperty(JS_EVENT_ONCHANGE,"validateText('"+path1+"')");
        attribute.setPath(path);
        addHTMLProperty(HTMLID, path);
        addHTMLProperty(HTMLNAME, path);

        buffer.append(HTML_ATTRIBUTE_START);

        if (isEditable()) {
            PortletRequest portletRequest = (PortletRequest) pageContext.getAttribute(
                    "portletRequest");
            PortletContext portletContext = ((PortletConfig) pageContext.getAttribute(
                    "portletConfig")).getContext();

            addScriptOpenWin(buffer, portletRequest, portletContext);

            buffer.append(HTML_SPACE).append(HTML_TEXT)
                  .append(getHTMLValue(HTMLID)).append(getHTMLValue(HTMLNAME))
                  .append(getHTMLValue(HTMLVALUE));
                  if(isShowLabel()) 
                  	buffer.append(getHTMLValue(JS_EVENT_ONCHANGE));
                  buffer.append(getHTMLValue(HTMLSTYLECLASS)).append(HTML_CLOSE_TAG);
            addAttachButton(id, buffer, portletRequest, portletContext);
            addPreviewButton(id, buffer, portletRequest, portletContext, value1);
            
            if (attribute.isMultiValued()) {
                buffer.append(displayPlusMinus(id));
            }
        } else {
			if(attribute.isMultiValued()) {
				buffer.append(displayOccurrenceNumber(id));	
			}        	
            buffer.append(value1);
        }

        buffer.append(HTML_ATTRIBUTE_END);

        return buffer.toString();
    }

    /**
     * @param id
     * @param buffer
     * @param portletRequest
     * @param portletContext
     */
    protected void addAttachButton(long id, StringBuffer buffer,
        PortletRequest portletRequest, PortletContext portletContext) {
        String path = getAttributeID(id+"");
		String path1 = path.substring(0,path.lastIndexOf('_'));
        buffer.append(HTML_BUTTON)
              .append(" NAME =\"" + 
            getAttributeID(String.valueOf(id)) + "_BUTTON\"")
              .append(" class=\"wpsButtonText\" size=\"10\" ")
              .append(" value=\"" +
            UIUtils.getTextLabel(portletContext, portletRequest.getLocale(),
                "ATTACH_BUTTON") + "\"").append(" onClick=\"javascript:openWin('" +
            getAttributeID(String.valueOf(id)) + "'); validateText('"+path1+"')\">");
    }

    /**
     * @param buffer
     * @param portletRequest
     * @return
     */
    protected void addScriptOpenWin(StringBuffer buffer,
        PortletRequest portletRequest, PortletContext portletContext) {
        PortletResponse portletResponse = (PortletResponse) pageContext.getAttribute(
                "portletResponse");
        PortletSession portletSession = portletRequest.getPortletSession();

        WPSContext context = UIUtils.getWPSContext(portletRequest,portletContext);
        WPCUser user = context.getWpcUser();
        String userName = "";
        String company = "";
		String supplierId = "";
		String wpcuserId = "";

        if (user != null) {
            userName = (String) user.getProperty(IWPCUser.KEY_USER_ID);
            company = (String) user.getProperty(IWPCUser.KEY_COMPANY);
			supplierId = (String) user.getProperty(IWPCUser.KEY_SUPPLIER_GLN);
			wpcuserId = (String) user.getProperty(IWPCUser.KEY_WPC_USERNAME);
        }

		buffer.append("<script language=\"JavaScript\">")
	  	.append("function openWin(e)	{ ")
	  	.append("window.open('" +
			portletResponse.encodeURL("/jsps/html/itemcatalog/AttachFile.jsp") +
			"?ele='+e+'&eurl=" + portletResponse.encodeURL("/") + "&" +
			IWPCUser.KEY_USER_ID + "=" + userName + "&" + 
			IWPCUser.KEY_COMPANY + "=" + company + "&" +
			IWPCUser.KEY_SUPPLIER_GLN + "=" + supplierId + "&" +
			IWPCUser.KEY_WPC_USERNAME + "=" + wpcuserId  +
            "','Upload',\"top=220,left=320,width=420,height=200,resizable=no,scrollbars=yes,menubar=no,toolbar=no,directories=no,location=no,status=no\");")
            .append("}").append("</script>");
    }

    /**
     * @param attribute
     * @param index
     * @return
     */
    protected String getStringValue(Attribute attribute, int index) {
        ImageAttribute imageAttrib = (ImageAttribute) attribute;

        if (imageAttrib.getCurrentOccurrence() > 0) {
            if (imageAttrib.isMultiValued()) {
                FileUploadResource[] files = imageAttrib.getValues();

                return (files[index].getName() != null)
                ? files[index].getName() : "";
            } else {
                FileUploadResource file = imageAttrib.getValue();

                return (file.getName() != null) ? file.getName() : "";
            }
        } else {
            return "";
        }
    }
    
    /**
     * @return Returns the preview.
     */
    public String getPreview() {
    	return preview;
    }
    /**
     * @param preview The preview to set.
     */
    public void setPreview(String preview) {
    	this.preview = preview;
    }
    /**
     * @param id
     * @param buffer
     * @param portletRequest
     * @param portletContext
     * @param value TODO
     */
    protected void addPreviewButton(long id, StringBuffer buffer,
        PortletRequest portletRequest, PortletContext portletContext, String value) {
    	if(preview.equalsIgnoreCase("false"))
    		return;
    	
    	if (UIUtils.isEmpty(value)) {
			return;
		}
    	int index = value.lastIndexOf('.');
    	String extn = (index == -1)?"":value.substring(index);

    	buffer.append(HTML_BUTTON)
              .append(" NAME =\"" + 
            getAttributeID(String.valueOf(id)) + "_VIEW_BUTTON\"")
              .append(" class=\"wpsButtonText\" size=\"10\" ")
			  .append(" value=\"")
			  .append(UIUtils.getTextLabel(portletContext, portletRequest.getLocale(),
                "PREVIEW_BUTTON") + "\" ");
				buffer.append(" onClick=\"javascript:openPreview(this.form."+getAttributeID(String.valueOf(id))+".value,'" +
			    getAttributeID(String.valueOf(id)) + "','"+attribute.getName()+"','"+ attribute.getType()+ "','"+extn + "')\"");
              
			buffer.append(">");
    }
}
