
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.tags.element;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletURI;

import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberEnumAttribute;
import com.ibm.ccd.ui.util.UIUtils;


public class NumberEnumerationElement extends BaseElement {
    private PortletRequest portletRequest = null;
    private PortletContext portletContext = null;
    private NumberEnumAttribute numberEnumAttribute = null;

    public NumberEnumerationElement(Attribute attribute, PortletURI portletURI) {
        super(attribute, portletURI);
        numberEnumAttribute = (NumberEnumAttribute) attribute;
    }

    /**
     * @see com.ibm.ccd.wpc.tags.BaseElement#renderElement()
     */
    public String renderElement() {
        portletRequest = (PortletRequest) pageContext.getAttribute(
                "portletRequest");
        portletContext = ((PortletConfig) pageContext.getAttribute(
                "portletConfig")).getContext();

        StringBuffer buffer = new StringBuffer();

        if (attribute.isMultiValued()) {
            for (int i = 0; i < attribute.getCurrOccurrence(); i++) {
            	String path = getAttributeID(String.valueOf(i));
            	String error = attribute.getValidationError(path);
				if(error != null){
					if(isShowLabel())
						buffer.append(displayError(error));
				}
				else{
					if(isShowLabel())
						buffer.append(displayLabel());
				}
                buffer.append(displayField(i, getStringValue(attribute, i)));
            }
        } else {
        	String path = getAttributeID(String.valueOf(0));
        	String error = attribute.getValidationError(path);
			if(error != null){
				if(isShowLabel())
					buffer.append(displayError(error));
			}
			else{
				if(isShowLabel())
		            buffer.append(displayLabel());
			}
            buffer.append(displayField(0, getStringValue(attribute, 0)));
        }

        return buffer.toString();
    }

    /**
     * @param id
     * @param value
     * @return
     */
    public String displayField(long id, Object value) {
        StringBuffer buffer = new StringBuffer();
        addHTMLProperty(HTMLVALUE, (String) value);

        String path = getAttributeID(String.valueOf(id));
        attribute.setPath(path);
		String path1 = path.substring(0,path.lastIndexOf('_'));
		addHTMLProperty(JS_EVENT_ONCHANGE,"validateText('"+path1+"')");   
        addHTMLProperty(HTMLID,path);
        addHTMLProperty(HTMLNAME,path);
        buffer.append(HTML_ATTRIBUTE_START);

        if (isEditable()) {
            double[] doubleArr = numberEnumAttribute.getEnumDb();
            buffer.append(HTML_SPACE).append(HTML_SELECT)
                  .append(getHTMLValue(HTMLSTYLECLASS))
                  .append(getHTMLValue(HTMLID)).append(getHTMLValue(HTMLNAME));
				if (isShowLabel())
					buffer.append(getHTMLValue(JS_EVENT_ONCHANGE));
                  buffer.append(HTML_CLOSE_TAG)
                  .append(generateHTMLOption(doubleArr,
                    (String)value)).append(HTML_SELECT_CLOSE);

            if (attribute.isMultiValued()) {
            	if(isShowLabel())
            		buffer.append(displayPlusMinus(id));
            }
        } else {
			if(attribute.isMultiValued()) {
				buffer.append(displayOccurrenceNumber(id));	
			}        	
            buffer.append(getStringValue(attribute, Integer.parseInt("0"+id)));
        }

        buffer.append(HTML_ATTRIBUTE_END);

        return buffer.toString();
    }

    /**
     * generates the HTML options for HTML Select tag
     *
     * @param values
     * @param selectedValue
     * @return
     */
    protected String generateHTMLOption(double[] values, String selectedValue) {
        StringBuffer buffer = new StringBuffer();

        if (values != null) {
            //Displays the HTML option value NONE
            buffer.append(generateHTMLNoneOption());

            //Displays the HTML select option values 		
            for (int i = 0; i < values.length; i++) {
                addHTMLProperty(HTMLVALUE, values[i] + "");

                if (String.valueOf(values[i]).equals(selectedValue)) {
                    buffer.append(HTML_OPTION_OPEN)
                          .append(getHTMLValue(HTMLVALUE) + " " +
                        HTML_SELECTED + " " + HTML_CLOSE_TAG).append(values[i])
                          .append(HTML_OPTION_CLOSE);
                } else {
                    buffer.append(HTML_OPTION_OPEN)
                          .append(getHTMLValue(HTMLVALUE) + HTML_CLOSE_TAG)
                          .append(values[i]).append(HTML_OPTION_CLOSE);
                }
            }
        } else {
            buffer.append(generateHTMLNoneOption());
        }

        return buffer.toString();
    }

    /**
     * generate HTML option with none as default selected
     * @return HTML None Option value eg., <option value="-1">NONE</option>
     */
    protected String generateHTMLNoneOption() {
        StringBuffer buffer = new StringBuffer();

		buffer.append(HTML_OPTION_OPEN).append(HTMLVALUE + HTML_OPEN_QUOTE + "" + HTML_CLOSE_QUOTE + HTML_CLOSE_TAG)
		.append(UIUtils.getTextLabel(portletContext, portletRequest,
                "DROPDOWN_VALUE_NONE")).append(HTML_OPTION_CLOSE);

        return buffer.toString();
    }

    /**
     * @param attribute
     * @param index
     * @return
     */
    protected String getStringValue(Attribute attribute, int index) {
        NumberEnumAttribute numberEnumAttrib = (NumberEnumAttribute) attribute;

        if (numberEnumAttrib.getCurrentOccurrence() > 0) {
            if (numberEnumAttrib.isMultiValued()) {
                String[] values = numberEnumAttrib.getValues();

                return values[index];
            } else {
                return numberEnumAttrib.getValue();
            }
        } else {
            return numberEnumAttrib.getDefaultValue();
        }
    }

}
