
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.tags.element;

import org.apache.jetspeed.portlet.PortletURI;

import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.PasswordAttribute;
import com.ibm.ccd.ui.util.UIUtils;


public class PasswordElement extends TextElement {
	
	private static final String JS_VALIDATE_PASSWORD = "validatePassword(this)";
	private static final String JS_SELTEXT_PASSWORD = "clearPasswordText(this)";
	 
    public PasswordElement(Attribute attribute, PortletURI portletURI) {
        super(attribute, portletURI);
    }

    /**
     * @see com.ibm.ccd.wpc.tags.BaseElement#renderElement()
     */
    public String renderElement() {
        StringBuffer buffer = new StringBuffer();

        if (attribute.isMultiValued()) {
            for (int i = 0; i < attribute.getCurrOccurrence(); i++) {
                buffer.append(displayLabel());
                buffer.append(displayField(i, getStringValue(attribute, i)));
            }
        } else {
            buffer.append(displayLabel());
            buffer.append(displayField(0, getStringValue(attribute, 0)));
        }

        return buffer.toString();
    }

    /**
     * @param id
     * @param value
     * @return
     */
    public String displayField(int id, Object value) {
    	String value1 = UIUtils.escapeForHTML((String)value);
        StringBuffer buffer = new StringBuffer();

        addHTMLProperty(HTMLVALUE,  getAsterisk(value1));

        String path = getAttributeID(String.valueOf(id));
        attribute.setPath(path);
		
		String path1 = path.substring(0,path.lastIndexOf('_'));
		addHTMLProperty(JS_EVENT_ONCHANGE,"validatePassword(this,'"+path1+"')");
		addHTMLProperty(JS_EVENT_ONFOCUS,"clearPasswordText(this,'"+path1+"')"); 
        
        addHTMLProperty(HTMLID,path);
        addHTMLProperty(HTMLNAME,path);
        buffer.append(HTML_ATTRIBUTE_START);

        if (isEditable()) {
            buffer.append(HTML_SPACE).append(HTML_PASSWORD)
                  .append(getHTMLValue(HTMLID)).append(getHTMLValue(HTMLNAME))
                  .append(getHTMLValue(HTMLSIZE)).append(getHTMLValue(HTMLVALUE))
                  .append(getHTMLValue(HTMLSTYLECLASS))
                  .append(getHTMLValue(JS_EVENT_ONCHANGE))
                  .append(getHTMLValue(JS_EVENT_ONFOCUS))
                  .append(HTML_CLOSE_TAG);
            
            addHTMLProperty(HTMLID,path+"_h");
            addHTMLProperty(HTMLNAME,path+"_h");
			addHTMLProperty(HTMLVALUE,"0");
                  
            buffer.append(HTML_SPACE).append(HTML_HIDDEN)
            	.append(getHTMLValue(HTMLID)).append(getHTMLValue(HTMLNAME))
            	.append(getHTMLValue(HTMLVALUE))
            	.append(HTML_CLOSE_TAG);
            	

            if (attribute.isMultiValued()) {
                buffer.append(displayPlusMinus(id));
            }
        } else {
			if(attribute.isMultiValued()) {
				buffer.append(displayOccurrenceNumber(id));	
			}        	
            buffer.append(HTML_SPACE).append(getAsterisk((String)value));
        }

        buffer.append(HTML_ATTRIBUTE_END);

        return buffer.toString();
    }

    /**
     * @param attribute
     * @param index
     * @return
     */
    protected String getStringValue(Attribute attribute, int index) {
        PasswordAttribute passAttrib = (PasswordAttribute) attribute;

        if (passAttrib.getCurrentOccurrence() > 0) {
            if (passAttrib.isMultiValued()) {
                String[] values = passAttrib.getValues();

                if ((values != null) || (values.length > 0)) {
                    return values[index];
                }
                else {
                    return "";
                }
            } else {
                return (passAttrib.getValue() != null) ? passAttrib.getValue()
                                                       : "";
            }
        } else {
            return (passAttrib.getDefaultValue() != null)
            ? passAttrib.getDefaultValue() : "";
        }
    }

	private String getAsterisk(String value) {
		String asktricks = "********************";
		if(value != null) {
			if(value.length() < 20)
		    	return asktricks.substring(0,value.length());
		    else 
		    	return asktricks;
		}else {
			return "";
		}
	}
}
