/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.tags.element;

import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletURI;

import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.PeriodAttribute;
import com.ibm.ccd.ui.util.UIUtils;


public class PeriodElement extends DateElement {

    public PeriodElement(Attribute attribute, PortletURI portletURI) {
        super(attribute, portletURI);
    }

    /**
     * @see com.ibm.ccd.wpc.tags.BaseElement#displayField(long, String)
     */
	public String displayField(long id, Object value) {
		StringBuffer buffer = new StringBuffer();

		PeriodAttribute valueAttribute = (PeriodAttribute) value;

		String path = getAttributeID(String.valueOf(id));
		attribute.setPath(path);
		addHTMLProperty(HTMLNAME, path);

		buffer.append(HTML_ATTRIBUTE_START);

		if (isEditable()) {

			renderCalendarScript(
				buffer,
				path + "_" + "V1",
				valueAttribute.getStartDate((int) id));

			buffer.append(HTML_SPACE);
			renderCalendarScript(
				buffer,
				path + "_" + "V2",
				valueAttribute.getEndDate((int) id));
			if (attribute.isMultiValued()) {
				buffer.append(displayPlusMinus(id));
			}
			
		} else {
			if(attribute.isMultiValued()) {
				buffer.append(displayOccurrenceNumber(id));	
			}        	

			buffer.append(valueAttribute.getStartDate((int) id)).append(
				HTML_SPACE).append(
				valueAttribute.getEndDate((int) id));
		}


		buffer.append(HTML_ATTRIBUTE_END);

		return buffer.toString();
	}
	
	

	protected void renderCalendarScript(
			StringBuffer htmlCode,
			String calendarId,
			String value) {
			StringBuffer buffer = new StringBuffer("");
			String calendarId1 = calendarId;

			calendarId = calendarId.substring(calendarId.lastIndexOf('/') + 1);

			for (int i = 0; i < calendarId.length(); i++) {
				if (Character.isSpaceChar(calendarId.charAt(i))) {
					continue;
				}

				if (calendarId.charAt(i) != '/') {
					buffer.append(calendarId.charAt(i));
				}
			}
			
			int occ = 0;
			int occ_Index = calendarId.substring(0,calendarId.lastIndexOf('_')).lastIndexOf('_');
			if(occ_Index != -1) {
				occ = Integer.parseInt(calendarId.substring(occ_Index+1,calendarId.lastIndexOf('_'))); 
			}
			
			String encodeurl =
				(
					(PortletResponse) pageContext.getAttribute(
						"portletResponse")).encodeURL(
					"/");

			PortletRequest portletRequest =
				(PortletRequest) pageContext.getAttribute("portletRequest");

			htmlCode.append("<script language=\"JavaScript\"> ");

			if (!UIUtils.isEmpty(value)) {
				htmlCode.append(
					"document.write(placeCalendar('"
						+ buffer
						+ "','"
						+ encodeurl
						+ "','"
						+ calendarId1
						+ "','"
						+ parseWPCDate(value)
						+ "'));");
			} else {
				htmlCode.append(
					"document.write(placeCalendar('"
						+ buffer
						+ "','"
						+ encodeurl
						+ "','"
						+ calendarId1
						+ "','"
						+ value
						+ "'));");
			}
			htmlCode.append("</script>");

			htmlCode.append(
				"<input type=\"hidden\" id=\""
					+ calendarId1
					+ "\" name=\""
					+ calendarId1
					+ "\" value=\""
					+ parseWPCDate(value)
					+ "\" >");
		}

    public String renderElement() {
        StringBuffer buffer = new StringBuffer();

        if (attribute.isMultiValued()) {
            for (int i = 0; i < attribute.getCurrOccurrence(); i++) {
            	String path = getAttributeID(String.valueOf(i));
            	String fromPath = path + "_" + "V1";
            	String endPath = path + "_" + "V2";
            	String fromError = attribute.getValidationError(fromPath);
            	String toError = attribute.getValidationError(endPath);
            	String periodError = attribute.getValidationError(path);
            	if(periodError != null){
            		buffer.append(displayError(periodError));
            	}
            	else if(fromError != null && toError != null){
            		buffer.append(displayError(fromError + toError));
            	}
            	else if(fromError != null){
            		buffer.append(displayError(fromError));
            	}
            	else if(toError != null){
            		buffer.append(displayError(toError));
            	}
            	else {
            		buffer.append(displayLabel());
            	}
                buffer.append(displayField(i, attribute));
            }
        } else {
        	String path = getAttributeID(String.valueOf(0));
        	String fromPath = path + "_" + "V1";
        	String endPath = path + "_" + "V2";
        	String fromError = attribute.getValidationError(fromPath);
        	String toError = attribute.getValidationError(endPath);
        	String periodError = attribute.getValidationError(path);
        	if(periodError != null){
        		buffer.append(displayError(periodError));
        	}
        	else if(fromError != null && toError != null){
        		buffer.append(displayError(fromError + toError));
        	}
        	else if(fromError != null){
        		buffer.append(displayError(fromError));
        	}
        	else if(toError != null){
        		buffer.append(displayError(toError));
        	}
        	else {
        		buffer.append(displayLabel());
        	}
            buffer.append(displayField(0, attribute));
        }

        return buffer.toString();
    }
	/**
	 * @param period
	 * @return
	 */
	private String getStartDate(String period){
		String startDate = period.substring(0,period.indexOf('|'));
		return startDate;	
	}
	/**
	 * @param period
	 * @return
	 */
	private String getEndDate(String period){
		String endDate = period.substring(period.indexOf('|')+1);
		return endDate ;	
	}	
}
