
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * Created on Jul 26, 2005
 *
 */
package com.ibm.ccd.ui.tags.element;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletURI;

import com.ibm.ccd.ui.dataobjects.NVPair;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.RelationshipAttribute;
import com.ibm.ccd.ui.tags.ITagConstants;
import com.ibm.ccd.ui.util.UIUtils;
import com.ibm.ccd.ui.util.ValidationUtil;


public class RelationElement extends BaseElement {
    private String LABEL_KEY_MENU_OPTION_SEARCH = "WORK_WITH_ITEM.VIEW.REL_ATTR_MENU_SEARCH";
    private String LABEL_KEY_MENU_OPTION_BROWSE = "WORK_WITH_ITEM.VIEW.REL_ATTR_MENU_BROWSE";
    private String LABEL_KEY_MENU_OPTION_VIEW = "WORK_WITH_ITEM.VIEW.REL_ATTR_MENU_VIEW";
    protected String value;
    protected String[] catalogNames = null;
    protected String maxLength = "1000";
    protected String size = "20";
    protected String styleClass = "wpsEditField";
    protected PortletContext portletContext = null;
    protected PortletRequest portletRequest = null;

    /**
     * @param attribute
     * @param portletURI
     */
    public RelationElement(Attribute attribute, PortletURI portletURI) {
        super(attribute, portletURI);
    }

    public String renderElement() {
        portletRequest = (PortletRequest) pageContext.getAttribute(ITagConstants.VAR_PORTLET_REQUEST);
        portletContext = ((PortletConfig) pageContext.getAttribute(ITagConstants.VAR_PORTLET_CONFIG)).getContext();

        StringBuffer buffer = new StringBuffer();

        if (attribute.isMultiValued()) {
            for (int i = 0; i < attribute.getCurrOccurrence(); i++) {
            	if(isShowLabel())
            		buffer.append(displayLabel());
                buffer.append(displayField(i, getValue(attribute, i)));
            }
        } else {
        	if(isShowLabel())
        		buffer.append(displayLabel());
            buffer.append(displayField(0, getValue(attribute, 0)));
        }

        return buffer.toString();
    }

    public String displayField(long id, Object pValue) {
        String path = null;
        String catalogName = "";
        String itemId = "";
        int index = 0;
        String menuName = null;
        String menuOptionSearch = null;
        String menuOptionBrowse = null;
        String menuOptionView = null;
        NVPair pair = null;
		boolean editable=false;
		
		editable=isEditable();
        path =  getAttributeID(String.valueOf(id));
        attribute.setPath(path);
        pair = (NVPair) pValue;

		String path1 = path.substring(0,path.lastIndexOf('_'));
		addHTMLProperty(JS_EVENT_ONCHANGE,"validateText('"+path1+"')");   

        if (pair != null) {
            catalogName = pair.getName();
            itemId = (String) pair.getValue();

            if (ValidationUtil.isNullOrEmpty(itemId)) {
                itemId = "";
            }
        }

        String imageURL = getImageURI();
        StringBuffer buffer = new StringBuffer();

        String selectName = path + "_V1";
        String textName = path + "_V2";

        //get localised values of menu options
        menuOptionSearch = getMenuOption(LABEL_KEY_MENU_OPTION_SEARCH);
        menuOptionBrowse = getMenuOption(LABEL_KEY_MENU_OPTION_BROWSE);
        menuOptionView = getMenuOption(LABEL_KEY_MENU_OPTION_VIEW);

        buffer.append("<TD width=\"100%\" ALIGN=\"LEFT\">")
              .append(" <TABLE><TR>")
              .append("<TD ALIGN=\"LEFT\">");
			
		if(attribute.isMultiValued() && !editable) {
			buffer.append(displayOccurrenceNumber(id));	
		}                 
          buffer.append("<SELECT id = \"relationshipSelect\" NAME=\"" +  selectName + "\"");
			if(isShowLabel()) buffer.append(getHTMLValue(JS_EVENT_ONCHANGE));
			if(!editable){
				buffer.append(" disabled=\"disabled\" ");
			}
			buffer.append(" >");

		String catName = null;
        if ((catalogNames != null) && (catalogNames.length > 0)) {
            for (int i = 0; i < catalogNames.length; i++) {
				catName = UIUtils.escapeForHTML(catalogNames[i].trim());
				buffer.append("<OPTION VALUE=\"" + catName + "\"");

				if (catName.equals(UIUtils.escapeForHTML(catalogName))) {
                    buffer.append(" SELECTED");
                }

                buffer.append(">");
				buffer.append(catName);
                buffer.append("</OPTION>");
            }
        }

        buffer.append("</SELECT>").append("</TD>");

        menuName = "menu" + selectName;

        buffer.append("<TD ALIGN=\"LEFT\">")
              .append("<INPUT TYPE=\"TEXT\"  id = \"relationshipText\" NAME=\"" + textName + "\"")
              .append("VALUE=\"" + UIUtils.escapeForHTML(itemId) + "\" ");
				if(!editable){
					buffer.append(" disabled=\"disabled\" ");
				}
			if(isShowLabel()) 
				buffer.append(getHTMLValue(JS_EVENT_ONCHANGE));
			buffer.append(" />")
              .append("</TD><TD  ALIGN=\"LEFT\">")
              .append("<table ALIGN=\"LEFT\"><tr>");

			if(editable){
				buffer.append("<td> ");
				if(isShowLabel())
					buffer.append("  <a href=\" # \" onClick= \"openSearch('" +	selectName + "','" + textName + "' " + " );validateText('"+path1+"');\">" );
				else	
					buffer.append("  <a href=\" # \" onClick= \"openSearch(\\'" +	selectName + "\\',\\'" + textName + "\\' " + " );\">"); 
				
				buffer.append("<IMG SRC=\"" + imageURL + "Search.gif\" title=\""+menuOptionSearch +"\" border=\"0\"/></a>")
				.append("</td>")
				.append("<td>");
				if(isShowLabel())
					buffer.append("   <a href=\" # \" onClick= \"openBrowse('" + selectName + "','" + textName + "' " + " );validateText('"+path1+"');\">");
				else
					buffer.append("   <a href=\" # \" onClick= \"openBrowse(\\'" + selectName + "\\',\\' " + textName + "\\' " + " );\">");
				 
				buffer.append("<IMG SRC=\"" + imageURL + "browseItem.gif\" title=\""+menuOptionBrowse +"\" border=\"0\" /></a>")
				.append("</td>");
				}
				buffer.append("<td>");
				  if(showLabel)
					buffer.append(" <a href=\" # \" onClick=\"openViewItem('" + selectName + "','" + textName + "' " + " );validateText('"+path1+"');\">");
				  else
				  	buffer.append(" <a href=\" # \" onClick=\"openViewItem(\\'" + selectName + "\\',\\'" + textName + "\\'" + " );\">");
				  	
				  buffer.append("<IMG SRC=\"" + imageURL + "itemDetails.gif\" title=\""+menuOptionView +"\" border=\"0\"/></a>")
				.append("</td>")
				.append(" </tr>")
				.append("</table>").append("</TD><TD ALIGN=\"LEFT\">");
/*				if(editable){
				buffer.append("<td> ")
				.append("  <a href=\" # \" onClick=\" openSearch(\"" +	selectName + "\",\" " + textName + "\" " + " );\">" + 
				"<IMG SRC=\"" + imageURL + "Search.gif\" title=\""+menuOptionSearch +"\" border=\"0\"/></a>")
				.append("</td>")
				.append("<td>")
				  .append("   <a href=\" # \" onClick=\" openBrowse(\"" + selectName + "\",\" " + textName + "\" " + " );\">" +
				  "<IMG SRC=\"" + imageURL + "browseItem.gif\" title=\""+menuOptionBrowse +"\" border=\"0\"/></a>")
				.append("</td>");
				}
				buffer.append("<td>")
			  .append(" <a href=\" # \" onClick=\" openViewItem(\"" + selectName + "\",\"" + textName + "\"" + " );\">" + 
			  "<IMG SRC=\"" + imageURL + "itemDetails.gif\" title=\""+menuOptionView +"\" border=\"0\"/></a>")
				.append("</td>")
				.append(" </tr>")
				.append("</table>").append("</TD><TD ALIGN=\"LEFT\">");*/

        if (attribute.isMultiValued()&& editable) {
            buffer.append(displayPlusMinus(id));
        }

        buffer.append("</TD></TR></TABLE>").append("</TD></TR>");

        return buffer.toString();
    }

    /**
     * @param LABEL_KEY_MENU_OPTION_SEARCH
     * @return
     */
    private String getMenuOption(String key) {
        String value = null;
        value = UIUtils.getTextLabel(portletContext, portletRequest, key);

        return value;
    }

    protected Object getValue(Attribute attribute, int index) {
        NVPair result = null;
        NVPair[] rvalues = null;
        RelationshipAttribute relationAttrib = (RelationshipAttribute) attribute;

        if (relationAttrib.getCurrentOccurrence() > 0) {
            if (relationAttrib.isMultiValued()) {
                rvalues = relationAttrib.getValues();
                result = rvalues[index];
            }else {
				result = relationAttrib.getValue();
			}
        }else{
			result = relationAttrib.getDefaultValue();
        }

        return result;
    }

    /**
     * @return
     */
    public String getMaxLength() {
        return maxLength;
    }

    /**
     * @return
     */
    public String getSize() {
        return size;
    }

    /**
     * @return
     */
    public String getStyleClass() {
        return styleClass;
    }

    /**
     * @return
     */
    public String getValue() {
        return value;
    }

    /**
     * @param string
     */
    public void setMaxLength(String string) {
        maxLength = string;
    }

    /**
     * @param string
     */
    public void setSize(String string) {
        size = string;
    }

    /**
     * @param string
     */
    public void setStyleClass(String string) {
        styleClass = string;
    }

    public void setValue(String string) {
        value = string;
    }

    /**
     * @return
     */
    public String[] getCatalogNames() {
        return catalogNames;
    }

    /**
     * @param strings
     */
    public void setCatalogNames(String[] strings) {
        catalogNames = strings;
    }
}
