
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.tags.element;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletURI;

import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringEnumAttribute;
import com.ibm.ccd.ui.util.UIUtils;


public class StringEnumerationElement extends BaseElement {
    private PortletContext portletContext = null;
    private PortletRequest portletRequest = null;

    /**
     * @param attribute
     * @param portletURI
     */
    public StringEnumerationElement(Attribute attribute, PortletURI portletURI) {
        super(attribute, portletURI);
    }

    /**
     * @see com.ibm.ccd.wpc.tags.BaseElement#renderElement()
     */
    public String renderElement() {
        portletRequest = (PortletRequest) pageContext.getAttribute(
                "portletRequest");
        portletContext = ((PortletConfig) pageContext.getAttribute(
                "portletConfig")).getContext();

        StringBuffer buffer = new StringBuffer();

        if (attribute.isMultiValued()) {
            for (int i = 0; i < attribute.getCurrOccurrence(); i++) {
            	String path = getAttributeID(String.valueOf(i));
            	String error = attribute.getValidationError(path);
				if(error != null){
					if(isShowLabel())
						buffer.append(displayError(error));
				}
				else{
					if(isShowLabel())
						buffer.append(displayLabel());
				}
                buffer.append(displayField(i, getStringValue(attribute, i)));
            }
        } else {
        	String path = getAttributeID(String.valueOf(0));
        	String error = attribute.getValidationError(path);
			if(error != null){
				if(isShowLabel())
					buffer.append(displayError(error));
			}
			else{
				if(isShowLabel())
		            buffer.append(displayLabel());
			}
            buffer.append(displayField(0, getStringValue(attribute, 0)));
        }

        return buffer.toString();
    }

    /**
     * @param id
     * @param value
     * @return
     */
    public String displayField(long id, Object value) {
		String value1 = UIUtils.escapeForHTML((String)value);
        StringBuffer buffer = new StringBuffer();
        addHTMLProperty(HTMLVALUE, value1);

        String path = getAttributeID(String.valueOf(id));
		String path1 = path.substring(0,path.lastIndexOf('_'));
		addHTMLProperty(JS_EVENT_ONCHANGE,"validateText('"+path1+"')");   
        attribute.setPath(path);
        addHTMLProperty(HTMLID,path);
        addHTMLProperty(HTMLNAME,path);
        buffer.append(HTML_ATTRIBUTE_START);

        if (isEditable()) {
            StringEnumAttribute stringEnumAttribute = (StringEnumAttribute) attribute;
            String[] stringArr = stringEnumAttribute.getEnumValues();
            buffer.append(HTML_SPACE).append(HTML_SELECT)
                  .append(getHTMLValue(HTMLNAME)).append(getHTMLValue(HTMLID))
                  .append(getHTMLValue(HTMLSTYLECLASS));
                  if(isShowLabel())
                  	buffer.append(getHTMLValue(JS_EVENT_ONCHANGE));
                  buffer.append(HTML_CLOSE_TAG)
				  .append(generateHTMLOption(stringArr, value1)).append(HTML_SELECT_CLOSE);

            if (attribute.isMultiValued()) {
            	if(isShowLabel())
            		buffer.append(displayPlusMinus(id));
            }
        } else {
			if(attribute.isMultiValued()) {
				buffer.append(displayOccurrenceNumber(id));	
			}        	
            buffer.append(UIUtils.escapeForHTML(getStringValue(attribute, Integer.parseInt("0"+id))));
        }

        buffer.append(HTML_ATTRIBUTE_END);

        return buffer.toString();
    }

    /**
     * generates the HTML options for HTML Select tag
     *
     * @param values
     * @param selectedValue
     * @return
     */
    protected String generateHTMLOption(String[] values, String selectedValue) {
        StringBuffer buffer = new StringBuffer();
        addHTMLProperty(HTMLVALUE, "");

        if (values != null) {
            //Displays the HTML option value NONE
            buffer.append(generateHTMLNoneOption());

            //Displays the HTML select option values 		
            for (int i = 0; i < values.length; i++) {
				String value = UIUtils.escapeForHTML(values[i].trim());
				addHTMLProperty(HTMLVALUE, value);

                if (value.equals(selectedValue)) {
                    buffer.append(HTML_OPTION_OPEN)
                          .append(getHTMLValue(HTMLVALUE) + " " +
                        HTML_SELECTED + " " + HTML_CLOSE_TAG).append(value)
                          .append(HTML_OPTION_CLOSE);
                } else {
                    buffer.append(HTML_OPTION_OPEN)
                          .append(getHTMLValue(HTMLVALUE) + HTML_CLOSE_TAG)
                          .append(value).append(HTML_OPTION_CLOSE);
                }
            }
        } else {
            buffer.append(generateHTMLNoneOption());
        }

        return buffer.toString();
    }

    /**
     * generate HTML option with none as default selected
     * @return HTML None Option value eg., <option value="">NONE</option>
     */
    protected String generateHTMLNoneOption() {
        StringBuffer buffer = new StringBuffer();

        buffer.append(HTML_OPTION_OPEN).append(HTMLVALUE + HTML_OPEN_QUOTE + "" + HTML_CLOSE_QUOTE + HTML_CLOSE_TAG)
              .append(UIUtils.getTextLabel(portletContext, portletRequest,
                "DROPDOWN_VALUE_NONE")).append(HTML_OPTION_CLOSE);

        return buffer.toString();
    }

    /**
     * @param attribute
     * @param index
     * @return
     */
    protected String getStringValue(Attribute attribute, int index) {
        StringEnumAttribute stringEnumAttrib = (StringEnumAttribute) attribute;

        if (stringEnumAttrib.getCurrentOccurrence() > 0) {
            if (stringEnumAttrib.isMultiValued()) {
                String[] values = stringEnumAttrib.getValues();

                return (values[index] != null) ? values[index] : "";
            } else {
                return (stringEnumAttrib.getValue() != null)
                ? stringEnumAttrib.getValue() : "";
            }
        } else {
            return (stringEnumAttrib.getDefaultValue() != null)
            ? stringEnumAttrib.getDefaultValue() : "";
        }
    }

}
