
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.tags.element;

import org.apache.jetspeed.portlet.PortletURI;

import com.ibm.ccd.ui.dataobjects.NVPair;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.TimeZoneAttribute;
import com.ibm.ccd.ui.util.UIUtils;


public class TimezoneElement extends BaseElement {
    public TimezoneElement(Attribute attribute, PortletURI portletURI) {
        super(attribute, portletURI);
    }

    /**
     * @see com.ibm.ccd.wpc.tags.BaseElement#renderElement()
     */
    public String renderElement() {
        StringBuffer buffer = new StringBuffer();

        if (attribute.isMultiValued()) {
            for (int i = 0; i < attribute.getCurrOccurrence(); i++) {
                buffer.append(displayLabel());
                buffer.append(displayField(i, getStringValue(attribute, i)));
            }
        } else {
            buffer.append(displayLabel());
            buffer.append(displayField(0, getStringValue(attribute, 0)));
        }

        return buffer.toString();
    }

    /**
     * @param attribute
     * @param index
     * @return
     */
    public String displayField(long id, Object value) {
        StringBuffer buffer = new StringBuffer();
        addHTMLProperty(HTMLVALUE, (String) value);

        String path = getAttributeID(String.valueOf(id));
		String path1 = path.substring(0,path.lastIndexOf('_'));
		addHTMLProperty(JS_EVENT_ONCHANGE,"validateText('"+path1+"')");   
        attribute.setPath(path);
        addHTMLProperty(HTMLID,path);
        addHTMLProperty(HTMLNAME,path);
        buffer.append(HTML_ATTRIBUTE_START);

        if (isEditable()) {
            TimeZoneAttribute timezoneAttribute = (TimeZoneAttribute) attribute;
            NVPair[] tzPair = TimeZoneAttribute.getTimeZoneEnum();
            buffer.append(HTML_SPACE).append(HTML_SELECT)
                  .append(getHTMLValue(HTMLID)).append(getHTMLValue(HTMLNAME))
                  .append(getHTMLValue(HTMLSTYLECLASS));
                  if(isShowLabel())
				  	buffer.append(getHTMLValue(JS_EVENT_ONCHANGE));
                  buffer.append(HTML_CLOSE_TAG);

            for (int i = 0; i < tzPair.length; i++) {
                addHTMLProperty(HTMLVALUE, tzPair[i].getName());

                buffer.append(HTML_OPTION_OPEN).append(getHTMLValue(HTMLVALUE));

                if (tzPair[i].getName().equals((String) value)) {
                    buffer.append(HTML_SELECTED);
                }

				buffer.append(HTML_CLOSE_TAG).append(UIUtils.escapeForHTML((String)tzPair[i].getValue()))
                      .append(HTML_OPTION_CLOSE);
            }

            buffer.append(HTML_SELECT_CLOSE);

            if (attribute.isMultiValued()) {
                buffer.append(displayPlusMinus(id));
            }
        } else {
			if(attribute.isMultiValued()) {
				buffer.append(displayOccurrenceNumber(id));	
			}        	
			buffer.append(UIUtils.escapeForHTML((String)getSelectedValue(attribute,
					getStringValue(attribute, id))));
        }

        buffer.append(HTML_ATTRIBUTE_END);

        return buffer.toString();
    }

    /**
     * @param attribute
     * @param index
     * @return
     */
    protected String getStringValue(Attribute attribute, long _index) {
        int index = Integer.parseInt(String.valueOf(_index));
        TimeZoneAttribute tzoneAttrib = (TimeZoneAttribute) attribute;

        if (tzoneAttrib.getCurrentOccurrence() > 0) {
            if (tzoneAttrib.isMultiValued()) {
                String[] values = tzoneAttrib.getValues();

                return (values[index] != null) ? values[index] : "";
            } else {
                return (tzoneAttrib.getValue() != null)
                ? tzoneAttrib.getValue() : "";
            }
        } else {
            return (tzoneAttrib.getDefaultValue() != null)
            ? tzoneAttrib.getDefaultValue() : "";
        }
    }

    /**
     * @param attribute
     * @param key
     * @return
     */
    public String getSelectedValue(Attribute attribute, String key) {
        TimeZoneAttribute tzoneAttrib = (TimeZoneAttribute) attribute;
        NVPair[] nvPair = TimeZoneAttribute.getTimeZoneEnum();

        for (int i = 0; i < nvPair.length; i++) {
            if (nvPair[i].getName().equals(key)) {
                return (String) nvPair[i].getValue();
            }
        }

        return "";
    }
}
