
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.text.CharacterIterator;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.text.StringCharacterIterator;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.StringTokenizer;

import javax.servlet.http.HttpSession;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.User;

import com.ibm.ccd.ui.cache.CacheConstants;
import com.ibm.ccd.ui.cache.CacheManager;
import com.ibm.ccd.ui.cache.CacheNotEnabledException;
import com.ibm.ccd.ui.context.IWPCUser;
import com.ibm.ccd.ui.context.WPCUser;
import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.AttributeTypes;
import com.ibm.ccd.ui.mediator.WPCMediator;
import com.ibm.ccd.ui.portlets.helper.PortletHelper;



public class UIUtils {
    public static final boolean isEmpty(String str) {
        return (str == null) || ("".equals(str.trim()));
    }

    public static int getIntValue(Object intObject) {
        if (intObject == null) {
            throw new IllegalArgumentException();
        }

        if (intObject instanceof String) {
            try {
                return new Integer((String) intObject).intValue();
            } catch (Exception e) {
                throw new IllegalArgumentException();
            }
        }

        ;

        if (intObject instanceof Integer) {
            return ((Integer) intObject).intValue();
        }

        if (intObject instanceof Long) {
            return ((Long) intObject).intValue();
        }

        // get also from the float and double objects
        throw new IllegalArgumentException();
    }

    public static float getFloatValue(Object floatObject) {
        if (floatObject == null) {
            throw new IllegalArgumentException();
        }

        if (floatObject instanceof String) {
            try {
                return new Float((String) floatObject).floatValue();
            } catch (Exception e) {
                throw new IllegalArgumentException();
            }
        }

        ;

        if (floatObject instanceof Integer) {
            return ((Integer) floatObject).floatValue();
        }

        if (floatObject instanceof Float) {
            return ((Float) floatObject).floatValue();
        }

        if (floatObject instanceof Double) {
            return ((Double) floatObject).floatValue();
        }

        throw new IllegalArgumentException();
    }

    public static double getDoubleValue(Object doubleObject) {
        if (doubleObject == null) {
            throw new IllegalArgumentException();
        }

        if (doubleObject instanceof String) {
            try {
                return new Double((String) doubleObject).doubleValue();
            } catch (Exception e) {
                throw new IllegalArgumentException();
            }
        }

        ;

        if (doubleObject instanceof Double) {
            return ((Double) doubleObject).doubleValue();
        }

        if (doubleObject instanceof Float) {
            return ((Float) doubleObject).floatValue();
        }

        if (doubleObject instanceof Integer) {
            return ((Integer) doubleObject).floatValue();
        }

        throw new IllegalArgumentException();
    }

    public static boolean getBooleanValue(Object booleanObject) {
        if (booleanObject == null) {
            throw new IllegalArgumentException();
        }

        if (booleanObject instanceof Boolean) {
            return ((Boolean) booleanObject).booleanValue();
        }

        if (booleanObject instanceof String) {
            try {
                return new Boolean((String) booleanObject).booleanValue();
            } catch (Exception e) {
                throw new IllegalArgumentException();
            }
        }

        ;

        throw new IllegalArgumentException();
    }

    public static boolean getFlagValue(Object flagObject) {
        if (flagObject == null) {
            throw new IllegalArgumentException();
        }

        if (flagObject instanceof Boolean) {
            return ((Boolean) flagObject).booleanValue();
        }

        // negative,0 is false any positive num is true;
        try {
            int i = new Integer(flagObject.toString()).intValue();

            return (i <= 0) ? false : true;
        } catch (Exception e) {
            throw new IllegalArgumentException();
        }
    }

    public static Date getDateValue(String dateStr) {
        return DateHelper.getDate(dateStr);
    }

    public static String getDateString(Date date) {
        return DateHelper.formatDate(date);
    }

    public static String getDateTimeString(Date date) {
        return DateHelper.formatDateTime(date);
    }

    public static String getFullDateTimeString(Date date) {
        return DateHelper.formatFullDateTime(date);
    }

    //	String to int array conversion utitlies		
    public static int[] getIntArray(String[] stringArray) {
        if ((stringArray == null) || (stringArray.length == 0)) {
            return null;
        }

        int intarraySize = stringArray.length;
        int[] intArray = new int[intarraySize];

        for (int i = 0; i < intarraySize; ++i) {
            intArray[i] = getIntValue(stringArray[i]);
        }

        return intArray;
    }

    //	String to double array conversion utitlies		
    public static double[] getDoubleArray(String[] stringArray) {
        if ((stringArray == null) || (stringArray.length == 0)) {
            return null;
        }

        int doublearraySize = stringArray.length;
        double[] doubleArray = new double[doublearraySize];

        for (int i = 0; i < doublearraySize; ++i) {
            doubleArray[i] = getDoubleValue(stringArray[i]);
        }

        return doubleArray;
    }

    // ArrayList to array conversion utitlies		
    public static int[] getIntArray(ArrayList arrayList) {
        if ((arrayList == null) || (arrayList.size() == 0)) {
            return null;
        }

        int arraySize = arrayList.size();
        int[] intArray = new int[arraySize];

        for (int i = 0; i < arraySize; ++i) {
            intArray[i] = getIntValue(arrayList.get(i));
        }

        return intArray;
    }

    //	ArrayList to array conversion utitlies		
    public static double[] getDoubleArray(ArrayList arrayList) {
        if ((arrayList == null) || (arrayList.size() == 0)) {
            return null;
        }

        int arraySize = arrayList.size();
        double[] doubleArray = new double[arraySize];

        for (int i = 0; i < arraySize; ++i) {
            doubleArray[i] = getDoubleValue(arrayList.get(i));
        }

        return doubleArray;
    }

    //	ArrayList to array conversion utitlies		
    public static String[] getStringArray(ArrayList arrayList) {
        if ((arrayList == null) || (arrayList.size() == 0)) {
            return null;
        }

        int arraySize = arrayList.size();
        String[] stringArray = new String[arraySize];

        for (int i = 0; i < arraySize; ++i) {
            stringArray[i] = ((arrayList.get(i) != null)
                ? arrayList.get(i).toString() : null);
        }

        return stringArray;
    }

    public static HashMap stringArrayToHashMap(String[] array) {
        HashMap hashMap = new HashMap();

        if (array == null) {
            return hashMap;
        }

        for (int i = 0; i < array.length; i++) {
            if (array[i] != null) {
                hashMap.put(array[i], array[i]);
            }
        }

        return hashMap;
    }

    public static String[] getNonEmptyStringArray(String[] values) {
        ArrayList strArrayList = null;

        if ((values != null) && (values.length > 0)) {
            strArrayList = new ArrayList(values.length);

            for (int i = 0; i < values.length; ++i) {
                if (!UIUtils.isEmpty(values[i])) {
                    strArrayList.add(values[i].trim());
                }
            }
        }

        return (strArrayList != null) ? UIUtils.getStringArray(strArrayList)
                                      : null;
    }

    /**
     * Method for extracting Label from resource Bundle
     * @param portletRequest
     * @param key
     * @return
     * @throws Exception
     */
    public static String getTextLabel(PortletContext context,
        PortletRequest portletRequest, String key) {
        String strTextLabel = "";

        strTextLabel = getTextLabel(context, portletRequest.getLocale(), key);

        return strTextLabel;
    }

    /**
     * Method for extracting Label from resource Bundle
     * @param portletRequest
     * @param key
     * @return
     * @throws Exception
     */
    public static String getTextLabel(PortletContext context, Locale locale,
        String key) {
        String strTextLabel = "";

        try {
            strTextLabel = context.getText(IReference.BUNDLE_WPC_UI_LABELS,
                    key, locale);
        } catch (MissingResourceException mm) {
            context.getLog().warn(context.getText(
                    IReference.BUNDLE_WPC_UI_MESSAGES, "MESSAGE_CODE_54", locale) +
                ":" + key);
            mm.printStackTrace();
        }

        return strTextLabel;
    }

    /**
     * Method for extracting Label from resource Bundle
     * @param portletRequest
     * @param key
     * @return
     * @throws Exception
     */
    public static String getTextMessages(PortletContext context,
        PortletRequest portletRequest, String key) {
        String strTextMessage = "";
        strTextMessage = getTextMessages(context, portletRequest.getLocale(),
                key);

        return strTextMessage;
    }

    /**
     * Method for extracting Label from resource Bundle
     * @param portletRequest
     * @param key
     * @return
     * @throws Exception
     */
    public static String getTextMessages(PortletContext context, Locale locale,
        String key) {
        String strTextLabel = "";

        try {
            strTextLabel = context.getText(IReference.BUNDLE_WPC_UI_MESSAGES,
                    key, locale);
        } catch (MissingResourceException mm) {
            context.getLog().warn(context.getText(
                    IReference.BUNDLE_WPC_UI_MESSAGES, "MESSAGE_CODE_54", locale) +
                ":" + key);
            mm.printStackTrace();
        }

        return strTextLabel;
    }
    
    /**
     * Method Basically filters the other locale attribute.Checks whether the passed attribute is localized attribute 
     * and if it is localized then checks whether the passed locale matches with the attribute locale.   
	 * @param attributeForDisplay
	 * @param localeName
	 * @return
	 */
	public static boolean isAttributeDisplayableForLocale(Attribute attributeForDisplay, String localeName){
    	boolean isDisplayable = false;
    	if(null == attributeForDisplay){
    		return isDisplayable;
    	}
		if(attributeForDisplay.isLocalized()){
			return isDisplayable;
		}    	
		if(isEmpty(attributeForDisplay.getLocaleName())){
			return true;
		}
		if(attributeForDisplay.getLocaleName().equalsIgnoreCase(localeName)){
			isDisplayable = true;
		}else{
			isDisplayable = false;
		}
	   	return isDisplayable;
    }

    //	return the date as a string yyyymmddhhmmss General Time String in LDAP 
    public static List tokenizeString(String inputStr, String tokenizer) {
        StringTokenizer strtoken = new StringTokenizer(inputStr, tokenizer);
        ArrayList arrList = new ArrayList();

        while (strtoken.hasMoreTokens()) {
            arrList.add(strtoken.nextToken());
        }

        return arrList;
    }

    public static String getCommaSeparatedId(Set transkeyset) {
        Iterator iter = transkeyset.iterator();
        StringBuffer commanSepkeys = new StringBuffer();

        while (iter.hasNext()) {
            commanSepkeys.append(iter.next());

            if (iter.hasNext()) {
                commanSepkeys.append(",");
            }
        }

        return commanSepkeys.toString();
    }

    public static String getCommaSeparatedId(String[] values) {
        StringBuffer buffer = new StringBuffer();

        if (values != null) {
            int size = values.length;

            if (size > 1) {
                for (int i = 0; i < (values.length - 1); i++) {
                    buffer.append(values[i] + ",");
                }

                buffer.append(values[size - 1]);
            } else if (size == 1) {
                buffer.append(values[0]);
            }
        }

        return buffer.toString();
    }

    public static String replaceSlash(String inputString) {
        inputString = inputString.replace('/', '_');

        return inputString;
    }

    public static String replaceEmptyStr(String inputString) {
        inputString = inputString.replace(' ', '_');

        return inputString;
    }

    public static ArrayList getStringArrayList(String[] StringObj) {
        ArrayList list = new ArrayList();

        for (int i = 0; i < StringObj.length; i++) {
            String value = new String(StringObj[i]);
            list.add(value);
        }

        return list;
    }

    /**
     * @param inputStream
     * @param completeFilePath
     * @return
     */
    public static boolean saveFileStream(InputStream inputStream,
        String completeFilePathtoSave) {
        boolean success = true;
        FileOutputStream outputstream = null;

        try {
            File filePath = new File(completeFilePathtoSave);

            if (!filePath.getParentFile().exists()) {
                if (!filePath.getParentFile().mkdir()) {
                    return false;
                }
            }

            outputstream = new FileOutputStream(completeFilePathtoSave);

            int singleByte;

            while ((singleByte = inputStream.read()) != -1) {
                outputstream.write(singleByte);
            }
        } catch (Exception e1) {
            e1.printStackTrace();
            success = false;
        } finally {
            if (outputstream != null) {
                try {
                    outputstream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }

        return success;
    }

    public static String formatNumber(String str, Locale locale) {
    	
        String number = str;
        /*
         * For Fix : WPC00028798
         */
        if(!isEmpty(str)){
        	
        	NumberFormat formatter = NumberFormat.getInstance(locale);
        	formatter.setGroupingUsed(false);
        	formatter.setMinimumFractionDigits(2);
        	formatter.setMaximumFractionDigits(3);
        	formatter.setMaximumIntegerDigits(30);
        	number = formatter.format(getDoubleValue(str));
        } 
        
        return number;
    }

	public static String formatInteger(String str,Locale locale) {
		
		String number = str;
		/*
         * For Fix : WPC00028798
         */
		if(!isEmpty(str)){
			
			NumberFormat formatter = NumberFormat.getInstance(locale);
			formatter.setGroupingUsed(false);
			formatter.setMaximumIntegerDigits(20);
			number = formatter.format(getIntValue(str));
		}
		
		return number;
	}
	
	public static String parseInteger(String str, Locale locale) 
		throws ParseException{
		
		/*
         * For Fix : WPC00028798
         */
		if(isEmpty(str))
			return str;
		
		NumberFormat formatter = NumberFormat.getInstance(locale);
		formatter.setGroupingUsed(false);
		formatter.setMaximumIntegerDigits(20);
		formatter.setMinimumFractionDigits(0);
		
		try {
			str = formatter.parse(str).toString();
			str = Math.round(Double.parseDouble(str)) +"";
		} catch (ParseException e) {
			throw e;
		}
		return str;		
	}
	
    public static String parseNumber(String str, Locale locale) 
    	throws ParseException {
    	
		/*
         * For Fix : WPC00028798
         */
		if(isEmpty(str))
			return str;
    	
        NumberFormat formatter = NumberFormat.getInstance(locale);
        formatter.setGroupingUsed(false);
		formatter.setMinimumFractionDigits(2);
		formatter.setMaximumFractionDigits(3);
		formatter.setMaximumIntegerDigits(30);
		
        try {
            str = formatter.parse(str).toString();
        } catch (ParseException e) {
        	throw e;
        }
        return str;
    }

    public static String[] parseNumber(String[] str, Locale locale) {
        NumberFormat formatter = NumberFormat.getInstance(locale);
		formatter.setGroupingUsed(false);
		formatter.setMinimumFractionDigits(2);
		formatter.setMaximumFractionDigits(3);
		formatter.setMaximumIntegerDigits(30);
		
        String[] str1 = null;

        if (str != null) {
            str1 = new String[str.length];

            for (int i = 0; i < str.length; i++) {
                try {
                    str1[i] = formatter.parse(str[i]).toString();
                } catch (ParseException e) {
                	str1[i] = str[i];
                    e.printStackTrace();
                }
            }
        }

        return str1;
    }

    public static Object[] copy(Object[] orig) {
        Object[] tar = new Object[orig.length];

        for (int i = 0; i < orig.length; i++) {
            tar[i] = copy(orig[i]);
        }

        return tar;
    }

    public static Object copy(Object orig) {
        Object obj = null;

        try {
            // Write the object out to a byte array
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            ObjectOutputStream out = new ObjectOutputStream(bos);
            out.writeObject(orig);
            out.flush();
            out.close();

            // Make an input stream from the byte array and read
            // a copy of the object back in.
            ObjectInputStream in = new ObjectInputStream(new ByteArrayInputStream(
                        bos.toByteArray()));
            obj = in.readObject();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException cnfe) {
            cnfe.printStackTrace();
        }

        return obj;
    }

    public static String getLookupTableName(WPCUser wpcUser){
    	return (String) wpcUser.getProperty(IWPCUser.KEY_WPC_CATALOGUE_NAME);
    }
    /**
     * @param wpcUser
     * @return
     */
    public static String getCatalogName(WPCUser wpcUser) {
        String catalogName = null;
        //code modified to get catalog name from the cache
        // Fix for Bug No WPC00031094
		CacheManager cManager = null;
					
		try {
			cManager=CacheManager.getInstance();
		} catch (CacheNotEnabledException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.out.println(wpcUser.getProperty(IWPCUser.KEY_USER_ID));
		String catalogNameKey = CacheConstants.CATALOG_NAME;
		String key = (String) wpcUser.getProperty(IWPCUser.KEY_USER_ID);
		if (CacheManager.isCacheEnabled()) {
			catalogNameKey=(String)cManager.getFromCache(catalogNameKey, key);
			System.out.println("on loine 703"+catalogNameKey);
				if(catalogNameKey==null){
					catalogNameKey = WPCMediator.getInstance().getDefaultCatalog();
					System.out.println("on loine 703"+catalogNameKey);
				}
				} else {
			catalogNameKey = (String) wpcUser.getProperty(IWPCUser.KEY_WPC_CATALOGUE_NAME);
			System.out.println("on loine 708"+catalogNameKey);
        }
        return catalogNameKey;
        // Fix for Bug No WPC00031094
    }
    
    public static String getCatalogNode(WPCUser wpcUser) {
        String catalogNode = null;
		// Fix for Bug No WPC00031094
		CacheManager cManager = null;
					
		 try {
			 cManager=CacheManager.getInstance();
		 } catch (CacheNotEnabledException e) {
			 // TODO Auto-generated catch block
			 e.printStackTrace();
		 }
		System.out.println(wpcUser.getProperty(IWPCUser.KEY_USER_ID));
		
		String catalogNodeKey = CacheConstants.CATALOG_NODE;
		String key = (String) wpcUser.getProperty(IWPCUser.KEY_USER_ID);
		
		 if (CacheManager.isCacheEnabled()) {
				// String currentCatalogId=(String)cManager.getFromCache(catalogNameKey, key);
			catalogNodeKey = (String)cManager.getFromCache(catalogNodeKey, key);
				 System.out.println("catalogNode at UiUtils"+catalogNodeKey);
			 } else {
			catalogNodeKey = (String) wpcUser.getProperty(IWPCUser.KEY_WPC_CATALOGUE_NODE);
        }

        return catalogNodeKey;
		// Fix for Bug No WPC00031094
    }

    public static String[] getAvailabelCatalogs(WPCUser wpcUser) {
        String[] catalogs = null;

        if (wpcUser != null) {
            catalogs = (String[]) wpcUser.getProperty(IWPCUser.KEY_AVAILABEL_CATALOGS);
        }

        return catalogs;
    }
    
	public static WPSContext getWPSContext(PortletRequest request,PortletConfig portletConfig ){
		 
		  PortletContext portletContext=null;
		  WPSContext context = null;
		  portletContext=portletConfig.getContext();
		  context = getWPSContext(request,portletContext);
		  return context;

	  }

	/**
	 * @param portletRequest
	 * @param portletContext
	 * @return
	 */
	public static WPSContext getWPSContext(PortletRequest portletRequest, PortletContext portletContext) {
			WPSContext context = null;
			PortletHelper phelper = new PortletHelper();
			phelper.setPortletContext(portletContext);
			context = phelper.getWPSContext(portletRequest);
			return context;
	}
	/**
	 * @param attribute
	 * @return
	 */
	public static boolean getEditableAttributeCheckforMass(Attribute attribute) {
		switch (attribute.getType()) {
		case AttributeTypes.STRING:
		case AttributeTypes.CURRENCY:
		case AttributeTypes.INTEGER:
		case AttributeTypes.NUMBER:
		case AttributeTypes.IMAGE_URL:
		case AttributeTypes.THUMB_NAIL_IMAGE_URL:
		case AttributeTypes.URL:
		case AttributeTypes.FLAG:
		case AttributeTypes.DATE:
		case AttributeTypes.STRING_ENUMERATION:
		case AttributeTypes.RELATIONSHIP:
		case AttributeTypes.NUMBER_ENUMERATION:
		case AttributeTypes.LOOKUP_TABLE:
		case AttributeTypes.TIMEZONE:
			return true;
		
		case AttributeTypes.IMAGE:
		case AttributeTypes.THUMB_NAIL_IMAGE:
		case AttributeTypes.UNSUPPORTED:
		case AttributeTypes.BINARY:
		case AttributeTypes.SEQUENCE:
		case AttributeTypes.PASSWORD:
			return false;
		}
		
		return false;
	}
	

	/**
	 * Replaces String with HTML escape character   
	 * @param str
	 * @return
	 */
	public static String escapeForHTML(String str) {
		if(UIUtils.isEmpty(str)) return "";

		StringBuffer result = new StringBuffer();
		StringCharacterIterator	iterator = new StringCharacterIterator(str);
		
		   char character =  iterator.current();
		   while (character != CharacterIterator.DONE ){
			 if (character == '&') {
				result.append("&#38;");
			 }
			 else if (character == '<') {
			   result.append("&#60;");
			 }
			 else if (character == '>') {
			   result.append("&#62;");
			 }
			 else 
			 if (character == '\"') {
			   result.append("&#34;");
			 }
			 else if (character == '\'') {
			   result.append("&#39;");
			 }
			 else if (character == '\\') {
				result.append("&#92;");
			 }
			 else if (character == '!') {
				result.append("&#33;");
			 }
			 else if (character == '@') {
				result.append("&#64;");
			 }
			 
			 else if (character == '#') {
				result.append("&#35;");
			 }
			 else if (character == '$') {
				result.append("&#36;");
			 }
			 
			 else if (character == '%') {
				result.append("&#37;");
			 }
			 else if (character == ';') {
				result.append("&#59;");
			 }
			 else if (character == ':') {
				result.append("&#58;");
			 }
			 else {
			   //the char is not a special one
			   //add it to the result as is
			   result.append(character);
			 }
			 character = iterator.next();
		   }
		   return result.toString();	
	}
	
	/**
	 * Replaces HTML escape character to normal Ascii char
	 * @param str
	 * @return
	 */
	public static String unescapeForHTML(String str) {
		if(UIUtils.isEmpty(str)) 
			return "";
		else 
			return str.replaceAll("&amp;lt;","<").replaceAll("&amp;gt;",">").replaceAll("&gt;",">").replaceAll("&lt;","<").replaceAll("&amp;","&").replaceAll("&#39;","'");
	}

	/**
	 * Replaces HTML escape character to normal Ascii char
	 * @param str
	 * @return
	 */
	public static String[] unescapeForHTML(String str[]) {
		if(str == null || str.length == 0) 
			return null;
		else {
			for(int i=0;i<str.length;i++) {
				str[i] = unescapeForHTML(str[i]); 
			}
		}
		return str;	
	}	

	/**
	 * Replaces all double quote char with double-double qoute and
	 * enclose the string with double quote otherwise the string itself   
	 * @param str
	 * @return
	 */
	public static String escapeForCSV(String str) {
		boolean hasSpecialChar = false;
		if (str != null) {
			if (str.indexOf('\"') != -1) {
				str = str.replaceAll("\"", "\"\"");
				hasSpecialChar = true;
			}
			if (str.indexOf(',') != -1) {
				hasSpecialChar = true;
			}
			if (hasSpecialChar) {
				str = "\"" + str + "\"";
			}
		}
		return str;
	}
	
	
	
	/**
	 * Formats the date received in Calendat format to a date in WPC format
	 * 
	 * @param date_str	date value entered by user
	 * @param locale	Locale for which the date needs to be formatted
	 * @return	a string representation of the date formatted in WPC format
	 */
	public static String formatWPCDate(String date_str,Locale locale) {
		//Convert the date from WPC format to the custom format to be displayed to the user
		if(date_str.equals(""))
			return date_str;

		Date date = null;
		ResourceBundle bundle =	ResourceBundle.getBundle(IReference.BUNDLE_WPC_UI_LABELS,locale);
		String wpcFormat = bundle.getString("WPC.DATE.FORMAT");
		wpcFormat = changeDateFormatCase(wpcFormat);
		String customFormat = bundle.getString("CALENDAR.DATE.FORMAT");
		customFormat = changeDateFormatCase(customFormat);
		
		SimpleDateFormat df = new SimpleDateFormat(customFormat,locale);
		try{
			date = df.parse(date_str);
			}catch(ParseException pe){
				System.err.println("ParseException " + pe);
			}
			SimpleDateFormat wpcf = new SimpleDateFormat(wpcFormat,locale);
			
			String	wpcDate = wpcf.format(date);
			return wpcDate;
	}
	
	/**
	 * The javascript date format used in calendarCtrl.js specifies the month as "mm", minutes as "MM", 
	 * and seconds as "SS". In java SimpleDateFormat class the month must be represented as "MM", minutes as "mm", 
	 * and seconds as "ss". 
	 * @param dateFormat Date Format as specified in wpclabels.properties
	 * @return a modified date format after changing the case of month, minute and seconds field
	 */
	public static String changeDateFormatCase(String dateFormat){
		dateFormat = dateFormat.replace('S','s');
		dateFormat = dateFormat.replaceAll("mm","MM");
		dateFormat = dateFormat.replaceAll(":MM",":mm");
		return dateFormat;
	}
}


class DateHelper {
    public static boolean DEBUG = false;
    public static final long DAY_SEC = 86400000; // number of seconds per day 24x60x60
    static final DateFormat simpleDateFormatter = new SimpleDateFormat(
            "yyyy-MM-dd");
    static final DateFormat simpleDateTimeFormatter = new SimpleDateFormat(
            "yyyy-MM-dd HH:mm:ss");
    static final DateFormat fullDateTimeFormatter = new SimpleDateFormat(
            "yyyy-MM-dd HH:mm:ss SSS");

    public static String formatDate(Date date) {
        if (date == null) {
            throw new IllegalArgumentException("Invalid Date");
        }

        return simpleDateFormatter.format(date);
    }

    public static String formatDateTime(Date date) {
        if (date == null) {
            throw new IllegalArgumentException("Invalid Date");
        }

        return simpleDateTimeFormatter.format(date);
    }

    public static String formatFullDateTime(Date date) {
        if (date == null) {
            throw new IllegalArgumentException("Invalid Date");
        }

        return fullDateTimeFormatter.format(date);
    }

    // Generalized Time Syntax methods
    public static Date getDate(String date) {
        try {
            return simpleDateFormatter.parse(date);
        } catch (ParseException pe) {
            if (DEBUG) {
                System.out.println("DateHelper_getTime() :- Error: " +
                    pe.toString());
            }

            return null;
        }
    }

    public static Date getDateTime(String dateTime) {
        try {
            return simpleDateTimeFormatter.parse(dateTime);
        } catch (ParseException pe) {
            if (DEBUG) {
                System.out.println("DateHelper_getTime() :- Error: " +
                    pe.toString());
            }

            return null;
        }
    }

    public static Date getFullDateTime(String dateTime) {
        try {
            return fullDateTimeFormatter.parse(dateTime);
        } catch (ParseException pe) {
            if (DEBUG) {
                System.out.println("DateHelper_getTime() :- Error: " +
                    pe.toString());
            }

            return null;
        }
    }

    public static boolean comapareDates(String dateStr1, String dateStr2) {
        if ((dateStr1 == null) || (dateStr2 == null)) {
            throw new IllegalArgumentException();
        }

        Date date1 = getDateTime(dateStr1);

        if (DEBUG) {
            System.out.println("date1 : " + dateStr1);
            System.out.println("date2:" + dateStr2);
            System.out.println("diff : '" +
                date1.compareTo(getDateTime(dateStr2)));
        }

        if (date1.compareTo(getDateTime(dateStr2)) >= 0) {
            return true;
        }

        return false;
    }

    // return the date as a string yyyymmddhhmmss General Time String in LDAP 
    public static String extendTime(String genTime, int days) {
        // gen time  is in the form of "20010706080000". get year,
        // month, day, hour, minute, and second from the genTime
        int year = Integer.parseInt(genTime.substring(0, 4));
        int mon = Integer.parseInt(genTime.substring(4, 6));
        int day = Integer.parseInt(genTime.substring(6, 8));
        int hour = Integer.parseInt(genTime.substring(8, 10));
        int minute = Integer.parseInt(genTime.substring(10, 12));
        int second = Integer.parseInt(genTime.substring(12, 14));

        Calendar genTimeCal = Calendar.getInstance();

        // set calendar to the time
        genTimeCal.set(year, mon - 1, day, hour, minute, second);

        // add days to the expiration Time
        genTimeCal.add(Calendar.DAY_OF_MONTH, days);

        Date date = genTimeCal.getTime();

        return simpleDateTimeFormatter.format(date);
    }

    /**
     * @param date1
     * @param date2
     * @return
     */
    public static int getDaysDifference(Date date1, Date date2) {
        long diff = (date1.getTime() - date2.getTime()) / DAY_SEC;

        return (int) diff;
    }
    
  
}
