/*
 * Created on Jun 19, 2006
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package com.ibm.ccd.ui.validator;

import java.math.BigInteger;
import java.text.MessageFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;

/**
 * @author Mohit
 *
 * ValidationHelper class to validate various WPC data types such as 
 * Number, Integer, Currency, Date, Period, URL and Image.
 * 
 */
public class ValidationHelper implements IValidatorMessage {
	
	/**
	 * 
	 * @param attribute		Attribute name with the occurence indicator, if any
	 * @param value			value to be validated
	 * @param precision		maximum number of digits allowed in the fraction portion
	 * @param locale		locale of the user
	 * @param error			MessageMap in which the error message,if any, is to be added
	 * @return	a string containing the error message,if any, or null
	 */
	public static String validateNumber(String attribute, String value, int precision,Locale locale,MessageMap error){
		
		if(value == null || value.equals("")){
        	return null;
        }
		
		NumberFormat nft = NumberFormat.getInstance(locale);
		nft.setGroupingUsed(false);
		/*
		 * Need to pass the parse position to parse method as the parse method does not throw an exception
		 * for invalid numbers but instead stops parsing at the first instance of a wrong character 
		 */
		ParsePosition pos = new ParsePosition(0);
		//First parse the input number to check if it is a valid number
		
		//NumberEnumeration values were entered with a leading space causing the validation to fail
		value = value.trim();
        Number num = nft.parse(value, pos);
        
        if (value.length() != pos.getIndex())
        {
        	error.addErrorMessage(IValidatorMessage.MSG_INVALID_NUMBER,attribute,value);
        	Object[] values = {attribute,value};
        	System.err.println("Invalid Number type " + attribute + " " + value);
        	return createErrorMessage(IValidatorMessage.MSG_INVALID_NUMBER,locale,values);
        }
        
        /*
         * Now format the valid number with the format set above
         * and compare with the Double value with that of the input number  
         */
        NumberFormat defaultFormat = NumberFormat.getInstance(Locale.ENGLISH);
        defaultFormat.setMaximumIntegerDigits(15);
        defaultFormat.setMaximumFractionDigits(precision);
        defaultFormat.setGroupingUsed(false);
        
        String formattedNumber = defaultFormat.format(num.doubleValue());
        if(Double.valueOf(formattedNumber).compareTo(Double.valueOf(num.toString())) == 0)
        	return null;
        else{
        	error.addErrorMessage(IValidatorMessage.MSG_INVALID_NUMBER,attribute,value);
        	Object[] values = {attribute,value};
        	System.err.println("Invalid Number type " + attribute + " " + value + " formattedNumber " + formattedNumber);
        	return createErrorMessage(IValidatorMessage.MSG_INVALID_NUMBER,locale,values);
        }
        	
	}
	
	/**
	 * 
	 * @param attribute		Attribute name with the occurence indicator, if any
	 * @param value			value to be validated
	 * @param precision		maximum number of digits allowed in the fraction portion
	 * @param locale		locale of the user
	 * @param error			MessageMap in which the error message,if any, is to be added
	 * @return	a string containing the error message,if any, or null
	 */
	public static String validateCurrency(String attribute, String value, int precision,Locale locale,MessageMap error){
		
		if(value == null || value.equals("")){
        	return null;
        }
		
		NumberFormat nft = NumberFormat.getInstance(locale);
		nft.setGroupingUsed(false);
		/*
		 * Need to pass the parse position to parse method as the parse method does not throw an exception
		 * for invalid numbers but instead stops parsing at the first instance of a wrong character 
		 */
		ParsePosition pos = new ParsePosition(0);
		//First parse the input number to check if it is a valid number
        Number num = nft.parse(value, pos);
        
        if (value.length() != pos.getIndex())
        {
        	error.addErrorMessage(IValidatorMessage.MSG_INVALID_CURRENCY,attribute,value);
        	Object[] values = {attribute,value};
        	System.err.println("Invalid Currency type " + attribute + " " + value);
        	return createErrorMessage(IValidatorMessage.MSG_INVALID_CURRENCY,locale,values);
        }
        /*
         * Now format the valid number with the format set above
         * and compare with the Double value with that of the input number 
         */
        NumberFormat defaultFormat = NumberFormat.getInstance(Locale.ENGLISH);
        defaultFormat.setMaximumIntegerDigits(15);
        defaultFormat.setMaximumFractionDigits(precision);
        defaultFormat.setGroupingUsed(false);
        
        String formattedNumber = defaultFormat.format(num.doubleValue());
        if(Double.valueOf(formattedNumber).compareTo(Double.valueOf(num.toString())) == 0)
        	return null;
        else{
        	error.addErrorMessage(IValidatorMessage.MSG_INVALID_CURRENCY,attribute,value);
        	Object[] values = {attribute,value};
        	System.err.println("Invalid Cuurency type " + attribute + " " + value + " formattedNumber " + formattedNumber);
        	return createErrorMessage(IValidatorMessage.MSG_INVALID_CURRENCY,locale,values);
        }
        	
	}
	
	/**
	 * 
	 * @param attribute		Attribute name with the occurence indicator, if any
	 * @param value			value to be validated
	 * @param locale		locale of the user
	 * @param error			MessageMap in which the error message,if any, is to be added
	 * @return	a string containing the error message,if any, or null
	 */
	public static String validateInteger(String attribute, String value, Locale locale,MessageMap error){
		
		if(value == null || value.equals("")){
        	return null;
        }
		
		NumberFormat nft = NumberFormat.getIntegerInstance(locale);
		nft.setGroupingUsed(false);
		/*
		 * Need to pass the parse position to parse method as the parse method does not throw an exception
		 * for invalid numbers but instead stops parsing at the first instance of a wrong character
		 */
		ParsePosition pos = new ParsePosition(0);
		//Parse the input value to check if it is a valid integer
        Number num = nft.parse(value, pos);
        
        if (value.length() != pos.getIndex())
        {
        	error.addErrorMessage(IValidatorMessage.MSG_INVALID_INTEGER,attribute,value);
        	Object[] values = {attribute,value};
        	System.err.println("Invalid Integer type " + attribute + " " + value);
        	return createErrorMessage(IValidatorMessage.MSG_INVALID_INTEGER,locale,values);
        }
        try{
        //Restrict the valid values to integers by creating a BigInteger object	
        BigInteger bigInt = new BigInteger(num.toString());
        }catch(NumberFormatException nfe){
        	error.addErrorMessage(IValidatorMessage.MSG_INVALID_INTEGER,attribute,value);
        	Object[] values = {attribute,value};
        	System.err.println("Invalid Integer type " + attribute + " " + value);
        	return createErrorMessage(IValidatorMessage.MSG_INVALID_INTEGER,locale,values);
        }
        return null;
		
	}
	
	/**
	 * 
	 * @param attribute		Attribute name with the occurence indicator, if any
	 * @param date			value to be validated
	 * @param locale		locale of the user
	 * @param error			MessageMap in which the error message,if any, is to be added
	 * @return	a string containing the error message,if any, or null
	 */
	public static String validateDate(String attribute, String date, Locale locale,MessageMap error){
		ResourceBundle bundle =	ResourceBundle.getBundle(IReference.BUNDLE_WPC_UI_LABELS,locale);
		String customFormat = bundle.getString("CALENDAR.DATE.FORMAT");
		// Modify the javascript date format to java date format
		String javaCustomFormat = UIUtils.changeDateFormatCase(customFormat);
		SimpleDateFormat df = new SimpleDateFormat(javaCustomFormat,locale);
		df.setLenient(false);
		try{
        Date value = df.parse(date);
        }catch(ParseException pe){
        	error.addErrorMessage(IValidatorMessage.MSG_INVALID_DATE,attribute,date,customFormat);
        	Object[] values = {attribute,date,customFormat};
        	System.err.println("Error in validateDate " + date);
        	return createErrorMessage(IValidatorMessage.MSG_INVALID_DATE,locale,values);
        }
        
		return null;
	}
	
	/**
	 * 
	 * @param attribute		Attribute name with the occurence indicator, if any
	 * @param fromValue		value of From Date
	 * @param toValue		value of To Date
	 * @param locale		locale of the user
	 * @param error			MessageMap in which the error message,if any, is to be added
	 * @return	a string containing the error message,if any, or null 
	 */
	public static String validatePeriod(String attribute, String fromValue, String toValue,Locale locale,MessageMap error){
		ResourceBundle bundle =	ResourceBundle.getBundle(IReference.BUNDLE_WPC_UI_LABELS,locale);
		String customFormat = bundle.getString("CALENDAR.DATE.FORMAT");
		// Modify the javascript date format to java date format
		customFormat = UIUtils.changeDateFormatCase(customFormat);
		SimpleDateFormat df = new SimpleDateFormat(customFormat,locale);
		df.setLenient(false);
		try{
        Date fromDate = df.parse(fromValue);
        Date toDate = df.parse(toValue);
        if(fromDate.compareTo(toDate) != -1){
        	error.addErrorMessage(IValidatorMessage.MSG_INVALID_PERIOD,attribute,fromValue,toValue);
        	Object[] values = {attribute,fromValue,toValue};
        	
        	System.err.println("Error in validatePeriod " + fromValue + " " + toValue);
        	return createErrorMessage(IValidatorMessage.MSG_INVALID_PERIOD,locale,values);
        }
        else
        	return null;
        }catch(ParseException pe){
        	System.err.println("ParseException in validatePeriod " + pe);
        	return createErrorMessage(IValidatorMessage.MSG_INVALID_PERIOD,locale,null);
        }
	}
	
	/**
	 * 
	 * @param attribute		Attribute name with the occurence indicator, if any
	 * @param url			url to be validated
	 * @param locale		locale of the user
	 * @param error			MessageMap in which the error message,if any, is to be added
	 * @return	a string containing the error message,if any, or null
	 */
	public static String validateURL(String attribute,String url,Locale locale,MessageMap error){
		try {
            Pattern pattern = Pattern.compile("(https?://|ftp://)([-\\w\\.]+)+(/([\\w/_\\.]*)?)?",Pattern.CASE_INSENSITIVE);
            Matcher matcher = pattern.matcher(url);
            if (!matcher.matches()) {
            	error.addErrorMessage(IValidatorMessage.MSG_INVALID_URL,attribute,url);
            	Object[] values = {attribute,url};
            	System.err.println("Invalid URL in validateURL " + attribute + " " + url);
				return createErrorMessage(IValidatorMessage.MSG_INVALID_URL,locale,values);
			}
		} catch (Exception e) {
	       System.err.println("Regular Expression Compilation Failed " + e);
		}
        return null;
        
	}
	
	/**
	 * 
	 * @param attribute		Attribute name with the occurence indicator, if any
	 * @param image			value to be validated
	 * @param locale		locale of the user
	 * @param error			MessageMap in which the error message,if any, is to be added
	 * @return	a string containing the error message,if any, or null
	 */
	public static String validateImage(String attribute,String image,Locale locale,MessageMap error){
		if(image.length() < 4){
			error.addErrorMessage(IValidatorMessage.MSG_INVALID_IMAGE,attribute,image);
        	Object[] values = {attribute,image};
        	System.err.println("Invalid Image in validateImage " + attribute + " " + image);
            return createErrorMessage(IValidatorMessage.MSG_INVALID_IMAGE,locale,values);
		}
		String sExtension = image.substring(image.length() - 4);
        if (!((sExtension.equalsIgnoreCase(".JPG")) ||
                (sExtension.equalsIgnoreCase(".GIF"))))  // XXX hardcoded extensions
        {
        	error.addErrorMessage(IValidatorMessage.MSG_INVALID_IMAGE,attribute,image);
        	Object[] values = {attribute,image};
        	System.err.println("Invalid Image in validateImage " + attribute + " " + image);
            return createErrorMessage(IValidatorMessage.MSG_INVALID_IMAGE,locale,values);
        }
        return null;
	}
	
	/**
     * Generate the error message to be stored in the attribute object for display as tooltip 
     * 
     * @param msgKey	Error message key
     * @param locale	Locale for which the message is to be generated
     * @param values	An array of values to be used to format the message
     * @return a string containing the error message to be shown as tooltip on the error image
     */
	public static String createErrorMessage(String msgKey,Locale locale,Object[] values){
		ResourceBundle bundle =	ResourceBundle.getBundle(IReference.BUNDLE_WPC_UI_MESSAGES,locale);
		String errorMsg = bundle.getString(msgKey);
		MessageFormat msgFormat = new MessageFormat(errorMsg);
		return msgFormat.format(values);
	}
}
