
#!      /bin/sh

#	Copyright 2009, IBM Corporation


#      Do cleanup for one collaboration area.

#	1. Run the SQL command "select coa_name, coa_id from coa order by coa_id;"
#		in your WPC DB Admin, or sqlplus, window, 
#		to see a list of collab areas and their ids.
#
#	2. Find (the name of) the collab area you want to do this report for,
#		 and write down its id.
#
#	3. Before running this script, fill in the correct value of the "coa_id" (i.e. in quotes) for the "7" in the two sed lines below.
#
#	4. Before running this script, fill in the correct <db username>/<db password>@<db schema> in the sqlplus command below.
#
#	5. Before running this script, decide whether you want fresh backup tables, or to re-use the existing ones (see below).  
#		The default is to re-use the tables.  To get fresh ones, uncomment the designated two lines below.
#
#	6. Output will be written to file clean.txt
#

# Usage of the file
if [[ $# -ne 4 ]]
then
 echo "Usage "
 echo "dooneca_clean_blob.sh --db=databasename --dbuser=dbusername --dbpassword=dbuserpassword --collabid=CollabAreaID"
 echo "Example dooneca_clean_blob.sh --db=wpcdb --dbuser=wpc1 --dbpassword=wpc1 --collabid=6"
  echo "  (Be sure the associated workflow of this collab area is an item, and not a category, workflow)"
 
 exit 0
fi

# Declare the variables
DBName=`echo $1 |cut -d "=" -f2`;export DBName
DBUSER=`echo $2 |cut -d "=" -f2`;export DBUSER
DBUserPassword=`echo $3 |cut -d "=" -f2`;export DBUserPassword
CollabID=`echo $4 |cut -d "=" -f2`;export CollabID
CleanFile="${CollabID}-{`date |awk '{print "blobcleanreport_item_ca_"$2"_"$3"_"$6"_"$4}'|sed s/":"//g`-.txt";export CleanFile


# replace symbolic constants
sed s/"= 4"/"= ${CollabID}"/ $TOP/src/integrity/SQL/db2/create_views_blob.sql > create_views_scratch.sql
#sed s/"clnCoaId"/"${CollabID}"/ $TOP/src/integrity/SQL/db2/clean.sql > clean_scratch.sql

db2 connect to ${DBName} user ${DBUSER} using ${DBUserPassword}

echo "*****************************************************************"
date
echo "Deleting messages from the messages table"
echo "*****************************************************************"

db2 "delete from messages"
db2 "commit"

echo "*********************************************************************"
echo " Dropping  views if they already exist"
echo " Ignore all DB21034E messages"
echo "*********************************************************************"

#db2 -tf $TOP/src/integrity/SQL/db2/drop_backup_tables_db2.sql
db2 -tf $TOP/src/integrity/SQL/db2/drop_views_db2.sql
db2 -tf $TOP/src/integrity/SQL/db2/drop_views_blob.sql

echo "*********************************************************************"
echo "Creating Views"
echo "Ignore all SQL0100W messages"
echo "*********************************************************************"

#db2 -tf $TOP/src/integrity/SQL/db2/create_backup_tables_db2.sql
db2 -tf create_views_scratch.sql

echo "*********************************************************************"
echo " Dropping the procedures if they already exist"
echo " Ignore all DB21034E messages"
echo "*********************************************************************"

db2 -tf $TOP/src/integrity/SQL/db2/drop_procedures_db2.sql


echo "*********************************************************************"
echo "Creating the procedures"
echo "*********************************************************************"

db2 -td@ -f $TOP/src/integrity/SQL/db2/create_procedures_db2.sql

echo "*********************************************************************"
echo "Invoking CLEAN_AND_BACKUP_DATABASE_ROWS PROCEDURE FOR COLLABID ${CollabID}"
echo "*****************************************************************"
# clean
db2 "call CLEAN_AND_BACKUP_DATABASE_ROWS(${CollabID})"

db2 "select substr(message_text,1,200) from messages " > ${CleanFile}

# Clean up views, to leave a clean situation for next creator/user of them
db2 -tf $TOP/src/integrity/SQL/db2/drop_views_blob.sql

#EOF

rm create_views_scratch.sql

# move logfile to standard location
if [ "$TOP" ]
then
mv ${CleanFile}  ${TOP}/logs/badBlobsCleanings
else
mv ${CleanFile}  logs/badBlobsCleanings
fi

exit $?

