#!/bin/bash
#	Copyright 2007, IBM Corporation

#      fullIntegrityRestore.sh - Restoring db based on basic integrty verify.sh, ghost items check, and bad blob reporting, 
#       for a given source catalog and all its associated collaboration areas catalogs.

#

if [[ $# -lt 5 ]]
then
    echo "Usage "
    echo "fullIntegrityRestore.sh --code=<cmp code> --ctgname=<catalog name> --basic=<y/n> --wf=<y/n> --serialization=<y/n>"
    echo "For example to run all the restorations"
    echo "fullIntegrityRestore.sh --code=trigo --ctgname='TestCatalog' --basic=y --wf=y --serialization=y"
    echo "To clean only serialization corruptions"
    echo "fullIntegrityRestore.sh --code=<cmp code> --ctgname=TestCatalog --basic=n --wf=n --serialization=y"

    exit 0
fi 

#Declare variable
CMPCODE=`echo $1 | cut -d "=" -f2`;export CMPCODE
CTG_NAME=`echo $2 | cut -d "=" -f2`;export CTG_NAME
RUNBASIC=`echo $3 | cut -d "=" -f2`;export RUNBASIC
RUNWF=`echo $4 | cut -d "=" -f2`;export RUNWF
RUNSER=`echo $5 | cut -d "=" -f2`;export RUNSER 

if [ ${CCD_DB} = "db2" ]
then
    ./fullIntegrityRestoreDb2.sh --code=$CMPCODE --ctgname=$CTG_NAME --basic=$RUNBASIC --wf=$RUNWF --serialization=$RUNSER
   #echo " This is for oracle database only" 
   exit 0
fi

curdir="`pwd`"
. ${TOP}/bin/get_params.sh
. ${TOP}/src/db/schema/util/set_db.sh --confirm_db=no

cd $curdir

echo "===================================================================================================================="
echo ">>>> STARTING FULL INTEGRITY CHECK;  DIRECTORY IS $curdir <<<<<<<<<<"
echo "===================================================================================================================="
echo " "
if [[ $RUNBASIC = 'y' ]]
then
        cd ../basic/oracle
        echo "==============================================="
        echo "NOW RUNNING BASIC INTEGRITY VERIFY.SH"
        echo "==============================================="
        echo $DB_USER_NAME
        CONNECTION=${DB_USER_NAME}/${DB_USR_PWD}@${DB_SERVICE_NAME}
        ./basic_integrity_restore.sh --code=$CMPCODE
       cd $curdir
# Redundant, but matches with analagous redundant line that will be produced in ghost items and bad blobs runs
        echo "==============================================="
        echo "DIRECTORY NOW IS `pwd`"
        echo "==============================================="

        # Be sure we're in original working directory
       echo "================================================"
       echo "DONE WITH BASIC INTEGRITY VERIFY.SH;  DIRECTORY NOW IS `pwd`"
   fi
    cd $curdir
   if [ $RUNWF = 'y' ]
   then
        cd workflow/oracle
        echo "==============================================="

        echo "NOW RUNNING WORKFLOW INTEGRITY CHECKS"
        echo "==============================================="

        ./workflow_integrity_restore.sh --code=$CMPCODE --ctgname="$CTG_NAME"
        #./docleanall.sh
        # Go back to original working directory
        cd $curdir
        echo "================================================"
        echo "DONE WITH WORKFLOW INTEGRITY ITEM CHECKS;  DIRECTORY NOW IS `pwd`"
        echo "================================================"
    fi
    if [ ${RUNSER} = 'y' ]
    then

        cd serialization/oracle
        echo "==============================================="
        echo "NOW RUNNING SERIALIZATION INTEGRITY CHECKS"
        echo "==============================================="
        echo $CMPCODE
        ./serialization_integrity_restore.sh --code=$CMPCODE --ctgname="$CTG_NAME" 
        #./doreportbloball.sh --code=$CMPCODE

        # Go back to original working directory
        cd $curdir
        echo "================================================"
        echo "DONE WITH SERIALIZATION INTEGRITY RESTORATION;  DIRECTORY NOW IS `pwd`"
        echo "================================================"
        echo " "
    fi
     


# Run master.sh

curdir="`pwd`"

# get src connection descriptor
#cd masterAndCleanup



# Run dobadblob checks

# get src connection descriptor
#./doreportbloball.sh --code=$CMPCODE

# Go back to original working directory
echo " "
echo " "





echo "===================================================================================================================="
echo ">>>> FINISHED FULL INTEGRITY CHECK; DIRETORY NOW IS `pwd` <<<<<<<<<<"
echo "===================================================================================================================="
echo " "



# done
exit $?

