#!/bin/bash
# Licensed Materials - Property of IBM
# 5724-I68
#
# (C) Copyright IBM Corp. 2000, 2006
#
# US Government Users Restricted Rights - Use, duplication, or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# 
# # Copyright (c) 2000, 2004 IBM Corp. - All Rights Reserved.
# # IBM makes no representations or warranties about the suitability of
# # this program, either express or implied, including but not limited to
# # the implied warranties of merchantability, fitness for a particular
# # purpose, or non-infringement. 
# 
#

. ${TOP}/bin/get_params.sh

printUsage()
{
	${ECHO} "USAGE: "$0" --code=<Company code> 
 	${ECHO} "EXAMPLE: "$0" --code=ibm
 	exit 0
}

cleanup_exit()
{
${RM} $tmpfile
${RM} $tmpsqlfile
exit
}

if [ $# -lt 1 ]
then
	printUsage
fi

. ${TOP}/src/db/schema/util/set_db.sh
_ERROR_LOGS=$TOP/logs/delete_old_wf_events_errfile.log
tmpfile=$TOP/logs/del_events_temp.out
tmpsqlfile=$TOP/logs/del_events_temp.sql

START_TIME=`date`

if [ "${_CCD_CODE}" = "" ]; then
    ${ECHO} "Usage: $0 --code=<company code> --end_date=<YYYY-MM-DD.HH24:MI:SS>"
    exit
fi

${TOUCH} tmpcmpid.sql
${CAT} $DB_SCRIPTS/get_cmpid_for_cmpcode.sql | ${SED} "s/%_CCD_CODE%/$_CCD_CODE/g" > tmpcmpid.sql
CMP_ID=`. ${TOP}/src/db/schema/util/run_db_cmd.sh --file=tmpcmpid.sql --errorfilter=yes | ${GREP} 'cmpid-'| cut -f2 -d "-"`

if [ "$CMP_ID" != "" ]; then
  #${ECHO} $CMP_ID_QRY
  ${ECHO} "COMPANY ID = " $CMP_ID >> $_ERROR_LOGS
  ${RM} tmpcmpid.sql
else 
  #. ${TOP}/src/db/schema/util/run_db_cmd.sh --file=tmpcmpid.sql
  ${ECHO} "Company code not exists:" ${_CCD_CODE}
  exit 0
fi

if [ "$CCD_DB" = "db2" ]; then
  WEE_QUERY="delete from wee where exists (select 1 from wfe where wfe_id = wee_event_id and wfe_event_status in ('DONE','ERROR') and wfe_start_time < (current timestamp - 7 days)  and wfe_cmp_id = $CMP_ID);"
  WFE_QUERY="delete from wfe where wfe_event_status in ('DONE','ERROR') and wfe_start_time < (current timestamp - 7 days)  and wfe_cmp_id = $CMP_ID;"

elif [ "$CCD_DB" = "oracle" ]; then
  WEE_QUERY="delete from wee where exists (select 1 from wfe where wfe_id = wee_event_id and wfe_event_status in ('DONE','ERROR') and wfe_start_time < sysdate - 7  and wfe_cmp_id = $CMP_ID);"
  WFE_QUERY="delete from wfe where wfe_event_status in ('DONE','ERROR') and wfe_start_time < sysdate - 7  and wfe_cmp_id = $CMP_ID;"
else  
  ${ECHO} "DB environment not set"
  cleanup_exit 
fi

${ECHO} "Executing script: " $0

${ECHO} $WEE_QUERY > $tmpsqlfile
. ${TOP}/src/db/schema/util/run_db_cmd.sh --file=$tmpsqlfile >> $_ERROR_LOGS
date >> $_ERROR_LOGS
${ECHO} $WFE_QUERY > $tmpsqlfile
. ${TOP}/src/db/schema/util/run_db_cmd.sh --file=$tmpsqlfile >> $_ERROR_LOGS
date >> $_ERROR_LOGS

END_TIME=`date` 
${ECHO} "START:" $START_TIME
${ECHO} "END  :" $END_TIME

${ECHO} " Please see the log file for further details : " $_ERROR_LOGS

cleanup_exit


